/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2001, Emmanuel CURIS
   Ce fichier : cration le 12 janvier 1999

   Cration d'un modle structural

   Choix d'un ou plusieurs atomes parmi une liste, ou suivant certains critres
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"
#include "exafs.h"

#include "atomes.h"
#include "interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define MAX_ATOMES_VISIBLES (CA_NOM_ATM_11 - CA_NOM_ATM_1 + 1)
#define LG_MAX_TXT_ATOME    12 /* Chanes dans l'arbre */
#define LG_MAX_TXT_FRAGMENT 16

static boolean *choisis = NULL;           /* pour marquer ceux qui sont choisis */
static unsigned long numero_choisi = 0;   /* Idem dans le cas de slection unique --- par dfaut, le 1er */
static unsigned long premier_visible = 0; /* Index du premier atome visible dans le slecteur */

static unsigned int nbr_atomes = 0;
static Atome **atomes = NULL;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void    prepare_choix_atome(void);
static int GER_choix_atome(unsigned int objet, int x, int y);

static void init_defileur(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Fonction qui s'occupe de l'interface -----------\
 |  Renvoie la liste des numros des atomes choisis         |
 |     ( librer aprs utilisation)                        |
 |                                                          |
 | atomes     : la table des atomes possibles               |
 |    (NULL : liste complte des atomes)                    |
 | nbr_atomes : nombre d'atomes dans la liste au-dessus     |
 |    (uniquement si slection multiple)                    |
 | multiple   : VRAI si la slection multiple est autorise |
 | num_choisi : le numro de l'atome choisi                 |
 |    (uniquement si slection simple)                      |
 \--------------------------------------------------------- */

unsigned int *ATM_choisir(Atome **liste_atomes, unsigned int nombre_atomes,
			  boolean multiple, int *num_choisi)
{
  if (liste_atomes == NULL) /* Pour l'instant, pas fait */
    return NULL;

  atomes = liste_atomes;
  nbr_atomes = nombre_atomes;

  numero_choisi = 0; /* Aucun atome choisi initialement */

  if (arbres[CHOIX_ATOME].drapeaux.prepare == FAUX)
    prepare_choix_atome();

  premier_visible = 0; /* On dmarre l'affichage sur le premier atome, toujours */
  numero_choisi = -1;  /* Aucun atome choisi initialement */
  init_defileur();

  OBJ_active(CHOIX_ATOME, CA_TOUS , multiple); /* Utiles seulement en cas de slection multiple */
  OBJ_active(CHOIX_ATOME, CA_AUCUN, multiple);

  if ( ARB_Bloquant( CHOIX_ATOME, "Choisir un atome", principale, 0 ) == CA_ANNULER ) {
    *num_choisi = -1;
    return NULL;
  }

  if (multiple == FAUX) /* Slection simple */
  {
    *num_choisi = numero_choisi;
    return NULL;
  }

  /* Slection multiple : reste  faire */

  return NULL;
}

/* -------- Initialiation du dfileur, affichant les atomes slectionns -------- */

static void init_defileur(void)
{
  unsigned int i;

  ARB_ascenseur(CHOIX_ATOME, CA_AVANT, CA_FOND_ASC, CA_BOUTON_ASC, CA_APRES, ASC_VERTICAL,
		nbr_atomes, MAX_ATOMES_VISIBLES, premier_visible, ROOT, -1, -1, METTRE);

  for (i = 0 ; i < MAX_ATOMES_VISIBLES ; i++) {
    OBJ_active(CHOIX_ATOME, CA_NOM_ATM_1 + i, premier_visible + i < nbr_atomes);
    OBJ_active(CHOIX_ATOME, CA_X_ATM_1 + i, premier_visible + i < nbr_atomes);
    OBJ_active(CHOIX_ATOME, CA_Y_ATM_1 + i, premier_visible + i < nbr_atomes);
    OBJ_active(CHOIX_ATOME, CA_Z_ATM_1 + i, premier_visible + i < nbr_atomes);
    OBJ_active(CHOIX_ATOME, CA_NOM_FRAG_1 + i, premier_visible + i < nbr_atomes);
    if (premier_visible + i < nbr_atomes) {
      Atome *atome = atomes[premier_visible + i];

      OBJ_choisit(CHOIX_ATOME, CA_NOM_ATM_1 + i, premier_visible + i == numero_choisi);
      OBJ_choisit(CHOIX_ATOME, CA_X_ATM_1 + i, premier_visible + i == numero_choisi);
      OBJ_choisit(CHOIX_ATOME, CA_Y_ATM_1 + i, premier_visible + i == numero_choisi);
      OBJ_choisit(CHOIX_ATOME, CA_Z_ATM_1 + i, premier_visible + i == numero_choisi);
      OBJ_choisit(CHOIX_ATOME, CA_NOM_FRAG_1 + i, premier_visible + i == numero_choisi);

      met_chaine(CHOIX_ATOME, CA_NOM_ATM_1 + i, LG_MAX_TXT_ATOME,
		 ATM_donnees[atome->num_atomique - 1].nom, TXT_COPIER);
      OBJ_MettreCoordonnees( CHOIX_ATOME, CA_X_ATM_1 + i, CA_Y_ATM_1 + i, CA_Z_ATM_1 + i, &(atome->position), atome->err_position );
      if (atome->fragment)
	met_chaine(CHOIX_ATOME, CA_NOM_FRAG_1 + i, LG_MAX_TXT_FRAGMENT,
		   atome->fragment->nom, TXT_COPIER);
      else
	met_chaine(CHOIX_ATOME, CA_NOM_FRAG_1 + i, LG_MAX_TXT_FRAGMENT,
		   "Atome isol"e_AIGU, TXT_COPIER);
    } else {
      Vecteur3D origine = { 0.0, 0.0, 0.0 };

      met_chaine(CHOIX_ATOME, CA_NOM_ATM_1 + i, LG_MAX_TXT_ATOME, "", TXT_COPIER);
      OBJ_MettreCoordonnees( CHOIX_ATOME, CA_X_ATM_1 + i, CA_Y_ATM_1 + i, CA_Z_ATM_1 + i, &origine, NULL );
      met_chaine(CHOIX_ATOME, CA_NOM_FRAG_1 + i, LG_MAX_TXT_FRAGMENT, "", TXT_COPIER);
    }
  }
}

/* -------- Gestion de l'arbre -------- */

static int GER_choix_atome(unsigned int objet, int x, int y)
{
  boolean clic_choisi = FAUX; /* VRAI si clic sur un nom d'atome */
  unsigned int prem_objet = ROOT;

  /* On dtecte la bonne ligne dans le dfileur */
  if ( (objet >= CA_NOM_ATM_1) && (objet <= CA_NOM_ATM_11) ) {
    clic_choisi = VRAI; prem_objet = CA_NOM_ATM_1;
  }
  if ( (objet >= CA_X_ATM_1) && (objet <= CA_X_ATM_11) ) {
    clic_choisi = VRAI; prem_objet = CA_X_ATM_1;
  }
  if ( (objet >= CA_Y_ATM_1) && (objet <= CA_Y_ATM_11) ) {
    clic_choisi = VRAI; prem_objet = CA_Y_ATM_1;
  }
  if ( (objet >= CA_Z_ATM_1) && (objet <= CA_Z_ATM_11) ) {
    clic_choisi = VRAI; prem_objet = CA_Z_ATM_1;
  }
  if ( (objet >= CA_NOM_FRAG_1) && (objet <= CA_NOM_FRAG_11) ) {
    clic_choisi = VRAI; prem_objet = CA_NOM_FRAG_1;
  }

  if (clic_choisi == VRAI) {
    if ( choisis != NULL ) { /* Slection multiple existe */
      OPTION_A_FAIRE( "Slection multiple" );
    } else { /* Slection simple */
      unsigned int i;

      if ( (numero_choisi > premier_visible) &&
	   (numero_choisi - premier_visible < MAX_ATOMES_VISIBLES) )
      {
	i = numero_choisi - premier_visible;
	OBJ_choisit(CHOIX_ATOME, CA_NOM_ATM_1 + i, FORCE_NON);
	OBJ_choisit(CHOIX_ATOME, CA_X_ATM_1 + i, FORCE_NON);
	OBJ_choisit(CHOIX_ATOME, CA_Y_ATM_1 + i, FORCE_NON);
	OBJ_choisit(CHOIX_ATOME, CA_Z_ATM_1 + i, FORCE_NON);
	OBJ_choisit(CHOIX_ATOME, CA_NOM_FRAG_1 + i, FORCE_NON);
      }

      numero_choisi = premier_visible + (objet - prem_objet);
      i = objet - prem_objet;
      OBJ_choisit(CHOIX_ATOME, CA_NOM_ATM_1 + i, FORCE_OUI);
      OBJ_choisit(CHOIX_ATOME, CA_X_ATM_1 + i, FORCE_OUI);
      OBJ_choisit(CHOIX_ATOME, CA_Y_ATM_1 + i, FORCE_OUI);
      OBJ_choisit(CHOIX_ATOME, CA_Z_ATM_1 + i, FORCE_OUI);
      OBJ_choisit(CHOIX_ATOME, CA_NOM_FRAG_1 + i, FORCE_OUI);
    }
    return BI_EVENEMENT_TRAITE;
  }

  switch (objet)
  {
  case CA_AVANT: /* Utilisation de l'ascenseur pour faire dfiler la liste */
  case CA_APRES:
  case CA_FOND_ASC:
  case CA_BOUTON_ASC:
    premier_visible = ARB_ascenseur(CHOIX_ATOME,CA_AVANT, CA_FOND_ASC, CA_BOUTON_ASC, CA_APRES, ASC_VERTICAL,
				    nbr_atomes, MAX_ATOMES_VISIBLES, premier_visible, objet, x, y, ECHANGE );

    init_defileur();
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparation de l'arbre de slection d'un atome parmi d'autres ------- */

static void prepare_choix_atome(void)
{
  if ( FAUX == arbres[ CHOIX_ATOME ].drapeaux.init ) {
    int retour;

    OPTION_A_FAIRE( "Documentation, " );
    retour = ARB_Initialiser( CHOIX_ATOME, NULL, GER_choix_atome, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }
  }

  arbres[CHOIX_ATOME].drapeaux.prepare = VRAI;
}
