/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996-2000 Emmanuel CURIS
   Cration : 12 mai 2000

   Modifications de la structure un peu plus ruses que changer  la main les coordonnes
   Mais pas le modeleur 3D avec changement  la souris
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "exafs.h"

#include "atomes.h"
#include "interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void prepare_arbre_distance(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~~~~~~~~ On dplace un atome de faon  changer sa distance par rapport  l'absorbeur courant
       en le gardant sur la droite passant par l'absorbeur et lui-mme.

       Renvoie VRAI si un dplacement a pu tre fait.
*/

boolean eloigne_absorbeur(Atome *absorbeur, Atome *atome)
{
  double d_init, t;
  Vecteur3D pos_finale;
  Vecteur3D_Erreurs err_pos_finale;

  if ( (absorbeur == NULL ) || (atome == NULL) ) return FAUX;
  if ( absorbeur == atome ) return FAUX; /* Pas de sens... */

  if (arbres[DIST_ABSORBEUR].drapeaux.prepare == FAUX)
    prepare_arbre_distance();

  /* On initialise l'arbre */
  OBJ_MettreCoordonnees( DIST_ABSORBEUR, DA_X_ATOME, DA_Y_ATOME, DA_Z_ATOME, &(atome->position), atome->err_position );

  d_init = U3D_Distance( &(absorbeur->position), &(atome->position) );
  if ( d_init <= 0.0 ) {
    EXF_ERREUR(DIV_ZERO, "Atomes confondus", txt_err_calc_annule);
    return FAUX;
  }
  OBJ_MettreNombre( DIST_ABSORBEUR, DA_DIST_INIT, d_init, 0.0 );
  OBJ_MettreNombre( DIST_ABSORBEUR, DA_DIST_FINALE, d_init, 0.0 );

  if ( ARB_Bloquant( DIST_ABSORBEUR, "Modifier la distance "a_GRAVE" l'absorbeur", fen_vue_3d, 0 ) == DA_ANNULER )
    return FAUX;

  /* Recherche de l'quation de la droite, en t = 0 : absorbeur, en t = 1 : atome, position initiale */
  t = OBJ_LireNombre( DIST_ABSORBEUR, DA_DIST_FINALE, NULL ) / d_init;
  (void) U3D_PositionSurDroite_PP( &(absorbeur->position), &(atome->position), t, &pos_finale,
				   absorbeur->err_position, atome->err_position, 0.0, &err_pos_finale );
#ifdef _DEVELOPPE_
  printf("Nouvelle position de l'atome : (%g, %g, %g)\n", pos_finale.x, pos_finale.y, pos_finale.z);
#endif
  atome->position = pos_finale;
  if ( ( absorbeur->err_position != NULL ) || ( atome->err_position != NULL ) ) {
    int retour;

    retour = ATM_CreerErreurs( atome );
    if ( retour >= 0 ) {
      *(atome->err_position) = err_pos_finale;
    }
    /* REMARQUE : aprs la 1re modification, on n'a plus indpendance entre les vecteurs des deux atomes
       ==> si on rappelle, les erreurs seront fausses */
  }

  if ( atome == ATM_courant ) { /* On modifie l'atome courant */
    OBJ_MettreCoordonnees( LISTE_MOLECULES, LM_COORD_X, LM_COORD_Y, LM_COORD_Z, &(atome->position), atome->err_position );
  }

  return VRAI;
}

static void prepare_arbre_distance(void)
{
#ifndef _DEVELOPPE_
  OBJ_active(DIST_ABSORBEUR, DA_TOUT_FRAGMENT, FORCE_NON);
  OBJ_active(DIST_ABSORBEUR, DA_TRANSLATION, FORCE_NON);
  OBJ_active(DIST_ABSORBEUR, DA_ROTATION, FORCE_NON);
  OBJ_active(DIST_ABSORBEUR, DA_CTR_GRAVITE, FORCE_NON);
  OBJ_active(DIST_ABSORBEUR, DA_ATOME, FORCE_NON);
  OBJ_active(DIST_ABSORBEUR, DA_X_ROTATION, FORCE_NON);
  OBJ_active(DIST_ABSORBEUR, DA_Y_ROTATION, FORCE_NON);
  OBJ_active(DIST_ABSORBEUR, DA_Z_ROTATION, FORCE_NON);
#endif

  arbres[DIST_ABSORBEUR].drapeaux.prepare = VRAI;
}
