/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2007, Emmanuel CURIS
   Création : janvier 2007

   Fonctions d'automatisation

   Routine d'appel des fonctions de calcul sur les graphes
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "automatise.h"
#include "graphes.h"

#include "xanes.h"		/* Automatisation des opérations de traitement du XANES */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Déclaration des fonctions locales ~~~~~~~~ */

static int AUTO_GRF_FonctionsGeneriques(struct automatiser *lot);

static int AUTO_GRF_FonctionsGenre_R(struct automatiser *lot);

static int AUTO_GRF_FonctionsType_Brut    (struct automatiser *lot);
static int AUTO_GRF_FonctionsType_MoinsMat(struct automatiser *lot);

/* ~~~~~~~~ Réalisation pratique ~~~~~~~~ */

int AUTO_EffectuerCalculs_GRF(struct automatiser *lot)
{
  if ( NULL == lot ) return 0;

  switch ( lot->classe_fonction )
  {
  case FONCTION_EST_GLOBALE:
    return AUTO_GRF_FonctionsGeneriques( lot );

  case FONCTION_EST_PAR_GENRE:
    switch ( lot->genre )
    {
    case GRF_FCT_REELLE:
      return AUTO_GRF_FonctionsGenre_R( lot );

    default:
      OPTION_A_FAIRE( "Automatiser ce genre de graphe" );
    }
    break;

  case FONCTION_EST_PAR_TYPE:
    switch ( lot->type )
    {
    case GR_BRUT:
      return AUTO_GRF_FonctionsType_Brut( lot );

    case GR_MOINSMAT:
      return AUTO_GRF_FonctionsType_MoinsMat( lot );

    default:
      OPTION_A_FAIRE( "Automatiser ce type de graphe" );
    }
    break;

  default:
    AVERTISSEMENT( "Champ d'action inconnu [%u]", lot->classe_fonction );
  }

  /* Terminé : si l'on est là, l'opération n'a pas pu être effectuée */
  return -10;
}

/* -------- Aiguillage pour les fonctions génériques -------- */

static int AUTO_GRF_FonctionsGeneriques(struct automatiser *lot __attribute__((__unused__)) )
{
  return -11;			/* Opération non effectuée */
}

/* -------- Aiguillage pour les fonctions sur graphe réel -------- */

static int AUTO_GRF_FonctionsGenre_R(struct automatiser *lot __attribute__((__unused__)) )
{
  return -11;			/* Opération non effectuée */
}

/* -------- Aiguillage pour les fonctions sur graphe de type GR_BRUT -------- */

static int AUTO_GRF_FonctionsType_Brut(struct automatiser *lot)
{
  switch ( lot->numero_fonction )
  {
  case FCT_GRF_BRUT_CONVERSION:
    OPTION_A_FAIRE( "Automatisation de la conversion fluorescence/absorption" );
    break;

  case FCT_GRF_BRUT_MATRICE:
    return XAN_MUM_Automatisation( lot );

  default:
    AVERTISSEMENT( "Option inconnue ou non automatisée [%u]", lot->numero_fonction );
  }

  return -11;			/* Opération non effectuée */
}

/* -------- Aiguillage pour les fonctions sur graphe de type GR_MOINSMAT -------- */

static int AUTO_GRF_FonctionsType_MoinsMat(struct automatiser *lot)
{
  switch ( lot->numero_fonction )
  {
  case FCT_GRF_MOINSMAT_EXAFS:
    return XAS_MU01_Automatisation( lot );

  default:
    AVERTISSEMENT( "Option inconnue ou non automatisée [%u]", lot->numero_fonction );
  }

  return -11;			/* Opération non effectuée */
}
