/* LASE - Logiciel d'Analyse des Spectres Expérimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines générales indépendantes du système

   Gestion des défileurs : liste de textes + ascenseur
*/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <math.h> /* Pour log10 */

#define _LIBRAIRIE_X_
#define _LIBRAIRIE_W_

#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void ARB_AfficheDefileur_O(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
				  unsigned long nombre_objets, boolean *choisi, unsigned long objet_choisi,
				  unsigned long premier_texte_affiche, const char *(*objet_v_texte)(unsigned int arbre, unsigned long numero));
static void ARB_AfficheDefileur_T(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
				  unsigned long nombre_textes, const char **textes, boolean *choisi, unsigned long texte_choisi,
				  unsigned long premier_texte_affiche);

static const char *ObjetVTexte(unsigned int arbre, unsigned long objet);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* Briques élémentaires pour la gestion */

static const char *ObjetVTexte(unsigned int arbre, unsigned long objet)
{
  const char **textes;

  textes = (const char **) ARB_LireComplement( arbre );
  if ( NULL == textes ) {
    (void) fprintf(stderr, "ObjetVTexte : Pas de table des textes\n");
    return NULL;
  }

  return textes[ objet ];
}

/* Choix du premier élément visible dans le défileur, pour être sûr de voir l'élément indiqué
   Renvoie le premier élément choisi

   nombre_noms         : le nombre de noms visibles dans le défileur
   nombre_objets       : le nombre total d'objets à afficher
   numero_objet_choisi : l'objet choisi, qui doit être visible dans le défileur
   premier_propose     : l'objet qui serait normalement le premier affiché dans le défileur
 */

unsigned long ARB_ChoisirDebutDefileur(unsigned long nombre_noms, unsigned long nombre_objets, unsigned long numero_objet_choisi, unsigned long premier_propose)
{
  /* Cas 1 : moins d'objets que de places dans le défileur */
  if ( nombre_objets <= nombre_noms ) {
    /* ==> on affiche toujours depuis le premier ! */
    return 0;
  }

  /* Cas 2 : objet choisi avant le premier proposé */
  if ( numero_objet_choisi < premier_propose ) {
    /* ==> le choisi devient le premier */
    premier_propose = numero_objet_choisi;
  }

  /* Cas 3 : objet choisi au-delà du défileur */
  if ( numero_objet_choisi >= premier_propose + nombre_noms ) {
    /* ==> le choisi devient le dernier */
    premier_propose = numero_objet_choisi - nombre_noms + 1;
  }

  /* Vérification : on évite qu'il y ait des places vides ! */
  if ( premier_propose + nombre_noms >= nombre_objets ) {
    premier_propose = nombre_objets - nombre_noms + 1;
  }

  /* Choix terminé */
  return premier_propose;
}

/* ~-~-~-~-~-~-~-~- Défileur de textes à partir d'une table d'objets quelconque ~-~-~-~-~-~-~-~- */

int ARB_PreparerDefileur_O(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
			   unsigned int obj_avant, unsigned int obj_fond_asc, unsigned int obj_bouton, unsigned int obj_apres,
			   unsigned long nombre_objets, boolean *choisi, unsigned long objet_choisi, unsigned long premier_objet_affiche,
			   const char *(*objet_v_texte)(unsigned int arbre, unsigned long numero_objet) )
{
  unsigned long i_obj, nbr_txt_visibles;
  int mode = TXT_COPIER;

  /* 1. Vérifications préliminaires */
  if ( obj_premier_texte > obj_dernier_texte ) {
    (void) fprintf(stderr, "ARB_PreparerDefileur_O : premier texte [%u] après le dernier texte [%u] --- arbre %u.\n",
		   obj_premier_texte, obj_dernier_texte, arbre);
    return -1;
  }
  if ( ( const char *(*)(unsigned int, unsigned long) ) NULL == objet_v_texte ) {
    (void) fprintf(stderr, "ARB_PreparerDefileur_O : Pas de fonction d'obtention des textes\n");
    return -2;
  }

  /* 2. Combien de textes sont visibles ? */
  nbr_txt_visibles = obj_dernier_texte - obj_premier_texte + 1;
  if ( 0 == lg_max_texte ) mode = TXT_REMPLACE;

  /* 3. Préparation, suivant la taille relative du défileur et de la table de textes */
  if ( 0 == nombre_objets ) { /* Aucun texte : défileur vide */
    ARB_ascenseur( arbre, obj_avant, obj_fond_asc, obj_bouton, obj_apres, ASC_VERTICAL, 0, nbr_txt_visibles, 0, 0, 0, 0, METTRE );

    for ( i_obj = obj_premier_texte ; i_obj <= obj_dernier_texte ; i_obj++ ) {
      OBJ_active ( arbre, i_obj, FORCE_NON );
      OBJ_choisit( arbre, i_obj, FORCE_NON );
      met_chaine ( arbre, i_obj, lg_max_texte, "", mode );
    }
  } else if ( nombre_objets <= nbr_txt_visibles ) { /* Moins de textes que de place */
    ARB_ascenseur( arbre, obj_avant, obj_fond_asc, obj_bouton, obj_apres, ASC_VERTICAL, 0, nbr_txt_visibles, 0, 0, 0, 0, METTRE );
    ARB_AfficheDefileur_O( arbre, obj_premier_texte, obj_dernier_texte, lg_max_texte, nombre_objets, choisi, objet_choisi, 0, objet_v_texte );
  } else {
    ARB_ascenseur( arbre, obj_avant, obj_fond_asc, obj_bouton, obj_apres, ASC_VERTICAL,
		   nombre_objets, nbr_txt_visibles, premier_objet_affiche, 0, 0, 0, METTRE );
    ARB_AfficheDefileur_O( arbre, obj_premier_texte, obj_dernier_texte, lg_max_texte, nombre_objets, choisi, objet_choisi, premier_objet_affiche, objet_v_texte );
  }

  /* Tout s'est bien passé */
  return 0;
}

static void ARB_AfficheDefileur_O(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
				  unsigned long nombre_objets, boolean *choisi, unsigned long objet_choisi,
				  unsigned long premier_texte_affiche, const char *(*objet_v_texte)(unsigned int arbre, unsigned long numero))
{
  unsigned int i_obj, i_txt;
  const char *texte;
  int mode = TXT_COPIER;

  if ( 0 == lg_max_texte ) mode = TXT_REMPLACE;

  if ( NULL == choisi ) { /* Un seul objet à la fois peut être sélectionné */
    for ( i_obj = obj_premier_texte, i_txt = premier_texte_affiche ; i_obj <= obj_dernier_texte ; i_obj++, i_txt++ ) {
      if ( i_txt < nombre_objets ) {
	texte = objet_v_texte( arbre, i_txt );
	OBJ_choisit( arbre, i_obj, i_txt == objet_choisi );
	OBJ_active ( arbre, i_obj, FORCE_OUI );
      } else {
	texte = "";
	OBJ_choisit( arbre, i_obj, FORCE_NON );
	OBJ_active ( arbre, i_obj, FORCE_NON );
      }

      met_chaine ( arbre, i_obj, lg_max_texte, texte, mode );
    }
  } else {
    int deborde = FAUX;

    for ( i_obj = obj_premier_texte, i_txt = premier_texte_affiche ; i_obj <= obj_dernier_texte ; i_obj++, i_txt++ ) {
      if ( i_txt < nombre_objets ) {
	texte = objet_v_texte( arbre, i_txt );
	OBJ_choisit( arbre, i_obj, choisi[ i_txt ] );
	OBJ_active ( arbre, i_obj, FORCE_OUI );
      } else {
	texte = "";
	if ( FAUX == deborde ) {
	  (void) fprintf(stderr, "%s, %s l.%u : + de textes que de place, mais places vides !\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  deborde = VRAI; /* Pour n'avoir le message qu'une fois */
	}
	OBJ_choisit( arbre, i_obj, FORCE_NON );
	OBJ_active ( arbre, i_obj, FORCE_NON );
      }

      met_chaine ( arbre, i_obj, lg_max_texte, texte, mode );
    }
  }
}

/* -------- Gestion du défileur -------- */

int ARB_GererDefileur_O(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
			unsigned int obj_avant, unsigned int obj_fond_asc, unsigned int obj_bouton, unsigned int obj_apres,
			unsigned long nombre_objets, boolean *choisi, unsigned long *objet_choisi, unsigned long *premier_objet_affiche,
			const char *(*objet_v_texte)(unsigned int arbre, unsigned long numero_objet), unsigned int objet, int y_souris)
{
  OBJECT *adresse __attribute__((__unused__));
  unsigned long premier_visible, nbr_obj_visibles;

  adresse = arbres[ arbre ].adresse;

  /* Vérifications préliminaires */
  if ( 0 == nombre_objets ) {
#ifdef _DEVELOPPE_ /* Car peut être le cas en situation presque normale, si depuis le clavier */
    printf("ARB_GererDefileur_O : appel alors qu'aucun objet n'est dans le défileur\n");
#endif
    return 0; /* Aucun texte */
  }
  if ( ( NULL == objet_choisi ) && ( NULL == choisi ) ) {
    (void) fprintf(stderr, "ARB_GererDefileur_O : impossible de mémoriser le ou les objets choisis\n");
    return -1;
  }
  if ( obj_premier_texte > obj_dernier_texte ) {
    (void) fprintf(stderr, "ARB_GererDefileur_O : premier texte [%u] après le dernier texte [%u] --- arbre %u.\n",
		   obj_premier_texte, obj_dernier_texte, arbre);
    return -2;
  }

  /* 2. Combien de textes sont visibles ? */
  nbr_obj_visibles = obj_dernier_texte - obj_premier_texte + 1;
  premier_visible = *premier_objet_affiche;

  /* Général : clic sur l'un des objets */
  if ( (objet >= obj_premier_texte) && (objet <= obj_dernier_texte) ) {
    unsigned long num_texte;

    num_texte = objet - obj_premier_texte + premier_visible;

    if ( choisi != NULL ) { /* Sélecteur avec sélection multiple */
      choisi[ num_texte ] = !( choisi[ num_texte ] );
      OBJ_choisit(arbre, objet, choisi[ num_texte ] );
    } else {
      unsigned long ancien_choisi;

      ancien_choisi = (obj_premier_texte + *objet_choisi) - premier_visible;	
      if ( (ancien_choisi >= obj_premier_texte) && (ancien_choisi <= obj_dernier_texte) )
	OBJ_choisit(arbre, (unsigned int) ancien_choisi, FORCE_NON);
	
      *objet_choisi = num_texte; /* Nouvel objet choisi */
      OBJ_choisit(arbre, objet, FORCE_OUI);
    }

    return 0; /* On a fini, tout s'est bien passé */
  }

  if ( ROOT == objet ) { /* Appel "clavier" : on modifie l'objet choisi à la main... */
    if ( NULL == choisi ) { /* Un seul objet choisi : on le déplace */
      if ( y_souris > 0 ) { /* On remonte = texte précédent devient choisi */
	if ( *objet_choisi == 0 )
	  return 0; /* On est en haut, rien à faire */

	(*objet_choisi)--;
	if ( *objet_choisi < premier_visible ) { /* On sort -> on fait défiler la liste pour que l'objet choisi reste en haut */
	  premier_visible = *objet_choisi;
	}
      } else if ( y_souris < 0 ) {
	if ( *objet_choisi >= ( nombre_objets - 1 ) )
	  return 0;		/* On est en bas, rien à faire */

	(*objet_choisi)++;
	if ( *objet_choisi > ( premier_visible + nbr_obj_visibles - 1 ) )
	  premier_visible++;
      }

      /* On positionne correctement l'ascenseur */
      ARB_ascenseur( arbre, obj_avant, obj_fond_asc, obj_bouton, obj_apres, ASC_VERTICAL,
		     nombre_objets, nbr_obj_visibles, premier_visible, ROOT, 0, 0, METTRE );
    } else {
      printf("ARB_GererDefileur_O : À FAIRE, gestion clavier avec sélection multiple\n");
    }

    /* On réaffiche le défileur */
    ARB_AfficheDefileur_O( arbre, obj_premier_texte, obj_dernier_texte, lg_max_texte, nombre_objets, choisi, *objet_choisi, premier_visible, objet_v_texte);

    /* On met à jour pour l'appelant */
    *premier_objet_affiche = premier_visible;

    /* On a fini, tout s'est bien passé */
    return 0;
  }

  if ( nbr_obj_visibles >= nombre_objets ) { /* Moins de textes que de cases */
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "ARB_GererDefileur_O : Clic sur l'ascenseur, mais ascenseur inutile...\n");
#endif
    return 0; /* Clic sur l'ascenseur sans rôle ! */
  }

  /* On est ici ==> on a cliqué sur l'ascenseur */
  premier_visible = ARB_ascenseur( arbre, obj_avant, obj_fond_asc, obj_bouton, obj_apres, ASC_VERTICAL,
				   nombre_objets, nbr_obj_visibles, premier_visible, objet, 0, y_souris, ECHANGE );

  /* On affiche les textes en fonction de cela */
  if ( NULL == objet_choisi ) {
    ARB_AfficheDefileur_O( arbre, obj_premier_texte, obj_dernier_texte, lg_max_texte, nombre_objets, choisi, 0, premier_visible, objet_v_texte );
  } else {
    ARB_AfficheDefileur_O( arbre, obj_premier_texte, obj_dernier_texte, lg_max_texte, nombre_objets, choisi, *objet_choisi, premier_visible, objet_v_texte );
  }

  /* On met à jour pour l'appelant */
  *premier_objet_affiche = premier_visible;

  /* On a fini, tout s'est bien passé */
  return 0;
}

/* ~-~-~-~-~-~-~-~- Défileur à partir d'une table de textes ~-~-~-~-~-~-~-~- */

/* -------- Préparer les objets pour que le défileur apparaisse correctement -------- */

int ARB_PreparerDefileur_T(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
			   unsigned int obj_avant, unsigned int obj_fond_asc, unsigned int obj_bouton, unsigned int obj_apres,
			   unsigned long nombre_textes, const char **textes, boolean *choisi, unsigned long texte_choisi,
			   unsigned long premier_texte_affiche)
{
  unsigned long i_obj, nbr_txt_visibles;
  int mode = TXT_COPIER;

  /* 1. Vérifications préliminaires */
  if ( obj_premier_texte > obj_dernier_texte ) {
    (void) fprintf(stderr, "ARB_PreparerDefileur_T : premier texte [%u] après le dernier texte [%u] --- arbre %u.\n",
		   obj_premier_texte, obj_dernier_texte, arbre);
    return -2;
  }

  /* 2. Combien de textes sont visibles ? */
  nbr_txt_visibles = obj_dernier_texte - obj_premier_texte + 1;
  if ( 0 == lg_max_texte ) mode = TXT_REMPLACE;

  /* 3. Préparation, suivant la taille relative du défileur et de la table de textes */
  if ( ( NULL == textes ) || ( 0 == nombre_textes ) ) { /* Aucun texte : défileur vide */
    ARB_ascenseur( arbre, obj_avant, obj_fond_asc, obj_bouton, obj_apres, ASC_VERTICAL, 0, nbr_txt_visibles, 0, 0, 0, 0, METTRE );

    for ( i_obj = obj_premier_texte ; i_obj <= obj_dernier_texte ; i_obj++ ) {
      OBJ_active ( arbre, i_obj, FORCE_NON );
      OBJ_choisit( arbre, i_obj, FORCE_NON );
      met_chaine ( arbre, i_obj, lg_max_texte, "", mode );
    }
  } else if ( nombre_textes <= nbr_txt_visibles ) { /* Moins de textes que de place */
    ARB_ascenseur( arbre, obj_avant, obj_fond_asc, obj_bouton, obj_apres, ASC_VERTICAL, 0, nbr_txt_visibles, 0, 0, 0, 0, METTRE );
    ARB_AfficheDefileur_T( arbre, obj_premier_texte, obj_dernier_texte, lg_max_texte, nombre_textes, textes, choisi, texte_choisi, 0 );
  } else {
    ARB_ascenseur( arbre, obj_avant, obj_fond_asc, obj_bouton, obj_apres, ASC_VERTICAL,
		   nombre_textes, nbr_txt_visibles, premier_texte_affiche, 0, 0, 0, METTRE );
    ARB_AfficheDefileur_T( arbre, obj_premier_texte, obj_dernier_texte, lg_max_texte, nombre_textes, textes, choisi, texte_choisi, premier_texte_affiche );
  }

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Gérer le défileur --------------------------------\
 |   Renvoie 0 si tout s'est bien passé,                      |
 |           != 0 en cas d'erreur (avec message sur stderr)   |
 |                                                            |
 | objet : objet cliqué, si ROOT : gestion au clavier...      |
 |  (dans ce cas : y_souris > 0 si on monte = texte précédent,|
 |                 y_souris < 0 si on descend = texte suivant)|
 \----------------------------------------------------------- */

int ARB_GererDefileur_T(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
			unsigned int obj_avant, unsigned int obj_fond_asc, unsigned int obj_bouton, unsigned int obj_apres,
			unsigned long nombre_textes, const char **textes, boolean *choisi, unsigned long *texte_choisi,
			unsigned long *premier_texte_affiche, unsigned int objet, int y_souris)
{
  OBJECT *adresse __attribute__((__unused__));
  void *ancien_pointeur;
  int retour;

  adresse = arbres[ arbre ].adresse;

  /* Vérifications préliminaires
     Rq : ( texte_choisi == NULL ) && ( choisi == NULL ) sera aussi vérifié dans ARB_GererDefileur_O
     Mais message plus clair si problème en le vérifiant ici aussi... */
  if ( ( nombre_textes == 0 ) || ( textes == NULL ) ) {
#ifdef _DEVELOPPE_ /* Car peut être le cas en situation presque normale, si depuis le clavier */
    printf("ARB_GererDefileur_T : appel alors qu'aucun texte n'est dans le défileur\n");
#endif
    return 0; /* Aucun texte */
  }
  if ( ( texte_choisi == NULL ) && ( choisi == NULL ) ) {
    (void) fprintf(stderr, "ARB_GererDefileur_T : impossible de mémoriser le ou les objets choisis\n");
    return -1;
  }
  if ( obj_premier_texte > obj_dernier_texte ) {
    (void) fprintf(stderr, "ARB_GererDefileur_T : premier texte [%u] après le dernier texte [%u] --- arbre %u.\n",
		   obj_premier_texte, obj_dernier_texte, arbre);
    return -2;
  }

  /* On mémorise l'ancien pointeur */
  ancien_pointeur = ARB_LireComplement( arbre );

  /* On installe le tableau de textes comme info complémentaire */
  ARB_MettreComplement( arbre, NULL );
  ARB_MettreComplement( arbre, textes );

  /* On appelle la routine générique */
  retour = ARB_GererDefileur_O( arbre, obj_premier_texte, obj_dernier_texte, lg_max_texte, obj_avant, obj_fond_asc, obj_bouton, obj_apres,
				nombre_textes, choisi, texte_choisi, premier_texte_affiche, ObjetVTexte, objet, y_souris );

  /* On remet l'ancien pointeur */
  ARB_MettreComplement( arbre, NULL );
  ARB_MettreComplement( arbre, ancien_pointeur );

  /* On renvoie le résultat */
  return retour;
}

/* -------- Afficher les textes dans le défileur -------- */

static void ARB_AfficheDefileur_T(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
				  unsigned long nombre_textes, const char **textes, boolean *choisi, unsigned long texte_choisi,
				  unsigned long premier_texte_affiche)
{
  int mode = TXT_COPIER;
  unsigned int i_obj, i_txt;

  if ( 0 == lg_max_texte ) mode = TXT_REMPLACE;

  if ( NULL == choisi ) { /* Un seul texte à la fois peut être sélectionné */
    for ( i_obj = obj_premier_texte, i_txt = premier_texte_affiche ; i_obj <= obj_dernier_texte ; i_obj++, i_txt++ ) {
      if ( i_txt < nombre_textes ) {
	OBJ_choisit( arbre, i_obj, i_txt == texte_choisi );
	OBJ_active ( arbre, i_obj, FORCE_OUI );
	met_chaine ( arbre, i_obj, lg_max_texte, textes[ i_txt ], mode );
      } else {
	OBJ_choisit( arbre, i_obj, FORCE_NON );
	OBJ_active ( arbre, i_obj, FORCE_NON );
	met_chaine ( arbre, i_obj, lg_max_texte, "", mode );
      }
    }
  } else {
    int deborde = FAUX;

    for ( i_obj = obj_premier_texte, i_txt = premier_texte_affiche ; i_obj <= obj_dernier_texte ; i_obj++, i_txt++ ) {
      if ( i_txt < nombre_textes ) {
	OBJ_choisit( arbre, i_obj, choisi[ i_txt ] );
	OBJ_active ( arbre, i_obj, FORCE_OUI );
	met_chaine ( arbre, i_obj, lg_max_texte, textes[ i_txt ], mode );
      } else {
	if ( FAUX == deborde ) {
	  (void) fprintf(stderr, "ARB_AfficheDefileur_T : + de textes que de place, mais places vides !\n");
	  deborde = VRAI; /* Pour n'avoir le message qu'une fois */
	}
	OBJ_choisit( arbre, i_obj, FORCE_NON );
	OBJ_active ( arbre, i_obj, FORCE_NON );
	met_chaine ( arbre, i_obj, lg_max_texte, "", mode );
      }
    }
  }
}
