/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2003, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Routines de dessin fondamentales
   Les quelques routines qui, a priori, ne dpendent pas du systme !
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <math.h> /* Pour isinf() ou quivalent */

#ifdef _UNIX_
# define _LIBRAIRIE_X_
#elif defined(_WINDOWS_)
# define _LIBRAIRIE_W_
#endif

#include "txt_utils.h"
#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Conversion cran <-> rgion 2D du plan ~-~-~-~-~-~-~-~-~ */

/* -------- Calcul des facteurs d'chelle --------
     Renvoie  0 si tout s'est bien pass
             <0 en cas d'erreur

   Les facteurs d'chelle sont renvoys dans *echelle_X et *echelle_Y, si non-NULL.
   Si NULL, pas calcul => on peut utiliser la fonction pour un seul axe aussi

   Les facteurs d'chelle sont exprims en vrai/pixel : utilisables avec les autres fonctions

   largeur : le nombre de pixels de large de la zone : on peut donc dessiner entre X [x_min] et X + largeur - 1 [x_max]
   hauteur : le nombre de pixels de large de la zone : on peut donc dessiner entre Y [y_min] et Y + hauteur - 1 [y_max]
   -------- */

int TRC_Echelles(unsigned int largeur, unsigned int hauteur, double x_min, double x_max, double y_min, double y_max,
		 double *echelle_X, double *echelle_Y)
{
  if ( echelle_X != NULL ) {
    if ( largeur < 2 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Largeur de la zone cran nulle ou rduite  un pixel\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
    if ( x_max == x_min ) {
      (void) fprintf(stderr, "%s,%s l.%u : Largeur de la zone relle nulle\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -2;
    }

    *echelle_X = ( x_max - x_min ) / (double) ( largeur - 1 );
  }

  if ( echelle_Y != NULL ) {
    if ( hauteur < 2 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Hauteur de la zone cran nulle ou rduire  un pixel\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -11;
    }
    if ( y_max == y_min ) {
      (void) fprintf(stderr, "%s,%s l.%u : Hauteur de la zone relle nulle\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -12;
    }

    *echelle_Y = ( y_max - y_min ) / (double) (hauteur - 1);
  }

  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Aides pour les styles de dessin ~-~-~-~-~-~-~-~-~ */

void TRC_CouleurSuivante(unsigned int *couleur, unsigned int *style, unsigned int *marque)
{
  if ( ( NULL == couleur ) && ( NULL == style ) && ( NULL == marque ) ) return;	/* Rien  faire ! */

  /* Premier cas : on veut faire voluer la couleur, donc potentiellement le style et la marque */
  if ( couleur != NULL ) {
    (*couleur)++;
    if ( *couleur >= BI_NOMBRE_COULEURS ) {
      /* On atteint la couleur maximale -> on recommence au noir */
      *couleur = NOIR;

      if ( style != NULL ) {
	/* On passe au style suivant */
	(*style)++;

	if ( *style >= BI_NOMBRE_STYLES_LIGNES ) {
	  /* On atteint le dernier style -> on recommence  la ligne continue [ou  l'absence de ligne, s'il y a des marques] */
	  if ( marque != NULL ) {
	    *style = BI_AUCUNE_LIGNE;
	    (*marque)++;
	    if ( *marque >= BI_NOMBRE_MARQUES ) {
	      /* On a atteint la dernire couleur, dernier style, dernire marque -> on recommence  0 ! */
	      *marque = BI_MARQUE_POINT;
	    }
	  } else {
	    *style = BI_CONTINU;
	  }
	}
      } else if ( marque != NULL ) {
	/* On passe  la marque suivante */
      }
    }
    return;
  }

  /* Deuxime cas : on ne change pas la couleur, juste le style & la marque */
  if ( style != NULL ) {
    /* On passe au style suivant */
    (*style)++;

    if ( *style >= BI_NOMBRE_STYLES_LIGNES ) {
      /* On atteint le dernier style -> on recommence  la ligne continue [ou  l'absence de ligne, s'il y a des marques] */
      if ( marque != NULL ) {
	*style = BI_AUCUNE_LIGNE;
	(*marque)++;
	if ( *marque >= BI_NOMBRE_MARQUES ) {
	  /* On a atteint le dernier style, dernire marque -> on recommence  0 ! */
	  *marque = BI_MARQUE_POINT;
	}
      } else {
	*style = BI_CONTINU;
      }
    }
    return;
  }

  /* Troisime cas : on ne change que la marque */
  if ( marque != NULL ) {
    (*marque)++;
    if ( *marque >= BI_NOMBRE_MARQUES ) {
      /* On a atteint le dernier style, dernire marque -> on recommence  0 ! */
      *marque = BI_MARQUE_POINT;
    }
  }
}

/* ~-~-~-~-~-~-~-~-~ Affichage de messages ~-~-~-~-~-~-~-~-~ */

inline void TRC_Message(Fenetre *fenetre, const char *texte)
{
  int x_centre, y_centre;

  if ( NULL == fenetre ) return;

  /* Le message est plac au centre de la fentre indique */
  x_centre = (fenetre->w_larg - fenetre->w_x) / 2;
  y_centre = (fenetre->w_haut - fenetre->w_y) / 2;

  /* On l'affiche normalement */
  TRC_Texte( fenetre, x_centre, y_centre, BI_TEXTE_CENTRER_H, BI_TEXTE_CENTRER_V, 0, texte );
}

void TRC_Z_Message(Fenetre *fenetre, int x_HG, int y_HG, unsigned int largeur, unsigned int hauteur, const char *texte)
{
  int x_centre, y_centre;

  /* Le message est plac au centre de la zone indique */
  x_centre = x_HG + largeur / 2;
  y_centre = y_HG + hauteur / 2;

  /* On l'affiche normalement */
  TRC_Z_Texte( fenetre, x_HG, y_HG, largeur, hauteur, x_centre, y_centre, BI_TEXTE_CENTRER_H, BI_TEXTE_CENTRER_V, 0, texte );
}
