/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Enjoliver des groupes d'objets
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define _LIBRAIRIE_X_
#define _LIBRAIRIE_W_

#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Enjoliver un ascenseur -------- */

int ARB_EnjoliverAscenseur(unsigned int arbre, unsigned int avant, unsigned int fond, unsigned int bouton, unsigned int apres,
			   int type, int delta)
{
  /* 1. Vrifications prliminaires */
  if ( arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "ARB_EnjoliverAscenseur : numro d'arbre incorrect [%u]\n", arbre);
    return -1;
  }
  if ( bouton > arbres[ arbre ].nbr_objets ) {
    (void) fprintf(stderr, "ARB_EnjoliverAscenseur : numro d'objet incorrect [arbre %u : objet %u]\n", arbre, bouton);
    return -2;
  }
  if ( ROOT == bouton ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "ARB_EnjoliverAscenseur : il faudrait identifier le bouton [arbre %u]\n", arbre);
#endif
    bouton = fond + 1; /* En gnral, fonctionne */
  }

  /* Enjolivation */
  if ( ASC_VERTICAL == type ) {
    OBJ_deplace ( arbre, fond, delta, +1, DEPLACE_PIXELS );
    OBJ_agrandit( arbre, fond, 0, -2 );
    if ( delta != 0 ) {
      OBJ_deplace( arbre, avant, delta, 0, DEPLACE_PIXELS );
      OBJ_deplace( arbre, apres, delta, 0, DEPLACE_PIXELS );
    }
  } else if ( ASC_HORIZONTAL == type ) {
    unsigned int lg_fleche;

    OBJ_deplace ( arbre, fond, +1, delta, DEPLACE_PIXELS );
    OBJ_agrandit( arbre, fond, -2, 0 );
    if ( delta != 0 ) {
      OBJ_deplace( arbre, avant, 0, delta, DEPLACE_PIXELS );
      OBJ_deplace( arbre, apres, 0, delta, DEPLACE_PIXELS );
    }

#ifdef _LIBRAIRIE_X_
    OBJ_taille( arbre, avant, -1, -1, &lg_fleche, NULL );
    if ( lg_fleche < BI_haut_car ) {
      unsigned int delta;

      delta = BI_haut_car - lg_fleche;
      /* On veut une flche carre, pour avoir une jolie flche ! */
      OBJ_taille ( arbre, avant, BI_haut_car, -1, NULL, NULL);
      OBJ_taille ( arbre, apres, BI_haut_car, -1, NULL, NULL);
      OBJ_deplace( arbre, apres, -delta, 0, DEPLACE_PIXELS);

      /* Consquence : l'ascenseur lui-mme est rduit... */
      OBJ_deplace ( arbre, fond, delta, 0, DEPLACE_PIXELS );
      OBJ_agrandit( arbre, fond, -2*delta, 0 );
    }
#endif
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Enjoliver un dfileur -------- */

int ARB_EnjoliverDefileur(unsigned int arbre, unsigned int fond_textes, unsigned int texte_1, unsigned int texte_N,
			  unsigned int avant, unsigned int fond, unsigned int bouton, unsigned int apres, int position)
{
  unsigned int nbr_textes, i;

  /* 1. Vrifications prliminaires */
  if ( arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s, %s l.%u : numro d'arbre incorrect [%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
  if ( ROOT == fond_textes ) {
    int nv_fond;

    /* On cherche l'objet qui a le 1er texte comme 1er fils */
    nv_fond = OBJ_TrouverPere( arbre, texte_1 );
#ifdef _DEVELOPPE_
    OPTION_A_FAIRE( "Il faudrait identifier le fond" );
    (void) fprintf( stderr, "      arbre = %u, texte 1 = %u\n", arbre, texte_1 );
#endif
    if ( nv_fond < 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : impossible de trouver le fond [arbre %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, arbre);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return -2;
    }
    fond_textes = (unsigned int) nv_fond;
  }

  nbr_textes = texte_N - texte_1 + 1;

  /* On agrandit le fond */
  OBJ_agrandit( arbre, fond_textes, 0, nbr_textes);

  /* On adapte l'ascenseur en consquence */
  if ( ARB_ASCENSEUR_DROITE == position ) {
    OBJ_agrandit( arbre, fond, 0, nbr_textes - 1 );
    ARB_EnjoliverAscenseur( arbre, avant, fond, bouton, apres, ASC_VERTICAL, +1 );
    OBJ_deplace( arbre, apres, 0, nbr_textes, DEPLACE_PIXELS );
  } else if ( ARB_ASCENSEUR_GAUCHE == position ) {
    OBJ_agrandit( arbre, fond, 0, ( (int) nbr_textes ) - 2 );
    ARB_EnjoliverAscenseur( arbre, avant, fond, bouton, apres, ASC_VERTICAL, -1 );
    OBJ_deplace( arbre, apres, 0, nbr_textes, DEPLACE_PIXELS );
  }

  /* On dcale les textes dans le dfileur */
  for ( i = 1 ; i < nbr_textes ; i++ ) {
    OBJ_deplace( arbre, texte_1 + i, 0, i, DEPLACE_PIXELS );
  }

  /* S'il y a beaucoup de textes : on remonte le tout un peu --- sauf si c'est la racine ! */
  if ( ( fond_textes != ROOT ) && ( nbr_textes > (BI_haut_car / 2) ) ) {
    int delta;

    delta = BI_haut_car / 2;
    OBJ_deplace( arbre, fond_textes, 0, -delta, DEPLACE_PIXELS );
    OBJ_deplace( arbre, avant, 0, -delta, DEPLACE_PIXELS );
    OBJ_deplace( arbre, fond , 0, -delta, DEPLACE_PIXELS );
    OBJ_deplace( arbre, apres, 0, -delta, DEPLACE_PIXELS );
  }

  /* Tout s'est bien pass */
  return 0;
}
