/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2003, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion des fentres : cration
   Remarque : comme c'est assez bas niveau, trs peu de routines sont indpendantes du systme !
*/

#include <stdio.h>
#include <stdlib.h>

#ifdef _UNIX_
# define _LIBRAIRIE_X_ /* Pour avoir accs aux fonctions internes */
#elif defined(_WINDOWS_)
# define _LIBRAIRIE_W_
#endif

#include "bib_interface.h"
#ifdef _DEVELOPPE_
# include "txt_utils.h"
#endif

/* -------- Fentre contenant une srie d'onglet basculant d'un arbre  l'autre --------
   Chaque onglet est une sous-fentre X, un clic dessus bascule d'arbre et d'onglet actif
 * ------------------------------------------------------------------------------------ */

Fenetre *FEN_CreerOnglets(int attributs, unsigned int nombre_onglets, struct bi_onglet *onglets)
{
  int retour, y_arbre;
  unsigned int largeur_fenetre, hauteur_fenetre;
  unsigned int largeur_arbres = 0, hauteur_arbres = 0;
  unsigned int i;
  Fenetre *fenetre = NULL;

  if ( ( 0 == nombre_onglets ) || ( NULL == onglets ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucun onglet indiqu, annul !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On cre la fentre de base */
  fenetre = FEN_NouvelleFenetre( BI_FENETRE_ONGLETS, attributs, NULL );
  if ( NULL == fenetre ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur en voulant crer la fentre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On dtecte la hauteur et la largeur de chaque arbre */
  y_arbre = BI_haut_car + 2;
  for ( i = 0 ; i < nombre_onglets ; i++ ) {
    if ( arbres[ onglets[ i ].numero_arbre ].adresse[ ROOT ].ob_width > largeur_arbres ) {
      largeur_arbres = arbres[ onglets[ i ].numero_arbre ].adresse[ ROOT ].ob_width;
    }
    if ( arbres[ onglets[ i ].numero_arbre ].adresse[ ROOT ].ob_height > hauteur_arbres ) {
      hauteur_arbres = arbres[ onglets[ i ].numero_arbre ].adresse[ ROOT ].ob_height;
    }
  }

  /* Chacun des arbres doit avoir ces dimensions */
  for ( i = 0 ; i < nombre_onglets ; i++ ) {
    arbres[ onglets[ i ].numero_arbre ].adresse[ ROOT ].ob_width  = largeur_arbres;
    arbres[ onglets[ i ].numero_arbre ].adresse[ ROOT ].ob_height = hauteur_arbres;

    /* Et on le place en 0, 1 */
    arbres[ onglets[ i ].numero_arbre ].adresse[ ROOT ].ob_x = 0;
    arbres[ onglets[ i ].numero_arbre ].adresse[ ROOT ].ob_y = y_arbre;
  }

  /* Les onglets sont, actuellement, en haut */
  largeur_fenetre = largeur_arbres;
  hauteur_fenetre = hauteur_arbres + BI_haut_car + 1; /* Sparateur en-dessous & une ligne de texte */

  /* On met  la taille la fentre (et on bloque sa taille) */
  FEN_ImposerTaille( fenetre, -1, -1, largeur_fenetre, hauteur_fenetre );

  /* On prpare les sous-fentres : une par arbre possible */
  fenetre->ss_fenetres = (struct fenetre **) malloc( sizeof( struct fenetre * ) * nombre_onglets );
  if ( NULL == fenetre->ss_fenetres ) {
    /* Impossible de crer la table des sous-fentres ! */
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la table des sous-fentres\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  fenetre->nbr_ss_fenetres = nombre_onglets;

  /* On cre les fentres des arbres associs  chaque onglet */
  for ( i = 0 ; i < nombre_onglets ; i++ ) {
    char *titre = NULL;		/* Utilise juste pour la version de dveloppement */

    /* S'il n'a pas t initialis, on le fait pour pouvoir crer les fentres X */
    if ( FAUX == arbres[ onglets[ i ].numero_arbre ].drapeaux.init ) {
      int retour;

      AVERTISSEMENT( "Arbre %u (onglet %u) non initialis", onglets[ i ].numero_arbre, i );
      retour = ARB_Initialiser( onglets[ i ].numero_arbre, NULL,
				(int (*)(unsigned int, int , int)) NULL, (int (*)(unsigned int, int, int *)) NULL );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : Problme en voulant initialiser l'arbre de l'onglet %u [code %d]\n",
		       __FILE__, __FUNCTION__, __LINE__, i, retour);
	FEN_Detruire( fenetre );
	return NULL;
      }
    }

    /* Une sous-fentre par arbre, cre et ouverte mais cache ! */
#ifdef _DEVELOPPE_
    titre = TXT_genere_chaine( "Onglet %u", i );
#endif
    fenetre->ss_fenetres[ i ] = FEN_OuvrirArbre( fenetre, 0, onglets[ i ].numero_arbre, titre, 0, y_arbre - 1 );
#ifdef _DEVELOPPE_
    free( titre ); titre = NULL;
#endif
  }

  /* On associe la table des onglets au PREMIER complment de la fentre */
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_1, onglets );

  /* On rserve les infos associes [spcifiques du systme] */
  retour = BI_Onglets_Creation( fenetre, nombre_onglets, onglets );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur  la cration des onglets ! Opration annule\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FEN_Detruire( fenetre );
    return NULL;
  }

  /* Termin, tout s'est bien pass */
  return fenetre;
}

/* -------- Changer l'onglet actif -------- */

int BI_Onglets_ChangerActif(Fenetre *fenetre, unsigned int nouvel_actif)
{
  BI_Onglet *onglets;
  int onglet_actif;

  /* Quel est l'onglet actuellement actif ?
     Dtecte les problmes comme fenetre=NULL, fenetre->type incorrect,... */
  onglet_actif = BI_QuelOngletEstActif( fenetre );
  if ( onglet_actif < 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans BI_QuelOngletEstActif() [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, onglet_actif);
    return -1;
  }

  /* Autres contrles prliminaires */
  if ( nouvel_actif >= fenetre->nbr_ss_fenetres ) {
    AVERTISSEMENT( "Numro d'onglet incorrect [%u ; 0  %lu autoris]", nouvel_actif, fenetre->nbr_ss_fenetres - 1 );
    return -3;
  }

  /* On rcupre les infos sur les onglets */
  onglets = FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
  if ( NULL == onglets ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucun onglet associ !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -4;
  }

  /* On enregistre les infos pour l'onglet actuellement actif */
  if ( onglets[ onglet_actif ].fermeture != ( int (*)(void) ) NULL ) {
    onglets[ onglet_actif ].fermeture();
  }

  if ( nouvel_actif == (unsigned int) onglet_actif ) {
    return 0; /* Rien d'autre  faire ! */
  }
  if ( onglets[ nouvel_actif ].etat & DISABLED ) {
    /* Onglet inactiv : rien  faire ! */
    return 0;
  }

  /* On cache la fentre de l'onglet actuellement actif */
  FEN_Cacher( fenetre->ss_fenetres[ onglet_actif ] );

  /* On mmorise que celui-ci est l'actif */
  BI_MemoriserOngletActif( fenetre, nouvel_actif );

  /* On fait apparatre la fentre du nouvel onglet actif */
  FEN_Afficher( fenetre->ss_fenetres[ nouvel_actif ] );

  /* Termin sans souci */
  return 0;
}
