/* Essai de la bib. X
   (C) 1999, Emmanuel CURIS

   Programme 1 : initialisation, ouverture d'une fentre qui contient un losange.
   Un appui sur une touche quitte le programme
   Il y a un rticule sur la fentre
*/

#include <stdio.h>
#include <stdlib.h>

#include "bib_interface.h"

static int x_centre = -10, y_centre = -10;

static void redessin(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  if ( NULL == fenetre ) return;

  /* Fond blanc classique */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  /* Losange : ct rouge */
  TRC_Couleur( ROUGE );
  TRC_Ligne( fenetre, 0, fenetre->w_haut >> 1, fenetre->w_larg >> 1, 0 );

  /* Losange : ct violet */
  TRC_Couleur( VIOLET );
  TRC_Ligne( fenetre, fenetre->w_larg >> 1, 0, fenetre->w_larg - 1, fenetre->w_haut >> 1 );

  /* Losange : ct noir */
  TRC_Couleur( NOIR );
  TRC_Ligne( fenetre, fenetre->w_larg - 1, fenetre->w_haut >> 1, fenetre->w_larg >> 1, fenetre->w_haut - 1 );

  /* Losange : ct vert */
  TRC_Couleur( VERT );
  TRC_Ligne( fenetre, fenetre->w_larg >> 1, fenetre->w_haut - 1, 0, fenetre->w_haut >> 1 );

  /* On place le point cliqu */
  TRC_Texte( fenetre, x_centre, y_centre, BI_TEXTE_CENTRER_H, BI_TEXTE_CENTRER_V, 0, "Clic..." );
}

static int clavier(Fenetre *fenetre, unsigned int touche, int etat)
{
  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p touche=%u etat=%d", (void *) fenetre, touche, etat );

  if ( NULL == fenetre ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [fenetre=NULL]", BI_EVENEMENT_ERREUR );
    return BI_EVENEMENT_ERREUR;
  }

  if ( ' ' == touche ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [espace]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }
  if ( K_CTRL & etat ) {
    (void) printf("Touche Ctrl appuye\n");
    SORTIE_FONCTION( __FUNCTION__, "%d [avec Ctrl]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }

  SORTIE_FONCTION( __FUNCTION__, "%d [touche quelconque]", BI_EVENEMENT_FIN_PRG );
  return BI_EVENEMENT_FIN_PRG;
}

static int clic(Fenetre *fenetre, int bouton, int n_clic, int etat, int x, int y)
{
  printf( "On clique au point X=%d, Y=%d avec le bouton %d [%d clics; etat=%d]\n",
	  x, y, bouton, n_clic, etat );

  /* On mmorise le lieu du clic */
  x_centre = x;
  y_centre = y;

  /* Et on force le redessin */
  FEN_redessin( fenetre, BI_RDS_TOUT );

  return BI_EVENEMENT_TRAITE;
}

static void rect(Fenetre *fenetre, int x, int y, int vx, int vy)
{
  TRC_Mode( BI_MODE_INVERSE );
  TRC_Couleur( GRIS );

  if ( ( vx >= 0 ) && ( vy >= 0 ) ) {
    TRC_LigneH( fenetre, 0, vy, fenetre->w_larg );
    TRC_LigneV( fenetre, vx, 0, fenetre->w_haut );
  }

  if ( ( x >= 0 ) && ( y >= 0 ) ) {
    TRC_LigneH( fenetre, 0, y, fenetre->w_larg );
    TRC_LigneV( fenetre, x, 0, fenetre->w_haut );
  }
}

int main(void)
{
  Fenetre *fenetre;
  boolean sortie = FAUX;

  if ( BI_Initialiser( NULL, 0, NULL, NULL ) == FAUX ) {
    return -1;
  }

  /* Cration de la fentre */
  fprintf( stdout, "Fentre simple [sans infos]...\n" );
  fenetre = FEN_CreerFenetreSimple( NAME|MOVER|FULLER|SIZER, "Fen"e_CIRC"tre d'essai", NULL,
				    redessin,
				    ( void (*)(Fenetre *, int, int, int) ) NULL, /* Pas d'ascenseur */
				    clavier,
				    clic,
				    ( boolean (*)(Fenetre *, Message *) ) NULL,	/* Pas de messages */
				    rect,
				    ( boolean (*)(Fenetre *) ) NULL );
  FEN_Ouvrir(fenetre);

  do {
    sortie = BI_AttendEvenement( 0,
				 0, 0, 0, 0, 0,
				 0UL,
				 NULL, NULL, NULL, NULL, NULL, NULL );
  } while ( sortie != BI_EVENEMENT_FIN_PRG );

/*    ferme_fenetre( fenetre ); fenetre = NULL; */

  /* --- Second essai : fentre avec infos & ascenseurs */
  printf( "Fentre complte...\n" );
  fenetre = FEN_CreerFenetreSimple( NAME|MOVER|FULLER|SIZER|INFO|ASC_HORIZONTAL|ASC_VERTICAL,
				    "Fen"e_CIRC"tre avec infos", "Texte informatif",
				    redessin,
				    ( void (*)(Fenetre *, int, int, int) ) NULL, 
				    clavier,
				    clic,
				    ( boolean (*)(Fenetre *, Message *) ) NULL,
				    rect,
				    ( boolean (*)(Fenetre *) ) NULL );
  FEN_Ouvrir( fenetre );

  do {
    sortie = BI_AttendEvenement( 0,
				 0, 0, 0, 0, 0,
				 0UL,
				 NULL, NULL, NULL, NULL, NULL, NULL);
  } while ( sortie != BI_EVENEMENT_FIN_PRG );

  BI_Terminer();
  return 0;
}
