/* Bibliothèque X-Window pour les interfaces
            Routines générales
	    Compatible avec gem_gene.c (version GEM : référence)
   (C) 1998-20015, Emmanuel CURIS - Création le 30 juillet 2015

   Tout ce qui concerne la fin du programme : libération...
*/

#define _LIBRAIRIE_X_ 

#include <stdio.h>
#include <stdlib.h>

#include <errno.h>

#include "bib_interface.h"

/* ****** Variables globales limitées à ce module ************** */

/**************** Fonctions internes à ce module *****************/

extern int BI_TermineRedessins(void); /* Dans X_redessins.c */

/**************** Fonctions proprement dites ******************* */

/* ~~~~~~~~~~ Lancement, fin du programme ~~~~~~~~~~~ */

/* ------------ Fin du programme : on ferme tout proprement et tout et tout ----------- */

void BI_Terminer(void)
{
  /* On enlève le menu */
  BI_SupprimerMenu();

  /* On ferme toutes les fenêtres */
#ifdef _DEVELOPPE_
  (void) printf( "%s,%s l.%u : Fermeture des fenêtres...\n",
		 __FILE__, __FUNCTION__, __LINE__ );
#endif
  while ( BI_premiere_fenetre != NULL )
    FEN_Detruire( BI_premiere_fenetre );

  /* On termine les redessins */
  BI_TermineRedessins();

#ifdef _DEVELOPPE_
  (void) printf( "%s,%s l.%u : Fermeture des arbres...\n",
		 __FILE__, __FUNCTION__, __LINE__ );
#endif
  /* S'il y a des arbres, on les ferme */
  if ( arbres != NULL ) {
    unsigned int i;

    /* On libère les chaînes préparées pour le sélecteur de fichier, si nécessaire */
    BIX_FSEL_Terminer();

    /* On s'occupe des autres arbres */
    for ( i = 0 ; i < BI_nbr_arbres - 3 ; i++ ) { /* -3 = boîte d'alerte, -2 = sélecteur, -1 = menu, déjà faits */
      BI_LibererArbre( i );
    }
    free( arbres ); arbres = NULL;
  }
  RSC_tt_effacer(); /* On efface toutes les ressources */

  /* On libère le GC pour les objets */
  XFreeGC( BI_Display, BI_GC_objets );

  /* On ferme la station VDI associée au programme */
  VDI_ferme( _PRG_station );

  /* On libère les fontes */
  if ( BI_FonteUTF8 != NULL ) {
    XFreeFontSet( BI_Display, BI_FonteUTF8 );
  }
  XFreeFont( BI_Display, BI_Fonte );

  /* On ferme la méthode d'entrée des données */
  if ( BI_Methode_entree != NULL ) {
    XCloseIM( BI_Methode_entree );
  }

  /* On termine X */
  XCloseDisplay( BI_Display );
}
