/* Bibliothque GEM pour X-Windows
   (C) 1998, Emmanuel CURIS

   Bibliothque de base VDI (fonctions v_xxx)
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <stdlib.h>

#include "bib_interface.h"

/****** Control library ******************************************************/

void v_get_driver_info(int device_id, int info_select, unsigned char *info_string)
{
  printf(" FAIRE : v_get_driver_info(%d, %d, %p)\n", device_id, info_select, (void *)  info_string);
}

void v_opnwk(int *work_in, int *handle, int *work_out)
{
  Station_VDI *station;

  station = VDI_ouvre(work_in[0]);
  if (station == NULL) {
    (void) fprintf(stderr, "v_opnwk : erreur dans VDI_ouvre\n");
    *handle = -1;
    return;
  }

  /* On initialise */
  station->lgn_style = work_in[1];
  station->lgn_couleur = work_in[2];

  station->mk_type = work_in[3];
  station->mk_couleur = work_in[4];

  station->txt_couleur = work_in[6]; /* 5 = index de fonte */
  station->txt_angle = 0;

  station->fil_couleur = work_in[9]; /* 7, 8 = style de remplissage */

  station->txt_angle = 0;	/* Textes normaux pas dfaut */

  *handle = station->handle;

  /* On remplit work_out */
  work_out[0] = station->larg;
  work_out[1] = station->haut;
  work_out[2] = 0;
  work_out[3] = (int) (station->x_pixel * 1000.0); /* Travail en m dans la VDI, en mm dans la station*/
  work_out[4] = (int) (station->y_pixel * 1000.0);
}

void v_clswk(int handle)
{
  Station_VDI *station;

  station = quelle_station(handle);
  if (station == NULL) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "v_cslwk : handle de station VDI incorrect\n");
#endif
    return;
  }

  VDI_ferme(station);
}

void v_clrwk(int handle)
{
  Station_VDI *station;

  station = quelle_station(handle);
  if (station == NULL)
    return;

  if ( (station->type >= 0) && (station->type < 10) ) /* Station cran */
    return; /* Rien  faire */

  printf(" FAIRE : seules les stations cran sont gres pour l'instant\n");
}

void v_updwk(int handle)
{
  Station_VDI *station;

  station = quelle_station(handle);
  if (station == NULL)
    return;

  if ( (station->type >= 0) && (station->type < 10) ) /* Station cran */
    return; /* Rien  faire */

  printf(" FAIRE : seules les stations cran sont gres pour l'instant\n");
}

#ifdef AVEC_VDI_V_COMPLETE_
void v_set_app_buff    (void *address, int nparagraphs);
int  v_bez_on          (int handle);
int  v_bez_off         (int handle);
int  v_bez_qual        (int handle, int prcnt);
void v_pat_rotate      (int handle, int angle);
#endif

/****** Output library *******************************************************/

/* ~~~~ Trac d'une ligne brise ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

int v_pline(int handle, unsigned int nbr_pts, int *coords)
{
  unsigned int i;
  Station_VDI *station;
  XPoint *points;

  if ( 0 == nbr_pts ) {
    (void) fprintf(stderr, "%s, %s l.%u : Nombre de points nul\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  station = quelle_station(handle);
  if (station == NULL)
    return -2;
#ifdef _DEVELOPPE_
  if ( NULL == station->gr_contexte ) {
    printf("Couleur demande : %d -> %lu [%p]\n", station->lgn_couleur, (unsigned long) station->couleurs[ station->lgn_couleur ].pixel,
	   (void *) station->gr_contexte );
    (void) fprintf(stderr, "%s,%s l.%u : STATION sans contexte ! Station = %p / _PRG_station = %p\n",
		   __FILE__, __FUNCTION__, __LINE__, (void *) station, (void *) _PRG_station);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -50;
  }
#endif

  XSetForeground( BI_Display, station->gr_contexte, station->couleurs[ station->lgn_couleur ].pixel );

  /* On met le style de ligne */
  if ( station->lgn_style <= SOLID ) {
    XSetLineAttributes( BI_Display, station->gr_contexte, station->lgn_epaisseur, LineSolid,
			CapButt, JoinMiter );
  } else {
    char tirets[6] = {5, 5, 6, 6, 8, 8};
    int nbr_tirets = 1;

    switch (station->lgn_style)
    {
    case LDASHED:
      tirets[0] =  4;
      tirets[1] =  4;
      nbr_tirets = 2;
      break;

    case DOTTED:
      tirets[0] = 2;
      tirets[1] = 6;
      nbr_tirets = 2;
      break;

    case DASHDOT:
      tirets[0] = 8;
      tirets[1] = 3;
      tirets[2] = 2;
      tirets[3] = 3;
      nbr_tirets = 4;
      break;

    case DASH:
      tirets[0] = 8;
      tirets[1] = 8;
      nbr_tirets = 2;
      break;

    case DASHDOTDOT:
      tirets[0] = 5;
      tirets[1] = 3;
      tirets[2] = 2;
      tirets[3] = 2;
      tirets[4] = 1;
      tirets[5] = 3;
      nbr_tirets = 6;
      break;

#ifdef _DEVELOPPE_
    default:
      (void) fprintf(stderr, "%s, %s l.%u : Code de style inconnu = %d\n",
		     __FILE__, __FUNCTION__, __LINE__, station->lgn_style);
      FIN_ANTICIPEE( FIN_ERREUR_COSMETIQUE );
      return -51;
#endif
    }

    XSetDashes(BI_Display, station->gr_contexte, 0, tirets, nbr_tirets);
    XSetLineAttributes( BI_Display, station->gr_contexte, station->lgn_epaisseur, LineOnOffDash,
			CapButt, JoinMiter );
  }

#ifdef _DEVELOPPE_ /* Serveur XFree 4.0 / Linux n'aime pas du tout les coordonnes absurdes :
		      Bloque absolument tout ! Mais ralentit -> vrification en dveloppement seulement */
  if ( (coords[ 0 ] < 0) || (coords[ 0 ] > 32000) ||
       (coords[ 1 ] < 0) || (coords[ 1 ] > 32000) ) {
    (void) fprintf(stderr, "%s, %s l.%u : premier point absurde (X=%d, Y=%d) [%u points]\n",
		   __FILE__, __FUNCTION__, __LINE__, coords[ 0 ], coords[ 1 ], nbr_pts);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -48;
  }
#endif
  if (nbr_pts == 1) { /* Un seul point */
    XDrawPoint(BI_Display, BI_Fen_redessin, station->gr_contexte, coords[0], coords[1]);
    return 0;
  }

#ifdef _DEVELOPPE_
  if ( (coords[ 2 ] < 0) || (coords[ 2 ] > 32000) ||
       (coords[ 3 ] < 0) || (coords[ 3 ] > 32000) ) {
    (void) fprintf(stderr, "%s, %s l.%u : deuxime point absurde (X=%d, Y=%d) [%u points]\n",
		   __FILE__, __FUNCTION__, __LINE__, coords[ 2 ], coords[ 3 ], nbr_pts);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -47;
  }
#endif
  if (nbr_pts == 2) { /* Deux points = une droite */
    XDrawLine(BI_Display, BI_Fen_redessin, station->gr_contexte,
	      coords[0], coords[1], coords[2], coords[3]);
    return 0;
  }

  /* Plus de points : il faut faire une conversion pour XDrawLines... */

  points = (XPoint *) calloc(nbr_pts, sizeof(XPoint));
  if (points == NULL) {
    (void) fprintf(stderr, "v_pline : mmoire insuffisante, table des points\n");
    return -10;
  }

  for (i = 0 ; i < nbr_pts ; i++) {
#ifdef _DEVELOPPE_
    if ( (coords[ 2*i ] < 0) || (coords[ 2*i ] > 32000) ||
	 (coords[ 2*i + 1 ] < 0) || (coords[ 2*i + 1 ] > 32000) ) {
      (void) fprintf(stderr, "%s, %s l.%u : %de point absurde (X=%d, Y=%d) [%u points]\n",
		     __FILE__, __FUNCTION__, __LINE__, i, coords[ 2 * i ], coords[ 2*i + 1 ], nbr_pts);
      free( points );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return -46;
    }
#endif
    points[i].x = (short) coords[2*i];
    points[i].y = (short) coords[2*i + 1];
  }

  XDrawLines(BI_Display, BI_Fen_redessin, station->gr_contexte, points, nbr_pts, CoordModeOrigin);

  free(points);

  /* Tout s'est bien pass */
  return 0;
}

/* ----------- Polymark -------- */

void v_pmarker(int handle, unsigned int nbr_pts, int *xy)
{
  unsigned int i;
  Station_VDI *station;

  station = quelle_station(handle);
  if (station == NULL)
    return;

  if ( 0 == station->mk_type ) return; /* Pas de marque dfinie ! */

  XSetForeground(BI_Display, station->gr_contexte, BI_Couleurs[station->mk_couleur].pixel);
  XSetLineAttributes(BI_Display, station->gr_contexte, 0, LineSolid, CapButt, JoinMiter);

  for (i = 0 ; i < nbr_pts ; i++) {
    BIX_TracerMarque( BI_Fen_redessin, station->gr_contexte, xy[ 2 * i ], xy[ 2 * i + 1 ], station->mk_type, station->mk_taille );
  }
}

#ifdef AVEC_VDI_V_COMPLETE_
void v_gtext(int handle, int x, int y, const char *texte)
{
  int x_texte, y_texte;
  unsigned int larg_texte, haut_texte;
  int angle;
  unsigned long lg_texte;
  GC gc_pixmap;
  Pixmap pixmap_texte, px_texte_tourne;
  Station_VDI *station;
  XGCValues parametres_GC;
  XImage *img_texte;

  if (texte == NULL) return;

  station = quelle_station(handle);
  if (station == NULL) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucune station VDI\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  XSetBackground( BI_Display, station->gr_contexte, BI_Couleurs[ BLANC ].pixel ); /* fond blanc sur atari */
  XSetForeground( BI_Display, station->gr_contexte, BI_Couleurs[ station->txt_couleur ].pixel );

  /* Quel est l'angle demand ? */
  angle = station->txt_angle;

  /* Dimensions du texte */
  lg_texte = strlen( texte );
  larg_texte = XTextWidth( BI_Fonte, texte, lg_texte );
  haut_texte = BI_Fonte->max_bounds.ascent + BI_Fonte->max_bounds.descent;

  if ( 0 == angle ) {
    /* Position du texte */
    if (station->txt_just == ALI_CENTER) /* On centre sur x */
      x_texte  = x - larg_texte / 2;
    else if (station->txt_just == ALI_RIGHT) /* On justifie  droite = fin du texte en x */
      x_texte = x - larg_texte;
    else
      x_texte = x; /* Justif.  gauche : dbut du texte en x */

    if ( station->txt_vert == ALI_TOP )
      y_texte = y + BI_Fonte->max_bounds.ascent;
    else if ( station->txt_vert == ALI_HALF )
      y_texte = y + (BI_Fonte->max_bounds.ascent - BI_Fonte->max_bounds.descent) / 2;
    else if (station->txt_vert == ALI_BOTTOM )
      y_texte = y - BI_Fonte->max_bounds.descent;
    else
      y_texte = y; /* Pour voir */

    XDrawImageString( BI_Display, BI_Fen_redessin, station->gr_contexte,
		      x_texte, y_texte, texte, lg_texte);
    return;
  }

  /* Si on est l : texte tourn */
  /* 1. Crer un pixmap qui contient le texte non-tourn */
  pixmap_texte = XCreatePixmap( BI_Display, BI_Fen_redessin, larg_texte, haut_texte, 1 );

  /* 2. Crer un GC associ pour pouvoir l'utiliser */
  parametres_GC.function   = GXcopy;
  parametres_GC.foreground = 1;
  parametres_GC.background = 0;
  gc_pixmap = XCreateGC( BI_Display, pixmap_texte, GCFunction | GCForeground | GCBackground, &parametres_GC );
  XSetFont( BI_Display, gc_pixmap, BI_Fonte->fid );

  /* 2. Dessin du texte dans ce pixmap [en noir] */
  XDrawImageString( BI_Display, pixmap_texte, gc_pixmap, 0, BI_Fonte->max_bounds.ascent, texte, lg_texte );

  /* 3. On en fait une image */
  img_texte = XGetImage( BI_Display, pixmap_texte, 0, 0, larg_texte, haut_texte, AllPlanes, XYPixmap );

  /* Le pixmap du texte original est donc inutile ! */
  XFreePixmap( BI_Display, pixmap_texte );

  /* 4. Rotation */
  if ( 900 == angle ) {
    unsigned int i_x, i_y;

    /* O se trouve le coin HG du rectangle du texte dans la fentre ? */
    if (station->txt_just == ALI_CENTER) /* On centre sur x */
      y_texte = y - larg_texte / 2;
    else if (station->txt_just == ALI_RIGHT) /* On justifie  droite = fin du texte en x */
      y_texte = y - larg_texte;
    else
      y_texte = y;

    if ( station->txt_vert == ALI_TOP )
      x_texte = x - BI_Fonte->max_bounds.ascent;
    else if ( station->txt_vert == ALI_HALF )
      x_texte = x - (BI_Fonte->max_bounds.ascent - BI_Fonte->max_bounds.descent) / 2;
    else if (station->txt_vert == ALI_BOTTOM )
      x_texte = x - BI_Fonte->max_bounds.descent;
    else
      x_texte = x; /* Pour voir */

    /* Le pixmap qui va contenir le texte tourn */
    px_texte_tourne = XCreatePixmap( BI_Display, BI_Fen_redessin, haut_texte, larg_texte, 1 );

/*     OPTION_A_FAIRE( "Vrifier s'il n'existe pas qq chose de + efficace pour la rotation" ); */

     for ( i_x = 0 ; i_x < larg_texte ; i_x++ ) {
       for ( i_y = 0 ; i_y < haut_texte ; i_y++ ) {
 	XPutImage( BI_Display, px_texte_tourne, gc_pixmap, img_texte,
 		   i_x, i_y, i_y, larg_texte - i_x - 1, 1, 1 );
       }
     } 

     /* Affichage sur la fentre */
     XCopyPlane( BI_Display, px_texte_tourne, BI_Fen_redessin, station->gr_contexte, 0, 0, haut_texte, larg_texte,
		 x_texte, y_texte, 1 );

    /* Le pixmap du texte tourn n'est plus utile */
    XFreePixmap( BI_Display, px_texte_tourne );
  } else {
#ifdef _DEVELOPPE_
    OPTION_A_FAIRE( "Rotation pour un angle autre que 0 ou 90 degrs" );
    fprintf( stderr, "\t angle demand = %g degrs\n", angle / 10.0 );
#endif
  }

  /* Plus besoin des pixmaps ni des images */
  XDestroyImage( img_texte );
  XFreeGC( BI_Display, gc_pixmap );
}

void v_etext           (int handle, int x, int y, unsigned char *string, int *offsets);
void v_fillarea        (int handle, int count, int *xy);
void v_cellarray       (int handle, int *pxyarray, int row_length,
			int el_used, int num_rows, int wrt_mode, int *colarray);
#endif

void v_bar(int handle, int *coords)
{
  Station_VDI *station;
  int x, y;
  unsigned int larg, haut;

  station = quelle_station(handle);
  if (station == NULL)
    return;

#ifdef _DEVELOPPE_
  if ( station != _PRG_station ) {
/*     printf("Couleur choisie : %lx [%d = %lx]\n", station->couleurs[station->fil_couleur].pixel, station->fil_couleur, */
/* 	   BI_Couleurs[ station->fil_couleur ].pixel ); */
  }
#endif
  XSetForeground(BI_Display, station->gr_contexte, station->couleurs[station->fil_couleur].pixel);

  if (coords[0] > coords[2]) {
    x = coords[2];
    larg = (coords[0] - coords[2]) + 1;
  } else {
    x = coords[0];
    larg = (coords[2] - coords[0]) + 1;
  }

  if (coords[1] > coords[3]) {
    y = coords[3];
    haut = (coords[1] - coords[3]) + 1;
  } else {
    y = coords[1];
    haut = (coords[3] - coords[1]) + 1;
  }

  XFillRectangle(BI_Display, BI_Fen_redessin, station->gr_contexte, x, y, larg, haut);
}

#ifdef AVEC_VDI_V_COMPLETE_
void v_arc             (int handle, int x, int y, int radius, int begang, int endang);
#endif

void v_pieslice(int handle, int x, int y, int rayon, int angle_debut, int angle_fin)
{
  Station_VDI *station;

  station = quelle_station(handle);
  if (station == NULL) {
    (void) fprintf(stderr, "v_pieslice : Station VDI incorrecte [handle = %d]\n", handle);
    return;
  }

  XSetArcMode( BI_Display, station->gr_contexte, ArcPieSlice ); /* Pour tre comme la VDI... */
  XSetForeground(BI_Display, station->gr_contexte, BI_Couleurs[ station->fil_couleur ].pixel);

  XFillArc(BI_Display, BI_Fen_redessin, station->gr_contexte, x - rayon, y - rayon,
	   rayon << 1, rayon << 1, angle_debut * 64, angle_fin * 64);
}

void v_circle(int handle, int x, int y, int rayon)
{
  Station_VDI *station;

  station = quelle_station(handle);
  if (station == NULL)
    return;

  XSetForeground(BI_Display, station->gr_contexte, BI_Couleurs[station->lgn_couleur].pixel);

  XDrawArc(BI_Display, BI_Fen_redessin, station->gr_contexte, x - rayon, y - rayon,
	   rayon << 1, rayon << 1, 0, 360 * 64);
}

void v_ellipse(int handle, int x, int y, int rayon_x, int rayon_y)
{
  Station_VDI *station;

  station = quelle_station(handle);
  if (station == NULL)
    return;

  XSetForeground(BI_Display, station->gr_contexte, BI_Couleurs[ station->lgn_couleur ].pixel);

  XDrawArc(BI_Display, BI_Fen_redessin, station->gr_contexte, x - rayon_x, y - rayon_y,
	   rayon_x << 1, rayon_y << 1, 0, 360 * 64);
}

#ifdef AVEC_VDI_V_COMPLETE_
void v_ellarc          (int handle, int x, int y, int xradius, int yradius,
			int begang, int endang);
void v_ellpie          (int handle, int x, int y, int xradius,
			int yradius, int begang, int endang);
void v_rbox            (int handle, int *xyarray);
void v_rfbox           (int handle, int *xyarray);
void v_justified       (int handle, int x, int y, char *string,
			int length, int word_space, int char_space);
void v_contourfill     (int handle, int x, int y, int index);
void v_bez             (int handle, int count, int *xyarr,
			unsigned char *bezarr, int *minmax, int *npts, int *nmove);
void v_bez_fill        (int handle, int count, int *xyarr,
                          unsigned char *bezarr, int *minmax, int *npts, int *nmove);
void v_get_pixel       (int handle, int x, int y, int *pel, int *index);
void v_show_c          (int handle, int reset);
void v_hide_c          (int handle);
void v_exit_cur        (int handle);
void v_enter_cur       (int handle);
void v_curup           (int handle);
void v_curdown         (int handle);
void v_curright        (int handle);
void v_curleft         (int handle);
void v_curhome         (int handle);
void v_eeos            (int handle);
void v_eeol            (int handle);
void v_curtext         (int handle, char *string);
void v_rvon            (int handle);
void v_rvoff           (int handle);
void v_hardcopy        (int handle);
void v_dspcur          (int handle, int x, int y);
void v_rmcur           (int handle);
void v_form_adv        (int handle);
void v_output_window   (int handle, int *xyarray);
void v_clear_disp_list (int handle);
void v_bit_image       (int handle, char *filename,
                          int aspect, int x_scale, int y_scale,
                          int h_align, int v_align, int *xyarray);
void v_alpha_text      (int handle, char *string);
void v_sound           (int handle, int frequency, int duration);
#endif

void v_meta_extents(int handle, int min_x, int min_y, int max_x, int max_y)
{
  (void) fprintf(stderr, "Stations Metafile  pas supportes dans cette version\n");
  (void) fprintf(stderr, "  v_meta_extents(%d, %d, %d, %d, %d)\n", handle, min_x, min_y, max_x, max_y);
}

void v_write_meta(int handle, int num_intin, int *intin, int num_ptsin, int *ptsin)
{
  (void) fprintf(stderr, "Stations Metafile  pas supportes dans cette version\n");
  (void) fprintf(stderr, "  v_write_meta(%d, %d, %p, %d, %p)\n",
		 handle, num_intin, (void *) intin, num_ptsin, (void *) ptsin);
}

#ifdef AVEC_VDI_V_COMPLETE_
void v_copies          (int handle, int count);
void v_orient          (int handle, int orientation);
void v_tray            (int handle, int tray);
int  v_xbit_image      (int handle, char *filename, int aspect,
			int x_scale, int y_scale,
			int h_align, int v_align, int rotate,
			int background, int foreground, int *xy);
void v_setrgbi         (int handle, int primtype, int r, int g, int b, int i);
void v_topbot          (int handle, int height,
			int *char_width, int *char_height,
			int *cell_width, int *cell_height);
void v_offset          (int handle, int offset);
void v_fontinit        (int handle, int fh_high, int fh_low);
void v_escape2000      (int times);
#endif
