/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1996 - 1998
   Dernière modification : 30 octobre 1998

   Les messages d'erreur généraux (+ quelques autres petites choses allant avec)
   Les fonctions de reconnaissance des faux flottants (qui conduiraient à des erreurs)

   30 octobre 1998
     * Rappatrié des messages d'erreur généraux qui n'avaient pas leur place dans la lib. d'interface
     * Intégré les codes d'erreur (avant dans ALERTES.H, maintenant inutile)

    6 janvier 1998 ; 16, 25 mars 1998
     * Textes des messages d'alerte fait dans Interface : ALERTES.RSC - plus sympa
     * Inclus "erreurs.h" (fichier associé...)
     * Modifié is_nan() : utilise 20 octets de moins...

   24, 25 & 27 novembre 1997 ; 15 décembre 1997
     * Fonction is_nan() créée et fonctionne correctement
     * Ajouté message d'erreur NON_SAUVE
     * Ajouté la fonction is_inf() : renvoie VRAI si un double vaut +Inf ou -Inf
*/

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "systeme.h"

#include "erreurs.txt"

#if !defined(A_FP_CLASS_) && !defined(A_FPCLASS_) && !defined(A_IS_INF_) && !defined(A_CLASS_)
    /* Aucune fonction de test par défaut */

/* ~~~~~~~~~~~~~~~ Fonctions de test évitant bien des erreurs... ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* --------------- Test de l'infini : renvoie VRAI si x = +Inf ou x = -Inf, FAUX sinon ------- */

int is_inf(double x)
{
  double p_infini = 1.0/0.0, m_infini = -1.0/0.0;

  if (x == p_infini) return VRAI; /* x vaut + l'infini */
  if (x == m_infini) return VRAI; /* x vaut - l'infini */

  return FAUX;
}

/* --------------- Test de Not a Number : renvoie VRAI si x = Nan, FAUX sinon ---------------- */

static char tst_nan[20]; /* On suppose 20 caractères au plus dans un flottant écrit par sprintf */

int is_nan(double x)
{
  sprintf( tst_nan, "%.1g", x );
  if ( 'N' == tst_nan[ 0 ] ) { /* Nan écrit NaN, NaNQ ou NaNS dans une chaîne par sprintf... */
    return VRAI;
  }

  return FAUX;
}

#endif /* !defined(A_FP_CLASS_) && !defined(A_IS_INF_) */

/* -------- Fonctions spécialisées dans certains messages -------- */

inline void ERR_MemoireInsuffisante(const char *fichier, const char *fonction, unsigned long num_ligne, unsigned long taille, const char *message)
{
#ifdef _DEVELOPPE_
  (void) fprintf(stderr, "%s, %s l.%lu : Impossible de réserver la mémoire [%lu octets]\n", fichier, fonction, num_ligne, taille);
#endif

  if ( message != NULL ) {
    BI_ERR_annuler( "%s [%lu octets ; module %s, fonction %s, ligne %lu]", message, taille, fichier, fonction, num_ligne );
  } else {
    BI_ERR_annuler( "Impossible de r"e_AIGU"server %lu octets (module %s, fonction %s, ligne %lu)", taille, fichier, fonction, num_ligne );
  }
}

/* ~~~~~~~~~~~~~~~ Messages d'erreur spécifiques du programme ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

#define NOMBRE_MESSAGES 27

#ifdef _ENGLISH_ /* Version anglaise */

static char *alertes[] = 
{
  "[1][---- Operate on a graph ----| "
     "|  This graph has some double"
     "|points (same X): this opera-"
     "|tion is not possible.][ Cancel ]",

  "[1][------ Computing error ------"
     "|  Doing this computing needs"
     "|a division by zero!|%.20s|%.20s][ Go on ]",

  "[1][------- Absurd value -------| "
     "|   The proposed value doesn't"
     "|have any meaning for the use"
     "|one wants to do][ Cancel ]",

  "[1][---- Operate on a graph ----| "
     "| This graph is not sorted,"
     "|which make the asked opera-"
     "|tion impossible][ Cancel ]",

  "[1][----- Too few points ----| "
     "| Doing such a calculation"
     "|would create a graph with only"
     "|one point, which is impossible][ Cancel ]",

  /*   v----------------------------v */
  "[1][-- Incoh"e_AIGU"rence des graphes ---| "
     "| Les graphes choisis n'ont pas"
     "|tous le m"e_CIRC"me nombre de points."
     "|  Op"e_AIGU"ration impossible.][Annuler]",

  "[1][--- Incoh"e_AIGU"rence des graphes ---| "
     "|  Les graphes choisis sont dis-"
     "|joints (pas de X commun)."
     "|  Op"e_AIGU"ration impossible.][Abandon]",

  "[1][------ Protected graphe ------| "
     "|       This graphe is used in"
     "|another computation and can't"
     "|be modified.][ Cancel ]",

  "[3][------ EXAFS extraction ------| "
     "| No defined atomic absorption!"
     "|The signal normalization is"
     "|not possible.][Continue]",

  /*   v----------------------------v */
  "[2][---- Quitter ce module ----| "
     "|   Que  faire en quittant ce"
     "|module : effectuer le calcul"
     "|ou fermer sans calculer ?][Calculer|Fermer|Annuler]",

  "[2][----- Module d"e_AIGU"j"a_GRAVE" actif -----"
     "|   Ce module est d"e_AIGU"j"a_GRAVE" utilis"e_AIGU"."
     "|Faut-il juste l'activer ou le"
     "|r"e_AIGU"initialiser  avec le graphe"
     "|courant ?][Activer|R"e_AIGU"initialiser|Annuler]",

  "[2][- "E_AIGU"limination d'une aberrance -| "
     "|      Ce spectre contient-il un"
     "|point aberrant que vous vouliez"
     "|"o_CIRC"ter ?][Aucun|Oui]",

  "[1][---- Param"e_AIGU"trage incorrect ----| "
     "|   Les bornes indiqu"e_AIGU"es sont"
     "|incorrectes (invers"e_AIGU"es ou se"
     "|r"e_AIGU"duisant "a_GRAVE" un seul point).][Abandon]",

  "[1][----- Param"e_GRAVE"tre incorrect -----| "
     "|  Le pas indiqu"e_AIGU" pour le calcul"
     "|est incorrect  (n"e_AIGU"gatif, nul ou"
     "|sup"e_AIGU"rieur "a_GRAVE" l'intervalle).][Abandon]",

  "[1][---- Calculs sur un graphe ----| "
     "|  Le graphe indiqu"e_AIGU" ne contient"
     "|pas assez de points pour que le"
     "|calcul soit pertinent.][Abandon]",

  "[1][---- Calculs sur un graphe ----| "
     "|  Ce type de calcul n'a de sens"
     "|que sur un graphe complexe.][Abandon]",

  "[1][---- Calculs sur un graphe ----| "
     "|  Ce type de calculs ne peut se"
     "|faire que sur un graphe conte-"
     "|nant 2"n_EXPOSANT" points...][Abandon]",

  "[3][------ Effacer un graphe ------| "
     "|  Ce graphe n'a pas "e_AIGU"t"e_AIGU" sauv"e_AIGU" !"
     "|Voulez-vous le sauver, l'effa-"
     "|cer ou annuler l'op"e_AIGU"ration ?][Sauver|Effacer|Annuler]",

  "[1][---- Calculs sur un graphe ----| "
     "|  Cette op"e_AIGU"ration ne peut pas"
     "|"e_CIRC"tre effectu"e_AIGU"e  sur un graphe"
     "|complexe.][Abandon]",

  /*  v----------------------------v */
  "[3][--- FATAL INTERNAL ERROR ---\n "
     "La liste des graphes en m"e_AIGU"moire"
     "n'est plus coh"e_AIGU"rente !\n Sauvez"
     "tout et quittez le programme.][Compris]",

  /*  v----------------------------v */
  "[1][----- Liste de graphes -----| "
     "|   Ce graphe se trouve d"e_AIGU"j"a_GRAVE
     "|dans la liste !][Abandon]",
     
  /*  v----------------------------v */
  "[3][----- Erreur survenue ------| "
     "|   Une erreur est survenue lors"
     "|des op"e_AIGU"rations demand"e_AIGU"es."
     "|  Origine de l'erreur inconnue.][Continuer]",

  /*  v----------------------------v */
  "[2][ Num"e_AIGU"ro de version incorrect.\n"
     " Le fichier a "e_AIGU"t"e_AIGU" cr"e_AIGU e_AIGU" par la"
     "version %.1f, mais la version"
     "du programme est la %.1f.][Annuler]",

  /*  v----------------------------v */
  "[3][ Format de fichier incorrect.\n"
     "Ce fichier n'a pas "e_AIGU"t"e_AIGU" cr"e_AIGU e_AIGU
     "par LASE sur cet ordinateur."
     "Essayez « Importer »][Compris]",

  /*   v----------------------------v */
  "[3][----- Point inexistant -----"
     "|Ce point n'appartient pas au"
     "|spectre. X = %f,"
     "|Xmin = %f ; Xmax = %f][ Bien ]",

  /*   v----------------------------v */
  "[3][Taille des doubles incorrecte"
     "| Dans ce fichier, les doubles"
     "|n'ont pas la m"e_CIRC"me taille que"
     "|sur cet ordinateur.][Annuler]",

  /*   v----------------------------v */
  "[3][--- Aucun graphe trouv"e_AIGU" ! ---"
     "|  Aucun graphe du type voulu"
     "|(%.28s)"
     "|n'a "e_AIGU"t"e_AIGU" trouv"e_AIGU".][ Annuler ]"
};

#else

static const char *alertes[] = 
{
  "[1][---- Traitement d'un graphe ---\n"
     "  Ce graphe contient des points"
     " doubles (m"e_CIRC"me X) : cette op"e_AIGU"ration est impossible.][Abandon]",

  "[1][------ Erreur de calcul ------\n"
     "Ce calcul conduit "a_GRAVE" effectuer une division par z"e_AIGU"ro"ESPACE_INSECABLE"!\n"
     "%.20s\n"
     "%.20s][Annuler]",

  "[1][------- Valeur absurde -------| "
     "|   La valeur propos"e_AIGU"e est tout"
     "|"a_GRAVE" fait incompatible avec l'uti-"
     "|lisation que l'on doit en faire][Annuler]",

  "[1][---- Traitement du graphe ----| "
     "|   Ce graphe n'a pas "e_AIGU"t"e_AIGU" tri"e_AIGU", "
     "|ce qui interdit la r"e_AIGU"alisation "
     "|de l'op"e_AIGU"ration demand"e_AIGU"e.][Abandon]",

  "[1][----- Trop peu de points ----| "
     "|   Un tel calcul conduirait "a_GRAVE
     "|un spectre  contenant un seul"
     "|point, ce qui est impossible.][Abandon]",

  "[1][-- Incoh"e_AIGU"rence des graphes ---| "
     "| Les graphes choisis n'ont pas"
     "|tous le m"e_CIRC"me nombre de points."
     "|  Op"e_AIGU"ration impossible.][Annuler]",

  "[1][--- Incoh"e_AIGU"rence des graphes ---| "
     "|  Les graphes choisis sont dis-"
     "|joints (pas de X commun)."
     "|  Op"e_AIGU"ration impossible.][Abandon]",

  "[1][------- Graphe prot"e_AIGU"g"e_AIGU" -------| "
     "|  Ce graphe est utilis"e_AIGU" dans un"
     "|calcul et ne peut "e_CIRC"tre modifi"e_AIGU".][Abandon]",

  /* --- NORM_IMPOSSIBLE --- */
  "[3][---- Extraction de l'EXAFS ----| "
     "|    Pas d'absorption atomique"
     "|d"e_AIGU"finie ! La normalisation du"
     "|signal est impossible.][Continuer]",

  /* --- FIN_MODULE --- */
  "[2][---- Quitter ce module ----| "
     "|   Que  faire en quittant ce"
     "|module : effectuer le calcul"
     "|ou fermer sans calculer ?][Calculer|Fermer|Annuler]",

  "[2][----- Module d"e_AIGU"j"a_GRAVE" actif -----"
     "|   Ce module est d"e_AIGU"j"a_GRAVE" utilis"e_AIGU"."
     "|Faut-il juste l'activer ou le"
     "|r"e_AIGU"initialiser  avec le graphe"
     "|courant ?][Activer|R"e_AIGU"initialiser|Annuler]",

  "[2][- "E_AIGU"limination d'une aberrance -| "
     "|      Ce spectre contient-il un"
     "|point aberrant que vous vouliez"
     "|"o_CIRC"ter ?][Aucun|Oui]",

  "[1][---- Param"e_AIGU"trage incorrect ----| "
     "|   Les bornes indiqu"e_AIGU"es sont"
     "|incorrectes (invers"e_AIGU"es ou se"
     "|r"e_AIGU"duisant "a_GRAVE" un seul point).][Abandon]",

  "[1][----- Param"e_GRAVE"tre incorrect -----| "
     "|  Le pas indiqu"e_AIGU" pour le calcul"
     "|est incorrect  (n"e_AIGU"gatif, nul ou"
     "|sup"e_AIGU"rieur "a_GRAVE" l'intervalle).][Abandon]",

  "[1][---- Calculs sur un graphe ----| "
     "|  Le graphe indiqu"e_AIGU" ne contient"
     "|pas assez de points pour que le"
     "|calcul soit pertinent.][Abandon]",

  "[1][---- Calculs sur un graphe ----| "
     "|  Ce type de calcul n'a de sens"
     "|que sur un graphe complexe.][Abandon]",

  "[1][---- Calculs sur un graphe ----| "
     "|  Ce type de calculs ne peut se"
     "|faire que sur un graphe conte-"
     "|nant 2"n_EXPOSANT" points...][Abandon]",

  "[3][------ Effacer un graphe -----| "
     "| Ce graphe n'a pas "e_AIGU"t"e_AIGU" sauv"e_AIGU" !"
     "|Voulez-vous le sauver, l'effa-"
     "|cer ou annuler l'op"e_AIGU"ration ?][Sauver|Effacer|Annuler]",

  "[1][---- Calculs sur un graphe ----| "
     "|  Cette op"e_AIGU"ration ne peut pas"
     "|"e_CIRC"tre effectu"e_AIGU"e  sur un graphe"
     "|complexe.][Abandon]",

  "[3][--- ERREUR INTERNE GRAVE ---| "
     "|La liste des graphes en m"e_AIGU"moire"
     "|n'est plus coh"e_AIGU"rente ! Sauvez"
     "|tout et quittez le programme.][Compris]",

  "[1][----- Liste de graphes -----| "
     "|   Ce graphe se trouve d"e_AIGU"j"a_GRAVE
     "|dans la liste !][Abandon]",
     
  "[3][----- Erreur survenue ------| "
     "|   Une erreur est survenue lors"
     "|des op"e_AIGU"rations demand"e_AIGU"es."
     "|  Origine de l'erreur inconnue.][Continuer]",

  "[2][ Num"e_AIGU"ro de version incorrect.\n"
     " Le fichier a "e_AIGU"t"e_AIGU" cr"e_AIGU e_AIGU" par la"
     "version %.1f, mais la version"
     "du programme est la %.1f.][Annuler]",

  "[3][Format de fichier incorrect.\n"
     "Ce fichier n'a pas "e_AIGU"t"e_AIGU" cr"e_AIGU e_AIGU" par LASE sur cet ordinateur.\n"
     "Essayez "GUILLEMET_OUVRANT" Importer "GUILLEMET_FERMANT"][Compris]",

  "[3][----- Point inexistant -----"
     "|Ce point n'appartient pas au"
     "|spectre. X = %f,"
     "|Xmin = %f ; Xmax = %f][ Bien ]",

  "[3][Taille des doubles incorrecte"
     "| Dans ce fichier, les doubles"
     "|n'ont pas la m"e_CIRC"me taille que"
     "|sur cet ordinateur.][Annuler]",

  /*   v----------------------------v */
  "[3][--- Aucun graphe trouv"e_AIGU" ! ---"
     "|  Aucun graphe du type voulu"
     "|(%.28s)"
     "|n'a "e_AIGU"t"e_AIGU" trouv"e_AIGU".][ Annuler ]"
};

#endif /* _ENGLISH_ */

extern char vrai_message[]; /* Définie dans variables.c, comme variable globale */

/* --------------- Routines d'affichage ------------------------------------------------------ */

int EXF_ERREUR(unsigned int code_erreur, ...)
{
  va_list params;
  double d_1, d_2, d_3;
  char *cp_1, *cp_2;
  int retour;

  if ( code_erreur >= NOMBRE_MESSAGES )
    return SYS_ERREUR( ERREUR_INCONNUE, code_erreur, "LASE, g"e_AIGU"n"e_AIGU"ral" );

  va_start( params, code_erreur );

  switch ( code_erreur )
  {
  case TROP_VIEUX: /* Deux numéros de version, en entiers à diviser par 100 */
    d_1 = ( (double) va_arg(params, int) ) / 10.0;
    d_2 = ( (double) va_arg(params, int) ) / 10.0;
    sprintf( vrai_message, alertes[ TROP_VIEUX ], d_1, d_2 );
    retour = form_alert(1, vrai_message);
    break;

  case POINT_INEXISTANT: /* Trois doubles : X et bornes du graphe */
    d_1 = va_arg(params, double);
    d_2 = va_arg(params, double);
    d_3 = va_arg(params, double);
    sprintf(vrai_message, alertes[POINT_INEXISTANT], d_1, d_2, d_3);
    retour = form_alert(1, vrai_message);
    break;

  case AUCUN_GRAPHE: /* Une chaîne : type de graphe cherché */
    cp_1 = va_arg(params, char *);
    sprintf(vrai_message, alertes[AUCUN_GRAPHE], cp_1);
    retour = form_alert(1, vrai_message);
    break;

  case DIV_ZERO: /* Deux chaînes : raison et conséquence */
    cp_1 = va_arg(params, char *);
    cp_2 = va_arg(params, char *);
    sprintf(vrai_message, alertes[DIV_ZERO], cp_1, cp_2);
    retour = form_alert(1, vrai_message);
    break;

  default: /* Messages sans paramètres */
    retour = form_alert(1, alertes[code_erreur]);
  }
  
  va_end(params);
  return retour;
}

/* -------- Messages d'erreur système -------- */

int SYS_ERREUR(unsigned int code_erreur, ...)
{
  va_list params;
  const char *ch_1, *ch_2 = NULL, *ch_3;
  unsigned int u_int;
  int retour = 0;
  FILE *fichier;

  if ( code_erreur >= NOMBRE_MESSAGES )
    return SYS_ERREUR(ERREUR_INCONNUE, code_erreur, "Interface GEM");

  va_start(params, code_erreur);

  switch (code_erreur)
  {
  case MANQUE_MEMOIRE:  /* Paramètres = 3 chaînes */
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "Manque de mémoire, raison : %s\n", strerror( errno ) );
#endif
    ch_1 = va_arg(params, const char *); /* Le module   */     
    ch_2 = va_arg(params, const char *), /* La fonction */
    ch_3 = va_arg(params, const char *); /* La ligne    */

    /* Note : obligé car ordre d'évaluation des paramètres non défini ->
       sprintf(,,va_arg, va_arg, va_arg) peut inverser les paramètres ! */
    retour = BI_ERR_alerte( NULL, &(textes_erreurs[ MANQUE_MEMOIRE ]), VRAI, ch_1, ch_2, ch_3 );
    break;

  case QUITTER_PROGRAMME: /* Aucun paramètre à utiliser */
  case TROP_FENETRES:
  case FEN_INEXISTANTE:
  case STATION_REFUSEE:
  case TROP_FICHIERS:
    retour = BI_ERR_alerte(NULL, &(textes_erreurs[code_erreur]), VRAI);
    break;

  case ERREUR_LECTURE:  /* Paramètres = 1 chaîne + errno */
  case ERREUR_ECRITURE:
    ch_1 = va_arg(params, const char *); /* Nom du fichier    */
    fichier = va_arg(params, FILE *); /* Fichier considéré, s'il existe */
    ch_2 = va_arg(params, const char *); /* Raison de l'échec */
    if (ch_1 == NULL) ch_1 = "Nom inconnu";
    if (ch_2 == NULL) {
      if ( (fichier) && ( feof(fichier) ) )
	ch_2 = "Fin de fichier atteinte";
      else
	ch_2 = txt_errno(errno); /* Raison de l'échec */
    }
#ifdef _DEVELOPPE_
    fprintf(stderr, "[SYS_ERREUR] Erreur avec le fichier '%s' : %s\n", ch_1, ch_2);
#endif
    retour = BI_ERR_alerte(NULL, &(textes_erreurs[code_erreur]), VRAI, ch_1, ch_2);
    break;

  case ERREUR_INCONNUE: /* Paramètres = 1 uint + 1 chaîne */
    u_int = va_arg(params, unsigned int); /* Numéro */
    ch_1  = va_arg(params, const char *); /* Module */
    retour = BI_ERR_alerte(NULL, &(textes_erreurs[code_erreur]), VRAI, u_int, ch_1);
    break;

  case FEN_OUVERTE:     /* Paramètre = 1 handle de fenêtre */
    retour = BI_ERR_alerte(NULL, &(textes_erreurs[code_erreur]), VRAI);
    break;
  }

  va_end(params);
  return retour;
}
