/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 2003 Emmanuel CURIS
   Cration : le 7 mars 2003 (Budapest)

   Rcupration des donnes exprimentales pour obtenir  total

   Fichiers enregistrs au NSLS
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
#include "serie.h"

#include "import_xas.h"

/* #include "ext_nsls.txt" */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int NSLS_PreparerSpectres(const char *nom_fichier, const char *nom_image,
				 EXT_ListeGraphes *graphes, struct serie *serie, unsigned long nbr_points);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int NSLS_LireSpectre(const char *nom, FILE *fichier, Infos_Extraction *infos, EXT_ListeGraphes *graphes, struct serie *serie)
{
  char *nom_image;
  unsigned long nbr_points, N;
  int c = 0, n_lu;
  double x_deb, x_fin, pas;
  double *x;

  if ( ( NULL == fichier ) || ( NULL == infos ) || ( NULL == graphes ) ) return -1;

  /* Premire ligne : nom du fichier, sans intrt --- a dj t passe ! */

  /* Deuxime ligne : nom de l'exprience = nom du spectre */
  while ( c != ':' ) {
    c = fgetc( fichier );
    if ( EOF == c ) {
      (void) fprintf(stderr, "NSLS_LireSpectre : fin de fichier atteinte [%s]\n", nom);
      return -2;
    }
  }
  nom_image = lit_ligne( fichier, 0 );

  /* Troisime ligne : nombre de mesures */
  c = fgetc( fichier );
  while ( c != ':' ) {
    if ( EOF == c ) {
      (void) fprintf(stderr, "NSLS_LireSpectre : fin de fichier atteinte [%s]\n", nom);
      return -2;
    }
    c = fgetc( fichier );
  }
  n_lu = fscanf( fichier, "%lu", &nbr_points );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : impossible de lire le nombre de points ! [%s][%s]\n", nom, nom_image);
    return -3;
  }
  if ( nbr_points < 2 ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : trop peu de points [%lu, \"%s\"]\n", nbr_points, nom);
    return -8;
  }
#ifdef _DEVELOPPE_
  printf("NSLS_LireSpectre : %lu points  lire\n", nbr_points);
#endif

  /* Quatrime ligne : nombre de lignes = doit tre gal  1 pour 1 spectre ! */
  c = fgetc( fichier );
  while ( c != ':' ) {
    if ( EOF == c ) {
      (void) fprintf(stderr, "NSLS_LireSpectre : fin de fichier atteinte [%s]\n", nom);
      return -2;
    }
    c = fgetc( fichier );
  }
  n_lu = fscanf( fichier, "%lu", &N );
  if ( n_lu != 1 ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : impossible de lire le nombre de spectres ! [%s]\n", nom);
    return -3;
  }
  if ( N != 1 ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : fichier multi-lignes. Pas de spectre ou multi-spectre ? [%s]\n", nom);
    return -4;
  }

  /* Cinquime ligne : nergies extrmes */
  c = fgetc( fichier );
  while ( c != ':' ) {
    if ( EOF == c ) {
      (void) fprintf(stderr, "NSLS_LireSpectre : fin de fichier atteinte [%s]\n", nom);
      return -2;
    }
    c = fgetc( fichier );
  }
  n_lu = fscanf( fichier, "%lf , %lf", &x_deb, &x_fin );
  if ( n_lu != 2 ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : impossible de lire les nergies extrmes ! [%s]\n", nom);
    return -3;
  }
  c = ignore_ligne(fichier); /* Ignore la fin de la ligne */
  if ( EOF == c ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : fin de fichier atteinte [%s]\n", nom);
    return -2;
  }
  if ( x_fin == x_deb ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : bornes en X identiques, chargement annul [%g, %g]\n", x_deb, x_fin);
    return -12;
  }
  if ( x_deb > x_fin ) {
    double tmp;

    (void) fprintf(stderr, "NSLS_LireSpectre : attention, X inverss ? [%s : %g -> %g]\n", nom, x_deb, x_fin);
    tmp = x_deb; x_deb = x_fin; x_fin = tmp;
  }

  /* Sixime ligne : dbut/fin en Y -- ignor */
  c = ignore_ligne(fichier);
  if ( EOF == c ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : fin de fichier atteinte [%s]\n", nom);
    return -2;
  }

  /* ------ On a toutes les infos pour crer les graphes ------ */
  n_lu = NSLS_PreparerSpectres( nom, nom_image, graphes, serie, nbr_points );
  if ( n_lu != 0 ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : ERREUR dans EXTRAIT_PreparerSpectres() [%d, \"%s\"]\n", n_lu, nom);
    return -5;
  }

  /* On remplit la table des X, si elle existe */
  (void) EXTRAIT_TrouverTableX( infos, graphes );
  if ( NULL == graphes->x ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : aucune information conserve !\n");
    return -11;
  }
  pas = ( x_fin - x_deb ) / (double) ( nbr_points - 1 );
  for ( N = 0, x = graphes->x ; N < nbr_points ; N++, x++ ) {
    *x = x_deb + pas * (double) N;
  }

  /* Titre de la ligne de fluorescence */
  c = ignore_ligne(fichier);
  if ( EOF == c ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : fin de fichier atteinte aprs titre If [%s]\n", nom);
    return -2;
  }

  /* Valeurs mesures pour I fluorescence */
  n_lu = lit_valeurs(fichier, nbr_points, '/', graphes->i_1[ 0 ]->y, NULL);
  if ( n_lu != 0 ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : ERREUR dans lit_valeurs / If\n");
    return -10;
  }

  /* Titre de la ligne pour I0 */
  c = ignore_ligne( fichier );
  if ( EOF == c ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : fin de fichier atteinte aprs titre I0  [%s]\n", nom);
    return -2;
  }

  /* Valeurs mesures pour I0 */
  n_lu = lit_valeurs(fichier, nbr_points, '/', graphes->i_0->y, NULL);
  if ( n_lu != 0 ) {
    (void) fprintf(stderr, "NSLS_LireSpectre : ERREUR dans lit_valeurs / I0\n");
    return -10;
  }

  /* Si ncessaire, on fait le calcul */
  if ( ( infos->que_garder & DETECT_GARDER_SIGNAL ) && ( graphes->spectres != NULL ) ) {
    int msg_i0 = FAUX, msg_i1 = FAUX;
    double *i1, *i0, *signal;
    unsigned long i;

    i0 = graphes->i_0->y;
    i1= graphes->i_1[ 0 ]->y;
    signal = graphes->spectres[ 0 ]->y;
    for ( i = 0 ; i < nbr_points ; i++, i0++, i1++, signal++ ) {
      if ( *i0 <= 0.0 ) {
	if ( FAUX == msg_i0 ) {
#ifdef _DEVELOPPE_
	  (void) fprintf(stderr, "NSLS_LireSpectre : I0 <= 0.0\n");
#endif
	}
	msg_i0 = VRAI;
	msg_i1 = VRAI;
	*signal = 0.0;
      } else {
	if ( *i1 < 0.0 ) {
	  if ( FAUX == msg_i1 ) {
#ifdef _DEVELOPPE_
	    (void) fprintf(stderr, "NSLS_LireSpectre : I1 < 0.0\n");
#endif
	  }
	  msg_i1 = VRAI;
	}
	*signal = *i1 / *i0;;
      }
    }
  }

  /* On efface ventuellement les graphes inutiles */
  if ( FAUX == ( infos->que_garder & DETECT_GARDER_I0 ) ) {
     GRF_efface( graphes->i_0, VRAI, VRAI );
     graphes->i_0 = NULL;
  }
  if ( FAUX == ( infos->que_garder & ( DETECT_GARDER_TRANS | DETECT_GARDER_FLUO | DETECT_GARDER_EL ) ) ) {
     GRF_efface( graphes->i_1[ 0 ], VRAI, VRAI );
     free( graphes->i_1 );
     graphes->i_1 = NULL;
  }

  /* Tout s'est bien pass */
  return 0;
}

static int NSLS_PreparerSpectres(const char *nom_fichier, const char *nom_image, EXT_ListeGraphes *graphes, struct serie *serie,
				 unsigned long nbr_points)
{
  char *titre;

  graphes->x = NULL;
  graphes->nbr_spectres = 1;

  /* Graphe pour I0 : toujours prsent... car facilite les choses */
  titre = TXT_genere_chaine("%F (%s) : I0", nom_fichier, nom_image);
  graphes->i_0 = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_I0, GR_NORMAL, titre );
  free( titre );
  if ( NULL == graphes->i_0 ) {
    (void) fprintf(stderr, "NSLS_PreparerSpectres : impossible de prparer I0\n");
    return -1;
  }

  /* Graphe pour I1 : forcment unique... */
  graphes->i_1 = (Graphe **) calloc( sizeof(Graphe *), 1 );
  if ( NULL == graphes->i_1 ) {
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "Extrait/extraire.c", "NSLS_PreparerGraphes", "graphes->i_1");
    GRF_efface( graphes->i_0, VRAI, VRAI );
    return -2;
  }
  titre = TXT_genere_chaine("%F (%s) : I fluo", nom_fichier, nom_image);
  graphes->i_1[ 0 ] = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_I1, GR_NORMAL, titre );
  free( titre );
  if ( NULL == graphes->i_1[ 0 ] ) {
    (void) fprintf(stderr, "NSLS_PreparerSpectres : ERREUR en crant le graphe pour I1\n");
    GRF_efface( graphes->i_0, VRAI, VRAI );
    graphes->i_0 = NULL;
    return -3;
  }

  /* Graphe pour le signal : idem */
  graphes->spectres = (Graphe **) calloc( sizeof(Graphe *), 1 );
  if ( NULL == graphes->spectres ) {
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "Extrait/NSLS.c", "NSLS_PreparerGraphes", "graphes->spectres");
  } else {
    titre = TXT_genere_chaine("%F (%s)", nom_fichier, nom_image);
    graphes->spectres[ 0 ] = Serie_nv_graphe( serie, serie->dernier_graphe, nbr_points, GR_BRUT, GR_NORMAL, titre );
    free( titre );
    if ( NULL == graphes->spectres[ 0 ] ) {
      (void) fprintf(stderr, "NSLS_PreparerGraphes : ERREUR dans Serie_nv_graphe / spectre brut\n");
      free( graphes->spectres );
      graphes->spectres = NULL;
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

