/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 1999, Emmanuel CURIS
   Cration : 27 mai 1998

   Calcul du spectre d'absorption  partir des fichiers exprimentaux

   Fichiers multilments : sparation des spectres de tous les lments

    faire : - sauter fin du fichier si I1 = 0 et utilisateur le demande
             - pas demander 36 fois si I0 = 0 et utilisateur demande d'ignorer
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "graphes.h"
#include "import_xas.h"

#include "separer.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Analyse de la ligne pour faire les calculs... --------*/

int EXTRAIT_AnalyseLigneSeparer(unsigned int nbr_valeurs, double *valeurs, Intervalle_I *filtre, unsigned long nbr_filtres,
				int *erreurs, int methode, double i0, unsigned long point,
				Graphe **spectres, Graphe **i1, const char *nom_fichier, Infos_Extraction *infos)
{
  unsigned int j;
  const BI_Boite_Alerte *message;

  if ( NULL == filtre ) return 0; /* Ne nous intresse pas...*/

  switch ( methode )
  {
  case DETECT_TRANSMISSION:
    message = &ERR_iTransmissionAbsurde;
    break;

  case DETECT_FLUORESCENCE:
    message = &ERR_iFluorescenceAbsurde;
    break;

  case DETECT_ELECTRONS:
    message = &ERR_iDetectionAbsurde;
    break;

  default:
    (void) fprintf(stderr, "EXTRAIT_AnalyseLigneSeparer : Mthode inconnue [%d]\n", methode );
    message = NULL;
    break;
  }

  for ( j = 0 ; j < nbr_valeurs ; j++ ) {
    if ( VRAI == TXT_est_dans_intervalle( j + 1, filtre, nbr_filtres ) ) {
      double val_i1;
	
      val_i1 = valeurs[ j ];
      /* Si on est en fluorescence : correction de temps mort, si demande */
      if ( ( DETECT_FLUORESCENCE == methode ) && ( infos->post_traitement & DETECT_POST_CORRECT ) ) {
	double tps_mort = 1.0, tps = infos->temps_mort.temps_mesure;

	if ( ( infos->temps_mort.col_tpsmort > 0 ) && ( infos->temps_mort.col_tpsmort <= nbr_valeurs ) )
	  tps_mort = valeurs[ infos->temps_mort.col_tpsmort - 1 ];
	if ( ( infos->temps_mort.col_temps > 0 ) && ( infos->temps_mort.col_temps <= nbr_valeurs ) )
	  tps = valeurs[ infos->temps_mort.col_temps - 1 ];

	val_i1 = EXTRAIT_CorrigeTempsMort( val_i1, infos->temps_mort.constante, tps, tps_mort );
      }

      if( i1 != NULL ) {
	/* Mmorisation de I1 */
	if ( i1[ j ] != NULL ) {
	  i1[ j ]->y[ point ] = val_i1;
	}
      }

      /* Calcul du signal [au moins 1] */
      if ( spectres != NULL ) {
	if ( spectres[ j ] != NULL ) {
	  double mu;

	  if ( i0 <= 0.0 ) {
	    mu = 0.0; /*DETECT_TRANSMISSION Signal arbitrairement nul */
	  } else if ( ( val_i1 < 0.0 ) || /* Ce qui suit : hack sauvage pour la transmission, sale mais... */
		      ( ( DETECT_TRANSMISSION == methode ) && ( val_i1 == 0.0 ) ) ) {
	    if ( ( NULL == erreurs) || ( FAUX == erreurs[ j ] ) ) {
	      int reponse;
	      
	      reponse = BI_ERR_alerte( NULL, message, VRAI, j + 1, nom_fichier );
	      if ( ANNULER_CHARGEMENT == reponse ) return ANNULER_CHARGEMENT;
	      if ( ARRETER_CHARGEMENT == reponse ) return ARRETER_CHARGEMENT;
	    }
	    erreurs[ j ] = VRAI;
	    if ( DETECT_TRANSMISSION == methode )
	      mu = 0.0;         /* Signal arbitrairement nul, en transmission, car non-dfini mathmatiquement */
	    else
	      mu = val_i1 / i0; /* Ne gne pas en fluo, on calcule (car peut arriver si courant noir trop soustrait...) */
	  } else {
	    if ( DETECT_TRANSMISSION == methode )
	      mu = log( i0 / val_i1 );
	    else
	      mu = val_i1 / i0;
	  }

	  spectres[ j ]->y[ point ] = mu;
	}
      }
    }
  }

  /* Tout s'est bien pass */
  return 0;
}
