/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2002, Emmanuel CURIS
   Ce fichier : cration le 8 janvier 1999

   Cration d'un modle thorique pour FEFF

   Paramtrage des potentiels
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
#include "exafs.h"

#include "absorption.h" /* Pour les seuils */
#include "atomes.h"
#include "feff.h"
#include "interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void    prepare_arbre_modele(void);
static int GER_modele(unsigned int num_objet, int x, int y);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

void FEFF_DefinirPotentiels(Modele_FEFF *modele)
{
  FEFF_INP *params_feff;
  boolean test;
  int i;

  if (NULL == modele) {
    modele = FEFF_ChoisirModele(FAUX); /* s'il y en a un, on le modifie sans demander */
    if (NULL == modele)
      return;
  }

  if ( NULL == modele->absorbeur ) {
    (void) fprintf(stderr, "FEFF_DefinirPotentiels : il n'y a aucun absorbeur associ au modle !\n");
    return;
  }
  params_feff = &(modele->parametres);

  /* -~-~-~-~-~ Prparation : choix des options raisonnables -~-~-~-~-~ */

  if ( arbres[FEFF_PARAM_MODEL].drapeaux.prepare == FAUX )
    prepare_arbre_modele();

  /* -~-~-~-~-~ On installe les paramtres actuel -~-~-~-~-~-~-~-~-~-~- */

  /* L'atome absorbeur -- il y en a forcment un puisque les modles sont dedans ! */
  met_chaine( FEFF_PARAM_MODEL, FEFF_POT_0, 0, ATM_Z_vers_symbole( modele->absorbeur->num_atomique ), TXT_COPIER);
  OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_ION_0, params_feff->ION[0], 0.0 );

  /* Les autres potentiels */
  for (i = 0 ; i < 7 ; i++) { /* Les autres atomes */
    met_chaine(FEFF_PARAM_MODEL, FEFF_POT_1 + i, 0, ATM_Z_vers_symbole( modele->potentiels[1 + i] ), TXT_COPIER);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_FLP_1 + i, !(modele->drapeaux.av_AFOLP) );

    if ( modele->potentiels[i + 1] > 0 ) { /* Le potentiel est dfini */
      OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_ION_1 + i, params_feff->ION[i + 1], 0.0 );

      if ( (params_feff->FOLP[i + 1] >= 0.0) && (FAUX == modele->drapeaux.av_AFOLP) ) {
	OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FLP_1 + i, FORCE_OUI);
	OBJ_active (FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, FORCE_OUI);
	OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, params_feff->FOLP[i + 1], 0.0 );
      } else {
	OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FLP_1 + i, FORCE_NON);
	OBJ_active (FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, FORCE_NON);
	OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, 0.0, 0.0 );
      }
    } else {
      OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_ION_1 + i, 0.0, 0.0 );
      OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FLP_1 + i, FORCE_NON);
      OBJ_active (FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, FORCE_NON);
      OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, 0.0, 0.0 );
    }
  }

  /* Recouvrement total ? */
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_AFOLP, modele->drapeaux.av_AFOLP);

  /* Modle de Debye pour le dsordre ? */
  test = ( params_feff->DEBYE.temperature > 0.0 ) && ( params_feff->DEBYE.t_debye > 0.0 );
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_DEBYE, test);
  OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_DEBYE_T , params_feff->DEBYE.temperature, 0.0 );
  OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_DEBYE_TD, params_feff->DEBYE.t_debye, 0.0 );
  OBJ_active (FEFF_PARAM_MODEL, FEFF_SIG2, !test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_VAL_SIGMA, !test);

  /* Valeur globale pour le facteur de Debye-Waller */
  test = (params_feff->SIG2 > 0.0);
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_SIG2, test);
  OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_VAL_SIGMA, params_feff->SIG2, 0.0 );
  OBJ_active (FEFF_PARAM_MODEL, FEFF_DEBYE, !test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_DEBYE_T, !test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_DEBYE_TD, !test);

  OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_DECALAGE, params_feff->CORRECTIONS.decalage, 0.0 );
  OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_ELARGIT, params_feff->CORRECTIONS.elargissement, 0.0 );

  test = params_feff->EXCHANGE.type >= 0;
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_EXCHANGE, test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_HEDIN, test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_DIRAC, test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_FONDAMENTAL, test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_MIXTE, test);
  if (test) {
    if ( params_feff->EXCHANGE.type == 0 )
      OBJ_choisit(FEFF_PARAM_MODEL, FEFF_HEDIN, FORCE_OUI);
    else if ( params_feff->EXCHANGE.type == 1)
      OBJ_choisit(FEFF_PARAM_MODEL, FEFF_DIRAC, FORCE_OUI);
    else if ( params_feff->EXCHANGE.type == 2)
      OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FONDAMENTAL, FORCE_OUI);
    else if ( params_feff->EXCHANGE.type == 3)
      OBJ_choisit(FEFF_PARAM_MODEL, FEFF_MIXTE, FORCE_OUI);
  }

  /* Seuil d'absorption */
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_K , SEUIL_K == modele->seuil);
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L1, SEUIL_LI == modele->seuil);
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L2, SEUIL_LII == modele->seuil);
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L3, SEUIL_LIII == modele->seuil);

  OBJ_MettreNombre( FEFF_PARAM_MODEL, FEFF_S0_2, VAR_LireValeur( &(modele->S0_2), VARIABLE_LIRE_VALEUR_INITIALE ), 0.0 );

  /* -~-~-~-~-~ On appelle l'arbre, on gre et tout et tout -~-~-~-~-~-~-~-~-~-~-~-~-~ */

  if ( ARB_Bloquant( FEFF_PARAM_MODEL, "Mod"e_GRAVE"le th"e_AIGU"orique pour FEFF", principale, 0 ) == FEFF_MODELE_ANN )
    return;

  /* -~-~-~-~-~ On enregistre les nouveaux paramtres ~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- */

  /* Recouvrements */
  modele->drapeaux.av_AFOLP = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_AFOLP, QUESTION);

  /* Absorbeur (nb : on ne peut pas modifier l'atome associ par cette mthode... */
  params_feff->ION[0] = OBJ_LireNombre( FEFF_PARAM_MODEL, FEFF_ION_0, NULL );

  /* Autres potentiels */
  for (i = 1 ; i < 8 ; i++) {
    modele->potentiels[i] = OBJ_NumeroAtomique( FEFF_PARAM_MODEL, FEFF_POT_1 + i - 1 );

    if ( modele->potentiels[i] == 0 ) {
      params_feff->FOLP[i] = -1.0; /* Pas de recouvrement */
      params_feff->ION[i]  = 0.0; /* Pas de charge */
    } else {
      params_feff->ION[i] = OBJ_LireNombre( FEFF_PARAM_MODEL, FEFF_ION_1 + i - 1, NULL );
      if ( (VRAI == modele->drapeaux.av_AFOLP) || (OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FLP_1 + i - 1, QUESTION) == FAUX ) )
	params_feff->FOLP[i] = -1.0; /* Pas de recouvrement */
      else
	params_feff->FOLP[i] = OBJ_LireNombre( FEFF_PARAM_MODEL, FEFF_FLP_1 + i - 1, NULL );
    }
  }

  if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_DEBYE, QUESTION) == VRAI ) {
    params_feff->DEBYE.temperature = OBJ_LireNombre( FEFF_PARAM_MODEL, FEFF_DEBYE_T, NULL );
    params_feff->DEBYE.t_debye     = OBJ_LireNombre( FEFF_PARAM_MODEL, FEFF_DEBYE_TD, NULL );
    params_feff->SIG2 = -1.0;
  } else {
    params_feff->DEBYE.temperature = 0.0;
    params_feff->DEBYE.t_debye     = 0.0;
    if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_SIG2, QUESTION) == VRAI ) {
      params_feff->SIG2 = OBJ_LireNombre( FEFF_PARAM_MODEL, FEFF_VAL_SIGMA, NULL );
    } else {
      params_feff->SIG2 = -1.0;
    }
  }

  params_feff->CORRECTIONS.decalage = OBJ_LireNombre( FEFF_PARAM_MODEL, FEFF_DECALAGE, NULL );
  params_feff->CORRECTIONS.elargissement = OBJ_LireNombre( FEFF_PARAM_MODEL, FEFF_ELARGIT, NULL );

  /*  FAIRE : grer les nouveaux potentiels / FEFF 7 */
  OPTION_A_FAIRE( "options EXCHANGE > 4 pas encore reconnues [feff 7]"); 

  if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_EXCHANGE, QUESTION) == VRAI ) {
    params_feff->EXCHANGE.decalage = params_feff->CORRECTIONS.decalage;
    params_feff->EXCHANGE.p_imag   = params_feff->CORRECTIONS.elargissement;
    if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_HEDIN, QUESTION) == VRAI )
      params_feff->EXCHANGE.type = 0;
    else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_DIRAC, QUESTION) == VRAI )
      params_feff->EXCHANGE.type = 1;
    else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FONDAMENTAL, QUESTION) == VRAI )
      params_feff->EXCHANGE.type = 2;
    else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_MIXTE, QUESTION) == VRAI )
      params_feff->EXCHANGE.type = 3;
  } else {
    params_feff->EXCHANGE.type = ABSENT; /* On n'utilise pas */
    params_feff->EXCHANGE.decalage = 0.0;
    params_feff->EXCHANGE.p_imag   = 0.0;
  }

  if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_K, QUESTION) == VRAI )
    modele->seuil = SEUIL_K;
  else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L1, QUESTION) == VRAI )
    modele->seuil = SEUIL_LI;
  else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L2, QUESTION) == VRAI )
    modele->seuil = SEUIL_LII;
  else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L3, QUESTION) == VRAI )
    modele->seuil = SEUIL_LIII;

  modele->S0_2.valeur_initiale = OBJ_LireNombre( FEFF_PARAM_MODEL, FEFF_S0_2, NULL );
}

/* ------------------------- Gestion effective de la bote de paramtrage ------------- */

static int GER_modele(unsigned int num_objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  int nv_etat, i;

  switch (num_objet)
  {
  case FEFF_EXCHANGE:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_EXCHANGE, ECHANGE);
    OBJ_active (FEFF_PARAM_MODEL, FEFF_HEDIN, nv_etat);
    OBJ_active (FEFF_PARAM_MODEL, FEFF_DIRAC, nv_etat);
    OBJ_active (FEFF_PARAM_MODEL, FEFF_FONDAMENTAL, nv_etat);
    OBJ_active (FEFF_PARAM_MODEL, FEFF_MIXTE, nv_etat);
    return BI_EVENEMENT_TRAITE;

  case FEFF_FLP_1:
  case FEFF_FLP_2:
  case FEFF_FLP_3:
  case FEFF_FLP_4:
  case FEFF_FLP_5:
  case FEFF_FLP_6:
  case FEFF_FLP_7:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, num_objet, ECHANGE);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + num_objet - FEFF_FLP_1, nv_etat);
    return BI_EVENEMENT_TRAITE;

  case FEFF_DEBYE:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_DEBYE, ECHANGE);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_SIG2, !nv_etat);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_VAL_SIGMA, !nv_etat);
    return BI_EVENEMENT_TRAITE;
    
  case FEFF_SIG2:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_SIG2, ECHANGE);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_DEBYE, !nv_etat);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_DEBYE_T, !nv_etat);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_DEBYE_TD, !nv_etat);
    return BI_EVENEMENT_TRAITE;

  case FEFF_AFOLP:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_AFOLP, ECHANGE);
    for (i = 0 ; i < 7 ; i++) {
      OBJ_active(FEFF_PARAM_MODEL, FEFF_FLP_1 + i , !nv_etat);
    }
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparation de l'arbre -------- */

static void prepare_arbre_modele(void)
{
  if ( FAUX == arbres[ FEFF_PARAM_MODEL ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( FEFF_PARAM_MODEL, "menu_feff.html#potentiel", GER_modele, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }
  }

  OBJ_agrandit(FEFF_PARAM_MODEL, FEFF_LACUNE_FOND, +3, 0);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_LACUNE_L1, +1, 0, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_LACUNE_L2, +2, 0, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_LACUNE_L3, +3, 0, DEPLACE_PIXELS);

  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_1_CHX, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_2_CHX, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_3_CHX, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_4_CHX, 0, +4, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_5_CHX, 0, +5, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_6_CHX, 0, +6, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_7_CHX, 0, +7, DEPLACE_PIXELS);

  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_1, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_2, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_3, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_4, 0, +4, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_5, 0, +5, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_6, 0, +6, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_POT_7, 0, +7, DEPLACE_PIXELS);

  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_FLP_1, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_FLP_2, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_FLP_3, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_FLP_4, 0, +4, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_FLP_5, 0, +5, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_FLP_6, 0, +6, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_FLP_7, 0, +7, DEPLACE_PIXELS);

  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_RECOUVRE_1, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_RECOUVRE_2, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_RECOUVRE_3, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_RECOUVRE_4, 0, +4, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_RECOUVRE_5, 0, +5, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_RECOUVRE_6, 0, +6, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_RECOUVRE_7, 0, +7, DEPLACE_PIXELS);

  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_ION_1, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_ION_2, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_ION_3, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_ION_4, 0, +4, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_ION_5, 0, +5, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_ION_6, 0, +6, DEPLACE_PIXELS);
  OBJ_deplace(FEFF_PARAM_MODEL, FEFF_ION_7, 0, +7, DEPLACE_PIXELS);

#ifndef _DEVELOPPE_
  OBJ_active(FEFF_PARAM_MODEL, FEFF_POT_1_CHX, FORCE_NON);
  OBJ_active(FEFF_PARAM_MODEL, FEFF_POT_2_CHX, FORCE_NON);
  OBJ_active(FEFF_PARAM_MODEL, FEFF_POT_3_CHX, FORCE_NON);
  OBJ_active(FEFF_PARAM_MODEL, FEFF_POT_4_CHX, FORCE_NON);
  OBJ_active(FEFF_PARAM_MODEL, FEFF_POT_5_CHX, FORCE_NON);
  OBJ_active(FEFF_PARAM_MODEL, FEFF_POT_6_CHX, FORCE_NON);
  OBJ_active(FEFF_PARAM_MODEL, FEFF_POT_7_CHX, FORCE_NON);
#endif

  arbres[FEFF_PARAM_MODEL].drapeaux.prepare = VRAI;
}

/* ~~~~~~~~ Aides  la conception du modle ~~~~~~~~ */
