%{
  /* Analyse d'une chane de texte pour en dduire une formule mathmatique
     = relation entre graphes, transformations de coordonnes,...

     (C) 2001 Emmanuel Curis, fichier yacc  l'origine

     Remarque : yacc rsoud les conflits reduce/reduce en choisissant la premire instance
       pour le choix entre 'expr' + 'nombre' et 'expr' + 'expr', on prfre la premire,
       donc on les place en premier [fait un arbre plus simple]

     En particulier, on place en premier le calcul des expressions avec que des nombres,
     comme a le calcul est fait ds l'analyse -> arbre beaucoup plus simple !

     Ceci dit, a n'a pas une grande importance parce que l'arbre cr est toujours correct,
     il sera juste analys diffremment...
  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h> /* Pour tolower() */
#include <math.h>

#include "systeme.h"

#include "c_formel.h"
#include "modeles.h"
#include "science.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static unsigned long nombre_parametres = 0;
static struct parametre *table_parametres = NULL;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

void yyerror (char *s);

extern char *formulestext; /* Dfinies par lex */
extern int formuleslex(void);
extern struct yy_buffer_state *formules_scan_string(const char *texte);
extern void formules_delete_buffer(struct yy_buffer_state *tampon);

static int       quelle_fonction(char *nom_fonction);
static Variable *CF_TrouverParametre(const char *nom_variable, unsigned long n_parametres, struct parametre *parametres);

static Expression *debut;
%}

%union {
  double nombre;
  char code_variable;
  int code_operation;
  char *nom_fonction;
  Expression *expression;
  Variable *variable;
}

%token <nombre>         NOMBRE
%token <code_variable>  VARIABLE
%token <code_operation> OPERATION
%token <nom_fonction>   FONCTION

%type <nombre>     exp_numerique
%type <expression> relation addition difference produit quotient puissance
%type <variable>   def_variable

%left '-' '+'
%left '*' '/'
%left NEG /* Moins de signe */
%right '^'

%%

lancement: relation { debut = $1; }
;

exp_numerique: '(' exp_numerique ')' { $$ = $2; }
             | exp_numerique '+' exp_numerique {$$ = $1 + $3;}
             | exp_numerique '-' exp_numerique {$$ = $1 - $3;}
             | exp_numerique '*' exp_numerique {$$ = $1 * $3;}
             | exp_numerique '/' exp_numerique {$$ = $1 / $3;}
             | exp_numerique '^' exp_numerique {$$ = pow($1, $3);}
             | '-' exp_numerique %prec NEG {$$ = -$2;}
             | NOMBRE {$$ = $1;}
             | FONCTION '(' exp_numerique ')' {
	                                        int code_fonction;

						code_fonction = quelle_fonction($1);
						switch(code_fonction)
						{
						case EXP_SINUS:   $$ = sin($3); break;
						case EXP_COSINUS: $$ = cos($3); break;
						case EXP_TANGENTE:$$ = tan($3); break;
						case EXP_COTANGENTE: $$ = 1.0/tan($3); break;
						case EXP_SECANTE: $$ = 1.0/cos($3); break;
						case EXP_COSECANTE: $$ = 1.0/sin($3); break;

						case EXP_COSH: $$ = cosh($3); break;
						case EXP_SINH: $$ = sinh($3); break;
						case EXP_TANH: $$ = tanh($3); break;

						case EXP_EXP:     $$ = exp($3); break;
						case EXP_LOG:     $$ = log($3); break;

						case EXP_RACINE: $$ = sqrt($3); break;
						case EXP_CARRE: $$ = $3 * $3; break;
						case EXP_CUBE: $$ = $3 * $3 * $3; break;
						case EXP_INVERSE: $$ = 1.0/$3; break;
						}
	                                      }
;

addition: exp_numerique '+' def_variable { Expression *nv_expression;
	                                   nv_expression = (Expression *) malloc(sizeof(Expression));
					   if (nv_expression == NULL) {
					     yyerror("Mmoire insuffisante\n");
					   }
					   nv_expression->type = EXP_VAR_CTE;
					   nv_expression->code = EXP_ADDITION;
					   nv_expression->arg_1.variable = $3;
					   nv_expression->arg_2.valeur = $1;
					   $$ = nv_expression;
                                         }

        | def_variable '+' exp_numerique { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_CTE;
				      nv_expression->code = EXP_ADDITION;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.valeur = $3;
				      $$ = nv_expression;
	                        }

        | def_variable '+' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_VAR;
				      nv_expression->code = EXP_ADDITION;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | exp_numerique '+' relation { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       nv_expression->type = EXP_ARG_CTE;
				       nv_expression->code = EXP_ADDITION;
				       nv_expression->arg_1.expression = $3;
				       nv_expression->arg_2.valeur = $1;
				       $$ = nv_expression;
	                             }

        | def_variable '+' relation { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_ARG_VAR;
				      nv_expression->code = EXP_ADDITION;
				      nv_expression->arg_1.expression = $3;
				      nv_expression->arg_2.variable = $1;
				      $$ = nv_expression;
	                        }


        | relation '+' exp_numerique { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       nv_expression->type = EXP_ARG_CTE;
				       nv_expression->code = EXP_ADDITION;
				       nv_expression->arg_1.expression = $1;
				       nv_expression->arg_2.valeur = $3;
				       $$ = nv_expression;
	                             }

        | relation '+' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_ARG_VAR;
				      nv_expression->code = EXP_ADDITION;
				      nv_expression->arg_1.expression = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | relation '+' relation { Expression *nv_expression;
	                          nv_expression = (Expression *) malloc(sizeof(Expression));
				  if (nv_expression == NULL) {
				    yyerror("Mmoire insuffisante\n");
				  }
				  nv_expression->type = EXP_ARG_ARG;
				  nv_expression->code = EXP_ADDITION;
				  nv_expression->arg_1.expression = $1;
				  nv_expression->arg_2.expression = $3;
				  $$ = nv_expression;
	                         }
;

difference: exp_numerique '-' def_variable { Expression *nv_expression;
	                                   nv_expression = (Expression *) malloc(sizeof(Expression));
					   if (nv_expression == NULL) {
					     yyerror("Mmoire insuffisante\n");
					   }
					   nv_expression->type = EXP_CTE_VAR;
					   nv_expression->code = EXP_SOUSTRACTION;
					   nv_expression->arg_1.valeur = $1;
					   nv_expression->arg_2.variable = $3;
					   $$ = nv_expression;
                                         }

        | def_variable '-' exp_numerique { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_CTE;
				      nv_expression->code = EXP_SOUSTRACTION;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.valeur = $3;
				      $$ = nv_expression;
	                        }

        | def_variable '-' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_VAR;
				      nv_expression->code = EXP_SOUSTRACTION;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | exp_numerique '-' relation { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       nv_expression->type = EXP_CTE_ARG;
				       nv_expression->code = EXP_SOUSTRACTION;
				       nv_expression->arg_1.valeur = $1;
				       nv_expression->arg_2.expression = $3;
				       $$ = nv_expression;
	                             }

        | def_variable '-' relation { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_ARG;
				      nv_expression->code = EXP_SOUSTRACTION;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.expression = $3;
				      $$ = nv_expression;
	                        }

        | relation '-' exp_numerique { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       nv_expression->type = EXP_ARG_CTE;
				       nv_expression->code = EXP_SOUSTRACTION;
				       nv_expression->arg_1.expression = $1;
				       nv_expression->arg_2.valeur = $3;
				       $$ = nv_expression;
	                             }

        | relation '-' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_ARG_VAR;
				      nv_expression->code = EXP_SOUSTRACTION;
				      nv_expression->arg_1.expression = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | relation '-' relation { Expression *nv_expression;
	                          nv_expression = (Expression *) malloc(sizeof(Expression));
				  if (nv_expression == NULL) {
				    yyerror("Mmoire insuffisante\n");
				  }
				  nv_expression->type = EXP_ARG_ARG;
				  nv_expression->code = EXP_SOUSTRACTION;
				  nv_expression->arg_1.expression = $1;
				  nv_expression->arg_2.expression = $3;
				  $$ = nv_expression;
	                         }
;

produit: exp_numerique '*' def_variable { Expression *nv_expression;
	                                   nv_expression = (Expression *) malloc(sizeof(Expression));
					   if (nv_expression == NULL) {
					     yyerror("Mmoire insuffisante\n");
					   }
					   nv_expression->type = EXP_VAR_CTE;
					   nv_expression->code = EXP_PRODUIT;
					   nv_expression->arg_1.variable = $3;
					   nv_expression->arg_2.valeur = $1;
					   $$ = nv_expression;
                                         }

        | def_variable '*' exp_numerique { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_CTE;
				      nv_expression->code = EXP_PRODUIT;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.valeur = $3;
				      $$ = nv_expression;
	                        }

        | def_variable '*' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_VAR;
				      nv_expression->code = EXP_PRODUIT;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | exp_numerique '*' relation { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       nv_expression->type = EXP_ARG_CTE;
				       nv_expression->code = EXP_PRODUIT;
				       nv_expression->arg_1.expression = $3;
				       nv_expression->arg_2.valeur = $1;
				       $$ = nv_expression;
	                             }

        | def_variable '*' relation { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_ARG_VAR;
				      nv_expression->code = EXP_PRODUIT;
				      nv_expression->arg_1.expression = $3;
				      nv_expression->arg_2.variable = $1;
				      $$ = nv_expression;
	                        }

        | relation '*' exp_numerique { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       nv_expression->type = EXP_ARG_CTE;
				       nv_expression->code = EXP_PRODUIT;
				       nv_expression->arg_1.expression = $1;
				       nv_expression->arg_2.valeur = $3;
				       $$ = nv_expression;
	                             }

        | relation '*' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_ARG_VAR;
				      nv_expression->code = EXP_PRODUIT;
				      nv_expression->arg_1.expression = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | relation '*' relation { Expression *nv_expression;
	                          nv_expression = (Expression *) malloc(sizeof(Expression));
				  if (nv_expression == NULL) {
				    yyerror("Mmoire insuffisante\n");
				  }
				  nv_expression->type = EXP_ARG_ARG;
				  nv_expression->code = EXP_PRODUIT;
				  nv_expression->arg_1.expression = $1;
				  nv_expression->arg_2.expression = $3;
				  $$ = nv_expression;
	                         }
;

quotient: exp_numerique '/' def_variable { Expression *nv_expression;
	                                   nv_expression = (Expression *) malloc(sizeof(Expression));
					   if (nv_expression == NULL) {
					     yyerror("Mmoire insuffisante\n");
					   }
					   nv_expression->type = EXP_CTE_VAR;
					   nv_expression->code = EXP_DIVISION;
					   nv_expression->arg_1.valeur = $1;
					   nv_expression->arg_2.variable = $3;
					   $$ = nv_expression;
                                         }

        | def_variable '/' exp_numerique { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_CTE;
				      nv_expression->code = EXP_DIVISION;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.valeur = $3;
				      $$ = nv_expression;
	                        }

        | def_variable '/' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_VAR;
				      nv_expression->code = EXP_DIVISION;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | exp_numerique '/' relation { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       nv_expression->type = EXP_CTE_ARG;
				       nv_expression->code = EXP_DIVISION;
				       nv_expression->arg_1.valeur = $1;
				       nv_expression->arg_2.expression = $3;
				       $$ = nv_expression;
	                             }

        | def_variable '/' relation { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_ARG;
				      nv_expression->code = EXP_DIVISION;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.expression = $3;
				      $$ = nv_expression;
	                        }

        | relation '/' exp_numerique { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       nv_expression->type = EXP_ARG_CTE;
				       nv_expression->code = EXP_DIVISION;
				       nv_expression->arg_1.expression = $1;
				       nv_expression->arg_2.valeur = $3;
				       $$ = nv_expression;
	                             }

        | relation '/' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_ARG_VAR;
				      nv_expression->code = EXP_DIVISION;
				      nv_expression->arg_1.expression = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | relation '/' relation { Expression *nv_expression;
	                          nv_expression = (Expression *) malloc(sizeof(Expression));
				  if (nv_expression == NULL) {
				    yyerror("Mmoire insuffisante\n");
				  }
				  nv_expression->type = EXP_ARG_ARG;
				  nv_expression->code = EXP_DIVISION;
				  nv_expression->arg_1.expression = $1;
				  nv_expression->arg_2.expression = $3;
				  $$ = nv_expression;
	                         }
;

puissance: exp_numerique '^' def_variable { Expression *nv_expression;
	                                    nv_expression = (Expression *) malloc(sizeof(Expression));
					    if (nv_expression == NULL) {
					      yyerror("Mmoire insuffisante\n");
					    }
					    nv_expression->type = EXP_CTE_VAR;
					    nv_expression->code = EXP_PUISSANCE;
					    nv_expression->arg_1.valeur = $1;
					    nv_expression->arg_2.variable = $3;
					    $$ = nv_expression;
                                          }

        | def_variable '^' exp_numerique { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_CTE;
				      nv_expression->code = EXP_PUISSANCE;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.valeur = $3;
				      $$ = nv_expression;
	                        }

        | def_variable '^' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_VAR;
				      nv_expression->code = EXP_PUISSANCE;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | exp_numerique '^' relation { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       nv_expression->type = EXP_CTE_ARG;
				       nv_expression->code = EXP_PUISSANCE;
				       nv_expression->arg_1.valeur = $1;
				       nv_expression->arg_2.expression = $3;
				       $$ = nv_expression;
	                             }

        | def_variable '^' relation { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_VAR_ARG;
				      nv_expression->code = EXP_PUISSANCE;
				      nv_expression->arg_1.variable = $1;
				      nv_expression->arg_2.expression = $3;
				      $$ = nv_expression;
	                        }

        | relation '^' exp_numerique { Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       OPTION_A_FAIRE( "Grer le cas du carr et du cube pour calcul plus rapide !" );
				       nv_expression->type = EXP_ARG_CTE;
				       nv_expression->code = EXP_PUISSANCE;
				       nv_expression->arg_1.expression = $1;
				       nv_expression->arg_2.valeur = $3;
				       $$ = nv_expression;
	                             }

        | relation '^' def_variable { Expression *nv_expression;
	                              nv_expression = (Expression *) malloc(sizeof(Expression));
				      if (nv_expression == NULL) {
					yyerror("Mmoire insuffisante\n");
				      }
				      nv_expression->type = EXP_ARG_VAR;
				      nv_expression->code = EXP_PUISSANCE;
				      nv_expression->arg_1.expression = $1;
				      nv_expression->arg_2.variable = $3;
				      $$ = nv_expression;
	                        }

        | relation '^' relation { Expression *nv_expression;
	                          nv_expression = (Expression *) malloc(sizeof(Expression));
				  if (nv_expression == NULL) {
				    yyerror("Mmoire insuffisante\n");
				  }
				  nv_expression->type = EXP_ARG_ARG;
				  nv_expression->code = EXP_PUISSANCE;
				  nv_expression->arg_1.expression = $1;
				  nv_expression->arg_2.expression = $3;
				  $$ = nv_expression;
	                         }
;

relation: '(' relation ')' { $$ = $2; }

        | FONCTION '(' def_variable ')' {  Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       /* Le deuxime paramtre ne sert pas, de toute faon */
				       nv_expression->type = EXP_VAR_CTE;
				       nv_expression->code = quelle_fonction($1);
				       free($1); /* Car cre avec strdup */
				       nv_expression->arg_1.variable = $3;
				       nv_expression->arg_2.valeur = 0.0;
				       $$ = nv_expression;
	                         }
        | FONCTION '(' relation ')' {  Expression *nv_expression;
	                               nv_expression = (Expression *) malloc(sizeof(Expression));
				       if (nv_expression == NULL) {
					 yyerror("Mmoire insuffisante\n");
				       }
				       /* Le deuxime paramtre ne sert pas, de toute faon */
				       nv_expression->type = EXP_ARG_CTE;
				       nv_expression->code = quelle_fonction($1);
				       free($1); /* Car cre avec strdup */
				       nv_expression->arg_1.expression = $3;
				       nv_expression->arg_2.valeur = 0.0;
				       $$ = nv_expression;
	                         }
        | addition   { $$ = $1; }
        | difference { $$ = $1; }
        | produit    { $$ = $1; }
        | quotient   { $$ = $1; }
        | puissance  { $$ = $1; }

        | exp_numerique { Expression *nv_expression;
	                  nv_expression = (Expression *) malloc(sizeof(Expression));
			  if (nv_expression == NULL) {
			    yyerror("Mmoire insuffisante\n");
			  }
			  nv_expression->type = EXP_CTE_CTE;
			  nv_expression->code = EXP_NOMBRE_ISOLE;
			  nv_expression->arg_1.valeur = $1;
			  $$ = nv_expression;
	                }

        | '-' relation %prec NEG { Expression *nv_expression;
	                            nv_expression = (Expression *) malloc(sizeof(Expression));
				    if (nv_expression == NULL) {
				      yyerror("Mmoire insuffisante\n");
				    }
				    nv_expression->type = EXP_CTE_ARG;
				    nv_expression->code = EXP_PRODUIT;
				    nv_expression->arg_1.valeur = -1.0;
				    nv_expression->arg_2.expression = $2;
				    $$ = nv_expression;
                                  }

        | def_variable { Expression *nv_expression;
	                 nv_expression = (Expression *) malloc(sizeof(Expression));
			 if (nv_expression == NULL) {
			   yyerror("Mmoire insuffisante\n");
			 }
			 nv_expression->type = EXP_VAR_CTE;
			 nv_expression->code = EXP_NOMBRE_ISOLE;
			 nv_expression->arg_1.variable = $1;
			 nv_expression->arg_2.variable = NULL;
			 $$ = nv_expression;
                       }
;

def_variable: VARIABLE { switch ($1)
                         {
			 case 'x': case 'X': $$ = &Var_X; break;
			 case 'y': case 'Y': $$ = &Var_Y; break;
			 case 'z': case 'Z': $$ = &Var_Z; break;
			 case 'i': case 'I': $$ = &Var_I; break;
			 case 't': case 'T': $$ = &Var_T; break;
			 case 'r': case 'R': $$ = &Var_R; break;
			 case 's': $$ = &Var_sX; break;
			 case 'S': $$ = &Var_sY; break;
			 default:
			   {
			     Variable *variable;
			     char nom_variable[ 2 ] = { $1, 0 };

			     /* On cherche donc dans la table des paramtres */
			     variable = CF_TrouverParametre( nom_variable, nombre_parametres, table_parametres );
			     if ( NULL == variable ) {
			       (void) fprintf(stderr, "%s, %s l.%u : Variable inconnue : %c\n",
					      __FILE__, __FUNCTION__, __LINE__, $1 );
			     }

			     $$ = variable;
			   }
			 }
                       }
            | FONCTION {
                         Variable *variable;

			 /* On cherche donc dans la table des paramtres */
			 variable = CF_TrouverParametre( $1, nombre_parametres, table_parametres );
			 if ( NULL == variable ) {
			   (void) fprintf(stderr, "%s, %s l.%u : Variable inconnue : %s\n",
					  __FILE__, __FUNCTION__, __LINE__, $1 );
			 }

			 $$ = variable;
	               }
;

%%

#ifdef _DEVELOPPE_

static void aff_operation(int code, unsigned int delta)
{
  unsigned int i;

  for (i = 0 ; i < delta ; i++)
    printf("| ");

  switch (code)
  {
  case EXP_NOMBRE_ISOLE: printf("|on ignore le 2e lment"); break;
  case EXP_ADDITION:     printf("+"); break;
  case EXP_SOUSTRACTION: printf("-"); break;
  case EXP_PRODUIT:      printf("*"); break;
  case EXP_DIVISION:     printf("/"); break;

  case EXP_COSINUS: printf("cos()"); break;
  case EXP_SINUS: printf("sin()"); break;
  case EXP_TANGENTE: printf("tg()"); break;
  case EXP_COSH: printf("ch()"); break;
  case EXP_SINH: printf("sh()"); break;
  case EXP_TANH: printf("th()"); break;

  case EXP_EXP:   printf("exp()"); break;
  case EXP_EXP10: printf("exp10()"); break;
  case EXP_LOG:   printf("log()"); break;
  case EXP_LOG10: printf("log10()"); break;
  }
  printf("\n");
}

static void aff_noeud(Expression *noeud, unsigned int delta)
{
  unsigned int i;

  if (noeud == NULL)
    return;

  /* On s'occupe de l'indentation */
  if (delta > 0) {
    for (i = 0 ; i < delta - 1 ; i++)
      printf("| ");
    printf("|-");
  }

  printf("Noeud de type %d, code %d\n", noeud->type, noeud->code);
  if (noeud->type == EXP_CTE_CTE) {
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Valeur = %f\n", noeud->arg_1.valeur);
    aff_operation(noeud->code, delta);
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Valeur = %f\n", noeud->arg_2.valeur);   
  } else if (noeud->type == EXP_CTE_ARG) {
    printf("|-> Valeur = %f\n", noeud->arg_1.valeur);
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    aff_operation(noeud->code, delta);
    aff_noeud(noeud->arg_2.expression, delta + 1);
  } else if (noeud->type == EXP_ARG_CTE) {
    aff_noeud(noeud->arg_1.expression, delta + 1);
    aff_operation(noeud->code, delta);
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Valeur = %f\n", noeud->arg_2.valeur);
  } else if (noeud->type == EXP_ARG_ARG) {
    aff_noeud(noeud->arg_1.expression, delta + 1);
    aff_operation(noeud->code, delta);
    aff_noeud(noeud->arg_2.expression, delta + 1);
  } else if (noeud->type == EXP_CTE_VAR) {
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Valeur = %f\n", noeud->arg_1.valeur);   
    aff_operation(noeud->code, delta);
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Variable [%p : %g = %g, type %d]\n", (void *) noeud->arg_2.variable, noeud->arg_2.variable->valeur_initiale, VAR_prend_valeur( noeud->arg_2.variable ), noeud->arg_2.variable->type);
  } else if (noeud->type == EXP_VAR_CTE) {
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Variable [%p : %g = %g, type %d]\n", (void *) noeud->arg_1.variable, noeud->arg_1.variable->valeur_initiale, VAR_prend_valeur( noeud->arg_1.variable ), noeud->arg_1.variable->type);
    aff_operation(noeud->code, delta);
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Valeur = %f\n", noeud->arg_2.valeur);   
  } else if (noeud->type == EXP_VAR_VAR) {
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Variable [%p : %g = %g, type %d]\n", (void *) noeud->arg_1.variable, noeud->arg_1.variable->valeur_initiale, VAR_prend_valeur( noeud->arg_1.variable ), noeud->arg_1.variable->type);
    aff_operation(noeud->code, delta);
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Variable [%p : %g = %g, type %d]\n", (void *) noeud->arg_2.variable, noeud->arg_2.variable->valeur_initiale, VAR_prend_valeur( noeud->arg_2.variable ), noeud->arg_2.variable->type);
  } else if (noeud->type == EXP_ARG_VAR) {
    aff_noeud(noeud->arg_1.expression, delta + 1);
    aff_operation(noeud->code, delta);
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Variable [%p : %g = %g, type %d]\n", (void *) noeud->arg_2.variable, noeud->arg_2.variable->valeur_initiale, VAR_prend_valeur( noeud->arg_2.variable ), noeud->arg_2.variable->type);
  } else if (noeud->type == EXP_VAR_ARG) {
    for (i = 0 ; i < delta ; i++)
      printf("| ");
    printf("|-> Variable [%p : %g = %g, type %d]\n", (void *) noeud->arg_1.variable, noeud->arg_1.variable->valeur_initiale, VAR_prend_valeur( noeud->arg_1.variable ), noeud->arg_1.variable->type);
    aff_operation(noeud->code, delta);
    aff_noeud(noeud->arg_2.expression, delta + 1);
  }
}

#endif

static int quelle_fonction(char *nom_fonction)
{
  size_t i;

  if (nom_fonction == NULL) {
    return EXP_INDEFINIE;
  }

  /* On commence par convertir en minuscules */
  for (i = 0 ; i < strlen(nom_fonction) ; i++) {
    nom_fonction[i] = tolower( nom_fonction[i] );
  }
#ifdef _DEVELOPPE_
  printf("Fonction  chercher : %s\n", nom_fonction);
#endif

  /* On compare aux fonctions connues */
  if (strcmp(nom_fonction, "log") == 0)
    return EXP_LOG;
  if (strcmp(nom_fonction, "log10") == 0)
    return EXP_LOG10;
  if (strcmp(nom_fonction, "exp") == 0)
    return EXP_EXP;
  if (strcmp(nom_fonction, "exp10") == 0)
    return EXP_EXP10;
  if (strcmp(nom_fonction, "cos") == 0)
    return EXP_COSINUS;
  if (strcmp(nom_fonction, "sin") == 0)
    return EXP_SINUS;
  if ( (strcmp(nom_fonction, "tan") == 0) || (strcmp(nom_fonction, "tg") == 0) )
    return EXP_TANGENTE;
  if ( (strcmp(nom_fonction, "cosh") == 0) || (strcmp(nom_fonction, "ch") == 0) )
    return EXP_COSH;
  if ( (strcmp(nom_fonction, "sinh") == 0) || (strcmp(nom_fonction, "sh") == 0) )
    return EXP_SINH;
  if ( (strcmp(nom_fonction, "tanh") == 0) || (strcmp(nom_fonction, "th") == 0) )
    return EXP_TANH;
  if ( strcmp( nom_fonction, "LW" ) == 0 )
    return EXP_LAMBERTW0;

  return EXP_INDEFINIE;
}

static Variable *CF_TrouverParametre(const char *nom_variable, unsigned long n_parametres, struct parametre *parametres)
{
  unsigned int i;

  if ( NULL == nom_variable ) return NULL;
  if ( ( 0 == n_parametres ) || ( NULL == parametres ) ) {
    /* Aucune variable utilisateur */
    return NULL;
  }

  for ( i = 0 ; i < n_parametres ; i++ ) {
    if ( strcmp( nom_variable, parametres[ i ].symbole ) == 0 ) {
      return &(parametres[ i ].variable);
    }
  }

  /* Variable non trouve */
  return NULL;
}

#ifdef _ISOLE_
int main(void)
{
  debut = NULL;
  yyparse();

  aff_noeud(debut, 0);


  return 0;
}
#else
Expression *construit_arbre(const char *formule)
{
  struct yy_buffer_state *tampon_flex;

  if (formule == NULL) return NULL;

  debut = NULL;
  tampon_flex = formules_scan_string( formule );
  formulesparse();
  formules_delete_buffer(tampon_flex);

#ifdef _DEVELOPPE_
  aff_noeud(debut, 0);
#endif

  return debut;
}

/* -------- Analyse d'une formule, pouvant comprendre des paramtres -------- */

Expression *CF_ConstruireArbre(const char *formule, unsigned long n_parametres, struct parametre *parametres)
{
  Expression *arbre;

  /* Contrle prliminaire */
  if ( NULL == formule ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucune formule indique !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On mmorise la table des paramtres */
  if ( n_parametres > 0 ) {
    if ( nombre_parametres > 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : Une table de paramtres existe dj : il y a donc une analyse en cours\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }

    nombre_parametres = n_parametres;
    table_parametres = parametres; 
  }

  /* On analyse la formule */
  arbre = construit_arbre( formule );

  /* On libre la table de paramtres */
  table_parametres = NULL;
  nombre_parametres = 0;

  /* On a termin : on renvoie l'arbre */
  return arbre;
}

#endif

void yyerror (char *s)
{
  fprintf (stderr, "%s\n", s);
#ifdef _DEVELOPPE_
  aff_noeud( debut, 0 );
#endif
}
