/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 2003, Emmanuel CURIS

   Paramtrage de l'affichage du graphe [fentre principale]
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "systeme.h"
#include "exafs.h" /* Pour la mise  jour de la liste des graphes */

#include "interface.h"
#include "serie.h"
#include "tracer.h"

#include "grf_affichage.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------------- Modification du titre d'un graphe ---------------------------------------\
 |
 | graphe : le graphe  modifier
 | titre  : pointeur de la chane contenant le titre  installer
 \---------------------------------------------------------------------------------------- */

int GRF_titre(Graphe *graphe, const char *titre)
{
  ENTREE_FONCTION( __FUNCTION__, "graphe=%p titre=%s", graphe, titre );

  if ( (graphe == NULL) || (NULL == titre) ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [Erreur de paramtre]", -1 );
    return -1; /* Pas de graphe, ni de titre */
  }

  HIST_Titre(4, "Nommer un graphe", "GRF");
  if ( graphe->nom != NULL ) {
    HIST_Texte(4, 4, "Ancien nom  : %s\n", graphe->nom);
  }
  HIST_Texte(4, 4, "Nouveau nom : %s\n", titre);

  /* On libre l'ancien nom */
  free( graphe->nom ); graphe->nom = NULL;

  /* On fait une copie du nouveau nom. Sera libre au prochain changement ou en effaant le graphe */
  graphe->nom = strdup( titre );

  /* On modifie le nom du graphe principal -> le titre de la fentre est chang ! */
  if ( graphe == graphe_courant ) {
    FEN_titre( principale, graphe_courant->nom );
    GRF_PreparerListe( FAUX );
  }

  /* Le graphe n'est donc plus sauv ! */
  graphe->drapeaux.sauve = FAUX;

  /* Tout s'est bien pass */
  SORTIE_FONCTION( __FUNCTION__, "%d", 0 );
  return 0;
}

/* -------- Les informations pour l'affichage du graphe -------- */

int GRF_CreerAffichage(Graphe *const graphe)
{
  struct grf_infos_affichage *affichage;

  if ( NULL == graphe ) return 0;
  if ( graphe->affichage != NULL ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "GRF_CreerAffichage : le graphe a dj un affichage...\n");
#endif
    return +1; /* Erreur mineure */
  }

  /* On rserve pour les infos d'affichage du graphe */
  affichage = ( struct grf_infos_affichage * ) malloc( sizeof( struct grf_infos_affichage ) );
  if ( NULL == affichage ) {
    (void) fprintf(stderr, "GRF_CreerAffichage : Imposible de rserver la mmoire !\n");
    return -1;
  }

  /* On initialise */
  if ( GRF_FCT_COMPLEXE == graphe->genre_donnees )
    affichage->methode = GRF_C_IM_MOD;
  else
    affichage->methode = 0;
  affichage->couleur = NOIR;
  affichage->style   = BI_CONTINU;
  affichage->marque  = BI_MARQUE_AUCUNE;
  affichage->axe_x = NULL; /* Pas de lgende par dfaut */
  affichage->axe_y = NULL;
  affichage->unite_x = NULL;
  affichage->unite_y = NULL;

  /* Faut-il faire un affichage dans l'espace ? */
  if ( GRF_COURBE_ESPACE == graphe->genre_donnees ) {
    affichage->complement.infos_3d = (Infos_3d *) malloc( sizeof(Infos_3d) );
    if ( affichage->complement.infos_3d == NULL ) {
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "Graphes/affichage.c", "GRF_CreerAffichage", "affichage->infos_3d");
      return -2;
    }

    affichage->complement.infos_3d->z_min = 0.0; /* Bornes */
    affichage->complement.infos_3d->z_max = 0.0;
    affichage->complement.infos_3d->x_vu = 0.0;  /* Conditions d'affichage */
    affichage->complement.infos_3d->y_vu = 0.0;
    affichage->complement.infos_3d->z_vu = 0.0;
    affichage->complement.infos_3d->x_oeil = 0.0;
    affichage->complement.infos_3d->y_oeil = 0.0;
    affichage->complement.infos_3d->z_oeil = 0.0;
    affichage->complement.infos_3d->x_h = 0.0;
    affichage->complement.infos_3d->y_h = 0.0;
    affichage->complement.infos_3d->z_h = 0.0;
    affichage->complement.infos_3d->angle_v = 20.0;
    affichage->complement.infos_3d->angle_h = 0.0;
  } else {
    affichage->complement.infos_3d = NULL;
  }

  /* On l'associe au graphe */
  graphe->affichage = affichage;

  /* Tout s'est bien pass */
  return 0;
}

/* -------- On libre les infos complmentaires d'affichage -------- */

int GRF_LibererAffichage(Graphe *graphe)
{
  if ( NULL == graphe ) return -1;
  if ( NULL == graphe->affichage ) return 0;

  /* On libre le contenu */
  free( graphe->affichage->unite_x );/* Units */
  free( graphe->affichage->unite_y );
  free( graphe->affichage->axe_x ); /* Les lgendes des axes */
  free( graphe->affichage->axe_y );
  free( graphe->affichage->complement.infos_3d ); /* Les infos pour la vue en 3D */

  /* Plus besoin de la structure */
  free( graphe->affichage );
  graphe->affichage = NULL;

  /* Tout s'est bien pass */
  return 0;
}

/* ~-~-~-~-~-~-~-~- Le cas des incertitudes ~-~-~-~-~-~-~-~- */

void GRF_AfficherErreurs(Graphe *graphe, int axe, boolean afficher)
{
  boolean modifie = FAUX;

  if ( NULL == graphe ) return;

  if ( axe & AXE_X ) {
    if ( ( NULL == graphe->erreur_x ) && ( VRAI == afficher ) ) {
      (void) fprintf(stderr, "%s,%s l.%u : Le graphe %s n'a pas d'erreurs en X\n",
		     __FILE__, __FUNCTION__, __LINE__, graphe->nom);
    } else {
      if ( afficher != graphe->drapeaux.erreur_x ) {
	graphe->drapeaux.erreur_x = afficher;
	modifie = VRAI;
      }
    }
  }

  if ( axe & AXE_Y ) {
    if ( ( NULL == graphe->erreur_y ) && ( VRAI == afficher ) ) {
      (void) fprintf(stderr, "%s,%s l.%u : Le graphe %s n'a pas d'erreurs en Y\n",
		     __FILE__, __FUNCTION__, __LINE__, graphe->nom);
    } else {
      if ( afficher != graphe->drapeaux.erreur_y ) {
	graphe->drapeaux.erreur_y = afficher;
	modifie = VRAI;
      }
    }
  }

  /* Les redessins... uniquement si ncessaire ! */
  if ( VRAI == modifie ) {
    GRF_MiseAJour( graphe, FAUX );
  }
}

/* ~-~-~-~-~-~-~-~- Le cas des couleurs ~-~-~-~-~-~-~-~- */

/* -------- Modifier couleur et style de trac d'un graphe --------\
 |   adapte aussi les barres d'erreur si elles n'ont pas t       |
 |   modifies par l'utilisateur auparavant                        |
 |                                                                 |
 | graphe  : le graphe  modifier                                  |
 | couleur : la couleur  mettre (-1 = inchange)                  |
 | style   : le style de ligne  mettre (-1 = inchange)           |
 | marque  : la marque  mettre en chaque point (-1 = inchange)   |
 \---------------------------------------------------------------- */

void GRF_couleur(Graphe *graphe, int couleur, short int style, short int marque)
{
  boolean modifie = FAUX;

  if ( graphe == NULL ) return;
  if ( NULL == graphe->affichage ) {
    AVERTISSEMENT( "Pas d'infos d'affichage pour ce graphe [%s]", graphe->nom );
    return;
  }

  if ( couleur > 0 ) { /* 0 = blanc ! */
    if (graphe->erreur_y != NULL) {
      Graphe *erreur = graphe->erreur_y;
      if ( ( erreur->affichage != NULL ) &&
	   ( erreur->affichage->couleur == graphe->affichage->couleur ) ) {
	erreur->affichage->couleur = couleur;
      }
    }

    graphe->affichage->couleur = couleur;

    modifie = VRAI;
  }

  if ( style >= 0 ) { /* 0 = pas de ligne trace ! */
    if (graphe->erreur_y != NULL) {
      Graphe *erreur = graphe->erreur_y;
      if ( ( erreur->affichage != NULL ) &&
	   ( erreur->affichage->style == graphe->affichage->style ) ) {
	erreur->affichage->style = style;
      }
    }

    graphe->affichage->style = style;

    modifie = VRAI;
  }

  if ( marque >= 0 ) { /* 0 : aucun symbole spcial */
    /* Essai : sans modifier les erreurs, car maintenant elles ne sont plus affiches... */

    graphe->affichage->marque = marque;

    modifie = VRAI;
  }

  if ( VRAI == modifie ) {
    /* On a modifi au moins l'un des trois paramtres : on voit s'il faut redessiner quelque chose
       Se charge aussi d'indiquer qu'il n'est plus sauv...
     */
    GRF_MiseAJour( graphe, FAUX ); /* Changement de couleur : on n'a pas  changer la rgion 2D affiche ! */
  }
}

/* -------- Changer la couleur et le style d'un graphe pour qu'il suive juste ceux du modle -------- */

void GRF_CouleurSuivante(Graphe *graphe, Graphe *modele)
{
  unsigned int couleur, style, marque;

  /* Contrle prliminaire */
  if ( ( NULL == graphe ) || ( NULL == modele ) ) {
    AVERTISSEMENT( "Paramtres absurdes [graphe=%p, modele=%p]", (void *) graphe, (void *) modele );
    return;
  }
  if ( NULL == modele->affichage ) {
    AVERTISSEMENT( "Graphe modle sans information de couleur [%s]", modele->nom );
    return;
  }

  /* Quels sont les paramtres du graphe modle ? */
  couleur = modele->affichage->couleur;
  style   = modele->affichage->style;
  marque = modele->affichage->marque;

  /* Quelle est la couleur suivante dans l'interface graphique ? */
  TRC_CouleurSuivante( &couleur, &style, &marque );

  /* On applique cela au graphe cible */
  GRF_couleur( graphe, couleur, style, marque );
}

/* ~-~-~-~-~-~-~-~- Le cas des axes ~-~-~-~-~-~-~-~- */

/* -------- Quel est le texte  utiliser comme lgende ? --------
   Cette fonction renvoie ncessairement quelque chose,
   ELLE NE DOIT PAS RENVOYER NULL sauf si graphe = NULL
 */

const char *GRF_QuelleLegende(const Graphe *graphe, unsigned int axe)
{
  const char *texte = NULL;
  const char *unite = NULL;

  if ( NULL == graphe ) return NULL;

  /* 1. Y a-t-il une lgende par l'utilisateur ? */
  if ( graphe->affichage != NULL ) {
    if ( AXE_X == axe ) {
      texte = graphe->affichage->axe_x;
    } else if ( AXE_Y == axe ) {
      texte = graphe->affichage->axe_y;
    }
  }
  if ( texte != NULL ) return texte; /* On a trouv une lgende... */

  /* 2. Le type du graphe permet-il de dduire la lgende ? */
  if ( ( graphe->type >= GR_BRUT ) && ( graphe->type < NOMBRE_TYPES_GRAPHES ) ) {
    if ( AXE_Y == axe ) {
      texte = txt_legendes_Y[ graphe->type ];
    } else if ( AXE_X == axe ) {
      /* Axe des X : la plupart des cas, l'unit permet de dduire
	 -> on se fonde ici seulement sur le type pour les cas restants */
      switch ( graphe->type )
      {
      case GR_TF:		/* Tranforme de Fourier */
	texte = txt_unite_longueur_apparente;
	break;

      case GR_RDF:		/* Fonction de distribution radiale */
	texte = txt_unite_distance;
	break;

      case GR_EPREUVE:		/* Tirages d'une preuve alatoire */
	texte = txt_unite_epreuve;
	break;

      case GR_PROFIL:		/* Profil de concentration linaire */
	texte = txt_unite_distance_au_centre;
	break;

      case GR_RESOL_NRJ:	/* Rsolution en nergie */
	texte = txt_unite_energie_relative;
	break;

      case GR_FLUOX:		/* Spectre de fluorescence X */
	texte = txt_unite_canal;
	break;

      case GR_CONCENTRATION:	/* Profil de concentration [pH ?] */
	texte = txt_unite_pH;
	break;

      case GR_REP_DETECTEUR:	/* Rponse d'un dtecteur */
	texte = txt_unite_intensite_reelle;
	break;
      }
    }
  }
  if ( texte != NULL ) return texte; /* Lgende trouve */

  /* 3. Une unit permet-elle de construire le texte ? */
  unite = GRF_QuelleUnite( graphe, axe );
  if ( unite != NULL ) {
    int type_unite;

    /* On retrouve le type de grandeur */
    (void) TXT_NormaliserUnite( unite, FAUX, &type_unite, NULL );

    /* On en dduit la nature de la grandeur physique */
    texte = TXT_LegendeParDefaut( type_unite );
  }
  if ( texte != NULL ) return texte; /* Lgende trouve */

  /* Aucune lgende trouve */
  if ( AXE_X == axe )
    texte = "Abscisse";
  else if ( AXE_Y == axe )
    texte = "Ordonn"e_AIGU"e";

  /* On renvoie la lgende trouve */
  return texte;
}

/* -------- Quel est le texte  utiliser comme unit ? -------- */

const char *GRF_QuelleUnite(const Graphe *graphe, unsigned int axe)
{
  const char *unite = NULL;

  if ( NULL == graphe ) return NULL;

  /* 0. Contrle rapide : cas impossibles */
  if ( ( AXE_X == axe ) && ( NULL == graphe->x ) ) {
    return NULL;
  }

  /* 1. Y a-t-il une unit impose ? */
  if ( graphe->affichage != NULL ) {
    if ( AXE_X == axe ) {
      unite = graphe->affichage->unite_x;
    } else if ( AXE_Y == axe ) {
      unite = graphe->affichage->unite_y;
    }
    if ( unite != NULL ) {
      if ( 0 == *unite ) return NULL; /* "" = sans unit : inutile de le trimbaler partout... ! */
      return unite; /* Unit trouve */
    }
  }

  /* 2. On essaye de trouver l'unit  partir du type de graphe
     [ABANDONN CAR TROP DLICAT ET INTERFRE AVEC LE RESTE !] */
  if ( unite != NULL ) { return unite; }

  /* 3. Aucune unit trouve... */
  return NULL;
}

/* -------- Unit par dfaut. Utilis lors du chargement des anciennes versions de fichier.
   Actuellement, pour limiter les problmes, l'unit est _toujours_ indique dans le graphe, s'il y en a une (connue)
   -------- */

const char *GRF_UniteParDefaut(int type, unsigned int axe)
{
  const char *unite = NULL;

  if ( ( type >= GR_BRUT ) && ( type < NOMBRE_TYPES_GRAPHES ) ) {
    if ( AXE_X == axe )
      unite = txt_unites_X[ type ];
    else if ( AXE_Y == axe ) {
      /* La plupart peuvent avoir n'importe-quoi en Y, ou sont sans unit, ou en unit arbitraire, ou normaliss... */
      if ( GR_LIBREP == type ) {
	unite = ANGSTROM;	/* Libre parcours moyen */
      } else if ( GR_FLUOX == type ) {
	unite = "coups";	/* Spectre de fluorescence */
      } else if ( ( GR_CINETIQUE == type ) || ( GR_CONCENTRATION == type ) ) {
	unite = "M";		/* Concentration : en M = mol/L par dfaut */
      } else if ( GR_SEC_EFFICACE == type ) {
	unite = "g/cm";	/* pseudo-Section efficace (rapporte  une masse) */
      }
    }
  } else {
  }

  return unite;
}

/* -------- Modification de la lgende d'un des axes d'un graphe --------\
 |  Renvoie 0 si tout s'est bien pass, != 0 sinon                       |
 |    Si la lgende a t mmorise, elle sera libre lorsque la        |
 |       suivante sera installe                                         |
 |    En cas d'chec (code != 0), c'est  a charge de l'appelant de la   |
 |       librer !                                                       |
 |                                                                       |
 | graphe  : le graphe  modifier                                        |
 | axe     : l'axe sur lequel on travaille (AXE_X ou AXE_Y)              |
 | legende : pointeur de la chane contenant la lgende                  |
 | copier  : VRAI == on ralise une copie du texte pass (avec strdup)   |
 |           FAUX == on place directement le texte pass                 |
 |                   [il est alors libr en cas de problme !]          |
 \---------------------------------------------------------------------- */

int GRF_ChangerLegende(Graphe *graphe, unsigned int axe, char *legende, boolean copier)
{
  char *legende_reelle = NULL;
  const char *unite;

  if ( NULL == legende ) return 0;

  if ( graphe == NULL ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "GRF_legende : Appel pour un graphe null\n");
#endif
    if ( FAUX == copier ) free( legende );
    return -1; /* Pas de graphe... */
  }
  if ( NULL == graphe->affichage ) { /* Pas d'infos d'affichage */
    (void) fprintf(stderr, "%s,%s l.%u : Aucune info d'affichage / graphe %s\n",
		   __FILE__, __FUNCTION__, __LINE__, graphe->nom);
    if ( FAUX == copier ) free( legende );
    return -2;
  }

  /* Doit-on vraiment mettre la lgende ? */
  unite = GRF_QuelleUnite( graphe, axe );
  if ( unite != NULL ) {
    int type_grandeur;
    const char *legende_par_defaut;

    (void) TXT_NormaliserUnite( unite, FAUX, &type_grandeur, NULL );
    legende_par_defaut = TXT_LegendeParDefaut( type_grandeur );

    if ( ( legende_par_defaut != NULL ) &&
	 ( 0 == strcmp( legende_par_defaut, legende ) ) ) {
      /* La lgende peut tre dduite par l'unit -> on n'en met aucune */
      if ( FAUX == copier ) {
	free( legende );
	legende_reelle = NULL;
      }
      legende = NULL;
    }
  }

  /* Aprs contrles, il reste une lgende  mettre */
  if ( legende != NULL ) {
    if ( VRAI == copier ) {
      legende_reelle = strdup( legende );
      if ( NULL == legende_reelle ) {
	(void) fprintf(stderr, "GRF_legende : Impossible de copier le texte [%s]\n", legende);
	return +1;
      }
    } else {
      legende_reelle = legende;
    }
  }

  if (axe == AXE_X) {
    free( graphe->affichage->axe_x ); /* Plus besoin de l'ancienne lgende */
    graphe->affichage->axe_x = legende_reelle; /* On met la nouvelle ---  librer avec free plus tard ! */
  } else if (axe == AXE_Y) {
    free( graphe->affichage->axe_y ); /* Plus besoin de l'ancienne lgende */
    graphe->affichage->axe_y = legende_reelle; /* On met la nouvelle ---  librer avec free plus tard ! */
  }

  /* Tout s'est bien pass */
  return 0;
}

int GRF_ChangerUnite(Graphe *graphe, unsigned int axe, const char *unite)
{
  int retour = 0;
  char *nv_unite = NULL;

  if ( NULL == graphe ) return -1;
  if ( NULL == graphe->affichage ) return -2;

  /* On commence par copier, car unite peut tre =  un texte dans le graphe, la libration poserait donc des problmes */
  if ( unite != NULL ) {
    nv_unite = strdup( unite );
  }

  if ( AXE_X == axe ) {
    free( graphe->affichage->unite_x );
    graphe->affichage->unite_x = nv_unite;
  } else if ( AXE_Y == axe ) {
    free( graphe->affichage->unite_y );
    graphe->affichage->unite_y = nv_unite;
  } else {
    OPTION_A_FAIRE( "Axe autre que X/Y" );
    retour = -5;
  }

  if ( 0 == retour ) {
    (void) fprintf(stderr, "%s,%s l.%u : Nouvelle unit du graphe : %s = %s = %s\n",
		   __FILE__, __FUNCTION__, __LINE__, nv_unite, unite, GRF_QuelleUnite( graphe, axe ) );

    /* Tout s'est bien pass -> redessins ventuels */
    GRF_MiseAJour( graphe, FAUX );
  }

  /* Termin */
  return retour;
}

/* -------- Renvoie la rgion du plan qui contient au mieux le graphe indiqu -------- */

int GRF_OptimiserAffichage(Graphe *graphe, double *x_min, double *x_max, double *y_min, double *y_max)
{
  int methode;

  if ( NULL == graphe ) return -1;

  /* Cas le plus frquent : les bornes conviennent bien */
  *x_min = graphe->x_min;
  *x_max = graphe->x_max;
  *y_min = graphe->y_min;
  *y_max = graphe->y_max;

  if ( NULL == graphe->affichage ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "GRF_OptimiserAffichage : Pas d'infos d'affichage => on utilise les bornes\n");
#endif
    return +1; /* Erreur mineure */
  }

  methode = graphe->affichage->methode;

  switch ( graphe->genre_donnees )
  {
  case GRF_FCT_REELLE:
    if ( GRF_R_CARTESIEN == methode ) {
      /* Rien de spcial  faire */
    } else if ( GRF_R_POLAIRE == methode ) {
      double x, y, *r, *theta;
      unsigned long i;

      *x_min = graphe->x[ 0 ] * cos( graphe->y[ 0 ] ); *x_max = *x_min;
      *y_min = graphe->x[ 0 ] * sin( graphe->y[ 0 ] ); *y_max = *y_min;
      for ( i = 1, r = graphe->x + 1, theta = graphe->y + 1 ; i < graphe->nombre_point ; i++, r++, theta++ ) {
	x = *r * cos( *theta );
	y = *r * sin( *theta );

	if ( x < *x_min ) *x_min = x; if ( x > *x_max ) *x_max = x;
	if ( y < *y_min ) *y_min = y; if ( y > *y_max ) *y_max = y;
      }
    } else {
      AVERTISSEMENT( "GRF_FCT_REELLE, mthode inconnue [%d]", methode);
    }
    break;

  case GRF_FCT_COMPLEXE:
    if ( GRF_C_IM_MOD == methode ) {
    } else if ( GRF_C_RE_MOD == methode ) {
    } else if ( GRF_C_RE_IM == methode ) {
    } else {
      AVERTISSEMENT( "GRF_FCT_COMPLEXE, mthode inconnue [%d]", methode);
    }
    break;

  case GRF_SERIE:
    if ( GRF_S_NORMAL == methode ) {
      /* Rien de spcial  faire */
    } else if ( ( GRF_S_BOXPLOT == methode ) || ( GRF_S_TUKEY == methode ) ) {
      /* Bote  moustache : valeurs en Y, mais la bote est horizontale -> passe en X
	 La hauteur n'a aucune importance concrte... */
      *x_min = graphe->y_min;
      *x_max = graphe->y_max;
    } else {
      AVERTISSEMENT( "GRF_SERIE, mthode inconnue [%d]", methode);
    }
    break;

  case GRF_COURBE_PLANE:
    break;

  case GRF_COURBE_ESPACE:
  case GRF_SURFACE:
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "GRF_OptimiserAffichage : Graphe 3D, pas de rgion plane !\n");
#endif
    return +2;

  case GRF_HISTOGRAMME:
    /* a. Optimisation des X */
    if ( graphe->z.effectifs[ 0 ] > 0 ) {
      /* Classe ]-infini, x_min] non-vide : on doit la dessiner
         on la prend de mme largeur que la premire classe (arbitraire !) */
      *x_min -= ( graphe->x[ 1 ] - graphe->x[ 0 ] );
    }
    if ( graphe->z.effectifs[ graphe->nombre_point ] > 0 ) {
      /* Classe [x_max, +infini[ non-vide : on doit la dessiner
         on la prend de mme largeur que la dernire classe (arbitraire !) */
      *x_max += ( graphe->x[ graphe->nombre_point - 1 ] - graphe->x[ graphe->nombre_point - 2 ] );
    }

    /* b. Optimisation des Y */
    *y_min = 0.0;		/* On commence toujours aux effectifs nuls ! */
    if ( GRF_H_EFFECTIF == methode ) {
      /* Rien de spcial  faire */
    } else if ( GRF_H_DP == methode ) {
      double d_max = 0.0, lg_classe;
      unsigned long i, e_total = 0;

      /* Quel est l'effectif total ? */
      for ( i = 0 ; i <= graphe->nombre_point ; i++ ) {
	e_total += graphe->z.effectifs[ i ];
      }

      /* On cherche quelle est la d.p. maximale [hors classes extrmes] */
      for ( i = 1 ; i < graphe->nombre_point ; i++ ) {
	lg_classe = graphe->x[ i ] - graphe->x[ i - 1 ];
	if ( lg_classe > 0.0 ) {
	  double dp;

	  dp = ((double) graphe->z.effectifs[ i ]) / ( lg_classe * (double) e_total );
	  if ( dp > d_max ) d_max = dp;
	}
      }

      /* Cette densit sert de maximum ; le minimum est 0 */
      *y_max = d_max;
    } else if ( GRF_H_FREQUENCE == methode ) {
      /* Une frquence est forcment comprise entre 0 et 1 */
      *y_max = 1.0;
      return 0;
    }
    break;

  case GRF_FAISCEAU:
    (void) fprintf(stderr, "GRF_OptimiserAffichage : GRF_FAISCEAU,  FAIRE\n");
    break;

  case GRF_MULTI_SERIE:
    (void) fprintf(stderr, "GRF_OptimiserAffichage : GRF_MULTI_SERIE,  FAIRE\n");
    break;

  default:
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "GRF_OptimiserAffichage : type de donnes inconnu [%d] !\n", graphe->genre_donnees);
#endif
    return +3;
  }

  /* Si une coordonne est constante : on ajoute 10% de chaque ct, pour y voir quelque chose ! */
  if ( *x_max == *x_min ) {
    *x_max *= 1.1;
    *x_min *= 0.9;
  }
  if ( *x_max == *x_min ) {
    /* Si c'est encore le cas : valeurs nulles ou quasi-nulles :arbitrairement, entre -1 et 1 ! */
    *x_min = -1.0;
    *x_max = 1.0;
  }

  if ( *y_max == *y_min ) {
    *y_max *= 1.1;
    *y_min *= 0.9;
  }
  if ( *y_max == *y_min ) {
    /* Si c'est encore le cas : valeurs nulles ou quasi-nulles :arbitrairement, entre -1 et 1 ! */
    *y_min = -1.0;
    *y_max = 1.0;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- On informe que le graphe a t mis  jour : redessins si ncessaire -------- */

void GRF_MiseAJour(Graphe *graphe, boolean remettre_a_l_echelle)
{
  struct serie *serie;

  if ( NULL == graphe ) return;

  /* Le graphe ne peut plus tre considr comme sauvegard */
  graphe->drapeaux.sauve = FAUX;

  /* Doit-on redessiner la fentre principale ? */
  if ( graphe == graphe_courant ) {
    /* On optimise si ncessaire l'affichage et on le met  jour (units, lgendes) */
    I_MettreAJourPrincipale( remettre_a_l_echelle );

    /* On remet  jour la liste des graphes */
    GRF_PreparerListe( FAUX );
  }

  /* Doit-on redessiner la fentre de superposition ? */
  serie = GRF_QuelleSerie( graphe );
  if ( serie == serie_courante ) {
    /* On est dans la meme srie : possible [la fonction ci-dessous contrle que c'est bien le cas !] */
    TRC_RedessinerSuperposition( graphe, FAUX, remettre_a_l_echelle );
  }

  /* Doit-on redessiner les figures ? */
  OPTION_A_FAIRE( "Vrifier le redessin ventuel des figures" );
}
