/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 9 janvier 2002

   Gestion gnrique des graphes

   Slecteur de graphes (slection simple ou multiple)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "serie.h"

#include "exafs.h"
#include "interface.h"
#include "tracer.h"

#include "grf_choisir.txt"

/* ~~~~~~~~ Structures locales ~~~~~~~ */

typedef struct {
  GRF_liste *liste;
  Serie_Graphes *serie;
  boolean strict;
  int type;
} Infos_ChoixGraphe_XX;

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LGMAX_TXT_TYPE_GRAPHE (49)
#define LGMAX_TXT_NOM_SERIE   (59)

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int Liste_InitialiserLongueur(GRF_liste *liste);

static int CLV_choix(unsigned int touche, int etat, int *renvoi);
static int GER_choix(unsigned int num_objet, int x, int y);

static boolean type_graphe_correct(int type, Graphe *graphe, boolean strict);

static unsigned long GRF_CHX_CompterValables(const struct serie *const serie, const Graphe *const modele, unsigned long reference, unsigned long strict);
static boolean       GRF_CHX_EstValable     (const Graphe *const graphe, const Graphe *const modele, unsigned long reference, unsigned long strict);
static boolean       GRF_CHX_GenreValable   (const Graphe *const graphe, const Graphe *const modele, boolean strict);
static boolean       GRF_CHX_TypeValable    (const Graphe *const graphe, const Graphe *const modele, boolean strict);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~- Cration d'une liste de graphes ~-~-~-~-~-~-~-~-~-~- */

/* -------- Forme la plus gnrique --------
     Renvoie la table des graphes slectionns ; le nombre sera dans *nombre
             NULL en cas de problme [ou si aucun graphe !]

  serie     : la srie dans laquelle chercher. NULL => on cherche dans toutes les sries
  modele    : graphe contenant les valeurs  utiliser dans la comparaison
  reference : masque de bits, quels sont les points  contrler
  strict    : masque de bits, parmi ces points, quels sont ceux qui doivent tre strictement gaux
  -------- */

Graphe **GRF_ChoisirMasque(const struct serie *const serie, const Graphe *const modele, unsigned long reference, unsigned long strict, unsigned long *const nombre)
{
  Graphe **table;
  unsigned long nombre_trouves = 0;

  /* A-t-on les informations indispensables ? */
  if ( ( NULL == nombre ) || ( NULL == modele ) ) return 0;

  if ( NULL == serie ) {
    const struct serie *serie;

    for ( serie = premiere_serie ; serie != NULL ; serie = serie->suivante ) {
      nombre_trouves += GRF_CHX_CompterValables( serie, modele, reference, strict );
    }
    /* Ici, serie = NULL => on le reprera encore dans la 2e tape : OK */
  } else {
    /* Combien de graphes conviennent ? */
    nombre_trouves = GRF_CHX_CompterValables( serie, modele, reference, strict );
  }
  *nombre = nombre_trouves;

  /* A-t-on trouv quelque chose ? */
  if ( 0 == nombre_trouves ) {
    /* Aucun graphe trouv ! */
    return NULL;
  }

  /* On rserve la table des graphes */
  table = (Graphe **) calloc( nombre_trouves, sizeof( Graphe * ) );
  if ( NULL == table ) {
    /* Mmoire insuffisante -> on fait comme si rien n'tait trouv ! */
    ERR_Memoire( "Table des graphes trouvs", nombre_trouves * sizeof( Graphe * ) );
    *nombre = 0;
    return NULL;
  }

  /* On remplit la table */
  if ( NULL == serie ) {
    OPTION_A_FAIRE( "Remplir la table, cas multi-sries" );
  } else {
    Graphe *graphe;
    unsigned long i;

    for ( i = 0, graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
      if ( GRF_CHX_EstValable( graphe, modele, reference, strict ) ) {
	/* Le graphe convient : on l'insre dans la table */
	table[ i ] = graphe;
	i++;
      }
    }
  }

  /* On renvoie la table ainsi cre */
  return table;
}

/* -------- Compter le nombre de graphes valables dans une srie -------- */

static unsigned long GRF_CHX_CompterValables(const struct serie *const serie, const Graphe *const modele, unsigned long reference, unsigned long strict)
{
  Graphe *graphe;
  unsigned long nombre_valables;

  for ( nombre_valables = 0, graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
    if ( VRAI == GRF_CHX_EstValable( graphe, modele, reference, strict ) ) {
      nombre_valables++;
    }
  }

  return nombre_valables;
}

/* -------- Un graphe est-il valable compte-tenu des critres de filtre indiqus ? -------- */

static boolean GRF_CHX_EstValable(const Graphe *const graphe, const Graphe *const modele, unsigned long reference, unsigned long strict)
{
  boolean reponse;

  if ( reference & GRF_CHOIX_GENRE ) {
    /* Contrler le genre du graphe */
    reponse = GRF_CHX_GenreValable( graphe, modele, strict & GRF_CHOIX_GENRE );
    if ( reponse != VRAI ) return FAUX;	/* On n'est pas en accord pour le genre */
  }

  if ( reference & GRF_CHOIX_TYPE ) {
    /* Contrler le type du graphe */
    reponse = GRF_CHX_TypeValable( graphe, modele, strict & GRF_CHOIX_TYPE );
    if ( reponse != VRAI ) return FAUX;	/* On n'est pas en accord pour le genre */
  }

  if ( reference & GRF_CHOIX_NOMBRE ) {
    /* Contrler le nombre de points */
    OPTION_A_FAIRE( "Contrler le nombre de points" );
  }

  if ( reference & GRF_CHOIX_UNITE_X ) {
    /* Contrler l'unit en X */
    OPTION_A_FAIRE( "Contrler l'unit en X" );
  }

  if ( reference & GRF_CHOIX_X ) {
    /* Contrler les valeurs en X */
    OPTION_A_FAIRE( "Contrler les valeurs en X" );
  }

  if ( reference & GRF_CHOIX_UNITE_Y ) {
    /* Contrler l'unit en Y */
    OPTION_A_FAIRE( "Contrler l'unit en Y" );
  }

  if ( reference & GRF_CHOIX_Y ) {
    /* Contrler les valeurs en Y */
    OPTION_A_FAIRE( "Contrler les valeurs en Y" );
  }

  /* Si l'on a tout pass, c'est que le graphe convient */
  return VRAI;
}

/* -------- Contrler que le genre du graphe convient -------- */

static boolean GRF_CHX_GenreValable(const Graphe *const graphe, const Graphe *const modele, boolean strict)
{
  /* Ont-ils le mme genre exactement ? */
  if ( modele->genre_donnees == graphe->genre_donnees ) return VRAI;

  /* Non : le contrle est-il strict ? */
  if ( strict != 0 ) return FAUX;

  OPTION_A_FAIRE( "Contrle de genre, non strict" );
  return FAUX;
}

/* -------- Contrler que le type du graphe convient --------- */

static boolean GRF_CHX_TypeValable(const Graphe *const graphe, const Graphe *const modele, boolean strict)
{
  /* Ont-ils le mme type exactement ? */
  if ( modele->type == graphe->type ) return VRAI;

  /* Non : le contrle est-il strict ? */
  if ( strict != 0 ) return FAUX;

  OPTION_A_FAIRE( "Contrle de type, non strict" );
  return FAUX;
}

/* ~-~-~-~-~-~-~- Gestion d'une liste de graphes slectionns ~-~-~-~-~-~-~-~-~-~- */

/* --------------- Prparation de la liste de tous les graphes du mme type -------\
 |   Renvoie un pointeur sur la liste cre ( librer aprs utilisation)          |
 |           NULL si la cration de la liste a choue                             |
 |                                                                                 |
 | type   : le type de graphe  prendre en compte                                  |
 |   (type < -100 : types spciaux, pour slections plus vastes. Voir graphes.h    |
 | alerte : VRAI ==> on prvient si aucun graphe n'a t trouv                    |
 | strict : FAUX ==> on intgre les graphes sans type                              |
 | serie  : la srie servant  constituer la liste                                 |
 |          NULL == srie courante uniquement                                      |
 \-------------------------------------------------------------------------------- */

GRF_liste *GRF_par_type(int type, unsigned int options, Serie_Graphes *serie)
{
  GRF_liste *liste;
  Graphe *graphe;
  unsigned long i;

  /* Avec quelle srie doit-on travailler ? */
  if ( NULL == serie ) {
    serie = serie_courante;
    if ( NULL == serie ) { /* Aucune srie -> aucun graphe ! */
      return NULL;
    }
  }

  liste = (GRF_liste *) malloc( (size_t) sizeof(GRF_liste) );
  if (liste == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "interface.c", "GRF_par_type", "liste");
    return NULL;
  }

  if ( options & GRF_LISTE_TYPES_MULTISERIE ) { /* On cherche dans toutes les sries */
    liste->nbr_graphe = GRF_compter_type( type, options & GRF_LISTE_TYPES_STRICT, NULL );
  } else {
    liste->nbr_graphe = GRF_compter_type( type, options & GRF_LISTE_TYPES_STRICT, serie );
  }
#ifdef _DEVELOPPE_
  if ( options & GRF_LISTE_TYPES_MULTISERIE )
    printf("%s : on compte %ld graphes valables au total, toutes sries confondues\n", __FUNCTION__, liste->nbr_graphe);
  else
    printf("%s : on compte %ld graphes valables dans cette srie [%s]\n", __FUNCTION__, liste->nbr_graphe, serie->nom);
#endif

  if (liste->nbr_graphe == 0) { /* Aucun graphe ne convient */
    if ( GRF_LISTE_TYPES_MESSAGES & options ) { /* On prvient l'utilisateur */
      const char *txt_type = NULL;

      if ( (type >= 0) || (type == GR_AUTRETYPE) ) { /* Seuls les graphes du mme type conviennent */
	txt_type = GRF_TexteTypeGraphe( type );
      } else if ( (type == GR_SSTYPE) || (type == GR_TOUS) ) {
	txt_type = "Tous les graphes";
      } else if ( type == GR_TT_EXAFS ) { /* Tous les spectres de type EXAFS */
	txt_type = "Tout spectre EXAFS";
      } else {
	fprintf(stderr, "GRF_par_type : type %d inconnu - annul\n", type);
      }

      EXF_ERREUR( AUCUN_GRAPHE, txt_type );
    }
    free(liste);
    return NULL;
  }

  liste->graphes = (Graphe **) calloc( liste->nbr_graphe, sizeof (Graphe *) );
  if (liste->graphes == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "interface.c", "GRF_par_type", "liste->graphes");
    free(liste);
    return NULL;
  }

  if ( options & GRF_LISTE_TYPES_MULTISELECTION ) {
    liste->choisi = ( boolean * ) calloc( liste->nbr_graphe, sizeof ( boolean ) );
    if ( NULL == liste->choisi ) {
      SYS_ERREUR( MANQUE_MEMOIRE, __FILE__, __FUNCTION__, "liste->choisi" );
      free( liste->graphes );
      free( liste );
      return NULL;
    }
  } else {
    liste->choisi = NULL;   /* N'existe qu'en cas de slection multiple */
  }

  liste->min = 1;         /* Choisir au moins un graphe */
  liste->max = 0;         /* Pas de limite suprieure   */
  liste->prem_graphe = 0; /* On commence au premier graphe, en haut de la liste */
  liste->num_select  = 0; /*   qui est le graphe choisi par dfaut */
  liste->delta_x = 0;     /* Affichage  la premire lettre du nom */

  i = 0;
  if ( options & GRF_LISTE_TYPES_MULTISERIE ) {
    Serie_Graphes *serie;

    for ( serie = premiere_serie ; serie != NULL ; serie = serie->suivante ) {
      for (graphe = serie->premier_graphe ; graphe ; graphe = graphe->suivant ) {
	if ( VRAI == type_graphe_correct(type, graphe, options & GRF_LISTE_TYPES_STRICT ) ) { /* On ajoute le graphe  la liste */
	  liste->graphes[i] = graphe;
	  i++; /* On prpare le suivant */
	}
      }
    }
  } else {
    for (graphe = serie->premier_graphe ; graphe ; graphe = graphe->suivant ) {
      if ( VRAI == type_graphe_correct(type, graphe, options & GRF_LISTE_TYPES_STRICT ) ) { /* On ajoute le graphe  la liste */
	liste->graphes[i] = graphe;
	i++; /* On prpare le suivant */
      }
    }
  }

  /* Longueur maximale des noms */
  Liste_InitialiserLongueur( liste );

  return liste; /* La liste est prte, normalement */
}

/* -------- Librer une liste de graphes -------- */

inline int GRF_LibererListe( GRF_liste *liste )
{
  if ( NULL == liste ) return -1;

  free( liste->choisi ); liste->choisi = NULL;
  free( liste->graphes ); liste->graphes = NULL;

  free( liste );

  /* Fini, tout s'est bien pass */
  return 0;
}

static boolean type_graphe_correct(int type, Graphe *graphe, boolean strict)
{
  if ( NULL == graphe ) return FAUX;

  if (type == graphe->type) return VRAI;

  if (FAUX == strict) {
    if ( GR_SSTYPE == graphe->type ) return VRAI;
  }

  if (GR_TOUS == type) return (graphe->type != GR_INTERNE);

  if ( GR_TT_EXAFS == type ) {
    if ( VRAI == graphe->drapeaux.exafs ) return VRAI;
  }

  return FAUX;
}

/* ------ Chercher la longueur maximale des noms de graphes -------- */

static int Liste_InitialiserLongueur(GRF_liste *liste)
{
  unsigned long i;

  if ( NULL == liste ) return -1;
  if ( NULL == liste->graphes ) return -2;

  liste->lgmax_txt = 0;
  for ( i = 0 ; i < liste->nbr_graphe ; i++ ) {
    const char *nom;
    size_t lg_nom;

    if ( NULL == liste->graphes[ i ] ) {
      AVERTISSEMENT( "Le graphe %lu est NULL", i );
      continue;
    }
    nom = liste->graphes[ i ]->nom;
    if ( NULL == nom ) continue; /* Pas de nom ! */

    lg_nom = strlen( nom );
    if ( lg_nom > liste->lgmax_txt )
      liste->lgmax_txt = lg_nom;
  }

  return 0;
}

/* --------------- Affiche le slecteur avec les infos de la liste ----------------\
 |   Pour l'instant, ne renvoie rien (-> toujours VRAI)                            |
 |                                                                                 |
 | liste : liste des graphes  afficher, avec numro du graphe slectionn et tout |
 | arbre : numro de l'arbre dans lequel se trouve le slecteur                    |
 | prem_nom : numro d'objet de la chane affichant le premier nom                 |
 | der_nom  : ---------------------------------------- dernier nom                 |
 | max_lg   : longueur maximale du texte affichable dans le dfileur               |
 | avant    : numro d'objet de la flche vers le haut                             |
 | apres    : ----------------------------------- bas                              |
 | fond     : numro d'objet du fond d'ascenseur                                   |
 | bouton   : numro d'objet du curseur de l'ascenseur                             |
 \-------------------------------------------------------------------------------- */

boolean GRF_affiche(GRF_liste *liste, unsigned int arbre, unsigned int prem_nom, unsigned int der_nom, unsigned int max_lg,
		    unsigned int avant, unsigned int apres, unsigned int fond, unsigned int bouton)
{
  unsigned int nbr_noms, i; /* Nombre de graphes affichable simultanment par le slecteur */
  char *nom_graphe;

  nbr_noms = der_nom - prem_nom + 1;
  if ( nbr_noms <= 0 ) /* Incohrence... */
    return FAUX;

  if (liste == NULL) { /* Aucune info de liste => on coupe tout */
#ifdef _DEVELOPPE_
    printf("GRF_affiche : liste nulle --- on coupe tout...\n");
#endif
    for (i = 0 ; i < nbr_noms ; i++) {
      OBJ_active (arbre, prem_nom + i, FORCE_NON);
      OBJ_choisit(arbre, prem_nom + i, FORCE_NON);
      met_chaine(arbre, prem_nom + i, max_lg, " ", TXT_COPIER);
    }
    ARB_ascenseur(arbre, avant, fond, bouton, apres, ASC_VERTICAL, 
		  0, nbr_noms, 0, ROOT, -1, -1, METTRE);
    
    return VRAI;
  }

  ARB_ascenseur(arbre, avant, fond, bouton, apres, ASC_VERTICAL, 
		liste->nbr_graphe, nbr_noms, liste->prem_graphe, ROOT, -1, -1, METTRE);

  if ( nbr_noms >= liste->nbr_graphe ) { /* Moins de graphes que de noms affichables */
    for (i = 0 ; i < liste->nbr_graphe ; i++) {
      OBJ_active (arbre, prem_nom + i, FORCE_OUI);
      if (liste->choisi)
	OBJ_choisit(arbre, prem_nom + i, liste->choisi[i] == VRAI);
      else
	OBJ_choisit(arbre, prem_nom + i, i == liste->num_select);
      if ( liste->graphes[ i ] != NULL ) {
	nom_graphe = liste->graphes[i]->nom;
      } else {
	nom_graphe = "";
#ifdef _DEVELOPPE_
	nom_graphe = " << NULL >> ";
#endif
      }
      if ( liste->delta_x < strlen( nom_graphe ) )
	nom_graphe += liste->delta_x;
      else
	nom_graphe = " ";
      met_chaine( arbre, prem_nom + i, max_lg, nom_graphe, TXT_COPIER );
    }

    /* On continue la prparation */
    for ( ; i < nbr_noms ; i++) {
      OBJ_choisit(arbre, prem_nom + i, FORCE_NON);
      OBJ_active (arbre, prem_nom + i, FORCE_NON);
      met_chaine (arbre, prem_nom + i, max_lg, " ", TXT_COPIER);
    }
    
    return VRAI; /* Fini */
  }

  /* Plus de graphes slectionns que de graphes affichables */
  for (i = 0 ; i < nbr_noms ; i++) {
    OBJ_active (arbre, prem_nom + i, FORCE_OUI);
    if (liste->choisi != NULL)
      OBJ_choisit(arbre, prem_nom + i, liste->choisi[liste->prem_graphe + i] == VRAI);
    else
      OBJ_choisit(arbre, prem_nom + i, i + liste->prem_graphe == liste->num_select);
    nom_graphe = liste->graphes[liste->prem_graphe + i]->nom;
    printf("Nom du graphe : %s\n", nom_graphe);
    if (liste->delta_x < strlen(nom_graphe))
      nom_graphe += liste->delta_x;
    else
      nom_graphe = " ";
    met_chaine(arbre, prem_nom + i, max_lg, nom_graphe, TXT_COPIER);
  }

  return VRAI;
}

/* --------------- Gestion d'un slecteur (un seul graphe parmi une liste) --------\
 |    Renvoie VRAI si tout s'est bien pass, FAUX sinon                            |
 |    Hypothse : il n'y aura jamais plus de max(unsigned int) graphes affichables |
 |       (raisonnable : fait 65536 graphes si sizeof(int) = 2 !                    |
 |                                                                                 |
 | num_objet : l'objet sur lequel on a cliqu                                      |
 | x, y      : coordonnes de la souris lors du clic (ncessaire pour l'ascenseur) |
 |    Les autres paramtres sont comme ci_dessus                                   |
 \-------------------------------------------------------------------------------- */

boolean GRF_selecteur(GRF_liste *liste, unsigned int arbre, unsigned int prem_nom, unsigned int der_nom, unsigned int max_lg,
                      unsigned int avant, unsigned int apres, unsigned int fond, unsigned int bouton,
		      unsigned int num_objet, int x, int y)
{
  unsigned int nbr_noms, i;

  if (liste == NULL) return FAUX; /* Pas de liste de graphes... */
  
  nbr_noms = der_nom - prem_nom + 1;
  if (nbr_noms <= 0) return FAUX; /* Absurde... */

  if ( (num_objet >= prem_nom) && (num_objet <= der_nom) ) {
    if (liste->choisi) { /* Slection multiple */
      liste->choisi[liste->prem_graphe + (num_objet - prem_nom)] = OBJ_choisit(arbre, num_objet, ECHANGE);
    } else { /* Slection simple */
      /* Position du graphe choisi par rapport au premier nom visible */
      i = (unsigned int)  (liste->num_select - liste->prem_graphe);

      if (num_objet - prem_nom == i) /* Clic sur le graphe dj choisi */
	return VRAI; /* Rien  faire */

      if ( i < nbr_noms ) { /* Graphe avant choisi est visible */
	OBJ_choisit(arbre, (int) (prem_nom + i), FORCE_NON); /* On dslectionne */
      }

      liste->num_select = liste->prem_graphe + (num_objet - prem_nom); /* Nouveau graphe choisi */
      OBJ_choisit(arbre, num_objet, FORCE_OUI); /* On slectionne */
    }
    
    return VRAI;
  }

  if ( (num_objet != avant) && (num_objet != apres) && (num_objet != fond) && (num_objet != bouton) ) {
#ifdef _DEVELOPPE_
    fprintf(stderr, "GRF_selecteur : objet pas dans le slecteur ! Arbre %d, objet cliqu %d\n", arbre, num_objet);
#endif
    return FAUX;
  }
  /* Donc si on est l, c'est que l'on a cliqu sur l'ascenseur */
  liste->prem_graphe = ARB_ascenseur(arbre, avant, fond, bouton, apres, ASC_VERTICAL,
				     liste->nbr_graphe, der_nom - prem_nom + 1, liste->prem_graphe,
				     num_objet, x, y, ECHANGE);
  GRF_affiche(liste, arbre, prem_nom, der_nom, max_lg, avant, apres, fond, bouton);

  return VRAI;
}

/* ~-~-~-~-~-~-~- Slection simple ~-~-~-~-~-~-~- */

/* -------------- Choisir un graphe d'un type donn ---------------------------------------\
 |   Renvoie le graphe choisi (NULL == erreur)                                             |
 |                                                                                         |
 | type : le type de graphes que l'on veut afficher                                        |
 |   (reconnat quelques types spciaux : si type <= -100 - cf. graphes.h                  |
 | strict : FAUX => les graphes sans type sont intgrs dans la liste                      |
 | multi_serie : VRAI => on a le droit de changer de srie dans le slecteur               |
 | serie       : la srie pour l'affichage (NULL => prendre la srie courante)             |
 \---------------------------------------------------------------------------------------- */

Graphe *GRF_choisir(int type, boolean strict, boolean multi_serie, Serie_Graphes *serie)
{
  Infos_ChoixGraphe_XX infos;
  Graphe *retour = NULL;
  int reponse;
  unsigned int options = 0U;

  if ( NULL == serie ) serie = serie_courante;

  if ( VRAI == strict ) options |= GRF_LISTE_TYPES_STRICT;
  if ( VRAI == multi_serie ) options |= GRF_LISTE_TYPES_MULTISERIE;
  infos.liste = GRF_par_type(type, options, serie);
  if (infos.liste == NULL) return NULL; /* Problme de slection */

  OBJ_active(CHX_GRAPHE, CHXG_OK, FORCE_OUI);
  OBJ_active(CHX_GRAPHE, CHXG_SERIE, multi_serie); /* Slection de la srie possible ? */

  /* Initialisation du dtecteur */
  OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, USR_graphe, (long) (infos.liste->graphes[infos.liste->num_select]), METTRE);
  ARB_PrepareSelecteurObjets( FAUX, GER_choix, CLV_choix );

  GRF_affiche(infos.liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
	      CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR);
  ARB_ascenseur(CHX_GRAPHE, CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL,
		infos.liste->lgmax_txt, LGMAX_TXT_NOM_OBJET_SELECTEUR, infos.liste->delta_x, ROOT, -1, -1, METTRE);

  /* On doit utiliser TXT_REMPLACE, car si v. traduite, texte constant => copie donne une seg. fault ! */
  if (type == GR_EXAFS)
    met_chaine(CHX_GRAPHE, CHXG_TITRE, LGMAX_TXT_TYPE_GRAPHE, txt_chx_grf_ttexafs, TXT_REMPLACE);
  else if (type == GR_TOUS)
    met_chaine(CHX_GRAPHE, CHXG_TITRE, LGMAX_TXT_TYPE_GRAPHE, txt_chx_grf_tous, TXT_REMPLACE);
  else
    met_chaine(CHX_GRAPHE, CHXG_TITRE, LGMAX_TXT_TYPE_GRAPHE, GRF_TexteTypeGraphe( type ), TXT_REMPLACE);
  infos.type  = type;
  infos.serie = serie;
  infos.strict = strict;
  met_chaine(CHX_GRAPHE, CHXG_SERIE, LGMAX_TXT_NOM_SERIE, serie->nom, TXT_COPIER);

  ARB_MettreComplement( CHX_GRAPHE, &infos );
  reponse = ARB_Bloquant( CHX_GRAPHE, titre_choix_graphe, NULL, CHX_GRAPHE );
  ARB_MettreComplement( CHX_GRAPHE, NULL );
  if ( CHXG_ANNULE == reponse ) {
    free(infos.liste->graphes);
    free(infos.liste); /* On libre */
    return NULL;
  }

  retour = (infos.liste->graphes)[infos.liste->num_select];
  free(infos.liste->graphes);
  free(infos.liste);
  return retour;
}

/* ~-~-~-~-~-~-~- Slection multiple ~-~-~-~-~-~-~- */

/* -------- Slection de plusieurs spectres ---------------------------\
 | renvoie une table de pointeurs sur les graphes choisis,             |
 |   NULL en cas d'erreur, ou si on annule                             |
 |                                                                     |
 | type      : le type de graphes  afficher dans la liste             |
 | min_choix : nombre minimal de graphes  choisir pour pouvoir sortir |
 |             0 = pas de minimum                                      |
 | max_choix : nombre maximal de graphes  choisir pour pouvoir sortir |
 |             0 = pas de minimum                                      |
 | nbr_choisis = nombre de graphes finalement choisis                  |
 \-------------------------------------------------------------------- */

Graphe **GRF_mchoisir(int type, unsigned int min_choix, unsigned int max_choix, unsigned long *nbr_choisis)
{
  Infos_ChoixGraphe_XX infos;
  Graphe **retour = NULL;
  int reponse;
  unsigned long i;

  /* Vrifications de cohrence */
  if ( NULL == nbr_choisis ) {
    (void) fprintf( stderr, "%s,%s l.%u : nbr_choisis = NULL, ABSURDE\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }
  if ( ( max_choix > 0 ) && ( max_choix < min_choix ) ) {
    (void) fprintf( stderr, "%s, %s l.%u : max_choix (%u) < min_choix (%u)\n",
		    __FILE__, __FUNCTION__, __LINE__, max_choix, min_choix );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return NULL;
  }

  infos.liste = GRF_par_type( type, GRF_LISTE_TYPES_MULTISELECTION | GRF_LISTE_TYPES_MESSAGES, serie_courante );
  if ( NULL == infos.liste ) return NULL; /* Problme de slection */

  infos.liste->min = min_choix;
  infos.liste->max = max_choix;

  /* Prparation de l'arbre */
  ARB_PrepareSelecteurObjets( VRAI, GER_choix, CLV_choix );
  OBJ_active( CHX_GRAPHE, CHXG_OK, min_choix == 0 );
  OBJ_active( CHX_GRAPHE, CHXG_SERIE, FORCE_NON ); /* On reste dans la mme srie */
  met_chaine( CHX_GRAPHE, CHXG_SERIE, LGMAX_TXT_NOM_SERIE, serie_courante->nom, TXT_COPIER );

  GRF_affiche( infos.liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
	       CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR );
  OBJ_utilisateur( CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) (infos.liste->graphes[ infos.liste->num_select ]), METTRE );
  ARB_ascenseur( CHX_GRAPHE, CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL,
		 infos.liste->lgmax_txt, LGMAX_TXT_NOM_OBJET_SELECTEUR, infos.liste->delta_x, ROOT, -1, -1, METTRE );

  /* On doit utiliser TXT_REMPLACE, car si v. traduite, texte constant => copie donne une seg. fault ! */
  if ( type == GR_EXAFS )
    met_chaine( CHX_GRAPHE, CHXG_TITRE, LGMAX_TXT_TYPE_GRAPHE, txt_chx_grf_ttexafs, TXT_REMPLACE );
  else if ( type == GR_TOUS )
    met_chaine( CHX_GRAPHE, CHXG_TITRE, LGMAX_TXT_TYPE_GRAPHE, txt_chx_grf_tous, TXT_REMPLACE );
  else
    met_chaine( CHX_GRAPHE, CHXG_TITRE, LGMAX_TXT_TYPE_GRAPHE, GRF_TexteTypeGraphe( type ), TXT_REMPLACE );
  infos.type = type;
  infos.serie = serie_courante;
  infos.strict = FAUX;

  ARB_MettreComplement( CHX_GRAPHE, &infos );
  reponse = ARB_Bloquant( CHX_GRAPHE, titre_choix_graphe, NULL, CHX_GRAPHE );
  ARB_MettreComplement( CHX_GRAPHE, NULL );
  if ( CHXG_ANNULE == reponse ) {
    (void) GRF_LibererListe( infos.liste );
    return NULL;
  }

  for ( *nbr_choisis = 0, i = 0 ; i < infos.liste->nbr_graphe ; i++ )
    if ( VRAI == infos.liste->choisi[ i ] )
      (*nbr_choisis)++;
  if ( 0 == *nbr_choisis ) {  /* Aucun graphe choisi */
    (void) GRF_LibererListe( infos.liste );
    return NULL;
  }

  retour = (Graphe **) calloc( *nbr_choisis, sizeof(Graphe *) );
  if ( NULL == retour ) {
    ERR_Memoire( "Table des graphes slectionns", *nbr_choisis * sizeof(Graphe *) );
    (void) GRF_LibererListe( infos.liste );
    return NULL;
  }
  for ( *nbr_choisis = 0, i = 0 ; i < infos.liste->nbr_graphe ; i++ )
    if ( VRAI == infos.liste->choisi[ i ] ) {
      retour[ *nbr_choisis ] = infos.liste->graphes[ i ];
      (*nbr_choisis)++;
    }

  (void) GRF_LibererListe( infos.liste );

  /* On a fini, la liste est prte */
#ifdef _DEVELOPPE_
  printf( "%s : %lu graphes slectionns\n", __FUNCTION__, *nbr_choisis );
  for ( i = 0 ; i < *nbr_choisis ; i++ ) {
    printf( " -> graphe %lu : %p [%s]\n", i, (void *) retour[ i ], retour[ i ]->nom );
  }
#endif
  return retour;
}

/* -------- Slection dans une liste indique par l'utilisateur -------- */

Graphe **GRF_ChoisirDansTable(Graphe **table, unsigned long nbr_ds_table, unsigned int options, unsigned long *nbr_choisis)
{
  GRF_liste liste_graphes;
  Graphe **graphes_choisis = NULL;
  Infos_ChoixGraphe_XX infos;
  int reponse;
  unsigned long i;

  /* Contrles prliminaires */
  if ( NULL == table ) {
    if ( nbr_ds_table > 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : Pas de table, mais %lu devraient tre dedans !\n",
		     __FILE__, __FUNCTION__, __LINE__, nbr_ds_table);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    }
    return NULL;
  }
  if ( 0 == nbr_ds_table ) {
    (void) fprintf(stderr, "%s, %s l.%u : Table indique, mais 0 dedans !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }
  if ( NULL == nbr_choisis ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas de variable pour mmoriser le nombre choisi\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  /* Le changement de srie n'a pas de sens, puisque la liste est dj prte */
  options &= ~OPTIONS_CHOIX_MULTISERIE;

  /* On prpare la liste des graphes */
  liste_graphes.nbr_graphe = nbr_ds_table;
  liste_graphes.prem_graphe = 0;
  liste_graphes.num_select = 0;
  liste_graphes.delta_x = 0;
  liste_graphes.min = 0;   liste_graphes.max = 0;
  liste_graphes.graphes = table;
  if ( options & OPTIONS_CHOIX_MULTIPLE ) {
    liste_graphes.choisi = (boolean *) calloc( nbr_ds_table, sizeof( boolean ) );
    if ( NULL == liste_graphes.choisi ) {
      (void) fprintf(stderr, "%s, %s l.%u : Mmoire insuffisante pour la slection multiple. Slection simple\n",
		     __FILE__, __FUNCTION__, __LINE__);
      options &= ~OPTIONS_CHOIX_MULTIPLE;
    }
  } else {
    liste_graphes.choisi = NULL;
  }
  Liste_InitialiserLongueur( &liste_graphes );

  /* Liste mmorise dans les infos */
  infos.liste = &liste_graphes;

  /* Prparation de l'arbre */
  ARB_PrepareSelecteurObjets( VRAI, GER_choix, CLV_choix );
  OBJ_active( CHX_GRAPHE, CHXG_OK, VRAI );
  OBJ_active( CHX_GRAPHE, CHXG_SERIE, FORCE_NON ); /* On reste dans la mme srie */
  met_chaine( CHX_GRAPHE, CHXG_SERIE, LGMAX_TXT_NOM_SERIE, serie_courante->nom, TXT_COPIER );

  GRF_affiche(infos.liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
	      CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR);
  OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) (infos.liste->graphes[infos.liste->num_select]), METTRE);
  ARB_ascenseur(CHX_GRAPHE, CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL,
		infos.liste->lgmax_txt, LGMAX_TXT_NOM_OBJET_SELECTEUR, infos.liste->delta_x, ROOT, -1, -1, METTRE);

  met_chaine(CHX_GRAPHE, CHXG_TITRE, LGMAX_TXT_TYPE_GRAPHE, "Choix dans la liste propose", TXT_REMPLACE);

  infos.type = GR_SSTYPE;
  infos.serie = NULL;
  infos.strict = FAUX;

  /* On appelle l'arbre */
  ARB_MettreComplement( CHX_GRAPHE, &infos );
  reponse = ARB_Bloquant( CHX_GRAPHE, titre_choix_graphe, NULL, CHX_GRAPHE );
  /* Plus besoin des infos associes  l'arbre */
  ARB_MettreComplement( CHX_GRAPHE, NULL );

  /* Si on a annul, on quitte directement */
  if ( CHXG_ANNULE == reponse ) {
    return NULL;
  }

  /* Combien de graphes ont t choisis ? */
  if ( options & OPTIONS_CHOIX_MULTIPLE ) {
    for (*nbr_choisis = 0, i = 0 ; i < infos.liste->nbr_graphe ; i++)
      if ( infos.liste->choisi[i] == VRAI )
	(*nbr_choisis)++;
  } else {
    *nbr_choisis = 1;
  }
  if (*nbr_choisis == 0) {  /* Aucun graphe choisi */
    return NULL;
  }

  /* On rserve la table des graphes choisis */
  graphes_choisis = (Graphe **) calloc( *nbr_choisis, sizeof(Graphe *) );
  if ( NULL == graphes_choisis ) {
    SYS_ERREUR(MANQUE_MEMOIRE, "Graphes/choisir.c", "GRF_mchoisir", "retour");
    return NULL;
  }
  if ( options & OPTIONS_CHOIX_MULTIPLE ) {
    for (*nbr_choisis = 0, i = 0 ; i < infos.liste->nbr_graphe ; i++)
      if ( infos.liste->choisi[ i ] == VRAI ) {
	graphes_choisis[ *nbr_choisis ] = infos.liste->graphes[ i ];
	(*nbr_choisis)++;
      }
  } else {
    graphes_choisis[ 0 ] = table[ infos.liste->num_select ];
  }

  /* On a fini, la liste est prte */
  return graphes_choisis;
}

/* -------- Gestion de l'interface associe -------- */

static int CLV_choix(unsigned int touche, int etat, int *renvoi)
{
  unsigned int nbr_noms, obj_choisi;
  Infos_ChoixGraphe_XX *infos;
  GRF_liste *liste;

  if ( renvoi != NULL ) *renvoi = 0; /* Aucun bouton en particulier... */

  infos = (Infos_ChoixGraphe_XX *) ARB_LireComplement( CHX_GRAPHE );
  liste = infos->liste;
  if ( liste == NULL ) return BI_EVENEMENT_NON_TRAITE;

  nbr_noms = (CHXG_DERNIER_NOM - CHXG_PREMIER_NOM) + 1;
  obj_choisi = liste->num_select - liste->prem_graphe;

  switch (touche)
  {
  case Fleche_Bas: /* On choisit le spectre suivant */
    if ( obj_choisi < nbr_noms ) /* Il est donc visible avant */
      OBJ_choisit(CHX_GRAPHE, CHXG_PREMIER_NOM + obj_choisi, FORCE_NON);
    if ( liste->choisi != NULL ) {
      return ARB_SimulerClic( CHX_GRAPHE, CHXG_APRES, 1 );
    }
    if ( etat & ( K_RSHIFT | K_LSHIFT ) ) { /* On va au dernier affich */
      obj_choisi = nbr_noms - 1;
      if ( liste->prem_graphe + obj_choisi >= liste->nbr_graphe )
	obj_choisi = liste->nbr_graphe - 1;
    } else {
      obj_choisi++;
    }
    if (obj_choisi + liste->prem_graphe >= liste->nbr_graphe) {
      OBJ_choisit(CHX_GRAPHE, CHXG_PREMIER_NOM + obj_choisi - 1, FORCE_OUI);
      /* Faire un bip */
      return BI_EVENEMENT_TRAITE;
    }
    if (obj_choisi >= nbr_noms) { /* On fait dfiler le tout d'un cran... */
      liste->num_select = -1; /* Aucun n'est slectionn */
      obj_choisi--; /* Puisque l'on a dfil d'un cran */
      if (GRF_selecteur(liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, 65,
			CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR, CHXG_APRES, 0, 0) == FAUX) {
	liste->num_select = (obj_choisi - CHXG_PREMIER_NOM) + liste->prem_graphe;
	OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) (liste->graphes[liste->num_select]), METTRE);
	/* Il faudrait un bip */
	return BI_EVENEMENT_TRAITE; /* En fait, il n'y a plus de graphes aprs */
      }
    }
    liste->num_select = obj_choisi + liste->prem_graphe;
    OBJ_choisit(CHX_GRAPHE, CHXG_PREMIER_NOM + obj_choisi, FORCE_OUI);
    OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) (liste->graphes[liste->num_select]), METTRE);
    return BI_EVENEMENT_TRAITE;

  case Fleche_Haut: /* On choisit le spectre prcdent */
    if ( obj_choisi < nbr_noms ) /* Il est donc visible avant */
      OBJ_choisit(CHX_GRAPHE, CHXG_PREMIER_NOM + obj_choisi, FORCE_NON);
    if ( liste->choisi != NULL) {
      return ARB_SimulerClic( CHX_GRAPHE, CHXG_AVANT, 1 );
    }
    if (obj_choisi == 0) { /* On est dj en haut */
      liste->num_select = -1; /* Aucun n'est slectionn */
      if (GRF_selecteur(liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, 65,
			CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR, CHXG_AVANT, 0, 0) == FAUX) {
	liste->num_select = liste->prem_graphe; /* Il faudrait un bip */
	OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) (liste->graphes[liste->num_select]), METTRE);
	OBJ_choisit(CHX_GRAPHE, CHXG_PREMIER_NOM, FORCE_OUI);
	return BI_EVENEMENT_TRAITE; /* Il n'y en a plus avant */
      }
      obj_choisi = 1;
    }
    obj_choisi--;
    liste->num_select = obj_choisi + liste->prem_graphe;
    OBJ_choisit(CHX_GRAPHE, CHXG_PREMIER_NOM + obj_choisi, FORCE_OUI);
    OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) (liste->graphes[liste->num_select]), METTRE);
    return BI_EVENEMENT_TRAITE;

  case Fleche_Gauche: /* Dfiler vers la droite */
    return ARB_SimulerClic( CHX_GRAPHE, CG_TXT_AVANT, 1 );
  case Fleche_Droite:
    return ARB_SimulerClic( CHX_GRAPHE, CG_TXT_APRES, 1 );

  case 'a': /* Aucun */
    return ARB_SimulerClic( CHX_GRAPHE, CHXG_AUCUN, 1 );

  case 's': /* Choix de la srie */
    return ARB_SimulerClic( CHX_GRAPHE, CHXG_SERIE, 1 );

  case 't': /* Tous */
    return ARB_SimulerClic( CHX_GRAPHE, CHXG_TOUS, 1 );
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Gestion de la souris -------- */

static int GER_choix(unsigned int num_objet, int x, int y)
{
  int retour = BI_EVENEMENT_NON_TRAITE;
  unsigned long nbr_choisis = 0;
  Infos_ChoixGraphe_XX *infos;
  GRF_liste *liste;

  infos = (Infos_ChoixGraphe_XX *) ARB_LireComplement( CHX_GRAPHE );
  liste = infos->liste;

  if (num_objet == CHXG_SERIE) { /* Choix de la srie de graphes. On peut avoir liste = NULL */
    Serie_Graphes *nv_serie;

    nv_serie = I_SERIE_GRF_Choisir( CHX_GRAPHE, CHXG_SERIE, LGMAX_TXT_NOM_SERIE );
    if ( (nv_serie != NULL) && (nv_serie != infos->serie) ) { /* Il faut refaire la liste */
      unsigned int options = 0U;

      if ( VRAI == infos->strict ) options |= GRF_LISTE_TYPES_STRICT;
      if ( liste->choisi != NULL ) options |= GRF_LISTE_TYPES_MULTISELECTION;

      GRF_LibererListe( liste ); /* On libre donc l'ancienne */

      infos->serie = nv_serie;
      infos->liste = GRF_par_type(infos->type, options, nv_serie);
#ifdef _DEVELOPPE_
      printf("Liste renvoye : %p\n", (void *) infos->liste);
#endif

      /* On affiche cette nouvelle srie */
      GRF_affiche(infos->liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
		  CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR);
      if (infos->liste != NULL) {
	OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, USR_graphe, (long) (infos->liste->graphes[infos->liste->num_select]), METTRE);
	ARB_ascenseur(CHX_GRAPHE, CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL,
		      infos->liste->lgmax_txt, LGMAX_TXT_NOM_OBJET_SELECTEUR, infos->liste->delta_x, ROOT, -1, -1, METTRE);
      } else {
#ifdef _DEVELOPPE_
	printf("Attention, srie nulle : aucun graphe dedans, forcment...\n");
#endif
	OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, USR_graphe, (long) (NULL), METTRE);
	ARB_ascenseur(CHX_GRAPHE, CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL,
		      1, LGMAX_TXT_NOM_OBJET_SELECTEUR, 0, ROOT, -1, -1, METTRE);
      }
    }
    return BI_EVENEMENT_TRAITE;
  }

  if ( liste == NULL ) return BI_EVENEMENT_TRAITE; /* Rien  faire quand la liste est vide ! */
  if ( liste->choisi != NULL ) {
    unsigned long i;

    /* Attention, on ne retourne pas maintenant car il faudra activer/dsactiver OK */
    if ( num_objet == CHXG_TOUS) {
      for (i = 0 ; i < liste->nbr_graphe ; i++)
	liste->choisi[i] = VRAI;
      GRF_affiche(liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
		  CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR);
      nbr_choisis = liste->nbr_graphe;
      /* Mais on indique que l'on a bien trait l'vnement
	 Reste dans le if() car sinon, fait  chaque appel -> rend le slecteur non-quittable !*/
      retour = BI_EVENEMENT_TRAITE;
    }
    if ( num_objet == CHXG_AUCUN) {
      for (i = 0 ; i < liste->nbr_graphe ; i++)
	liste->choisi[i] = VRAI;
      GRF_affiche(liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
		  CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR);
      nbr_choisis = 0;
      retour = BI_EVENEMENT_TRAITE;
    }
  }

  /* Pour le dcalage des noms : ascenseur horizontal */
  if ( (num_objet == CG_TXT_AVANT) || (num_objet == CG_FND_ASC_TXT) ||
       (num_objet == CG_ASC_TXT  ) || (num_objet == CG_TXT_APRES) ) {
    liste->delta_x = ARB_ascenseur(CHX_GRAPHE, CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL,
				   liste->lgmax_txt, LGMAX_TXT_NOM_OBJET_SELECTEUR, liste->delta_x, num_objet, x, y, ECHANGE);
    GRF_affiche(liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
		CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR);
    return BI_EVENEMENT_TRAITE;
  }

  /* Pour le dcalage de la liste : ascenseur vertical */
  if ( (num_objet == CHXG_AVANT) || (num_objet == CHXG_FOND_ASC) ||
       (num_objet == CHXG_APRES) || (num_objet == CHXG_POUSSOIR) ) {
    (void) GRF_selecteur(liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
			 CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR,
			 num_objet, x, y);
    OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) (liste->graphes[liste->num_select]), METTRE);
    return BI_EVENEMENT_TRAITE;
  }

  /* Clic dans la liste : slection */
  if ( ( num_objet >= CHXG_PREMIER_NOM ) && ( num_objet <= CHXG_DERNIER_NOM ) ) {
    (void) GRF_selecteur(liste, CHX_GRAPHE, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
			 CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR,
			 num_objet, x, y);
    OBJ_utilisateur(CHX_GRAPHE, CG_AFFICHAGE, NULL, (long) (liste->graphes[liste->num_select]), METTRE);
    if ( (liste->choisi) && ((liste->min > 0) || (liste->max > 0)) ) {
      unsigned long i;

      for (i = 0 ; i < liste->nbr_graphe ; i++)
	if (liste->choisi[i] == VRAI) nbr_choisis++;
    }

    /* On indique que l'on a bien trait l'vnement */
    retour = BI_EVENEMENT_TRAITE;
  }

  /* On contrle si l'on peut, ou non, activer le bouton OK */
  if ( infos->liste->choisi != NULL ) {
    if ( nbr_choisis < infos->liste->min ) /* Pas assez de graphes choisis */
      OBJ_active(CHX_GRAPHE, CHXG_OK, FORCE_NON);
    else if ( (infos->liste->max > 0) && (nbr_choisis > infos->liste->max) ) /* Trop de graphes choisis */
      OBJ_active(CHX_GRAPHE, CHXG_OK, FORCE_NON);
    else /* Nombre de graphes choisis acceptable */
      OBJ_active(CHX_GRAPHE, CHXG_OK, FORCE_OUI);
  }

  return retour; /* Autrement, traitement par defaut */
}

/* -------- Prparation de l'interface [Non-statique car utilise aussi pour les images] -------- */

void ARB_PrepareSelecteurObjets(boolean multiple, int (*gestion)(unsigned int, int, int), int (*clavier)(unsigned int, int, int *) )
{
  /* Positionnement correct des objets */
  if ( FAUX == arbres[ CHX_GRAPHE ].drapeaux.prepare ) {
    int retour;

    /* ATTENTION, l'objet utilisateur doit dj avoir t initialis pour ne pas le perdre ! */
    /* Initialisation de l'arbre */
    retour = ARB_Initialiser( CHX_GRAPHE, "communs.html", gestion, clavier );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible d'initialiser l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }

    OBJ_deplace (CHX_GRAPHE, CHXG_TXT_SERIE, 0, +500, DEPLACE_RELATIF);
    OBJ_deplace (CHX_GRAPHE, CHXG_SERIE    , 0, +500, DEPLACE_RELATIF);
    OBJ_deplace (CHX_GRAPHE, CHXG_TITRE   , 0, -500, DEPLACE_RELATIF);
    OBJ_deplace (CHX_GRAPHE, CHXG_TXT_TYPE, 0, -500, DEPLACE_RELATIF);

    ARB_EnjoliverDefileur( CHX_GRAPHE, CHXG_FOND, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM,
			   CHXG_AVANT, CHXG_FOND_ASC, CHXG_POUSSOIR, CHXG_APRES, ARB_ASCENSEUR_DROITE );
    ARB_EnjoliverAscenseur( CHX_GRAPHE, CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL, 0 );

#ifdef TRADUCTION_COMPLETE_
    OBJ_TraduireTexte( CHX_GRAPHE, CHXG_OK   , txt_chx_grf_ok    );
    OBJ_TraduireTexte( CHX_GRAPHE, CHXG_TITRE, titre_choix_graphe);
    OBJ_TraduireTexte( CHX_GRAPHE, CHXG_TOUS , txt_chx_grf_tous2 );
    OBJ_TraduireTexte( CHX_GRAPHE, CHXG_AUCUN, txt_chx_grf_aucun );
#endif

    arbres[CHX_GRAPHE].drapeaux.prepare = VRAI;
  }

  if ( FAUX == multiple ) {  /* Pas de slection multiple ici ! */
    OBJ_active(CHX_GRAPHE, CHXG_TOUS, FORCE_NON);
    OBJ_active(CHX_GRAPHE, CHXG_AUCUN, FORCE_NON);
  } else {
    OBJ_active(CHX_GRAPHE, CHXG_TOUS, FORCE_OUI);
    OBJ_active(CHX_GRAPHE, CHXG_AUCUN, FORCE_OUI);
  }

  /* On installe les fonctions de gestion, car changent suivant que l'on utilise des graphes ou des images ! */
  arbres[ CHX_GRAPHE ].clavier = clavier;
  arbres[ CHX_GRAPHE ].fonction.arbre = gestion;
}

/* -------- Redessin d'un objet utilisateur contenant un graphe -------- */

int cdecl USR_graphe(USR_FONCTION *parametres)
{
  Graphe *graphe;

  /* Prparation du dessin */
  TRC_InitDessins( NULL, parametres->pb_x, parametres->pb_y, parametres->pb_w, parametres->pb_h );

  /* Trac du graphe : optimal dans ce cadre */
  graphe = (Graphe *) (parametres->pb_parm);
  TRC_GrapheDansRectangle( NULL, graphe, NOIR, 1, parametres->pb_x, parametres->pb_y, parametres->pb_w, parametres->pb_h );

  /* On a fini */
  return parametres->pb_currstate;
}
