/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2003, Emmanuel CURIS

   Informations sur un graphe
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "interface.h"
#include "serie.h"

#define NOMBRE_GENRE_GRAPHES (GRF_ERREUR_C + 1)

#include "grf_infos.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

struct infos_graphe {
  int nouveau_type;
  int nouveau_genre;
  int nouvel_affichage;
  Graphe *graphe;
};

#define LGMAX_NOM_TYPE_GRAPHE 30
#define LGMAX_VALEUR_BORNE    20
#define LGMAX_GENRE_GRAPHE    19
#define LGMAX_AFFICHAGE       24

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int  GER_infos_graphe(unsigned int objet, int x, int y);
static void prepare_infos_graphe(void);

inline static unsigned long GRF_NombreAffichages(int genre);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Affichage des infos sur un graphe, modifications ventuelles
   Renvoie VRAI s'il y a eu des modifications du graphe, FAUX sinon
 */

boolean GRF_Informations(Graphe *graphe)
{
  int retour = FAUX;
  unsigned int lg = 50;
  struct infos_graphe infos;
  char val_borne[ 50 ];

  if (graphe == NULL) return -1;

  if ( arbres[INFOS_GRAPHE].drapeaux.prepare == FAUX )
    prepare_infos_graphe();

  /* On initialise l'arbre */
  /* 0. Les bornes en X & Y */
  (void) TXT_CNV_D_vTexte( graphe->x_min, 0.0, 4, 3, 2, val_borne, NULL );
  met_chaine( INFOS_GRAPHE, IG_XMIN, LGMAX_VALEUR_BORNE, val_borne, TXT_COPIER );
  (void) TXT_CNV_D_vTexte( graphe->x_max, 0.0, 4, 3, 2, val_borne, NULL );
  met_chaine( INFOS_GRAPHE, IG_XMAX, LGMAX_VALEUR_BORNE, val_borne, TXT_COPIER );

  (void) TXT_CNV_D_vTexte( graphe->y_min, 0.0, 4, 3, 2, val_borne, NULL );
  met_chaine( INFOS_GRAPHE, IG_YMIN, LGMAX_VALEUR_BORNE, val_borne, TXT_COPIER );
  (void) TXT_CNV_D_vTexte( graphe->y_max, 0.0, 4, 3, 2, val_borne, NULL );
  met_chaine( INFOS_GRAPHE, IG_YMAX, LGMAX_VALEUR_BORNE, val_borne, TXT_COPIER );

  /* 1. Le nom du graphe courant */
  met_chaine( INFOS_GRAPHE, GRF_INFOS_NOM, 0, graphe->nom, TXT_REMPLACE );

  /* 2. Le nombre de points */
  (void) TXT_CNV_UL_vTexte( graphe->nombre_point, val_borne, &lg );
  met_chaine( INFOS_GRAPHE, GRF_INFOS_NBRTPS, 10, val_borne, TXT_COPIER );

  OBJ_choisit(INFOS_GRAPHE, IG_SAUVE, (graphe->drapeaux.sauve == VRAI));
  OBJ_choisit(INFOS_GRAPHE, IG_PROTEGE, (graphe->drapeaux.protege == VRAI));

  /* 4. Le type, genre & mode d'affichage */
  OBJ_ChoisirTypeGraphe( INFOS_GRAPHE, IG_TYPE, LGMAX_NOM_TYPE_GRAPHE, graphe->type, METTRE, VRAI );
  OBJ_ChoisirGenreGraphe( INFOS_GRAPHE, IG_GNR_GRAPHE, LGMAX_GENRE_GRAPHE, graphe->genre_donnees, METTRE );

  if ( NULL == graphe->affichage ) {
    OBJ_active( INFOS_GRAPHE, IG_AFFICHAGE, FORCE_NON );
    met_chaine( INFOS_GRAPHE, IG_AFFICHAGE, 0, err_sans_affichage, TXT_REMPLACE );
    infos.nouvel_affichage = -21975;
  } else {
    OBJ_active( INFOS_GRAPHE, IG_AFFICHAGE, FORCE_OUI );
    infos.nouvel_affichage = OBJ_ChoisirAffichage( INFOS_GRAPHE, IG_AFFICHAGE, graphe->genre_donnees, graphe->affichage->methode, METTRE );
  }

  /* 5. Peut-on avoir des incertitudes ? */
  if ( ( NULL == graphe->x ) || ( GRF_SERIE == graphe->genre_donnees ) ) {
    OBJ_active( INFOS_GRAPHE, IG_BERR_X, FORCE_NON );
  } else {
    OBJ_active( INFOS_GRAPHE, IG_BERR_X, FORCE_OUI );
  }

  if ( ( VRAI == GRF_EstComplexe( graphe ) ) || ( VRAI == GRF_EstReel( graphe ) ) ) {
    OBJ_active( INFOS_GRAPHE, IG_BERR_Y, FORCE_OUI );
  } else {
    OBJ_active( INFOS_GRAPHE, IG_BERR_Y, FORCE_NON );
  }

  OBJ_choisit(INFOS_GRAPHE, IG_BERR_X, graphe->erreur_x != NULL);
  OBJ_choisit(INFOS_GRAPHE, IG_BERR_Y, graphe->erreur_y != NULL);

  /* On prpare les informations pour la bote de dialogue */
  infos.graphe = graphe;
  infos.nouveau_type = graphe->type;
  infos.nouveau_genre = graphe->genre_donnees;

  /* On appelle l'arbre */
  ARB_MettreComplement( INFOS_GRAPHE, &infos );
  retour = ARB_Bloquant( INFOS_GRAPHE, titre_boite_infos, principale, 0 );
  ARB_MettreComplement( INFOS_GRAPHE, NULL );
  /* On remet une chane vide pour viter des ennuis en fin de programme */
  met_chaine( INFOS_GRAPHE, GRF_INFOS_NOM, 0, "", TXT_REMPLACE );

  if (retour == GRF_INFOS_ANNULE) return FAUX; /* Rien  faire */
  retour = FAUX;

  /* A-t-on modifi le mode d'affichage du graphe ? */
  if ( ( graphe->affichage != NULL ) &&( graphe->affichage->methode != infos.nouvel_affichage ) ) {
    retour = VRAI;
    graphe->affichage->methode = infos.nouvel_affichage;
  }

  /* A-t-on modifi le genre du graphe ? */
  if ( graphe->genre_donnees != infos.nouveau_genre ) {
    int reponse;

    reponse = BI_ERR_alerte( principale, &MSG_ChangerGenre, VRAI );
    if ( 1 == reponse ) {
      retour = VRAI;
      reponse = GRF_ChangerGenre( graphe, infos.nouveau_genre );
      if ( reponse != 0 ) {
	(void) BI_ERR_alerte( principale, &ERR_ConversionGenreImpossible, VRAI );
      }
    }
  }

  /* A-t-on modifi le type du graphe ? */
  if ( graphe->type != infos.nouveau_type ) {
    int reponse;

    reponse = BI_ERR_alerte( principale, &MSG_ChangerType, VRAI );
    if ( 1 == reponse ) {
      retour = VRAI;
      GRF_ChangerType( graphe, infos.nouveau_type );
    }
  }

  return retour;
}

/* -------- Quel est le genre du graphe ? -------- */

const char *GRF_TXT_Genre(int genre)
{
  if ( ( genre < 0 ) || ( genre > GRF_ERREUR_C ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Genre de graphe inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, genre);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return "Genre inconnu";
  }

  return textes_genres_graphe[ genre ];
}

/* -------- Gestion de l'arbre donnant les infos sur le graphe ------------
   Le graphe concern est forcment le graphe courant */

static int GER_infos_graphe(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  Graphe *graphe;
  struct infos_graphe *infos;
  int reponse;

  infos = (struct infos_graphe *) ARB_LireComplement( INFOS_GRAPHE );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'informations associes  l'arbre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }
  graphe = infos->graphe;
  if ( NULL == graphe ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas de graphe dans les informations associes  l'arbre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }

  switch (objet)
  {
  case IG_TYPE: /* Changer le type du graphe [dangereux !] */
    infos->nouveau_type = OBJ_ChoisirTypeGraphe( INFOS_GRAPHE, IG_TYPE, LGMAX_NOM_TYPE_GRAPHE, infos->nouveau_type, QUESTION, VRAI );
    return BI_EVENEMENT_TRAITE;

  case IG_GNR_GRAPHE: /* Changer le genre du graphe [trs dangereux] */
    infos->nouveau_genre = OBJ_ChoisirGenreGraphe( INFOS_GRAPHE, IG_GNR_GRAPHE, LGMAX_GENRE_GRAPHE, infos->nouveau_genre, QUESTION );
    return BI_EVENEMENT_TRAITE;

  case IG_BERR_X:
  case IG_BERR_Y:
    {
      Graphe *nv_erreurs;
      Serie_Graphes *serie;
      boolean meme_X = FAUX;
      int axe;

      if ( IG_BERR_X == objet ) {
	axe = AXE_X;
      } else {
	axe = AXE_Y;
      }

      serie = GRF_QuelleSerie( graphe );
      nv_erreurs = GRF_choisir( GR_DEVIATION, VRAI, FAUX, serie ); /* Les barres d'erreur DOIVENT tre dans la mme srie */
      if ( nv_erreurs == NULL ) return VRAI; /* Rien  faire */
      
      if ( ( graphe->x_min >= nv_erreurs->x_max ) ||
	   ( graphe->x_max <= nv_erreurs->x_min ) ) {
	EXF_ERREUR(ERR_DISJOINTS);
	return BI_EVENEMENT_TRAITE;
      }

      /* Nb : mme si mme nombre de points, il vaut mieux interpoler parce que les X
	 ne sont pas forcment corrects... Mais on demande quand mme */      
      if ( graphe->nombre_point == nv_erreurs->nombre_point ) {
	int retour;

	retour = BI_ERR_alerte(principale, &Q_AssignerErreurs, VRAI);
	if (retour == 3) return VRAI;	
	if (retour == 2) {
	  meme_X = VRAI;
	}
      }

      /* Assignation --- s'occupe de faire l'interpolation et tout et tout */
      GRF_AssignerErreurs( graphe, nv_erreurs, meme_X, axe );
    }
    /* On adapte ! */
    OBJ_choisit( INFOS_GRAPHE, IG_BERR_X, graphe->erreur_x != NULL );
    OBJ_choisit( INFOS_GRAPHE, IG_BERR_Y, graphe->erreur_y != NULL );
    return BI_EVENEMENT_TRAITE;

  case IG_PROTEGE:
  case IG_SAUVE:
    /* Ces boutons sont des indicateurs non-modifiables */
    return BI_EVENEMENT_TRAITE;

    /* Les boutons qui suivent sont des indicateurs modifiables */
  case IG_EST_THEORIQUE:
    reponse = BI_ERR_alerte( principale, &MSG_ChangerIndicateur, VRAI );
    if ( 1 == reponse ) {
      graphe->drapeaux.theorique = !graphe->drapeaux.theorique;
      OBJ_choisit( INFOS_GRAPHE, IG_EST_THEORIQUE, graphe->drapeaux.theorique != 0 );
    }
    return BI_EVENEMENT_TRAITE;

  case IG_EST_NORME:
    reponse = BI_ERR_alerte( principale, &MSG_ChangerIndicateur, VRAI );
    if ( 1 == reponse ) {
      graphe->drapeaux.normalise = !graphe->drapeaux.normalise;
      OBJ_choisit( INFOS_GRAPHE, IG_EST_NORME, graphe->drapeaux.normalise != 0 );
    }
    return BI_EVENEMENT_TRAITE;

  case IG_EST_XTRIE:
    reponse = BI_ERR_alerte( principale, &MSG_ChangerIndicateur, VRAI );
    if ( 1 == reponse ) {
      graphe->drapeaux.X_tries = !graphe->drapeaux.X_tries;
      OBJ_choisit( INFOS_GRAPHE, IG_EST_XTRIE, graphe->drapeaux.X_tries != 0 );
    }
    return BI_EVENEMENT_TRAITE;

  case IG_EST_X_UNIQUES:
    reponse = BI_ERR_alerte( principale, &MSG_ChangerIndicateur, VRAI );
    if ( 1 == reponse ) {
      graphe->drapeaux.X_uniques = !graphe->drapeaux.X_uniques;
      OBJ_choisit( INFOS_GRAPHE, IG_EST_X_UNIQUES, graphe->drapeaux.X_uniques != 0 );
    }
    return BI_EVENEMENT_TRAITE;

  case IG_AFFICHAGE: /* Modifier le mode d'affichage (si c'est faisable...) */
    infos->nouvel_affichage = OBJ_ChoisirAffichage( INFOS_GRAPHE, IG_AFFICHAGE, infos->nouveau_genre, infos->nouvel_affichage, QUESTION );
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Choisir le mode d'affichage -------- */

int OBJ_ChoisirAffichage(unsigned int arbre, unsigned int objet, int genre, int affichage, int mode)
{
  unsigned long i, nbr_affichage;
  const char **noms_affichage;
  int nv_affichage;

  if ( METTRE == mode ) {
    met_chaine( arbre, objet, 0, GRF_TXT_TypeAffichage( genre, affichage ), TXT_REMPLACE );
    return affichage;
  }

  /* Nombre d'affichages */
  nbr_affichage = GRF_NombreAffichages( genre );
  if ( nbr_affichage < 2 ) { /* Il n'y a pas le choix de la mthode d'affichage */
    return affichage;
  }

  /* Prparation du tableau */
  noms_affichage = (const char **) malloc( sizeof( char *) * nbr_affichage );
  if ( NULL == noms_affichage ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la table des affichages / mmoire insuffisante \n",
		   __FILE__, __FUNCTION__, __LINE__);
    OBJ_choisit( arbre, objet, FORCE_NON );
    return genre;
  }

  /* On remplit : les types classiques en premier */
  for ( i = 0 ; i < nbr_affichage ; i++ ) {
    noms_affichage[ i ] = GRF_TXT_TypeAffichage( genre, i );
  }

  /* On appelle ce sous-menu */
  nv_affichage = SS_menu_texte( noms_affichage, nbr_affichage, titre_menu_affichage, arbre, objet );
  free( noms_affichage ); noms_affichage = NULL;
  if ( nv_affichage < 0 ) {
    OBJ_choisit( arbre, objet, FORCE_NON );
    return genre;
  }

  if ( nv_affichage == affichage ) {
    OBJ_choisit( arbre, objet, FORCE_NON );
    return nv_affichage;
  }

  met_chaine( arbre, objet, 0, GRF_TXT_TypeAffichage( genre, nv_affichage ), TXT_REMPLACE );
  OBJ_choisit( arbre, objet, FORCE_NON );

  return nv_affichage;
}

/* -------- Choisir le genre du graphe -------- */

int OBJ_ChoisirGenreGraphe(unsigned int arbre, unsigned int objet, unsigned long lgmax_texte, int genre, int mode)
{
  unsigned long i, nbr_genres;
  const char **noms_genres;
  int nv_genre;

  if ( METTRE == mode ) {
    met_chaine( arbre, objet, lgmax_texte, GRF_TXT_Genre( genre ), TXT_COPIER );
    return genre;
  }

  OBJ_choisit( arbre, objet, FORCE_OUI );

  nbr_genres = NOMBRE_GENRE_GRAPHES;

  noms_genres = (const char **) malloc( sizeof( char *) * nbr_genres );
  if ( NULL == noms_genres ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la table des genres / mmoire insuffisante \n",
		   __FILE__, __FUNCTION__, __LINE__);
    OBJ_choisit( arbre, objet, FORCE_NON );
    return genre;
  }

  /* On remplit : les types classiques en premier */
  for ( i = 0 ; i < nbr_genres ; i++ ) {
    noms_genres[ i ] = GRF_TXT_Genre( i );
  }

  /* On appelle ce sous-menu */
  nv_genre = SS_menu_texte( noms_genres, nbr_genres, titre_menu_genres, arbre, objet );
  free( noms_genres ); noms_genres = NULL;
  if ( nv_genre < 0 ) {
    OBJ_choisit( arbre, objet, FORCE_NON );
    return genre;
  }

  if ( nv_genre == genre ) {
    OBJ_choisit( arbre, objet, FORCE_NON );
    return nv_genre;
  }

  OBJ_RemplacerTexte( arbre, objet, GRF_TXT_Genre( nv_genre ), VRAI );
  OBJ_choisit( arbre, objet, FORCE_NON );
  return nv_genre;
}

/* -------- Prparation de l'arbre donnant les infos sur un graphe -------- */

static void prepare_infos_graphe(void)
{
  int retour;

  if ( FAUX == arbres[INFOS_GRAPHE].drapeaux.init ) {
    retour = ARB_Initialiser( INFOS_GRAPHE, "principale.html#informations", GER_infos_graphe, ( int (*)(unsigned int, int, int *) ) NULL );
    if ( retour != 0 ) return;
  }

  OBJ_deplace(INFOS_GRAPHE, IG_XMAX, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(INFOS_GRAPHE, IG_YMAX, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(INFOS_GRAPHE, IG_TXT_BERR, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(INFOS_GRAPHE, IG_BERR_X, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(INFOS_GRAPHE, IG_BERR_Y, 0, +3, DEPLACE_PIXELS);

  OBJ_deplace(INFOS_GRAPHE, IG_PROTEGE      , 0,  +2, DEPLACE_PIXELS);
  OBJ_deplace(INFOS_GRAPHE, IG_EST_THEORIQUE, 0,  +4, DEPLACE_PIXELS);
  OBJ_deplace(INFOS_GRAPHE, IG_EST_NORME    , 0,  +6, DEPLACE_PIXELS);
  OBJ_deplace(INFOS_GRAPHE, IG_EST_XTRIE    , 0,  +8, DEPLACE_PIXELS);
  OBJ_deplace(INFOS_GRAPHE, IG_EST_X_UNIQUES, 0, +10, DEPLACE_PIXELS);

#ifndef _DEVELOPPE_
  OBJ_active(INFOS_GRAPHE, IG_BERR_X, FORCE_NON);
#endif

#ifdef AVEC_TRADUCTION_
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_TXT_TYPE  , texte_type );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_TXT_NOMBRE, texte_nombre_points );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_TXT_BERR  , texte_barres_erreur );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_BERR_X    , texte_pour_les_X );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_BERR_Y    , texte_pour_les_Y );

  OBJ_TraduireTexte( INFOS_GRAPHE, IG_TXT_GRF_EST  , texte_le_graphe_est );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_SAUVE        , texte_sauve );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_PROTEGE      , texte_protege );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_EST_THEORIQUE, texte_theorique );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_EST_NORME    , texte_normalise );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_EST_XTRIE    , texte_trie_en_X );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_EST_X_UNIQUES, texte_a_X_uniques );

  OBJ_TraduireTexte( INFOS_GRAPHE, GRF_INFOS_OK , texte_ok );
  OBJ_TraduireTexte( INFOS_GRAPHE, GRF_INFOS_ANNULE, texte_annuler );
  OBJ_TraduireTexte( INFOS_GRAPHE, IG_TXT_AFFICHAGE, texte_affichage );
#endif

  arbres[INFOS_GRAPHE].drapeaux.prepare = VRAI;
}

inline static unsigned long GRF_NombreAffichages(int genre)
{
  switch ( genre )
  {
  case GRF_FCT_REELLE:   return 2;
  case GRF_FCT_COMPLEXE: return 4;
  case GRF_SERIE:        return 3;
  default:
    return 0;
  }
}

/* -------- Texte correspondant au type d'affichage choisi -------- */

const char *GRF_TXT_TypeAffichage(int genre, int affichage)
{
  switch ( genre )
  {
  case GRF_FCT_REELLE:
    if ( GRF_R_CARTESIEN == affichage ) return "cart"e_AIGU"sien (X,Y)";
    if ( GRF_R_POLAIRE   == affichage ) return "polaire (r,T)";
    break;

  case GRF_FCT_COMPLEXE:
    if ( GRF_C_CHEMIN == affichage ) return "-X=Re, Y=Im";
    if ( GRF_C_IM_MOD == affichage ) return "Im + module";
    if ( GRF_C_RE_MOD == affichage ) return "Re + module";
    if ( GRF_C_RE_IM  == affichage ) return "Im + Re";
    break;

  case GRF_SERIE:
    if ( GRF_S_NORMAL == affichage ) return "Valeurs";
    if ( GRF_S_BOXPLOT == affichage ) return "Bo"i_CIRC"te "a_GRAVE" moustaches";
    if ( GRF_S_TUKEY == affichage ) return "Diagramme de Tukey";
    break;

  case GRF_COURBE_PLANE:
    break;

  case GRF_COURBE_ESPACE:
    break;

  case GRF_SURFACE:
    break;

  case GRF_HISTOGRAMME:
    break;

  case GRF_FAISCEAU:
    break;

  case GRF_MULTI_SERIE:
    break;
  }

  return "Inconnu";
}
