/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS
   Cration : 17 juillet 2003

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Paramtres d'affichage des images
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "stats_utils.h"

#include "graphes.h" /* Pour les images contenant des graphes ! */
#include "images.h"
#include "interface.h"
#include "tracer.h" /* Pour l'adaptation dans un rectangle */

#include "img_affichage.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Changer le titre d'une image -------- */

void IMG_Titre(Image *image, const char *titre)
{
  if ( (image == NULL) || (titre == NULL) ) return;

  /* Une mention dans l'historique (dtaill) */
  HIST_Titre( 4, "Nommer une image", "IMG" );
  if ( image->nom != NULL ) {
    HIST_Texte( 4, 4, "Ancien nom  : %s\n", image->nom );
  }
  HIST_Texte( 4, 4, "Nouveau nom : %s\n", titre );

  /* On efface l'ancien titre */
  free( image->nom );

  /* On met en place le nouveau */
  image->nom = strdup( titre );

  /* Interface */
  if ( image == image_courante ) {
    FEN_titre( fen_images, image->nom );
    LIMG_RafraichirListe( VRAI );
  }
}

/* -------- Changer la lgende d'un axe -------- */

int IMG_ChangerLegende(Image *image, unsigned int axe, const char *legende)
{
  char **original = NULL;
  const char *unite;

  if ( NULL == image ) return 0;
  if ( NULL == image->infos_affichage ) return -1;
  if ( NULL == legende ) return -2;

  /* Doit-on vraiment mettre la lgende ? */
  unite = IMG_QuelleUnite( image, axe );
  if ( unite != NULL ) {
    int type_grandeur;
    const char *legende_par_defaut;

    (void) TXT_NormaliserUnite( unite, FAUX, &type_grandeur, NULL );
    legende_par_defaut = TXT_LegendeParDefaut( type_grandeur );

    if ( ( legende_par_defaut != NULL ) &&
	 ( 0 == strcmp( legende_par_defaut, legende ) ) ) {
      /* On n'a donc aucune lgende  mettre ! */
      legende = NULL;
    }
  }

  if ( AXE_X == axe ) {
    original = &(image->infos_affichage->legende_X);
  } else if ( AXE_Y == axe ) {
    original = &(image->infos_affichage->legende_Y);
  } else if ( AXE_Z == axe ) {
    original = &(image->infos_affichage->legende_Z);
  } else {
    (void) fprintf(stderr, "%s, %s l.%u : Axe inconnu, ou combinaison d'axe interdite [%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, axe);
    return 0;
  }

  /* On efface l'ancien texte */
  free( *original ); *original = NULL;

  /* On installe une copie de la nouvelle lgende, s'il faut en mettre une */
  if ( legende != NULL ) {
    *original = strdup( legende );
  }

  /* Interface */
  if ( image == image_courante ) {
    FEN_redessin( fen_images, BI_RDS_UTIL );
    LIMG_RafraichirListe( VRAI );
  }

  return 0;
}

/* -------- Lire la lgende associe  une image -------- */

const char *IMG_QuelleLegende(const Image *image, unsigned int axe)
{
  const char *legende = NULL;
  struct aff_image *infos;

  if ( NULL == image ) return NULL;
  infos = image->infos_affichage;

  if ( AXE_X == axe ) {
    /* Une lgende spciale est-elle disponible ? */
    if ( infos != NULL ) {
      legende = infos->legende_X;
    }

    if ( NULL == legende ) {
      OPTION_A_FAIRE( "lgende par dfaut / X" );
    }
  }

  if ( AXE_Y == axe ) {
    /* Une lgende spciale est-elle disponible ? */
    if ( infos != NULL ) {
      legende = infos->legende_Y;
    }

    if ( NULL == legende ) {
      OPTION_A_FAIRE( "lgende par dfaut / Y" );
    }
  }

  if ( AXE_Z == axe ) {
    /* Une lgende spciale est-elle disponible ? */
    if ( infos != NULL ) {
      legende = infos->legende_Z;
    }

    if ( NULL == legende ) {
      if ( ( image->type >= 0 ) && ( image->type < IMG_NOMBRE_TYPES ) ) {
	legende = txt_legendes_Z[ image->type ];
      }
    }
  }

  /* Aucune lgende trouve */
  if ( NULL == legende ) legende = txt_sans_legende;

  return legende;
}

/* -------- Lire l'unit associe  une image -------- */

const char *IMG_QuelleUnite(const Image *image, unsigned int axe)
{
  const char *unite = NULL;

  if ( NULL == image ) return NULL;

  /* 0. Contrle rapide : cas impossibles */

  /* 1. Y a-t-il une unit impose ? */
  if ( image->infos_affichage != NULL ) {
    if ( AXE_X == axe ) {
      unite = image->infos_affichage->unite_X;
    } else if ( AXE_Y == axe ) {
      unite = image->infos_affichage->unite_Y;
    } else if ( AXE_Z == axe ) {
      unite = image->infos_affichage->unite_Z;
    }
    if ( unite != NULL ) {
      if ( 0 == *unite ) return NULL; /* "" = sans unit : inutile de le trimbaler partout... ! */
      return unite; /* Unit trouve */
    }
  }

  /* 2. On essaye de trouver l'unit  partir du type de image
     [ABANDONN CAR TROP DLICAT ET INTERFRE AVEC LE RESTE !] */
  if ( unite != NULL ) { return unite; }

  /* 3. Aucune unit trouve... */
  return NULL;
}

/* -------- Modifier l'unit associe  une image -------- */

int IMG_ChangerUnite(Image *image, unsigned int axe, const char *unite)
{
  int retour = 0;
  char *nv_unite = NULL;

  if ( NULL == image ) return -1;
  if ( NULL == image->infos_affichage ) return -2;

  /* On commence par copier, car unite peut tre =  un texte dans l'image, la libration poserait donc des problmes */
  if ( unite != NULL ) {
    nv_unite = strdup( unite );
  }

  if ( AXE_X == axe ) {
    free( image->infos_affichage->unite_X );
    image->infos_affichage->unite_X = nv_unite;
  } else if ( AXE_Y == axe ) {
    free( image->infos_affichage->unite_Y );
    image->infos_affichage->unite_Y = nv_unite;
  } else if ( AXE_Z == axe ) {
    free( image->infos_affichage->unite_Z );
    image->infos_affichage->unite_Z = nv_unite;
  } else {
    OPTION_A_FAIRE( "Axe autre que X/Y/Z" );
    retour = -5;
  }

  if ( 0 == retour ) {
/*     (void) fprintf(stderr, "%s,%s l.%u : Nouvelle unit de l'image : %s = %s = %s\n", */
/* 		   __FILE__, __FUNCTION__, __LINE__, nv_unite, unite, IMG_QuelleUnite( image, axe ) ); */

    /* Tout s'est bien pass -> redessins ventuels */
/*     IMG_MiseAJour( image, FAUX ); */
  }

  /* Termin */
  return retour;
}

/* -------- Changer la couleur d'une image -------- */

void IMG_Couleur(Image *image, int couleur)
{
  if ( NULL == image ) return;
  if ( NULL == image->infos_affichage ) return;
  if ( IMG_GENRE_PHOTO == image->genre ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de changer les couleurs d'une photo\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return;
  }

  /* On met la nouvelle couleur */
  image->infos_affichage->couleur = couleur;

  /* Interface */
  if ( image == image_courante ) {
    FEN_redessin( fen_images, BI_RDS_UTIL );
  }
}

/* -------- Initialiser les valeurs moyennes -------- */

void IMG_Moyennes(Image *image)
{
  unsigned long i, nombre_points;
  double moyenne;

  if ( ( NULL == image ) || ( NULL == image->infos_affichage ) ) return;
  if ( image->genre != IMG_GENRE_GRAPHE ) {
    (void) fprintf(stderr, "%s,%s l.%u : L'image %s ne contient pas un graphe par point !\n",
		   __FILE__, __FUNCTION__, __LINE__, image->nom );
    return;
  }
  if ( NULL == image->infos_affichage->moyennes ) {
    (void) fprintf(stderr, "%s,%s l.%u : Table des moyennes non initialisee [image %s] !\n",
		   __FILE__, __FUNCTION__, __LINE__, image->nom );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* Combien de points dans l'image ? */
  nombre_points = image->larg * (unsigned long) image->haut;

  /* Pour chaque point, determination de la valeur */
  for ( i = 0 ; i < nombre_points ; i++ ) {
    (void) STAT_Moyenne_D( image->valeurs.G[ i ]->y, image->valeurs.G[ i ]->nombre_point, &moyenne );
    image->infos_affichage->moyennes[ i ] = moyenne;
  }
}

/* -------- Changer la coupe associe  l'image -------- */

int IMG_AssocierCoupe(Image *image, Graphe *coupe, double x1, double y1, double x2, double y2)
{
  struct aff_image *infos;
  if ( NULL == image ) return -1;
  if ( NULL == image->infos_affichage ) return -2;

  if ( ( coupe != NULL ) && ( coupe->type != GR_PROFIL ) && ( coupe->type != GR_SSTYPE ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Type de graphe incorrect [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, coupe->type);
    return -10;
  }

  infos = image->infos_affichage;
  /* On efface l'ancienne coupe [en demandant ventuellement] */
  if ( infos->coupe != NULL ) {
    GRF_efface( infos->coupe, FAUX, VRAI );
    infos->coupe = NULL;
  }

  if ( NULL == coupe ) { /* On se contente d'effacer */
    infos->xp_1 = 0.0; infos->yp_1 = 0.0;
    infos->xp_2 = 0.0; infos->yp_2 = 0.0;
  } else {
    infos->xp_1 = x1; infos->yp_1 = y1;
    infos->xp_2 = x2; infos->yp_2 = y2;
  }

  /* Interface */
  if ( image == image_courante ) {
    FEN_redessin( fen_images, BI_RDS_UTIL );
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Optimiser pour l'affichage dans une rgion -------- */

void IMG_AdapterAffichage(DES_image *dessin_image)
{
  Image *image;

  /* Par dfaut : pour la fentre principale */
  if ( NULL == dessin_image ) {
    dessin_image = ( DES_image * ) FEN_LireComplement( fen_images, FEN_COMPLEMENT_1 );
  }
  if ( NULL == dessin_image ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR, pas d'infos pour le trac des images !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* De quel image s'occupe-t-on ? */
  image = dessin_image->image;
  if ( NULL == image ) return;

  /* Bornes de dessin : on garde un peu d'espace autour... */
  dessin_image->x_min = image->x_min - image->l_reel / 2.0;
  dessin_image->y_min = image->y_min - image->h_reel / 2.0;

  dessin_image->x_max = image->x_min + image->l_reel * ( image->larg - 0.5 );
  dessin_image->y_max = image->y_min + image->h_reel * ( image->haut - 0.5 );
}
