/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 11 fevrier 2002

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Pour une carto complete, que doit-on afficher comme image ?
     => recherche des tranches...
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "images.h"
#include "tracer.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

typedef struct {
  boolean dedans;    /* VRAI => on garde l'intrieur */
  unsigned int x, y; /* Debut de la zone rectangulaire */
  unsigned int larg, haut; /* Dimensions de la zone rectangulaire */
} Infos_Decoupe;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static Image *IMG_decoupe_rectangle(Image *image, Infos_Decoupe *infos);

static int prepare_arbre_decoupe(unsigned int);
static int GER_decoupe(unsigned int objet, int x, int y);

/* static void    RCT_decoupe(Fenetre *, int, int, int, int); */
static void RDS_decoupe(Fenetre *, int, int, unsigned int, unsigned int);
static int  CLC_decoupe(Fenetre *, int, int, int, int, int);
/* static int CLV_decoupe(Fenetre *fen, unsigned int touche, int etat); */
static int  MSG_decoupe(Fenetre *fen, Message *message);

/* ~~~~~~~~ Realisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~- Calculs ~-~-~-~-~-~-~-~- */

static Image *IMG_decoupe_rectangle(Image *image, Infos_Decoupe *infos)
{
  unsigned long i, j;
  Image *resultat= NULL;

  if (infos == NULL) return NULL;
  if (image == NULL) return NULL;

  if (VRAI == infos->dedans) {
    unsigned long numero_case;

    resultat = IMG_CreerImage( NULL, image, image->genre, image->type, infos->larg, infos->haut );
    if (resultat == NULL) return NULL;
    resultat->l_reel = image->l_reel;
    resultat->h_reel = image->h_reel;
    resultat->x_min = image->x_min + image->l_reel * infos->x; /* Nouveau minimum */
    resultat->y_min = image->y_min + image->h_reel * infos->y;

    for (i = 0 ; i < infos->larg ; i++) {
      for (j = 0 ; j < infos->haut ; j++) {
	numero_case = (infos->y + j) * image->larg + (infos->x + i);
	resultat->valeurs.R[ j * infos->larg + i ] = image->valeurs.R[ numero_case ];
      }
    }
  } else {
    unsigned long numero_case;

    resultat = IMG_CopierImage( image, NULL, FAUX );
    if (NULL == resultat) return NULL;

    for (i = infos->x ; i < infos->x + infos->larg ; i++) {
      for (j = infos->y ; j < infos->y + infos->haut ; j++) {

	IMG_IJ_v_N( image, i, j, &numero_case );
	resultat->valeurs.R[ numero_case ] = 0.0;
      }
    }
  }

  IMG_bornes(resultat);
  return resultat;
}

int IMG_decouper(Image *image)
{
  Fenetre *fenetre;
  Infos_Decoupe *infos;

  if ( image == NULL ) return FAUX;

  infos = (Infos_Decoupe *) malloc( sizeof(Infos_Decoupe) );
  if (infos == NULL) {
    ERR_Memoire( "Op"e_AIGU"ration annul"e_AIGU"e", sizeof( Infos_Decoupe ) );
    return FAUX;
  }

  fenetre = FEN_CreerAvecArbre( TOUS_SF_ASC, "Isoler une portion de l'image", NULL,
				DECOUPE_IMAGE, BI_FEN_ARBRE_HAUT, prepare_arbre_decoupe,
				RDS_decoupe, NULL /* Clavier */, CLC_decoupe, MSG_decoupe,
				NULL/*RCT_pointeur*/, ( int (*)(Fenetre *) ) NULL);
  if (fenetre == NULL) {
    free( infos ); infos = NULL;
    return FAUX;
  }
  FEN_Ouvrir( fenetre );

  infos->dedans = (OBJ_choisit(DECOUPE_IMAGE, DI_INTERIEUR, QUESTION) == VRAI);
  infos->x = 0; infos->y = 0; /* On prend toute l'image, par defaut */
  infos->larg = image->larg; infos->haut = image->haut;

  /* On lie les infos complmentaires */
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_1, image );
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, infos );

  fenetre->drapeaux.redessin = VRAI; /* Redessin si changement de taille */
  fenetre->msg_masque.fermeture = VRAI; /* Messages traits par MSG_tranche */

  return VRAI;
}

/* -------- Redessin de la fenetre de travail -------- */

static void RDS_decoupe(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  DES_image dessin_image;
  Infos_Decoupe *infos;
  Image *image;
  double x_echelle;
  int x_image, y_image;

  if (fenetre == NULL) return;

  /* Prparation du dessin */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  /* Separation entre les deux parties */
  TRC_Separateur( fenetre, VERTICAL, fenetre->w_larg / 2, 0, fenetre->w_haut );

  infos = (Infos_Decoupe *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  image = (Image *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
  if ( ( NULL == infos ) || ( NULL == image ) ) return;

  /* Apercu de l'image moyenne */
  x_image = fenetre->w_x + 5;
  y_image = fenetre->w_y + 5;
  dessin_image.image   = image;
  dessin_image.couleur = ROUGE;
  /*  dessin_image.type_grille = GRILLE_AUCUNE; */
  TRC_image( fenetre, &dessin_image, x_image, y_image, (fenetre->w_larg >> 1) - 10, fenetre->w_haut - 10, IMG_TRACE_NORMAL);

  /* Rectangle pour la zone  dcouper */
  {
    double l_case, h_case;

    x_echelle = IMG_dim_trace(image, (fenetre->w_larg >> 1) - 10, fenetre->w_haut - 10, NULL, NULL);
    l_case = image->l_reel * x_echelle;
    h_case = image->h_reel * x_echelle;

    TRC_Couleur( ROUGE );
    TRC_Rectangle( fenetre, x_image + (infos->x ) * l_case, y_image + (infos->y ) * h_case, infos->larg * l_case, infos->haut * h_case );

    if (VRAI == infos->dedans) { /* On garde l'intrieur */
    } else {
      /* Les diagonales ! */
      TRC_Ligne( fenetre, x_image + (infos->x ) * l_case                       , y_image + (infos->y ) * h_case,
		          x_image + (infos->x ) * l_case + infos->larg * l_case, y_image + (infos->y ) * h_case + infos->haut * h_case );
      TRC_Ligne( fenetre, x_image + (infos->x ) * l_case + infos->larg * l_case, y_image + (infos->y ) * h_case,
		          x_image + (infos->x ) * l_case                       , y_image + (infos->y ) * h_case + infos->haut * h_case );
    }
  }

  /* MOITIE GAUCHE : HISTOGRAMME DE L'IMAGE TOTAL ET POUR LA REGION CHOISIE (?) */
}

/* -------- Gestion des messages -------- */

static int MSG_decoupe(Fenetre *fenetre, Message *message)
{
  Image *image;
  Infos_Decoupe *infos;

  infos = (Infos_Decoupe *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  image = (Image *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );

  if ( message->type == WM_CLOSED ) { /* On veut fermer la fentre */
    Image *resultat;
    int retour;

    if ( (image == NULL) || (message->fenetre.haut != 0) ) /* Demande de fermeture force */
      retour = 2; /* On ferme sans rien demander et sans rien faire */
    else
      retour = EXF_ERREUR(FIN_MODULE); /* On demande s'il faut fermer */

    if (retour == 3) return BI_EVENEMENT_TRAITE; /* Fermeture annulee */

    /* On dtache les infos de la fentre */
    FEN_StockerComplement( fenetre, FEN_COMPLEMENT_1, NULL );
    FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );

    if ( 1 == retour ) { /* Calculs  faire ! */
      /* Faire la dcoupe relle */
      resultat = IMG_decoupe_rectangle(image, infos);
      IMG_AllerImage( resultat );
    }

    free( infos ); infos = NULL;
    FEN_Detruire( fenetre );
    return BI_EVENEMENT_FEN_FERMEE;
  }

  AVERTISSEMENT( "Message non trait [%d]", message->type );
  return BI_EVENEMENT_NON_TRAITE; /* Message non traite */
}

/* -------- Clic de la souris sur la zone utilisateur -------- */

static int CLC_decoupe(Fenetre *fenetre, int bouton __attribute__((__unused__)), int n_clic __attribute__((__unused__)), int etat __attribute__((__unused__)), int x, int y)
{
  Image *image;
  Infos_Decoupe *infos;

  /* On rcupre les infos. Si fenetre = NULL, FEN_LireComplement renvoie NULL : OK */
  infos = (Infos_Decoupe *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  image = (Image *)         FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
  if ( ( NULL == infos ) || ( NULL == image ) ) return BI_EVENEMENT_TRAITE;

  if ( x < (int) (fenetre->w_larg >> 1) ) { /* Clic sur la partie image */
    float l_case, h_case;
    float echelle;
    unsigned int x_image, y_image;
    int i_1, j_1, i_2, j_2;

    x_image = fenetre->w_x + 5;
    y_image = fenetre->w_y + 5;
    echelle = IMG_dim_trace(image, (fenetre->w_larg >> 1) - 10, fenetre->w_haut - 10, NULL, NULL);
    l_case = image->l_reel * echelle;
    h_case = image->h_reel * echelle;

    /* Premier point cliqu */
    i_1 = (int) ( (x - x_image)/l_case );
    j_1 = (int) ( (y - y_image)/h_case );
    if ( prepare_zoom(fenetre, x, y, &i_2, &j_2) == VRAI ) {
      i_2 = (int) ( (i_2 - x_image)/l_case );
      j_2 = (int) ( (j_2 - y_image)/h_case );

      if (i_2 < i_1) { int i = i_2; i_2 = i_1; i_1 = i;}
      if (j_2 < j_1) { int j = j_2; j_2 = j_1; j_1 = j;}

      if ( (i_2 < 0) || (j_2 < 0) || (i_1 >= (int) image->larg) || (j_1 >= (int) image->haut) )
	return VRAI; /* Clic en-dehors, on ne fait rien ! */
      if (i_1 < 0) i_1 = 0; /* On se recadre dans la zone permise */
      if (j_1 < 0) j_1 = 0;
      if (i_2 >= (int) image->larg) i_2 = image->larg - 1;
      if (j_2 >= (int) image->haut) j_2 = image->haut - 1;

      infos->x = i_1; infos->y = j_1;
      /*      printf("Coordonnees debut : %u, %u\n", infos->x, infos->y); */
      infos->larg = i_2 - i_1 + 1; infos->haut = j_2 - j_1 + 1;

      FEN_redessin(fenetre, BI_RDS_TOUT); /* On affiche */
    }
  }

  return BI_EVENEMENT_TRAITE;
}

/* -------- Gestion de l'arbre dans la fentre -------- */

static int GER_decoupe(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  Fenetre *fenetre;
  Infos_Decoupe *infos;

  fenetre = ARB_QuelleFenetre(arbres + DECOUPE_IMAGE, FAUX);
  infos = (Infos_Decoupe *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  switch (objet)
  {
  case DI_INTERIEUR:
  case DI_EXTERIEUR:
    OBJ_choisit(DECOUPE_IMAGE, objet, FORCE_OUI);
    infos->dedans = (objet == DI_INTERIEUR);
    FEN_redessin(fenetre, BI_RDS_UTIL);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparation de l'arbre -------- */

static int prepare_arbre_decoupe(unsigned int numero_arbre)
{
  arbres[ numero_arbre ].drapeaux.prepare = VRAI;
  arbres[ numero_arbre ].fonction.arbre = GER_decoupe;

  return 0;
}
