/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Effacement des images
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "images.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LIENS_RESTANTS 2102 /* 21 fvrier = anniversaire va (quitte  prendre une valeur arbitraire...) */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int IMG_DefaireLiens( Image *image );
static int IMG_EffacerGraphes( Image *image );
static int IMG_EffacerInfosAffichage( Image *image );

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int IMG_EffacerImage(Image *image, boolean eff_graphes)
{
  int retour;

  if ( NULL == image ) return 0;

  ENTREE_FONCTION( __FUNCTION__, "image=%p [%s] eff_graphes=%d", image, image->nom, eff_graphes);

  if ( FAUX == image->drapeaux.isolee ) {
    retour = IMG_IsolerImage( image );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible d'isoler l'image\n",
		     __FILE__, __FUNCTION__, __LINE__);
      SORTIE_FONCTION( __FUNCTION__, "%d [ERREUR dans IsolerImage]", -1);
      return -1;
    }

    /* On libre, met  jour,... tout ce qui pourrait utiliser cette image
       Postulat : si l'image est isole, elle n'est utilise et efface qu'une fois */
    do {
      retour = IMG_DefaireLiens( image );
    } while ( LIENS_RESTANTS == retour );
  }

  /* On efface les ventuelles barres d'erreur associes */
  IMG_EffacerImage( image->erreurs, FAUX );
  image->erreurs = NULL;

  /* On libre */
  IMG_EffacerInfosAffichage( image );
  if ( VRAI == eff_graphes ) {
    IMG_EffacerGraphes( image );
  }

  free( image->valeurs.R ); image->valeurs.R = NULL;
  free( image->nom ); image->nom = NULL;
  free( image->ddl ); image->ddl = NULL;
  free( image->y ); image->y = NULL;
  free( image->x ); image->x = NULL;

  free( image );

  /* Tout s'est bien pass */
  SORTIE_FONCTION( __FUNCTION__, "%d", 0);
  return 0;
}

/* -------- Dfaire les liens vers l'image considre -------- */

static int IMG_DefaireLiens( Image *image )
{
  (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Contrle des liens [%s]\n",
		 __FILE__, __FUNCTION__, __LINE__, image->nom );

  return 0; /* Aucun lien trouv -> on peut passer  la suite */
}

static int IMG_EffacerGraphes( Image *image )
{
  struct graphe **graphes;
  struct serie *serie_graphes;
  int retour;
  unsigned long i, nbr_valeurs;

  if ( ( IMG_GENRE_GRAPHE != image->genre ) && ( IMG_GENRE_DIAGRAMME != image->genre ) )
    return 0; /* Pas de graphe  effacer */

  /* Combien de valeurs ? */
  nbr_valeurs = image->larg * (unsigned long) image->haut;

  /* Quelle srie contient les graphes ? */
  serie_graphes = IMG_TrouverSerieGraphes( image );
  if ( serie_graphes != NULL ) {
    /* On dissocie la srie pour pouvoir effacer tranquillement les graphes */
    retour = Serie_DissocierImage( serie_graphes );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans Serie_DissocierImage()\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }

    /* On efface la serie intgralement, avec ses graphes */
    SERIE_GRF_EffacerSerie( serie_graphes, VRAI );
  } else {

    /* On efface tous les graphes, isols */
    for ( i = 0, graphes = image->valeurs.G ; i < nbr_valeurs ; i++, graphes++ ) {
      GRF_eff_isole( *graphes );
    }
  }

  /* On met tout  zro */
  for ( i = 0, graphes = image->valeurs.G ; i < nbr_valeurs ; i++, graphes++ ) {
    *graphes = NULL;
  }

  /* Pas de problme survenu */
  return 0;
}

static int IMG_EffacerInfosAffichage( Image *image )
{
  struct aff_image *infos;

  infos = image->infos_affichage;
  if ( NULL == infos ) return 0; /* Rien  faire... */

  ENTREE_FONCTION( __FUNCTION__, "image=%p [%s, infos=%p]", image, image->nom, image->infos_affichage);

  /* Infos internes maintenant inutiles */
  free( infos->palette.RVB );
  free( infos->legende_X );
  free( infos->legende_Y );
  free( infos->legende_Z );

  /* La coupe est efface aussi [mais sur demande !] */
  GRF_efface( infos->coupe, FAUX, FAUX );
  infos->coupe = NULL;

  /* On peut les librer */
  free( infos );
  image->infos_affichage = NULL;

  SORTIE_FONCTION( __FUNCTION__, "%d", 0);
  return 0;
}
