/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 11 avril 2001

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Construction d'une table d'images en filtrant la liste chane
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "images.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

Image **IMG_FiltrerType(Image *premiere, int type, boolean strict, unsigned long *nombre)
{
  unsigned long i, nbr_convenables = 0;
  Image **table_images = NULL, *idx_img;

  if ( NULL == premiere ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune image indique\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  if ( NULL == nombre ) {
    AVERTISSEMENT( "Pas de variable pour mmoriser le nombre d'images trouves [%p]", nombre );
    return NULL;
  }

  /* Combien d'images conviennent ? */
  for ( idx_img = premiere ; idx_img != NULL ; idx_img = idx_img->suivante ) {
    if ( type == idx_img->type ) {
      nbr_convenables++;
    } else if ( ( FAUX == strict ) && ( IMG_SANS_TYPE == idx_img->type ) ) {
      nbr_convenables++;
#ifdef _DEVELOPPE_
    } else {
      (void) fprintf(stderr, "%s,%s l.%u : Image %s rejete, type = %d au lieu de %d\n",
		     __FILE__, __FUNCTION__, __LINE__, idx_img->nom, idx_img->type, type);
#endif
    }
  }

  if ( 0 == nbr_convenables ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune image ne convient\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

#ifdef _DEVELOPPE_
  (void) fprintf(stdout, "%s : %lu images conviennent\n", __FUNCTION__, nbr_convenables);
#endif

  /* On rserve la table des images */
  table_images = (Image **) malloc( sizeof( Image * ) * nbr_convenables );
  if ( NULL == table_images ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver la table des images\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On la remplit */
  for ( idx_img = premiere, i = 0 ; idx_img != NULL ; idx_img = idx_img->suivante ) {
    if ( type == idx_img->type ) {
      table_images[ i ] = idx_img;
      i++;
    } else if ( ( FAUX == strict ) && ( IMG_SANS_TYPE == idx_img->type ) ) {
      table_images[ i ] = idx_img;
      i++;
    }
  }

  *nombre = nbr_convenables;

  return table_images;
}
