/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Cration, effacement des images (gestion "fondamentale")
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "images.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~- Gestion des index 2D pour une image ~-~-~-~-~-~-~-~- */

inline int IMG_N_v_IJ(Image *image, unsigned long num_case, unsigned int *colonne, unsigned int *ligne)
{
#ifdef _DEVELOPPE_
  if ( NULL == image ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec une image NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    abort();
  }
  if ( ( NULL == colonne ) || ( NULL == ligne ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : colonne = NULL ou ligne = NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    abort();
  }
#endif

  *ligne = (unsigned int) ( num_case / image->larg );
  *colonne = (unsigned int) ( num_case % image->larg );

  return 0;
}

/* -------- Index 2D vers numro de case --------
   colonne = coordonne en X = i
   ligne   = coordonne en Y = j

   Les donnes sont codes ligne par ligne => i varie en premier
 */

inline int IMG_IJ_v_N(Image *image, unsigned int colonne, unsigned int ligne, unsigned long *n_case)
{
  unsigned long numero_case;

#ifdef _DEVELOPPE_
  if ( NULL == image ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec une image NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  }
  if ( NULL == n_case ) {
    (void) fprintf(stderr, "%s,%s l.%u : n_case = NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  }
#endif
  if ( ( ligne >= image->haut ) || ( colonne >= image->larg ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Coordonnes de case incorrectes [l=%u, c=%u][image %u  %u \"%s\"]\n",
		   __FILE__, __FUNCTION__, __LINE__, ligne, colonne, image->larg, image->haut, image->nom);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -25;
  }

  /* Les images sont mmorises en lignes */
  numero_case  = ligne * image->larg;
  numero_case += colonne;

  /* Mmorisation */
  *n_case = numero_case;

  return 0;
}

/* -------- Index 2D vers position (X,Y) -------- */

inline int IMG_IJ_v_XY(Image *image, unsigned int i, unsigned int j, double *x, double *y)
{
#ifdef _DEVELOPPE_
  if ( NULL == image ) {
    (void) fprintf(stderr, "%s, %s l.%u : Appel avec image=NULL, interdit\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
#endif
  if ( ( 0.0 >= image->l_reel ) || ( 0.0 >= image->h_reel ) ) {
    AVERTISSEMENT( "Unit des repres non-dfinie ou absurde [X : %g, Y : %g]\n", image->l_reel, image->h_reel);
    return -2;
  }

  /* Attention : j = indice de colonne => donne X
                 i = indice de ligne   => donne Y */
  *x = image->x_min + image->l_reel * (double) j;
  *y = image->y_min + image->h_reel * (double) i;

  if ( j >= image->larg ) return +12; /* On est hors de l'image ; erreur mineure */
  if ( i >= image->haut ) return +13;

  return 0;
}

/* -------- Position 2D vers index (X,Y) -------- */

inline int IMG_XY_v_IJ(Image *image, double x, double y, unsigned int *i, unsigned int *j)
{
#ifdef _DEVELOPPE_
  if ( NULL == image ) {
    (void) fprintf(stderr, "%s, %s l.%u : Appel avec image=NULL, interdit\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
#endif
  if ( ( 0.0 >= image->l_reel ) || ( 0.0 >= image->h_reel ) ) {
    AVERTISSEMENT( "Unit des repres non-dfinie ou absurde [X : %g, Y : %g]\n", image->l_reel, image->h_reel);
    return -2;
  }

  /* On passe dans le repre local de l'image */
  x -= image->x_min; y -= image->y_min;
  if ( x < 0.0 ) return +2; /* Hors de l'image... */
  if ( y < 0.0 ) return +3;

  x /= image->l_reel;
  y /= image->h_reel;
  /* Attention : j = indice de colonne => donn par X
                 i = indice de ligne   => donn par Y */
  *j = (unsigned int) rint( x );
  *i = (unsigned int) rint( y );
  if ( *j >= image->larg ) return +12;
  if ( *i >= image->haut ) return +13;

  /* Conversion effectue, on est dans l'image */
  return 0;
}

/* -------- Position (X,Y) vers numro de case -------- */

inline int IMG_XY_v_N(Image *image, double x, double y, unsigned long *num_case)
{
  unsigned int i, j;
  int retour;

#ifdef _DEVELOPPE_
  if ( NULL == image ) {
    (void) fprintf(stderr, "%s, %s l.%u : Appel avec image=NULL, interdit\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
#endif

  /* On convertit en index 2D de case : i = indice de ligne */
  retour = IMG_XY_v_IJ( image, x, y, &i, &j );
  if ( retour != 0 ) return -1;

  /* On convertit en index de case */
  return IMG_IJ_v_N( image, j, i, num_case );
}

/* -------- Position du centre du repre par rapport  l'image -------- */

short int IMG_Position_Centre(Image *image)
{
  float x_min, y_min, x_max, y_max;

  if (NULL == image) return -1;

  x_min = image->x_min;
  y_min = image->y_min;
  x_max = x_min + image->larg * image->l_reel;
  y_max = y_min + image->haut * image->h_reel;

  if ( x_min < 0.0 ) {
    if ( y_min < 0.0 ) {
      if ( x_max < 0.0 ) {
	if ( y_max < 0.0 )
	  return IMG_CENTRE_QUADRANT_HD;
	else if ( y_max == 0.0 )
	  return IMG_CENTRE_EXT_BORD_HD;
	else
	  return IMG_CENTRE_COTE_D;
      } else if ( x_max == 0.0 ) {
	if ( y_max < 0.0 )
	  return IMG_CENTRE_EXT_BORD_DH;
	else if ( y_max == 0.0 )
	  return IMG_CENTRE_COIN_HD;
	else
	  return IMG_CENTRE_BORD_D;
      } else {
	if ( y_max < 0.0 )
	  return IMG_CENTRE_COTE_H;
	else if ( y_max == 0.0 )
	  return IMG_CENTRE_BORD_H;
	else
	  return IMG_CENTRE_DANS_IMAGE;
      }
    } else if ( y_min == 0.0 ) { /* y_max est donc forcment positif ! */
      if ( x_max < 0.0 )
	return IMG_CENTRE_EXT_BORD_BD;
      else if ( x_max == 0.0 )
	return IMG_CENTRE_COIN_BD;
      else if ( x_max > 0.0 )
	return IMG_CENTRE_BORD_B;
    } else { /* y_max est forcment positif ! */
      if ( x_max < 0.0 )
	return IMG_CENTRE_QUADRANT_BD;
      else if ( x_max == 0.0 )
	return IMG_CENTRE_EXT_BORD_DB;
      else
	return IMG_CENTRE_COTE_B;
    }
  } else if ( x_min == 0.0 ) { /* x_max est forcment positif */
    if ( y_min < 0.0 ) {
      if ( y_max < 0.0 )
	return IMG_CENTRE_EXT_BORD_GH;
      else if ( y_max == 0.0 )
	return IMG_CENTRE_COIN_HG;
      else if ( y_max > 0.0 )
	return IMG_CENTRE_BORD_G;
    } else if ( y_min == 0.0 ) /* y_max est forcment positif */
      return IMG_CENTRE_COIN_BG; /* -- Cas normal -- */
    else if ( y_min > 0.0 ) /* y_max est forcment positif */
      return IMG_CENTRE_EXT_BORD_GB;
  } else { /* x_max est forcment positif */
    if ( y_min < 0.0 ) {
      if ( y_max < 0.0 )
	return IMG_CENTRE_QUADRANT_HG;
      else if ( y_max == 0.0 )
	return IMG_CENTRE_EXT_BORD_HG;
      else
	return IMG_CENTRE_COTE_G;
    } else if ( y_min == 0.0 )
      return IMG_CENTRE_EXT_BORD_BG;
    else
      return IMG_CENTRE_QUADRANT_BG;
  }

  /* On ne devrait _jamais_ arriver ici */
  return -3;
}
