/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Gestion de base des sries d'image
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "images.h"

#include "img_series.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int IMG_CreerInfosAffichage(Image *image);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~- Cration d'une srie ~-~-~-~-~-~-~-~- */

Serie_Images *IMG_SERIE_CreerSerie(Serie_Images *serie_avant, const char *nom_serie)
{
  Serie_Images *nv_serie;

  ENTREE_FONCTION( __FUNCTION__, "serie_avant=%p nom_serie=%s", serie_avant, nom_serie);

  /* On rserve pour la srie */
  nv_serie = ( Serie_Images * ) calloc( 1, sizeof( Serie_Images ) );
  if ( NULL == nv_serie ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante --- Impossible de crer la srie\n",
		   __FILE__, __FUNCTION__, __LINE__);
    SORTIE_FONCTION( __FUNCTION__, "%p [mmoire insuffisante]", NULL);
    return NULL;
  }

  /* On l'initialise ; calloc => pointeurs dj  NULL, doubles  0.0 [pas sr 100%, mais...] */
  if ( nom_serie != NULL ) {
    nv_serie->nom = strdup( nom_serie );
  }

  /* On la chane */
  if ( NULL == serie_avant ) { /* On l'insre en dbut de chane */
    nv_serie->suivante = IMG_premiere_serie;
    if ( NULL == IMG_premiere_serie ) { /* On fait la premire srie */
      IMG_premiere_serie = nv_serie;
      IMG_AllerSerie( nv_serie );
    } else {
      IMG_premiere_serie = nv_serie;
    }
  } else { /* On l'insre dans la liste des sries */
    nv_serie->suivante = serie_avant->suivante;
    serie_avant->suivante = nv_serie;
  }

  /* Fini, tout va bien */
  SORTIE_FONCTION( __FUNCTION__, "%p", nv_serie);
  return nv_serie;
}

/* ~-~-~-~-~-~-~-~- Effacement d'une srie ~-~-~-~-~-~-~-~- */

int IMG_SERIE_EffacerSerie(Serie_Images *serie, boolean question, boolean eff_images)
{
  int retour;

  if ( NULL == serie ) return 0;

  ENTREE_FONCTION( __FUNCTION__, "serie=%p question=%d eff_images=%d", serie, question, eff_images);

  /* On ne pose la question que si la srie contient des images ! */
  if ( serie->nbr_images > 0 ) {
    if ( VRAI == question ) {
      int reponse;

      reponse = BI_ERR_alerte( fen_images, &Q_EffacerSerie, VRAI, serie->nom, serie->nbr_images );
      if ( 2 == reponse ) {
	SORTIE_FONCTION( __FUNCTION__, "%d [Annulation]", +1);
	return +1; /* On a annul */
      }
    } else if ( FAUX == eff_images ) {
      (void) fprintf(stderr, "%s,%s l.%u : La srie contient encore des images\n",
		     __FILE__, __FUNCTION__, __LINE__);
      SORTIE_FONCTION( __FUNCTION__, "%d [Images dedans]", -1);
      return -1;
    }
  }
  DANS_FONCTION("Questions posees, on continue...", 0);

  /* On efface les images, sans poser de question */
  while ( serie->premiere_image != NULL ) {
    retour = IMG_EffacerImage( serie->premiere_image, VRAI );
    if ( retour != 0 ) { /* Problme  l'effacement -> on annule ! */
      return +1;
    }
  }

  /* On isole la srie. _Aprs_ avoir effac les images, car sinon
     - images non isoles non trouves par IMG_QuelleSerie => abort en v. dveloppement
     - si une image n'est pas efface [demande de sauvegarde / annulation], la srie n'est pas perdue
  */
  if ( IMG_premiere_serie == serie ) {
    IMG_premiere_serie = serie->suivante;
    if ( serie == IMG_serie_courante )
      IMG_serie_courante = serie->suivante;
  } else {
    Serie_Images *idx_serie;

    idx_serie = IMG_premiere_serie;
    while ( idx_serie->suivante != serie ) {
      idx_serie = idx_serie->suivante;
#ifdef _DEVELOPPE_
      if ( NULL == idx_serie ) {
	(void) fprintf(stderr, "%s,%s l.%u : Srie %s hors chanage ???\n",
		       __FILE__, __FUNCTION__, __LINE__, serie->nom );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      }
#endif
    }

    /* Ici, idx_serie est donc la srie juste avant la srie  effacer */
    idx_serie->suivante = serie->suivante;

    /* On repre l'ventuelle nouvelle srie courante : juste aprs si possible, sinon juste avant */
    if ( serie == IMG_serie_courante ) {
      if ( serie->suivante != NULL )
	IMG_serie_courante = serie->suivante;
      else
	IMG_serie_courante = idx_serie;
    }
  }
  serie->suivante = NULL;

  DANS_FONCTION("Srie hors-chanage...", 0);

  /* On peut librer le contenu */
  free( serie->nom ); serie->nom = NULL;

  /* et la srie elle-mme */
  free( serie ); serie = NULL;

  /* Fini, tout va bien */
  SORTIE_FONCTION( __FUNCTION__, "%d", 0);
  return 0;
}

/* -------- Renommer une srie d'images -------- */

int IMG_SERIE_Titre(Serie_Images *serie, const char *nouveau_nom_serie)
{
  if ( NULL == serie ) return 0;
  if ( NULL == nouveau_nom_serie ) return -1;

  /* On efface l'ancien nom */
  free( serie->nom ); serie->nom = NULL;

  /* On installe une copie du nouveau,  librer avec free() ensuite */
  serie->nom = strdup( nouveau_nom_serie );

  /* Termin sans souci */
  return 0;
}

/* ~-~-~-~-~-~-~-~- Gestion du chanage ~-~-~-~-~-~-~-~- */

/* -------- Aller  l'image indique -------- */

void IMG_AllerSerie(Serie_Images *serie)
{
  unsigned long num_serie = 0, nbr_series;
  Serie_Images *idx_serie;

  if ( NULL == serie ) {
    /* Reste-t-il des sries ? */
    if ( NULL == IMG_premiere_serie ) {
      (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Mise  jour fentre / aucune srie en mmoire\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }
    return;
  }

  if ( IMG_serie_courante == serie ) return; /* On ne change pas de srie : rien  faire */

  /* C'est maintenant la nouvelle srie courante */
  IMG_serie_courante = serie;

  /* Position de la srie dans la liste des sries */
  for ( idx_serie = IMG_premiere_serie, nbr_series = 0 ; idx_serie != NULL ; idx_serie = idx_serie->suivante, nbr_series++ ) {
    if ( idx_serie == serie )
      num_serie = nbr_series;
  }

  /* On en dduit taille et position de l'ascenseur horizontal */
  if ( 1 == nbr_series ) {
    FEN_ascenseur( fen_images, ASC_HORIZONTAL, 0, 1000 );
  } else {
    float taille, position;

    taille = 1000.0 / (float) nbr_series;
    position = (1000.0 * num_serie) / (nbr_series - 1.0 );

    FEN_ascenseur( fen_images, ASC_HORIZONTAL, (int) position, (int) taille );
  }

  /* On va  la premire image de cette nouvelle srie */
  IMG_AllerImage( serie->premiere_image );
}

/* -------- Changer l'image de srie -------- */

int IMG_ChangerSerie(Image *image, Serie_Images *nouvelle_serie, Image *avant)
{
  int retour;

  /* 0. Contrles... */
  if ( NULL == image ) return -1;
  if ( ( NULL == nouvelle_serie ) && ( NULL == avant ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de trouver la srie dans laquelle chaner\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -2;
  }

  if ( ( nouvelle_serie != NULL ) && ( avant != NULL ) && ( nouvelle_serie != IMG_QuelleSerie( avant ) ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Contradiction entre l'image avant et la srie indique !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -3;
  }

  /* Remarque : si par hasard on reste dans la mme srie, l'image change de position dans la srie,
     mais cela ne devrait pas avoir d'autre effet secondaire ! */

  /* 1. On isole l'image */
  retour = IMG_IsolerImage( image );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur dans l'isolement, code %d\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -5;
  }

  /* 2. On l'insre dans la nouvelle srie */
  retour = IMG_InsererDansSerie( image, nouvelle_serie, avant );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur dans l'insertion, code %d\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -10;
  }

  /* Tout s'est bien pass */
  return 0;
}


/* -------- Insrer l'image indique dans la srie -------- */

int IMG_InsererDansSerie(Image *image, Serie_Images *serie, Image *avant )
{
  if ( NULL == image ) return -1;
  if ( ( NULL == serie ) && ( NULL == avant ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de trouver la srie dans laquelle chaner\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -2;
  }
  if ( image->suivante != NULL ) {
    (void) fprintf(stderr, "%s,%s l.%u : Image dj dans un chanage\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -3;
  }

  ENTREE_FONCTION( __FUNCTION__, "image=%p [%s] serie=%p avant=%p", image, image->nom, serie, avant);

  if ( NULL == serie ) {
    /* On prend celle  laquelle appartient l'image indique */
    serie = IMG_QuelleSerie( avant );
    if ( NULL == serie ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de trouver la srie : avant [%s] isole ?\n",
		     __FILE__, __FUNCTION__, __LINE__, avant->nom);
      SORTIE_FONCTION( __FUNCTION__, "%d [ERREUR dans IMG_QuelleSerie()]", -2);
      return -2;
    }
#ifdef _DEVELOPPE_
  } else if ( avant != NULL ) {
    Image *img;

    /* Contrle de cohrence */
    for ( img = serie->premiere_image ; img != avant ; img = img->suivante ) {
      if ( NULL == img ) {
	(void) fprintf(stderr, "%s,%s l.%u : avant [%s] n'appartient pas  la srie indique [%s]\n",
		       __FILE__, __FUNCTION__, __LINE__, avant->nom, serie->nom );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      }
    }
#endif
  }

  /* On chane */
  if ( NULL == avant ) { /* Premire image */
    image->suivante = serie->premiere_image;
    serie->premiere_image = image;
  } else { /* On place aprs l'image indique */
    image->suivante = avant->suivante;
    avant->suivante = image;
  }
  serie->nbr_images++; /* Une image de plus dans la srie */

  /* L'image n'est plus isole */
  image->drapeaux.isolee = FAUX;

  /* Si elle est dans une srie, elle est visible, a priori... */
  if ( NULL == image->infos_affichage ) {
    IMG_CreerInfosAffichage( image );
  }

  /* Interface */
  if ( IMG_serie_courante == serie ) {
    /* Adapter l'ascenseur vertical de la fentre, si besoin est */
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- ascenseur vertical fen_images\n",
		   __FILE__, __FUNCTION__, __LINE__);

    if ( 1 == serie->nbr_images ) {
      IMG_AllerImage( image ); /* Premire image de la srie : on l'affiche donc */
    }
  }

  /* Chanage russi */
  SORTIE_FONCTION( __FUNCTION__, "%d", 0);
  return 0;
}

/* -------- Isoler une image -------- */

int IMG_IsolerImage(Image *image)
{
  Serie_Images *serie;

  if ( NULL == image ) return 0;

  /* De quelle srie doit-on isoler ? */
  serie = IMG_QuelleSerie( image );
  if ( NULL == serie ) {
    (void) fprintf(stderr, "%s,%s l.%u : Image %s dj isole ?\n",
		   __FILE__, __FUNCTION__, __LINE__, image->nom);
    return -1;
  }

  /* On isole rellement l'image de la srie */
  if ( image == serie->premiere_image ) { /* Image en dbut de chane */
    serie->premiere_image = image->suivante;
  } else {
    Image *idx;

    idx = serie->premiere_image;
    while ( idx->suivante != image ) {
      idx = idx->suivante;
#ifdef _DEVELOPPE_
      if ( NULL == idx ) {
	(void) fprintf(stderr, "%s,%s l.%u : l'image %s n'appartient pas au chanage %s [pourtant trouv comme tel]\n",
		       __FILE__, __FUNCTION__, __LINE__, image->nom, serie->nom );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      }
#endif
    }

    idx->suivante = image->suivante;
  }

  /* Il y a donc une image de moins dans la srie */
  serie->nbr_images--;
#ifdef _DEVELOPPE_
  if ( 0 == serie->nbr_images ) {
    if ( NULL != serie->premiere_image ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans le dcompte des images [reste 0, premire %p [%s]\n",
		     __FILE__, __FUNCTION__, __LINE__, (void *) serie->premiere_image, serie->premiere_image->nom );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    }
  } else if ( ( NULL == serie->premiere_image ) && ( serie->nbr_images > 0 ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans le dcompte des images [reste %lu, premire NULL]\n",
		   __FILE__, __FUNCTION__, __LINE__, serie->nbr_images);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  }
#endif

  /* S'il s'agit de l'image courante : on va  l'image suivante dans la srie, s'il y en avait une */
  if ( image_courante == image ) {
    if ( NULL == image->suivante ) {
      IMG_AllerImage( serie->premiere_image );
    } else {
      IMG_AllerImage( image->suivante );
    }
  }

  /* et l'image est isole */
  image->suivante = NULL;
  image->drapeaux.isolee = VRAI;

  /* Les erreurs sont forcment des images isoles : pas besoin d'y toucher... */

  /* Tout s'est bien pass */
  return 0;
}

/* --------  quelle srie appartient l'image indique -------- */

Serie_Images *IMG_QuelleSerie(Image *image)
{
  Serie_Images *serie;

  if ( NULL == image ) return NULL;

  ENTREE_FONCTION( __FUNCTION__, "image=%p [%s]", image, image->nom);

  /* Image isole ? */
#ifndef _DEVELOPPE_
  if ( VRAI == image->drapeaux.isolee ) {
    /* V. dveloppement : contrles de cohrence */
    return NULL;
  }
#endif

  for ( serie = IMG_premiere_serie ; serie != NULL ; serie = serie->suivante ) {
    Image *idx_image;

    for ( idx_image = serie->premiere_image ; idx_image != NULL ; idx_image = idx_image->suivante ) {
      if ( idx_image == image ) {
#ifdef _DEVELOPPE_
	if ( VRAI == image->drapeaux.isolee ) {
	  (void) fprintf(stderr, "%s,%s l.%u : Image trouve dans le chanage, marque comme isole : ERREUR GRAVE\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	}
#endif
	SORTIE_FONCTION( __FUNCTION__, "%p", serie);
	return serie; /* On a trouv la srie, tout va bien */
      }
    }
  }

#ifdef _DEVELOPPE_
  if ( FAUX == image->drapeaux.isolee ) {
    (void) fprintf(stderr, "%s,%s l.%u : Image non-trouve dans le chanage, non-marque comme isole : ERREUR GRAVE\n",
		   __FILE__, __FUNCTION__, __LINE__);
    abort();
  }
#endif

  SORTIE_FONCTION( __FUNCTION__, "%p [non-trouve]", NULL);
  return NULL;
}

/* -------- Crer les infos d'affichage pour une image -------- */

static int IMG_CreerInfosAffichage(Image *image)
{
  struct aff_image *infos;

  ENTREE_FONCTION( __FUNCTION__, "image=%p [%s]", image, image->nom);

  if ( image->infos_affichage != NULL ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : L'image a dj des infos d'affichage\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    SORTIE_FONCTION( __FUNCTION__, "%d [Dj des infos : %p]", 0, image->infos_affichage);
    return 0;
  }

  /* On rserve */
  infos = ( struct aff_image * ) calloc( 1, sizeof( struct aff_image ) );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver les infos d'affichage --- Mmoire insuffisante\n",
		   __FILE__, __FUNCTION__, __LINE__);
    SORTIE_FONCTION( __FUNCTION__, "%d [mmoire insuffisante]", -1);
    return -1;
  }

  /* On initialise [calloc -> pointeurs dj NULL, doubles dj 0.0 */
  infos->proprietes = IMG_AVEC_CADRE; /* On trace avec un cadre autour */
  if ( IMG_GENRE_PHOTO == image->genre ) {
    infos->couleur = IMG_PALETTE;
  } else {
    infos->couleur = IMG_FAUSSES_COULEURS; /* Dessin en fausses couleurs par dfaut */
  }

  /* Reservations complementaires, si necessaire */
  if ( IMG_GENRE_GRAPHE == image->genre ) {
    unsigned long nombre_valeurs;

    nombre_valeurs = (image->larg) * (unsigned long) (image->haut);
    infos->moyennes = (double *) calloc( sizeof(double), nombre_valeurs );
    if ( NULL == infos->moyennes ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver la table des moyennes --- Mmoire insuffisante\n",
		     __FILE__, __FUNCTION__, __LINE__);
      SORTIE_FONCTION( __FUNCTION__, "%d [mmoire insuffisante]", -1);
      return -2;
    }
  }
  /* On associe  l'image */
  image->infos_affichage = infos;

  /* Tout s'est bien pass */
  SORTIE_FONCTION( __FUNCTION__, "%d [infos : %p]", 0, image->infos_affichage);
  return 0;
}
