/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS
   Cration : 23 avril 2003

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Transformation des images (matrice quelconque)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "images.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void IMG_DimensionsImage(Image *source, double *matrice, double *x, double *y);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

Image *IMG_TransformationQuelconque(Image *source, double *matrice, double pas_x, double pas_y)
{
  Image *resultat;
  double x_p[ 4 ], y_p[ 4 ];
  double x_min, x_max, y_min, y_max;
  unsigned int i, j, larg, haut;

  if ( NULL == source ) return NULL;
  if ( source->genre != IMG_GENRE_REELLE ) {
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Image de genre %d\n",
		   __FILE__, __FUNCTION__, __LINE__, source->genre);
    return NULL;
  }
  if ( NULL == matrice ) {
    (void) fprintf(stderr, "IMG_TransformationQuelconque : Pas de matrice de transformation indique\n");
    return NULL;
  }

  /* Calcul des dimensions de l'image obtenue  la fin */
  IMG_DimensionsImage( source, matrice, x_p, y_p );
  x_min = x_p[ 0 ]; y_min = y_p[ 0 ];
  x_max = x_p[ 0 ]; y_max = y_p[ 0 ];
  for ( i = 1 ; i < 4 ; i++ ) {
    if ( x_p[ i ] < x_min ) x_min = x_p[ i ];
    if ( y_p[ i ] < y_min ) y_min = y_p[ i ];
    if ( x_p[ i ] > x_max ) x_max = x_p[ i ];
    if ( y_p[ i ] > y_max ) y_max = y_p[ i ];
  }
  larg = 1.0 + ( x_max - x_min ) / pas_x;
  haut = 1.0 + ( y_max - y_min ) / pas_y;

  /* Inversion de la matrice de projection */
  {
    double det, tmp;

    det = matrice[ 0 ] * matrice[ 3 ] - matrice[ 1 ] * matrice[ 2 ];
    if ( 0.0 == det ) {
      (void) fprintf(stderr, "IMG_TransformationQuelconque : Matrice non-inversible\n");
      return NULL;
    }

    tmp = matrice[ 0 ];
    matrice[ 0 ] =  matrice[ 3 ];
    matrice[ 1 ] = -matrice[ 1 ];
    matrice[ 2 ] = -matrice[ 2 ];
    matrice[ 3 ] = tmp;

    for ( i = 0 ; i < 4 ; i++ ) {
      matrice[ i ] /= det;
    }
  }

  /* Cration de l'image rsultante */
  resultat = IMG_CreerImage( NULL, source, source->genre, source->type, larg, haut );
  if ( NULL == resultat ) {
    (void) fprintf(stderr, "IMG_TransformationQuelconque : ERREUR dans creer_image()\n");
    return NULL;
  }
  resultat->x_min = x_min; resultat->y_min = y_min;
  resultat->l_reel = pas_x; resultat->h_reel = pas_y;

  /* Remplissage de l'image */
  for ( i = 0 ; i < resultat->larg ; i++ ) {
    double x, y;

    x = resultat->x_min + resultat->l_reel * (double) i;
    for ( j = 0 ; j < resultat->haut ; j++ ) {
      double x_avant, y_avant;
      double valeur;
      unsigned long n_case;

      y = resultat->y_min + resultat->h_reel * (double) j;

      x_avant = x * matrice[ 0 ] + y * matrice[ 1 ]; /* Matrice 22 en lignes */
      y_avant = x * matrice[ 2 ] + y * matrice[ 3 ];

      IMG_IJ_v_N( resultat,  i, j, &n_case );
      valeur = IMG_lit_valeur( source, x_avant, y_avant, IMG_INTERPOLE_BILINEAIRE, NULL );
      resultat->valeurs.R[ n_case ] = valeur;
    }
  }

  /* Calcul termin */
  IMG_bornes( resultat );
  return resultat;
}

static void IMG_DimensionsImage(Image *source, double *matrice, double *x_p, double *y_p)
{
  double x, y;

  /* Coin BG */
  x = source->x_min; y = source->y_min;
  x_p[ 0 ] = x * matrice[ 0 ] + y * matrice[ 1 ]; /* Matrice 22 en lignes */
  y_p[ 0 ] = x * matrice[ 2 ] + y * matrice[ 3 ];

  /* Coin HG */
  x = source->x_min; y = source->y_min + source->haut * source->h_reel;
  x_p[ 1 ] = x * matrice[ 0 ] + y * matrice[ 1 ]; /* Matrice 22 en lignes */
  y_p[ 1 ] = x * matrice[ 2 ] + y * matrice[ 3 ];

  /* Coin HD */
  x = source->x_min + source->larg * source->l_reel; y = source->y_min + source->haut * source->h_reel;
  x_p[ 2 ] = x * matrice[ 0 ] + y * matrice[ 1 ]; /* Matrice 22 en lignes */
  y_p[ 2 ] = x * matrice[ 2 ] + y * matrice[ 3 ];

  /* Coin BD */
  x = source->x_min + source->larg * source->l_reel; y = source->y_min;
  x_p[ 3 ] = x * matrice[ 0 ] + y * matrice[ 1 ]; /* Matrice 22 en lignes */
  y_p[ 3 ] = x * matrice[ 2 ] + y * matrice[ 3 ];
}
