/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Création : 11 avril 2001

   Imports/export, des fichiers expérimentaux entre autres
   Import d'un fichier contenant une carto réalisée sur ID22 à l'ESRF
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h> /* Pour stat */
#include <unistd.h>

#include "txt_utils.h"
#include "systeme.h"

#include "exafs.h"

#include "esrf.h"
#include "fichiers.h"
#include "images.h"
#include "serie.h"

#include "import_esrf.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* La ligne sur laquelle on travaille */
static unsigned int num_ligne_esrf = ESRF_ID22;

/* Noms des lignes */

static const char *esrf_lignes[ESRF_NOMBRE_LIGNES] = {
  "-ID-01 (diffusion anomale)",
  "-ID-02 (haute brillance)",
  "-ID-03 (surfaces)",
  "-ID-08 (dragon, polaris"e_AIGU")",
  "-ID-09 (faisceau blanc)",
  "-ID-10A (Tro"i_TREMA"ka)",
  "-ID-10B (Tro"i_TREMA"ka)",
  "-ID-11 (mat"e_AIGU"riaux)",
  "-ID-12 (polarisation circulaire)",
  "-ID-13 (microfaisceau)",
  "-ID-14,1 (cristallo)",
  "-ID-14,2 (cristallo)",
  "-ID-14,3 (cristallo)",
  "-ID-14,4 (cristallo)",
  "-ID-15A (diffraction haute "e_AIGU"nergie)",
  "-ID-15B (diffusion in"e_AIGU"lastique)",
  "-ID-16 (diffusion in"e_AIGU"lastique)",
  "-ID-17 (ligne m"e_AIGU"dicale)",
  "-ID-18 (r"e_AIGU"sonance nucl"e_AIGU"aire)",
  "-ID-19 (topographie)",
  "-ID-20 (diffusion magn"e_AIGU"tique)",
  "ID-21 (microscopie X)",
  "ID-22 (microscopie X)",
  "-ID-23 (cristallo.)",
  "-ID-24 (exafs dispersif)",
  "-ID-26 (XAS)",
  "-ID-27 (fluo. X)",
  "-ID-28 (diffusion in"e_AIGU"lastique)",
  "-ID-29 (MAD)",
  "-ID-30 (haute pression)",
  "-ID-31 (diffraction de poudre)",
  "-ID-32 (SEXAFS)",

  "-SNBL/BM-01A (diffraction)",
  "SNBL/BM-01B (exafs)",
  "-D2AM/BM-02 (diffusion anomale)",
  "-BM-05 (Optique)",
  "-GILDA/BM-08",
  "-BM-14 (MAD)",
  "-BM-16 (MAD)",
  "-ROBL/BM-20 (Radiochimie)",
  "-SpLine/BM-25A (exafs,...)",
  "-SpLine/BM-25B (cristallo.)",
  "-Dubble/BM-26 (XAS)",
  "-XMaS/BM-28 (Diffusion magn"e_AIGU"tique)",
  "-FIP/BM-30A (Cristallo.)",
  "-FAME/BM-30B (exafs)",
  "-BM-32 (interfaces)"
};

/* Nombre de commandes differentes pour chaque ligne */

static const unsigned int nbr_types_ligne[ESRF_NOMBRE_LIGNES] = {
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  9,/* ID-21 ; cf à restreindre */
  7,/* ID-22 ; idem */
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,

  0,
  1, /* SNBL/BM-1B : exafs */
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0
};

#define ESRF_NOMBRE_TYPES 9 /* Nombre _maximal_ de types */
#define LG_MAX_TYPE_ESRF  12

static const char *esrf_types_ID21[ESRF_NOMBRE_TYPES] = {
  "ascan",   /* Scan absolu, un moteur */
  "dscan",   /* Scan relatif, un moteur */
  "Escan",   /* ? Vu sur -ID 21 */
  "mcaacq",  /* Spectre de fluo complet 4096 canaux */
  "mesh",    /* Grille 2D -> image, sur id22 */
  "timescan",/* Suivi au cours du temps (id21) */
  "z3escan", /* XANES sur id21 */
  "zapimage",/* Image 2D sur id21 */
  "zapline"  /* Ligne de carto sur id21 */
};

static const char *esrf_types_ID22[7] = {
  "ascan",   /* Scan absolu, un moteur */
  "dscan",   /* Scan relatif, un moteur */
  "Escan",   /* ? Vu sur -ID 21 */
  "mcaacq",  /* Spectre de fluo complet 4096 canaux */
  "mesh",    /* Grille 2D -> image, sur id22 */
  "timescan",/* Suivi au cours du temps (id21) */
  "z3escan", /* XANES sur id21 */
};

static const char *esrf_types_BM01B[1] = {
  "exafs" /* Scan en energie pour un spectre EXAFS */
};

static const char **esrf_types_lignes[ESRF_NOMBRE_LIGNES] = {
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  esrf_types_ID21,
  esrf_types_ID22,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,

  NULL,
  esrf_types_BM01B,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL
};

static boolean esrf_types_choisis[ESRF_NOMBRE_TYPES];
static unsigned long prem_type_affiche = 0;

#define LG_MAX_MOUVEMENT_ESRF  12
#define ESRF_NOMBRE_MOUVEMENTS 21

static const char *esrf_mouvement[ ESRF_NOMBRE_MOUVEMENTS ] = {
  "detx", "dety", "detz",	/* Detecteur */
  "energy", "enmono",		/* Mono */
  "menergy",		        /* Mono sans suivi de l'onduleur - id22 */
  "osax", "osay", "osaz",	/* Order selection area - id21 */
  "pinh", "pinv",		/* Pin hole - id22 */
  "r2theta",			/* Parallelisme 2e cristal - id21 */
  "samh", "samv", "samd",	/* Echantillon - id22 */
  "sampx", "sampy", "sampz",	/* Echantillon - id22 [2011] */
  "samx", "samy", "samz",	/* Echantillon - id21 */
};

static boolean esrf_mvmts_choisis[ ESRF_NOMBRE_MOUVEMENTS ];
static unsigned long prem_mvmt_affiche = 0;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int ESRF_lit_scan(FILE *fichier, unsigned int ligne, int n_scan, const char *nom_fichier,
			 const char *commande, int mode);

static int esrf_charge_type(char *commande, unsigned int ligne);

static void    esrf_installe_types(unsigned int ligne);
static int GER_esrf           (unsigned int objet, int x, int y);
static void    prepare_arbre_esrf (unsigned int ligne);

/* ~~~~~~~~ Realisation pratique ~~~~~ */

int import_ESRF(unsigned int ligne)
{
  char **liste_noms;
  unsigned int nbr_fichier = 0, i;
  const char *extensions[] = {"*.dat", "*.spec"};
  Intervalle_I *intervalles;
  unsigned long nbr_intervalles;

  if ( ligne < ESRF_NOMBRE_LIGNES ) {
    num_ligne_esrf = ligne;
  }

  prepare_arbre_esrf(num_ligne_esrf);

  if ( ARB_Bloquant( IMPORT_ESRF_1, titre_import_esrf, principale, 0 ) == IE1_ANNULER) {
    return FAUX;
  }

  intervalles = TXT_genere_intervalles(lit_chaine(IMPORT_ESRF_1, IE1_SCANS, 0, NULL, TXT_REMPLACE),
				       &nbr_intervalles);
  if (intervalles == NULL) { /* Charger tous les spectres ? */
    if ( 2 == BI_ERR_alerte(NULL, &Question_Intervalle, VRAI) )
      return FAUX;
  }

  liste_noms = CH_selecteur("*.spec,*.dat", titre_import_esrf, 2, extensions, &nbr_fichier, principale );
  if (liste_noms == NULL) {
    free(intervalles);
    return FAUX;
  }

  for (i = 0 ; i < nbr_fichier ; i++) {
    lit_esrf( liste_noms[i], intervalles, nbr_intervalles, num_ligne_esrf);
    free( liste_noms[i] );
  }

  free(liste_noms);
  free(intervalles);
  return VRAI;
}

/* -------- Lecture d'un fichier SPEC / ESRF : routine principale -------- */

int lit_esrf(const char *nom, Intervalle_I *intervalles, unsigned long nbr_intervalles, unsigned int ligne)
{
  FILE *fichier;
  char *commande = NULL;
  int c, n_lu, n_scan, mode;

  if (nom == NULL) return FAUX;

  fichier = fopen(nom, "rt");
  if (fichier == NULL) {
    (void) SYS_ERREUR(ERREUR_LECTURE, nom, NULL);
    return FAUX;
  }

  do {
    c = fgetc(fichier);

    do {
      while (c != '#') { /* On saute les lignes initiales qui ne commencent pas par un # */
#ifdef _DEVELOPPE_
	/*	printf("  ... on saute une ligne de fin du scan precedent\n"); */
#endif
	if (ignore_ligne(fichier) == EOF) {
	  (void) fclose(fichier);
	  return FAUX;
	}
	c = fgetc(fichier);
      }
      /* Ici, on a lu le premier # : quel est le code qui suit ? */
      c = fgetc(fichier);
#ifdef _DEVELOPPE_
      /*	printf("Commentaire de type : %c (%d)\n", c, c); */
#endif
    } while (c != 'S');
    /* Ici, on est tombé sur une ligne #S = scan, on pourra donc charger... */

    n_lu = fscanf(fichier, "%d", &n_scan);
    if (n_lu != 1) {
      (void) SYS_ERREUR(ERREUR_LECTURE, nom, fichier, "Num"e_AIGU"ro de scan manquant");
      (void) fclose(fichier);
      return FAUX;
    }
    if ( TXT_est_dans_intervalle(n_scan, intervalles, nbr_intervalles) == VRAI ) {
      commande = lit_commande(fichier);
      if (commande == NULL) {
	(void) SYS_ERREUR(ERREUR_LECTURE, nom, fichier, "Impossible de lire la commande");
	(void) fclose(fichier);
	return FAUX;
      }
    } else {
#ifdef _DEVELOPPE_
      printf("Spectre ignoré : %d\n", n_scan);
#endif
      commande = NULL; /* On ignore => pas de commande ! */
    }
#ifdef _DEVELOPPE_
    /*    printf("scan n° %d : commande = %s\n", n_scan, commande); */
#endif
    switch ( mode = esrf_charge_type(commande, ligne) )
    {
    case ESRF_MCA:    /* Commande mcaacq = spectre de fluo X */
      {
	Graphe *spectre;

	spectre = id22_lit_mca(fichier, n_scan, nom, serie_courante);
	GRF_couleur( spectre, (n_scan % 15) + 1, ((n_scan / 15) % 15) + 1, 0 );
	GRF_aller( spectre );
	if ( spectre == NULL ) { /* Problème grave */
#ifdef _DEVELOPPE_
	  printf("Problème grave au chargement...\n");
#endif
	  (void) fclose(fichier);
	  free(commande);
	  return FAUX;
	}
      }
      break;

    case ZAP_IMAGE:   /* Commande zapimage = carto sur id21 */
      {
	Image *image __attribute__((__unused__));
	
	image = id21_lit_zapimg(fichier, n_scan, nom);
	/*	if (image == NULL) {
	  (void) fclose(fichier);
	  return FAUX;
	  }*/
      }
      break;

    case GRILLE_MESH: /* Commande mesh = carto sur id22 */
      {
	Image *image = NULL;
	int reponse;

	reponse = BI_ERR_alerte(principale, &Q_Commandes_mesh, VRAI, n_scan);
	if ( 1 == reponse )
	  image = id22_lit_mesh(fichier, n_scan, nom);
	else if ( 2 == reponse )
	  image = id22_lit_mesh_MCA(fichier, n_scan, nom);
	if (image == NULL) {
	  (void) fclose(fichier);
	  return FAUX;
	}
      }
      break;

    case SCAN_ABSOLU:
    case SCAN_RELATIF: /* Spectre classique, absolu ou relatif */
      if (FAUX == ESRF_lit_scan(fichier, ligne, n_scan, nom, commande, mode)) {
	free(commande);
	return FAUX;
      }
      break;

    case ESRF_AUTRE: /* Type inconnu, mais à charger ! */
      break;

    default: /* Ignorer */
      if (ignore_ligne(fichier) == EOF) { /* On saute la fin de la ligne */
	(void) SYS_ERREUR(ERREUR_LECTURE, nom, fichier);
	(void) fclose(fichier);
	free(commande);
	return FAUX;
      }
      break;
    }

    free(commande);
  } while (feof(fichier) == 0); /* Tant que l'on n'est pas à la fin du fichier */

  (void) fclose(fichier); /* Fini */

  fin_colonnes();
  return VRAI; /* On a chargé */
}

/* -------- Renvoie comment l'on doit charger la commande indiquee
   FAUX si on l'ignore
   SCAN_ABSOLU pour ascan, z3escan, timescan
   SCAN_RELATIF pour dscan
   GRILLE_MESH pour mesh
   ZAP_IMAGE pour zapimage
   ZAP_LIGNE pour zapligne
   ESRF_AUTRE pour une autre commande, inconnue
 -------- */

static int esrf_charge_type(char *commande, unsigned int ligne)
{
  unsigned int i;
  const char **esrf_types;

  if (commande == NULL) return FAUX;
  if (ligne >= ESRF_NOMBRE_LIGNES) return FAUX;

  if ( 0 == nbr_types_ligne[ ligne ] ) { /* Aucun type connu */
    (void) fprintf(stderr, "esrf_charge_type : ligne sans type reconnu ! [%u]", ligne);
    return ESRF_AUTRE;
  }

  esrf_types = esrf_types_lignes[ ligne ];
  if ( NULL == esrf_types ) {
    (void) fprintf(stderr, "esrf_charge_type : tableau de types absent ! [ligne %u]", ligne);
    return FAUX;
  }

  /* On cherche dans les types connus */
  for (i = 0 ; i < nbr_types_ligne[ ligne ] ; i++) {
    if ( strcmp( commande, esrf_types[i] ) == 0 ) {
      if ( esrf_types_choisis[i] == FAUX )
	return FAUX;
      
      if (i == SCAN_RELATIF) /* Commande dscan */
	return SCAN_RELATIF;
      else if (i == ESRF_MCA)
	return ESRF_MCA;
      else if (i == GRILLE_MESH)
	return GRILLE_MESH;
      else if (i == ZAP_IMAGE)
	return ZAP_IMAGE;
      else if (i == ZAP_LIGNE)
	return ZAP_LIGNE;
      else
	return SCAN_ABSOLU;
    }
  }

  /* Si on est la : commande inconnue */
#ifdef _DEVELOPPE_
  printf("Import ESRF, commande inconnue : %s\n", commande);
#endif
  if ( OBJ_choisit(IMPORT_ESRF_1, IE1_TYPE_IGNORER, QUESTION) ==  VRAI ) {
    return FAUX;/* On ignore */
  }

  return ESRF_AUTRE; /* Commande inconnue, a charger */
}

/* -------- Lecture d'un scan = type "classique", depend de la ligne -------- */

static int ESRF_lit_scan(FILE *fichier, unsigned int ligne, int n_scan, const char *nom_fichier,
			 const char *commande, int mode)
{
  Graphe *graphe;
  boolean ok = FAUX;
  char *mouvement;

  if (NULL == fichier) return FAUX;

  switch ( ligne )
  {
  case ESRF_ID21: /* Compteurs simples, sur un mouvement */
  case ESRF_ID22:
    mouvement = lit_commande(fichier); /* Quel mouvement a été réalisé ? */
    if (mouvement == NULL) {
      (void) SYS_ERREUR(ERREUR_LECTURE, "Fichier en cours - commande", fichier, "Mouvement absent");
      (void) fclose(fichier);
      return FAUX;
    }
#ifdef _DEVELOPPE_
    printf("Mouvement : %s [%s]\n", mouvement, commande);
#endif

    /* On vérifie : doit-on faire le chargement ? */
    ok = VRAI;
	
    if (ok == VRAI) { /* Mouvement conservé */
      graphe = id22_lit_scan(fichier, mode, n_scan, mouvement, nom_fichier, &ok);
      GRF_couleur( graphe, (n_scan % 15) + 1, ((n_scan / 15) % 15) + 1, 0 );
      GRF_aller(graphe);
      if ( (graphe == NULL) && (ok == FAUX) ) { /* Problème grave, fichier a été fermé */
#ifdef _DEVELOPPE_
	printf("Problème grave au chargement...\n");
#endif
	free(mouvement);
	return FAUX;
      }
    }
    free( mouvement ); mouvement = NULL;
    break;

  case ESRF_BM01B:
    bm01b_lit_exafs(fichier, n_scan, nom_fichier);
    break;
  }

  /* Tout s'est bien passe */
  return VRAI;
}

/* -~-~-~-~-~-~-~-~ Interface proprement dite -~-~-~-~-~-~-~-~ */

/* -------- Gestion de la boite de dialogue pour choisir les spectres a garder -------- */

static int GER_esrf(unsigned int objet, int x __attribute__((__unused__)), int y)
{
  int nv_ligne;
  const char **esrf_types;

  esrf_types = esrf_types_lignes[ num_ligne_esrf ];

  if ( ( (objet >= IE1_TYPE_TXT_1) && (objet <= IE1_TYPE_TXT_4 ) ) ||
       (objet == IE1_TYPE_AVANT  ) || (objet == IE1_TYPE_APRES )   ||
       (objet == IE1_TYPE_FND_ASC) || (objet == IE1_TYPE_BOUTON) ) {
    ARB_GererDefileur_T(IMPORT_ESRF_1, IE1_TYPE_TXT_1, IE1_TYPE_TXT_4, 0,
			IE1_TYPE_AVANT, IE1_TYPE_FND_ASC, IE1_TYPE_BOUTON, IE1_TYPE_APRES,
			nbr_types_ligne[ num_ligne_esrf ], esrf_types, esrf_types_choisis, NULL, &prem_type_affiche, objet, y);
    return BI_EVENEMENT_TRAITE;
  }

  if ( ( (objet >= IE1_TXT_MVMT_1) && (objet <= IE1_TXT_MVMT_4) ) ||
       (objet == IE1_MVMT_AVANT) || (objet == IE1_MVMT_APRES) ||
       (objet == IE1_MVMT_FND_ASC) || (objet == IE1_MVMT_BOUTON) ) {
    ARB_GererDefileur_T(IMPORT_ESRF_1, IE1_TXT_MVMT_1, IE1_TXT_MVMT_4, 0,
			IE1_MVMT_AVANT, IE1_MVMT_FND_ASC, IE1_MVMT_BOUTON, IE1_MVMT_APRES, 
			ESRF_NOMBRE_MOUVEMENTS, (const char **) esrf_mouvement, esrf_mvmts_choisis, NULL,
			&prem_mvmt_affiche, objet, y);
    return BI_EVENEMENT_TRAITE;
  }

  switch (objet)
  {
  case IE1_LIGNE: /* On modifie le type de ligne */
    nv_ligne = SS_menu_texte(esrf_lignes, ESRF_NOMBRE_LIGNES, "Ligne exp"e_AIGU"rimentale / ESRF",
			     IMPORT_ESRF_1, IE1_LIGNE);
    if (nv_ligne < 0) return VRAI; /* On annule */
    num_ligne_esrf = (unsigned int) nv_ligne;
    esrf_installe_types(num_ligne_esrf);
    return BI_EVENEMENT_TRAITE;

  case IE1_TYPE_IGNORER: /* Que faire d'un type inconnu ? */
  case IE1_TYPE_DEMANDE:
    OBJ_choisit(IMPORT_ESRF_1, objet, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Préparation de l'arbre de premier choix / ESRF -------- */

static void prepare_arbre_esrf(unsigned int ligne)
{
  unsigned int i;

  /* Ligne par defaut */
  esrf_installe_types(ligne);

  /* Initialement, aucun type ni mouvement choisi */
  for (i = 0 ; i < ESRF_NOMBRE_MOUVEMENTS ; i++) {
    esrf_mvmts_choisis[i] = FAUX;
  }

  ARB_PreparerDefileur_T(IMPORT_ESRF_1, IE1_TXT_MVMT_1, IE1_TXT_MVMT_4, 0,
			 IE1_MVMT_AVANT, IE1_MVMT_FND_ASC, IE1_MVMT_BOUTON, IE1_MVMT_APRES, 
			 ESRF_NOMBRE_MOUVEMENTS, (const char **) esrf_mouvement, esrf_mvmts_choisis, 0, 0);

  if (arbres[IMPORT_ESRF_1].drapeaux.prepare == FAUX) {
    /* Un peu d'esthétique */
    OBJ_deplace(IMPORT_ESRF_1, IE1_MVMT_DEMANDE, 0, +1, DEPLACE_PIXELS);
    OBJ_deplace(IMPORT_ESRF_1, IE1_TYPE_DEMANDE, 0, +1, DEPLACE_PIXELS);

    arbres[IMPORT_ESRF_1].drapeaux.prepare = VRAI;
    arbres[IMPORT_ESRF_1].fonction.arbre = GER_esrf;
  }
}

/* -------- Installation des types disponibles sur une ligne -------- */

static void esrf_installe_types(unsigned int ligne)
{
  unsigned int i, nbr_types;

  if ( ligne > ESRF_NOMBRE_LIGNES ) {
    (void) fprintf(stderr, "esrf_installe_types : ligne inconnue (%u)\n", ligne);
    return;
  }

  /* Le nom de la ligne */
  met_chaine(IMPORT_ESRF_1, IE1_LIGNE, LG_MAX_LIGNE_ESRF, esrf_lignes[ ligne ], TXT_COPIER);

  /* Nombre de types pour la ligne consideree */
  nbr_types = nbr_types_ligne[ ligne ];
#ifdef _DEVELOPPE_
  printf("esrf_installe_types : ligne = %u, soit %u types\n", ligne, nbr_types);
  for (i = 0 ; i < nbr_types ; i++) {
    printf("   %s\n", esrf_types_lignes[ ligne ][i]);
  }
#endif

  /* On desactive les types... */
  for (i = 0 ; i < nbr_types ; i++) {
    esrf_types_choisis[i] = FAUX;
  }

  /* On installe la liste des types, en commençant au premier */
  prem_type_affiche = 0;
  ARB_PreparerDefileur_T(IMPORT_ESRF_1, IE1_TYPE_TXT_1, IE1_TYPE_TXT_4, 0,
			 IE1_TYPE_AVANT, IE1_TYPE_FND_ASC, IE1_TYPE_BOUTON, IE1_TYPE_APRES,
			 nbr_types, esrf_types_lignes[ ligne ], esrf_types_choisis, 0, 0);
}

/* -~-~-~-~-~-~-~- Briques générales -~-~-~-~-~-~-~- */

/* -------- Nombre de lignes dans un seul scan
   Attention : scans mca commencent par une @, et ensuite lignes "classiques"
               ---> contrôler que cela fonctionne tout de même !
   -------- */

unsigned long ESRF_CompterLignesScan( FILE *fichier )
{
  unsigned long nbr_lignes = 0;
  long position;
  int c;

  /* Position de départ */
  position = ftell( fichier );

  do {
    c = fgetc( fichier );
    ungetc( c, fichier );
    while ( '#' != c ) {
      nbr_lignes++;
      c = ignore_ligne( fichier ); /* On passe à la ligne suivante */
      if ( EOF == c ) break;
    }

    /* # : changement de scan ou coupure de faisceau ? */
    (void) fgetc( fichier ); /* On saute le # */
    c = fgetc( fichier );
    ungetc( c, fichier );
    if ( 'S' == c ) break; /* Passage au scan suivant */
    c = ignore_ligne( fichier );
  } while ( c != EOF );

  /* On se remet au début */
  fseek( fichier, position, SEEK_SET );

#ifdef _DEVELOPPE_
  printf("%s : %lu lignes comptées\n", __FUNCTION__, nbr_lignes);
#endif

  return nbr_lignes;
}

/* -------- Se place au début de la ligne du scan suivant
   Un scan commence par #S

   Renvoie VRAI si on a trouvé un scan, FAUX s'il n'y en a pas après
   -------- */

boolean ESRF_TrouverScanSuivant(FILE *fichier)
{
  int c;

  if ( NULL == fichier ) return FAUX;

  /* On saute les lignes de données du scan courant, si nécessaire */
  do {
    c = fgetc( fichier );
    if ( EOF == c ) return FAUX;
    ungetc( c, fichier ); /* Pour être compatible avec ignore_ligne() */
    while ( c != '#' ) {
      c = ignore_ligne( fichier );
      if ( EOF == c ) return FAUX; /* Fin de fichier atteinte, pas de scan trouvé */
    }

    /* Ici, on est sur un commentaire */
    (void) fgetc( fichier ); /* Le # du commentaire */
    c = fgetc( fichier );
    if ( 'S' == c ) { /* #S trouvé : on a trouvé le scan suivant */
      fseek( fichier, -2, SEEK_CUR ); /* On se remet au début de la ligne */
      return VRAI; /* Et on a trouvé */
    }
    /* Autre commentaire : en-tête du fichier ou coupure de faisceau ? */

    c = ignore_ligne( fichier );
  } while ( c != EOF );

  /* Si on sort comme ça, c'est que l'on a atteint la fin du fichier ! */
  return FAUX;
}
