/* LASE = Logiciel d'analyse des spectres EXAFS
   (c) 1998, Emmanuel CURIS - Cration de ce fichier : 1er juillet 1998
   Dernire modification : 1er juillet 1998

   Tout ce qui concerne l'import de fichiers contenant des graphes  des formats autres

   Import des fichiers sortis par FEFF
   Rcupr de FICHIERS.c
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "fichiers.h"
#include "graphes.h"

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~~~~ */

/* -------- On passe l'en-tte FEFF : on lit jusqu' tomber sur @#
            qui marque la fin d'un en-tte dans un fichier FEFF normal */

int FEFF_SauterEnTete(FILE *fichier)
{
  int car[2];

  car[0] = 0; car[1] = 0;
  while ( ! ( (car[0] == '@') && (car[1] == '#') ) ) {
    car[0] = car[1];
    car[1] = fgetc(fichier);

    if (car[1] == EOF) {
      (void) fprintf(stderr, "%s,%s l.%u : Marque @# non trouve.\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
  }
  /* On se place au dbut de la ligne suivante */
  ignore_ligne( fichier );

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Travail  faire aprs le chargement des spectres -------- */

int FEFF_PostChargement(int format, unsigned int colonne_X, unsigned int nombre, struct graphe **table, const char *nom_fichier)
{
  char *titre;

  if ( NULL == table ) return -1;
  if ( nombre < 4 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Nombre de colonnes incorrect [%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, nombre);
    return -1;
  }

  switch ( format )
  {
  case ASCII_FORMAT_FEFF_Cp: /* chi partiels */
    if ( nombre > 4 ) {
      /* Colonne 5 : dphasage "pur" (terme en 2kr t) */
      GRF_ChangerType( table[ 4 ], GR_EXAFSPHA );
      titre = TXT_genere_chaine( "%F : d"e_AIGU"phasage "E_AIGU"lectronique", nom_fichier );
      GRF_titre( table[ 4 ], titre );
      free( titre ); titre = NULL;
    }

  case ASCII_FORMAT_FEFF_C: /* chi.dat */
    /* Colonne 2 : EXAFS en fonction de k, k en A^-1 => lgende, unit par dfaut conviennent */
    GRF_ChangerType( table[ 1 ], GR_EXAFS );
    titre = TXT_genere_chaine( "%F : oscillations EXAFS", nom_fichier );
    GRF_titre( table[ 1 ], titre );
    free( titre ); titre = NULL;

    /* Colonne 3 : amplitude (totale => module complexe) */
    GRF_ChangerType( table[ 2 ], GR_COMPMOD );
    titre = TXT_genere_chaine( "%F : amplitude totale", nom_fichier );
    GRF_titre( table[ 2 ], titre );
    free( titre ); titre = NULL;

    /* Colonne 4 : dphasage (total => argument complexe) */
    GRF_ChangerType( table[ 3 ], GR_COMPARG );
    titre = TXT_genere_chaine( "%F : dphasage total", nom_fichier );
    GRF_titre( table[ 3 ], titre );
    free( titre ); titre = NULL;
    break;

  case ASCII_FORMAT_FEFF_F: /* feff*.dat : */
    if ( nombre < 7 ) {
      (void) fprintf(stderr, "%s,%s l.%u : feff*.dat, nombre de colonnes incorrect [%u, 7 attendu]\n",
		     __FILE__, __FUNCTION__, __LINE__, nombre);
      return -2;
    }

    /* Colonne 2 : Re[ 2 * phc ] = dphasage de l'atome central */
    GRF_ChangerType( table[ 1 ], GR_EXAFSPHA );
    titre = TXT_genere_chaine( "%F : dphasage de l'atome central", nom_fichier );
    GRF_titre( table[ 1 ], titre );
    free( titre ); titre = NULL;

    /* Colonne 3 : amplitude lectronique "pure" */
    GRF_ChangerType( table[ 2 ], GR_EXAFSAMP );
    titre = TXT_genere_chaine( "%F : amplitude lectronique", nom_fichier );
    GRF_titre( table[ 2 ], titre );
    free( titre ); titre = NULL;

    /* Colonne 4 : dphasage pur */
    GRF_ChangerType( table[ 3 ], GR_EXAFSPHA );
    titre = TXT_genere_chaine( "%F : dphasage lectronique", nom_fichier );
    GRF_titre( table[ 3 ], titre );
    free( titre ); titre = NULL;

    /* Colonne 5 : facteur de rduction S0 */
    GRF_ChangerType( table[ 4 ], GR_FAC_RED );
    titre = TXT_genere_chaine( "%F : facteur de rduction", nom_fichier );
    GRF_titre( table[ 4 ], titre );
    free( titre ); titre = NULL;

    /* Colonne 6 : libre parcours moyen */
    GRF_ChangerType( table[ 5 ], GR_LIBREP );
    titre = TXT_genere_chaine( "%F : libre parcours moyen", nom_fichier );
    GRF_titre( table[ 5 ], titre );
    free( titre ); titre = NULL;

    /* Colonne 7 : Re[ p ] */
    GRF_ChangerType( table[ 6 ], GR_AUTRETYPE );
    titre = TXT_genere_chaine( "%F : Re(p)", nom_fichier );
    GRF_titre( table[ 6 ], titre );
    free( titre ); titre = NULL;
    break;

  case ASCII_FORMAT_FEFF_X: /* xmu.dat */
    if ( nombre < 6 ) {
      (void) fprintf(stderr, "%s,%s l.%u : xmu.dat, nombre de colonnes incorrect [%u, 6 attendu]\n",
		     __FILE__, __FUNCTION__, __LINE__, nombre);
      return -3;
    }

    /* Colonne 4 : coefficient d'absorption atomique total, =0(1+khi) */
    GRF_ChangerType( table[ 3 ], GR_MOINSMAT );
    titre = TXT_genere_chaine( "%F : Coefficient d'absorption atomique total", nom_fichier );
    GRF_titre( table[ 3 ], titre );
    free( titre ); titre = NULL;
    if ( 2 == colonne_X ) {
      GRF_ChangerLegende( table[ 3 ], AXE_X, E_AIGU"nergie relative", VRAI );
      GRF_ChangerUnite( table[ 3 ], AXE_X, "eV" );
    } else if ( 3 == colonne_X ) {
      GRF_ChangerLegende( table[ 3 ], AXE_X, "Nombre d'onde", VRAI );
      GRF_ChangerUnite( table[ 3 ], AXE_X, ANGSTROM"^-1" );
    }

    /* Colonne 5 : coefficient d'absorption atomique intrinsque, 0 */
    GRF_ChangerType( table[ 4 ], GR_MU0 );
    titre = TXT_genere_chaine( "%F : Coefficient d'absorption atomique intrinsque", nom_fichier );
    GRF_titre( table[ 4 ], titre );
    free( titre ); titre = NULL;
    if ( 2 == colonne_X ) {
      GRF_ChangerLegende( table[ 4 ], AXE_X, E_AIGU"nergie relative", VRAI );
      GRF_ChangerUnite( table[ 4 ], AXE_X, "eV" );
    } else if ( 3 == colonne_X ) {
      GRF_ChangerLegende( table[ 4 ], AXE_X, "Nombre d'onde", VRAI );
      GRF_ChangerUnite( table[ 4 ], AXE_X, ANGSTROM"^-1" );
    }

    /* Colonne 6 : oscillations EXAFS */
    GRF_ChangerType( table[ 5 ], GR_EXAFS );
    titre = TXT_genere_chaine( "%F : Oscillations EXAFS", nom_fichier );
    GRF_titre( table[ 5 ], titre );
    free( titre ); titre = NULL;
    if ( 1 == colonne_X ) {
      GRF_ChangerLegende( table[ 5 ], AXE_X, E_AIGU"nergie", VRAI );
      GRF_ChangerUnite( table[ 5 ], AXE_X, "eV" );
    } else if ( 2 == colonne_X ) {
      GRF_ChangerLegende( table[ 5 ], AXE_X, E_AIGU"nergie relative", VRAI );
      GRF_ChangerUnite( table[ 5 ], AXE_X, "eV" );
    }
    break;
  }

  /* Tout s'est bien pass */
  return 0;
}
