/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
   Dfinitions des messages d'erreur pour le traitement gnral des spectres
   (c) Emmanuel CURIS 1998-2001

  16, 25 mars 1998
    * Rcupr les messages d'erreur dans graphes.h
*/

#ifndef ERREURS_H_
#define ERREURS_H_ /* Protection contre l'inclusion multiple */

/* ~~~~~~~~ Fonctions de contrle des valeurs relles ~~~~~~~~ */

#if defined(A_FP_CLASS_) /* Fonctions IEEE prsentes sous forme de fp_class */

# include <math.h>
# include <fp_class.h>
# define is_inf(x) ( (fp_class(x) == FP_POS_INF) || (fp_class(x) == FP_NEG_INF) )
# define is_nan(x) ( (fp_class(x) == FP_SNAN) || (fp_class(x) == FP_QNAN) )

#elif defined(A_FPCLASS_) /* Fonctions IEEE prsentes sous la forme de fpclass */

# include <ieeefp.h>
# define is_inf(x) ( (fpclass(x) == FP_PINF) || (fpclass(x) == FP_NINF) )
# define is_nan(x) ( (fpclass(x) == FP_SNAN) || (fpclass(x) == FP_QNAN) )

#elif defined(A_CLASS_)   /* Fonctions IEEE prsentes sous la forme de class */

# include <math.h>
# include <float.h>
# define is_inf(x) ( (class(x) == FP_PLUS_INF) || (class(x) == FP_MINUS_INF) )
# define is_nan(x) ( isnan(x) )

/* D'aprs man class, on pourrait faire comme suit :

# define is_nan(x) ( (class(x) == FP_NANS) || (class(x) == FP_NANQ) )

  mais il se trouve que FP_NANS et FP_NANQ semblent ne pas tre dfinis dans le math.h et le float.h
  (essais sur IBM AIX 4.1)
*/

#elif defined(A_FPCLASSIFY) /* Fonctions IEEE prsentes sous la forme de fpclassify */

# include <math.h>
# define is_inf(x) ( (fpclassify(x) == FP_MINUS_INF) || (fpclassify(x) == FP_PLUS_INF) )
# define is_nan(x) ( (fpclassify(x) == FP_SNAN) || (fpclassify(x) == FP_QNAN) )

#elif defined(A_IS_INF_)  /* Fonctions IEEE prsentes sous forme de isinf/isnan */

# include <math.h>
# define is_inf(x) isinf(x)
# define is_nan(x) isnan(x)

#else /* Fonctions IEEE absentes, il faut les routines de test personnelles */
int is_inf(double x); /* Renvoie VRAI si x = +/-Inf */
int is_nan(double x); /* Renvoie VRAI si x = NaN    */
#endif

/* -------- Messages d'erreur partiels "standards" (dans variables.c) -- */

extern const char *txt_err_calc_annule;
extern const char *txt_err_val_inchangee;
extern const char *txt_err_val_nulle;

/* -------- Macros facilitant les messages -------- */

#define ERR_Memoire(message,taille) ERR_MemoireInsuffisante( __FILE__, __FUNCTION__, __LINE__, (unsigned long) (taille), message )

/* -------- Fonctions plus spcifiques -------- */

inline void ERR_MemoireInsuffisante(const char *fichier, const char *fonction, unsigned long num_ligne, unsigned long taille, const char *message);

/* ~~~~~~~~ Codes d'erreur : messages systmes ~~~~~~~~ */

#define NOMBRE_SYS_MESSAGES 12

#define MANQUE_MEMOIRE     0 /* Mmoire insuffisante                    */
#define QUITTER_PROGRAMME  1 /* Faut-il quitter le programme ?          */
#define TROP_FENETRES      2 /* Trop de fentres ouvertes...            */
#define FEN_INEXISTANTE    3 /* Fentre n'existe pas                    */
#define ERREUR_LECTURE     4 /* Erreur  la lecture d'un fichier        */
#define ERREUR_ECRITURE    5 /* Erreur  l'criture d'un fichier        */
#define STATION_REFUSEE    6 /* VDI ne peut pas ouvrir de station       */
#define TROP_FICHIERS      7 /* Trop de fichiers slectionns           */
#define ERREUR_INCONNUE    8 /* Erreur inconnue                         */
#define FEN_OUVERTE        9 /* Fentre dj ouverte !                  */

#define ERREUR_RESSOURCE  10 /* Windows : impossible de charger le ressource */
#define ERREUR_MENUCREATE 11 /* Windows : erreur  la cration d'un menu */

int SYS_ERREUR(unsigned int code_erreur, ...); /* Affichage d'un message d'erreur gnral */

/* -------- Codes de retour des fonctions, si renvoient un code... -------- */

#define PAS_DE_PROBLEME 0	/* Calcul OK */

/* -------- Codes d'erreur pour les messages d'erreur gnraux de LASE -------- */

#define ERR_DOUBLONS      0
#define DIV_ZERO          1
#define VALEUR_ABSURDE    2
#define ERR_NON_TRIE      3
#define ERR_UN_POINT      4
#define NBRPTS_INCORRECT  5
#define ERR_DISJOINTS     6
#define GRF_PROTEGE       7
#define NORM_IMPOSSIBLE   8
#define FIN_MODULE        9
#define DEJA_ACTIF       10
#define OTE_ABERRANCE    11
#define BORNES_FAUSSES   12
#define PAS_FAUX         13
#define TROP_PEU_POINTS  14
#define NON_COMPLEXE     15
#define PUISSANCE_2      16
#define NON_SAUVE        17
#define GRAPHE_COMPLEXE  18
#define ERR_STRUCT_GRF   19
#define GRF_DEJA_PRESENT 20
#define PROBLEME_DIVERS  21
#define TROP_VIEUX       22
#define FORMAT_INCONNU   23
#define POINT_INEXISTANT 24
#define MAUVAIS_DOUBLE   25
#define AUCUN_GRAPHE     26

int EXF_ERREUR(unsigned int num_erreur, ...); /* Affiche un message d'erreur du programme */

/* ~~~~~~~~ Erreurs du module de chargement des fichiers exprimentaux ~~~~~~~~ */

#define EXT_I0_NUL         0
#define EXT_I1_NUL         1
#define AUCUN_ELEMENT      2
#define PAS_D_EN_TETE      3
#define CAR_INCONNU        4
#define AUCUN_DOUBLON      5
#define NOMBRE_DOUBLONS    6
#define AUCUNE_INFORMATION 7
#define EXT_SATURATION     8

int EXT_ERREUR(unsigned int num_erreur, ...); /* Dans Extrait/erreurs.c */

/* ~~~~~~~~ Erreurs lors des calculs statistiques ~~~~~~~~ */

#define NBR_PTS_DIFFERENTS   0
#define TYPES_DIFFERENTS     1
#define STAT_INCOHERENCE     2
#define SANS_RECOUVREMENT    3
#define SIGMA_MOYEN          4
#define MOY_TROP_PEU_GRAPHES 5
#define MOY_TYPES_DIFFERENTS 6

int STAT_ERREUR(unsigned int num_erreur, ...); /* Dans Stats/erreurs.c   */

/* ~~~~~~~~ Erreurs du module de traitement du XANES ~~~~~~~~ */

#define XAN_E1_TROPPETIT 0
#define XAN_E1_EGAL_E1P  1

int XAN_ERREUR(unsigned int code_erreur);    /* Dans XANES/erreurs.c */

/* ~~~~~~~~ Erreurs sur la structure molculaire ~~~~~~~~ */

#define TROP_LIAISONS       0 /* Cet atome a trop de liaisons        */
#define ATOMES_SUPERPOSES   1 /* Deux ou plus atomes sont superposs */
#define ATOMES_CHEVAUCHANTS 2 /* Deux atomes se chevauchent          */
#define ELEMENT_INCONNU     3 /* Numro atomique incorrect           */
#define AUCUN_FRAGMENT      4 /* Molcule sans aucun fragment !      */

int ATM_ERREUR(unsigned int num_erreur, ...); /* Dans Atomes/atm_err.c  */

/* ~~~~~~~~ Erreurs sur le modle pour FEFF ~~~~~~~~ */

#define AUCUN_ATOME             0
#define PAS_ABSORBEUR           1
#define ABSORBEUR_DEFINI        2
#define PLUS_POTENTIEL          3
#define POTENTIEL_FAUX          4
#define FF_TOUT_CHOISIR         5
#define FATM_EFFACER            6
#define FATM_EXISTE_DEJA        7
#define FATM_TT_EFFACER         8
#define FSS_FAUX_NUMERO         9
#define FSS_EFFACER            10
#define FSS_EXISTE_DEJA        11
#define FSS_DEG_FAUSSE         12
#define FF_DIST_NEGATIVE       13
#define EFFACER_ATOMES         14
#define FEF_AUCUN_ATOME        15
#define POT_SS_ATOME           16
#define FEF_INCOHERENCE        17
#define POT_NON_DEFINI         18
#define COMMANDE_IORDER        19
#define COMMANDE_INCONNUE      20
#define Z_ABSORBEUR_DIFFERENTS 21
#define PAS_ATOME_ABSORBEUR    22
#define COMMANDE_END           23
#define FEFF_TERMINE           24
#define FEFF_INTERROMPU        25
#define FEFF_NON_LANCE         26

int FEFF_ERREUR(unsigned int num_erreur, ...); /* Dans FEFF/erreurs.c    */

/* ~~~~~~~~ Erreurs du module ce calcul matriciel ~~~~~~~~~~~ */

#define MAT_NON_INVERSIBLE     0 /* Matrice non inversible */
#define MAT_NON_CARREE         1 /* La matrice n'est pas carre */
#define MAT_NON_DEF_POS        2 /* La matrice n'est pas dfinie, positive */
#define MAT_VAL_ABSURDE        3 /* Valeur indique incohrente avec le type de matrice */
#define MAT_TYPE_IMPOSSIBLE    4 /* Le type de matrice indiqu n'est pas possible */
#define MAT_ELEMENT_INEXISTANT 5 /* L'lment indiqu n'est pas dans la matrice */

int MAT_ERREUR(unsigned int num_erreur, ...); /* Dans Matrices/erreurs.c */

/* ~~~~~~~~ Erreurs lors de l'ajustement  n couches ~~~~~~~~ */

#define AUCUNE_COUCHE         0 /* Aucune couche dfinie pour le modle      */
#define PAS_DE_PHASE          1 /* Pas de dphasage dfini pour la couche    */
#define PAS_D_AMPLITUDE       2 /* Pas d'amplitude dfinie pour la couche    */
#define AUCUNE_VARIABLE       3 /* Aucune variable dfinie pour l'ajustement */
#define INIT_INCORRECTE       4 /* Valeur initiale en-dehors de l'intervalle dfini */
#define TYPE_INCORRECT        5 /* Ce type de graphe ne peut tre ajust     */
#define ARRETER_AJUSTEMENT    6
#define AJUSTEMENT_EN_COURS   7
#define NC_INTERVALLE_VIDE    8
#define NC_DISJOINTS          9
#define RELANCER_AJUSTEMENT  10 /* On relance : que faire des rsultats prcdents ? */
#define PAS_DE_SIGMA         11 /* Demande d'utilisation des sigma sans sigma disponible ! */
#define DEMIN_INCORRECT      12 /* DE min pourrait conduire  des k corrigs ngatifs */
#define DELTAX_TROP_FAIBLE   13 /* Variation d'un paramtre trop petite */
#define BORNE_ATTEINTE       14 /* Un paramtre a atteint la borne de l'intervalle autoris */
#define DELTAERR_TROP_FAIBLE 15 /* Variation de l'erreur trop faible */
#define ERR_MIN_ATTEINTE     16 /* Erreur souhaite atteinte */
#define IMPOSSIBLE_MINIMISER 17 /* Impossible de diminuer davantage l'erreur */
#define ARRET_D_URGENCE      18 /* Plus de graphe de rfrence (effac !)    */
#define ABSORBEUR_SS_SPECTRE 19 /* L'absorbeur slectionn n'a pas de spectre exprimental */
#define VERIF_BORNE_ATTEINTE 20 /* Borne atteinte pour un des paramtres */
#define AJUSTEMENT_DIVERGE   21 /* Par exemple, on trouve +infini pour certaines valeurs */

int NC_ERREUR(unsigned int num_erreur, ...); /* Dans N_couches/nc_erreurs.c */

#endif /* ERREURS_H_ */
