/* LASE - Logiciel d'Analyse des Spectres Expérimentaux
   (c) Emmanuel CURIS 1998 - 2007
   Création : 23 janvier 2007
   
   Codage pour toutes les fonctions de traitement individuel
   FICHIER À INCLURE

   Idée : chaque fonction individuelle est classée dans trois catégories (générique, par genre, par type)
    * générique : s'applique à chaque objet de la nature considérée (graphe, image,...)
    * par genre : s'applique à tout objet du genre considéré (graphe : réel, complexe,...)
    * par type  : s'applique à tout objet du type considéré (graphe réel : exafs, µ,...)

   En fonction de cela, dans la triade (générique/genre/type), chaque fonction a un numéro unique

   But : - menus de fonctions utilisables partout
         - automatisation
	 -...

   Il y a aussi les tableaux de texte, qui sont dans Interface/fonctions.c et Textes/__/fonctions.txt
*/

#ifndef LASE_FONCTIONS_H_
# define LASE_FONCTIONS_H_ /* Protection contre l'inclusion multiple */

/* ~-~-~-~-~-~-~-~-~ Les catégories de fonction ~-~-~-~-~-~-~-~-~ */

/* Champ d'action de la fonction */
#define FONCTION_EST_GLOBALE   0 /* S'applique à tous les objets */
#define FONCTION_EST_PAR_GENRE 1 /* S'applique à tout objet du genre indiqué */
#define FONCTION_EST_PAR_TYPE  2 /* S'applique à tout objet du type indiqué */

/* Nature de l'objet traité par la fonction */
#define FONCTION_POUR_GRAPHE   0 /* S'applique à un graphe */
#define FONCTION_POUR_IMAGE    1 /* S'applique à une image */
#define FONCTION_POUR_MOLECULE 2 /* S'applique à une molécule */

/* Codage du genre : codes de genre pour graphe/image/... */
/* Codage du type : codes de type pour graphe/image/... */

/* ~-~-~-~-~-~-~-~-~ Les codes de fonction ~-~-~-~-~-~-~-~-~ */

/* -------- Fonctions appliquées aux graphes -------- */

/* ---- Fonctions génériques ---- */

#define FCT_GRF_SAUVER_GRAPHE       0
#define FCT_GRF_EXPORTER_GRAPHE     1
#define FCT_GRF_EXPORT_POSTSCRIPT   2 /*  3 : séparateur */
#define FCT_GRF_EFFACER_GRAPHE      4
#define FCT_GRF_COPIER_GRAPHE       5
#define FCT_GRF_DEPLACER_GRAPHE     6
#define FCT_GRF_ISOLER_GRAPHE       7 /*  8 : séparateur */
#define FCT_GRF_ERREUR_V_GRAPHE     9
#define FCT_GRF_COVARIANCE_V_IMAGE 10

#define FCT_GRF_NOMBRE_OPT_GENERIQUES 11 /* Inclue les séparateurs */

/* ---- Fonctions par genre ---- */

/* Les options valables pour une fonction Y=f(X), réelle */
#define FCT_GRF_INVERSER_XY       0
#define FCT_GRF_TRIER_X           1 /*  2 : séparateur */
#define FCT_GRF_DECOMPOSER_R      3
#define FCT_GRF_OTER_ABERRANCE    4
#define FCT_GRF_TRONQUER_R        5 /*  6 : séparateur */
#define FCT_GRF_NORMALISER_R      7
#define FCT_GRF_INTERPOLER_R      8
#define FCT_GRF_LISSER_R          9
#define FCT_GRF_TRANSFORMER_R    10
#define FCT_GRF_TRANSFORMER_LOG  11 /* 12 : séparateur */
#define FCT_GRF_DERIVER_R        13
#define FCT_GRF_INTEGRER_R       14 /* 15 : séparateur */
#define FCT_GRF_APPROX_DROITE    16
#define FCT_GRF_MODELISER        17 /* 17 : séparateur */
#define FCT_GRF_BRUITER          19

#define FCT_GRF_NOMBRE_OPT_REEL 20

/* Les options valables pour une fonction Z=f(X), à valeurs complexes */
#define FCT_GRF_PARTIE_REELLE     0
#define FCT_GRF_PARTIE_IMAGINAIRE 1
#define FCT_GRF_MODULE            2
#define FCT_GRF_ARGUMENT          3 /* 4 : séparateur */
#define FCT_GRF_TRONQUER_C        5 /* 6 : séparateur */
#define FCT_GRF_NORMALISER_C      7
#define FCT_GRF_INTERPOLER_C      8
#define FCT_GRF_LISSER_C          9
#define FCT_GRF_TRANSFORMER_C    10 /* 11 : séparateur */
#define FCT_GRF_DERIVER_C        12
#define FCT_GRF_INTEGRER_C       13

#define FCT_GRF_NOMBRE_OPT_COMPLEXE 14

/* Les options valables pour une série de valeurs, à valeurs réelles */
#define FCT_GRF_CHOISIR_X            0 /* 1 : séparateur */
#define FCT_GRF_STAT_DESCRIPTIVE     2
#define FCT_GRF_FONCTION_REPARTITION 3
#define FCT_GRF_HISTOGRAMME          4
#define FCT_GRF_DROITE_HENRY         5
#define FCT_GRF_ANVA_1F              7 /* 6 : séparateur "Analyse de variance" */

#define FCT_GRF_NOMBRE_OPT_SERIE 8

/* Les options pour les graphes GR_BRUT (réels) */

#define FCT_GRF_BRUT_CONVERSION 0 /* Conversion fluorescence/absorption */
#define FCT_GRF_BRUT_OTER_BLANC 2 /* Ôter un blanc contenu dans un autre spectre */
#define FCT_GRF_BRUT_MATRICE    3 /* Ôter la contribution de la matrice */
#define FCT_GRF_BRUT_TOUT_FAIRE 4 /* 5 : Séparateur */
#define FCT_GRF_BRUT_REPRODUIRE 6
#define FCT_GRF_BRUT_ERREUR_X   8 /* Estimer l'erreur [systématique] en X */

#define FCT_GRF_NOMBRE_OPT_BRUT 9

/* Les options pour les graphes GR_MOINSMAT (réels) */

#define FCT_GRF_MOINSMAT_DERIVER1   1 /* [0 : séparateur] Dérivée première */
#define FCT_GRF_MOINSMAT_DERIVER2   2 /* Dérivée seconde */
#define FCT_GRF_MOINSMAT_NORMALISER 3 /* Normaliser */
#define FCT_GRF_MOINSMAT_INTEGRER   4 /* Intégrer le préseuil */
#define FCT_GRF_MOINSMAT_EXAFS      6 /* [5 : séparateur] Extraire l'EXAFS */

#define FCT_GRF_NOMBRE_OPT_MOINSMAT 7

/* Les options pour les graphes GR_CINETIQUE (réels) */

#define FCT_GRF_CINETIQUE_BASIQUE        1 /* [0 : séparateur] Calcul non-compartimental : AUC,... */
#define FCT_GRF_CINETIQUE_MODELE_1CBOLUS 2 /* Ajuster un modèle IV bolus monocompartimental */
#define FCT_GRF_CINETIQUE_MODELE_2CBOLUS 3 /* Ajuster un modèle IV bolus bicompartimental   */

#define FCT_GRF_NOMBRE_OPT_CINETIQUE  4

/* -------- Fonctions appliquées aux images -------- */

/* ---- Fonctions génériques ---- */

#define FCT_IMG_SAUVER_IMAGE       0
#define FCT_IMG_EXPORTER_IMAGE     1
#define FCT_IMG_EXPORT_POSTSCRIPT  2 /*  3 : séparateur */
#define FCT_IMG_EFFACER_IMAGE      4
#define FCT_IMG_COPIER_IMAGE       5
#define FCT_IMG_DEPLACER_IMAGE     6
#define FCT_IMG_ISOLER_IMAGE       7 /*  8 : séparateur */
#define FCT_IMG_ERREUR_V_IMAGE     9

#define FCT_IMG_NOMBRE_OPT_GENERIQUES 10 /* Inclue les séparateurs */

/* -------- Fonctions appliquées aux molécules -------- */

/* ---- Fonctions génériques ---- */

#define FCT_MOL_SAUVER_MOLECULE     0
#define FCT_MOL_EXPORTER_MOLECULE   1
#define FCT_MOL_EXPORT_POSTSCRIPT   2 /*  3 : séparateur */
#define FCT_MOL_EFFACER_MOLECULE    4
#define FCT_MOL_COPIER_MOLECULE     5 /* 6 : séparateur */
#define FCT_MOL_CENTRER_MOLECULE    7
#define FCT_MOL_TRANSLATER_MOLECULE 8
#define FCT_MOL_TOURNER_MOLECULE    9  /* 10 : séparateur */
#define FCT_MOL_COMPOSITION        11

#define FCT_MOL_NOMBRE_OPT_GENERIQUES 12 /* Inclue les séparateurs */

/* ~-~-~-~-~-~-~-~-~ Récupérer les noms des fonctions ~-~-~-~-~-~-~-~-~ */

const char **FCT_ListeFonctions(unsigned int nature_objet, unsigned int champ,
				unsigned int genre, unsigned int type, unsigned long *nombre_fonctions);

#define FCT_ListeFonctions_GRF(champ,genre,type,nombre_fonctions) FCT_ListeFonctions(FONCTION_POUR_GRAPHE,champ,genre,type,nombre_fonctions)
#define FCT_ListeFonctions_IMG(champ,genre,type,nombre_fonctions) FCT_ListeFonctions(FONCTION_POUR_IMAGE,champ,genre,type,nombre_fonctions)

#endif	/* LASE_FONCTIONS_H_ */
