/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1998-2003, Emmanuel CURIS

   Export au format PostScript (niveau 1 ou 2)
   Fichier des dclarations locales  ce module
*/

#ifndef EXPORT_PS_H_
# define EXPORT_PS_H_ /* Protection contre l'inclusion multiple */
# include "figures.h"

struct graphe;			/* Dans graphes.h */

/* -------- Dfinition des options de l'export PS -------- */

#define PS_PAGE_TOURNEE 1	/* Bit mis : on tourne la page (mode portrait) */
#define PS_NIVEAU_2     2	/* Bit mis : on travaille avec du PS niveau 2  */

struct infos_ps {
  unsigned int proprietes;	/* Options --- masque de bits, cf. ci-dessus */
  double larg_page, haut_page;	/* Dimensions de la page (en mm) */
  struct {
    double haut, bas;
    double gauche, droite;
  } marges;			/* Marges (en mm) */
  double larg_dessin, haut_dessin; /* Dimensions de la zone o l'on dessine (en mm) */
  double epaisseur;             /* paisseur des lignes traces (mm) */
  double haut_marque;		/* Hauteur de marques  utiliser (mm) */
  double haut_texte;		/* Hauteur de texte par dfaut (pt) */
  const char *nom_fonte;	/* Nom de la fonte  utiliser par dfaut */
};

struct rect_ps {
  double x_BG, y_BG;		/* Position de l'origine sur la feuille : coin bas, gauche du rectangle (1/100 mm) */
  double larg, haut;		/* Dimensions du rectangle (1/100 mm) */

  struct infos_dessin2D cadre;	/* Le cadre proprement dit */
};

int PS_Parametrage(struct infos_ps *infos);

/* Ouverture du fichier */

FILE *PS_OuvrirFichier(const char *nom, const char *titre, struct infos_ps *infos);
int   PS_FermerFichier(FILE *fichier);

/* Options de dessin */

inline int PS_Couleur     (FILE *fichier, unsigned int couleur);
inline int PS_EpaisLigne  (FILE *fichier, double epaisseur);
inline int PS_StyleLigne  (FILE *fichier, unsigned int style);
inline int PS_TailleMarque(FILE *fichier, double haut_marque);
inline int PS_TailleTexte (FILE *fichier, double haut_texte);

inline const char *PS_Marque(unsigned int marque);

/* Primitives de dessin */

int PS_Texte    (FILE *fichier, double x, double y, const char *chaine, int align_hor, int align_ver, double angle, int couleur);
int PS_Rectangle(FILE *fichier, boolean niveau2, double epaisseur_bord, int couleur_bord, int couleur_fond,
		 double x, double y, double largeur, double hauteur);

/* Objets de dessin fondamentaux */

int PS_AxeGradue(FILE *fichier, double x_debut, double y_debut, double x_fin, double y_fin, InfosAxe *axe, double delta,
		 double haut_texte);

int PS_BlocFigure(FILE *fichier, struct infos_ps *infos, struct figure *figure, struct bloc_figure *bloc);
int PS_Cadre (FILE *fichier, struct infos_ps *infos, struct rect_ps *cadre);
int PS_Graphe(FILE *fichier, struct infos_ps *infos, struct rect_ps *cadre, struct graphe *graphe, int erreurs,
	      int couleur, int style_lgn, int style_mk);
int PS_Grille(FILE *fichier, struct rect_ps *cadre, InfosAxe *axe);

#endif /* EXPORT_PS_H_ */
