/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1998 - 1999
   Cration : 13 mars 1998
   
   Toutes les fonctions de traitement statistique de l'E.X.A.F.S.
   FICHIER  INCLURE
*/

#ifndef EXAFS_STAT_H_

#define EXAFS_STAT_H_ /* Protection contre l'inclusion multiple */

#include "graphes.h" /* Pour la structure graphes */

/* ~~~~~~~~ Statistiques descriptives ~~~~~~~~ */

/* -------- Travail sur une srie de valeurs -------- */

/* -- Travail sur la fonction de rpartition -- */

#define STAT_FDR_EMPIRIQUE_BASE 0 /* Mthode vidente : /n */
#define STAT_FDR_EMPIRIQUE_NP1  1 /* Mthode pour droite de Henry : /(n+1) [transformation de Wald ?] */

#define STAT_FDR_EMPIRIQUE_AVEC_ERREURS 1 /* Bit mis : estimer aussi les incertitudes */

struct graphe *STAT_FdREmpirique (struct graphe *valeurs, unsigned int methode, unsigned int options);
struct graphe *STAT_DroiteDeHenry(struct graphe *valeurs, unsigned int methode);

/* -- Paramtres de position -- */

#define STAT_MEDIANE_FDR     0	/* Inversion de la fonction de rpartition exprimentale */
#define STAT_MEDIANE_USUELLE 1	/* Dfinition usuelle de la mdiane */

double STAT_Mediane  (struct graphe *valeurs, unsigned int methode);
int    STAT_Quartiles(struct graphe *valeurs, unsigned int methode, double quartiles[ 3 ]);

/* ------------- dispersion standard des spectres E.X.A.F.S. ------------------- */

void ERR_dispersion(void); /* Paramtrage de la dispersion standard */

#define DISP_X_MOYENNER   0 /* Les diffrents modes de traitement en fonction des X */
#define DISP_X_INTERPOLER 1
#define DISP_X_RECALER    2
#define DISP_X_FUSION     3

#define DISP_X_NOMBRE_METHODES 4

Graphe *MoyennerGraphes(Graphe **graphes, unsigned long nbr_graphes, int type, boolean erreur_x, boolean err_moyenne);

double calc_param_stat(Graphe *graphe);
double esperance(Graphe *graphe);
double variance (Graphe *graphe, double esperance);

double STAT_VarianceCommune(double *variances, int *ddl, unsigned long nbr_groupes, unsigned long *ddl_total);

double covariance(Graphe *graphe1, Graphe *graphe2, double m1, double m2);

Graphe *estime_distribution(Graphe *graphe);
Graphe *estime_densite     (Graphe *graphe);
Graphe *estime_liaison     (Graphe *graphe1, Graphe *graphe2);
Graphe *estime_repartition (Graphe *experience);

/* ~~~~~~~~ Tests statistiques ~~~~~~~~ */

#define STAT_TEST_EXACT      1 /* Bit mis : le test est exact */
#define STAT_TEST_NP         2 /* Bit mis : le test est non-paramtrique */
#define STAT_TEST_UNILATERAL 4 /* Bit mis : le test est unilatral */

/* Dfinition d'un test statistique */
struct test_stats {
  unsigned int proprietes; /* Masque de bits, cf. ci-dessus */

  const char *nom; /* Nom du test */
  const char *H0;  /* Hypothse H0 */
  const char *H1;  /* Hypothse H1 */

  const char *table; /* Radical du nom de la table  utiliser */

  double alpha;    /* Le risque alpha accept [0 <= alpha <= 1] */
  double v_ideale; /* Valeur idale attendue sous H0 */
  double v_esperee;/* Valeur attendue en moyenne sous H0 */
  double variance; /* Variance du test sous H0 */

  unsigned long nombre_valeurs;	/* utilises pour calculer le critre */
  double resultat;		/* Valeur calcule du critre */
  double p_critique; /* Valeur de la probabilit critique pour ce rsultat, si calculable */
};

int condenser_erreur(Graphe *graphe);

#define TEST_HOMO_LOIUNIFORME 0

int STAT_TesterHomogeneite(struct graphe **graphes, unsigned long nbr_graphes, int methode,
			   struct test_stats *resultat, struct graphe **repartition);

typedef struct {
  double x_min, x_max; /* Intervalle pour fusion des moyennes */
  unsigned long somme_ddl; /* Somme des ddl de chaque variance */
  unsigned long nbr_classes;

  struct test_stats test; /* Le test statistique */

  /* Pour le redessin de l'aperu */
  struct graphe *graphe;
  int methode; /* Slection de la mthode de test */
  double echelle;
} Infos_ERRM;

int STAT_TesterVariances(Infos_ERRM *infos);

/* Analyse de variance & apparents */

int STAT_ANVA_1facteur(struct graphe **graphes, unsigned long nbr_facteurs, struct test_stats *resultat);
int STAT_ANVA_droite  (Graphe **graphes, unsigned long nbr_modalites, Graphe *x, struct test_stats *resultat);

/* -------- Routines pour tirer au sort des nombres alatoires -------- */

#define DP_GAUSSIENNE     0
#define DP_UNIFORME       1
#define DP_CAUCHY         2
#define DP_EXPONENTIEL    3
#define DP_EXP_SYMETRIQUE 4
#define DP_VECTEUR_G      5
#define DP_BETA_I         6
#define DP_BETA_II        7
#define DP_FISHER         8
#define DP_GAMMA          9
#define DP_CHIDEUX       10
#define DP_GUMBEL        11
#define DP_STUDENT       12
#define DP_PEARSON       13
#define DP_LOGNORMALE    14
#define DP_WEIBULL       15

#define DP_NOMBRE_LOIS 16
#ifndef VARIABLES_GENERALES_
extern const char *txt_loi_probas_continues[]; /* Dans variables.c : les noms des lois */
#endif

#define DP_PROPRIETE_DISCRETE 1 /* Bit mis si on veut la loi discrte */

struct loi_probabilite {
  unsigned int famille;
  unsigned int proprietes;
  union {
    struct {
      unsigned int ddl_num, ddl_den;
    } Fisher;
    struct {
      double alpha, beta;
    } Gamma;
    struct {
      double mu;		/* Esprance */
      double sigma;		/* cart-type */
    } Gauss;
    struct {
      unsigned int ddl;		/* Degrs de libert */
    } KhiDeux;			/*  utilisable aussi pour un Student ! */
    struct {
      unsigned int ddl;		/* Degrs de libert */
    } Student;			/*  utilisable aussi pour un Student ! */
    struct {
      double min, max;
    } Uniforme;
  } parametres;
};

int init_aleatoire(long racine);
void STAT_TerminerAleatoire(void);

double gaussienne(void); /* Gaussienne centre rduite */
double uniforme  (void); /* Loi uniforme sur [0, 1] */

double STAT_NombreAleatoire(struct loi_probabilite *loi);
double STAT_NombreAleatoire_UniformeC(double min, double max);

/* -------- Routines de gnration d'un processus alatoire (centr, rduit) -------- */

#define PA_BRUIT_BLANC 0 /* Bruit blanc : points indpendants */
#define PA_AR          1 /* Processus autorgressif */

struct processus_aleatoire {
  unsigned int type;
  union {
    struct {
      int ordre; /* Ordre du processus auto-rgressif */
    } AR;
  } parametres;
  struct loi_probabilite loi; /* Loi de probabilit pour un point */
};

Graphe *PA_GenererStationnaire(struct processus_aleatoire *processus, unsigned long nbr_valeurs);
int     PA_GenererSurGraphe   (struct processus_aleatoire *processus, Graphe *graphe);

/* -------- Routines de bruitage -------- */

#define STAT_BRUITER_CONSTANTE       0 /* Erreur constante avec X */
#define STAT_BRUITER_PROPORTIONNELLE 1 /* Erreur proportionnelle  X */
#define STAT_BRUITER_QUELCONQUE      2 /* Erreur en fonction de X indique dans un graphe */

struct infos_bruitage {
  unsigned int methode;
  unsigned int loi_proba;
  double sigma;
  struct graphe *sigma_fX;
};

int STAT_GRF_Bruiter(struct graphe *graphe, struct infos_bruitage *parametres);

/* -------- Routines pour gnrer des densits ou des fonctions de rpartition -------- */

Graphe *modele_probas(void); /* L'interface */

Graphe *DP_beta_I       (double delta, double n, double p, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_beta_II      (double delta, double n, double p, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_cauchy       (double mode, double a, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_double_exp   (double delta, double lambda, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_exponentielle(double delta, double lambda, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_fisher       (double mode, double n, double p, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_gamma        (double delta, double alpha, double beta, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_gaussienne   (double mu, double sigma, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_gumbel       (double delta, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_lognormale   (double mu, double sigma, double gamma, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_pearson      (double mode, double a, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_student      (double mode, double n, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_uniforme     (double min, double  max, double x_min, double x_max, unsigned long nbr_pts);
Graphe *DP_weibull      (double alpha, double beta, double gamma, double x_min, double x_max, unsigned long nbr_pts);

/* -------- Travail sur les lois discrtes -------- */

unsigned int STAT_TirerValeurDiscrete(unsigned int nbr_valeurs, double *probabilites);

/* -------- Diffrents tests statistiques sur les distributions ------- */

void cramer_vonmises   (Graphe *experience);
void durbin_watson     (Graphe *experience);
void kolmogorov_smirnov(Graphe *experience);

/* ~-~-~-~-~-~-~-~-~ Mthodes de comparaisons (multiples) ~-~-~-~-~-~-~-~-~ */

struct modalite;		/* Dfinie ci-dessous */

struct comparaison {
  unsigned int methode;		/* Mthode utilise pour obtenir le fractile [cf ci-dessus] */
  unsigned int drapeaux;	/* Masque de bits, cf. ci-dessous */
  int sens;			/* Sens de la comparaison : >0 si l'on teste si [0]-[1] > 0 ; <0  si l'on teste si [0]-[1] < 0 ; 0 si l'on teste si [0]-[1] != 0 */
  struct modalite *modalites[ 2 ];  /* Les deux modalits impliques. La comparaison sera sur la diffrence [0] - [1] */
  double difference;		/* La diffrence observe */
  double variance_difference;	/* Variance de la diffrence */
  double fractile;		/* Fractile  utiliser pour l'intervalle de confiance */
  double ic_bas, ic_haut;	/* Bornes de l'intervalle de confiance [quand il existe ; si uni, l'une des deux vaut l'infini : inutilis alors */
};

#define STAT_IC_FRACTILE_EXACT     0  /* On utilise la formule exacte pour obtenir le fractile, mme si elle est complique */
#define STAT_IC_FRACTILE_BONFERONI 1  /* On utilise l'ingalit de Bonferoni pour estimer le fractile [si possible] */
#define STAT_IC_FRACTILE_SIDAK     2  /* On utilise l'ingalit de Sidak pour estimer le fractile [si possible] */
#define STAT_IC_FRACTILE_SLEPIAN   3  /* On utilise l'ingalit de Slepian pour estimer le fractile [si possible] */
#define STAT_IC_FRACTILE_HW        4  /* On utilise l'ingalit de Hunter-Worsley pour estimer le fractile */
#define STAT_IC_FRACTILE_MILLERW   5  /* Mthode de Miller-Winer [pour d de Dunnett] : DCONSEILLE */
#define STAT_IC_FRACTILE_TABULE    6  /* On utilise une table [ lire dans $CHEMIN/Donnees] */
#define STAT_IC_FRACTILE_DEMANDER  7  /* Demander la valeur  l'utilisateur [test spcial !] */

#define STAT_IC_NOMBRE_METHODES_APPROXIMATION 8

#define STAT_CMP_UNILATERALE   1 /* Bit mis : la comparaison est de type unilatral */
#define STAT_CMP_AVEC_IC       2 /* Bit mis : on peut construire un IC sur la diffrence */
#define STAT_CMP_SIGNIFICATIVE 4 /* Bit mis : la comparaison est significative au seuil GLOBAL choisi */

struct comparaison *STAT_CreerComparaisons         (unsigned int nombre_comparaisons);
struct comparaison *STAT_Init_ComparaisonsAUnTemoin(unsigned int nombre_modalites /* Total */, struct modalite *modalites, unsigned int modalite_temoin, int sens);
void                STAT_EffacerComparaisons       (unsigned int nombre_comparaisons, struct comparaison *comparaisons);

double STAT_Fractile_Dunnett(double alpha_global, unsigned int nombre_comparaisons, struct comparaison *comparaisons, unsigned long ddl, int sens);

int STAT_ComparerAUnTemoin_Dunnett(unsigned int nombre_comparaisons, struct comparaison *comparaisons, int sens, double alpha_global, unsigned int approximation,
				   double variance_commune, unsigned long ddl_SR, boolean est_equilibree);

int STAT_CMP_CalculerIC(struct comparaison *comparaison);

struct graphe *STAT_GRF_ComparerGraphes(unsigned long nombre_graphes, struct graphe **graphes, struct graphe *reference, struct graphe **difference);

/* ~-~-~-~-~-~-~-~-~ Mthodes d'analyse de variance ~-~-~-~-~-~-~-~-~ */

struct modalite {
  char *nom;			/* Nom de la modalit [purement informatif] */
  double valeur;		/* Dans le cas d'un facteur quantitatif */
  double effet;			/* Effet estim de cette modalit sur la moyenne globale */

  double moyenne;		/* Moyenne dans la modalit */
  double SCR;			/* Somme des carrs rsiduelle pour cette seule modalit [/ddl -> variance dans cette modalit] */

  struct graphe *valeurs;	/* Le graphe contenant la srie des valeurs pour cette "modalit" */
};

struct anva_facteur {
  char *nom;			/* Nom du facteur [purement informatif] */
  unsigned int proprietes;	/* Masque de bits, cf. ci-dessous */
  unsigned int nbr_modalites;	/* Nombre de modalits de ce facteur */
  struct modalite *modalites;
};

#define ANVA_FACTEUR_ALEATOIRE   (1) /* Bit mis : le facteur est  effets alatoires ; sinon : il est  effets fixes */
#define ANVA_FACTEUR_QUANTITATIF (2) /* Bit mis : le facteur est quantitatif --- on peut faire une rgression linaire dessus ! */

int ANVA_InitialiserFacteur(struct anva_facteur *facteur, unsigned int nombre_modalites);

int ANVA_FACTEUR_AjouterModalite(struct anva_facteur *facteur, unsigned int numero_modalite_avant);
int ANVA_FACTEUR_OterModalite   (struct anva_facteur *facteur, unsigned int numero_modalite);
int ANVA_FACTEUR_Renommer       (struct anva_facteur *facteur, const char *nouveau_nom);

int ANVA_MODALITE_CalculsInitiaux(struct modalite *modalite);
int ANVA_MODALITE_Renommer(struct modalite *modalite, char *nouveau_nom);

/* -------- Cas particulier de l'analyse de variance  1 facteur -------- */

struct modele_ajuste;		/* Dfinie dans Include/modeles.h */
struct modele_droite {
  double covariance;		/* Covariance Y/X */
  double SCE_X;			/* Somme des carrs des carts pour X */
  double moyenne_X;		/* Moyenne des X */
  double pente, o_a_lo;		/* Pente et ordonne  l'origine */
};

struct infos_ANVA_1F {
  struct anva_facteur facteur;	/* Informations sur le facteur */
  struct graphe *moyennes;	/* Moyennes par modalit --- pour affichage ! */
  struct graphe *variances;	/* Variances par modalit --- pour affichage ! */
  double moyenne;		/* Moyenne globale --- pour affichage ! */

  int modalite_controle;  /* Le numro de la modalit qui sert de contrle [<0 : pas de contrle] */

  double QT, QR, QF;		/* Les sommes de carrs de la dcomposition */
  unsigned long ddl_T, ddl_R, ddl_F; /* Les degrs de libert de la dcomposition */
  double ST, SR, SF;		/* Les variances */
  double rapport;
  double proba_critique;

  unsigned int type_modele;	/* Type de modle Y=f(X) pour une quantitative ; cf ci-dessous */
  double QE, QM;		/* Somme des carrs dcomposant le factoriel : E pour cart au modle, M pour modle */
  unsigned long ddl_E, ddl_M;	/* Degrs de libert, mme chose */
  double SE, SM;		/* Variances associes */
  double RE, RM, pc_E, pc_M;

  union {
    struct modele_droite *rl;	/* Droite de rgression linaire */
    struct modele_ajuste *nl;	/* Pour un modle non-linaire ou CL... */
  } modele;
  struct graphe *resultat;

  unsigned int nombre_comparaisons;  /* Nombre de comparaisons de moyennes faites */
  struct comparaison *comparaisons;  /* Ces comparaisons */
};

#define ANVA_MODELE_DROITE   0	/* Rgression linaire simple : y = a + b * X */
#define ANVA_MODELE_POLYNOME 1	/* Rgression polynomiale, de degr quelconque : y = a + b * X + c * X + ...*/
#define ANVA_MODELE_CL       2	/* Rgression linaire gnrique : y = a * f1(X) + b * f2(X) + ... */
#define ANVA_MODELE_NL       3	/* Rgression non-linaire quelconque : y = f(X;a,b,c,...) */

int ANVA_1F_Calculs       (struct infos_ANVA_1F *infos_ANVA);
int ANVA_1F_CalculerModele(struct infos_ANVA_1F *infos_ANVA);
int ANVA_1F_LibererModele (struct infos_ANVA_1F *infos_ANVA);
int ANVA_1F_PreparerModele(struct infos_ANVA_1F *infos_ANVA, unsigned int type);

boolean ANVA_1F_EstEquilibree(struct infos_ANVA_1F *infos_ANVA);

#define ANVA_CMP_TEMOIN_DUNETT 0  /* Comparaison au tmoin : mthode de Dunnett */
#define ANVA_CMP_TEMOIN_NMPG   1  /* Comparaison au tmoin : mthode de Naik, Marcus, Peritz & Gabriel */
#define ANVA_CMP_TEMOIN_DT     2  /* Comparaison au tmoin : mthode de Dunnett & Tamhane */
#define ANVA_CMP_TEMOIN_STEEL  3  /* Comparaison au tmoin : mthode non-paramtrique de Steel */
#define ANVA_CMP_TEMOIN_BOFINGER 4
#define ANVA_CMP_TEMOIN_SKH    5

#define ANVA_NOMBRE_METHODES_TEMOIN 6

int ANVA_1F_ComparerAuTemoin(struct infos_ANVA_1F *infos_ANVA, unsigned int methode, unsigned int methode_calcul, int sens, double alpha_global);

#endif /* EXAFS_STAT_H_ */
