/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion de la bote de modification des lgendes d'un axe
*/

#include <stdlib.h>
#include <stdio.h>

#include "systeme.h"
#include "exafs.h"

#include "interface.h"
#include "tracer.h"

#include "bornes.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LGMAX_TXT_BORNES  26

struct infos_bornes {
  double x_min, x_max;
  double y_min, y_max;
  double z_min, z_max;
  void *infos;			/* Infos complmentaires pour le calcul des bornes */
  int (*automatique)(unsigned int mode, double bornes[ 6 ], void *infos);
};

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void BORNES_GererSousMenu(unsigned int arbre, unsigned int objet, unsigned long mode);
static void BORNES_PreparerArbre(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Fonction visible de l'extrieur : paramtrage complet
   Renvoie 0 si tout s'est bien pass, != 0 sinon

   axe_X, axe_Y, axe_Z : les axes  modifier (NULL : non considr)
   automatique         : fonction  appeler pour le calcul automatique des bornes
      mode   = le mode de calcul (X, Y, X & Y, X/Y, Y/X)
      bornes = la table des bornes (x,X, y,Y, z,Z)
      infos  = informations complmentaires, ventuellement ncessaires
   infos               : les infos pour l'appel  automatique
   mere                : la fentre concerne... [pour interface seulement]
   -------- */

int I_ParametrerBornes(struct infos_axe *axe_X, struct infos_axe *axe_Y, struct infos_axe *axe_Z,
		       int (*automatique)(unsigned int mode, double bornes[ 6 ], void *infos), void *infos_complementaires, Fenetre *mere)
{
  struct infos_bornes infos = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, ( int (*)(unsigned int, double [6], void *) ) NULL, NULL };
  int retour;

  /* Prparer l'arbre [avant pour convertir les ditables en G_NOMBRE] */
  BORNES_PreparerArbre();

  /* Bornes en X */
  if ( NULL == axe_X ) {
    OBJ_active( CHG_BORNES, BORNE_XMIN, FORCE_NON );
    OBJ_active( CHG_BORNES, BORNE_XMAX, FORCE_NON );
  } else {
    OBJ_active( CHG_BORNES, BORNE_XMIN, FORCE_OUI );
    OBJ_active( CHG_BORNES, BORNE_XMAX, FORCE_OUI );

    OBJ_OptionsEditable( CHG_BORNES, BORNE_XMIN, BI_EDITABLE_UNITE, 0.0, 0.0, axe_X->unite );
    OBJ_OptionsEditable( CHG_BORNES, BORNE_XMAX, BI_EDITABLE_UNITE, 0.0, 0.0, axe_X->unite );

    infos.x_min = axe_X->min;
    infos.x_max = axe_X->max;
  }

  /* Bornes en Y */
  if ( NULL == axe_Y ) {
    OBJ_active( CHG_BORNES, BORNE_YMIN, FORCE_NON );
    OBJ_active( CHG_BORNES, BORNE_YMAX, FORCE_NON );
  } else {
    OBJ_active( CHG_BORNES, BORNE_YMIN, FORCE_OUI );
    OBJ_active( CHG_BORNES, BORNE_YMAX, FORCE_OUI );

    OBJ_OptionsEditable( CHG_BORNES, BORNE_YMIN, BI_EDITABLE_UNITE, 0.0, 0.0, axe_Y->unite );
    OBJ_OptionsEditable( CHG_BORNES, BORNE_YMAX, BI_EDITABLE_UNITE, 0.0, 0.0, axe_Y->unite );

    infos.y_min = axe_Y->min;
    infos.y_max = axe_Y->max;
  }

  /* Bornes en Z */
  if ( NULL == axe_Z ) {
    OBJ_active( CHG_BORNES, CB_ZMIN, FORCE_NON );
    OBJ_active( CHG_BORNES, CB_ZMAX, FORCE_NON );
  } else {
    OBJ_active( CHG_BORNES, CB_ZMIN, FORCE_OUI );
    OBJ_active( CHG_BORNES, CB_ZMAX, FORCE_OUI );

    OBJ_OptionsEditable( CHG_BORNES, CB_ZMIN, BI_EDITABLE_UNITE, 0.0, 0.0, axe_Z->unite );
    OBJ_OptionsEditable( CHG_BORNES, CB_ZMAX, BI_EDITABLE_UNITE, 0.0, 0.0, axe_Z->unite );

    infos.z_min = axe_Z->min;
    infos.z_max = axe_Z->max;
  }

  /* Fonction automatique, si prsente */
  infos.automatique = automatique;
  if ( NULL == automatique ) {
    OBJ_active( CHG_BORNES, CB_OPTIMISER, FORCE_NON );
  } else {
    infos.infos = infos_complementaires;
    OBJ_active( CHG_BORNES, CB_OPTIMISER, FORCE_OUI );
  }

  /* Mise en place des valeurs */
  OBJ_MettreIntervalle( CHG_BORNES, BORNE_XMIN, BORNE_XMAX, 0, 0, infos.x_min, infos.x_max, 0.0, 0 );
  OBJ_MettreIntervalle( CHG_BORNES, BORNE_YMIN, BORNE_YMAX, 0, 0, infos.y_min, infos.y_max, 0.0, 0 );
  OBJ_MettreIntervalle( CHG_BORNES, CB_ZMIN, CB_ZMAX, 0, 0, infos.z_min, infos.z_max, 0.0, 0 );

  /* Appel de l'arbre */
  ARB_MettreComplement( CHG_BORNES, &infos );
  retour = ARB_Bloquant( CHG_BORNES, titre_arbre_bornes, mere, CHG_BORNES );
  ARB_MettreComplement( CHG_BORNES, NULL );

  if ( retour == BORNE_ANNULE ) { /* On annule tout */
    return +1;
  }

  /* Lecture des nouvelles bornes */
  infos.x_min = OBJ_LireNombre( CHG_BORNES, BORNE_XMIN, NULL );
  infos.x_max = OBJ_LireNombre( CHG_BORNES, BORNE_XMAX, NULL );
  if ( infos.x_min > infos.x_max ) { /* On vrifie le bon ordre... */
    double inter;

    inter = infos.x_min;
    infos.x_min = infos.x_max;
    infos.x_max = inter;
  }

  infos.y_min = OBJ_LireNombre( CHG_BORNES, BORNE_YMIN, NULL );
  infos.y_max = OBJ_LireNombre( CHG_BORNES, BORNE_YMAX, NULL );
  if ( infos.y_min > infos.y_max ) { /* On vrifie le bon ordre... */
    double inter;

    inter = infos.y_min;
    infos.y_min = infos.y_max;
    infos.y_max = inter;
  }

  infos.z_min = OBJ_LireNombre( CHG_BORNES, CB_ZMIN, NULL );
  infos.z_max = OBJ_LireNombre( CHG_BORNES, CB_ZMAX, NULL );
  if ( infos.z_min > infos.z_max ) { /* On vrifie le bon ordre... */
    double inter;

    inter = infos.z_min;
    infos.z_min = infos.z_max;
    infos.z_max = inter;
  }

  /* Mmorisation pour les axes concerns */
  if ( axe_X != NULL ) {
    axe_X->min = infos.x_min;
    axe_X->max = infos.x_max;
  }
  if ( axe_Y != NULL ) {
    axe_Y->min = infos.y_min;
    axe_Y->max = infos.y_max;
  }
  if ( axe_Z != NULL ) {
    axe_Z->min = infos.z_min;
    axe_Z->max = infos.z_max;
  }

  /* On a bien chang les bornes */
  return 0;
}

/* -------- Gestion des options d'optimisation des bornes -------- */

static void BORNES_GererSousMenu(unsigned int arbre, unsigned int objet __attribute__((__unused__)), unsigned long mode)
{
  double bornes[ 6 ];
  struct infos_bornes *infos;

  infos = (struct infos_bornes *) ARB_LireComplement( arbre );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas d'infos associes !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* Lecture des bornes actuelles */
  bornes[ 0 ] = OBJ_LireNombre( CHG_BORNES, BORNE_XMIN, NULL );
  bornes[ 1 ] = OBJ_LireNombre( CHG_BORNES, BORNE_XMAX, NULL );

  bornes[ 2 ] = OBJ_LireNombre( CHG_BORNES, BORNE_YMIN, NULL );
  bornes[ 3 ] = OBJ_LireNombre( CHG_BORNES, BORNE_YMAX, NULL );

  bornes[ 4 ] = OBJ_LireNombre( CHG_BORNES, CB_ZMIN, NULL );
  bornes[ 5 ] = OBJ_LireNombre( CHG_BORNES, CB_ZMAX, NULL );

  /* Appel de la fonction d'optimisation */
  infos->automatique( mode, bornes, infos->infos );

  /* Affichage des valeurs qui ont pu changer */
  OBJ_MettreIntervalle( CHG_BORNES, BORNE_XMIN, BORNE_XMAX, 0, 0, bornes[ 0 ], bornes[ 1 ], 0.0, 0 );
  OBJ_MettreIntervalle( CHG_BORNES, BORNE_YMIN, BORNE_YMAX, 0, 0, bornes[ 2 ], bornes[ 3 ], 0.0, 0 );
  OBJ_MettreIntervalle( CHG_BORNES, CB_ZMIN, CB_ZMAX, 0, 0, bornes[ 4 ], bornes[ 5 ], 0.0, 0 );
}

/* ------ Prparation de l'arbre ------ */

static void BORNES_PreparerArbre(void)
{
  if ( VRAI == arbres[ CHG_BORNES ].drapeaux.prepare ) return;
  if ( FAUX == arbres[ CHG_BORNES ].drapeaux.init ) {
    int retour;

    /* Le sous-menu qui permet de calculer des bornes est gr automatiquement */
    OBJ_ChangerType( CHG_BORNES, CB_OPTIMISER, G_SSMENU );
    OBJ_MettreMenu ( CHG_BORNES, CB_OPTIMISER, NOMBRE_MODES_AUTOMATIQUES, txt_bornes, titre_modes_optimisation, BORNES_GererSousMenu );

    retour = ARB_Initialiser( CHG_BORNES, NULL, ( int (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );
    if ( retour != 0 ) return;
  }

  OBJ_deplace( CHG_BORNES, BORNE_XMAX, 0, 1, DEPLACE_PIXELS );
  OBJ_deplace( CHG_BORNES, BORNE_YMAX, 0, 1, DEPLACE_PIXELS );
/*   OBJ_deplace( CHG_BORNES, BORNE_ZMAX, 0, 1, DEPLACE_PIXELS ); */

  /* Le titre du sous-menu pour optimiser les bornes */
  OBJ_RemplacerTexte( CHG_BORNES, CB_OPTIMISER, txt_optimiser_bornes, FAUX );

#ifdef AVEC_TRADUCTION_
  OPTION_A_FAIRE( "Traduire l'arbre" !);
#endif

  arbres[CHG_BORNES].drapeaux.prepare = VRAI;
}
