/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2009, Emmanuel CURIS
   Cration : 6 octobre 2009

   Interface graphique : fonctions de type pharmacocintique
    l'origine : calculs pour Fatima-Zara el-Wafi
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "serie.h"

#include "interface.h"
#include "operations.h"
#include "stats_utils.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#include "int_pharmaco.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Calculs des paramtres basiques de pharmacocintique -------- */

int I_PHARMACO_ParametresBasiques(struct graphe *profil)
{
  double auc, c_max, t_max, auc_corrigee;
  double valeur_basale, correction_auc;
  double erreur_AUC = 0.0, *x, *y;
  unsigned long i, n_basales;
  char *unite_AUC = NULL;

  if ( NULL == profil ) return 0;

  /* tape 1 : calcul de l'AUC globale, du Cmax et du tmax */
  auc = GRF_Integrer_R( profil, 0.0, profil->x_max, INTG_TRAPEZE, &erreur_AUC );
  STAT_ChercherExtrema_D( profil->y, profil->nombre_point, NULL, &c_max );
  
  for ( i = 0, y = profil->y ; *y != c_max ; i++, y++ );
  t_max = profil->x[ i ];

  /* tape 2 : correction de l'AUC & du Cmax en tant la ligne de base */
  /* a. Dtermination de la ligne de base : moyenne des valeurs pour t ngatif */
  valeur_basale = 0.0;
  for ( i = 0, n_basales = 0, x = profil->x, y = profil->y ;
	i < profil->nombre_point ;
	i++, x++, y++ ) {
    if ( *x <= 0.0 ) {
      valeur_basale += *y;
      n_basales++;
    }
  }
  if ( n_basales > 0 ) {
    valeur_basale /= (double) n_basales;
  }

  /* b. Correction */
  correction_auc = valeur_basale * profil->x_max;
  auc_corrigee = auc - correction_auc;

  /* tape 3 : on affiche */
  unite_AUC = TXT_genere_chaine( "%s%s", GRF_QuelleUnite( profil, AXE_Y ), GRF_QuelleUnite( profil, AXE_X ) );
  (void) BI_ERR_alerte( NULL, &MSG_ParametresPharmacocinetiques, FAUX,
			auc, unite_AUC,
			c_max, GRF_QuelleUnite( profil, AXE_Y ),
			t_max, GRF_QuelleUnite( profil, AXE_X ),
			valeur_basale, GRF_QuelleUnite( profil, AXE_Y ),
			auc_corrigee, unite_AUC,
			c_max - valeur_basale, GRF_QuelleUnite( profil, AXE_Y ) );

  /* Termin */
  free( unite_AUC ); unite_AUC = NULL;
  return 0;
}
