/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2006, Emmanuel CURIS
   Cration : mai 2006

   Interface graphique avec l'utilisateur

   Gnralits pour les stats [macros de gestion de boutons & appel de fonctions internes]
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "interface.h"
#include "serie.h"
#include "statistiques.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

struct infos_moyenner_GRF {
  unsigned long premier_graphe_affiche;
  unsigned long numero_graphe_choisi;
  unsigned long nombre_graphes_choisis;
  Graphe **graphes_choisis;	/* Table dynamique */
};

#define LGMAX_TXT_NOM_GRAPHE 42

#define NOMBRE_GRAPHES_VISIBLES (BERR_GRF_DER_NOM - BERR_GRF_PREMNOM + 1)

#include "int_stat_moyenne.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int  I_GRF_GererArbreMoyenner   (unsigned int num_objet, int x, int y);
static void I_GRF_PreparerArbreMoyenner(void);

static void        I_STAT_MOY_ActiverObjets  (unsigned long nombre_choisis);
static int         I_STAT_MOY_AjouterGraphes (struct infos_moyenner_GRF *infos, unsigned long n_a_ajouter, Graphe **graphes);
static boolean     I_STAT_MOY_AjouterUnGraphe(struct infos_moyenner_GRF *infos, Graphe *graphe);
static int         I_STAT_MOY_EnleverGraphe  (struct infos_moyenner_GRF *infos, unsigned long numero_graphe_a_enlever);
static const char *I_STAT_MOY_GrapheVersTexte(unsigned int arbre, unsigned long numero_objet);

static void I_STAT_MOY_NommerGraphe(Graphe *resultat, const char *masque, unsigned long nombre_moyennes);

/* ~~~~~~~~ Ralisation pratique ~~~~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Moyenne de plusieurs graphes ~-~-~-~-~-~-~-~-~ */

/* -------- Bote de dialogue permettant de paramtrer tout a [Appel depuis l'extrieur] -------- */

struct graphe *I_GRF_Moyenner(void)
{
  struct infos_moyenner_GRF infos = {0, 0, 0, NULL};
  int reponse = 0, methode = 0;
  unsigned int homogeneite, homo_a_controler = 0;
  Graphe **selection;
  Graphe *moyen;
  Graphe modele;	  /* Ce qui est commun entre les graphes... */
  boolean valide = FAUX;

  if ( ( NULL == serie_courante ) || ( NULL == serie_courante->premier_graphe ) ) {
    /* Aucun graphe : inutile d'en faire des moyennes ! */
    return NULL;
  }

  /* On prpare l'arbre, si ncessaire */
  I_GRF_PreparerArbreMoyenner();

  /* On initialise les objets */
  I_STAT_MOY_ActiverObjets( 0 );

  /* On prpare le slecteur */
  do {
    /* On associe les infos  l'arbre
       (dans la boucle car sinon, pb qd on rappelle  pas d'information associe  l'arbre ) */
    ARB_MettreComplement( BARRE_ERREUR, &infos );

    ARB_PreparerDefileur_O( BARRE_ERREUR, BERR_GRF_PREMNOM, BERR_GRF_DER_NOM, LGMAX_TXT_NOM_GRAPHE,
			    BERR_GRF_AVANT, BERR_GRF_ASC_FND, BERR_GRF_ASC_BOU, BERR_GRF_APRES,
			    infos.nombre_graphes_choisis, NULL, infos.numero_graphe_choisi, infos.premier_graphe_affiche,
			    I_STAT_MOY_GrapheVersTexte );

    /* On appelle l'arbre */
    reponse = ARB_Bloquant( BARRE_ERREUR, txt_nom_dialogue, principale, 0 );
    ARB_MettreComplement( BARRE_ERREUR, NULL );
    if ( BERR_ANNULER == reponse ) {
      free( infos.graphes_choisis ); infos.graphes_choisis = NULL;
      return NULL;
    }

    /* Table des graphes slectionns */
    selection = infos.graphes_choisis;
    if ( NULL == selection ) {
      (void) fprintf(stderr, "%s,%s l.%u : Aucun graphe n'a t choisi !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }

    /* On est l : on veut le calcul effectif */
    valide = VRAI;

    /* Comment doit-on calculer les erreurs ? */
    methode = OBJ_OptionMenu( BARRE_ERREUR, BERR_METHODE_X, 0, QUESTION );

    /* On regarde ce qui est en commun d'un graphe  l'autre */
    modele.affichage = NULL;	/* Sera mis par le test d'homognit, si opportun */
    homo_a_controler = GRF_CONTROLE_TYPE | GRF_CONTROLE_GENRE | GRF_CONTROLE_LEGENDE_X | GRF_CONTROLE_UNITE_X | GRF_CONTROLE_LEGENDE_Y | GRF_CONTROLE_UNITE_Y ;
    if ( DISP_X_MOYENNER == methode ) {
      /* Dans ce cas, il faut le mme nombre de points obligatoirement ! */
      homo_a_controler |= GRF_CONTROLE_NBR_PTS;
    }
    homogeneite = GRF_ControlerCompatibilite( selection, infos.nombre_graphes_choisis, homo_a_controler, &modele );
    if ( homogeneite != homo_a_controler ) {
      /* Problme le plus grave : si pas le mme nombre de points */
      if ( ( DISP_X_MOYENNER == methode ) && ( ( homogeneite & GRF_CONTROLE_NBR_PTS ) == 0 ) ) {
	reponse = BI_ERR_alerte( principale, &ERR_NombrePointsDifferent, VRAI );
	if ( 3 == reponse ) {
	  /* Calcul annul */
	  free( selection ); selection = NULL;
	  infos.graphes_choisis = NULL;
	  return NULL;
	}
	if ( 2 == reponse ) {
	  /* Vrification demande */
	  valide = FAUX;
	}
      }

      /* Autre problme trs embtant : les graphes n'ont pas le mme genre */
      if ( ( ( homogeneite & GRF_CONTROLE_GENRE ) == 0 ) && ( VRAI == valide ) ) {
	reponse = BI_ERR_alerte( principale, &ERR_GenreDifferent, VRAI );
	if ( 3 == reponse ) {
	  /* Calcul annul */
	  free( selection ); selection = NULL;
	  infos.graphes_choisis = NULL;
	  return NULL;
	}
	if ( 2 == reponse ) {
	  /* Vrification demande */
	  valide = FAUX;
	}
      }

      /* Problme un peu moins gnant : les graphes n'ont pas le mme type */
      if ( ( ( homogeneite & GRF_CONTROLE_GENRE ) == 0 ) && ( VRAI == valide ) ) {
	reponse = BI_ERR_alerte( principale, &ERR_TypeDifferent, VRAI );
	if ( 3 == reponse ) {
	  /* Calcul annul */
	  free( selection ); selection = NULL;
	  infos.graphes_choisis = NULL;
	  return NULL;
	}
	if ( 2 == reponse ) {
	  /* Vrification demande */
	  valide = FAUX;
	}
      }

      /* Problmes secondaires : units (plus pour viter des surprises !) */
      if ( ( ( homogeneite & GRF_CONTROLE_UNITE_X ) == 0 ) && ( VRAI == valide ) ) {
	/* On ne signale le problme que si aucun problme n'a dj t dtect... */
	reponse = BI_ERR_alerte( principale, &ERR_UniteXDifferente, VRAI );
	if ( 3 == reponse ) {
	  /* Calcul annul */
	  free( selection ); selection = NULL;
	  infos.graphes_choisis = NULL;
	  return NULL;
	}
	if ( 2 == reponse ) {
	  /* Vrification demande */
	  valide = FAUX;
	}
      }

      if ( ( ( homogeneite & GRF_CONTROLE_UNITE_Y ) == 0 ) && ( VRAI == valide ) ) {
	/* On ne signale le problme que si aucun problme n'a dj t dtect... */
	reponse = BI_ERR_alerte( principale, &ERR_UniteYDifferente, VRAI );
	if ( 3 == reponse ) {
	  /* Calcul annul */
	  free( selection ); selection = NULL;
	  infos.graphes_choisis = NULL;
	  return NULL;
	}
	if ( 2 == reponse ) {
	  /* Vrification demande */
	  valide = FAUX;
	}
      }

      /* Problme mineur : faut-il le signaler ? */
      if ( ( ( homogeneite & GRF_CONTROLE_LEGENDE_X ) == 0 ) && ( VRAI == valide ) ) {
	OPTION_A_FAIRE( "Lgendes diffrentes en X : faut-il avertir ?" );
      }

      if ( ( ( homogeneite & GRF_CONTROLE_LEGENDE_Y ) == 0 ) && ( VRAI == valide ) ) {
	OPTION_A_FAIRE( "Lgendes diffrentes en Y : faut-il avertir ?" );
      }
    }

    /* Veut-on contrler l'homognit ? */
    if ( OBJ_choisit( BARRE_ERREUR, BE_TESTS, QUESTION ) ) {
      struct test_stats test;
      Graphe *repartition = NULL;
      int retour;

      /* On fait le contrle */
      retour = STAT_TesterHomogeneite( selection, infos.nombre_graphes_choisis, TEST_HOMO_LOIUNIFORME, &test, &repartition );
      if ( retour != 0 ) {
	reponse = BI_ERR_alerte( principale, &ERR_ControleHomogeneite, VRAI );
	if ( 2 == reponse ) {
	  /* Calcul annul */
	  free( selection ); selection = NULL;
	  infos.graphes_choisis = NULL;
	  return NULL;
	}
      }

      /* On affiche le rsultat du contrle */
      test.nom = txt_nom_test_homogeneite;
      test.H0  = txt_H0_test_homogeneite;
      test.H1  = txt_H1_test_homogeneite;
      test.table = NULL;
      reponse = I_STAT_TEST_AfficherResultat( &test );

      /* On mmorise le graphe des maxima */
      GRF_titre( repartition, txt_nom_graphe_repartition );
      I_GRF_InsererDansSerie( NULL, NULL, repartition );

      /* Plus facile  voir avec des marques que des lignes ! Pas noir, pour mieux distinguer sur les bords */
      GRF_couleur( repartition, ROUGE, 0, 2 );

      if ( STATS_ACCEPTER_H1 == reponse ) {
	/* On annule la moyenne */
	free( selection ); selection = NULL;
	infos.graphes_choisis = NULL;
	return NULL;
      }
    }
  } while ( FAUX == valide );

  /* On calcule le graphe moyen avec mmorisation des barres d'erreur */
  moyen = MoyennerGraphes( selection, infos.nombre_graphes_choisis, methode,
			   OBJ_choisit( BARRE_ERREUR, BERR_GARDER_X, QUESTION ), OBJ_choisit( BARRE_ERREUR, BE_SIGMA_MOYEN, QUESTION ) );

  /* Plus besoin de la table des graphes choisis */
  free( selection ); selection = NULL;
  infos.graphes_choisis = NULL;

  /* ventuellement, on fait une correction */
  if ( OBJ_choisit( BARRE_ERREUR, BERR_MUL_SIGMA, QUESTION ) == VRAI ) {
    double correction;

    correction = OBJ_LireNombre( BARRE_ERREUR, BERR_MUL_SIGMA_V, NULL );
    if ( correction > 0.0 ) {
      GRF_FacteurMultiplicatif( moyen->erreur_y, correction, AXE_Y, NULL );
    }
  }

  /* On nomme le graphe moyen */
  I_STAT_MOY_NommerGraphe( moyen, lit_chaine( BARRE_ERREUR, BE_NOM_GRAPHE, 0, NULL, TXT_REMPLACE ), infos.nombre_graphes_choisis );

  /* On insre le rsultat, si possible en fin de la srie courante */
  I_GRF_InsererDansSerie( NULL, NULL, moyen );

  /* On installe, si faisable, units et lgendes */
  if ( homogeneite & GRF_CONTROLE_LEGENDE_X ) {
    GRF_ChangerLegende( moyen, AXE_X, (char *) GRF_QuelleLegende( &modele, AXE_X ), VRAI );
  }
  if ( homogeneite & GRF_CONTROLE_UNITE_X ) {
    GRF_ChangerUnite( moyen, AXE_X, GRF_QuelleUnite( &modele, AXE_X ) );
  }
  if ( homogeneite & GRF_CONTROLE_LEGENDE_Y ) {
    GRF_ChangerLegende( moyen, AXE_Y, (char *) GRF_QuelleLegende( &modele, AXE_Y ), VRAI );
  }
  if ( homogeneite & GRF_CONTROLE_UNITE_Y ) {
    GRF_ChangerUnite( moyen, AXE_Y, GRF_QuelleUnite( &modele, AXE_Y ) );
  }

  /* On renvoie le rsultat */
  return moyen;
}

/* -------- Affichage de la liste des graphes slectionns -------- */

static const char *I_STAT_MOY_GrapheVersTexte(unsigned int arbre, unsigned long numero_objet)
{
  struct infos_moyenner_GRF *infos;
  Graphe *graphe;

  infos = ARB_LireComplement( arbre );
  if ( NULL == infos ) return NULL;

  if ( numero_objet >= infos->nombre_graphes_choisis ) {
    AVERTISSEMENT( "Appel pour un objet hors-liste [%lu]", numero_objet );
    return NULL;
  }

  graphe = infos->graphes_choisis[ numero_objet ];
  if ( NULL == graphe ) {
    AVERTISSEMENT( "Appel pour un objet correspondant  un graphe NULL [%lu]", numero_objet );
    return NULL;
  }

  return graphe->nom;
}

/* -------- Gestion de l'arbre -------- */

static int I_GRF_GererArbreMoyenner(unsigned int num_objet, int x __attribute__((__unused__)), int y)
{
  boolean etat;
  struct infos_moyenner_GRF *infos;
  int n_doublons;

  infos = (struct infos_moyenner_GRF *) ARB_LireComplement( BARRE_ERREUR );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'informations associes  l'arbre !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }

  /* Clic sur le slecteur */
  if ( ( ( num_objet >= BERR_GRF_PREMNOM) && (num_objet <= BERR_GRF_DER_NOM ) ) || /* Clic sur un graphe */
       ( BERR_GRF_AVANT   == num_objet ) || ( BERR_GRF_ASC_FND == num_objet ) ||
       ( BERR_GRF_ASC_BOU == num_objet ) || ( BERR_GRF_APRES   == num_objet ) ) { /* Ou le dfileur */
    /* Gestion du dfileur */
    ARB_GererDefileur_O( BARRE_ERREUR, BERR_GRF_PREMNOM, BERR_GRF_DER_NOM, LGMAX_TXT_NOM_GRAPHE,
			 BERR_GRF_AVANT, BERR_GRF_ASC_FND, BERR_GRF_ASC_BOU, BERR_GRF_APRES,
			 infos->nombre_graphes_choisis, NULL, &(infos->numero_graphe_choisi), &(infos->premier_graphe_affiche),
			 I_STAT_MOY_GrapheVersTexte, num_objet, y );
    return BI_EVENEMENT_TRAITE;
  }

  /* On gre les autres objets un peu particuliers */
  switch ( num_objet )
  {
  case BE_AJOUTER: /* On ajoute un spectre  la liste */
    if ( 0 == infos->nombre_graphes_choisis ) { /* Aucun spectre choisi */
      Graphe *choisi;

      /* On ne choisit qu'un seul graphe : permettra de fixer le type, le genre,... */
      choisi = GRF_choisir( GR_TOUS, FAUX, FAUX, NULL ); /* Tous les types visibles [Dans la srie courante seulement] */
      if ( choisi == NULL ) {
	return BI_EVENEMENT_TRAITE; /* On n'a rien choisi */
      }

      /* On ajoute ce graphe  la liste */
      n_doublons = I_STAT_MOY_AjouterGraphes( infos, 1, &choisi );
    } else {
      Graphe **choisis;
      int type = GR_AUCUN;
      unsigned long i, nbr_choisis;

      /* On cherche le type des graphes... */
      for ( i = 0 ; i < infos->nombre_graphes_choisis ; i++ ) {
	type = infos->graphes_choisis[ i ]->type; /* Donc on prend ce type */
	if ( ( type != GR_SSTYPE ) && ( type != GR_INTERNE ) ) /* Rellement un type */
	  break; /* On s'arrte donc l */
      }

      /* On ne choisit que les graphes de mme type (large) */
      choisis = GRF_mchoisir( type, 1, 0, &nbr_choisis );
      if ( NULL == choisis ) {
	return BI_EVENEMENT_TRAITE; /* Aucun graphe choisi : on ne fait rien */
      }

      /* On ajoute ces graphes  la liste */
      n_doublons = I_STAT_MOY_AjouterGraphes( infos, nbr_choisis, choisis );

      /* Plus besoin de cette slection */
      free( choisis ); choisis = NULL;
    }
    if ( n_doublons != 0 ) {
      /* Problme dans l'insertion des graphes */
      if ( n_doublons < 0 ) {
	BI_ERR_alerte( NULL, &ERR_AjouterGraphes, VRAI );
      } else if ( 1 == n_doublons ) {
	BI_ERR_alerte( NULL, &MSG_UnDoublon, VRAI );
      } else {
	BI_ERR_alerte( NULL, &MSG_PlusieursDoublons, VRAI, (unsigned int) n_doublons );
      }
    }
    return BI_EVENEMENT_TRAITE;

  case BE_EFFACER: /* On efface le graphe actuellement slectionn */
    I_STAT_MOY_EnleverGraphe( infos, infos->numero_graphe_choisi );
    return BI_EVENEMENT_TRAITE;

  case BE_TT_AJOUTER: /* On ajoute _tous_ les spectres de mme type */
    {
      Graphe **choisis;
      int type = GR_AUCUN; /* Le type qu'il faut utiliser */
      unsigned long i, nombre_choisis;

      /* On cherche le type des graphes... */
      for ( i = 0 ; i < infos->nombre_graphes_choisis ; i++ ) {
	type = infos->graphes_choisis[ i ]->type; /* Donc on prend ce type */
	if ( ( type != GR_SSTYPE ) && ( type != GR_INTERNE ) ) /* Rellement un type */
	  break; /* On s'arrte donc l */
      }

      /* On rcupre les graphes de la srie courante qui ont ce type [strict !] */
      choisis = GRF_ChoisirType( serie_courante, type, VRAI, &nombre_choisis );
      if ( NULL == choisis ) {
	return BI_EVENEMENT_TRAITE;
      }

      /* On les ajoute  la slection */
      n_doublons = I_STAT_MOY_AjouterGraphes( infos, nombre_choisis, choisis );
      if ( n_doublons != 0 ) {
	/* Problme dans l'insertion des graphes */
	if ( n_doublons < 0 ) {
	  BI_ERR_alerte( NULL, &ERR_AjouterGraphes, VRAI );
	} else if ( 1 == n_doublons ) {
	  BI_ERR_alerte( NULL, &MSG_UnDoublon, VRAI );
	} else {
	  BI_ERR_alerte( NULL, &MSG_PlusieursDoublons, VRAI, (unsigned int) n_doublons );
	}
      }

      /* Plus besoin de cette slection */
      free( choisis ); choisis = NULL;
    }
    return BI_EVENEMENT_TRAITE;

  case BE_TT_EFFACER: /* On efface _tous_ les graphes de la liste */
    free( infos->graphes_choisis ); infos->graphes_choisis = NULL;
    infos->nombre_graphes_choisis = 0;

    ARB_PreparerDefileur_O( BARRE_ERREUR, BERR_GRF_PREMNOM, BERR_GRF_DER_NOM, LGMAX_TXT_NOM_GRAPHE,
			    BERR_GRF_AVANT, BERR_GRF_ASC_FND, BERR_GRF_ASC_BOU, BERR_GRF_APRES,
			    0, NULL, 0, 0, I_STAT_MOY_GrapheVersTexte );
    return BI_EVENEMENT_TRAITE;

  case BERR_MUL_SIGMA:
    etat = OBJ_choisit( BARRE_ERREUR, BERR_MUL_SIGMA, ECHANGE );
    OBJ_active( BARRE_ERREUR, BERR_MUL_SIGMA_V, etat );
    OBJ_active( BARRE_ERREUR, BERR_MUL_SIGMA_S, etat & ( infos->nombre_graphes_choisis > 2 ) );
    return BI_EVENEMENT_TRAITE;

  case BERR_MUL_SIGMA_S: /* Pour la suggestion, d'aprs Vlaic & coll., JSR, 1999 (6), p. 262-266
			    Les rsultats sont cependant douteux  mon avis */
    OBJ_choisit( BARRE_ERREUR, BERR_MUL_SIGMA_S, FORCE_OUI );
    if ( infos->nombre_graphes_choisis == 3 )
      OBJ_MettreNombre( BARRE_ERREUR, BERR_MUL_SIGMA_V, 2.2, 0.0 );
    if ( infos->nombre_graphes_choisis == 4 )
      OBJ_MettreNombre( BARRE_ERREUR, BERR_MUL_SIGMA_V, 1.5, 0.0 );
    if ( infos->nombre_graphes_choisis == 5 )
      OBJ_MettreNombre( BARRE_ERREUR, BERR_MUL_SIGMA_V, 1.2, 0.0 );
    if ( infos->nombre_graphes_choisis == 6 )
      OBJ_MettreNombre( BARRE_ERREUR, BERR_MUL_SIGMA_V, 1.1, 0.0 );
    if ( infos->nombre_graphes_choisis >= 7 ) /* Au-del de 7 spectres, suppos correct */
      OBJ_MettreNombre( BARRE_ERREUR, BERR_MUL_SIGMA_V, 1.0, 0.0 );
    OBJ_choisit( BARRE_ERREUR, BERR_MUL_SIGMA_S, FORCE_NON );
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE; /* Objet non utilis */
}

/* -------- Ajouter des graphes  la liste des graphes choisis (aprs le graphe actif) -------- */

static int I_STAT_MOY_AjouterGraphes(struct infos_moyenner_GRF *infos, unsigned long nombre_graphes, Graphe **graphes)
{
  unsigned int i, n_doublons = 0;
  boolean reussi;

  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Infos = NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -1;
  }
  if ( ( 0 == nombre_graphes ) || ( NULL == graphes ) ) return +1;

  if ( NULL == infos->graphes_choisis ) {
    /* On doit donc crer la table */
    infos->graphes_choisis = (Graphe **) malloc( sizeof( Graphe * ) * nombre_graphes );
    if ( NULL == infos->graphes_choisis ) {
      ERR_Memoire( "Table des graphes slectionns", sizeof( Graphe * ) * nombre_graphes );
      return -2;
    }

    infos->nombre_graphes_choisis = 0; /* En thorie superflu, mais soyons prudent... */
  } else {
    /* On augmente la taille de la table */
    Graphe **nv_table;

    nv_table = (Graphe **) realloc( infos->graphes_choisis, sizeof(Graphe *) * ( infos->nombre_graphes_choisis + nombre_graphes ) );
    if ( NULL == nv_table ) {
      ERR_Memoire( "Extension de la table des graphes slectionns", sizeof(Graphe *) * (infos->nombre_graphes_choisis + nombre_graphes) );
      return -2;
    }
    infos->graphes_choisis = nv_table;
  }

  /* On ajoute les graphes */
  for ( i = 0 ; i < nombre_graphes ; i++ ) {
    reussi = I_STAT_MOY_AjouterUnGraphe( infos, graphes[ i ] );
    if ( FAUX == reussi ) {
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s,%s l.%u : Graphe en double = %s\n",
		     __FILE__, __FUNCTION__, __LINE__, graphes[ i ]->nom);

#endif
      n_doublons++;
    }

    /* L'incrmentation de infos est normalement dj faite */
  }

  /* On raffiche la liste */
  ARB_PreparerDefileur_O( BARRE_ERREUR, BERR_GRF_PREMNOM, BERR_GRF_DER_NOM, LGMAX_TXT_NOM_GRAPHE,
			  BERR_GRF_AVANT, BERR_GRF_ASC_FND, BERR_GRF_ASC_BOU, BERR_GRF_APRES,
			  infos->nombre_graphes_choisis, NULL, infos->numero_graphe_choisi, infos->premier_graphe_affiche,
			  I_STAT_MOY_GrapheVersTexte );

  /* On active les options, si besoin est */
  I_STAT_MOY_ActiverObjets( infos->nombre_graphes_choisis );

  /* Termin : on renvoie le nombre de doublons */
  return n_doublons;
}

/* -------- Ajouter un graphe  la fin de la liste ; renvoie VRAI si l'a fait (pas NULL, pas en doublon...) -------- */

static boolean I_STAT_MOY_AjouterUnGraphe(struct infos_moyenner_GRF *infos, Graphe *graphe)
{
  unsigned int i;

  if ( NULL == graphe ) return FAUX;

  /* On vrifie que le graphe n'est pas dans la liste */
  for ( i = 0 ; i < infos->nombre_graphes_choisis ; i++ ) {
    if ( graphe == infos->graphes_choisis[ i ] ) {
      /* Graphe dj prsent : on ne va pas l'insrer encore ! */
      return FAUX;
    }
  }

  /* i pointe ici sur la dernire case, suppose exister... On insre donc le graphe */
  infos->graphes_choisis[ i ] = graphe;

  /* Et l'on contient donc un graphe de plus */
  infos->nombre_graphes_choisis++;

  /* Termin */
  return VRAI;
}

/* -------- Enlever un graphe de la liste des graphes choisis -------- */

static int I_STAT_MOY_EnleverGraphe(struct infos_moyenner_GRF *infos, unsigned long numero_graphe_a_enlever)
{
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Infos = NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -1;
  }
  if ( numero_graphe_a_enlever >= infos->nombre_graphes_choisis ) {
    AVERTISSEMENT( "Numro de graphe absurde [%lu, 0  %lu possible]", numero_graphe_a_enlever, infos->nombre_graphes_choisis - 1 );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -2;
  }

  if ( 1 == infos->nombre_graphes_choisis ) {
    /* Dernier graphe : on peut effacer la table ! */
    free( infos->graphes_choisis ); infos->graphes_choisis = NULL;
  } else {
    unsigned long i;

    /* On dplace tous les graphes suivants */
    for ( i = numero_graphe_a_enlever ; i < ( infos->nombre_graphes_choisis - 1 ) ; i++ ) {
      infos->graphes_choisis[ i ] = infos->graphes_choisis[ i + 1 ];
    }
    infos->graphes_choisis[ infos->nombre_graphes_choisis ] = NULL;
  }

  /* Un graphe de moins dans la liste */
  infos->nombre_graphes_choisis--;

  /* On raffiche la liste */
  ARB_PreparerDefileur_O( BARRE_ERREUR, BERR_GRF_PREMNOM, BERR_GRF_DER_NOM, LGMAX_TXT_NOM_GRAPHE,
			  BERR_GRF_AVANT, BERR_GRF_ASC_FND, BERR_GRF_ASC_BOU, BERR_GRF_APRES,
			  infos->nombre_graphes_choisis, NULL, infos->numero_graphe_choisi, infos->premier_graphe_affiche,
			  I_STAT_MOY_GrapheVersTexte );

  /* Contrles ventuels */
  I_STAT_MOY_ActiverObjets( infos->nombre_graphes_choisis );

  /* Termin */
  return 0;
}

/* ----------- Prparation du slecteur dans la bote de dialogue --------------- */

static void I_STAT_MOY_ActiverObjets(unsigned long nombre_choisis)
{
  if ( 0 == nombre_choisis ) {
    /* Aucun graphe choisi : pas mal d'objets inactifs ! */
    OBJ_active( BARRE_ERREUR, BE_TT_AJOUTER, FORCE_NON );

    OBJ_active( BARRE_ERREUR, BE_EFFACER, FORCE_NON );
    OBJ_active( BARRE_ERREUR, BE_TT_EFFACER, FORCE_NON );

    OBJ_active( BARRE_ERREUR, BERR_OK, FORCE_NON );
  } else {
    OBJ_active( BARRE_ERREUR, BE_TT_AJOUTER, FORCE_OUI );

    OBJ_active( BARRE_ERREUR, BE_EFFACER, FORCE_OUI );
    OBJ_active( BARRE_ERREUR, BE_TT_EFFACER, FORCE_OUI );

    OBJ_active( BARRE_ERREUR, BERR_OK, FORCE_OUI );
  }

  if ( nombre_choisis <= 2 ) {
    /* Pas de proposition pour la correction de variance */
    OBJ_active( BARRE_ERREUR, BERR_MUL_SIGMA_S, FORCE_NON );
  } else {
    if ( VRAI == OBJ_choisit( BARRE_ERREUR, BERR_MUL_SIGMA, QUESTION ) ) {
      /* En-dessous de 3 graphes, pas de proposition de correction des incertitudes ! */
      OBJ_active( BARRE_ERREUR, BERR_MUL_SIGMA_S, FORCE_OUI );
    }
  }
}

/* -------- Prparation de l'arbre de paramtrage des calculs d'cart-type -------- */

static void I_GRF_PreparerArbreMoyenner(void)
{
  int retour;

  if ( VRAI == arbres[BARRE_ERREUR].drapeaux.prepare ) return; /* Dj initialis ! */

  /* On commence par les conversions de menus */
  OBJ_ChangerType( BARRE_ERREUR, BERR_METHODE_X, G_SSMENU );
  OBJ_MettreMenu ( BARRE_ERREUR, BERR_METHODE_X, DISP_X_NOMBRE_METHODES, options_moyenne_X, titre_menu_X, NULL );

  /* On initialise ensuite */
  retour = ARB_Initialiser( BARRE_ERREUR, "menu_erreurs.html#b_erreur", I_GRF_GererArbreMoyenner, ( int (*)(unsigned int, int, int *) ) NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme en initialisant l'arbre [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return;
  }

  /* Amliorations esthtiques */
  ARB_EnjoliverDefileur( BARRE_ERREUR, ROOT, BERR_GRF_PREMNOM, BERR_GRF_DER_NOM,
			 BERR_GRF_AVANT, BERR_GRF_ASC_FND, BERR_GRF_ASC_BOU, BERR_GRF_APRES, ARB_ASCENSEUR_GAUCHE );

  OBJ_deplace( BARRE_ERREUR, BERR_TITRE_X    , 0, 500, DEPLACE_RELATIF );
  OBJ_deplace( BARRE_ERREUR, BERR_TITRE_Y    , 0, 500, DEPLACE_RELATIF );

#ifdef AVEC_TRADUCTION_ /* Traduction des textes */
  OBJ_TraduireTexte(BARRE_ERREUR, BE_AJOUTER    , txt_arb_ajouter   );
  OBJ_TraduireTexte(BARRE_ERREUR, BE_EFFACER    , txt_arb_effacer   );
  OBJ_TraduireTexte(BARRE_ERREUR, BE_TT_AJOUTER , txt_arb_tt_ajouter);
  OBJ_TraduireTexte(BARRE_ERREUR, BE_TT_EFFACER , txt_arb_tt_effacer);

  OBJ_TraduireTexte(BARRE_ERREUR, BERR_GARDER_X   , txt_arb_converser );
  OBJ_TraduireTexte(BARRE_ERREUR, BERR_TITRE_X    , txt_arb_X         );

  OBJ_TraduireTexte(BARRE_ERREUR, BE_TXT_MULTIPLIE, txt_arb_mulsigma   );
  OBJ_TraduireTexte(BARRE_ERREUR, BERR_MUL_SIGMA_S, txt_arb_proposition);
  OBJ_TraduireTexte(BARRE_ERREUR, BE_TXT_DIVISER  , txt_arb_sigmamoyen );
  OBJ_TraduireTexte(BARRE_ERREUR, BERR_TITRE_Y    , txt_arb_Y          );

  OBJ_TraduireTexte( BARRE_ERREUR, BE_TXT_NOM     , txt_arb_nom       );
  OBJ_TraduireTexte( BARRE_ERREUR, BE_TXT_NV_SERIE, txt_arb_nlle_serie);
  OBJ_TraduireTexte( BARRE_ERREUR, BE_TXT_TESTS   , txt_arb_test_homo );
  OBJ_TraduireTexte( BARRE_ERREUR, BERR_OK        , txt_arb_ok        );
  OBJ_TraduireTexte( BARRE_ERREUR, BERR_COVAR     , txt_arb_covariance);
  OBJ_TraduireTexte( BARRE_ERREUR, BERR_ANNULER   , txt_arb_annuler   );
#endif

#ifndef _DEVELOPPE_
  OBJ_active( BARRE_ERREUR, BERR_COVAR, FORCE_NON );
#endif

  arbres[ BARRE_ERREUR ].drapeaux.prepare = VRAI; /* Pour ne pas le refaire  chaque appel */
}

/* ~-~-~-~-~-~-~-~-~ Oprations sur le graphe rsultat ~-~-~-~-~-~-~-~-~ */

/* -------- Nommer le graphe -------- */

static void I_STAT_MOY_NommerGraphe(Graphe *resultat, const char *masque, unsigned long nombre_moyennes)
{
  char *titre, *masque_converti = NULL;
  int nombre_remplacements = 0;

  if ( masque != NULL ) {
    masque_converti = conv_chaine( masque , DIESE_LONG, &nombre_remplacements );
  }
  if ( masque_converti != NULL ) {
    if ( 0 == nombre_remplacements ) {
      titre = TXT_genere_chaine( masque_converti );
    } else if ( 1 == nombre_remplacements ) {
      titre = TXT_genere_chaine( masque_converti, nombre_moyennes );
    } else {
      (void) fprintf(stderr, "%s,%s l.%u : Double # : non gr !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      titre = TXT_genere_chaine( txt_nom_grf_par_defaut, nombre_moyennes );
    }
  } else {
    titre = TXT_genere_chaine( txt_nom_grf_par_defaut, nombre_moyennes );
  }

  /* Plus besoin du masque converti */
  free( masque_converti ); masque_converti = NULL;

  /* On nomme le graphe */
  GRF_titre( resultat, titre );

  /* Plus besoin du titre */
  free( titre ); titre = NULL;
}
