/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 2005, Emmanuel CURIS
   Cration : 6 mars 2005

   Interface graphique

   Transformer, de faon "autonome", un objet --- avec transformations prdfinies possibles !
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "ascii_utils.h"
#include "bin_utils.h"		/* Fichiers d'un dossier */

#include "systeme.h"
#include "exafs.h"

#include "c_formel.h"
#include "figures.h"
#include "interface.h"
#include "modeles.h"		/* Pour MODELE_AjusterDroite() */
#include "operations.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

struct transformation {
  char *nom_fichier;
  char *nom;
};

struct infos_arbre {
  unsigned long nombre_transformations;
  struct transformation *transformations;
  unsigned long premiere_visible;
  unsigned long choisie;
  Graphe *graphe;		/* Le graphe transform */
};

#ifndef PREFIXE_REP
# define PREFIXE_REP "/usr/local/share/Lase"
#endif
#define CHEMIN_TRANSFORMATIONS PREFIXE_REP"/Transfos"

#define LGMAX_NOM_TRANSFORMATION 21

#include "int_transformer.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void    I_TRF_PreparerArbre(void);
static int     I_TRF_GererArbre   (unsigned int objet, int x, int y);
static void    I_TRF_ActiverX     (int sens, const Graphe *graphe);
static void    I_TRF_ActiverY     (int sens, const Graphe *graphe);
static void    I_TRF_ActiversX    (int sens, const Graphe *graphe);
static void    I_TRF_ActiversY    (int sens, const Graphe *graphe);

static void I_LibererTransformations (struct infos_arbre *infos);
static int  I_LireTransformation     (struct transformation *transformation, const Graphe *graphe);
static int  I_PreparerTransformations(struct infos_arbre *infos, const char *prefixe);

static boolean T_EstApplicable    (const char *applicabilite, const Graphe *graphe);
static boolean T_EstTransformation(const char *nom_fichier, const Graphe *graphe);
static char   *T_LireNom          (const char *nom_fichier);

static const char *T_AfficheNom(unsigned int arbre, unsigned long numero_objet);

#define T_TrouverSigneEgal {\
  c = fgetc( fichier );\
  while ( c != '=' ) {\
    if ( EOF == c ) {\
      (void) fprintf(stderr, "%s, %s l.%u : Format de fichier incorrect, ligne sans signe =\n",\
		     __FILE__, __FUNCTION__, __LINE__);\
      (void) fclose( fichier );\
      return -2;\
    }\
    c = fgetc( fichier );\
  }\
  c = fgetc( fichier ); /* Premier caractre aprs le gal */\
  while ( isspace( c ) ) {\
    if ( EOF == c ) {\
      (void) fprintf(stderr, "%s, %s l.%u : Format de fichier incorrect, signe = en fin de fichier\n",\
		     __FILE__, __FUNCTION__, __LINE__);\
      (void) fclose( fichier );\
      return -2;\
    }\
    c = fgetc( fichier );\
  }\
  ungetc( c, fichier ); /* On remet le caractre non = ! */\
}

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Transformation type papier semi-log ~-~-~-~-~-~-~-~-~ */

Graphe *I_GRF_SemiLog(Graphe *graphe)
{
  struct serie *serie = NULL;
  Bloc_Figure *bloc = NULL;
  Graphe *resultat = NULL, *graphes[ 2 ] = { NULL, NULL };
  Fenetre *fenetre;
  Figure *figure = NULL;
  double pente, o_a_lo;
  int retour;

  /* On prpare le rsultat : mmes X, mmes valeurs en Y  l'origine ; isol pour commencer */
  resultat = GRF_Copier( graphe, NULL, VRAI );
  if ( NULL == resultat ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur en voulant prparer le calcul\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On fait la transformation, en log dcimal : par cohrence avec l'affichage de l'chelle log. de LASE [en place] */
  retour = GRF_SemiLog_Y( resultat, 10.0 );
  if ( retour != 0 ) {
    /* Erreur survenue : conversion impossible */
  }

  /* Le calcul est fait : on insre le rsultat dans la srie */
  serie = I_GRF_InsererDansSerie( NULL, graphe, resultat );
  if ( NULL == serie ) {
    /* On n'a pas pu, ou pas voulu, conserver les rsultats */
    GRF_eff_isole( resultat );
    resultat = NULL;

    return NULL;
  } else {
    char *texte;

    /* On met un nom, les lgendes,... */
    /* a. En X : mme lgende, mme unit ; mme style de dessin */
    GRF_CopierAffichage( resultat, graphe, GRF_CP_AFF_LEGENDE_X | GRF_CP_AFF_UNITE_X | GRF_CP_AFF_DESSIN );

    /* b. En Y : lgende = log( lgende ) ; devient sans unit */
    texte = TXT_genere_chaine( "log(%s)", GRF_QuelleLegende( graphe, AXE_Y ) );
    GRF_ChangerLegende( resultat, AXE_Y, texte, FAUX );
    /* La libration sera faite avec la lgende... */

    /* c. Le nom : comme pour la lgende... */
    texte = TXT_genere_chaine( "log(%s)", graphe->nom );
    GRF_titre( resultat, texte );
    free( texte ); texte = NULL;
  }

  /* On cherche la droite des MC sur le modle transform */
  retour = MODELE_AjusterDroite( resultat, graphes + 1, NULL, &pente, &o_a_lo );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur dans MODELE_AjusterDroite [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    /* On ne fait donc pas de figure */
    return resultat;
  }
  /* On prpare pour les affichages, mme si graphe isol... Car permettra de l'inclure correctement dans une srie si l'utilisateur le veut ! */
  GRF_CopierAffichage( graphes[ 1 ], resultat, GRF_CP_AFF_LEGENDE_X | GRF_CP_AFF_LEGENDE_Y | GRF_CP_AFF_UNITE_X | GRF_CP_AFF_UNITE_Y );
  GRF_CouleurSuivante( graphes[ 1 ], resultat );
  GRF_couleur( graphes[ 1 ], -1, BI_CONTINU, -1 );  /* On force l'existence d'un trait continu : c'est une droite, donc normal ! */

  /* On affiche la bote d'alerte avec les rsultats de l'ajustement */
  retour = BI_ERR_alerte( principale, &MSG_ValeursAjusteesSemiLog, VRAI, o_a_lo, pente, pow( 10.0, o_a_lo ), pente * log( 10.0 ), pow( 10.0, o_a_lo ), pente );

  /* On cre une figure avec le graphe transform et son modle linaire */
  figure = FIG_CreerFigure();
  if ( NULL == figure ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de crer la figure !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return resultat;
  }

  /* Cette figure contient un seul bloc qui contient le graphe transform et son modle linaire */
  bloc = FIG_CreerBloc( figure, NULL, BLOCFIGURE_TYPE_GRAPHES,
			figure->marge_gauche, figure->marge_basse,
			figure->largeur - ( figure->marge_gauche + figure->marge_droite ), figure->hauteur - ( figure->marge_basse + figure->marge_haute ) );
  if ( NULL == bloc ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de crer le bloc dans la figure !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIG_EffacerFigure( figure ); figure = NULL;
    return NULL;
  }

  graphes[ 0 ] = resultat;
  retour = BLOC_AjouterGraphes( bloc, graphes, 2 );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur en voulant placer les graphes dans la figure !\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }

  /* Axes de ce bloc */
  AXE_MettreLegende( &(bloc->dessin.axe_X), GRF_QuelleLegende( graphe, AXE_X ) );
  AXE_MettreUnite  ( &(bloc->dessin.axe_X), GRF_QuelleUnite  ( graphe, AXE_X ) );

  bloc->dessin.axe_Y.type_graduation = GRADUATION_LOGARITHMIQUE;
  AXE_MettreLegende( &(bloc->dessin.axe_Y), GRF_QuelleLegende( graphe, AXE_Y ) );
  AXE_MettreUnite  ( &(bloc->dessin.axe_Y), GRF_QuelleUnite  ( graphe, AXE_Y ) );

  /* On ouvre la fentre affichant cette figure */
  fenetre = FIG_OuvrirFenetre( figure );
  if ( NULL == fenetre ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur en ouvrant la figure !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    /* On efface la figure pour viter des ennuis... */
    FIG_EffacerFigure( figure );
  }

  /* On a fini : on renvoie le rsultat */
  return resultat;
}

/* ~-~-~-~-~-~-~-~-~ Transformation quelconque ~-~-~-~-~-~-~-~-~ */

/* -------- Interface graphique : transformer un graphe (X,Y) -------- *
    Renvoie le graphe transform
 * ------------------------------------------------------------------- */

Graphe *I_GRF_Transformer(Graphe *graphe)
{
  int reponse;
  struct infos_arbre infos = { 0, NULL, 0, 0, NULL };
  char *formule;
  Graphe *resultat = NULL;
  Expression *tr_X = NULL, *tr_Y = NULL, *tr_sX = NULL, *tr_sY = NULL;

  if ( NULL == graphe ) return NULL;

  /* Le graphe est-il du bon type ? */
  if ( FAUX == GRF_EstReel( graphe ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Opration impossible sur les graphes  valeurs non relles\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  infos.graphe = graphe;

  /* Prparation de l'arbre de transformations */
  I_TRF_PreparerArbre( );
  ARB_MettreComplement( TRANSFORME_GRAF, &infos );

  /* Peut-on transformer en X ? */
  if ( ( GRF_SERIE == graphe->genre_donnees ) || ( GRF_ERREUR_R == graphe->genre_donnees ) ) {
    /* X absents ou dfinis extrieurement */
    OBJ_active ( TRANSFORME_GRAF, TG_AVEC_TF_X, FORCE_NON );
    I_TRF_ActiverX( -1, graphe );
  } else {
    /* X prsents : on peut les transformer */
    OBJ_active( TRANSFORME_GRAF, TG_AVEC_TF_X, FORCE_OUI );

    if ( NULL == graphe->erreur_x ) {
      OBJ_active( TRANSFORME_GRAF, TG_AVEC_SX, FORCE_NON ); 
      I_TRF_ActiversX( -1, graphe );
    } else if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_X, QUESTION ) ) {
      OBJ_active( TRANSFORME_GRAF, TG_AVEC_SX, FORCE_OUI );
      if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SX, QUESTION ) ) {
	OBJ_active( TRANSFORME_GRAF, TG_TRANSFORME_SX, FORCE_OUI );
      }
    }
  }

  /* La transformation en Y peut-elle concerner les erreurs ? */
  if ( NULL == graphe->erreur_y ) {
    OBJ_active( TRANSFORME_GRAF, TG_AVEC_SY, FORCE_NON );
    I_TRF_ActiversY( -1, graphe );
  } else if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_Y, QUESTION ) ) {
    OBJ_active( TRANSFORME_GRAF, TG_AVEC_SY, FORCE_OUI );
    if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SY, QUESTION ) ) {
      OBJ_active( TRANSFORME_GRAF, TG_TRANSFORME_SY, FORCE_OUI );
    }
  }

  /* Prparation de la liste des transformations prdfinies */
  reponse = I_PreparerTransformations( &infos, CHEMIN_TRANSFORMATIONS );
  if ( reponse != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Problme dans le chargement des transformations prdfinies !\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }

  /* On prpare le dfileur des transformations */
  (void) ARB_PreparerDefileur_O( TRANSFORME_GRAF, TG_PREDEF_1, TG_PREDEF_N, LGMAX_NOM_TRANSFORMATION,
				 TG_AVANT_PREDEF, TG_FDASC_PREDEF, TG_BTASC_PREDEF, TG_APRES_PREDEF,
				 infos.nombre_transformations, NULL, infos.choisie, infos.premiere_visible,
				 T_AfficheNom );

  /* Ouverture de l'arbre */
  reponse = ARB_Bloquant( TRANSFORME_GRAF, "Transformer un graphe", principale, 0 );
  ARB_MettreComplement( TRANSFORME_GRAF, NULL );
  I_LibererTransformations( &infos );
  if ( TG_ANNULER == reponse ) {
    return NULL;
  }

  /* Lecture des donnes de l'arbre */
  if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_X, QUESTION ) ) {
    formule = lit_chaine( TRANSFORME_GRAF, TG_TRANSFORME_X, 0, NULL, TXT_COPIER );
    tr_X = construit_arbre( formule );

    /* Veut-on aussi transformer les erreurs en X ? */
    if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SX, QUESTION ) ) {
      formule = lit_chaine( TRANSFORME_GRAF, TG_TRANSFORME_SX, 0, NULL, TXT_COPIER );
      tr_sX = construit_arbre( formule );
    } else {
      tr_sX = NULL;
    }
  } else {
    tr_X = NULL;
  }

  if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_Y, QUESTION ) ) {
    formule = lit_chaine( TRANSFORME_GRAF, TG_TRANSFORME_Y, 0, NULL, TXT_COPIER );
    tr_Y = construit_arbre( formule );

    /* Veut-on aussi transformer les erreurs en Y ? */
    if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SY, QUESTION ) ) {
      formule = lit_chaine( TRANSFORME_GRAF, TG_TRANSFORME_SY, 0, NULL, TXT_COPIER );
      tr_sY = construit_arbre( formule );
    } else {
      tr_sY = NULL;
    }
  } else {
    tr_Y = NULL;
  }

  /* Application de la transformation, si besoin est... */
  resultat = GRF_Transformer_R( graphe, tr_X, tr_sX, tr_Y, tr_sY );

  /* On libre les arbres d'expression construits */
  effacer_arbre( tr_X ); tr_X = NULL;
  effacer_arbre( tr_Y ); tr_Y = NULL;
  effacer_arbre( tr_sX ); tr_sX = NULL;
  effacer_arbre( tr_sY ); tr_sY = NULL;

  /* Termin : on renvoit le rsultat */
  return resultat;
}

/* -------- Prparation de l'arbre -------- */

static void I_TRF_PreparerArbre(void)
{
  int retour;

  if ( VRAI == arbres[ TRANSFORME_GRAF ].drapeaux.prepare ) return; /* Prparation dj effectue ! */

  /* Initialisation interne */
  retour = ARB_Initialiser( TRANSFORME_GRAF, NULL, I_TRF_GererArbre, NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Problme  l'initialisation [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return;
  }

  /* Enjolivements visuels */
  OBJ_deplace( TRANSFORME_GRAF, TG_AVEC_TF_Y    , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( TRANSFORME_GRAF, TG_TXT_YP       , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( TRANSFORME_GRAF, TG_TRANSFORME_Y , 0, +1, DEPLACE_PIXELS );

  OBJ_deplace( TRANSFORME_GRAF, TG_TXT_SY        , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( TRANSFORME_GRAF, TG_AVEC_SY       , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( TRANSFORME_GRAF, TG_TRANSFORME_SY , 0, +1, DEPLACE_PIXELS );

  (void) ARB_EnjoliverDefileur( TRANSFORME_GRAF, TG_FOND_PREDEF, TG_PREDEF_1, TG_PREDEF_N,
				TG_AVANT_PREDEF, TG_FDASC_PREDEF, TG_BTASC_PREDEF, TG_APRES_PREDEF, ARB_ASCENSEUR_DROITE );

  /* Traduction ventuelle */
#ifdef AVEC_TRADUCTION_
#endif

  /* Termin */
  arbres[ TRANSFORME_GRAF ].drapeaux.prepare = VRAI;
}

/* -------- Grer l'arbre -------- */

static int I_TRF_GererArbre(unsigned int objet, int x __attribute__((__unused__)), int y)
{
  struct infos_arbre *infos;

  /* Infos sur l'arbre */
  infos = (struct infos_arbre *) ARB_LireComplement( TRANSFORME_GRAF );
  if ( NULL == infos ) return BI_EVENEMENT_TRAITE;

  /* Gestion du dfileur de transformations prdfinies */
  if ( ( objet >= TG_PREDEF_1 ) && ( objet <= TG_PREDEF_N ) ) {
    /* On rcupre le nouvel objet slectionn */
    (void) ARB_GererDefileur_O( TRANSFORME_GRAF, TG_PREDEF_1, TG_PREDEF_N, LGMAX_NOM_TRANSFORMATION,
				TG_AVANT_PREDEF, TG_FDASC_PREDEF, TG_BTASC_PREDEF, TG_APRES_PREDEF,
				infos->nombre_transformations, NULL, &(infos->choisie), &(infos->premiere_visible), T_AfficheNom, objet, y );

    /* On en dduit la transformation slectionne */
    if ( infos->choisie < infos->nombre_transformations ) {
      (void) I_LireTransformation( infos->transformations + infos->choisie, infos->graphe );
    }
    return BI_EVENEMENT_TRAITE;
  }

  /* Gestion des boutons particuliers */
  switch ( objet )
  {
  case TG_AVEC_TF_X:		/* Doit-on transformer les X ? */
    if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_X, QUESTION ) ) {
      I_TRF_ActiverX( -1, infos->graphe );
    } else {
      I_TRF_ActiverX( +1, infos->graphe );
    }
    return BI_EVENEMENT_TRAITE;

  case TG_AVEC_TF_Y:		/* Doit-on transformer les Y ? */
    if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_Y, QUESTION ) ) {
      I_TRF_ActiverY( -1, infos->graphe );
    } else {
      I_TRF_ActiverY( +1, infos->graphe );
    }
    return BI_EVENEMENT_TRAITE;

  case TG_AVEC_SX:		/* Doit-on transformer les erreurs en X ? */
    if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SX, QUESTION ) ) {
      I_TRF_ActiversX( -1, infos->graphe );
    } else {
      I_TRF_ActiversX( +1, infos->graphe );
    }
    return BI_EVENEMENT_TRAITE;

  case TG_AVEC_SY:		/* Doit-on transformer les erreurs en Y ? */
    if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SY, QUESTION ) ) {
      I_TRF_ActiversY( -1, infos->graphe );
    } else {
      I_TRF_ActiversY( +1, infos->graphe );
    }
    return BI_EVENEMENT_TRAITE;

  case TG_AVANT_PREDEF:		/* Dfileur : on se contente de faire dfiler la liste */
  case TG_FDASC_PREDEF:
  case TG_BTASC_PREDEF:
  case TG_APRES_PREDEF:
    (void) ARB_GererDefileur_O( TRANSFORME_GRAF, TG_PREDEF_1, TG_PREDEF_N, LGMAX_NOM_TRANSFORMATION,
				TG_AVANT_PREDEF, TG_FDASC_PREDEF, TG_BTASC_PREDEF, TG_APRES_PREDEF,
				infos->nombre_transformations, NULL, &(infos->choisie), &(infos->premiere_visible), T_AfficheNom, objet, y );
    return BI_EVENEMENT_TRAITE;

  case TG_RELIRE:		/* Relire la liste des transformations prdfinies */
    OBJ_choisit( TRANSFORME_GRAF, objet, FORCE_OUI );
    I_PreparerTransformations( infos, CHEMIN_TRANSFORMATIONS );

    /* On remet  jour le dfileur des transformations */
    (void) ARB_PreparerDefileur_O( TRANSFORME_GRAF, TG_PREDEF_1, TG_PREDEF_N, LGMAX_NOM_TRANSFORMATION,
				   TG_AVANT_PREDEF, TG_FDASC_PREDEF, TG_BTASC_PREDEF, TG_APRES_PREDEF,
				   infos->nombre_transformations, NULL, infos->choisie, infos->premiere_visible,
				   T_AfficheNom );

    OBJ_choisit( TRANSFORME_GRAF, objet, FORCE_NON );
    return BI_EVENEMENT_TRAITE;
  }

  /* Objet inconnu : gestion par dfaut */
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Macros d'activation des objets -------- */

static void I_TRF_ActiverX(int sens, const Graphe *graphe)
{
  if ( sens > 0 ) {		/* On active */
    if ( graphe->x != NULL ) {
      OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_X, FORCE_OUI );
      OBJ_active ( TRANSFORME_GRAF, TG_TRANSFORME_X, FORCE_OUI );

      if ( graphe->erreur_x != NULL ) {
	OBJ_active( TRANSFORME_GRAF, TG_AVEC_SX, FORCE_OUI );
	if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SX, QUESTION ) ) {
	  OBJ_active( TRANSFORME_GRAF, TG_TRANSFORME_SX, FORCE_OUI );
	}
      }
    }
  } else {			/* On dsactive */
    OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_X, FORCE_NON );
    OBJ_active ( TRANSFORME_GRAF, TG_TRANSFORME_X, FORCE_NON );

    I_TRF_ActiversX( -1, graphe );
    OBJ_active ( TRANSFORME_GRAF, TG_AVEC_SX, FORCE_NON );
  }
}

static void I_TRF_ActiverY(int sens, const Graphe *graphe)
{
  if ( sens > 0 ) {		/* On active */
    OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_Y, FORCE_OUI );
    OBJ_active ( TRANSFORME_GRAF, TG_TRANSFORME_Y, FORCE_OUI );

    if ( graphe->erreur_y != NULL ) {
      OBJ_active( TRANSFORME_GRAF, TG_AVEC_SY, FORCE_OUI );
      if ( OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SY, QUESTION ) ) {
	OBJ_active( TRANSFORME_GRAF, TG_TRANSFORME_SY, FORCE_OUI );
      }
    }
  } else {			/* On dsactive */
    OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_TF_Y, FORCE_NON );
    OBJ_active ( TRANSFORME_GRAF, TG_TRANSFORME_Y, FORCE_NON );

    I_TRF_ActiversY( -1, graphe );
    OBJ_active ( TRANSFORME_GRAF, TG_AVEC_SY, FORCE_NON );
  }
}

static void I_TRF_ActiversX(int sens, const Graphe *graphe)
{
  if ( sens > 0 ) {		/* On active */
    if ( graphe->erreur_x != NULL ) {
      OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SX, FORCE_OUI );
      OBJ_active( TRANSFORME_GRAF, TG_TRANSFORME_SX, FORCE_OUI );
    }
  } else {			/* On dsactive */
    OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SX, FORCE_NON );
    OBJ_active( TRANSFORME_GRAF, TG_TRANSFORME_SX, FORCE_NON );
  }
}

static void I_TRF_ActiversY(int sens, const Graphe *graphe)
{
  if ( sens > 0 ) {		/* On active */
    if ( graphe->erreur_y != NULL ) {
      OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SY, FORCE_OUI );
      OBJ_active( TRANSFORME_GRAF, TG_TRANSFORME_SY, FORCE_OUI );
    }
  } else {			/* On dsactive */
    OBJ_choisit( TRANSFORME_GRAF, TG_AVEC_SY, FORCE_NON );
    OBJ_active( TRANSFORME_GRAF, TG_TRANSFORME_SY, FORCE_NON );
  }
}

/* ~-~-~-~-~-~-~-~-~ Gestion des transformations prdfinies ~-~-~-~-~-~-~-~-~ */

/* -------- Prparer la liste des transformations -------- */

static int I_PreparerTransformations(struct infos_arbre *infos, const char *prefixe)
{
  unsigned long i, j, nombre_fichiers;
  char **noms;

  /* On libre d'ventuelles anciennes transformations */
  if ( infos->nombre_transformations > 0 ) {
    for ( i = 0 ; i < infos->nombre_transformations ; i++ ) {
      free( infos->transformations[ i ].nom_fichier );
      free( infos->transformations[ i ].nom );
    }
    free( infos->transformations ); infos->transformations = NULL;

    infos->nombre_transformations = 0;
  }

  /* On rcupre la liste des fichiers du dossier indiqu */
  noms = DIR_ListeFichiers( prefixe, NULL, &nombre_fichiers );
  if ( NULL == noms ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucun fichier de transformation prdfinie !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0;
  }

  /* Pour chaque fichier : on vrifie qu'il contient une transformation et qu'il est lisible... */
  infos->nombre_transformations = 0;
  for ( i = 0 ; i < nombre_fichiers ; i++ ) {
    boolean retour;

    retour = T_EstTransformation( noms[ i ], infos->graphe );
    if ( VRAI == retour ) {
      infos->nombre_transformations++;      
    } else {
#ifdef _DEVELOPPE_
      AVERTISSEMENT( "Fichier %s ignor", noms[ i ] );
#endif
      free( noms[ i ] ); noms[ i ] = NULL;
    }
  }

  if ( 0 == infos->nombre_transformations ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucune transformation trouve !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    /* On a alors dj libr tous les noms de fichier ! */
    free( noms ); noms = NULL;
    return 0;
  }

  /* On rserve la liste des transformations */
  infos->transformations = (struct transformation *) calloc( infos->nombre_transformations, sizeof( struct transformation ) );
  if ( NULL == infos->transformations ) {
    (void) fprintf(stderr, "%s, %s l.%u : Mmoire insuffisante pour lire les transformations\n",
		   __FILE__, __FUNCTION__, __LINE__);
    /* On libre tous les noms de fichier */
    for ( i = 0 ; i < nombre_fichiers ; i++ ) {
      free( noms[ i ] ); noms[ i ] = NULL;
    }
    free( noms ); noms = NULL;
    infos->nombre_transformations = 0;
    return +1;
  }

  /* On mmorise la liste des transformations : fichier et nom */
  for ( i = 0, j = 0 ; i < nombre_fichiers ; i++ ) {
    if ( noms[ i ] != NULL ) {	/* Transformation utilisable */
      infos->transformations[ j ].nom_fichier = noms[ i ];
      infos->transformations[ j ].nom = T_LireNom( noms[ i ] );
      j++;

      /* Le nom de fichier sera libr en mme temps que les infos de transformation... */
    }
  }

  /* Plus besoin de la table des noms de fichier */
  free( noms ); noms = NULL;

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Librer la liste des transformations... -------- */

static void I_LibererTransformations (struct infos_arbre *infos)
{
  unsigned long i;

  for ( i = 0 ; i < infos->nombre_transformations ; i++ ) {
    free( infos->transformations[ i ].nom_fichier );
    free( infos->transformations[ i ].nom );
  }
  free( infos->transformations ); infos->transformations = NULL;

  infos->nombre_transformations = 0;
}

/* -------- Vrifie si une transformation peut s'appliquer au graphe indiqu -------- */

static boolean T_EstApplicable(const char *applicabilite, const Graphe *graphe)
{
  boolean retour;

  if ( NULL == applicabilite ) return VRAI; /* Par dfaut, on peut l'appliquer  tout ! */
  if ( NULL == graphe ) return FAUX; /* On ne peut rien appliquer  l'absence de graphe ! */

  if ( ( 'F' == *applicabilite ) || ( 'f' == *applicabilite ) ) {
    char *ss_type;

    /* F : fonction -> on cherche  savoir si relle ou complexe ? */
    retour = GRF_EstFonction( graphe );
    if ( FAUX == retour ) {	/* Le graphe n'est pas une fonction ! */
      return FAUX;
    }

    /* A-t-on prcis le type de fonction sur lequel s'applique la transformation ? */
    ss_type = strchr( applicabilite, ' ' );
    if ( NULL == ss_type ) {	/* On ne prcise pas : fonction relle par dfaut */
      return GRF_EstFonctionReelle( graphe );
    }

    while ( isspace( *ss_type ) ) {
      ss_type++;
    }
    if ( ( 'R' == *ss_type ) || ( 'r' == *ss_type ) ) {
      return GRF_EstFonctionReelle( graphe );
    } else if ( ( 'C' == *ss_type ) || ( 'c' == *ss_type ) ) {
      return GRF_EstComplexe( graphe );
    } else {
      (void) fprintf(stderr, "%s, %s l.%u : Type de fonction inconnu [%s], ignor\n",
		     __FILE__, __FUNCTION__, __LINE__, applicabilite);
      return VRAI;
    }
  } else {
    (void) fprintf(stderr, "%s, %s l.%u : Code d'applicabilit inconnu [%s], fichier ignor\n",
		   __FILE__, __FUNCTION__, __LINE__, applicabilite);
    return FAUX;
  }

  /* On ne devrait jamais tre l */
  AVERTISSEMENT( "On ne devrait jamais arriver ici [commande %s] !", applicabilite );
  return FAUX;
}

/* -------- Ouvre un fichier pour vrifier qu'il contient une transformation utilisable -------- */

static boolean T_EstTransformation(const char *nom_fichier, const Graphe *graphe)
{
  FILE *fichier;
  boolean retour;
  int c;
  char *commande;

  if ( NULL == nom_fichier ) return FAUX;

  /* On essaye d'ouvrir le fichier */
  fichier = fopen( nom_fichier, "rb" );
  if ( NULL == fichier ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible d'ouvrir le fichier %s\n",
		   __FILE__, __FUNCTION__, __LINE__, nom_fichier);
    return FAUX;
  }

  /* On lit le premier caractre du fichier */
  c = fgetc( fichier );
  if ( EOF == c ) {
    (void) fprintf(stderr, "%s, %s l.%u : Fichier %s vide !\n",
		   __FILE__, __FUNCTION__, __LINE__, nom_fichier);
    (void) fclose( fichier );
    return FAUX;
  }

  /* On saute les commentaires ventuels */
  while ( '#' == c ) {
    c = ignore_ligne( fichier );
    if ( EOF == c ) {
      (void) fprintf(stderr, "%s, %s l.%u : Fichier %s vide !\n",
		     __FILE__, __FUNCTION__, __LINE__, nom_fichier);
      (void) fclose( fichier );
      return FAUX;
    }
  }

  /* La premire ligne DOIT commencer par "Applicable" */
  if ( ( c != 'A' ) && ( c != 'a' ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Format de fichier incorrect\n",
		   __FILE__, __FUNCTION__, __LINE__);
    (void) fclose( fichier );
    return FAUX;
  }
  /* On saute donc jusqu'au signe gal */
  T_TrouverSigneEgal;

  /* Et on lit la nature de graphe sur lequel est applicable la transformation */
  commande = lit_mot( fichier );
  if ( NULL == commande ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de contrler la validit de la transformation !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    (void) fclose( fichier );
    return VRAI;		/* On l'autorise par dfaut ! */
  }

  /* On contrle qu'elle est en accord avec le graphe trait */
  retour = T_EstApplicable( commande, graphe );

  /* A priori, la transformation devrait fonctionner... (optimiste !) */
  free( commande ); commande = NULL;
  (void) fclose( fichier );
  return retour;
}

/* -------- Ouvrir un fichier pour en extraire le nom de la transformation -------- */

static char *T_LireNom(const char *nom_fichier)
{
  FILE *fichier;
  char *nom_transformation;
  int c;

  /* On ouvre le fichier */
  fichier = fopen( nom_fichier, "rb" );
  if ( NULL == fichier ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible d'ouvrir le fichier %s\n",
		   __FILE__, __FUNCTION__, __LINE__, nom_fichier);
    return NULL;
  }

  /* On saute l'en-tte et les lignes inutiles, pour reprer la ligne contenant le nom */
  c = fgetc( fichier );
  ungetc( c, fichier );		/* Lecture annule pour compatibilit avec ignore_ligne() ensuite */
  do {
    while ( ( c != 'N' ) && ( c != 'n' ) ) { /* Nom = ; pas de distinction majuscule/minuscule */
      if ( EOF == c ) {
	(void) fprintf(stderr, "%s, %s l.%u : Fin de fichier atteinte, pas de nom trouv !\n",
		       __FILE__, __FUNCTION__, __LINE__);
	(void) fclose( fichier );

	/* Pas de nom trouv ==> on renvoie le nom du fichier */
	nom_transformation = TXT_genere_chaine( "<%F>", nom_fichier );
	return nom_transformation;
      }

      /* On passe  la ligne suivante */
      c = ignore_ligne( fichier );
    }

    /* On saute ce caractre [N en dbut de ligne, non-lu par ignore_ligne */
    (void) fgetc( fichier );
    c = fgetc( fichier );
  } while ( ( 'o' != c ) && ( 'O' != c ) );

  /* Si on est l : on  trouver la ligne No[m] = <Nom> */
  /* 1. On ignore jusqu'au = */
  c = fgetc( fichier );
  while ( c != '=' ) {
    if ( EOF == c ) {
      (void) fprintf(stderr, "%s, %s l.%u : Fin de fichier atteinte, ligne nom sans = !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      (void) fclose( fichier );

      /* Format de fichier incorrect */
      return NULL;
    }

    c = fgetc( fichier );
  }

  /* Ici, c vaut '=' => on peut lire le nom directement, aprs avoir saut les espaces [fait par lit_mot()] */
  nom_transformation= lit_mot( fichier );
  (void) fclose( fichier );

  /* Termin */
  return nom_transformation;
}

/* -------- Renvoie le nom de la i-me transformation, pour le dfileur -------- */

static const char *T_AfficheNom(unsigned int arbre, unsigned long numero_objet)
{
  struct infos_arbre *infos;

  /* Infos sur l'arbre */
  infos = (struct infos_arbre *) ARB_LireComplement( arbre );
  if ( NULL == infos ) return NULL;

  if ( numero_objet >= infos->nombre_transformations ) return NULL;

  return infos->transformations[ numero_objet ].nom;
}

/* -------- Lire les informations sur la transformation dans le fichier et les afficher -------- */

static int I_LireTransformation(struct transformation *transformation, const Graphe *graphe)
{
  FILE *fichier;
  int c;
  char *texte;
  unsigned long numero_ligne = 0;

  if ( NULL == transformation ) return -1;

  /* On dcoche les transformations en X, en Y */
  I_TRF_ActiverX( -1, graphe );
  I_TRF_ActiverY( -1, graphe );
  I_TRF_ActiversX( -1, graphe );
  I_TRF_ActiversY( -1, graphe );

  /* On ouvre le fichier */
  fichier = fopen( transformation->nom_fichier, "rb" );
  if ( NULL == fichier ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible d'ouvrir le fichier %s\n",
		   __FILE__, __FUNCTION__, __LINE__, transformation->nom_fichier);
    return -2;
  }

  /* Analyse ligne  ligne du fichier */
  c = fgetc( fichier );
  ungetc( c, fichier );		/* Lecture annule pour compatibilit avec ignore_ligne() ensuite */
  do {
    numero_ligne++;		/* On traite la ligne suivante ! */
    switch ( c )
    {
    case '#':			/* Commentaire -> ligne ignore */
      c = ignore_ligne( fichier );
      break;

    case 'a':			/* "Applicable" : contrle de la nature du graphe */
    case 'A':
      /* On cherche le signe = */
      T_TrouverSigneEgal;

      /* On lit la nature du graphe  contrler */
      texte = lit_mot( fichier );
      if ( texte != NULL ) {
	boolean retour;

	retour = T_EstApplicable( texte, graphe );
	if ( FAUX == retour ) {
	  OPTION_A_FAIRE( "Message : transformation non applicable !" );
	  free( texte ); texte = NULL;
	  (void) fclose( fichier );
	  return -1;
	}
	free( texte ); texte = NULL;
      }
      c = ignore_ligne( fichier );
      break;

    case 'n':			/* Pas de distinction majuscule/minuscule */
    case 'N':
      (void) fgetc( fichier );
      c = fgetc( fichier );
      if ( ( 'o' == c ) || ( 'O' == c ) ) { /* Nom de la transformation : ignor ici */
	c = ignore_ligne( fichier );
      } else if ( ( 'a' == c ) || ( 'A' == c ) ) { /* Nature des grandeurs ncessaire */
	OPTION_A_FAIRE( "Contrle de la nature des grandeurs en X/en Y" );
	c = ignore_ligne( fichier );
      } else {
	(void) fprintf(stderr, "%s, %s l.%u : Commande inconnue dans le fichier %s, ligne %lu : ignore\n",
		       __FILE__, __FUNCTION__, __LINE__, transformation->nom_fichier, numero_ligne);
	c = ignore_ligne( fichier );
      }
      break;

    case 'e':			/* Formules de correction des carts-types */
    case 'E':
      c = ignore_car( fichier, 6 ); /* On saute le Erreur */
      if ( ( c != 'r' ) && ( c != 'R' ) ) {
	(void) fprintf(stderr, "%s, %s l.%u : Commande inconnue, l. %lu (ignore)\n",
		       __FILE__, __FUNCTION__, __LINE__, numero_ligne);
	c = ignore_ligne( fichier );
      } else {
	c = fgetc( fichier );
	if ( ( 'x' == c ) || ( 'X' == c ) ) {
	  /* Il y a une transformation des erreurs en X ! */
	  I_TRF_ActiversX( +1, graphe );

	  /* Formule de transformation : erreurs en X */
	  T_TrouverSigneEgal;

	  texte = lit_ligne( fichier, 0 );
	  met_chaine( TRANSFORME_GRAF, TG_TRANSFORME_SX, 0, texte, TXT_COPIER );
	  free( texte ); texte = NULL;

	  c = fgetc( fichier );
	  ungetc( c, fichier );		/* Lecture annule pour compatibilit avec ignore_ligne() ensuite */
	} else if ( ( 'y' == c ) || ( 'Y' == c ) ) {
	  /* Il y a une transformation des erreurs en Y ! */
	  I_TRF_ActiversY( +1, graphe );

	  /* Formule de transformation : erreurs en Y */
	  T_TrouverSigneEgal;

	  texte = lit_ligne( fichier, 0 );
	  met_chaine( TRANSFORME_GRAF, TG_TRANSFORME_SY, 0, texte, TXT_COPIER );
	  free( texte ); texte = NULL;

	  c = fgetc( fichier );
	  ungetc( c, fichier );		/* Lecture annule pour compatibilit avec ignore_ligne() ensuite */
	} else {
	  (void) fprintf(stderr, "%s, %s l.%u : Commande inconnue, l. %lu (ignore)\n",
			 __FILE__, __FUNCTION__, __LINE__, numero_ligne);
	  c = ignore_ligne( fichier );
	}
      }
      break;

    case 'x':
    case 'X':			/* Formule de la transformation en X */
      /* Il y a une transformation en X ! */
      I_TRF_ActiverX( +1, graphe );

      /* On cherche le signe = */
      T_TrouverSigneEgal;

      /* On lit la formule qui suit */
      texte = lit_ligne( fichier, 0 );
      met_chaine( TRANSFORME_GRAF, TG_TRANSFORME_X, 0, texte, TXT_COPIER );
      free( texte ); texte = NULL;

      /* Avec lit_ligne, on se trouve au dbut de la ligne suivante */
      c = fgetc( fichier );
      ungetc( c, fichier );		/* Lecture annule pour compatibilit avec ignore_ligne() ensuite */
      break;

    case 'y':
    case 'Y':			/* Formule de la transformation en Y */
      /* Il y a une transformation en Y ! */
      I_TRF_ActiverY( +1, graphe );

      /* On cherche le signe = */
      T_TrouverSigneEgal;

      /* On lit la formule qui suit */
      texte = lit_ligne( fichier, 0 );
      met_chaine( TRANSFORME_GRAF, TG_TRANSFORME_Y, 0, texte, TXT_COPIER );
      free( texte ); texte = NULL;

      /* Avec lit_ligne, on se trouve au dbut de la ligne suivante */
      c = fgetc( fichier );
      ungetc( c, fichier );		/* Lecture annule pour compatibilit avec ignore_ligne() ensuite */
      break;

    default:
      (void) fprintf(stderr, "%s, %s l.%u : Commande ignore [%c = %d], l. %lu\n",
		     __FILE__, __FUNCTION__, __LINE__, c, c, numero_ligne);
      c = ignore_ligne( fichier );
      break;
    }

  } while ( c != EOF );

  /* Termin ici */
  (void) fclose( fichier );
  return 0;
}
