/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Lissage de valeurs

   Lissage par les moindres carrs [modle linaire pour les paramtres]
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "lissage.h"
#include "matrices.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Rsolution "simple" de l'quation des MC -------- */

int MC_Lineaire_Resolution(struct matrice *mat_MDC, struct matrice *v_constant, struct matrice *v_parametres)
{
  int retour;

  /* Contrles */
  if ( ( NULL == mat_MDC ) || ( NULL == v_constant ) || ( NULL == v_parametres ) ) return -1;

  /* Cas simples : 1, 2 inconnues */
  /* 1. Une inconnue : ax = b */
  if ( 1 == mat_MDC->n_lignes ) {
    double coefficient;

    coefficient = mat_MDC->valeurs[ 0 ];
    if ( 0.0 == coefficient ) {
      (void) fprintf(stderr, "%s,%s l.%u : Pas de solution\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }

    /* Rsolution directe */
    v_parametres->valeurs[ 0 ] = v_constant->valeurs[ 0 ] / coefficient;
    return 0;
  }

  /* 2. Deux inconnues : mthode de Cramer [plus stable ?] */
  if ( 2 == mat_MDC->n_lignes ) {
    double determinant, Dx, Dy;

#ifdef _DEVELOPPE_
    (void) printf("%s,%s : Deux quations  deux inconnues\n",  __FILE__, __FUNCTION__);
#endif

    determinant = lit_element( mat_MDC, 0, 0 ) * lit_element( mat_MDC, 1, 1 ) - lit_element( mat_MDC, 0, 1 ) * lit_element( mat_MDC, 1, 0 );
    if ( 0.0 == determinant ) {
      (void) fprintf(stderr, "%s,%s l.%u : Pas de solution, dterminant nul\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }

    Dx = v_constant->valeurs[ 0 ] * lit_element( mat_MDC, 1, 1 ) - v_constant->valeurs[ 1 ] * lit_element( mat_MDC, 0, 1 );
    Dy = lit_element( mat_MDC, 0, 0 ) * v_constant->valeurs[ 1 ] - lit_element( mat_MDC, 1, 0 ) * v_constant->valeurs[ 0 ];

    v_parametres->valeurs[ 0 ] =  Dx / determinant ;
    v_parametres->valeurs[ 1 ] =  Dy / determinant ;
    return 0;
  }

  /* Dcomposition Cholesky de la matrice des moindres carrs */
  if ( MATRICE_SYMETRIQUE == mat_MDC->type ) {
    struct matrice *mat_Cholesky = NULL;

    /* On essaye la mthode de Cholesky, plus efficace */
    mat_Cholesky = MAT_DecompositionCholesky( mat_MDC );
    if ( NULL == mat_Cholesky ) {
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans la dcomposition Cholesky\n",
		     __FILE__, __FUNCTION__, __LINE__);
#endif
    } else {
      int i, j;
      double somme;

      /* Rsolution du systme */
      /* 1. Triangulaire infrieur */
      for ( i = 0 ; i < (int) mat_MDC->n_lignes ; i++ ) {
	somme = v_constant->valeurs[ i ];

	for ( j = i - 1 ; j >= 0 ; j-- ) {
	  somme -= lit_element( mat_Cholesky, i, j ) * v_parametres->valeurs[ j ];
	}
	if ( 0.0 == lit_element( mat_Cholesky, i, i ) ) {
	  (void) fprintf(stderr, "%s,%s l.%u : lment diagonal nul\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  effacer_matrice( mat_Cholesky );
	  return -2;
	}
	v_parametres->valeurs[ i ] = somme / lit_element( mat_Cholesky, i, i );
      }

      /* 2. Triangulaire suprieur */
      for ( i = mat_MDC->n_lignes - 1 ; i >= 0 ; i-- ) {
	somme = v_parametres->valeurs[ i ];

	for ( j = i + 1 ; j < (int) mat_MDC->n_lignes ; j++ ) {
	  somme -= lit_element( mat_Cholesky, j, i ) * v_parametres->valeurs[ j ];
	}
	v_parametres->valeurs[ i ] = somme / lit_element( mat_Cholesky, i, i );
      }

      /* Plus besoin d'elle */
      effacer_matrice( mat_Cholesky );

      /* Tout s'est bien pass */
      return 0;
    }
  }

  /* Si on est l : non-symtrique ou trop instable -> Gauss personnel */
  retour = MAT_ResoudGauss( mat_MDC, v_parametres, v_constant );
  if ( retour != 0 ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans MAT_ResoudGauss() [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
#endif
  }

#ifdef _DEVELOPPE_
/*   { */
/*     unsigned int i; */
/*     printf("%s : Coefficients trouvs = \n( %g", __FUNCTION__, v_parametres->valeurs[ 0 ] ); */
/*     for ( i = 1 ; i < v_parametres->n_lignes ; i++ ) { */
/*       printf(" , %g", v_parametres->valeurs[ i ]); */
/*     } */
/*     printf(" )\n"); */
/*   } */
#endif

  return 0;
}
