/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Lissage de valeurs

   Lissage par un polynome, obtenu par les moindres carrs
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "lissage.h"
#include "matrices.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

double MC_Polynome_PreparationMatrices(unsigned int degre, double *table_X, double *table_Y, unsigned long nbr_points,
				       Matrice *mat_MDC, Matrice *v_constant)
{
  unsigned long i, j;
  double X, Y;
  double Sx = 0.0, norme = 1.0;

#ifdef _DEVELOPPE_
  /* On autorise les matrices plus grandes, pour les modles polynme +... */
  if ( ( mat_MDC->n_lignes <= degre ) || ( mat_MDC->n_colonnes <= degre ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Dimensions de la matrice MDC absurdes [%u  %u, degr %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, mat_MDC->n_lignes, mat_MDC->n_colonnes, degre);
    abort();
  }
  if ( ( v_constant->n_lignes <= degre ) || ( v_constant->n_colonnes != 1 ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Dimensions du vecteur colonne absurdes [%u  %u, degr %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, v_constant->n_lignes, v_constant->n_colonnes, degre);
    abort();
  }
  if ( mat_MDC->type != MATRICE_SYMETRIQUE ) {
    (void) fprintf(stderr, "%s,%s l.%u : la matrice des moindres carrs doit tre symtrique !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    abort();
  }
#endif

  /* On met  zro les deux matrices */
  for ( i = 0 ; i < mat_MDC->n_lignes ; i++ ) {
    for ( j = 0 ; j < mat_MDC->n_colonnes ; j++ ) {
      met_element( mat_MDC, i, j, 0.0 );
    }
  }
  for ( i = 0 ; i < v_constant->n_lignes ; i++ ) {
    v_constant->valeurs[ i ] = 0.0;
  }

  /* Calcul de la somme des X : facteur de normalisation... */
  for ( i = 0, Sx = 0.0 ; i < nbr_points ; i++ ) {
    Sx += table_X[ i ];
  }
  if ( Sx != 0.0 ) {
    norme = nbr_points / Sx;
/*     printf("Facteur de normalisation : %g\n", norme); */
  }

  /* Remplissage : la matrice carre est code par lignes, de toute faon.
     Mais attention, matrice symtrique -> on remplit en colonnes car plus facile !
  */
  /* 1. Premire colonne : n, Sx, Sx2,..., Sxn */
  mat_MDC->valeurs[ 0 ] = (double) nbr_points;
  for ( j = 0 ; j < nbr_points ; j++ ) {
    X = 1.0;
    Y = table_Y[ j ];

    v_constant->valeurs[ 0 ] += Y;
    for ( i = 1 ; i <= degre ; i++ ) {
      X *= ( norme * table_X[ j ] );
      mat_MDC->valeurs[ MAT_SYM_element( i, 0 ) ] += X;
      v_constant->valeurs[ i ] += Y * X;
    }

    /* On continue pour la dernire ligne */
    for ( i = 1 ; i <= degre ; i++ ) {
      X *= ( norme * table_X[ j ] );
      mat_MDC->valeurs[ MAT_SYM_element( degre, i ) ] += X;
    }
  }

#ifdef _DEVELOPPE_
/*   printf("%s : matrice partielle pour un polynme de degr %u =\n", __FUNCTION__, degre ); */
/*   MAT_AfficherMatrice( mat_MDC ); */
#endif

  /* 2. Reste des autres lignes : on remplit en recopiant... */
  for ( i = 1 ; i <= degre ; i++ ) {
    for ( j = i ; j < degre ; j++ ) {
      met_element( mat_MDC, j, i, lit_element( mat_MDC, j + 1, i - 1 ) );
    }
  }

#ifdef _DEVELOPPE_
/*   printf("%s : matrice pour un polynme de degr %u =\n", __FUNCTION__, degre ); */
/*   MAT_AfficherMatrice( mat_MDC ); */

/*   printf("%s : vecteur constant pour un polynme de degr %u =\n", __FUNCTION__, degre ); */
/*   MAT_AfficherMatrice( v_constant ); */
#endif

  /* Calcul termin, la matrice et le vecteur sont prts
     On renvoie le facteur de normalisation ncessaire */
  return norme;
}
