/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996-2002, Emmanuel CURIS
   Cration de ce fichier : 6 septembre 2002

   Routines gnriques pour les ajustements

   Ajustement proprement dit d'un modle quelconque
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "systeme.h"

#include "couches.h"
#include "decomposition.h"
#include "modeles.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Interface gnrale pour lancer la minimisation --------
   Suppose que les variables aient dj t initialises

  modele         = le modle  minimiser
  calcule_modele = la fonction utilise pour calculer le modle dans son ensemble
*/

int MODELE_AjusterModele(ModeleAjustement *modele, int (*calcule_modele)(struct modele_ajuste *))
{
  int retour = +1;
  int (*fct_Increments)(struct modele_ajuste *) = ( int (*)(struct modele_ajuste *) ) NULL;

  if ( NULL == modele ) return -1;

  if ( MODELE_DECOMPOSITION == modele->type ) {
    int retour;

    /* Les routines sont particulires pour ce type de modle ! */
    retour = MODELE_Decomposition_Decomposer( modele->experience, modele->infos_modele.decomposition );

    /* Mais on calcule l'erreur ici, pour les affichages... */
    modele->erreur = MODELE_CalculerErreur( modele, MODELE_ERREUR_CARRES, FAUX, FAUX );
    return retour;
  }

/*   printf("MODELE_AjusterModele :  FAIRE, slection de la routine en fonction de l'estimateur utilis...\n"); */
  if ( (int (*)(struct modele_ajuste *)) NULL == calcule_modele ) {
    /* On essaye de deviner la bonne fonction de calcul */
    switch (modele->type)
    {
    case MODELE_EXAFS_CHEMINS:
      calcule_modele = NC_CalculeEXAFS;
      break;

    case MODELE_GENERIQUE:
      /* vite un appel intermdiaire   MODELE_CalculerModele() : gagne du temps ! */
      calcule_modele = MODELE_Generique_CalculerModele;
      break;

    default:
      calcule_modele = MODELE_CalculerModele;
      break;
    }
  }

  /* Prparation, en fonction de l'algorithme utilis */
  switch (modele->algorithme)
  {
  case ALGO_PSEUDOGRADIENT:
    if ( MODELE_EXAFS_CHEMINS == modele->type )
      fct_Increments = NC_IncrementsPG;
    if ( MODELE_GENERIQUE == modele->type ) {
      fct_Increments = MODELE_Generique_IncrementsPG;
    }

    retour = MODELE_AjustementPG(modele, NULL,
				 ( double (*)(struct modele_ajuste *, double) ) NULL,
				 calcule_modele, fct_Increments, NULL);
    break;

  default:
    (void) fprintf(stderr, "MODELE_AjusterModele : algorithme pas encore support [%d]\n", modele->algorithme);
  }

  /* Tout s'est bien pass, ajustement fini */
  return retour;
}
