/* LASE - Logiciel d'Analyse des Spectres Expérimentaux
   (C) 1996-2006, Emmanuel Curis

   Création et gestion de modèles en vue d'un ajustement

   Ajustement direct d'une droite sur une série de données
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "graphes.h"
#include "lissage.h" /* Pour moindre2() */
#include "modeles.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Réalisation concrète ~~~~~ */

/* -------- Meilleure droite, au sens des moindres carrés --------
    Renvoie 0 si tout s'est bien passé
            != 0 en cas d'erreur (<0 : grave ; >0 : avertissement)

   donnees : les données à modéliser
   modele  : contiendra la droite obtenue [comme graphe ISOLÉ]
   residus : si non-NULL, contiendra les résidus [comme graphe ISOLÉ]
   pente   : contiendra la pente
   o_a_lo  : contiendra l'ordonnée à l'origine
 */

int MODELE_AjusterDroite(struct graphe *donnees, struct graphe **modele, struct graphe **residus, double *pente, double *o_a_lo)
{
  double resultat, coefficients[ 2 ];
  unsigned long i;

  /* Contrôles préliminaires */
  if ( NULL == donnees ) return -1;
  if ( ( NULL == modele ) || ( NULL == pente ) || ( NULL == o_a_lo ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de mémoriser le résultat [paramètre NULL]\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -2;
  }
  if ( FAUX == GRF_EstFonctionReelle( donnees ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Genre de graphe inadapté\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -50;
  }

  /* On appelle les moindres carrés */
  resultat = MC_Polynome_SurPartieGraphe( donnees, 1, 0, NULL, coefficients );
  if ( resultat < 0.0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur survenue dans le calcul [code %g]\n",
		   __FILE__, __FUNCTION__, __LINE__, resultat);
    return -10;
  }

  /* On mémorise les paramètres obtenus */
  *o_a_lo = coefficients[ 0 ];
  *pente  = coefficients[ 1 ];

  /* On construit le modèle */
  *modele = GRF_nv_TableX( NULL, donnees->type, GRF_FCT_REELLE, "Droite des moindres carrés", VRAI, donnees->x, donnees->nombre_point, VRAI );
  if ( NULL == *modele ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de construire le modèle !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return +1;
  }
  for ( i = 0 ; i < donnees->nombre_point ; i++ ) {
    (*modele)->y[ i ] = *o_a_lo + (*modele)->x[ i ] * *pente;
  }
  GRF_bornes( *modele, NE_PAS_FAIRE, DESORDONNE );
  (*modele)->drapeaux.theorique = VRAI;	 /* C'est un modèle théorique ! */

  /* Veut-on aussi le résidu ? [expérience - modèle] */
  if ( residus != NULL ) {
    *residus = GRF_nv_TableX( NULL, donnees->type, GRF_FCT_REELLE, "Droite des moindres carrés", VRAI, donnees->x, donnees->nombre_point, VRAI );
    if ( *residus != NULL ) {
      for ( i = 0 ; i < donnees->nombre_point ; i++ ) {
	(*residus)->y[ i ] = donnees->y[ i ] - (*modele)->y[ i ];
      }
      GRF_bornes( *residus, NE_PAS_FAIRE, DESORDONNE );
    }
  }

  /* Terminé sans souci particulier */
  return 0;
}

