/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Cration et gestion de modles en vue d'un ajustement

   Paramtrage d'un modle (partie gnrique -- interface graphique)
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

/* #include "c_formel.h" /\* Pour les variables *\/ --- inclus par modeles.h */
#include "couches.h"  /* Pour NC_Verifier */
#include "modeles.h"
#include "serie.h"
#include "statistiques.h"	/* Pour gnrer au hasard des valeurs */

#include "modele_validite.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation concrte ~~~~~ */

/* -~-~-~-~-~-~-~-~ Contrle des bornes autorises pour les variables -~-~-~-~-~-~-~-~ */

/* -------- Interface gnrique pour tout type de modle -------- */

int MODELE_ControlerBornes(struct modele_ajuste *modele)
{
  int retour = 0;
  unsigned int i;

  if ( NULL == modele ) return -1;
  if ( NULL == modele->infos_modele.modeles ) {
    (void) fprintf( stderr, "%s,%s l.%u : pas d'information complmentaire sur le modle !\n",
		    __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  switch ( modele->type )
  {
  case MODELE_MULTI_MODELES:
    if ( NULL == modele->infos_modele.modeles) {
      (void) fprintf(stderr, "MODELE_ControlerBornes : pas de table des modles [multi-modles]\n");
      return -3;
    }

    for ( i = 0 ; i < modele->infos_modele.modeles->nbr_modeles ; i++ )
      retour |= MODELE_ControlerBornes( modele->infos_modele.modeles->modeles[ i ] );
    break;

  case MODELE_CL_MODELES:
    if (NULL == modele->infos_modele.cl_modeles) {
      (void) fprintf(stderr, "MODELE_ControlerBornes : pas de table des modles [CL de modles]\n");
      return -3;
    }

    for (i = 0 ; i < modele->infos_modele.cl_modeles->nbr_termes ; i++) {
      retour |= MODELE_ControlerBornes( modele->infos_modele.modeles->modeles[i] );
      if ( 0 != VAR_ControleBornes( &( modele->infos_modele.cl_modeles->coefs[i] ) ) ) {
	int reponse;

	(void) printf("MODELE_ControlerBornes : Bornes atteintes pour le coefficient %u de la combinaison linaire\n", i);
	reponse = BI_ERR_alerte( NULL, &MSG_CoefficientCL, VRAI, i + 1, VAR_LireValeur( &( modele->infos_modele.cl_modeles->coefs[i] ), VARIABLE_LIRE_VALEUR_AJUSTEE ),
				modele->infos_modele.cl_modeles->coefs[i].min, modele->infos_modele.cl_modeles->coefs[i].max);
	if (2 == reponse)
	  return +1;
	retour = +1;
      }
    }
    break;

  case MODELE_EXAFS_CHEMINS:
    retour = NC_Verifier(modele->infos_modele.feff, MODELE_VAL_BORNES);
    break;

  case MODELE_EXAFS_GNXAS:
  case MODELE_SEUIL_BASIQUE:
  case MODELE_FLUOX:
  case MODELE_DECOMPOSITION:
    (void) fprintf(stderr, "MODELE_ControlerBornes : type de modle pas encore support [%d]\n", modele->type);
    break;

  case MODELE_GENERIQUE:
    retour = MODELE_Generique_Verifier( modele->infos_modele.generique, MODELE_VAL_BORNES );
    break;

  default:
    (void) fprintf(stderr, "MODELE_ControlerBornes : type de modle inconnu [%d]\n", modele->type);
    return modele->nbr_variables; /* Suppose que c'est fourni par l'utilisateur... */
  }

  /* On a fini ! */
  return retour;
}

/* -~-~-~-~-~-~-~-~ Contrle du sens physique des valeurs des variables -~-~-~-~-~-~-~-~ */

int MODELE_ControlerPhysique(struct modele_ajuste *modele)
{
  int retour = 0;
  unsigned int i;

  if (NULL == modele) return -1;
  if (NULL == modele->infos_modele.modeles) {
    (void) fprintf(stderr, "MODELE_ControlerPhysique : pas d'informations sur le modle !\n");
    return -2;
  }

  switch (modele->type)
  {
  case MODELE_MULTI_MODELES:
    if (NULL == modele->infos_modele.modeles) {
      (void) fprintf(stderr, "MODELE_ControlerPhysique : pas de table des modles [multi-modles]\n");
      return -3;
    }

    for (i = 0 ; i < modele->infos_modele.modeles->nbr_modeles ; i++)
      retour |= MODELE_ControlerPhysique( modele->infos_modele.modeles->modeles[i] );
    break;

  case MODELE_CL_MODELES:
    if (NULL == modele->infos_modele.cl_modeles) {
      (void) fprintf(stderr, "MODELE_ControlerPhysique : pas de table des modles [CL de modles]\n");
      return -3;
    }

    for (i = 0 ; i < modele->infos_modele.cl_modeles->nbr_termes ; i++) {
      retour |= MODELE_ControlerPhysique( modele->infos_modele.modeles->modeles[i] );
    }
    break;

  case MODELE_EXAFS_CHEMINS:
    retour = NC_Verifier(modele->infos_modele.feff, MODELE_VAL_PHYSIQUE);
    break;

  case MODELE_EXAFS_GNXAS:
  case MODELE_SEUIL_BASIQUE:
  case MODELE_FLUOX:
  case MODELE_DECOMPOSITION:
    (void) fprintf(stderr, "MODELE_ControlerPhysique : type de modle pas encore support [%d]\n", modele->type);
    break;

  default:
    (void) fprintf(stderr, "MODELE_ControlerPhysique : type de modle inconnu [%d]\n", modele->type);
    return modele->nbr_variables; /* Suppose que c'est fourni par l'utilisateur... */
  }

  /* On a fini ! */
  return retour;
}

/* -------- Toutes les operations prealables a une serie d'ajustements... --------
 * Renvoie 0 si tout s'est bien passe
 * Renvoie <0 en cas d'erreur grave
 */

int MODELE_VALIDITE_PreparerModele(struct modele_ajuste *modele)
{
  int retour;

  /* A-t-on bien passe un modele ?  */
  if ( NULL == modele ) return -1;

  /* Ce modele ne doit pas etre en cours d'ajustement */
  if ( VRAI == modele->drapeaux.en_cours ) {
    (void) fprintf( stderr, "%s : ajustement en cours sur le modle choisi !\n",
		    __FUNCTION__ );
    return -2;
  }

  /* On contrle que le modle pass est bien utilisable */
  retour = MODELE_ControlerModele( modele, FAUX );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s : le contrle du modle a chou [code %d].\n",
		    __FUNCTION__, retour);
    return -3;
  }

  /* On prpare l'ajustement du modle considr */
  retour = MODELE_PreparerAjustement( modele, Q_MODELE_ANCIENS_RESULTATS_GARDER );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s : la prparation du modle a chou [code %d].\n",
		    __FUNCTION__, retour );
    return -4;
  }

  /* On initialise le modle */
  retour = MODELE_Initialiser( modele );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s : l'initialisation du modle a chou [code %d].\n",
		    __FUNCTION__, retour );
    MODELE_TerminerAjustement( modele, FAUX );
    return -5;
  }

  /* Tout va bien... */
  return 0;
}

/* -~-~-~-~-~-~-~-~ Etude de l'importance des valeurs initiales -~-~-~-~-~-~-~-~ */

Serie_Graphes *MODELE_VALIDITE_ValeursInitiales_MC(struct modele_ajuste *modele, unsigned long nombre_tirages)
{
  Graphe **graphes_tirages, **graphes_resultats, *graphe_residu;
  Serie_Graphes *serie = NULL;
  double *valeurs_initiales_originales = NULL;
  int retour;
  unsigned long i, j, nombre_parametres = 0;
  int (*fonction_Modele)(struct modele_ajuste *) = ( int (*)(struct modele_ajuste *) ) NULL;

  /* Controles preliminaires */
  if ( 0 == nombre_tirages ) {
    (void) fprintf( stderr, "%s : aucun tirage demand !\n", __FUNCTION__ );
    return NULL;
  }
  retour = MODELE_VALIDITE_PreparerModele( modele );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s : erreur en voulant preparer l'ajustement [code %d]\n",
		    __FUNCTION__, retour );
    return NULL;
  }

  /* Combien de parametres ajustes ? */
  nombre_parametres = MODELE_CompterVariables( modele );
  if ( 0 == nombre_parametres ) {
    (void) fprintf( stderr, "%s/%s l. %u : aucun parametre ajustable dans le modele...\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    MODELE_TerminerAjustement( modele, FAUX );
    return NULL;
  }

  /* On prpare les variables d'ajustement */
  retour = MODELE_InitVariables( modele );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s : l'initialisation du modle a chou [code %d].\n",
		    __FUNCTION__, retour );
    MODELE_TerminerAjustement( modele, FAUX );
    return NULL;
  }

  /* On cre la srie qui contiendra les rsultats */
  serie = SERIE_GRF_CreerSerie( NULL, "[MODELE_VALIDITE_ValeursInitiales_MC]" );
  if ( NULL == serie ) {
    (void) fprintf( stderr, "%s/%s l. %u : impossible de creer la serie des resultats...\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    MODELE_TerminerAjustement( modele, FAUX );
    return NULL;
  }

  /* On mmorise les valeurs initiales choisies par l'utilisateur */
  valeurs_initiales_originales = (double *) malloc( sizeof( double ) * nombre_parametres );
  if ( NULL == valeurs_initiales_originales ) {
    ERR_Memoire( "Table des valeurs initiales originales", sizeof( double ) * nombre_parametres );
    SERIE_GRF_EffacerSerie( serie, VRAI );
    MODELE_TerminerAjustement( modele, FAUX );
    return NULL;
  }
  for ( i = 0 ; i < nombre_parametres ; i++ ) {
    valeurs_initiales_originales[ i ] = MODELE_ValeurVariable( modele, i, VARIABLE_LIRE_VALEUR_INITIALE );
  }

  /* On cre les graphes correspondants */
  /* a. Les rsidus */
  graphe_residu = Serie_nv_graphe( serie, NULL, nombre_tirages, GR_EPREUVE, GRF_SERIE,
				   "Variation des valeurs initiales :  r"e_AIGU"sidus des ajustements" );
  if ( NULL == graphe_residu ) {
    (void) fprintf( stderr, "%s/%s l. %u : impossible de crer le graphe des rsidus...\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    free( valeurs_initiales_originales ); valeurs_initiales_originales = NULL;
    SERIE_GRF_EffacerSerie( serie, VRAI );
    MODELE_TerminerAjustement( modele, FAUX );    
    return NULL;
  }
  graphe_residu->drapeaux.protege = VRAI;

  /* b. Les rsultats des ajustements */
  graphes_resultats = (Graphe **) malloc( sizeof( Graphe * ) * nombre_parametres );
  if ( NULL == graphes_resultats ) {
    ERR_Memoire( "Table des graphes des r"e_AIGU"sultats", sizeof( Graphe * ) * nombre_parametres );
    GRF_efface( graphe_residu, VRAI, VRAI );
    SERIE_GRF_EffacerSerie( serie, VRAI );
    free( valeurs_initiales_originales ); valeurs_initiales_originales = NULL;
    MODELE_TerminerAjustement( modele, FAUX );
    return NULL;
  }
  graphes_tirages = (Graphe **) malloc( sizeof( Graphe * ) * nombre_parametres );
  if ( NULL == graphes_resultats ) {
    ERR_Memoire( "Table des graphes des r"e_AIGU"sultats", sizeof( Graphe * ) * nombre_parametres );
    GRF_efface( graphe_residu, VRAI, VRAI );
    SERIE_GRF_EffacerSerie( serie, VRAI );
    free( valeurs_initiales_originales ); valeurs_initiales_originales = NULL;
    MODELE_TerminerAjustement( modele, FAUX );
    free( graphes_resultats ); graphes_resultats = NULL;
    return NULL;
  }
  for ( i = 0 ; i < nombre_parametres ; i++ ) {
    char *titre, *nom_variable, *unite;

    nom_variable = MODELE_InfosVariable( modele, i );

    titre = TXT_genere_chaine( "R"o_CIRC"le des valeurs initiales : var. %u [%s] (Valeurs finales)",
			       (i + 1), nom_variable );
    graphes_resultats[ i ] = Serie_nv_graphe( serie, (i > 0) ? (graphes_resultats[i - 1]) : graphe_residu, nombre_tirages,
					      GR_EPREUVE, GRF_SERIE, titre );
    free( titre ); titre = NULL;

    titre = TXT_genere_chaine( "R"o_CIRC"le des valeurs initiales : var. %u [%s] (Valeurs tir"e_AIGU"es)",
			       (i + 1), nom_variable );
    graphes_tirages[ i ] = Serie_nv_graphe( serie, (i > 0) ? (graphes_tirages[i - 1]) : graphe_residu, nombre_tirages,
					    GR_EPREUVE, GRF_SERIE, titre );
    free( titre ); titre = NULL;

    GRF_ChangerLegende( graphes_resultats[ i ], AXE_Y, nom_variable, VRAI );
    GRF_ChangerLegende( graphes_tirages[ i ], AXE_Y, nom_variable, VRAI );
    free( nom_variable ); nom_variable = NULL;

    if ( ( NULL == graphes_resultats[ i ] ) || ( NULL == graphes_tirages[ i ] ) ) {
      unsigned int j;

      (void) fprintf( stderr, "%s : impossible de gnrer les graphes de valeurs (variable %lu) !\n",
		      __FUNCTION__, i );
      for ( j = 0 ; j < i ; j++ ) {
	GRF_efface( graphes_resultats[ j ], VRAI, VRAI );
	GRF_efface( graphes_tirages[ j ], VRAI, VRAI );
      }
      GRF_efface( graphe_residu, VRAI, VRAI );
      free( graphes_resultats ); graphes_resultats = NULL;
      free( graphes_tirages ); graphes_tirages = NULL;
      MODELE_TerminerAjustement(modele, FAUX);
      return NULL;
    }

    /* On met des bornes en Y provisoires, pour les affichages [fondes sur les valeurs autorisees] */
    graphes_tirages[ i ]->y_min =  MODELE_ValeurVariable( modele, i, VARIABLE_LIRE_VALEUR_MINIMALE );
    graphes_tirages[ i ]->y_max =  MODELE_ValeurVariable( modele, i, VARIABLE_LIRE_VALEUR_MAXIMALE );

    graphes_resultats[ i ]->y_min = graphes_tirages[ i ]->y_min;
    graphes_resultats[ i ]->y_max = graphes_tirages[ i ]->y_max;
    /* On met les bonnes units en Y */
    unite = MODELE_UniteVariable( modele, i );
    GRF_ChangerUnite( graphes_resultats[ i ], AXE_Y, unite ) ;
    GRF_ChangerUnite( graphes_tirages[ i ], AXE_Y, unite ) ;
    free( unite ); unite = NULL;

    /* Le graphe est protg pendant les calculs */
    graphes_resultats[ i ]->drapeaux.protege = VRAI;
    graphes_tirages[ i ]->drapeaux.protege = VRAI;
  }

  /* 3. La srie des valeurs est protge... */
  serie->drapeaux.protegee = VRAI;

  switch ( modele->type )
  {
  case MODELE_EXAFS_CHEMINS:
    fonction_Modele = NC_CalculeEXAFS;
    break;

  case MODELE_GENERIQUE:
    fonction_Modele = MODELE_Generique_CalculerModele;
    break;

  default:
    fonction_Modele = MODELE_CalculerModele;
    break;
  }

  /* On peut faire les tirages et les ajustements... */
  for ( i = 0 ; i < nombre_tirages ; i++ ) {
    /* 1. Tirage au sort des valeurs initiales :
          lois uniformes dans l'intervalle de variation utilise */
    for ( j = 0 ; j < nombre_parametres ; j++ ) {
      double p_min, p_max, valeur_tiree;

      /* L'intervalle autorise pour ce parametre */
      p_min = graphes_tirages[ j ]->y_min;
      p_max = graphes_tirages[ j ]->y_max;

      /* On tire au sort une valeur dans cet intervalle : loi uniforme */
      valeur_tiree = STAT_NombreAleatoire_UniformeC( p_min, p_max );

      /* On la memorise... */
      graphes_tirages[ j ]->y[ i ] = valeur_tiree;
      MODELE_FixerValeurVariable( modele, j, VARIABLE_LIRE_VALEUR_INITIALE, valeur_tiree );
      MODELE_ReinitialiseVariable( modele, j ); /* Et on part de cette valeur... */
    }

    /* On peut faire l'ajustement : meme reference, memes conditions... */
    modele->iteration = 0; /* On recommence un tour...
			      Sinon, on risque de dpasser le maximum d'itrations autoris (100 000) */
    MODELE_AjusterModele( modele, fonction_Modele );

    /* Ajustement termine : on memorise les resultats */
    graphe_residu->y[ i ] = modele->erreur;
    for ( j = 0 ; j < nombre_parametres ; j++ ) {
      graphes_resultats[ j ]->y[ i ] = MODELE_ValeurVariable( modele, j, VARIABLE_LIRE_VALEUR_AJUSTEE );
    }
  }

  /* Ici, tous les tirages ont eu lieu... */
  /* a. On remet les valeurs initiales originales */
  for ( j = 0 ; j < nombre_parametres ; j++ ) {
    MODELE_FixerValeurVariable( modele, j, VARIABLE_LIRE_VALEUR_INITIALE, valeurs_initiales_originales[ j ] );
  }
  free( valeurs_initiales_originales ); valeurs_initiales_originales = NULL;

  /* b. On adapte les bornes des graphes de resultats */
  GRF_bornes( graphe_residu, NE_PAS_FAIRE, DESORDONNE );
  for ( j = 0 ; j < nombre_parametres ; j++ ) {
    GRF_bornes( graphes_tirages[ j ], NE_PAS_FAIRE, DESORDONNE );
    GRF_bornes( graphes_resultats[ j ], NE_PAS_FAIRE, DESORDONNE );

    graphes_tirages[ j ]->drapeaux.protege = FAUX;
    graphes_resultats[ j ]->drapeaux.protege = FAUX;
  }

  /* Termine */
  serie->drapeaux.protegee = FAUX;
  graphe_residu->drapeaux.protege = FAUX;

  free( graphes_resultats ); graphes_resultats = NULL;
  free( graphes_tirages ); graphes_tirages = NULL;
  MODELE_TerminerAjustement( modele, FAUX );
  return serie;
}
