/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2002, Emmanuel CURIS
   Cration              : 5 juillet 2002

   Module d'ajustement par un modle  n couches

   Arrt de l'ajustement (normal ou anticip) et actions subsquentes
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "systeme.h"

/* #include "exafs.h" */
#include "graphes.h"
/* #include "ERREURS.H" */

#include "atomes.h"   /* Pour les absorbeurs */
#include "c_formel.h" /* Pour les variables & l'historique associ */
#include "couches.h"
#include "feff.h"     /* Pour le modle FEFF */
#include "filtrage.h" /* Pour les infos Nyquist */
#include "modeles.h"  /* Pour les modles */
#include "tracer.h"

/* #ifdef _ENGLISH_ */
/* # include "nc_arret.uk" */
/* #else */
/* # include "nc_arret.fr" */
/* #endif */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

void HIST_NC_AfficherResultats(struct modele_feff *modele);

/* -------- On n'utilise plus le modle indiqu pour l'ajustement */

int NC_LibererModele(struct modele_feff *modele, boolean eff_coef_splines)
{
  Couche *couche;

  if (NULL == modele) return -1;

  for ( couche = modele->couches ; couche != NULL ; couche = couche->suivante ) {
    if ( VRAI == eff_coef_splines ) {
      /* Libration des coefficients de la spline / amplitude */
      free(couche->b_amp); couche->b_amp = NULL;
      free(couche->c_amp); couche->c_amp = NULL;
      free(couche->d_amp); couche->d_amp = NULL;

      /* Libration des coefficients de la spline / dphasage */
      free(couche->b_pha); couche->b_pha = NULL;
      free(couche->c_pha); couche->c_pha = NULL;
      free(couche->d_pha); couche->d_pha = NULL;
    }

    /* Dprotection des graphes */
    if (couche->amplitude != NULL) 
      couche->amplitude->drapeaux.protege = FAUX;
    if (couche->phase != NULL)
      couche->phase->drapeaux.protege = FAUX;
  }

  if ( VRAI == eff_coef_splines ) {
    /* Libration des coefficients de la spline / libre parcours moyen */
    free(modele->b_lpm); modele->b_lpm = NULL;
    free(modele->c_lpm); modele->c_lpm = NULL;
    free(modele->d_lpm); modele->d_lpm = NULL;
  }

  /* Dprotection des graphes */
  if (modele->lpm != NULL)
    modele->lpm->drapeaux.protege = FAUX;
  if (modele->phase_central != NULL)
    modele->phase_central->drapeaux.protege = FAUX;

  /* Tout s'est bien pass */
  return 0;
}

void HIST_NC_AfficherResultats(struct modele_feff *modele)
{
  Couche *couche;

  if (NULL == modele) return;

  HIST_Titre(2, "Valeurs des variables "a_GRAVE" l'issue de l'ajustement", "Ajustement");

  /* Variables gnrales */
  HIST_VAR_ValeurVariable( &(modele->S0_2), "S0"DEUX_EXPOSANT);
  if (VRAI == modele->drapeaux.DE0_general) {
    HIST_VAR_ValeurVariable( &(modele->Delta_E), DELTA_MAJUSCULE"E_0 (eV)");
  }
  if (VRAI == modele->drapeaux.GAM_general) {
    HIST_VAR_ValeurVariable( &(modele->Gamma), GAMMA_MAJUSCULE" ("ANGSTROM"^-1)");
  }

  /* Variables des couches */
  for (couche = modele->couches ; couche != NULL ; couche = couche->suivante) {
    if (FAUX == couche->utilisee)
      continue;

    /* La couche est effectivement utilise */ 
    HIST_Titre(3, "Couche", couche->nom);
    HIST_VAR_ValeurVariable( &(couche->N), "Nombre de voisins ");
    HIST_VAR_ValeurVariable( &(couche->R), "Longueur de chemin");
    HIST_VAR_ValeurVariable( &(couche->Debye_Waller), "Debye-Waller");
    HIST_VAR_ValeurVariable( &(couche->Cumul_1), "Premier cumulant");
    HIST_VAR_ValeurVariable( &(couche->Cumul_2), "Deuxi"e_GRAVE"me cumulant");
    HIST_VAR_ValeurVariable( &(couche->Cumul_3), "Troisi"e_GRAVE"me cumulant");
    HIST_VAR_ValeurVariable( &(couche->Cumul_4), "Quatri"e_GRAVE"me cumulant");
    if (FAUX == modele->drapeaux.DE0_general) {
      HIST_VAR_ValeurVariable( &(couche->Delta_E), DELTA_MAJUSCULE"E_0 (eV)");
    }
    if (FAUX == modele->drapeaux.GAM_general) {
      HIST_VAR_ValeurVariable( &(couche->Gamma), GAMMA_MAJUSCULE" ("ANGSTROM"^-1)");
    }

    /* Pour contrle, seulement si historique complet... */
    HIST_Texte(5, 0, "D"e_AIGU"phasage : %s\n", couche->phase->nom);
    HIST_Texte(5, 0, "Amplitude : %s\n", couche->amplitude->nom);
  }
}
