/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2002, Emmanuel CURIS
   Ce fichier : cration le 10 aot 2002

   Module d'ajustement par un modle  n couches

   Slection directe d'une couche parmi l'ensemble des couches du modle
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "couches.h"
#include "feff.h"

#include "couches_choisir.txt"

/* ~~~~~~~~ Variables locales  ce fichier ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales  ce fichier ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique des diffrentes fonctions ~~~~~~~~ */

Couche *COUCHE_Choisir(struct modele_feff *modele, const char *titre, int arbre, int objet)
{
  Couche *couche;
  const char **noms;
  int reponse;
  unsigned int i, nv_couche;

  if (NULL == modele) return NULL;
  if (NULL == modele->couches) {
    (void) fprintf(stderr, "COUCHE_Choisir : le modele ne contient aucune couche !\n");
    return NULL;
  }

  if ( 1 == modele->nbr_couches ) {
    (void) fprintf(stderr, "COUCHE_Choisir : le modle ne contient qu'une seule couche !\n");
    return modele->couches;
  }

  if ( NULL == titre ) {
    titre = txt_titre_par_defaut;
  }

  /* On prpare la table des noms des couches */
  noms = (const char **) malloc( sizeof(char *) * (modele->nbr_couches) );
  if (NULL == noms) {
    (void) fprintf(stderr, "COUCHE_Choisir : impossible de rserver la table des noms !\n");
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "N_couches/choisir.c", "COUCHE_Choisir", "noms");
    return modele->couches;
  }

  for (i = 0, couche = modele->couches ; i < modele->nbr_couches ; i++, couche = couche->suivante) {
    if (NULL == couche) {
      (void) fprintf(stderr, "COUCHE_Choisir : ERREUR, %u indiques dans le modle, il n'y en a que %u !\n",
		     modele->nbr_couches, i);
#ifdef _DEVELOPPE_
      /* On arrte tout pour chercher l'erreur ! */
      abort();
#else
      /* On essaye de corriger, mais assez sale... */
      modele->nbr_couches = i;
#endif
      break;
    }

    noms[i] = couche->nom;
  }

  /* On appelle le sous-menu pour choisir la couche */
  reponse = SS_menu_texte(noms, modele->nbr_couches, titre, arbre, objet);
  free(noms);
  if (reponse < 0) return NULL; /* On annule le changement */
  nv_couche= (unsigned int) reponse;

  for (i = 0, couche = modele->couches ; i < nv_couche ; couche = couche->suivante, i++);

  return couche;
}
