/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996-2001, Emmanuel CURIS
   Cration du fichier : 5 mars 1999

   Oprations gnriques sur les graphes
   Recalage d'un graphe sur les X ou les Y
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
#include "exafs.h"

#include "operations.h"

/* ~~~~~~~~ variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ ralisation pratique ~~~~~ */

/* -------- Recaler un graphe en ordonnes -------------------\
 |  Renvoie le graphe recal (NULL si erreur)                 |
 |  Ne fonctionne pas sur les graphes complexes               |
 |                                                            |
 | graphe = le graphe qu'il faut recaler                      |
 | valeur = la valeur que doit avoir la moyenne des y         |
 |    on ralise ensuite nv_y = y + (valeur - moyenne_y)      |
 | nouveau = VRAI s'il faut crer un nouveau graphe           |
 | err_valeur = erreur connue sur la nouvelle valeur          |
 \----------------------------------------------------------- */

Graphe *GRF_recale_y(Graphe *graphe, double valeur, boolean nouveau, double err_valeur)
{
  Graphe *resultat = NULL;
  char *titre;
  unsigned long i;
  double moyenne_y = 0.0;
  register double *y;

  if (graphe == NULL) return NULL;

  if (nouveau == VRAI) {
    resultat = GRF_Copier( graphe, NULL, FAUX );
    if (resultat == NULL)
      return NULL;
  } else
    resultat = graphe;

  titre = TXT_genere_chaine( "%s recal"e_AIGU, graphe->nom );
  GRF_titre( resultat, titre );
  free( titre ); titre = NULL;

  /* Calcul de la valeur moyenne */
  for (i = 0, y = graphe->y ; i < graphe->nombre_point ; i++, y++)
    moyenne_y += *y;
  moyenne_y /= graphe->nombre_point;

  HIST_Texte(1, 0, "Graphe recal"e_AIGU" : %s\n", graphe->nom);
  HIST_Texte(5, 2, "D"e_AIGU"calage utilis"e_AIGU" : %.10g\n", valeur - moyenne_y);

  if (moyenne_y == valeur) return resultat; /* Pas de recalage  faire ! */

  moyenne_y = valeur - moyenne_y; /* Incrment  utiliser */

  /* Recalage effectif */
  for (i = 0, y = resultat->y ; i < resultat->nombre_point ; i++, y++)
    *y += moyenne_y;

  resultat->y_min += moyenne_y;
  resultat->y_max += moyenne_y;

  if (err_valeur <= 0.0) /* Pas d'erreur -> les barres d'erreur ne sont pas modifies */
    return resultat;

  printf("GRF_recale_y :  FAIRE, cas d'une erreur connue [%g] sur la nouvelle valeur...\n", err_valeur);
  /* Les erreurs ne sont pas modifies, puisque l'on ajoute une constante !
     Il faudrait voir ventuellement  estimer l'erreur sur la moyenne ? */

  return resultat;
}
/* -------- Recaler un graphe en ordonnes -------------------\
 |  Renvoie le graphe recal (NULL si erreur)                 |
 |  Ne fonctionne pas sur les graphes complexes               |
 |                                                            |
 | graphe = le graphe qu'il faut recaler                      |
 | valeur = la valeur que doit avoir la moyenne des y         |
 |    on ralise ensuite nv_y = y * (valeur / moyenne_y)      |
 | nouveau = VRAI s'il faut crer un nouveau graphe           |
 | err_valeur = erreur connue sur la nouvelle valeur          |
 \----------------------------------------------------------- */

Graphe *GRF_mrecale_y(Graphe *graphe, double valeur, boolean nouveau, double err_valeur)
{
  Graphe *resultat = NULL;
  char *titre;
  unsigned long i;
  double moyenne_y = 0.0;
  register double *y;

  if (graphe == NULL) return NULL;

  if (nouveau == VRAI) {
    resultat = GRF_Copier( graphe, NULL, FAUX );
    if (resultat == NULL)
      return NULL;
  } else
    resultat = graphe;

  titre = TXT_genere_chaine( "%s recal"e_AIGU, graphe->nom );
  GRF_titre(resultat, titre);
  free( titre ); titre = NULL;

  /* Calcul de la valeur moyenne */
  for (i = 0, y = graphe->y ; i < graphe->nombre_point ; i++, y++)
    moyenne_y += *y;
  moyenne_y /= graphe->nombre_point;

  if (moyenne_y == 0.0) {
    EXF_ERREUR(DIV_ZERO, "Moyenne nulle", txt_err_calc_annule);
    return NULL;
  }

  HIST_Texte(1, 0, "Graphe recal"e_AIGU" : %s\n", graphe->nom);
  HIST_Texte(5, 2, "Facteur utilis"e_AIGU" : %.10g\n", valeur/moyenne_y);

  if (moyenne_y == valeur) return resultat; /* Pas de recalage  faire ! */

  moyenne_y = valeur/moyenne_y; /* Coefficient  utiliser */

  /* Recalage effectif */
  GRF_FacteurMultiplicatif( resultat, moyenne_y, AXE_Y, NULL ); /* S'occupe des erreurs et tout et tout */

  if (err_valeur <= 0.0) /* Pas d'erreur -> les barres d'erreur ont dj t modifies */
    return resultat;

  printf("GRF_mrecale_y :  FAIRE, cas d'une erreur connue [%g] sur la nouvelle valeur...\n", err_valeur);

  return resultat;
}
