/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 2006, Emmanuel CURIS
   Cration : 22 septembre 2006

   Calculs statistiques divers

   Estimation d'une fonction de rpartition empirique [et outils autour...]
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"
#include "utils_tables.h"

#include "graphes.h"
#include "proba.h"
#include "statistiques.h"
#include "tri.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Estimation de la fonction de rpartition empirique d'une srie de valeurs -------- */

Graphe *STAT_FdREmpirique(Graphe *valeurs, unsigned int methode, unsigned int options)
{
  Graphe *fdr;
  double facteur;
  unsigned long i;

  if ( NULL == valeurs ) return NULL;
  /* Le contrle du genre est  la charge de l'appelant ! */

  /* La FDR est un graphe isole, qui a en X les valeurs */
  fdr = GRF_nv_TableX( NULL, GR_REPARTITION, GRF_FCT_REELLE, "Fonction de r"e_AIGU"partition", VRAI,
		       valeurs->y, valeurs->nombre_point, VRAI );
  if ( NULL == fdr ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de prparer la fonction de rpartition\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On doit donc maintenant classer les X par ordre croissant ; tri bte */
  TABLE_TrierTable( fdr->x, fdr->nombre_point, 1 );

  /* On remplit la table des Y avec les rangs */
  for ( i = 0 ; i < fdr->nombre_point ; i++ ) {
    fdr->y[ i ] = (double) (i + 1);
  }

  OPTION_A_FAIRE( "Vrifier ce qu'il se passe en cas de doublons !" );
#ifdef a_voir
  /* On supprime les doublons [brutal car aucune valeur utile actuellement] */
  GRF_TraiterDoublons( fdr, GRF_DOUBLONS_SIMPLE, 0, FAUX, FAUX );
#endif

  /* On prpare pour les incertitudes */
  if ( options & STAT_FDR_EMPIRIQUE_AVEC_ERREURS ) {
    OPTION_A_FAIRE( "Intgrer l'estimation de l'incertitude..." );
  }

  /* On multiplie par le terme correctif, en fonction de la mthode */
  switch ( methode )
  {
  case STAT_FDR_EMPIRIQUE_BASE:
    facteur = 1.0 / (double) (valeurs->nombre_point); 
    break;

  case STAT_FDR_EMPIRIQUE_NP1:
    /* Correction de Wald [?] pour ne pas perdre le dernier point dans la droite de Henry */
    facteur = 1.0 / (double) (valeurs->nombre_point + 1);
    break;

  default:
    AVERTISSEMENT( "Mthode inconnue [%d]", methode );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    GRF_eff_isole( fdr ); fdr = NULL;
    return NULL;
  }

  /* On fait le calcul */
  GRF_FacteurMultiplicatif( fdr, facteur, AXE_Y, NULL );

  /* De toute faon, une fonction de rpartition est comprise entre 0 et 1 */
  fdr->y_min = 0.0;
  fdr->y_max = 1.0;

  /* On a termin, sans souci */
  return fdr;
}

/* -------- Construction de la droite de Henry pour une srie de valeurs -------- */

Graphe *STAT_DroiteDeHenry(Graphe *valeurs, unsigned int methode)
{
  Graphe *resultat;
  double fractile;
  unsigned long i;

  /* On construit la fonction de rpartition empirique
     Contrle aussi que non-NULL & ce genre de choses... */
  resultat = STAT_FdREmpirique( valeurs, methode, 0 );
  if ( NULL == resultat ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la fonction de rpartition empirique\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Pour chaque valeur en Y, qui est F_X(x_i), on calcule le fractile d'une gaussienne centre rduite */
  for ( i = 0 ; i < resultat->nombre_point ; i++ ) {
    fractile = PROBA_Fractile_Gauss( resultat->y[ i ] );
    resultat->y[ i ] = fractile;
  }

  /* On recalcule les bornes en Y ; normalement, croissants */
  GRF_bornes( resultat, NE_PAS_FAIRE, ORDONNE );

  /* Termin */
  return resultat;
}
