/* LASE - Logiciel d'Analyse des Spectres EXAFS
   Copyright (C) 1996 - 1998 Emmanuel CURIS
   Cration du fichier : 6 novembre 1998

   Module de calculs statistiques

   Fusion de plusieurs spectres pour en faire un seul
   Suppose les spectres originaux tris par X croissants
   Le spectre final est tri suivant les X croissants
   Les doublons peuvent tre ignors, limins ou moyenns.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "serie.h"

#include "statistiques.h"
#include "tri.h"

#include "fusion.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

typedef struct {
  unsigned long index_dep;
  boolean *a_fondre;
  Graphe *prem_graphe;
} inf_asc;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* Pas static car dfinies dans moyennes.c : utilises aussi pour les moyennes */

boolean gere_arbre_fusion(unsigned int objet, int x, int y);
void    gere_asc_fusion  (inf_asc *infos, int objet, int y);
int     init_asc_fusion  (inf_asc *infos);
void    prep_arbre_fusion(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------------- Fusion de n graphes dj en mmoire (appel) ---------------------------- */

/* ~~~~~~~~ Interfaage des routines ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* -------- Fusion : procdure d'appel, avec arbre GEM et tout et tout ------------\
 |   Renvoie le graphe contenant la fusion de tous les graphes choisis             |
 |                                                                                 |
 | Cette fonction permet  l'utilisateur de crer une liste de graphes  fusionner |
 | Il y a quelques contrles sur la cohrence des graphes choisis, mais ce n'est   |
 | pas encore parfait ! Il faudrait amliorer a...                                |
 \-------------------------------------------------------------------------------- */

Graphe *fondre(void)
{
  Graphe *resultat,parametres;
  TRI_infos *infos_tri, fusion = { NULL, NULL, 0, 0, 0, NULL, NULL };
  char *titre;
  int renvoi;
  unsigned long total_points = 0;
  unsigned long i, compteur;
  unsigned long nbr_a_fondre = 0; /* Nombre de graphes  fusionner */
  inf_asc infos_ascenseur;

  /* Prparation de l'ascenseur */
  if (init_asc_fusion(&infos_ascenseur) != VRAI)
    return NULL;

  met_chaine(FUSION_GRAPHES, FUSION_OK, 0, txt_fusionner, TXT_REMPLACE);
  OBJ_cache(FUSION_GRAPHES, FUS_DOUBLE_NOM, FORCE_NON);
  OBJ_cache(FUSION_GRAPHES, FUS_DOUBLE_FOND, FORCE_NON);

  do {
    renvoi = ARB_Bloquant( FUSION_GRAPHES, titre_arbre_fusion, principale, 0 );
    if (renvoi == FUSION_ANNULE) { /* On annule la fusion */
      free(infos_ascenseur.a_fondre); /* On libre la mmoire */
      return NULL;
    }
  } while (renvoi != FUSION_OK);

  for (i = 0 ; i < serie_courante->nbr_graphes ; i++) {
    if ( (infos_ascenseur.a_fondre)[i] == VRAI)
      nbr_a_fondre++;
  }

  if (nbr_a_fondre < 2) {
    form_alert(1, txt_erreur_2graphes);
    free(infos_ascenseur.a_fondre); /* On libre la mmoire */
    return NULL;
  }

  infos_tri = (TRI_infos *) malloc( sizeof(TRI_infos) * nbr_a_fondre );
  if (infos_tri == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "fusion.c", "fondre", "infos_tri");
    free(infos_ascenseur.a_fondre); /* On libre la mmoire */
    return NULL;
  }

  /* On prpare les blocs  trier */
  resultat = serie_courante->premier_graphe;
  compteur = 0; /* Les graphes slectionns */
  i = 0;
  while ( (infos_ascenseur.a_fondre)[i] == FAUX) { /* Recherche du premier graphe  fondre */
    i++;
    resultat = resultat->suivant;
    if (resultat == NULL) { /* Dernier graphe */
      form_alert(1, txt_erreur_0graphe);
      free(infos_ascenseur.a_fondre); /* On libre la mmoire */
      free(infos_tri);
      return NULL;
    }
  }
  parametres.x_min = resultat->x_min; parametres.x_max = resultat->x_max;
  parametres.y_min = resultat->y_min; parametres.y_max = resultat->y_max;
  parametres.type = resultat->type;
  parametres.genre_donnees = resultat->genre_donnees;
  OPTION_A_FAIRE( "Adaptation des lgendes en X, en Y" );

  HIST_Titre(1, "Fusion de graphes", "Stats");
  HIST_Texte(1, 0, "Graphes pris en compte :\n");
  HIST_Texte(2, 1, "  %s\n", resultat->nom);

  while ( i < serie_courante->nbr_graphes ) {
    if ( (infos_ascenseur.a_fondre)[i] == VRAI) {
      total_points += resultat->nombre_point; /* Nombre de points du total */
      infos_tri[ compteur ].x = resultat->x;
      infos_tri[ compteur ].y = resultat->y;
      infos_tri[ compteur ].position   = 0;
      infos_tri[ compteur ].nbr_points = resultat->nombre_point;
      compteur++;
      
      if (parametres.x_min > resultat->x_min) parametres.x_min = resultat->x_min;
      if (parametres.x_max < resultat->x_max) parametres.x_max = resultat->x_max;
      if (parametres.y_min > resultat->y_min) parametres.y_min = resultat->y_min;
      if (parametres.y_max < resultat->y_max) parametres.y_max = resultat->y_max;

      if ( resultat->genre_donnees != parametres.genre_donnees ) {
	(void) fprintf(stderr, "Fusion de graphes complexes et rels impossible. Opration annule\n");
	free(infos_ascenseur.a_fondre);
	free(infos_tri);
	return NULL;
      }

      if ( ( parametres.type != resultat->type ) && (resultat->type != -1) ) {
	if (parametres.type == -1) 
	  parametres.type = resultat->type;
	else {
	  renvoi = form_alert(1, txt_erreur_type_different);
	  if (renvoi == 2) {
	    free(infos_ascenseur.a_fondre); /* On libre la mmoire */
	    free(infos_tri);
	    return NULL;
	  }
	}
      }

      HIST_Texte(2, 1, "  %s\n", resultat->nom);
    }
    resultat = resultat->suivant;
    i++;
  }
  free(infos_ascenseur.a_fondre); /* On libre la mmoire */

  titre = TXT_genere_chaine(txt_titre_graphe_fusion, nbr_a_fondre);
  resultat = GRF_nouveau(NULL, total_points, parametres.type, parametres.genre_donnees, titre);
  free(titre);
  if (resultat == NULL) {
    free(infos_tri);
    return NULL;
  }

  HIST_Texte(1, 0, "R"e_AIGU"sultat : %s\n", resultat->nom);

  fusion.x = resultat->x;
  fusion.y = resultat->y;
  fusion.nbr_points = total_points;
  TRI_Blocs( &fusion, infos_tri, nbr_a_fondre );

  /* Reste l'limination des doublons */
  if ( OBJ_choisit(FUSION_GRAPHES, FUS_GARDER, QUESTION) == FAUX ) {
    compteur = GRF_TraiterDoublons( resultat, GRF_DOUBLONS_SIMPLE, 0.0, 
				    OBJ_choisit(FUSION_GRAPHES, FUS_MOYENNE, QUESTION), VRAI );

    if (compteur > 0) { /* Des doublons => les bornes calcules ne conviennent pas */
      /* On met  jour les bornes en X et en Y */

      parametres.x_min = resultat->x_min;
      parametres.x_max = resultat->x_max;
      parametres.y_min = resultat->y_min;
      parametres.y_max = resultat->y_max;
    }
  }

  /* On assigne les bornes au graphe fusion */
  resultat->x_min = parametres.x_min;
  resultat->x_max = parametres.x_max;
  resultat->y_min = parametres.y_min;
  resultat->y_max = parametres.y_max;

  free(infos_tri);
  return resultat;
}
