/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
   Tout ce qui concerne la superposition de divers graphes, ainsi que leur trac...
   (c) Emmanuel CURIS 1996 - 1998

   16, 26 mars 1998
     * Utilisation des stations VDI avances (Station_VDI) -> plus portable [j'espre...]
     * Remplac _tous_ les vdi_handle par _PRG_station->handle
     * Inclus "erreurs.h" pour prototypes de isnan() et isinf()

   6, 12 & 13, 19 janvier 1998
     * Corrig RCT_trace : plantait si rect == NULL (aucun rectangle) (sur Falcon en tout cas)
     * Dans TRC_poly, utilis _PRG_station->handle au lieu de vdi_handle
     * Impression des graphes superposs : premier essai - pour voir ! a tourne !
     * Modifi le protocole d'appel de l'export d'images
     * Inclus export.h

  24 - 28 novembre 1997
     * Ajout l'affichage des coordonnes dans la fentre de trac des graphes
     * Complt le paramtrage des lgendes : texte, style de grille, taille de fonte, unit
     * Corrig erreur de calcul des y dans CLC_rect (le calcul des bornes choisies par un rectangle)
     * Corrig le calcul de lieu_centre -> vite la succession des 2 qd on clic sur le bord bas
     * Modifi TRC_poly() : ne trace que si le point apparat dans la zone indique en paramtres
     * Commenc le squelette de RCT_trace (affichage d'un pointeur sur des graphes superposs)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "serie.h"

#include "exafs.h"
#include "interface.h" /* Pour USR_... */ 
#include "fichiers.h"  /* Pour export PS */

#include "tracer.h"

#ifdef _TOS_ /* L'impression ne marche que sous TOS actuellement... */
# include "imprime.h"
#endif

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ------ Cration et modification d'un cadre ------ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ----------- Dessin d'un rectangle contenant des graphes ------------------------------- */


/* ~~~~~~~~~~~~~~ "Primitives" de trac des graphes ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ---------------------- Trac d'un polynme dont on donne les X et les coefficients ------------\
 |    Ne renvoie rien
 |    Ne permet de dessiner qu' l'cran
 |
 | nbr_pts              : nombre de points  tracer
 | degre                : degr du polynome  tracer
 | coefs                : les coefficients
 | x                    : le tableau des X
 | x_echelle, y_echelle : chelles en X et Y
 | x_min    , x_max     : bornes admissibles en X
 | y_min    , y_max     : bornes admissibles en Y
 | x_trace  , y_trace   : origine du trac
 \---------------------------------------------------------------------------------------------- */

void TRC_poly(unsigned long nbr_pts, unsigned short int degre, double *coefs, double *x,
              double x_echelle, double y_echelle, int x_trace, int y_trace, double x_min, double x_max, double y_min, double y_max)
{
  unsigned int compteur = 0;
  unsigned long j = 0;
  double y, x_n;
  short int i;

  if ( (x == NULL) || (coefs == NULL) ) return; /* Paramtres idiots... */

  /* On contrle le degr maximal */
  if ( degre > 0 ) {
    while ( coefs[ degre ] == 0.0 ) {
      degre--;
      if ( 0 == degre ) break;
    }
  }

  if (degre == 0) { /* On trace une constante... */
    if ( ( coefs[ 0 ] < y_min ) || ( coefs[ 0 ] > y_max ) ) return; /* On est hors de la zone */

    points[0] = x_trace + x_echelle * (x[0]  - x_min);
    points[1] = y_trace + y_echelle * (y_max - coefs[0]);
    points[2] = x_trace + x_echelle * (x[nbr_pts - 1] - x_min);
    points[3] = points[1];
    v_pline(_PRG_station->handle, 2, points);
    return;
  }

  if ( 1 == degre ) { /* On trace une droite */
    double x_d, x_f;

    /* Premier point : le plus  gauche (x le plus petit) */
    y = coefs[ 0 ] + coefs[ 1 ] * x_min;
    if ( ( y > y_max ) || ( y < y_min ) ) { /* Bord haut ou bas ! */
      if ( coefs[ 1 ] > 0.0 ) { /* Croissante -> bord bas coup en premier  */
	x_d = ( y_min - coefs[ 0 ] ) / coefs[ 1 ];
      } else { /* Dcroissante -> bord haut coup en premier */
	x_d = ( y_max - coefs[ 0 ] ) / coefs[ 1 ];
      }

      if( ( x_d < x_min ) || ( x_d > x_max ) ) return; /* On est entirement en-dehors */
    } else { /* On coupe le bord gauche en premier : cas simple... */
      x_d = x_min;
    }

    /* Second point : le plus  droite (x le plus grand) */
    y = coefs[ 0 ] + coefs[ 1 ] * x_max;
    if ( ( y > y_max ) || ( y < y_min ) ) { /* Bord haut ou bas ! */
      if ( coefs[ 1 ] > 0.0 ) { /* Croissante -> bord haut coup en dernier  */
	x_f = ( y_max - coefs[ 0 ] ) / coefs[ 1 ];
      } else { /* Dcroissante -> bord bas coup en dernier */
	x_f = ( y_min - coefs[ 0 ] ) / coefs[ 1 ];
      }

      if( ( x_f < x_min ) || ( x_f > x_max ) ) return; /* On est entirement en-dehors */
    } else { /* On coupe le bord gauche en premier : cas simple... */
      x_f = x_max;
    }

    /* On trace entre ces deux points */
    y = coefs[ 0 ] + coefs[ 1 ] * x_d;
    points[ 0 ] = x_trace + x_echelle * ( x_d - x_min ); points[ 1 ] = y_trace + y_echelle * ( y_max - y );

    y = coefs[ 0 ] + coefs[ 1 ] * x_f;
    points[ 2 ] = x_trace + x_echelle * ( x_f - x_min ); points[ 3 ] = y_trace + y_echelle * ( y_max - y );

    v_pline(_PRG_station->handle, 2, points);
    return;
  }

  /* Polynme de degr suprieur : on trace donc point par point... */

  /* Calcul du premier point */
  x_n = *x;
  y = coefs[ 0 ] + x_n * coefs[ 1 ];
  for (i = 2 ; i <= degre ; i++) {
    x_n *= *x;
    y += coefs[ i ] * x_n;
  }

  while ( j < nbr_pts ) { /* Tant qu'il reste des points */
    while ( (*x < x_min) || (*x > x_max) || ( y < y_min) || ( y > y_max) ) { /* On dborde */
      x++; /* On passe au point suivant */
      j++;
      if ( j >= nbr_pts ) return; /* On s'arrte, plus de points aprs */
      
      x_n = *x;
      y = coefs[0] + x_n * coefs[1];
      for ( i = 2 ; i <= degre ; i++ ) {
	x_n *= *x;
	y += coefs[i] * x_n;
      }
    }

    /* On est l <=> le point (x, y) courant doit tre trac */

    compteur = 0; /* On commence un nouveau trac */
    while ( (*x >= x_min) && (*x <= x_max) && ( y >= y_min) && ( y <= y_max) ) { /* On reste dans la zone de trac */
      /* (x, y) convient, on l'a vrifi et calcul */
      points[ compteur ] = x_trace + ( *x - x_min ) * x_echelle;
      points[ compteur + 1 ] = y_trace + ( y_max - y ) * y_echelle;

#ifdef _DEVELOPPE_
      if ( points[ compteur + 1 ] < 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : Valeur absurde pour Y [Y = %d / y_trace = %d, y = %g, y_max = %g, echelle =%g / x = %g]\n",
		       __FILE__, __FUNCTION__, __LINE__, points[ compteur + 1 ], y_trace, y, y_max, y_echelle, *x );
      }
#endif

      compteur += 2;
      if ( compteur >= 256 ) { /* Tableau intin plein */
	v_pline(_PRG_station->handle, 128, points); /* On trace ces 128 points */
	points[0] = points[254];
	points[1] = points[255];
	compteur = 2;
      }

      /* On passe au suivant */
      x++; j++;
      if ( j >= nbr_pts ) /* Fini */
	break;

      x_n = *x;
      y = coefs[ 0 ] + x_n * coefs[ 1 ];
      for (i = 2 ; i <= degre ; i++) {
	x_n *= *x;
	y += coefs[ i ] * x_n;
      }
    }

    /* On trace s'il reste des points */
    if ( (compteur < 256) && (compteur > 1) ) {
      v_pline(_PRG_station->handle, compteur >> 1, points); /* On trace ce qui reste */
    }
  }
}
