/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 2003

   Modles structurels d'une station exprimentale
   Chargement d'une exprience dj enregistre
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"
#include "bin_utils.h"

#include "experience.h"
#include "fichiers.h"
#include "graphes.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Chargement de l'exprience dans son ensemble -------- */

Experience *EXP_ChargerExperience( FILE *fichier, int version, const char *nom_fichier )
{
  Experience *experience;
  Faisceau **table_faisceaux = NULL;
  struct composant **table_composants = NULL;
  struct matiere **table_compositions = NULL;
  int methode;
  unsigned long nbr_objets, num_objet;

  if ( NULL == fichier ) return NULL;

  /* On lit la mthode */
  fread( &methode, sizeof( int ), 1, fichier );

  /* On peut donc crer l'exprience */
  experience = EXP_CreerExperience( methode );
  if ( NULL == experience ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans EXP_CreerExperience()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Proprits */
  fread( &(experience->proprietes), sizeof( int ), 1, fichier );

  /* nergie(s) d'excitation */
  fread( &(experience->E_debut), sizeof( double ), 1, fichier );
  if ( VRAI == EXP_EstSpectroscopie( experience->methode ) ) {
    fread( &(experience->E_fin), sizeof( double ), 1, fichier ); /* nergie d'excitation / fin du spectre */
    fread( &(experience->pas_E), sizeof( double ), 1, fichier ); /* nergie d'excitation / pas */
  }

  fread( &(experience->ordre), sizeof( unsigned int ), 1, fichier ); /* Ordre de modlisation */

  fread( &(experience->Z), sizeof( unsigned int ), 1, fichier );     /* lment tudi */
  fread( &(experience->seuil), sizeof( unsigned int ), 1, fichier ); /* Seuil tudi */

  if ( VRAI == EXP_UtiliseFluorescence( experience ) ) {
    fread( &(experience->Ef_debut), sizeof( double ), 1, fichier ); /* Dbut de l'intervalle de fluorescence */
    fread( &(experience->Ef_fin), sizeof( double ), 1, fichier );   /* Fin de l'intervalle de fluorescence */
    fread( &(experience->pas_Ef), sizeof( double ), 1, fichier );   /* Pas */

    fread( &(experience->ROI_debut), sizeof( double ), 1, fichier ); /* Dbut de la ROI */
    fread( &(experience->ROI_fin), sizeof( double ), 1, fichier );   /* Fin de la ROI */
  }

  /* Les faisceaux */
  fread( &nbr_objets, sizeof( unsigned long ), 1, fichier );
  if ( nbr_objets > 0 ) {
    unsigned long i;

    table_faisceaux = ( Faisceau ** ) malloc( sizeof( Faisceau * ) * nbr_objets );
    if ( NULL == table_faisceaux ) {
      (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante -> aucun faisceau ne sera affect  l'exprience\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return experience;
    }

    for ( i = 0 ; i < nbr_objets ; i++ ) {
      table_faisceaux[ i ] = FAISCEAU_Charger( fichier, version, nom_fichier );
      FAISCEAU_ChainerFaisceau( NULL, table_faisceaux[ i ]);
    }
  }

  /* Lequel est utilis par l'exprience ? */
  fread( &num_objet, sizeof( unsigned long ), 1, fichier );
  if ( table_faisceaux != NULL ) {
    experience->faisceau = table_faisceaux[ num_objet ];
    free( table_faisceaux ); table_faisceaux = NULL;
  }

  /* Les constituants */
  fread( &nbr_objets, sizeof( unsigned long ), 1, fichier );
  if ( nbr_objets > 0 ) {
    unsigned long i;

#ifdef _DEVELOPPE_
    printf( "%s : %lu constituants dans le fichier\n", __FUNCTION__, nbr_objets );
#endif

    table_composants = ( struct composant ** ) malloc( sizeof( struct composant * ) * nbr_objets );
    if ( NULL == table_composants ) {
      (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante -> aucun composant ne sera utilis dans l'exprience\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return experience;
    }

    for ( i = 0 ; i < nbr_objets ; i++ ) {
      table_composants[ i ] = Composant_Charger( fichier, version, nom_fichier );
      printf( "%s : constituant %lu = %s\n", __FUNCTION__, i , table_composants[ i ]->nom);
      if ( NULL == table_composants[ i ] ) {
	(void) fprintf(stderr, "%s,%s l.%u : ERREUR dans Composant_Charger(), composant %lu sur %lu\n",
		       __FILE__, __FUNCTION__, __LINE__, i + 1, nbr_objets);
      }
    }
  }

  /* Les compositions */
  printf("%s : Position dans le fichier = %ld = %lx \n", __FUNCTION__, ftell( fichier ), ftell( fichier ));
  fread( &nbr_objets, sizeof( unsigned long ), 1, fichier );
  if ( nbr_objets > 0 ) {
    unsigned long i;

    printf("%s : %lu compositions a lire\n", __FUNCTION__, nbr_objets);

    table_compositions = ( struct matiere ** ) malloc( sizeof( struct matiere * ) * nbr_objets );
    if ( NULL == table_compositions ) {
      (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante -> aucune composition ne sera utilise dans l'exprience\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }

    for ( i = 0 ; i < nbr_objets ; i++ ) {
      table_compositions[ i ] = Composition_Charger( fichier, version, nom_fichier, table_composants );
      if ( NULL == table_compositions[ i ] ) {
	(void) fprintf(stderr, "%s,%s l.%u : ERREUR dans Composition_Charger(), composition %lu sur %lu\n",
		       __FILE__, __FUNCTION__, __LINE__, i + 1, nbr_objets);
      }
      printf("%s : compositions %lu = %s\n", __FUNCTION__, i, table_compositions[ i ]->nom);

    }
  }

  /* Plus besoin de la table des composants, car ils sont forcment dans des compositions s'ils sont utiliss */
  free( table_composants ); table_composants = NULL;

  /* Les chantillons */
  fread( &nbr_objets, sizeof( unsigned long ), 1, fichier );
  if ( nbr_objets > 0 ) {
    Echantillon **table_echantillons;
    unsigned long i;

    table_echantillons = ( struct echantillon ** ) malloc( sizeof( struct echantillon * ) * nbr_objets );
    if ( NULL == table_echantillons ) {
      (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante -> aucun chantillon ne sera attribu  l'exprience\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }

    for ( i = 0 ; i < nbr_objets ; i++ ) {
      Echantillon *echantillon;

      echantillon = ECHANTILLON_Charger( fichier, version, nom_fichier, table_compositions );
      if ( NULL == echantillon ) {
	(void) fprintf(stderr, "%s,%s l.%u : ERREUR dans ECHANTILLON_Charger(), chantillon %lu sur %lu\n",
		       __FILE__, __FUNCTION__, __LINE__, i + 1, nbr_objets);
      }

      if ( table_echantillons != NULL )
	table_echantillons[ i ] = echantillon;

      /* On chane cet chantillon aux autres */
      ECH_ChainerEchantillon( NULL, echantillon );
    }

    /* Lequel d'entre-eux est utilis ? */
    fread( &num_objet, sizeof( unsigned long ), 1, fichier );
    if ( table_echantillons != NULL ) {
      experience->echantillon = table_echantillons[ num_objet ];
      free( table_echantillons ); table_echantillons = NULL;
    }
  }

  /* Nombre de dtecteurs */
  fread( &nbr_objets, sizeof( unsigned long ), 1, fichier );
  if ( nbr_objets > 0 ) {
    Detecteur **table_detecteurs;
    unsigned long i;

    table_detecteurs = ( struct detecteur ** ) malloc( sizeof( struct detecteur * ) * nbr_objets );
    if ( NULL == table_detecteurs ) {
      (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante -> aucun dtecteur ne sera attribu  l'exprience\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }

    for ( i = 0 ; i < nbr_objets ; i++ ) {
      Detecteur *detecteur;

      detecteur = DETECTEUR_Charger( fichier, version, nom_fichier, table_compositions );
      if ( NULL == detecteur ) {
	(void) fprintf(stderr, "%s,%s l.%u : ERREUR dans DETECTEUR_Charger(), dtecteur %lu sur %lu\n",
		       __FILE__, __FUNCTION__, __LINE__, i + 1, nbr_objets);
      }

      if ( table_detecteurs != NULL )
	table_detecteurs[ i ] = detecteur;

      /* On chane cet chantillon aux autres */
      DET_ChainerDetecteur( NULL, detecteur );
    }

    /* Lequel d'entre-eux est utilis pour I0 ? */
    fread( &num_objet, sizeof( unsigned long ), 1, fichier );
    if ( table_detecteurs != NULL ) {
      experience->i0 = table_detecteurs[ num_objet ];
    }

    /* Lequel d'entre-eux est utilis pour I1 ? */
    fread( &num_objet, sizeof( unsigned long ), 1, fichier );
    if ( table_detecteurs != NULL ) {
      experience->i1 = table_detecteurs[ num_objet ];
    }

    free( table_detecteurs ); table_detecteurs = NULL;
  }

  /* On n'a plus besoin de la table des compositions */
  free( table_compositions ); table_compositions = NULL;

  /* Chargement russi */
  return experience;
}

/* -------- Chargement d'un constituant lmentaire -------- */

struct composant *Composant_Charger(FILE *fichier, int version, const char * nom_fichier)
{
  char *nom;
  struct composant *composant;
  unsigned int i;

  if ( NULL == fichier ) return NULL;

  /* On cre le composant */
  composant = CreerComposant( NULL );
  if ( NULL == composant ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans CreerComposant()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Le nom */
  nom = BIN_CH_chaine( fichier );
  Composant_ChangerNom( composant, nom );
  free( nom );

  /* La densit */
  fread( &(composant->densite), sizeof( float ), 1, fichier );

  /* Les atomes constitutifs */
  for ( i = 0 ; i < ZMAX_TABLE_CONSTITUANTS ; i++ ) {
    fread( composant->atomes + i, sizeof( float ), 1, fichier );

    if ( composant->atomes[ i ] > 0.0 ) {
      composant->mu[ i ] = CH_Graphe( fichier, version, nom_fichier, NULL, FAUX );
      printf( "  %f pour Z=%i, graphe = %s\n", composant->atomes[ i ], i + 1,
	      (composant->mu[ i ] == NULL) ? ("AUCUN") : (composant->mu[ i ]->nom));
    }
  }

  /* /rho global */
  composant->mu_total = CH_Graphe( fichier, version, nom_fichier, NULL, FAUX );
  printf("  mu total : %s\n", (composant->mu_total == NULL) ? ("AUCUN") : (composant->mu_total->nom));

  /* Chargement termin, avec succs (esprons !) */
  return composant;
}

/* -------- Chargement d'une composition -------- */

struct matiere *Composition_Charger(FILE *fichier, int version __attribute__((__unused__)), const char *nom_fichier __attribute__((__unused__)), struct composant **table_composants)
{
  char *nom;
  double densite;
  struct matiere *composition;
  unsigned int nbr_composants;

  if ( NULL == fichier ) return NULL;

  /* Le nom */
  nom = BIN_CH_chaine( fichier );

  /* La densit */
  fread( &densite, sizeof( double ), 1, fichier );

  /* Le nombre de composants */
  fread( &nbr_composants, sizeof( unsigned int ), 1, fichier );

  /* On cre la composition */
  composition = Matiere_CreerComposition( NULL, nbr_composants );
  if ( NULL == composition ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans Matiere_CreerComposition()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( nom );
    return NULL;
  }
  composition->densite = densite;
  Matiere_Nommer( composition, nom );
  free( nom ); /* Plus besoin de lui */

  /* Chargement des composants */
  if ( nbr_composants > 0 ) {
    if ( NULL == composition->fraction_massique ) {
      (void) fprintf(stderr, "%s,%s l.%u : Erreur dans la cration de la table des composants\n",
		     __FILE__, __FUNCTION__, __LINE__);

    } else {
      unsigned int i, num_composant;
      struct composant *composant;

      /* Les fractions massiques */
      fread( composition->fraction_massique, sizeof( float ), nbr_composants, fichier );

      /* Les composants */
      for ( i = 0 ; i < nbr_composants ; i++ ) {
	fread( &num_composant, sizeof( unsigned int ), 1, fichier );

	composant = table_composants[ num_composant ];
	composition->composants[ i ] = composant;
      }
    }
  }

  /* Chargement termin, avec succs (esprons !) */
  return composition;
}

/* -------- Chargement d'un dtecteur -------- */

Detecteur *DETECTEUR_Charger(FILE *fichier, int version, const char * nom_fichier, struct matiere **table_compositions)
{
  Detecteur *detecteur;
  char *nom;
  int retour;
  unsigned int proprietes;

  if ( NULL == fichier ) return NULL;

  /* Nom du dtecteur */
  nom = BIN_CH_chaine( fichier );

  /* Proprits */
  fread( &proprietes, sizeof( unsigned int ), 1, fichier );

  /* On cre le dtecteur */
  detecteur = DET_CreerDetecteur( DETECTEUR_CHAMBRE, proprietes );
  if ( NULL == detecteur ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans DET_CreerDetecteur()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( nom );
    return NULL;
  }
  DET_NommerDetecteur( detecteur, nom );
  free( nom );

  /* L'orientation */
  retour = BIN_CH_orientation( fichier, &(detecteur->orientation) );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans BIN_CH_orientation() [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return NULL;
  }

  /* Angle solide intercept */
  fread( &(detecteur->angle_solide), sizeof( double ), 1, fichier );

  /* Dimensions du dtecteur */
  fread( &(detecteur->largeur), sizeof( double ), 1, fichier );
  fread( &(detecteur->hauteur), sizeof( double ), 1, fichier );

  /* Longueur traverse */
  fread( &(detecteur->longueur), sizeof( double ), 1, fichier );
  if ( detecteur->longueur > 0.0 ) {
    unsigned int num_composition;

    fread( &num_composition, sizeof( unsigned int ), 1, fichier );
    if ( table_compositions != NULL )
      detecteur->composition = table_compositions[ num_composition ];
  }

  /* Coups  vide */
  fread( &(detecteur->coups_a_vide), sizeof( double ), 1, fichier );

  printf("Dtecteur : %s\n", detecteur->nom );

  /* Rponse avec l'intensit */
  detecteur->r_signal = CH_Graphe( fichier, version, nom_fichier, NULL, FAUX );

  /* Rponse avec l'nergie */
  detecteur->r_energie = CH_Graphe( fichier, version, nom_fichier, NULL, FAUX );

  /* Rponse avec la surface */
  detecteur->r_surface = CH_Image( fichier, version, nom_fichier, FAUX );

  return detecteur;
}

/* -------- Chargement d'un chantillon -------- */

Echantillon *ECHANTILLON_Charger(FILE *fichier, int version, const char *nom_fichier __attribute__((__unused__)), struct matiere **table_compositions)
{
  Echantillon *echantillon;
  char *nom;
  double largeur, hauteur, epaisseur;
  int type;
  unsigned int n_x, n_y, n_z;
  unsigned int proprietes;

  if ( NULL == fichier ) return NULL;

  /* Le nom */
  nom = BIN_CH_chaine( fichier );

  /* Le type */
  fread( &type, sizeof( int ), 1, fichier );

  /* Les dimensions */
  fread( &largeur, sizeof( double ), 1, fichier );
  fread( &hauteur, sizeof( double ), 1, fichier );
  fread( &epaisseur, sizeof( double ), 1, fichier );

  /* Les pas */
  fread( &n_x, sizeof( unsigned int ), 1, fichier );
  fread( &n_y, sizeof( unsigned int ), 1, fichier );
  fread( &n_z, sizeof( unsigned int ), 1, fichier );

  /* Les proprits */
  fread( &proprietes, sizeof( unsigned int ), 1, fichier );

  /* On cre l'chantillon */
  echantillon = ECH_CreerEchantillonN( largeur, hauteur, epaisseur, n_x, n_z, n_y, type, proprietes );
  if ( NULL == echantillon ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans ECH_CreerEchantillonN()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( nom );
    return NULL;
  }
  ECHANTILLON_Nommer( echantillon, nom );
  free( nom );

  /* Chargement du contenu */
  if ( proprietes & ECHANTILLON_HOMOGENE ) {
    unsigned int num_composition;

    /* On lit juste la composition */
    fread( &num_composition, sizeof( unsigned int ), 1, fichier );
    if ( table_compositions != NULL )
      echantillon->valeurs.homogene.composition = table_compositions[ num_composition ];
  } else if ( ECHANTILLON_GRADIENT == echantillon->type ) {
    unsigned int num_composition;
    unsigned long nbr_cases;

    /* On lit la composition */
    fread( &num_composition, sizeof( unsigned int ), 1, fichier );
    if ( table_compositions != NULL )
      echantillon->valeurs.homogene.composition = table_compositions[ num_composition ];

    /* On lit la table des valeurs */
    nbr_cases = 1 + ECH_IJK_v_Case( echantillon, n_x - 1, n_y - 1, n_z - 1 );
    fread( echantillon->valeurs.homogene.concentration, sizeof( double ), nbr_cases, fichier );
  } else if ( version > 32 ) { /* chantillon inhomogne : v. 3.3 */
    int num_composition;
    unsigned long i;
    unsigned long nbr_cases;

    /* Combien de cases ? */
    nbr_cases = 1 + ECH_IJK_v_Case( echantillon, n_x - 1, n_y - 1, n_z - 1 );

    /* Composition en chaque point */
    for ( i = 0 ; i < nbr_cases ; i++ ) {
      fread( &num_composition, sizeof( int ), 1, fichier );
      if ( -1 == num_composition ) { /* Trou */
	echantillon->valeurs.inhomogene[ i ] = NULL;
      } else if ( table_compositions != NULL ) {
	echantillon->valeurs.inhomogene[ i ] = table_compositions[ (unsigned int) num_composition ];
      }
    }
  }

  /* Chargement termin */
  return echantillon;
}

/* -------- Chargement d'un faisceau -------- */

Faisceau *FAISCEAU_Charger(FILE *fichier, int version, const char *nom_fichier)
{
  Faisceau *faisceau;
  char *nom;
  double largeur, hauteur, I;
  int retour;
  unsigned int proprietes, nbr_harmoniques;

  if ( NULL == fichier ) return NULL;

  /* Le nom */
  nom = BIN_CH_chaine( fichier );

  /* Les dimensions */
  fread( &largeur, sizeof( double ), 1, fichier );
  fread( &hauteur, sizeof( double ), 1, fichier );

  /* L'intensit */
  fread( &I, sizeof( double ), 1, fichier );

  /* Les proprits */
  fread( &proprietes, sizeof( unsigned int ), 1, fichier );
  if ( ( 0.0 >= largeur ) && ( 0.0 >= hauteur ) ) {
    proprietes |= FAISCEAU_PONCTUEL; /* N'existait pas dans les premires versions */
  }
#ifdef _DEVELOPPE_
  printf("%s : faisceau [%s], %f  %f, I=%g, p=%i\n", __FUNCTION__, nom, largeur, hauteur, I, proprietes );
#endif

  /* On cre le faisceau associ */
  faisceau = FAISCEAU_Creer( largeur, hauteur, proprietes, 0, I );
  if ( NULL == faisceau ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le faisceau\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( nom );
    return NULL;
  }
  FAISCEAU_Nommer( faisceau, nom );
  free( nom );

  /* L'orientation */
  retour = BIN_CH_orientation( fichier, &(faisceau->orientation) );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans BIN_CH_orientation() [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return NULL;
  }

  /* Si ncessaire, le point focal */
  if ( FAISCEAU_FOCALISE & faisceau->proprietes ) {
    retour = BIN_CH_vecteur_3d( fichier, &(faisceau->point_focal) );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans BIN_CH_vecteur_3d() point focal [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return NULL;
    }
  }

  /* Intensit avec l'nergie nominale */
  faisceau->g_energie = CH_Graphe( fichier, version, nom_fichier, NULL, FAUX );

  /* Spectre du faisceau  Monochomatique  */
  faisceau->g_E = CH_Graphe( fichier, version, nom_fichier, NULL, FAUX );

  /* Nombre d'harmoniques */
  fread( &nbr_harmoniques, sizeof( unsigned int ), 1, fichier );
  if ( nbr_harmoniques > 0 ) {
    unsigned int i;

    retour = FAISCEAU_CreerHarmoniques( faisceau, nbr_harmoniques );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la table des harmoniques\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }

    for ( i = 0 ; i < nbr_harmoniques ; i++ ) {
      fread( faisceau->i_harmoniques + i, sizeof( float ), 1, fichier );
      faisceau->g_harmoniques[ i ] = CH_Graphe( fichier, version, nom_fichier, NULL, FAUX );
    }
  }

  /* Carte de rpartition spatiale */
  faisceau->g_espace = CH_Image( fichier, version, nom_fichier, FAUX );

  return faisceau;
}
