/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 1999
   Cration du fichier   : 14 dcembre 1999
   Dernire modification : 14 dcembre 1999

   Module de calculs thoriques divers
   Calcul d'un signal EXAFS thorique pour 1 couche,  partir des lments constitutifs
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"

#include "lissage.h"

#include "extrait.h" /* Pour EXF_k_p_n */
#include "theorie.h"

#include "th_exafs.txt"

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

typedef struct
{
  Graphe *amplitude;
  Graphe *phase;
  Graphe *lpm;
} TH_calc_infos;

#define LGMAX_NOM_AMPLITUDE 34 /* Longueur des textes dans les boutons */
#define LGMAX_NOM_PHASE     34
#define LGMAX_NOM_LPM       25

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

static int GER_calc_exafs(unsigned int objet, int x, int y);
static void prepare_arbre_exafs(void);

/* ~~~~~~~~ Ralisation pratique des fonctions ~~~ */

Graphe *TH_calc_exafs(void)
{
  Graphe *exafs;
  double k_min, k_max, pas;
  double N, R, sigma, gamma = 0.0, eta = 0.0;
  double *b_amp = NULL, *c_amp = NULL, *d_amp = NULL;
  double *b_pha = NULL, *c_pha = NULL, *d_pha = NULL;
  double *b_lpm = NULL, *c_lpm = NULL, *d_lpm = NULL;
  register double *k, *khi;
  unsigned long i, nbr_points;
  int retour, deg_pondere;
  TH_calc_infos infos = { NULL, NULL, NULL };

  if (arbres[CALC_SIGNAL_XAS].drapeaux.prepare == FAUX)
    prepare_arbre_exafs();

  met_chaine(CALC_SIGNAL_XAS, CSX_AMPLITUDE, LGMAX_NOM_AMPLITUDE, texte_err_amplitude, TXT_REMPLACE);
  met_chaine(CALC_SIGNAL_XAS, CSX_PHASE, LGMAX_NOM_PHASE, texte_err_phase, TXT_REMPLACE);
  met_chaine(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM, LGMAX_NOM_LPM, texte_err_lpm, TXT_REMPLACE);

  OBJ_active(CALC_SIGNAL_XAS, CSX_CALCULER, FORCE_NON); /* Tant que ni phase, ni amplitude */

  ARB_MettreComplement( CALC_SIGNAL_XAS, &infos );
  retour = ARB_Bloquant( CALC_SIGNAL_XAS, titre_arbre_calcul, principale, 0 );
  ARB_MettreComplement( CALC_SIGNAL_XAS, NULL );
  if ( retour == CSX_ANNULER )
    return NULL;

  /* ------ On prpare pour le calcul ------ */
  N = OBJ_LireNombre( CALC_SIGNAL_XAS, CSX_N, NULL );
  R = OBJ_LireNombre( CALC_SIGNAL_XAS, CSX_R, NULL );
  sigma = OBJ_LireNombre( CALC_SIGNAL_XAS, CSX_DW, NULL );

  if ( (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMA, QUESTION) == VRAI) ||
       (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMAETA, QUESTION) == VRAI) ) {
    gamma = OBJ_LireNombre( CALC_SIGNAL_XAS, CSX_VAL_GAMMA, NULL );
    if (gamma < 0.0) {
      (void) BI_ERR_alerte(NULL, &ERR_LPM_gamma, FAUX, gamma);
      return NULL;
    }
    if ( OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMAETA, QUESTION) == VRAI ) {
      eta = OBJ_LireNombre( CALC_SIGNAL_XAS, CSX_VAL_ETA, NULL );
      if (eta == 0.0) {
	(void) BI_ERR_alerte(NULL, &ERR_LPM_eta, FAUX, gamma);
	return NULL;
      }
    }
  }

  k_min = OBJ_LireNombre( CALC_SIGNAL_XAS, CSX_KMIN, NULL );
  k_max = OBJ_LireNombre( CALC_SIGNAL_XAS, CSX_KMAX, NULL );
  if (k_max < k_min) {
    double k = k_min; k_min = k_max; k_max = k;
  }
  if (k_max == k_min) {
    EXF_ERREUR(BORNES_FAUSSES);
    return NULL;
  }

  pas = OBJ_LireNombre( CALC_SIGNAL_XAS, CSX_PAS, NULL );
  if (pas <= 0.0) {
    EXF_ERREUR(PAS_FAUX);
    return NULL;
  }

  nbr_points = (unsigned long) (k_max - k_min) / pas;
  k_max = k_min + pas * (double) (nbr_points);
  nbr_points++;

  exafs = GRF_nouveau( NULL, nbr_points, GR_AJUSTE, GR_NORMAL, titre_grf_exafs );
  if (exafs == NULL) return NULL;

  /* ---- Les splines amplitude / phase / lpm si ncessaire ---- */
  if ( SPL_tt_prep(infos.amplitude, &b_amp, &c_amp, &d_amp) < 0.0 ) {
    GRF_efface(exafs, VRAI, VRAI);
    return NULL;
  }
  if ( SPL_tt_prep(infos.phase, &b_pha, &c_pha, &d_pha) < 0.0 ) {
    free(c_amp); free(b_amp); free(d_amp);
    GRF_efface(exafs, VRAI, VRAI);
    return NULL;
  }
  if ( OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GRAPHE, QUESTION) == VRAI ) {
    if ( SPL_tt_prep(infos.lpm, &b_lpm, &c_lpm, &d_lpm) < 0.0 ) {
      free(c_amp); free(b_amp); free(d_amp);
      free(c_pha); free(b_pha); free(d_pha);
      GRF_efface(exafs, VRAI, VRAI);
      return NULL;
    }
  }

  for (i = 0, k = exafs->x, khi = exafs->y ; i < nbr_points ; i++, k++, khi++) {
    double x = k_min + pas * (double) i;
    double y = N;
    double phase = 2 * R;

    *k = x;
    if (x == 0.0) {
      *khi = 0.0; continue;
    }

    y /= x * R * R;
    y *= exp(-2 * x * x * sigma);
    y *= SPL_tt_calc(infos.amplitude, x, b_amp, c_amp, d_amp);

    phase *= x;
    phase += SPL_tt_calc(infos.phase, x, b_pha, c_pha, d_pha);
    y *= sin(phase);

    /* Maintenant, le lpm */
    if (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMA, QUESTION) == VRAI) {
      double lpm = -2 * R * gamma;
      lpm /= x;
      y *= exp(lpm);
    } else if (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMAETA, QUESTION) == VRAI) {
      double lpm, inter;

      inter = eta / x;
      inter *= inter;
      inter *= inter;
      lpm = 2.0 * R * gamma * (x + inter);
      y *= exp(lpm);
    } else if (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GRAPHE, QUESTION) == VRAI) {
      double lpm;

      lpm = SPL_tt_calc(infos.lpm, x, b_lpm, c_lpm, d_lpm);
      if (lpm == 0.0) {
	EXF_ERREUR(DIV_ZERO, "Libre parcours moyen nul", txt_err_val_inchangee);
      } else {
	lpm = -2 * R / lpm;
	y *= exp(lpm);
      }
    }

    *khi = y;
  }

  free(b_amp); free(c_amp); free(d_amp);
  free(c_pha); free(b_pha); free(d_pha);
  free(c_lpm); free(b_lpm); free(d_lpm);

  HIST_Titre(1, "Oscillations EXAFS d'un chemin", "TH");
  HIST_Texte(1, 0, "EXAFS calcul"e_AIGU" : %s\n", exafs->nom);
  HIST_Texte(3, 0, "D"e_AIGU"g"e_AIGU"n"e_AIGU"rescence : %f\n", N);
  HIST_Texte(3, 0, "Longueur de chemin      : %f "ANGSTROM"\n", R);
  HIST_Texte(3, 0, "Facteur de Debye-Waller : %f "ANGSTROM""DEUX_EXPOSANT"\n", sigma);
  HIST_Texte(3, 0, "Amplitude dans %s\n", infos.amplitude->nom);
  HIST_Texte(3, 0, "Phase dans %s\n", infos.phase->nom);
  HIST_Texte(3, 0, "Libre parcours moyen : ");
  if (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMA, QUESTION) == VRAI)
    HIST_Texte(3, 0, "constant, "GAMMA_MAJUSCULE" = %f\n", gamma);
  else if (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMAETA, QUESTION) == VRAI)
    HIST_Texte(3, 0, "variable, "GAMMA_MAJUSCULE" = %f, "ETA" = %f\n", gamma, eta);
  else if (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GRAPHE, QUESTION) == VRAI)
    HIST_Texte(3, 0, "dans %s\n", infos.lpm->nom);
  else
    HIST_Texte(3, 0, "inclus "a_GRAVE" l'amplitude\n");
  HIST_Texte(5, 0, "Calcul de %f  %f par pas de %g\n", k_min, k_max, pas);

  deg_pondere = OBJ_LireEntier_U( CALC_SIGNAL_XAS, CSX_VAL_PONDERE );
  (void) EXF_k_p_n(exafs, deg_pondere, VRAI);

  GRF_bornes(exafs, ORDONNE, DESORDONNE);
  return exafs;
}

/* -------- Gestion de l'arbre -------- */

static int GER_calc_exafs(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  TH_calc_infos *infos;
  Graphe *nv_graphe;

  infos = (TH_calc_infos *) ARB_LireComplement( CALC_SIGNAL_XAS );

  switch (objet)
  {
  case CSX_AMPLITUDE:
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_AMPLITUDE, FORCE_OUI);
    nv_graphe = GRF_choisir(GR_EXAFSAMP, VRAI, VRAI, NULL);
    if (nv_graphe != NULL) {
      infos->amplitude = nv_graphe;
      met_chaine(CALC_SIGNAL_XAS, CSX_AMPLITUDE, LGMAX_NOM_AMPLITUDE, nv_graphe->nom, TXT_REMPLACE);
      if ( (infos->phase != NULL) &&
	   ( (infos->lpm != NULL) || (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GRAPHE, QUESTION) == FAUX) ) )
	OBJ_active(CALC_SIGNAL_XAS, CSX_CALCULER, FORCE_OUI);
    }
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_AMPLITUDE, FORCE_NON);
    return BI_EVENEMENT_TRAITE;

  case CSX_PHASE:
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_PHASE, FORCE_OUI);
    nv_graphe = GRF_choisir(GR_EXAFSPHA, VRAI, VRAI, NULL);
    if (nv_graphe != NULL) {
      infos->phase = nv_graphe;
      met_chaine(CALC_SIGNAL_XAS, CSX_PHASE, LGMAX_NOM_PHASE, nv_graphe->nom, TXT_REMPLACE);
      if ( (infos->amplitude != NULL) &&
	   ( (infos->lpm != NULL) || (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GRAPHE, QUESTION) == FAUX) ) )
	OBJ_active(CALC_SIGNAL_XAS, CSX_CALCULER, FORCE_OUI);
    }
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_PHASE, FORCE_NON);
    return BI_EVENEMENT_TRAITE;

  case CSX_LPM_INCLUS:
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_INCLUS, FORCE_OUI);
    OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_GAMMA, FORCE_NON);
    OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_ETA, FORCE_NON);
    OBJ_active(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM, FORCE_NON);
    if ( (infos->amplitude != NULL) && (infos->phase != NULL) )
      OBJ_active(CALC_SIGNAL_XAS, CSX_CALCULER, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;

  case CSX_LPM_GAMMA:
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMA, FORCE_OUI);
    OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_GAMMA, FORCE_OUI);
    OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_ETA, FORCE_NON);
    OBJ_active(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM, FORCE_NON);
    if ( (infos->amplitude != NULL) && (infos->phase != NULL) )
      OBJ_active(CALC_SIGNAL_XAS, CSX_CALCULER, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;

  case CSX_LPM_GAMMAETA:
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMAETA, FORCE_OUI);
    OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_GAMMA, FORCE_OUI);
    OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_ETA, FORCE_OUI);
    OBJ_active(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM, FORCE_NON);
    if ( (infos->amplitude != NULL) && (infos->phase != NULL) )
      OBJ_active(CALC_SIGNAL_XAS, CSX_CALCULER, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;

  case CSX_LPM_GRAPHE:
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GRAPHE, FORCE_OUI);
    OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_GAMMA, FORCE_NON);
    OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_ETA, FORCE_NON);
    OBJ_active(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM, FORCE_OUI);
    if ( (infos->amplitude != NULL) && (infos->phase != NULL) &&
	 ( (infos->lpm != NULL) || (OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GRAPHE, QUESTION) == FAUX) ) )
      OBJ_active(CALC_SIGNAL_XAS, CSX_CALCULER, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;

  case CSX_GRAPHE_LPM:
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM, FORCE_OUI);
    nv_graphe = GRF_choisir(GR_LIBREP, VRAI, VRAI, NULL);
    if (nv_graphe != NULL) {
      infos->lpm = nv_graphe;
      met_chaine(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM, LGMAX_NOM_LPM, nv_graphe->nom, TXT_REMPLACE);
      if ( (infos->amplitude != NULL) && (infos->phase != NULL) )
	OBJ_active(CALC_SIGNAL_XAS, CSX_CALCULER, FORCE_OUI);
    }
    OBJ_choisit(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM, FORCE_NON);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparation de l'arbre ---- */

static void prepare_arbre_exafs(void)
{
  boolean etat;

  OBJ_deplace(CALC_SIGNAL_XAS, CSX_TXT_R, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_R, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_TXT_DW, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_DW, 0, +2, DEPLACE_PIXELS);

  OBJ_deplace(CALC_SIGNAL_XAS, CSX_LPM_INCLUS, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_TXT_INCLUS, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_LPM_GAMMA, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_LPM_GAMMAETA, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_LPM_GRAPHE, 0, +4, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_VAL_GAMMA, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_VAL_ETA, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM, 0, +4, DEPLACE_PIXELS);

  OBJ_deplace(CALC_SIGNAL_XAS, CSX_KMAX, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_TXT_PONDERE, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_SIGNAL_XAS, CSX_VAL_PONDERE, 0, +1, DEPLACE_PIXELS);

  OBJ_active(CALC_SIGNAL_XAS, CSX_GRAPHE_LPM,
	     OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GRAPHE, QUESTION));
  etat = OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMAETA, QUESTION); /* VRAI si gamma + ta */
  OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_ETA, etat);
  OBJ_active(CALC_SIGNAL_XAS, CSX_VAL_GAMMA,
	     OBJ_choisit(CALC_SIGNAL_XAS, CSX_LPM_GAMMA, QUESTION) | (etat));

#ifdef AVEC_TRADUCTION_ /* On traduit les textes de l'arbre */
  met_chaine(CALC_SIGNAL_XAS, CSX_TXT_DEG, 0, txt_arbre_degenerescence, TXT_REMPLACE);
  met_chaine(CALC_SIGNAL_XAS, CSX_TXT_R  , 0, txt_arbre_longueur, TXT_REMPLACE);
  met_chaine(CALC_SIGNAL_XAS, CSX_TXT_AMP, 0, txt_arbre_amplitude, TXT_REMPLACE);
  met_chaine(CALC_SIGNAL_XAS, CSX_TXT_LPM, 0, txt_arbre_lpm, TXT_REMPLACE);
  met_chaine(CALC_SIGNAL_XAS, CSX_TXT_INCLUS, 0, txt_arbre_lpm_inclus, TXT_REMPLACE);
  met_chaine(CALC_SIGNAL_XAS, CSX_TXT_PHASE , 0, txt_arbre_dephasage, TXT_REMPLACE);
  met_chaine(CALC_SIGNAL_XAS, CSX_TXT_PONDERE, 0, txt_arbre_ponderation, TXT_REMPLACE);

  OBJ_TraduireTexte( CALC_SIGNAL_XAS, CSX_CALCULER, txt_arbre_calculer );
#endif

  arbres[CALC_SIGNAL_XAS].fonction.arbre = GER_calc_exafs;
  arbres[CALC_SIGNAL_XAS].drapeaux.prepare = VRAI; 
}
