/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 2002, Emmanuel CURIS
   Cration : le 21 janvier 2001

   Module de calculs thoriques (simples)
   Calcul de l'absorption par une molcule de composition donne
   (Calcul du fond continu en particulier, ie sans le seuil...)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "graphes.h"

#include "absorption.h"
#include "atomes.h"
#include "chimie_utils.h"
#include "interface.h"
#include "science.h" /* Au cas o M_PI n'est pas dfini */
#include "theorie.h"

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

#define NOM_FICHIER_SOLVANT   "solvants.txt"
#define NOM_FICHIER_PASTILLES "pastilles.txt"

#define LGMAX_NOM_SOLVANT   17
#define LGMAX_NOM_LIANT     22 /* ex : cellulose, pour faire la pastille */
#define LGMAX_GRF_REFERENCE 52
#define LGMAX_NOM_SOLUTE    23

#define NOMBRE_SOLUTES (CCM_SOL_NAT_DER - CCM_SOL_NAT_1 + 1)

/* static unsigned long nombre_pastilles = 0, nombre_solvants = 0; */
/* static ListeComposes *liste_pastilles = NULL, *liste_solvants = NULL; */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

static void Victoreen_matrice(Graphe *graphe, int Z, float proportion, int seuil);
static void McMaster_matrice (Graphe *graphe, int Z, float proportion, int seuil);

static void    prepare_calc_matrice(void);
static boolean GER_ArbreMatrice(unsigned int objet, int x, int y);

static void I_ActiverPastille(boolean activer);
static void I_ActiverSolution(boolean activer);

/* ~~~~~~~~ Ralisation pratique du module ~~~~~~~ */

/* ------ Interface : entre de la formule,...
   fond : si non-NULL, par dfaut on lui rajoute l'absorption ainsi calcule
*/

Graphe *TH_fond_continu(Graphe *fond)
{
  int retour, methode = MUM_SELON_VICTOREEN;
  char *texte;
  double e_min, e_max;
  float masse_molaire;
  Composition *compo;
  Graphe *graphe = NULL; /* Contient le nouveau fond continu calcul */

  if (arbres[CALC_MUM].drapeaux.prepare == FAUX)
    prepare_calc_matrice();

  retour = ARB_Bloquant( CALC_MUM, "Calcul de (matrice)", principale, 0 );
  if ( CMM_ANNULER == retour ) return NULL;

  texte = lit_chaine( CALC_MUM, CMM_FORMULE, 0, NULL, TXT_REMPLACE );
  if ( NULL == texte ) return NULL;

#ifdef _DEVELOPPE_
  printf("Formule  analyser : %s\n", texte);
#endif

  compo = Chimie_AnalyseFormule(texte);
  if (compo == NULL) {
    (void) fprintf(stderr, "TH_fond_continu : ERREUR dans Chimie_AnalyseFormule()\n");
    return NULL;
  }

  /* Calcul de la masse molaire */
  for (retour = 0, masse_molaire = 0.0; retour < 120 ; retour++) {
    if (compo->atomes[retour] > 0.0) {
      masse_molaire += compo->atomes[retour] * ATM_donnees[retour].m_molaire;
    }
  }
  printf("Masse molaire obtenue : %f gmol-1\n", masse_molaire);
  if (masse_molaire <= 0.0) {
    (void) fprintf(stderr, "TH_fond_continu : Masse molaire ngative ou nulle ! Calcul impossible\n");
    return NULL;
  }

  if ( OBJ_choisit(CALC_MUM, CMM_NV_SPECTRE, QUESTION) == VRAI ) {
    double pas;

    retour = OBJ_LireIntervalle( CALC_MUM, CMM_XMIN, CMM_XMAX, CMM_PAS, 0, &e_min, &e_max, &pas, NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Paramtres de l'intervalle incorrects\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }

    graphe = GRF_nv_GrilleXP( NULL, GR_MUMATRICE, GR_NORMAL, "Absorption modlise", FAUX, e_min, e_max, pas );
    if (graphe == NULL) {
      (void) fprintf(stderr, "%s,%s l.%u : Erreur  la cration du graphe\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }
  } else { /* On ajoute  un graphe prexistant */
    printf("TH_fond_continu :  FAIRE, ajout au graphe existant [%p / %s]\n", (void *) fond, fond->nom);
  }

  if ( OBJ_choisit(CALC_MUM, CMM_VICTOREEN, QUESTION) == VRAI ) {
    methode = MUM_SELON_VICTOREEN;
  } else if ( OBJ_choisit(CALC_MUM, CMM_MACMASTER, QUESTION) == VRAI ) {
    methode = MUM_SELON_MCMASTER;
  }

  /* lment par lment, on ajoute la contribution  l'absorption */
  for (retour = 0 ; retour < 120 ; retour++) {
    if (compo->atomes[retour] > 0.0) {
      float frc_massique;

      frc_massique = (float) compo->atomes[retour] * ATM_donnees[retour].m_molaire / masse_molaire;
      printf("On ajoute la contribution de l'atome Z = %d (%f%% en masse)\n",
	     retour + 1, frc_massique * 100.0);
      TH_AjouterMum(graphe, retour + 1, frc_massique, methode, VRAI);
    }
  }
  /* Ici, on a donc le coefficient d'absorption massique de l'chantillon
     Il correspond aussi  l'absorbance pour 1g de produit sur 1 cm
     ==> en divisant par la hauteur de seuil, on a la masse de produit  mettre
         pour avoir un seuil unitaire
  */

  GRF_bornes(graphe, ORDONNE, DESORDONNE);
  free(compo);
  return graphe;
}

/* -------- Estimation de l'absorption due  un type d'atome ----------------------\
 |  Calcule le coefficient d'absorption massique, /rho - en fait,                 |
 |          la section efficace massique (grandeur additive)                       |
 |                                                                                 |
 | graphe     : le graphe auquel doit s'ajouter la contribution.                   |
 |              ses X servent  dfinir l'intervalle d'nergie et le pas           |
 | Z          : le type d'atome pour lequel le calcul est demand                  |
 | proportion : la fraction de cet atome au total (fraction massique)              |
 | methode    : faon de faire le calcul                                           |
 |   MUM_SELON_VICTOREEN = modle de Victoreen                                     |
 |   MUM_SELON_MCMASTER  = modle de McMaster                                      |
 | avec_seuils : VRAI => on tient compte des seuils rencontrs (calcul de  total) |
 |               FAUX => on calcule seulement pour le fond continu                 |
 \-------------------------------------------------------------------------------- */

void TH_AjouterMum(Graphe *graphe, int Z, float proportion, int methode, boolean avec_seuils)
{
  double e_seuil;
  int seuil;

  if (graphe == NULL) return;

  e_seuil = TH_QuelSeuil(Z, graphe->x_min, graphe->x_max, &seuil);

  if (e_seuil <= 0.0 ) { /* L'lment ne possde pas de seuil dans l'intervalle considr */
    switch (methode)
    {
    case MUM_SELON_VICTOREEN: /* Estimation selon Victoreen */
      Victoreen_matrice(graphe, Z, proportion, seuil);
      break;

    case MUM_SELON_MCMASTER:  /* Estimation selon McMaster */
      McMaster_matrice(graphe, Z, proportion, seuil);
      break;
    }
    return;
  }

  if (FAUX == avec_seuils) { /* lment avec seuil, ne nous intresse pas => on calcule juste la contribution hors-seuil */
#ifdef _DEVELOPPE_
    printf("L'lment %d possde un seuil entre %f et %f : on n'en tient pas compte\n", Z, graphe->x_min, graphe->x_max);
#endif
    switch (methode)
    {
    case MUM_SELON_VICTOREEN: /* Estimation selon Victoreen */
      Victoreen_matrice( graphe, Z, proportion, seuil );
      break;

    case MUM_SELON_MCMASTER:  /* Estimation selon McMaster */
      McMaster_matrice( graphe, Z, proportion, seuil );
      break;
    }
    return;
  }

  /* lment avec un seuil dans l'intervalle, on en tient compte */
  switch (methode)
  {
  case MUM_SELON_VICTOREEN: /* Estimation selon Victoreen */
    victorine_seuil(graphe, Z, proportion, seuil, e_seuil);
    break;

  case MUM_SELON_MCMASTER:  /* Estimation selon McMaster */
    McMaster_seuil(graphe, Z, proportion, seuil, e_seuil);
    break;
  }
}

/* -------- Fond continu estim avec le modle de Victoreen -------- */

static void Victoreen_matrice(Graphe *graphe, int Z, float proportion, int seuil)
{
  unsigned long i;
  register double *x, *y;
  double C = 0.0, D = 0.0;

  if ( TH_coef_Victoreen(Z, seuil, &C, &D) == FAUX ) {
    (void) fprintf(stderr, "Victoreen_matrice : ERREUR dans TH_coef_Victoreen()\n");
    return;
  }

#ifdef _DEVELOPPE_
  printf("Valeurs de C et D : %f & %f\n", C, D);
#endif

  for (i = 0, x = graphe->x, y = graphe->y ; i < graphe->nombre_point ; i++, x++, y++) {
    double mu;

    /* Calcul de la valeur correspondante */
    mu  = TH_Victoreen(*x, Z, C, D);
    mu *= proportion; /* Pondration par la fraction massique */
    *y += mu;
  }
}

static void McMaster_matrice(Graphe *graphe, int Z, float proportion, int seuil)
{
  unsigned long i;
  double *x, *y;
  double A, B, C, D;
  double Ae, Be, Ce, De;
  double Ai, Bi, Ci, Di;

  if ( TH_coef_McMaster(Z, seuil, &A, &B, &C, &D) == FAUX ) {
    (void) fprintf(stderr, "McMaster_matrice : valeurs inconnues / lment %d, seuil %d\n", Z, seuil);
    return;
  }
  TH_coef_McMaster(Z, DIFFUSION_COHERENTE  , &Ae, &Be, &Ce, &De);
  TH_coef_McMaster(Z, DIFFUSION_INCOHERENTE, &Ai, &Bi, &Ci, &Di);

  for (i = 0, x = graphe->x, y = graphe->y ; i < graphe->nombre_point ; i++, x++, y++) {
    double coef_McM;

    coef_McM  = TH_McMaster(*x, Z, A, B, C, D); /*  Coefficient d'absorption  massique */
/*    coef_McM += TH_McMaster(*x, Z, Ae, Be, Ce, De);*/ /* Contribution de la diffusion lastique */
/*    coef_McM += TH_McMaster(*x, Z, Ai, Bi, Ci, Di);*/ /* Contribution de la diffusion inlastique */
    coef_McM *= proportion;
    *y += coef_McM;
  }
}

/* ------ Prparation et gestion de l'interface ------ */

static void prepare_calc_matrice(void)
{
  int retour;

  /* Initialisation */
  retour = ARB_Initialiser( CALC_MUM, NULL, GER_ArbreMatrice, ( int (*)(unsigned int, int, int *) ) NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR en initialisant l'arbre [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return;
  }
  /*   OBJ_ChangerType(CALC_MUM, CMM_FORMULE, G_TXT_INFINI); *//* Pour avoir une formule de longueur quelconque */

  /* Enjolivation */
  OBJ_deplace(CALC_MUM, CMM_CHOISIR, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CMM_TXT_MODELE, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CMM_FND_MODELE, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CMM_SOLVANT, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CCM_NV_SOLUTE, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CCM_EFFACER, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CCM_TT_EFFACER, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CCM_FND_SOLUTES, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CCM_AVANT, +3, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CCM_FND_ASC_SOL, +3, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CCM_SOL_APRES, +3, +1, DEPLACE_PIXELS);

  OBJ_deplace(CALC_MUM, CMM_XMAX, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(CALC_MUM, CMM_PAS, 0, +2, DEPLACE_PIXELS);

  arbres[CALC_MUM].drapeaux.prepare = VRAI;
}

static boolean GER_ArbreMatrice(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  int etat;

  switch ( objet )
  {
  case CMM_SOLUTION:
    etat = OBJ_choisit( CALC_MUM, objet, ECHANGE );
    I_ActiverSolution( etat );
    if ( VRAI == etat ) {	/* On vient d'indiquer que c'est une solution */
      OBJ_choisit( CALC_MUM, CCM_PASTILLE, FAUX );
      I_ActiverPastille( FAUX );
    }
    return BI_EVENEMENT_TRAITE;

  case CCM_PASTILLE:
    etat = OBJ_choisit( CALC_MUM, objet, ECHANGE );
    I_ActiverPastille( etat );
    if ( VRAI == etat ) {	/* On vient d'indiquer que c'est une pastille */
      OBJ_choisit( CALC_MUM, CMM_SOLUTION, FAUX );
      I_ActiverSolution( FAUX );
    }
    return BI_EVENEMENT_TRAITE;
  }

  /* Ici : bouton grable avec les fonctions par dfaut */
  return BI_EVENEMENT_NON_TRAITE;
}

static void I_ActiverPastille(boolean activer)
{
  OBJ_active( CALC_MUM, CCM_POURCENT, activer );
  OBJ_active( CALC_MUM, CCM_MATRICE , activer );
  OBJ_active( CALC_MUM, CCM_DECRIT_MATRI, activer );
/*   OBJ_active( CALC_MUM, , activer ); */
}

static void I_ActiverSolution(boolean activer)
{
  OBJ_active( CALC_MUM, CMM_SOLVANT, activer );
  OBJ_active( CALC_MUM, CMM_CONC   , activer );
  OBJ_active( CALC_MUM, CMM_CHX_SOLVANT, activer );
  OBJ_active( CALC_MUM, CCM_CHARGER, activer );
  OBJ_active( CALC_MUM, CCM_SAUVER, activer );
  OBJ_active( CALC_MUM, CCM_NV_SOLUTE, activer );
  OBJ_active( CALC_MUM, CCM_TT_EFFACER, activer );

  /* Le dfileur des autres composants en solution */
/*   OBJ_active( CALC_MUM, , activer ); */
}
