/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998, Emmanuel CURIS

   Module d'affichage graphique des rsultats

   Trac des graphes, un seul graphe, graphes 2D rels ou complexes (X, f(X))
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"

#include "tracer.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* Toutes ces fonctions demandent la position de (x_min, y_min)  l'cran [x_BG, y_BG]
   Toutes ces fonctions demandent les chelles en vrai/pixel
 */

static void trace_reel  (Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
			 double x_min, double x_max, double y_min, double y_max,
			 double x_echelle, double y_echelle);

static void trace_creel (Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
			 double x_min, double x_max, double y_min, double y_max,
			 double x_echelle, double y_echelle);

static void trace_imag  (Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
			 double x_min, double x_max, double y_min, double y_max,
			 double x_echelle, double y_echelle);

static void trace_module(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
			 double x_min, double x_max, double y_min, double y_max,
			 double x_echelle, double y_echelle);

static void TRC_GrapheComplexe_Re (Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
				   double x_min, double x_max, double y_min, double y_max,
				   double x_echelle, double y_echelle, int erreurs, int couleur, int style, int marque);
static void TRC_GrapheComplexe_Im (Fenetre *fenetre, Graphe *graphe, int x, int y,
				   double x_min, double x_max, double y_min, double y_max,
				   double x_echelle, double y_echelle, int erreurs, int couleur, int style, int marque);
static void TRC_GrapheComplexe_Mod(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
				   double x_min, double x_max, double y_min, double y_max,
				   double x_echelle, double y_echelle, int erreurs, int couleur, int style, int marque);

static void TRC_GrapheReel_rT (Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
			       double x_min, double x_max, double y_min, double y_max,
			       double x_echelle, double y_echelle, int erreurs, int couleur, int style, int marque);

static void TRC_GrapheReel_XY (Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
			       double x_min, double x_max, double y_min, double y_max,
			       double x_echelle, double y_echelle, int erreurs, int couleur, int style, int marque);

static void TRC_GrapheSerie   (Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
			       double x_min, double x_max, double y_min, double y_max,
			       double x_echelle, double y_echelle, int erreurs, int couleur, int style, int marque);

static void TRC_Histogramme_DP(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG, 
			       double x_min, double x_max, double y_min, double y_max, 
			       double x_echelle, double y_echelle, int erreurs, int couleur, int style);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Trac d'un graphe dans un rectangle donn, de faon  occuper au mieux le rectangle -------- */

void TRC_GrapheDansRectangle(Fenetre *fenetre, struct graphe *graphe, int couleur, int style,
			     int x_HG, int y_HG, unsigned int larg, unsigned int haut)
{
  double echelle_X, echelle_Y;
  int retour;

  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p graphe=%p couleur=%d style=%d x_HG=%d y_HG=%d larg=%u haut=%u",
		  fenetre, graphe, couleur, style, x_HG, y_HG, larg, haut);

  if ( NULL == graphe ) {
    SORTIE_FONCTION("TRC_GrapheDansRectangle", "ERREUR : graphe==NULL", 0);
    return;
  }
  if ( ( 0 == larg ) || ( 0 == haut ) ) {
    (void) fprintf(stderr, "TRC_GrapheDansRectangle : dimensions absurdes [L=%u, H=%u]\n", larg, haut);
    SORTIE_FONCTION("TRC_GrapheDansRectangle", "ERREUR : dimensions absurdes", 0);
    return;
  }
  if ( graphe->x_min >= graphe->x_max ) {
    char *texte;

    texte = TXT_genere_chaine( "%s : X constants", graphe->nom );
    TRC_Z_Message( fenetre, x_HG, y_HG, larg, haut, texte );
    free( texte ); texte = NULL;

    SORTIE_FONCTION("TRC_GrapheDansRectangle", "ERREUR : X constants [min=%g, max=%g]", graphe->x_min, graphe->x_max);
    return;
  }
  if ( graphe->y_min >= graphe->y_max ) {
    char *texte;

    texte = TXT_genere_chaine( "%s : Y constants", graphe->nom );
    TRC_Z_Message( fenetre, x_HG, y_HG, larg, haut, texte );
    free( texte ); texte = NULL;

    SORTIE_FONCTION("TRC_GrapheDansRectangle", "ERREUR : X constants [min=%g, max=%g]", graphe->y_min, graphe->y_max);
    return;
  }

  /* chelles de trac, en vrai/pixel */
  retour = TRC_Echelles( larg, haut, graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, &echelle_X, &echelle_Y );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : problme dans le calcul des chelles !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }

  /* On trace, sans barres d'erreur
     Attention, le rectangle est indiqu avec le coin SUPRIEUR gauche, mais TRC_Graphe veut le coin INFRIEUR gauche
     ==> Si le rectangle dmarre au pixel y_HG et fait haut pixels de haut, il va jusqu'au pixel y_HG + (haut - 1)
  */
  TRC_Graphe( fenetre, graphe, couleur, style, BI_MARQUE_AUCUNE, 0, x_HG, y_HG + haut - 1, 
	      graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, echelle_X, echelle_Y);

  SORTIE_FONCTION("TRC_GrapheDansRectangle", "Fini (chelles : X=%g, Y=%g pixels/ux)", echelle_X, echelle_Y);
}

/* -------- Trac d'un graphe dans une rgion 2D du plan dj dfinie ---------\
 |  Renvoie les chelles utilises en X et en Y                                |
 |          ces chelles sont en unit vraie / pixels                          |
 |                                                                             |
 | fenetre          : fentre dans laquelle il faut dessiner [NULL : objet]    |
 | region           : la rgion 2D du plan qui est effectivement visible       |
 | axes_secondaires : indique quels axes sont  prendre comme axes secondaires |
 | graphe           : le graphe  dessiner (en fait, les infos qui vont avec)  |
 | couleur          : la couleur  utiliser  (-1 : celle du graphe)            |
 | style_ligne      : le style de ligne  utiliser (-1 : celui du graphe)      |
 | style_marque     : le style de marques  utiliser (-1 : celui du graphe)    |
 | mode_dessin      : indique les axes sur lesquels tracer des erreurs         |
 | echelle_X_p      : si non-NULL, contiendra l'chelle en X utilise          |
 | echelle_Y_p      : si non-NULL, contiendra l'chelle en Y utilise          |
 \---------------------------------------------------------------------------- */

void TRC_GrapheDansRegion2D(Fenetre *fenetre, InfosRegion2D *region, int axes_secondaires, struct graphe *graphe,
			    int couleur, int style_ligne, int style_marque, int mode_dessin,
			    double *echelle_X_p, double *echelle_Y_p)
{
  double echelle_X, echelle_Y;
  int retour;
  InfosAxe *axe_X, *axe_Y;

  /* Taille de la zone */
  if ( NULL == region ) return;
  if ( ( region->largeur < 2 ) || ( region->hauteur < 2 ) ) {
    /* Rgion trop petite : on ne va pas tracer ! */
    AVERTISSEMENT( "Rgion trop petite : L=%u, H=%u", region->largeur, region->hauteur );
    return;
  }

  /* On rcupre les axes  utiliser */
  if ( ( axes_secondaires & AXE_X ) && ( region->X_secondaire != NULL ) ) {
    axe_X = region->X_secondaire;
  } else {
    axe_X = &(region->axe_X);
  }

  if ( ( axes_secondaires & AXE_Y ) && ( region->Y_secondaire != NULL ) ) {
    axe_Y = region->Y_secondaire;
  } else {
    axe_Y = &(region->axe_Y);
  }

  /* Contrle des axes */
  retour = AXE_ValiderBornes( axe_X );
  if ( retour != 0 ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s, %s l.%u : Problme sur l'axe des X [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
#endif
    return;	/* Problme, axe des Y */
  }

  retour = AXE_ValiderBornes( axe_Y );
  if ( retour != 0 ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s, %s l.%u : Problme sur l'axe des Y [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
#endif
    return;	/* Problme, axe des Y */
  }

  /* Calcul des chelles, en vrai/pixel */
  retour = TRC_Z2D_Echelles( region, axes_secondaires, &echelle_X, &echelle_Y );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme en calculant les chelles [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return;
  }

  /* Trac du graphe --- Rgion & TRC_Graphe travaillent avec le coin INFRIEUR gauche comme rfrence */
  TRC_Graphe( fenetre, graphe, couleur, style_ligne, style_marque, mode_dessin, region->x_BG, region->y_BG,
	      axe_X->min, axe_X->max, axe_Y->min, axe_Y->max, echelle_X, echelle_Y );

  /* Renvoi des valeurs d'chelles, si ncessaire */
  if ( echelle_X_p != NULL ) *echelle_X_p = echelle_X;
  if ( echelle_Y_p != NULL ) *echelle_Y_p = echelle_Y;
}

/* -------- Trac d'un graphe dans la rgion du plan indique -------------------\
 |                                                                               |
 | fenetre              : la fentre  utiliser pour le dessin [NULL autoris]   |
 | graphe               : le graphe  dessiner                                   |
 | couleur              : la couleur de dessin  utiliser [<0 : celle du graphe] |
 | style_ligne          : le style de ligne  utiliser [<0 : celui du graphe]    |
 | style_marque         : le style de marques  utiliser [<0 : celui du graphe]  |
 | mode_dessin          : quelles barres d'erreur dessiner                       |
 |                               AXE_X = erreurs en X                            |
 |                               AXE_Y = erreurs en Y                            |
 |                               DEUX_AXES = les deux axes  la fois             |
 | x_BG, y_BG           : coin infrieur gauche de la zone o dessiner           |
 |                        == position de (x_min, y_min)  l'cran                |
 | x_min, x_max         : intervalle de dessin en X                              |
 | y_min, y_max         : intervalle de dessin en Y                              |
 | echelle_X, echelle_Y : les chelles de trac en X et en Y [en vrai/pixel]     |
 \------------------------------------------------------------------------------ */

void TRC_Graphe(Fenetre *fenetre, Graphe *graphe, int couleur, int style_ligne, int style_marque, int mode_dessin, int x_BG, int y_BG, 
		double x_min, double x_max, double y_min, double y_max, double echelle_X, double echelle_Y)
{
  int methode = 0;		/* Soit cartsien pour les y=f(x), et mode par dfaut pour la plupart */

  /* Vrifications usuelles de scurit... */
  if ( is_inf(x_min) || is_inf(x_max) || is_inf(y_min) || is_inf(y_max) ) return;
  if ( is_nan(x_min) || is_nan(x_max) || is_nan(y_min) || is_nan(y_max) ) return;
  if ( NULL == graphe ) {
    (void) fprintf(stderr, "%s, %s l.%u : Graphe NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return;
  }

  /* Conditions de trac */
  if ( graphe->affichage != NULL ) {
    methode = graphe->affichage->methode;
    if ( style_ligne < 0 ) { /* Valeurs par dfaut */
      style_ligne = graphe->affichage->style;
    }
    if( style_marque < 0 ) {
      style_marque = graphe->affichage->marque;
    }
    if ( couleur < 0 ) { /* Valeurs par dfaut */
      couleur = graphe->affichage->couleur;
    }
  } else {
    /* Le message ci-dessous est un peu gnant, car il arrive de devoir dessiner des graphes isols --> non-utilis en gnral ! */
/*     AVERTISSEMENT( "graphe sans paramtres d'affichage [%p = %s]", (void *) graphe, graphe->nom ); */
    if ( couleur < 0 ) couleur = NOIR;
    if ( style_ligne  < 0 ) style_ligne  = BI_CONTINU;
    if ( style_marque < 0 ) style_marque = BI_MARQUE_AUCUNE;
    if ( GRF_FCT_COMPLEXE == graphe->genre_donnees ) methode = GRF_C_IM_MOD; /* Car pas 0 */
  }

  switch ( graphe->genre_donnees )
  {
  case GRF_FCT_REELLE:
    if ( GRF_R_CARTESIEN == methode ) {
      if ( x_max < graphe->x_min ) return; /* Graphe hors de la zone choisie */
      if ( x_min > graphe->x_max ) return; /* Non-mettable en gnrique avant car bornes sans lien graphique direct si graphe polaire,... */
      if ( y_max < graphe->y_min ) return;
      if ( y_min > graphe->y_max ) return;

      TRC_GrapheReel_XY( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			 mode_dessin, couleur, style_ligne, style_marque);
    } else if ( GRF_R_POLAIRE == methode ) {
      TRC_GrapheReel_rT( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			 mode_dessin, couleur, style_ligne, style_marque);
    } else {
      AVERTISSEMENT( "Mthode inconnue [%d], graphes rels", methode );
    }
    return;

  case GRF_FCT_COMPLEXE:
    if ( x_max < graphe->x_min ) return; /* Graphe hors de la zone choisie */
    if ( x_min > graphe->x_max ) return; /* Non-mettable en gnrique avant car bornes sans lien graphique direct si graphe polaire,... */
    if ( y_max < graphe->y_min ) return;
    if ( y_min > graphe->y_max ) return;

    if ( GRF_C_IM_MOD == methode ) {
      TRC_GrapheComplexe_Im ( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			      mode_dessin, couleur, style_ligne, style_marque );
      TRC_GrapheComplexe_Mod( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			      mode_dessin, couleur + 1, style_ligne, style_marque );
    } else if ( GRF_C_RE_MOD == methode ) {
      TRC_GrapheComplexe_Re ( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			      mode_dessin, couleur, style_ligne, style_marque );
      TRC_GrapheComplexe_Mod( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			      mode_dessin, couleur + 1, style_ligne, style_marque );
    } else if ( GRF_C_RE_IM == methode ) {
      TRC_GrapheComplexe_Re ( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			      mode_dessin, couleur, style_ligne, style_marque );
      TRC_GrapheComplexe_Im ( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			      mode_dessin, couleur + 1, style_ligne, style_marque );
    } else {
      AVERTISSEMENT( "Mthode inconnue [%d], graphes complexes", methode );
    }
    return;

  case GRF_SERIE:
  case GRF_ERREUR_R:		/* En pratique, se redessine pareil ! */
    if ( x_max < graphe->x_min ) return; /* Graphe hors de la zone choisie */
    if ( x_min > graphe->x_max ) return; /* Non-mettable en gnrique avant car bornes sans lien graphique direct si graphe polaire,... */
    if ( y_max < graphe->y_min ) return;
    if ( y_min > graphe->y_max ) return;

    if ( graphe->x != NULL ) {
      TRC_GrapheReel_XY( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			 mode_dessin, couleur, style_ligne, style_marque );
      return;
    }
    TRC_GrapheSerie( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
		     mode_dessin, couleur, style_ligne, style_marque );
    return;

  case GRF_HISTOGRAMME:
    if ( GRF_H_EFFECTIF == methode ) {
      OPTION_A_FAIRE( "Histogramme en effectifs" );
    } else if ( GRF_H_DP == methode ) {
      TRC_Histogramme_DP( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y,
			  mode_dessin, couleur, style_ligne );
    } else if ( GRF_H_FREQUENCE == methode ) {
      OPTION_A_FAIRE( "Histogramme en frquence" );
    }

  default:
    AVERTISSEMENT( "Type de graphe inconnu ou non adapt [%d]", graphe->genre_donnees );
    break;
  }
}

/* ~-~-~-~-~-~-~-~-~ Dessin lmentaire de graphes, d'un type prcis ~-~-~-~-~-~-~-~-~ */

/* -------- Trac d'un graphe  valeurs complexes -------- */

static void TRC_GrapheComplexe_Im(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
				  double x_min, double x_max, double y_min, double y_max, 
				  double echelle_X, double echelle_Y,
				  int erreurs, int couleur, int style_ligne, int marque)
{
  Complexe *y;
  double *x;
  unsigned long i;

  /* Prparation du dessin */
  TRC_Couleur( couleur );

  /* Trac du graphe proprement dit */
  if ( style_ligne > 0 ) {
    TRC_Style( style_ligne );

    trace_imag( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y );
  }
  if ( marque > 0 ) {
    int x_ecran, y_ecran;

    for ( i = 0, x = graphe->x, y = (Complexe *) graphe->y ; i < graphe->nombre_point ; i++, x++, y++ ) {
      if ( ( *x > x_max ) || ( *x < x_min ) ) continue;
      if ( ( y->im > y_max ) || ( y->im < y_min ) ) continue;

      x_ecran = TRC_ConvertitX_PvE(    *x, x_min, x_BG, echelle_X );
      y_ecran = TRC_ConvertitY_PvE( y->im, y_min, y_BG, echelle_Y );

      TRC_Marque( fenetre, x_ecran, y_ecran, marque, 6 );
    }
  }

  /* Trac ventuel des barres d'erreur, en continu forcment */
  TRC_Style( SOLID );

  /* 1. Trac des erreurs en X */
  if ( ( erreurs & AXE_X ) && ( graphe->erreur_x != NULL ) ) {
    register double *err_x;
    int dx, x_vrai, y_vrai, x_fin;
    int points[ 4 ];

    /* Valeur maximale en X autorise */
    x_fin = TRC_ConvertitX_PvE( x_max, x_min, x_BG, echelle_X );

    for ( i = 0, x = graphe->x, y = (Complexe *) graphe->y, err_x = graphe->erreur_x->y ;
	  i < graphe->nombre_point ; i++, x++, y++, err_x++ ) {
      if ( ( y->im > y_max ) || ( y->im < y_min ) ) continue;
      if ( ( *x + *err_x < x_min ) || ( *x - *err_x > x_max ) ) continue;

      /* Position du point central */
      x_vrai = TRC_ConvertitX_PvE(    *x, x_min, x_BG, echelle_X );
      y_vrai = TRC_ConvertitY_PvE( y->im, y_min, y_BG, echelle_Y );

      /* Dimensions de la barre d'erreur  l'cran */
      dx = (int) rint( *err_x / echelle_X );

      /* Bornes de la droite */
      points[ 0 ] = x_vrai - dx; points[ 1 ] = y_vrai;
      points[ 2 ] = x_vrai + dx; points[ 3 ] = y_vrai;
      if ( points[ 0 ] < x_BG ) points[ 0 ] = x_BG;
      if ( points[ 2 ] > x_fin   ) points[ 2 ] = x_fin;

      TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );
    }
  }

  /* 2. Trac des erreurs en Y */
  if ( ( erreurs & AXE_Y ) && ( graphe->erreur_y != NULL ) ) {
    Complexe *err_y;
    unsigned long i;
    int x_ecran, y_msigma, y_psigma, y_HD;

    /* Valeur maximale en Y autorise */
    y_HD = TRC_ConvertitY_PvE( y_max, y_min, y_BG, echelle_Y );

    for ( i = 0, x = graphe->x, y = (Complexe *) graphe->y, err_y = (Complexe *) graphe->erreur_y->y ;
	  i < graphe->nombre_point ; i++, x++, y++, err_y++ ) {
      if ( ( *x > x_max ) || ( *x < x_min ) ) continue;
      if ( ( y->im + err_y->im < y_min ) || ( y->im - err_y->im > y_max ) ) continue;

      /* Position du point central */
      x_ecran = TRC_ConvertitX_PvE(*x, x_min, x_BG, echelle_X );

      /* Bornes de la droite */
      y_msigma = TRC_ConvertitY_PvE( (y->im - err_y->im), y_min, y_BG, echelle_Y );
      y_psigma = TRC_ConvertitY_PvE( (y->im + err_y->im), y_min, y_BG, echelle_Y );

      /* On bloque  la zone dfinie */
      if ( y_msigma > y_BG ) y_msigma = y_BG;
      if ( y_psigma < y_HD ) y_psigma = y_HD;

      TRC_Ligne( fenetre, x_ecran, y_msigma, x_ecran, y_psigma );
    }
  }
}

static void TRC_GrapheComplexe_Mod(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
				   double x_min, double x_max, double y_min, double y_max, 
				   double echelle_X, double echelle_Y,
				   int erreurs, int couleur, int style_ligne, int marque)
{
  Complexe *y;
  double *x, module;
  unsigned long i;

  /* Prparation du dessin */
  TRC_Couleur( couleur );

  /* Trac du graphe proprement dit */
  if ( style_ligne > 0 ) {
    TRC_Style( style_ligne );

    trace_module( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y );
  }
  if ( marque > 0 ) {
    int x_ecran, y_ecran;

    for ( i = 0, x = graphe->x, y = (Complexe *) graphe->y ; i < graphe->nombre_point ; i++, x++, y++ ) {
      if ( ( *x > x_max ) || ( *x < x_min ) ) continue;

      module = sqrt( y->re * y->re + y->im * y->im );
      if ( ( module > y_max ) || ( module < y_min ) ) continue;

      x_ecran = TRC_ConvertitX_PvE(     *x, x_min, x_BG, echelle_X );
      y_ecran = TRC_ConvertitY_PvE( module, y_min, y_BG, echelle_Y );

      TRC_Marque( fenetre, x_ecran, y_ecran, marque, 6 );
    }
  }

  /* Trac ventuel des barres d'erreur */
  TRC_Style( SOLID );

  /* 1. Trac des erreurs en X */
  if ( ( erreurs & AXE_X ) && ( graphe->erreur_x != NULL ) ) {
    register double *err_x;
    int dx, x_vrai, y_vrai, x_fin;
    int points[ 4 ];

    /* Valeur maximale en X autorise */
    x_fin = TRC_ConvertitX_PvE( x_max, x_min, x_BG, echelle_X );

    for ( i = 0, x = graphe->x, y = (Complexe *) graphe->y, err_x = graphe->erreur_x->y ;
	  i < graphe->nombre_point ; i++, x++, y++, err_x++ ) {
      module = sqrt( y->re * y->re + y->im * y->im );

      if ( ( module > y_max ) || ( module < y_min ) ) continue;
      if ( ( *x + *err_x < x_min ) || ( *x - *err_x > x_max ) ) continue;

      /* Position du point central */
      x_vrai = TRC_ConvertitX_PvE(     *x, x_min, x_BG, echelle_X );
      y_vrai = TRC_ConvertitY_PvE( module, y_min, y_BG, echelle_Y );

      /* Dimensions de la barre d'erreur  l'cran */
      dx = (int) rint( *err_x / echelle_X );

      /* Bornes de la droite */
      points[ 0 ] = x_vrai - dx; points[ 1 ] = y_vrai;
      points[ 2 ] = x_vrai + dx; points[ 3 ] = y_vrai;
      if ( points[ 0 ] < x_BG  ) points[ 0 ] = x_BG;
      if ( points[ 2 ] > x_fin ) points[ 2 ] = x_fin;

      TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );
    }
  }

  /* 2. Trac des erreurs en Y */
  if ( ( erreurs & AXE_Y ) && ( graphe->erreur_y != NULL ) ) {
    Complexe *err_y;
    unsigned long i;
    int x_ecran, y_msigma, y_psigma, y_HD;
    double A_FAIRE = 0.0;

    OPTION_A_FAIRE( "Estimation de la barre d'erreur sur le module" );

    /* Valeur maximale en Y autorise */
    y_HD = TRC_ConvertitY_PvE( y_max, y_min, y_BG, echelle_Y );

    for ( i = 0, x = graphe->x, y = (Complexe *) graphe->y, err_y = (Complexe *) graphe->erreur_y->y ;
	  i < graphe->nombre_point ; i++, x++, y++, err_y++ ) {
      if ( ( *x > x_max ) || ( *x < x_min ) ) continue;

      module = sqrt( y->re * y->re + y->im * y->im );
      if ( ( y->re + err_y->re < y_min ) || ( y->re - err_y->re > y_max ) ) continue;

      /* Position du point central */
      x_ecran = TRC_ConvertitX_PvE( *x, x_min, x_BG, echelle_X );

      /* Bornes de la droite */
      y_msigma = TRC_ConvertitY_PvE( ( module - A_FAIRE ), y_min, y_BG, echelle_Y );
      y_psigma = TRC_ConvertitY_PvE( ( module + A_FAIRE ), y_min, y_BG, echelle_Y );

      /* On bloque  la zone dfinie */
      if ( y_msigma > y_BG ) y_msigma = y_BG;
      if ( y_psigma < y_HD ) y_psigma = y_HD;

      TRC_Ligne( fenetre, x_ecran, y_msigma, x_ecran, y_psigma );
    }
  }
}

static void TRC_GrapheComplexe_Re(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG,
				  double x_min, double x_max, double y_min, double y_max, 
				  double echelle_X, double echelle_Y,
				  int erreurs, int couleur, int style_ligne, int marque)
{
  double *x;
  unsigned long i;
  Complexe *y;

  /* Prparation du dessin */
  TRC_Couleur( couleur );

  /* Trac du graphe proprement dit */
  if ( style_ligne > 0 ) {
    TRC_Style( style_ligne );

    trace_creel( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y );
  }
  if ( marque > 0 ) {
    int x_ecran, y_ecran;

    for ( i = 0, x = graphe->x, y = (Complexe *) graphe->y ; i < graphe->nombre_point ; i++, x++, y++ ) {
      if ( ( *x > x_max ) || ( *x < x_min ) ) continue;
      if ( ( y->re > y_max ) || ( y->re < y_min ) ) continue;

      x_ecran = TRC_ConvertitX_PvE(    *x, x_min, x_BG, echelle_X );
      y_ecran = TRC_ConvertitY_PvE( y->re, y_min, y_BG, echelle_Y );

      TRC_Marque( fenetre, x_ecran, y_ecran, marque, 6 );
    }
  }

  /* Trac ventuel des barres d'erreur */
  TRC_Style( SOLID );

  /* 1. Trac des erreurs en X */
  if ( ( erreurs & AXE_X ) && ( graphe->erreur_x != NULL ) ) {
    register double *err_x;
    int dx, x_vrai, y_vrai, x_fin;
    int points[ 4 ];

    /* Valeur maximale en X autorise */
    x_fin = TRC_ConvertitX_PvE( x_max, x_min, x_BG, echelle_X );

    for ( i = 0, x = graphe->x, y = (Complexe *) graphe->y, err_x = graphe->erreur_x->y ;
	  i < graphe->nombre_point ; i++, x++, y++, err_x++ ) {
      if ( ( y->re > y_max ) || ( y->re < y_min ) ) continue;
      if ( ( *x + *err_x < x_min ) || ( *x - *err_x > x_max ) ) continue;

      /* Position du point central */
      x_vrai = TRC_ConvertitX_PvE(    *x, x_min, x_BG, echelle_X );
      y_vrai = TRC_ConvertitY_PvE( y->re, y_min, y_BG, echelle_Y );

      /* Dimensions de la barre d'erreur  l'cran */
      dx = (int) rint( *err_x / echelle_X );

      /* Bornes de la droite */
      points[ 0 ] = x_vrai - dx; points[ 1 ] = y_vrai;
      points[ 2 ] = x_vrai + dx; points[ 3 ] = y_vrai;
      if ( points[ 0 ] < x_BG )  points[ 0 ] = x_BG;
      if ( points[ 2 ] > x_fin   ) points[ 2 ] = x_fin;

      TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );
    }
  }

  /* 2. Trac des erreurs en Y */
  if ( ( erreurs & AXE_Y ) && ( graphe->erreur_y != NULL ) ) {
    Complexe *err_y;
    unsigned long i;
    int x_ecran, y_msigma, y_psigma, y_HD;

    /* Valeur maximale en Y autorise */
    y_HD = TRC_ConvertitY_PvE( y_max, y_min, y_BG, echelle_Y );

    for ( i = 0, x = graphe->x, y = (Complexe *) graphe->y, err_y = (Complexe *) graphe->erreur_y->y ;
	  i < graphe->nombre_point ; i++, x++, y++, err_y++ ) {
      if ( ( *x > x_max ) || ( *x < x_min ) ) continue;
      if ( ( y->re + err_y->re < y_min ) || ( y->re - err_y->re > y_max ) ) continue;

      /* Position du point central */
      x_ecran = TRC_ConvertitX_PvE(*x, x_min, x_BG, echelle_X );

      /* Bornes de la droite */
      y_msigma = TRC_ConvertitY_PvE( (y->re - err_y->re), y_min, y_BG, echelle_Y );
      y_psigma = TRC_ConvertitY_PvE( (y->re + err_y->re), y_min, y_BG, echelle_Y );

      /* On bloque  la zone dfinie */
      if ( y_msigma > y_BG ) y_msigma = y_BG;
      if ( y_psigma < y_HD ) y_psigma = y_HD;

      TRC_Ligne( fenetre, x_ecran, y_msigma, x_ecran, y_psigma );
    }
  }
}

/* -------- Trac d'un graphe  valeurs relles (coordonnes polaires) -------- */

static void TRC_GrapheReel_rT(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG, 
			      double x_min, double x_max, double y_min, double y_max, 
			      double echelle_X, double echelle_Y, int erreurs, int couleur, int style_ligne, int marque)
{
  /* Contrle pour comprendre ce qu'il se passe */
  if ( ( NULL == graphe->x ) || ( NULL == graphe->y ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Appel avec un graphe contenant NULL pour x et y [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, graphe->nom );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* Prparation du dessin */
  TRC_Couleur( couleur );

  /* Trac du graphe proprement dit */
  if ( style_ligne > 0 ) {
    register double *r = graphe->x, *theta = graphe->y;
    double x, y;
    unsigned long i = 0;
    unsigned int compteur = 0;
    Point_Ecran points[ BI_NOMBRE_MAXIMAL_POINTS ];

    TRC_Style( style_ligne );

    while ( i < graphe->nombre_point ) { /* Tant qu'il reste des points */
      if ( isnan( *r ) || isnan( *theta ) ) {
	(void) fprintf(stderr, "%s,%s l.%u : Valeur NaN [X=%g, Y=%g i=%lu/%lu], abandon pour viter une boucle sans fin\n",
		       __FILE__, __FUNCTION__, __LINE__, *r, *theta, i, graphe->nombre_point );
	TRC_LigneBrisee( fenetre, compteur, points );
	FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
	return;
      }
      /* On convertit en cartsien */
      x = *r * cos( *theta ); y = *r * sin( *theta );

      while ( ( x < x_min ) || ( x > x_max ) || /* On dborde */
	      ( y < y_min ) || ( y > y_max ) ) {
	r++; theta++; /* On passe au point suivant */
	i++;
	if ( i >= graphe->nombre_point ) return; /* On s'arrte */

	x = *r * cos( *theta ); y = *r * sin( *theta );
      }

      compteur = 0; /* On commence un nouveau trac */
      while ( ( x >= x_min ) && ( x <= x_max ) && /* On est dans la zone de trac */
	      ( y >= y_min ) && ( y <= y_max ) ) {
	points[ compteur ].x = TRC_ConvertitX_PvE( x, x_min, x_BG, echelle_X );
	points[ compteur ].y = TRC_ConvertitY_PvE( y, y_min, y_BG, echelle_Y );
	compteur++;

	if ( compteur >= BI_NOMBRE_MAXIMAL_POINTS ) { /* Tableau de points plein */
	  TRC_LigneBrisee( fenetre, BI_NOMBRE_MAXIMAL_POINTS, points );
	  points[ 0 ] = points[ BI_NOMBRE_MAXIMAL_POINTS - 1 ];
	  compteur = 1;
	}
	r++; theta++;
	i++; /* Point suivant */

	if ( i >= graphe->nombre_point ) {
	  break;
	}

	x = *r * cos( *theta ); y = *r * sin( *theta );
      }
    
      /* On trace s'il reste des points */
      if ( compteur > 0 ) {
	TRC_LigneBrisee( fenetre, compteur, points );
      }
    }
  }
  if ( marque > 0 ) {
    int x_ecran, y_ecran;
    double *r, *theta, x, y;
    unsigned long i;

    for ( i = 0, r = graphe->x, theta = graphe->y ; i < graphe->nombre_point ; i++, r++, theta++ ) {
      /* Conversion polaire -> cartsien */
      x = *r * cos( *theta ); y = *r * sin( *theta );

      /* Le point est-il dans la zone ? */
      if ( ( x > x_max ) || ( x < x_min ) ) continue;
      if ( ( y > y_max ) || ( y < y_min ) ) continue;

      /* Oui : conversion en coordonnes cran */
      x_ecran = TRC_ConvertitX_PvE( x, x_min, x_BG, echelle_X );
      y_ecran = TRC_ConvertitY_PvE( y, y_min, y_BG, echelle_Y );

      /* Et affichage */
      TRC_Marque( fenetre, x_ecran, y_ecran, marque, 6 );
    }
  }

  /* Demande-t-on des erreurs ? */
  if ( 0 == erreurs ) return;	/* Non => termin */

  /* Trac des barres d'erreur */
  OPTION_A_FAIRE( "Incertitudes sur un graphe en polaire..." );
}

/* -------- Trac d'un graphe  valeurs relles (coordonnes cartsiennes) -------- */

static void TRC_GrapheReel_XY(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG, 
			      double x_min, double x_max, double y_min, double y_max, 
			      double echelle_X, double echelle_Y, int erreurs, int couleur, int style_ligne, int marque)
{
  /* Contrle pour comprendre ce qu'il se passe */
  if ( ( NULL == graphe->x ) || ( NULL == graphe->y ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Appel avec un graphe contenant NULL pour x et y [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, graphe->nom );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* Prparation du dessin */
  TRC_Couleur( couleur );

  /* Trac du graphe proprement dit */
  if ( style_ligne > 0 ) {
    TRC_Style( style_ligne );

    trace_reel( fenetre, graphe, x_BG, y_BG, x_min, x_max, y_min, y_max, echelle_X, echelle_Y );
  }
  if ( marque > 0 ) {
    int x_ecran, y_ecran;
    double *x, *y;
    unsigned long i;

    for ( i = 0, x = graphe->x, y = graphe->y ; i < graphe->nombre_point ; i++, x++, y++ ) {
      if ( ( *x > x_max ) || ( *x < x_min ) ) continue;
      if ( ( *y > y_max ) || ( *y < y_min ) ) continue;

      x_ecran = TRC_ConvertitX_PvE( *x, x_min, x_BG, echelle_X );
      y_ecran = TRC_ConvertitY_PvE( *y, y_min, y_BG, echelle_Y );

      TRC_Marque( fenetre, x_ecran, y_ecran, marque, 6 );
    }
  }

  /* Trac ventuel des barres d'erreur */
  TRC_Style( SOLID );

  /* 1. Trac des erreurs en X */
  if ( ( erreurs & AXE_X ) && ( graphe->erreur_x != NULL ) ) {
    register double *x, *y, *err_x;
    unsigned long i;
    int dx, x_vrai, y_vrai, x_fin;
    int points[ 4 ];

    /* Valeur maximale en X autorise */
    x_fin = TRC_ConvertitX_PvE( x_max, x_min, x_BG, echelle_X );

    for ( i = 0, x = graphe->x, y = graphe->y, err_x = graphe->erreur_x->y ;
	  i < graphe->nombre_point ; i++, x++, y++, err_x++ ) {
      if ( ( *y > y_max ) || ( *y < y_min ) ) continue;
      if ( ( *x + *err_x < x_min ) || ( *x - *err_x > x_max ) ) continue;

      /* Position du point central */
      x_vrai = TRC_ConvertitX_PvE( *x, x_min, x_BG, echelle_X );
      y_vrai = TRC_ConvertitY_PvE( *y, y_min, y_BG, echelle_Y );

      /* Dimensions de la barre d'erreur  l'cran */
      dx = (int) rint( *err_x / echelle_X );

      /* Bornes de la droite */
      points[ 0 ] = x_vrai - dx; points[ 1 ] = y_vrai;
      points[ 2 ] = x_vrai + dx; points[ 3 ] = y_vrai;
      if ( points[ 0 ] < x_BG  ) points[ 0 ] = x_BG;
      if ( points[ 2 ] > x_fin ) points[ 2 ] = x_fin;

      TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );
    }
  }

  /* 2. Trac des erreurs en Y */
  if ( ( erreurs & AXE_Y ) && ( graphe->erreur_y != NULL ) ) {
    register double *x, *y, *err_y;
    unsigned long i;
    int x_ecran, y_msigma, y_psigma, y_HD;

    /* Valeur maximale en Y autorise */
    y_HD = TRC_ConvertitY_PvE( y_max, y_min, y_BG, echelle_Y );

    for ( i = 0, x = graphe->x, y = graphe->y, err_y = graphe->erreur_y->y ;
	  i < graphe->nombre_point ; i++, x++, y++, err_y++ ) {
      if ( ( *x > x_max ) || ( *x < x_min ) ) continue;
      if ( ( *y + *err_y < y_min ) || ( *y - *err_y > y_max ) ) continue;

      /* Position du point central */
      x_ecran = TRC_ConvertitX_PvE( *x, x_min, x_BG, echelle_X );

      /* Bornes de la droite */
      y_msigma = TRC_ConvertitY_PvE( (*y - *err_y), y_min, y_BG, echelle_Y );
      y_psigma = TRC_ConvertitY_PvE( (*y + *err_y), y_min, y_BG, echelle_Y );

      /* On bloque  la zone dfinie */
      if ( y_msigma > y_BG ) y_msigma = y_BG;
      if ( y_psigma < y_HD ) y_psigma = y_HD;

      TRC_Ligne( fenetre, x_ecran, y_msigma, x_ecran, y_psigma );
    }
  }
}

/* -------- Srie de valeurs avec X implicites -------- */

static void TRC_GrapheSerie(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG, 
			    double x_min, double x_max, double y_min, double y_max, 
			    double echelle_X, double echelle_Y, int erreurs, int couleur, int style, int marque)
{
  unsigned long i_debut = 0;
  double *y;

  /* Prparation du dessin */
  TRC_Couleur( couleur );

  /* Trac du graphe proprement dit */
  /* 1. On saute les points initiaux */
  while ( i_debut < x_min ) {
    i_debut++;
    if ( i_debut >= graphe->nombre_point ) return; /* On est en-dehors de la rgion trace */
  }

  /* 2. On trace tous les points dedans */
  if ( style > 0 ) {
    unsigned int compteur;
    unsigned long i;
    Point_Ecran points[ BI_NOMBRE_MAXIMAL_POINTS ];

    TRC_Style( style );

    i = i_debut;
    y = graphe->y + i;
    while ( i <= x_max ) {
      while ( ( *y < y_min) || ( *y > y_max ) ) {
	i++; y++;
	if ( i >= graphe->nombre_point ) break; /* On s'arrte */
      }
      if ( i >= graphe->nombre_point ) break;

      compteur = 0; /* On commence un nouveau trac */
      while ( ( *y >= y_min ) && ( *y <= y_max ) ) {
	points[ compteur ].x = TRC_ConvertitX_PvE( (double) i, x_min, x_BG, echelle_X );
	points[ compteur ].y = TRC_ConvertitY_PvE(         *y, y_min, y_BG, echelle_Y );
	compteur++;


	if ( compteur >= BI_NOMBRE_MAXIMAL_POINTS ) { /* Tableau de points plein */
	  TRC_LigneBrisee( fenetre, BI_NOMBRE_MAXIMAL_POINTS, points );
	  points[ 0 ] = points[ BI_NOMBRE_MAXIMAL_POINTS - 1 ];
	  compteur = 1;
	}

	i++; y++; /* Point suivant */
	if ( i >= graphe->nombre_point ) break;
      }
    
      /* On trace s'il reste des points */
      if ( compteur > 0 ) {
	TRC_LigneBrisee( fenetre, compteur, points );
      }
    }
  }
  if ( marque > 0 ) {
    int x_ecran, y_ecran;
    unsigned long i;

    i = i_debut;
    y = graphe->y + i;
    while ( i <= x_max ) {
      if ( i >= graphe->nombre_point ) break;

      if ( ( *y >= y_min ) && ( *y <= y_max ) ) {
	x_ecran = TRC_ConvertitX_PvE( (double) i, x_min, x_BG, echelle_X );
	y_ecran = TRC_ConvertitY_PvE(         *y, y_min, y_BG, echelle_Y );

	TRC_Marque( fenetre, x_ecran, y_ecran, marque, 6 );
      }

      i++; y++;
    }
  }

  /* Trac ventuel des barres d'erreur */
  TRC_Style( BI_CONTINU );

  if ( ( erreurs & AXE_Y ) && ( graphe->erreur_y != NULL ) ) {
    register double *y, *err_y;
    unsigned long i;
    int x_ecran, y_msigma, y_psigma, y_HD;

    /* Valeur maximale en Y autorise */
    y_HD = TRC_ConvertitY_PvE( y_max, y_min, y_BG, echelle_Y );

    for ( i = 0, y = graphe->y, err_y = graphe->erreur_y->y ;
	  i < graphe->nombre_point ; i++, y++, err_y++ ) {
      if ( ( i > x_max ) || ( i < x_min ) ) continue;
      if ( ( *y + *err_y < y_min ) || ( *y - *err_y > y_max ) ) continue;

      /* Position du point central */
      x_ecran = TRC_ConvertitX_PvE( (double) i, x_min, x_BG, echelle_X );

      /* Bornes de la droite */
      y_msigma = TRC_ConvertitY_PvE( (*y - *err_y), y_min, y_BG, echelle_Y );
      y_psigma = TRC_ConvertitY_PvE( (*y + *err_y), y_min, y_BG, echelle_Y );

      /* On bloque  la zone dfinie */
      if ( y_msigma > y_BG ) y_msigma = y_BG;
      if ( y_psigma < y_HD ) y_psigma = y_HD;

      TRC_Ligne( fenetre, x_ecran, y_msigma, x_ecran, y_psigma );
    }
  }
}

/* ~-~-~-~-~-~-~-~-~ Trac d'lments de graphes, qui reviennent souvent... ~-~-~-~-~-~-~-~-~ */

/* -------- Trac simple d'un graphe, la couleur et le style tant mis, avec une droite ------ */

static void trace_reel(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG, 
		       double x_min, double x_max, double y_min, double y_max, 
		       double echelle_X, double echelle_Y)
{
  register double *x = graphe->x, *y = graphe->y;
  unsigned long i = 0;
  unsigned int compteur = 0;
  Point_Ecran points[ BI_NOMBRE_MAXIMAL_POINTS ];

  while ( i < graphe->nombre_point ) { /* Tant qu'il reste des points */
    if ( isnan( *x ) || isnan( *y ) ) {
      (void) fprintf(stderr, "%s,%s l.%u : Valeur NaN [X=%g, Y=%g i=%lu/%lu], abandon pour viter une boucle sans fin\n",
		     __FILE__, __FUNCTION__, __LINE__, *x, *y, i, graphe->nombre_point );
      FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
      return;
    }
    while ( (*x < x_min) || (*x > x_max) /* On dborde */
	 || (*y < y_min) || (*y > y_max) ) {
      x++; y++; /* On passe au point suivant */
      i++;
      if ( i >= graphe->nombre_point ) return; /* On s'arrte */
    }

    compteur = 0; /* On commence un nouveau trac */
    while ( (*x >= x_min) && (*x <= x_max) /* On dborde */
	 && (*y >= y_min) && (*y <= y_max) ) {
      points[ compteur ].x = TRC_ConvertitX_PvE( *x, x_min, x_BG, echelle_X );
      points[ compteur ].y = TRC_ConvertitY_PvE( *y, y_min, y_BG, echelle_Y );
      compteur++;

      if ( compteur >= BI_NOMBRE_MAXIMAL_POINTS ) { /* Tableau de points plein */
	TRC_LigneBrisee( fenetre, BI_NOMBRE_MAXIMAL_POINTS, points );
	points[ 0 ] = points[ BI_NOMBRE_MAXIMAL_POINTS - 1 ];
	compteur = 1;
      }
      x++; y++;
      i++; /* Point suivant */

      if ( i >= graphe->nombre_point ) {
	break;
      }
    }
    
    /* On trace s'il reste des points */
    if ( compteur > 0 ) {
      TRC_LigneBrisee( fenetre, compteur, points );
    }
  }
}

/* -------- Histogramme, reprsent en densit de frquence -------- */

static void TRC_Histogramme_DP(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG, 
			       double x_min, double x_max, double y_min, double y_max, 
			       double echelle_X, double echelle_Y, int erreurs, int couleur, int style __attribute__((__unused__)))
{
  double dp;
  unsigned long i, e_total = 0;
  int points[ 4 ];

  /* Calcul de l'effectif total */
  for ( i = 0 ; i <= graphe->nombre_point ; i++ )
    e_total += graphe->z.effectifs[ i ];

  /* La couleur du rectangle */
  TRC_Couleur( couleur );

  /* Les rectangles dmarrent  Y = 0, de toute faon */
  if ( y_min < 0.0 ) {
    if ( y_max <= 0.0 ) return; /* Une dp ne peut pas tre ngative ! */

    /* 0 est visible en Y : les rectangles commencent forcment l */
    points[ 1 ] = TRC_ConvertitY_PvE( 0.0, y_min, y_BG, echelle_Y );
  } else {
    /* 0 n'est pas visible */
    points[ 1 ] = TRC_ConvertitY_PvE( y_min, y_min, y_BG, echelle_Y );
  }

  /* La classe infrieure est-elle reprsente ? */
  /* Question sans rel fondement, puisque de toute faon, elle est de largeur infinie
     ==> sa densit de frquence relative est nulle ! */

  /* On reprsente chacune des classes */
  for ( i = 1 ; i < graphe->nombre_point ; i++ ) {
    double x_debut, x_fin;

    /* Est-elle reprsente ? */
    if ( graphe->x[ i - 1 ] <= x_min ) continue; /* On n'a pas encore atteint la rgion trace */
    if ( graphe->x[ i ] >= x_max ) break; /* On a fini, forcment ! */
    if ( graphe->x[ i ] >= graphe->x[ i - 1 ] ) {
      AVERTISSEMENT( "Doublons ou X non-tris [graphe %s, i=%lu]", graphe->nom, i );
    }

    /* Si on est l, c'est donc que l'on est reprsent */
    /* a. Coordonnes X de la classe  utiliser */
    x_debut = graphe->x[ i - 1 ];
    if ( x_debut < x_min ) x_debut = x_min;

    x_fin = graphe->x[ i ];
    if ( x_fin > x_max ) x_fin = x_max;

    /* b. Dfinition des points correspondants */
    points[ 0 ] = TRC_ConvertitX_PvE( x_debut, x_min, x_BG, echelle_X );
    points[ 2 ] = TRC_ConvertitX_PvE( x_fin  , x_min, x_BG, echelle_X );

    /* c. Calcul de la densit de frquence */
    dp = ( (double) graphe->z.effectifs[ i ] ) / (double) e_total;
    dp /= ( graphe->x[ i ] - graphe->x[ i - 1 ] );

    /* d. On vrifie qu'elle est visible en Y */
    if ( dp >= y_min ) { /* Oui */
      points[ 3 ] = TRC_ConvertitY_PvE( dp, y_min, y_BG, echelle_Y );

      TRC_RectanglePlein( fenetre, points[ 0 ], points[ 1 ], points[ 2 ] - points[ 0 ], points[ 3 ] - points[ 1 ] );

      /* e. Cela vaut alors la peine de tracer l'erreur, si on le demande */
      if ( erreurs & AXE_Y ) {
	OPTION_A_FAIRE( "Tracer l'erreur : barre verticale en milieu de classe" );
      }
    }
  }

  /* Inutile de regarder la dernire classe : de toute faon, sa dp vaut 0, elle est de largeur infinie ! */
}

/* -------- Trac simple de la partie relle d'un graphe,
            la couleur et le style tant mis, avec une droite ------ */

static void trace_creel(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG, 
			double x_min, double x_max, double y_min, double y_max, 
			double echelle_X, double echelle_Y)
{
  register double   *x = graphe->x;
  register Complexe *y = (Complexe *) graphe->y;
  unsigned long i = 0;
  unsigned int compteur = 0;
  Point_Ecran points[ BI_NOMBRE_MAXIMAL_POINTS ];

  while ( i < graphe->nombre_point ) { /* Tant qu'il reste des points */
    while ( ( *x    < x_min ) || ( *x    > x_max ) /* On dborde */
	 || ( y->re < y_min ) || ( y->re > y_max ) ) {
      x++; y++; /* On passe au point suivant */
      i++;
      if ( i >= graphe->nombre_point ) return; /* On s'arrte */
    }

    compteur = 0; /* On commence un nouveau trac */
    while ( ( *x    >= x_min ) && ( *x    <= x_max ) /* On est dedans */
	 && ( y->re >= y_min ) && ( y->re <= y_max ) ) {
      points[ compteur ].x = TRC_ConvertitX_PvE(    *x, x_min, x_BG, echelle_X );
      points[ compteur ].y = TRC_ConvertitY_PvE( y->re, y_min, y_BG, echelle_Y );
      compteur++;

      if ( compteur >= BI_NOMBRE_MAXIMAL_POINTS ) { /* Tableau de points plein */
	TRC_LigneBrisee( fenetre, BI_NOMBRE_MAXIMAL_POINTS, points );
	points[ 0 ] = points[ BI_NOMBRE_MAXIMAL_POINTS - 1 ];
	compteur = 1;
      }
      x++; y++;
      i++; /* Point suivant */

      if ( i >= graphe->nombre_point ) {
	break;
      }
    }
    
    /* On trace s'il reste des points */
    if ( compteur  > 0 ) {
      TRC_LigneBrisee( fenetre, compteur, points );
    }
  }
}

/* -------- Trac simple de la partie imaginaire d'un graphe,
            la couleur et le style tant mis, avec une droite ------ */

static void trace_imag(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG, 
		       double x_min, double x_max, double y_min, double y_max, 
		       double echelle_X, double echelle_Y)
{
  register double   *x = graphe->x;
  register Complexe *y = (Complexe *) graphe->y;
  unsigned long i = 0;
  unsigned int compteur = 0;
  Point_Ecran points[ BI_NOMBRE_MAXIMAL_POINTS ];

  while ( i < graphe->nombre_point ) { /* Tant qu'il reste des points */
    while ( ( *x    < x_min ) || ( *x    > x_max ) || /* On dborde */
	    ( y->im < y_min ) || ( y->im > y_max ) ) {
      x++; y++; /* On passe au point suivant */
      i++;
      if ( i >= graphe->nombre_point ) return; /* On s'arrte */
    }

    compteur = 0; /* On commence un nouveau trac */
    while ( ( *x    >= x_min ) && ( *x     <= x_max ) && /* On dborde */
	    ( y->im >= y_min ) && (  y->im <= y_max ) ) {
      points[ compteur ].x = TRC_ConvertitX_PvE(    *x, x_min, x_BG, echelle_X );
      points[ compteur ].y = TRC_ConvertitY_PvE( y->im, y_min, y_BG, echelle_Y );
      compteur++;

      if ( compteur >= BI_NOMBRE_MAXIMAL_POINTS ) { /* Tableau de points plein */
	TRC_LigneBrisee( fenetre, BI_NOMBRE_MAXIMAL_POINTS, points );
	points[ 0 ] = points[ BI_NOMBRE_MAXIMAL_POINTS - 1 ];
	compteur = 1;
      }
      x++; y++;
      i++; /* Point suivant */

      if ( i >= graphe->nombre_point ) {
	break;
      }
    }
    
    /* On trace s'il reste des points */
    if ( compteur  > 0 ) {
      TRC_LigneBrisee( fenetre, compteur, points );
    }
  }
}
/* -------- Trac simple du module d'un graphe,
            la couleur et le style tant mis, avec une droite ------ */

static void trace_module(Fenetre *fenetre, Graphe *graphe, int x_BG, int y_BG, 
		       double x_min, double x_max, double y_min, double y_max, 
		       double echelle_X, double echelle_Y)
{
  register double   *x = graphe->x;
  register Complexe *y = (Complexe *) graphe->y;
  double module;
  unsigned long i = 0;
  unsigned int compteur = 0;
  Point_Ecran points[ BI_NOMBRE_MAXIMAL_POINTS ];

  if ( y_max <= 0.0 ) return; /* Un module ne pouvant pas tre ngatif... */

  while ( i < graphe->nombre_point ) { /* Tant qu'il reste des points */
    module = sqrt( y->re * y->re + y->im * y->im );

    while ( ( *x < x_min ) || ( *x > x_max ) || /* On dborde */
	  (module  < y_min ) || ( module  > y_max ) ) {
      x++; y++; /* On passe au point suivant */
      i++;
      if ( i >= graphe->nombre_point ) return; /* On s'arrte */

      module = sqrt( y->re * y->re + y->im * y->im );
    }

    compteur = 0; /* On commence un nouveau trac */
    while ( (     *x >= x_min ) && (     *x <= x_max ) && /* On dborde */
	    ( module >= y_min ) && ( module <= y_max ) ) {
      points[ compteur ].x = TRC_ConvertitX_PvE(     *x, x_min, x_BG, echelle_X );
      points[ compteur ].y = TRC_ConvertitY_PvE( module, y_min, y_BG, echelle_Y );
      compteur++;

      if ( compteur >= BI_NOMBRE_MAXIMAL_POINTS ) { /* Tableau de points plein */
	TRC_LigneBrisee( fenetre, BI_NOMBRE_MAXIMAL_POINTS, points );
	points[ 0 ] = points[ BI_NOMBRE_MAXIMAL_POINTS - 1 ];
	compteur = 1;
      }
      x++; y++;
      i++; /* Point suivant */

      if ( i >= graphe->nombre_point ) {
	break;
      }
      module = sqrt( y->re * y->re + y->im * y->im );
    }
    
    /* On trace s'il reste des points */
    if ( compteur > 0 ) {
      TRC_LigneBrisee( fenetre, compteur, points );
    }
  }
}
