/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998, Emmanuel CURIS

   Module d'affichage graphique des rsultats

   Trac des images "classiques"
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "fichiers.h"
#include "figures.h"
#include "interface.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static Fenetre *fenetre_superposition = NULL;
static struct infos_superpose infos_superposition = {
  TRC_SUPERPOSE_COCHE | TRC_SUPERPOSE_TYPE | TRC_SUPERPOSE_AUTO,
  GR_TOUS,
  {
    0, 0, 0, 0, 8, 8, 90, 90,
    CADRE2D_AVEC_CADRE | CADRE2D_ECRAN,
    { NULL, NULL, AXE_AVEC_BORNES | AXE_AVEC_GRADUATIONS | AXE_AVEC_LEGENDE | AXE_AVEC_GRILLE | AXE_ECRAN,
      GRADUATION_DECIMALE, 0, 0.0, 0.0, 0.0, ' ', 0, 10, NOIR, NOIR, VIOLET, GRIS_FONCE, 3}, /* Axe X */
    { NULL, NULL, AXE_AVEC_BORNES | AXE_AVEC_GRADUATIONS | AXE_AVEC_LEGENDE | AXE_AVEC_GRILLE | AXE_VERTICAL | AXE_ECRAN,
      GRADUATION_DECIMALE, 0, 0.0, 0.0, 0.0, ' ', 0, 10, NOIR, NOIR, VIOLET, GRIS_FONCE, 3}, /* Axe Y */
    NULL, NULL,
    NOIR, BLANC
  }
};

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int TRC_OuvrirFenetre(void);

static int  CLC_Superposition(Fenetre *fenetre, int bouton, int n_clic, int etat, int x, int y);
static int  MSG_Superposition(Fenetre *fenetre, Message *message);
static void RCT_Superposition(Fenetre *fenetre, int x, int y, int vx_x, int vx_y);
static void RDS_Superposition(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut);

static void TRC_S_BoxplotsMultiples(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut);

static int TRC_InitialiserEchelle(struct infos_superpose *infos);
static int TRC_OptimiserEchelle  (unsigned int mode, double bornes[ 6 ], void *infos);

static inline void S_CalculerCoordonnesZoneDessin(Fenetre *fenetre);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Ouverture de la fentre de superposition -------- */

void TRC_OuvrirSuperposition(void)
{
  int retour;

  /* Doit-on ouvrir la fentre ? */
  if ( NULL == fenetre_superposition ) {
    retour = TRC_OuvrirFenetre();
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible d'ouvrir la fentre... [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return;
    }
  }

  /* On adapte en fonction des donnes actuelles */
  TRC_MettreAJourSuperposition();
}

/* -------- Cration & ouverture de la fentre -------- */

static int TRC_OuvrirFenetre(void)
{
  /* Prparation de la fentre */
  fenetre_superposition = FEN_CreerFenetreSimple( TOUS_SF_ASC, "Superposer des graphes", "",
						  RDS_Superposition, (void (*)(Fenetre *, int, int, int)) NULL,
						  (int (*)(Fenetre *, unsigned int, int)) NULL,
						  CLC_Superposition, MSG_Superposition, RCT_Superposition,
						  (int (*)(Fenetre *)) NULL );
  if ( NULL == fenetre_superposition ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la fentre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On place les drapeaux d'utilisation de la fentre */
  fenetre_superposition->drapeaux.redessin = VRAI;
  fenetre_superposition->msg_masque.plein_ecran  = VRAI;
  fenetre_superposition->msg_masque.taille       = VRAI;
  fenetre_superposition->msg_masque.fermeture    = VRAI;

  /* Ouverture de la fentre */
  FEN_Ouvrir( fenetre_superposition );

  /* Adaptation de la rgion de trac */
  TRC_RelatifVAbsolu( &(infos_superposition.dessin),
		      0, fenetre_superposition->w_haut - 1,
		      fenetre_superposition->w_larg, fenetre_superposition->w_haut );

  /* Cration russie, tout s'est bien pass */
  return 0;
}

/* -------- Mise  jour du cadre de superposition -------- */

void TRC_MettreAJourSuperposition(void)
{
  boolean convient, uX_compatibles = VRAI, lX_compatibles = VRAI, uY_compatibles = VRAI, lY_compatibles = VRAI;
  int retour;
  const char *lX_ref = NULL, *lY_ref = NULL, *uX_ref = NULL, *uY_ref = NULL;
  struct graphe *graphe, *graphe_type = NULL;
  unsigned long nbr_graphes = 0;

  if ( NULL == fenetre_superposition ) return; /* Pas de fentre ! */
  if ( FAUX == ( infos_superposition.type & TRC_SUPERPOSE_AUTO ) ) return; /* On ne met pas  jour ! */
  if ( NULL == serie_courante ) return;
  if ( NULL == serie_courante->premier_graphe ) return;

  /* a. Initialiser les chelles */
  retour = TRC_InitialiserEchelle( &infos_superposition );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans l'initialisation de l'chelle [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return;
  }

  /* b. tendre pour pouvoir tout afficher */
  for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
    convient = TRC_GrapheConvient( &infos_superposition, graphe );
    if ( VRAI == convient ) {
      nbr_graphes++; /* Un graphe de plus est affich */

      if ( graphe->x_min < infos_superposition.dessin.axe_X.min )
	infos_superposition.dessin.axe_X.min = graphe->x_min;
      if ( graphe->x_max > infos_superposition.dessin.axe_X.max )
	infos_superposition.dessin.axe_X.max = graphe->x_max;

      if ( graphe->y_min < infos_superposition.dessin.axe_Y.min )
	infos_superposition.dessin.axe_Y.min = graphe->y_min;
      if ( graphe->y_max > infos_superposition.dessin.axe_Y.max )
	infos_superposition.dessin.axe_Y.max = graphe->y_max;

      /* Y a-t-il compatibilit des lgendes ? */
      if ( VRAI == lX_compatibles ) {
	if ( graphe_type != NULL ) {
	  const char *legende;

	  legende = GRF_QuelleLegende( graphe, AXE_X );
	  if ( legende != lX_ref ) {
	    if ( strcmp( legende, lX_ref ) != 0 ) {
	      lX_compatibles = FAUX;
	    }
	  }
	} else {
	  lX_ref = GRF_QuelleLegende( graphe, AXE_X );
	}
      }

      if ( VRAI == lY_compatibles ) {
	if ( graphe_type != NULL ) {
	  const char *legende;

	  legende = GRF_QuelleLegende( graphe, AXE_Y );
	  if ( legende != lY_ref ) {
	    if ( strcmp( legende, lY_ref ) != 0 ) {
	      lY_compatibles = FAUX;
	    }
	  }
	} else {
	  lY_ref = GRF_QuelleLegende( graphe, AXE_Y );
	}
      }

      /* Y a-t-il compatibilit des units ? */
      if ( VRAI == uX_compatibles ) {
	if ( graphe_type != NULL ) {
	  const char *unite;

	  unite = GRF_QuelleUnite( graphe, AXE_X );
	  if ( ( unite != uX_ref ) && ( unite != NULL ) && ( uX_ref != NULL ) ) {
	    if ( strcmp( unite, uX_ref ) != 0 ) {
	      uX_compatibles = FAUX;
	    }
	  }
	} else {
	  uX_ref = GRF_QuelleUnite( graphe, AXE_X );
	}
      }

      if ( VRAI == uY_compatibles ) {
	if ( graphe_type != NULL ) {
	  const char *unite;

	  unite = GRF_QuelleUnite( graphe, AXE_Y );
	  if ( ( unite != uY_ref ) && ( unite != NULL ) && ( uY_ref != NULL ) ) {
	    if ( strcmp( unite, uY_ref ) != 0 ) {
	      uY_compatibles = FAUX;
	    }
	  }
	} else {
	  uY_ref = GRF_QuelleUnite( graphe, AXE_Y );
	}
      }

      /* Graphe modle : le dernier repr dans la liste */
      graphe_type = graphe;
    }
  }

  /* c. Mettre en place les lgendes en X, Y */
  if ( 1 == nbr_graphes ) {
    AXE_MettreLegende( &(infos_superposition.dessin.axe_X), GRF_QuelleLegende( graphe_type, AXE_X ) );
    AXE_MettreUnite  ( &(infos_superposition.dessin.axe_X), GRF_QuelleUnite  ( graphe_type, AXE_X ) );

    AXE_MettreLegende( &(infos_superposition.dessin.axe_Y), GRF_QuelleLegende( graphe_type, AXE_Y ) );
    AXE_MettreUnite  ( &(infos_superposition.dessin.axe_X), GRF_QuelleUnite  ( graphe_type, AXE_Y ) );
  } else {
    if ( VRAI == lX_compatibles ) {
      AXE_MettreLegende( &(infos_superposition.dessin.axe_X), lX_ref );
    } else {
      AXE_MettreLegende( &(infos_superposition.dessin.axe_X), "Grandeurs multiples" );
    }
    if ( VRAI == lY_compatibles ) {
      AXE_MettreLegende( &(infos_superposition.dessin.axe_Y), lY_ref );
    } else {
      AXE_MettreLegende( &(infos_superposition.dessin.axe_Y), "Grandeurs multiples" );
    }

    if ( VRAI == uX_compatibles ) {
      AXE_MettreUnite( &(infos_superposition.dessin.axe_X), uX_ref );
    } else {
      AXE_MettreUnite( &(infos_superposition.dessin.axe_X), "Unit"e_AIGU"s multiples" );
    }
    if ( VRAI == uY_compatibles ) {
      AXE_MettreUnite( &(infos_superposition.dessin.axe_Y), uY_ref );
    } else {
      AXE_MettreUnite( &(infos_superposition.dessin.axe_Y), "Unit"e_AIGU"s multiples" );
    }
  }

  /* Le redessin, s'il est forc, doit tre fait dans la fonction appelante */
}

/* ~-~-~-~-~-~-~-~-~ Gestion des messages de la fentre ~-~-~-~-~-~-~-~-~ */

static int MSG_Superposition(Fenetre *fenetre, Message *message)
{
  if ( WM_SIZED == message->type ) {	/* Taille de la fentre change */
    S_CalculerCoordonnesZoneDessin( fenetre );
    return VRAI;
  }

  if ( WM_CLOSED == message->type ) { /* Fermeture demande */
    (void) TRC_FermerSuperposition();
    return VRAI;
  }

  /* Message inconnu */
  return FAUX;
}

/* -------- Fermeture de la fenetre de superposition -------- */

int TRC_FermerSuperposition(void)
{
  /* Librer les lgendes & units des axes */
  free( infos_superposition.dessin.axe_X.legende ); infos_superposition.dessin.axe_X.legende = NULL;
  free( infos_superposition.dessin.axe_X.unite ); infos_superposition.dessin.axe_X.unite = NULL;
  free( infos_superposition.dessin.axe_Y.legende ); infos_superposition.dessin.axe_Y.legende = NULL;
  free( infos_superposition.dessin.axe_Y.unite ); infos_superposition.dessin.axe_Y.unite = NULL;

  /* Fermer la fenetre */
  FEN_Fermer( fenetre_superposition, VRAI );
  fenetre_superposition = NULL;

  /* Termin */
  return 0;
}

/* -------- Est-ce qu'un graphe doit tre pris en compte dans la superposition ? -------- */

boolean TRC_GrapheConvient(struct infos_superpose *infos, struct graphe *graphe)
{
  if ( NULL == graphe ) return FAUX; /* Aucun graphe... */
  if ( NULL == infos ) {
    /* Appel externe : on utilise la fenetre de superposition */
    infos = &infos_superposition;
  }

  /* Contrle le plus facile : on demande la coche, graphe non-coch */
  if ( infos->type & TRC_SUPERPOSE_COCHE ) {
    if ( FAUX == graphe->drapeaux.superpose )
      return FAUX;
  }

  /* Doit-on avoir concordance des types ? */
  if ( infos->type & TRC_SUPERPOSE_TYPE ) {
    if ( graphe->type != infos->type_graphe )
      return FAUX;
  }

  /* Doit-on avoir concordance des axes [X] ? = concordance des units */
  if ( infos->type & TRC_SUPERPOSE_AXES ) {
    const char *unite_ref, *unite;

    /* Unit du graphe contrl */
    unite = GRF_QuelleUnite( graphe, AXE_X );

    /* On compare au graphe courant */
    unite_ref = GRF_QuelleUnite( graphe_courant, AXE_X );

    /* Unit diffrentes */
    if ( unite_ref != unite ) {
      if ( ( NULL == unite_ref ) || ( NULL == unite ) )
	return FAUX; /* Un avec, un sans unit */

      if ( strcmp( unite, unite_ref ) != 0 )
	return FAUX;
    }
  }

  /* Si on est l : on n'a pas pu trouver d'incompatibilit */
  return VRAI;
}

/* -------- Trouver le 1er graphe qui convient : initialisation de l'chelle -------- */

static int TRC_InitialiserEchelle(struct infos_superpose *infos)
{
  Graphe *graphe;
  int type;

  /* Tient-on compte d'autre chose que de la coche ? */
  type = infos->type & ~( TRC_SUPERPOSE_AUTO | TRC_SUPERPOSE_COCHE );
  if ( 0 == type ) {
    if ( infos->type & TRC_SUPERPOSE_COCHE ) {
      /* On ne tient compte que de la coche : premier graphe coch de la srie convient pour initialiser */
      for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
	if ( VRAI == graphe->drapeaux.superpose ) {
	  /* Trouv : on initialise */
	  infos->dessin.axe_X.min = graphe->x_min;
	  infos->dessin.axe_X.max = graphe->x_max;
	  infos->dessin.axe_Y.min = graphe->y_min;
	  infos->dessin.axe_Y.max = graphe->y_max;

	  /* Inutile d'aller plus loin */
	  return 0;
	}
      }

      /* Si on est l : aucun graphe n'est marqu */
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s, %s l.%u : Aucun graphe coch\n",
		     __FILE__, __FUNCTION__, __LINE__);
#endif
      return -1;
    }

    /* Si on est l : on affiche tous les graphes sans aucun contrle */
    infos->dessin.axe_X.min = serie_courante->premier_graphe->x_min;
    infos->dessin.axe_X.max = serie_courante->premier_graphe->x_max;
    infos->dessin.axe_Y.min = serie_courante->premier_graphe->y_min;
    infos->dessin.axe_Y.max = serie_courante->premier_graphe->y_max;

    return 0;
  }

  /*  partir d'ici : on se fie aussi au type ou aux axes
     Dtermins  partir du graphe courant */
  if ( infos->type & TRC_SUPERPOSE_COCHE ) {
    if ( FAUX == graphe_courant->drapeaux.superpose ) {
      (void) fprintf(stderr, "%s, %s l.%u : Graphe courant non coch, dtermination du type/des axes impossibles\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -5;
    }
  }

  if ( infos->type & TRC_SUPERPOSE_TYPE ) {
    infos->type_graphe = graphe_courant->type;
  }

  /* Tout est dfini par le graphe courant */
  infos->dessin.axe_X.min = graphe_courant->x_min;
  infos->dessin.axe_X.max = graphe_courant->x_max;
  infos->dessin.axe_Y.min = graphe_courant->y_min;
  infos->dessin.axe_Y.max = graphe_courant->y_max;

  /* Initialisation correctement effectue */
  return 0;
}

/* -------- Optimiser l'chelle -------- */

static int TRC_OptimiserEchelle(unsigned int mode, double bornes[ 6 ], void *infos __attribute__((__unused__)))
{
  boolean premier = VRAI, convient;
  struct graphe *graphe;

  if ( NULL == serie_courante ) {
    bornes[ 0 ] = 0.0; bornes[ 1 ] = 0.0;
    bornes[ 2 ] = 0.0; bornes[ 3 ] = 0.0;
    bornes[ 4 ] = 0.0; bornes[ 5 ] = 0.0;
    return 0;
  }

  switch ( mode )
  {
  case BORNES_XY:
    for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
      convient = TRC_GrapheConvient( &infos_superposition, graphe );
      if ( VRAI == convient ) {
	if ( VRAI == premier ) {
	  bornes[ 0 ] = graphe->x_min;
	  bornes[ 1 ] = graphe->x_max;

	  bornes[ 2 ] = graphe->y_min;
	  bornes[ 3 ] = graphe->y_max;

	  premier = FAUX;
	} else {
	  if ( graphe->x_min < bornes[ 0 ] ) bornes[ 0 ] = graphe->x_min;
	  if ( graphe->x_max > bornes[ 1 ] ) bornes[ 1 ] = graphe->x_max;

	  if ( graphe->y_min < bornes[ 2 ] ) bornes[ 2 ] = graphe->y_min;
	  if ( graphe->y_max > bornes[ 3 ] ) bornes[ 3 ] = graphe->y_max;
	}
      }
    }
    break;

  case BORNES_X:
    for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
      convient = TRC_GrapheConvient( &infos_superposition, graphe );
      if ( VRAI == convient ) {
	if ( VRAI == premier ) {
	  bornes[ 0 ] = graphe->x_min;
	  bornes[ 1 ] = graphe->x_max;

	  premier = FAUX;
	} else {
	  if ( graphe->x_min < bornes[ 0 ] ) bornes[ 0 ] = graphe->x_min;
	  if ( graphe->x_max > bornes[ 1 ] ) bornes[ 1 ] = graphe->x_max;
	}
      }
    }
    break;

  case BORNES_Y:
    for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
      convient = TRC_GrapheConvient( &infos_superposition, graphe );
      if ( VRAI == convient ) {
	if ( VRAI == premier ) {
	  bornes[ 2 ] = graphe->y_min;
	  bornes[ 3 ] = graphe->y_max;

	  premier = FAUX;
	} else {
	  if ( graphe->y_min < bornes[ 2 ] ) bornes[ 2 ] = graphe->y_min;
	  if ( graphe->y_max > bornes[ 3 ] ) bornes[ 3 ] = graphe->y_max;
	}
      }
    }
    break;

  case BORNES_XpourY:
    for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
      convient = TRC_GrapheConvient( &infos_superposition, graphe );
      if ( VRAI == convient ) {
	int retour;
	double x_min, x_max;

	retour = GRF_QuelsXPourY( graphe, bornes[ 2 ], bornes[ 3 ], &x_min, &x_max );
	if ( retour != 0 ) continue; /* Graphe hors-intervalle */
	if ( VRAI == premier ) {
	  bornes[ 0 ] = x_min;
	  bornes[ 1 ] = x_max;

	  premier = FAUX;
	} else {
	  if ( x_min < bornes[ 0 ] ) bornes[ 0 ] = x_min;
	  if ( x_max > bornes[ 1 ] ) bornes[ 1 ] = x_max;
	}
      }
    }
    break;

  case BORNES_YpourX:
    for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
      convient = TRC_GrapheConvient( &infos_superposition, graphe );
      if ( VRAI == convient ) {
	int retour;
	double y_min, y_max;

	retour = GRF_QuelsYPourX( graphe, bornes[ 0 ], bornes[ 1 ], &y_min, &y_max );
	if ( retour != 0 ) continue; /* Graphe hors-intervalle */
	if ( VRAI == premier ) {
	  bornes[ 2 ] = y_min;
	  bornes[ 3 ] = y_max;

	  premier = FAUX;
	} else {
	  if ( y_min < bornes[ 2 ] ) bornes[ 2 ] = y_min;
	  if ( y_max > bornes[ 3 ] ) bornes[ 3 ] = y_max;
	}
      }
    }
    break;

  default:
    (void) fprintf(stderr, "%s, %s l.%u : Mode d'optimisation inconnu [%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, mode);
    return -1;
  }

  return 0;
}

/* -------- Demander un redessin, si quelque chose a chang --------
    Renvoie VRAI si le graphe indiqu appartient  la superposition

   Si VRAI == effacement, on se renseigne juste pour savoir s'il faudra redessiner aprs effacement
   sinon, on redessine (aprs avoir ventuellement refait un affichage, si VRAI = mise_a_jour)
 */

boolean TRC_RedessinerSuperposition(struct graphe *graphe, boolean effacement, boolean mise_a_jour)
{
  boolean convient;

  if ( NULL == fenetre_superposition ) return FAUX; /* Fentre ferme, rien  faire ! */
  if ( ( NULL == graphe ) && ( VRAI == effacement ) ) {
    /* On a effac un graphe qui tait sur la superposition : on force le redessin ! */
    TRC_MettreAJourSuperposition( ); /* Car bornes peuvent avoir chang */
    FEN_redessin( fenetre_superposition, BI_RDS_TOUT );
  }

  /* Est-ce que le graphe indiqu convient ? */
  convient = TRC_GrapheConvient( &infos_superposition, graphe );
  if ( FAUX == convient ) return FAUX;

  /* Ne fait-on que se renseigner avant effacement ? */
  if ( VRAI == effacement ) return VRAI;

  /* On remet ventuellement  jour */
  if ( VRAI == mise_a_jour ) {
    TRC_MettreAJourSuperposition();
  }

  /* On force le redessin */
  FEN_redessin( fenetre_superposition, BI_RDS_TOUT );

  /* Le graphe convenait */
  return VRAI;
}

/* -------- Redessin -------- */

static void RDS_Superposition(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  boolean convient;
  int retour;
  double echelle_X, echelle_Y;
  Graphe *graphe;

#ifdef _DEVELOPPE_
  if ( NULL == fenetre ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune fentre  l'appel !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
#endif

  /* On trace en blanc la rgion  redessiner */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  /* Dimensions de la zone de dessin  partir de celles de la fentre */
  if ( ( fenetre->w_haut < 100 ) || ( fenetre->w_larg < 100 ) ) {
    TRC_Message( fenetre, "Fentre trop petite" );
    return;
  }

  /* On trace le cadre */
  TRC_DessineRegion2D( fenetre, &(infos_superposition.dessin) );

  /* On en dduit les chelles */
  retour = TRC_Z2D_Echelles( &(infos_superposition.dessin), 0, &echelle_X, &echelle_Y );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans les chelles [X : %g, Y : %g]\n",
		   __FILE__, __FUNCTION__, __LINE__, echelle_X, echelle_Y);
    return;
  }

  /* Y a-t-il des graphes traables ? */
  if ( NULL == serie_courante ) {
    return;
  }

  /* On trace les graphes, avec leur style et couleur */
  for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
    convient = TRC_GrapheConvient( &infos_superposition, graphe );
    if ( FAUX == convient ) continue;

    if ( GRF_SERIE == graphe->genre_donnees ) {
      if ( ( graphe->affichage != NULL ) && ( GRF_S_BOXPLOT == graphe->affichage->methode ) ) {
	/* On n'affiche que les botes  moustache ! */
	OPTION_A_FAIRE( "Mieux organiser les appels pour le redessin / superposition" );

	TRC_S_BoxplotsMultiples( fenetre, x, y, larg, haut );

	return;
      }
    }

    TRC_Graphe( fenetre, graphe, -1, -1, -1, AXE_Y,
		infos_superposition.dessin.x_BG, infos_superposition.dessin.y_BG,
		infos_superposition.dessin.axe_X.min, infos_superposition.dessin.axe_X.max,
		infos_superposition.dessin.axe_Y.min, infos_superposition.dessin.axe_Y.max,
		echelle_X, echelle_Y );
  }
}

/* -------- Juxtaposition de boxplot multiples -------- */

static void TRC_S_BoxplotsMultiples(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  Graphe *graphe;
  InfosRegion2D zd_boxplot;
  boolean convient;
  int espace_boites, position_boite;
  unsigned long nbr_boites = 0, i = 0;

  /* On remet  0 le dessin */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  /* On trace l'axe en bas */
  zd_boxplot = infos_superposition.dessin;
  zd_boxplot.axe_X = infos_superposition.dessin.axe_Y;
  zd_boxplot.axe_X.proprietes &= ~AXE_VERTICAL;

  TRC_LigneH( fenetre, zd_boxplot.x_BG, zd_boxplot.y_BG, zd_boxplot.largeur );
  TRC_AxeGradue( fenetre, zd_boxplot.x_BG, zd_boxplot.y_BG, zd_boxplot.x_BG + zd_boxplot.largeur - 1, zd_boxplot.y_BG, &(zd_boxplot.axe_X), 10, 8 );

  /* Compter combien de graphes sont impliqus */
  for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
    convient = TRC_GrapheConvient( &infos_superposition, graphe );
    if ( FAUX == convient ) continue;

    if ( GRF_SERIE == graphe->genre_donnees ) {
      if ( ( graphe->affichage != NULL ) && ( GRF_S_BOXPLOT == graphe->affichage->methode ) ) {
	nbr_boites++;
      }
    }
  }

  /* On en dduit les carts entre les diffrentes botes / dessin horizontal
     En fait, compte l'espace associ  chaque bote...
   */
  espace_boites = infos_superposition.dessin.hauteur / nbr_boites;

  /* On trace les botes, une par une */
  for ( graphe = serie_courante->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
    convient = TRC_GrapheConvient( &infos_superposition, graphe );
    if ( FAUX == convient ) continue;

    if ( GRF_SERIE == graphe->genre_donnees ) {
      if ( ( graphe->affichage != NULL ) && ( GRF_S_BOXPLOT == graphe->affichage->methode ) ) {
	/* Dtermination de la position */
	position_boite  = zd_boxplot.y_BG - ( espace_boites >> 1 );
	position_boite -= i * espace_boites;

	/* Trac */
	if ( NULL == graphe->affichage->complement.infos_bmoustache ) {
	  int retour;

	  retour = TRC_GRF_PreparerBoiteAMoustache( graphe, 0.25, 0.75, 0.0, 1.0 );
	  if ( retour != 0 ) {
	    TRC_Message( fenetre, "Erreur dans la prparation du dessin" );
	    return;
	  }
	}

	(void) TRC_BoiteAMoustache( fenetre, &zd_boxplot, graphe, graphe->affichage->complement.infos_bmoustache, position_boite );
	TRC_Texte( fenetre, zd_boxplot.x_BG + (zd_boxplot.largeur >> 1), position_boite + ( espace_boites >> 1 ), BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_BAS, 0,
		   graphe->nom );

	/* La prochaine sera au-dessus */
	i++;
      }
    }
  }
}

/* -------- Convertir en coordonnes cran absolues la zone de dessin -------- */

static inline void S_CalculerCoordonnesZoneDessin(Fenetre *fenetre)
{
  TRC_RelatifVAbsolu( &infos_superposition.dessin, fenetre->w_x, fenetre->w_y + fenetre->w_haut, fenetre->w_larg, fenetre->w_haut );

#ifdef _DEVELOPPE_
  if ( infos_superposition.dessin.y_BG < infos_superposition.dessin.hauteur ) {
    (void) fprintf(stderr, "%s, %s l.%u : Bloc HORS-FENTRE [X=%d(%g%%), Y=%d(%g%%), L=%u(%g%%), H=%u(%g%%)]\n",
		   __FILE__, __FUNCTION__, __LINE__, infos_superposition.dessin.x_BG, infos_superposition.dessin.x_BG_pc, infos_superposition.dessin.y_BG, infos_superposition.dessin.y_BG_pc,
		   infos_superposition.dessin.largeur, infos_superposition.dessin.larg_pc, infos_superposition.dessin.hauteur, infos_superposition.dessin.haut_pc);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
  }
#endif
}

/* -------- Gestion d'un clic souris -------- */

static int CLC_Superposition(Fenetre *fenetre __attribute__((__unused__)), int bouton, int n_clic, int etat, int x, int y)
{
  int retour;

  if ( etat & K_CTRL ) n_clic++; /* Ctrl-clic simule le double-clic */

  if ( 1 == bouton ) {
    unsigned int position_clic;

    position_clic = TRC_PositionDansRegion2D( &(infos_superposition.dessin), x, y, 5 );
    if ( TRC_EST_HORS_REGION == position_clic ) return VRAI;

    if ( 1 == n_clic ) {
      /* Clic simple : on essaye de changer la taille de la rgion 2D
	 Tous les bords sont actifs : si on est sur un bord, on fait croire que l'on est au milieu !
      */
      if ( TRC_EST_BORD_GAUCHE == position_clic ) position_clic = TRC_EST_MILIEU_BORD_GAUCHE;
      if ( TRC_EST_BORD_DROIT  == position_clic ) position_clic = TRC_EST_MILIEU_BORD_DROIT;
      if ( TRC_EST_BORD_BAS    == position_clic ) position_clic = TRC_EST_MILIEU_BORD_BAS;
      if ( TRC_EST_BORD_HAUT   == position_clic ) position_clic = TRC_EST_MILIEU_BORD_HAUT;
      retour = I_BLOC_ChangerTaille_Souris( fenetre, &(infos_superposition.dessin),
					    0, fenetre->w_haut - 1, fenetre->w_larg, fenetre->w_haut,
					    position_clic );
      if ( retour < 0 ) return VRAI;
      if ( 0 == retour ) {
	S_CalculerCoordonnesZoneDessin( fenetre );
	FEN_redessin( fenetre, BI_RDS_UTIL );
	return VRAI;
      }
 
      /* 2e essai : le dplacement du bloc, si shift est enfonce */
      if ( ( etat & K_LSHIFT ) || ( etat & K_RSHIFT ) ) {
	retour = I_BLOC_Deplacer_Souris( fenetre, &(infos_superposition.dessin),
					 0, fenetre->w_haut - 1, fenetre->w_larg, fenetre->w_haut,
					 x - infos_superposition.dessin.x_BG, infos_superposition.dessin.y_BG - y );
	if ( 0 == retour ) {	/* Changement confirm */
	  S_CalculerCoordonnesZoneDessin( fenetre );
	  FEN_redessin( fenetre, BI_RDS_UTIL );
	}
	return VRAI;
      }

      /* 3e essai : zooms dans le bloc, si le bloc s'y prte */
      retour = I_BLOC_ZoomerSouris( fenetre, &infos_superposition.dessin, 0, x, y );
      if ( retour < 0 ) return VRAI; /* Erreur ou annulation */
      if ( 0 == retour ) {
	FEN_redessin( fenetre, BI_RDS_UTIL );
	return VRAI;
      }
    } else {
      boolean redessiner = FAUX;

      if ( TRC_EST_AU_CENTRE == position_clic ) {
	int retour;

	retour = I_ParametrerBornes( &(infos_superposition.dessin.axe_X), &(infos_superposition.dessin.axe_Y), NULL, TRC_OptimiserEchelle, NULL, fenetre_superposition );
	if ( 0 == retour ) redessiner = VRAI;
      }
      if ( position_clic & TRC_EST_BORD_GAUCHE ) {
	I_ParametrerLegendes( &(infos_superposition.dessin.axe_Y), AXE_Y, VRAI, fenetre_superposition );
	redessiner = VRAI;
      }
      if ( position_clic & TRC_EST_BORD_BAS ) {
	I_ParametrerLegendes( &(infos_superposition.dessin.axe_X), AXE_X, VRAI, fenetre_superposition );
	redessiner = VRAI;
      }

      if ( VRAI == redessiner ) {
	FEN_redessin( fenetre, BI_RDS_UTIL );
      }
    }

    return VRAI;
  }

  /* Si on est l : on appelle un menu contextuel */
  /* Pour l'instant : appel de l'export PS */
  retour = EXP_PS_Superposition( &infos_superposition );

  /* Termin */
  return VRAI;
}

/* ~~~~~~~~~~~~~ Gestion du "rticule" sur la superposition ~~~~~~~~~~~~~~~~~~~~~~~~~~ */

static void RCT_Superposition(Fenetre *fenetre, int x, int y, int vx_x, int vx_y)
{
  int avec_zone_dessin;
  char *infos;

  if ( NULL == graphe_courant ) {
    return; /* Pas de graphe -> pas de pointeur */
  }

  /* Utilise-t-on une zone de dessin complte ? */
  avec_zone_dessin = 1/* P_UtiliserZoneDessin() */;

  /* Cas simple : aucune zone de dessin -> pas de pointeur */
  if ( -1 == avec_zone_dessin ) {
    return;
  }

  /* On prpare le dessin */
  TRC_InitPointeur;

  /* Cas gnral : on utilise compltement la zone de dessin */
  if ( 1 == avec_zone_dessin ) {
    boolean etait_dedans = FAUX;
    int retour;
    int x_HD, y_HD;
    Bloc_Figure pseudo_bloc;	/* Juste pour profiter des fonctions de dessin correspondantes ! */

    /* Contrle de la validit des bornes */
    retour = AXE_ValiderBornes( &(infos_superposition.dessin.axe_X) );
    if ( retour != 0 ) {
      return;
    }
    retour = AXE_ValiderBornes( &(infos_superposition.dessin.axe_Y) );
    if ( retour != 0 ) {
      return;
    }
    pseudo_bloc.dessin = infos_superposition.dessin;

    /* Coordonnes de la zone de dessin */
    x_HD = infos_superposition.dessin.x_BG + infos_superposition.dessin.largeur - 1;
    y_HD = infos_superposition.dessin.y_BG - infos_superposition.dessin.hauteur - 1;

    /* Effacement du rticule, si on tait dedans */
    if ( ( vx_x >= (int) infos_superposition.dessin.x_BG ) && ( vx_x <= x_HD ) &&
	 ( vx_y <= (int) infos_superposition.dessin.y_BG ) && ( vx_y >= y_HD ) ) {
      BLOC_TracerReticule( fenetre, &pseudo_bloc, AXE_X | AXE_Y, vx_x, vx_y );
      etait_dedans = VRAI;	/* On tait donc dj dedans */
    }

    /* Est-on maintenant dedans ou dehors ? */
    if ( ( x >= (int) infos_superposition.dessin.x_BG ) && ( x <= x_HD ) &&
	 ( y <= (int) infos_superposition.dessin.y_BG ) && ( y >= y_HD ) ) {
      double x_reel, y_reel;

      /* On est dedans */
      BLOC_TracerReticule( fenetre, &pseudo_bloc, AXE_X | AXE_Y, x, y );

      /* Recherche des coordonnes */
      retour = TRC_Z2D_EcranVXY( &infos_superposition.dessin, 0, x, y, &x_reel, &y_reel );
      if ( retour != 0 ) {
	infos = TXT_genere_chaine( "X = ??, Y = ??" );
      } else {
	infos = TXT_genere_chaine( "X = %f, Y = %f", x_reel, y_reel );
      }

      FEN_infos(fenetre, infos);
      free( infos ); infos = NULL;
    } else {
      /* On est dehors */
      if ( VRAI == etait_dedans ) {
	/* On vient de sortir ! */
	FEN_infos( fenetre, "Superposition de graphes..." );
      }
    }

    return;
  }

  OPTION_A_FAIRE( "Rticule, autre type de graphe" );
}
