/* Fonctions utilitaires générales
   Développées pour LASE, mais plus génériques
   (C) 1996 - 2002, Emmanuel CURIS

   Lecture "avancée" des fichiers en ASCII
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>		/* Pour la définition de NAN */

#define DEBOGUE_MALLOC_ 1

#include "ascii_utils.h"
#include "txt_utils.h" /* Pour TXT_CNV_vNombre_D */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static char *lit_ligne_taille_fixe(FILE *fichier, unsigned long taille);
static char *lit_ligne_quelconque (FILE *fichier);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* -------- Décompte du nombre de colonnes de la ligne active --------
            NE MODIFIE PAS LA POSITION DANS lE FICHIER
*/

unsigned int compte_colonnes(FILE *fichier, int separateur)
{
  char *ligne;
  long debut;
  unsigned int nombre_colonnes = 0;

  if ( NULL == fichier ) return 0;

  debut = ftell( fichier ); /* On se place là où on commence à compter */

  /* On lit la ligne [longueur inconnue] */
  ligne = lit_ligne_quelconque( fichier );
  if ( NULL == ligne ) {
    return 0;
  }

  /* Si un séparateur a été indiqué */
  if ( separateur != 0 ) {
    char *c;

    c = ligne;
    do {
      if ( separateur == *c ) {
	nombre_colonnes++;
      }
      c++;
    } while ( *c != 0 ); /* Tant que l'on n'a pas atteint la fin de la chaîne */

    /* nombre_colonnes contient ici le nombre de séparateurs, or on veut le nombre de colonnes : +1 */
    nombre_colonnes++;

    /* Remarque : on peut donc avoir, ici, des colonnes vides si deux séparateurs se suivent... */
  } else { /* espaces et assimilés */
    char *c;

    c = ligne;
    do {
      /* Espaces, tabs,... : on est entre deux colonnes [ou avant la première ou après la dernière] */
      while ( isspace( *c ) ) {
	c++;
	if ( 0 == *c ) break;
      }
      if ( 0 == *c ) break; /* Fin de chaîne atteinte */

      /* Tant que l'on est sur autre chose : on est sur une colonne */
      nombre_colonnes++;
      while ( FAUX == isspace( *c ) ) {
	c++;
	if ( 0 == *c ) break;
      }
    } while ( *c != 0 ); /* Tant que l'on n'a pas atteint la fin de la chaîne */
  }

  free( ligne ); /* Plus besoin d'elle */
  fseek(fichier, debut, SEEK_SET); /* On revient avant le début du décompte */
  return nombre_colonnes;
}

/* -------- Décompte du nombre de lignes
            compatible mac/unix/pc si le fichier a été ouvert en mode binaire !
	    Les lignes commençant par commentaire sont ignorées (sauf si commentaire = 0)
   -------- */

unsigned long compte_lignes(FILE *fichier, int commentaire)
{
  int c;
  unsigned long nombre_lignes = 0;
  long debut;

  debut = ftell(fichier); /* On se place là où on commence à compter */

  do {
    c = fgetc(fichier); /* On lit un caractère */
    if (c == EOF) break;

    if ( (commentaire == 0) || (c != commentaire) ) { /* Ce n'est pas une ligne de commentaire */
      nombre_lignes++; /* Donc une ligne de données de plus */
    }

    c = ignore_ligne(fichier); /* On ignore la fin de la ligne */
  } while (c != EOF); /* Tant qu'il y a des données */

  fseek(fichier, debut, SEEK_SET); /* On revient avant le début du décompte */

  return nombre_lignes;
}

/* -------- Ignorer un nombre donné de caractères
     Lit & ignore les n caractères suivants du fichier
     Renvoie EOF si on atteint la fin du fichier
             -1 si on atteint la fin de la ligne avant d'avoir sauté les n cars
             -2 si on demande de ne sauter aucun caractère
             le dernier caractère ignoré sinon

     Note : ne saute jamais à la ligne suivante, s'arrête avant !
   -------- */

int ignore_car(FILE *fichier, unsigned int n_cars)
{
  unsigned int i;
  int c = -2; /* Si n_cars = 0. Mais suprenant ! */

  for (i = 0 ; i < n_cars ; i++) {
    c = fgetc(fichier);
    if (EOF == c) {
      return EOF;
    } else if ( ('\r' == c) || ('\n' == c) ) {
      (void) ungetc(c, fichier); /* On annule le saut de ligne... */
      return -1;
    }
  }

  return c;
}

/* -------- Ignorer une fin de ligne
     Lit le fichier jusqu'à ce que l'on rencontre une fin de ligne (mac, pc, unix)
     puis se place au premier caractère suivant qui n'est pas une fin de ligne
     Renvoie EOF si on atteint la fin du fichier au passage, le premier caractère de la nouvelle ligne sinon

     Note : ignore aussi les lignes vides (ou équivalentes : que espaces, que tab) qui suivent
   -------- */

int ignore_ligne(FILE *fichier)
{
  int ligne_vide = FAUX;
  int c;

/*   printf("ignore_ligne : ligne ignorée ci-dessous...\n"); */
  /* On lit tout jusqu'à la fin d'une ligne */
  do {
    c = fgetc(fichier);
/*     putchar( c ); */
    if (c == EOF)
      return EOF;
  } while ( (c != '\n') && (c != '\r') );

  do {
    /* On saute la fin de ligne */
    while ( ( c == '\n' ) || ( c == '\r' ) ) {
      c = fgetc(fichier);
      if (c == EOF)
	return EOF;
    }

    ungetc(c, fichier); /* On annule la lecture du dernier = début de la ligne suivante */
    /* Ligne commençant par espace : est-ce une ligne vide ? */
    if ( isspace( c ) ) {
      long debut;
      int c_debut;

      c_debut = c;
      debut = ftell( fichier );
      while ( (c != '\n') && (c != '\r') && isspace( c ) ) {
	c = fgetc( fichier );
	if ( c == EOF ) /* Fichier se termine par une ligne avec espaces seulement */
	  return EOF;
      }
      if ( (c != '\n') && (c != '\r') ) { /* Ligne non vide, commençant par des espaces */
	fseek(fichier, debut, SEEK_SET ); /* On se remet donc au début */
	return c_debut;
      }
    } else { /* Ligne ne commençant pas par un espace : aucun problème */
      return c;
    }
  } while ( ligne_vide == VRAI );

  /*   putchar( '\n' ); */

  /* Si on est là, c'est qu'il y a un souci ! */
  (void) fprintf(stderr, "%s,%s l.%u : On ne devrait pas être là\n",
		 __FILE__, __FUNCTION__, __LINE__);
  return 0; 
}

/* -------- Sauter n lignes
            Lit le fichier jusqu'à avoir passé nbr_lignes lignes
	    Fin de ligne compatible mac, pc, unix
	    Après l'opération, on est au début de la ligne suivant les lignes sautées
   -------- */

int sauter_lignes(FILE *fichier, unsigned int nbr_lignes)
{
  int c;
  unsigned int i;

  for (i = 0 ; i < nbr_lignes ; i++) {
    c = fscanf(fichier, "%*[^\r\n]"); /* Sauter tout ce qui n'est pas une fin de ligne */
    if (c == EOF) {
      fprintf(stderr, "sauter_lignes : Moins de lignes que de lignes à passer!\n");
      return FAUX;
    }
#ifdef _PURE_C_ /* Problème dans la lib. standard Pure C ? */
    while ( ( (c = fgetc(fichier)) == '\r' ) || (c == '\n') );
    ungetc(c, fichier);
#else
    c = fscanf(fichier, "%*[\r\n]"); /* Sauter tous les codes de fins de ligne */
#endif
    if (c == EOF) {
      fprintf(stderr, "Moins de lignes que de lignes à passer!\n");
      return FAUX;
    }
  }

  return VRAI; /* On a pu sauter les n lignes */
}

/* -------- Sauter un en-tête (cad x lignes, mais commentaires ignorés) -------- */

int sauter_en_tete(FILE *fichier, unsigned int nbr_lignes, int commentaire)
{
  int c;
  unsigned int i = 0;

  if ( NULL == fichier ) return EOF;
  if ( 0 == nbr_lignes ) {
    c = fgetc(fichier);
    ungetc( c, fichier );
    return c; /* Rien à faire ! Mais on doit renvoyer le 1er caractère du fichier... */
  }

  c = fgetc(fichier);
  do {
    if ( EOF == c ) {
      (void) fprintf(stderr, "sauter_en_tete : fin de fichier atteinte !\n");
      return EOF;
    }
    if ( c != commentaire ) /* Les lignes de commentaire ne sont pas comptées dans l'en-tête */
      i++;
    c = ignore_ligne( fichier );
  } while ( i < nbr_lignes );

  /* Tout s'est bien passé */
  return c;
}

/* -------- Lire une ligne, de longueur a priori inconnue, dans un fichier ---------------\
 |  Renvoie la chaîne correspondant à la ligne (NULL en cas d'erreur) créée avec malloc() |
 |                                                                                        |
 | S'arrête sur une fin de ligne ou de fichier, mais se positionne sur la prochaine ligne |
 | Saute les lignes vides, ou avec seulement des espaces                                  |
 | lg_ligne : la longueur de ligne attendue (message d'avertissement si différente)       |
 |             si 0, on ne sait pas -> on lit quelle que soit sa taille                   |
 \--------------------------------------------------------------------------------------- */

char *lit_ligne(FILE *fichier, unsigned long lg_ligne)
{
  char *ligne = NULL;

  if ( NULL == fichier ) {
    (void) fprintf( stderr, "%s,%s l.%u : aucun fichier indiqué\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return NULL;
  }

  do {
    unsigned long i;
    int est_vide = VRAI;

    if ( 0 == lg_ligne )
      ligne = lit_ligne_quelconque( fichier );
    else
      ligne = lit_ligne_taille_fixe( fichier, lg_ligne );

    if ( NULL == ligne ) { /* Erreur survenue */
      return NULL;
    }

    /* On contrôle qu'il y a autre-chose que des espaces */
    for ( i = 0 ; i < strlen( ligne ) ; i++ ) {
      if ( FAUX == isspace( ligne[ i ] ) ) {
	est_vide = FAUX;
	break;
      }
    }
    if ( VRAI == est_vide ) { /* Ligne d'espaces => ignorée */
      free( ligne );
      ligne = NULL;
    }

  } while ( NULL == ligne );

  return ligne;
}

/* -------- Lecture d'une ligne de taille donnée (pour fichiers formatés type PDB) --------
  S'assure que la chaîne contenant la ligne fait toujours exactement cette taille,
  ignore la fin si la ligne est trop longue, ne déborde jamais sur la suivante,
  remplit de 0 si elle est trop courte, ne néglige pas les espaces finaux ou initiaux
 */

static char *lit_ligne_taille_fixe(FILE *fichier, unsigned long taille)
{
  char *ligne;
  unsigned long i;
  int c = 0;

  ligne = (char *) malloc( sizeof(char) * (taille + 1) ); /* +1 : le 0 final ! */
  if (NULL == ligne) {
    (void) fprintf(stderr, "lit_ligne_taille_fixe : mémoire insuffisante pour une ligne de %lu caractères\n", taille);
    return NULL;
  }

  /* On lit les caractères */
  for (i = 0 ; i < taille ; i++) {
    c = fgetc(fichier);
    if ( EOF == c ) {
      if ( 0 == i ) { /* Fin de fichier atteinte : ligne vide ! */
	(void) fprintf(stderr, "lit_ligne_taille_fixe : ligne vide, fin de fichier atteinte !\n");
	free( ligne ); ligne = NULL;
	/* On renvoie une ligne vide */
      } else {
	(void) fprintf(stderr, "lit_ligne_taille_fixe : ligne incomplète, fin de fichier atteinte ! [%lu caractères seulement]\n",
		       i + 1);
	/* On renvoie la ligne partielle */
	*ligne = 0;
      }
      return ligne;
    } else if ( ('\r' == c) || ('\n' == c) ) {
      (void) fprintf(stderr, "lit_ligne_taille_fixe : ligne trop courte ! [%lu caractères seulement]\n",
		     i + 1);
      break;
    } else {
      ligne[i] = (char) c;
    }
  }
  /* Au moins le 0 final */
  for ( ; i <= taille ; i++)
    ligne[i] = 0;

  /* On vérifie que l'on a bien atteint la fin de la ligne */
  if ( ('\r' != c) && ('\n' != c) ) { /* Ligne assez longue */
    unsigned long nbr_ignores = 0;

    c = fgetc(fichier);
    while ( ('\r' != c) && ('\n' != c) ) { /* Ligne trop longue */
      if (0 == nbr_ignores) {
	(void) fprintf(stderr, "lit_ligne_taille_fixe : ligne trop longue, on ignore !\n\t[");
      }
      nbr_ignores++;
      fputc(c, stderr);
      c = fgetc(fichier);
      if (c == EOF)
	break;
    }
    if ( nbr_ignores > 0 ) {
      (void) fprintf(stderr, "] = %lu caractères\n", nbr_ignores);
    }
  }

  /* On saute les fins de ligne, mais _pas_ les espaces en début de ligne suivante */
  while ( ('\r' == c) || ('\n' == c) ) {
    c = fgetc(fichier);
    if (c == EOF)
      break;
  }
  if ( c != EOF )
    ungetc(c, fichier); /* On annule la lecture du dernier = ligne suivante [sauf si fin de fichier !] */

  return ligne;
}

/* -------- Ligne de longueur a priori quelconque --------
  Lit la ligne entière, mais rabote les espaces finaux
*/

static char *lit_ligne_quelconque(FILE *fichier)
{
  char *ligne_lue;
  int c, retour;
  size_t lg_ligne;
  long pos_debut_ligne;

  /* Repérer la prochaine fin de ligne
     Remarque ! fgets() ne marche pas car ne reconnaît pas \r (= ^M) seul ==> fichiers Mac mal lus ! */
  pos_debut_ligne = ftell( fichier );
  lg_ligne = 0;
  c = fgetc( fichier );
  while ( ( c != '\n' ) && ( c != '\r' ) && ( c != EOF ) ) {
    lg_ligne++;
    c = fgetc( fichier );
  }

  /* On revient en début de ligne */
  retour = fseek( fichier, pos_debut_ligne, SEEK_SET );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s,%s l.%u : Problème dans fseek, code %d\n",
		    __FILE__, __FUNCTION__, __LINE__, retour );
    return strdup( " " );
  }

  /* A-t-on pu lire une ligne ? */
  if ( 0 == lg_ligne ) {
    if ( EOF == c ) {
      (void) fprintf( stderr, "%s,%s l.%u : Fin de fichier !\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      return NULL;
    } else {
      (void) fprintf( stderr, "%s,%s l.%u : ligne vide\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      return strdup(" "); /* Ligne d'espaces uniquement !! */
    }
  }

  /* On mémorise la ligne ainsi détectée */
  ligne_lue = (char *) malloc( lg_ligne + 1 );
  if ( NULL == ligne_lue ) {
    (void) fprintf( stderr, "%s,%s l.%u : mémoire insuffisante pour une ligne de %lu caractères\n",
		    __FILE__, __FUNCTION__, __LINE__, (unsigned long) (lg_ligne + 1) );
  } else {
    unsigned long i;

    for ( i = 0 ; i < lg_ligne ; i++ ) {
      ligne_lue[ i ] = (char) fgetc( fichier );
    }
    ligne_lue[ i ] = 0;
  }

  /* On saute les lignes vides après, si nécessaire */
  c = fgetc( fichier );
  while ( ( c == '\r' ) || ( c == '\n' ) ) {
    c = fgetc( fichier );
  }
  ungetc( c, fichier );

  /* On a fini, on renvoie la ligne lue */
  return ligne_lue;
}

/* -------- Lire un mot de longueur a priori inconnue dans un fichier, type %s
   Le mot ainsi lu est dans un chaîne allouée dynamiquement, qu'il faut libérer ensuite avec free().
   En cas de problème, la routine renvoie NULL
   Un mot est fini par un seul « espace » ou équivalent (fonction isspace)
 -------- */

char *lit_commande(FILE *fichier)
{
  size_t i, lg_mot = 0;
  int c_lu;
  char *chaine;
  long pos_debut;

  if (fichier == NULL) {
    return NULL;
  }

  /* On saute les espaces initiaux */
  while ( isspace(c_lu = fgetc(fichier)) != 0 ) ;
  if (c_lu == EOF) {
    fprintf(stderr, "lit_commande : erreur en voulant lire la commande, FIN DE FICHIER ATTEINTE !\n");
    return NULL;
  }
  ungetc(c_lu, fichier); /* On remet le dernier lu */

  pos_debut = ftell(fichier);
  if (pos_debut == -1) {
    fprintf(stderr, "lit_commande : erreur en voulant lire la commande, problème ftell !\n");
    return NULL;
  }

  while ( isspace(c_lu = fgetc(fichier)) == 0) {
    if (c_lu == EOF)
      break;
    lg_mot++;
  }

/*   printf("[lit_commande : lg = %lu]\n", (unsigned long) lg_mot); */
  chaine = (char *) malloc( lg_mot + 1 ); /* +1 = zéro final */
  if (chaine == NULL) {
    fprintf(stderr, "lit_ascii.c/lit_commande : MÉMOIRE INSUFFISANTE\n");
    fprintf(stderr, "  Réservation de la chaîne impossible\n");
    fprintf(stderr, "  Taille demandée = %lu octets\n", (unsigned long) (lg_mot + 1) );
    return NULL;
  }

  fseek(fichier, pos_debut, SEEK_SET);
  for ( i = 0 ; i < lg_mot ; i++ ) {
    chaine[ i ] = fgetc( fichier );
  }
  chaine[ lg_mot ] = 0;

  /* Chargement de la commande réussi */
  return chaine;
}

/* -------- Lire un mot de longueur a priori inconnue dans un fichier : %s amélioré...
   Le mot ainsi lu est dans un chaîne allouée dynamiquement, qu'il faut libérer ensuite.
   En cas de problème, la routine renvoie NULL
   Un mot est séparé par deux « espaces » ou équivalents (fonctions isspace)
 -------- */

char *lit_mot(FILE *fichier)
{
  size_t lg_mot = 0, i;
  int c_lu;
  char *chaine;
  long pos_debut;
  int sortie = FAUX;

  if (fichier == NULL) return NULL;

  /* On saute les espaces initiaux */
  while ( isspace(c_lu = fgetc(fichier)) != 0 ) ;
  if (c_lu == EOF) {
    fprintf(stderr, "lit_mot : erreur en voulant lire le mot, FIN DE FICHIER ATTEINTE !\n");
    return NULL;
  }
  ungetc(c_lu, fichier); /* On remet le dernier lu */

  pos_debut = ftell(fichier);
  if (pos_debut == -1) {
    fprintf(stderr, "lit_mot : erreur en voulant lire le mot, problème ftell !\n");
    return NULL;
  }

  do {
    while ( isspace(c_lu = fgetc(fichier)) == 0) {
      if ( (c_lu == '\n') || (c_lu == '\r') ) /* Fin de ligne... */
	break;
      if (c_lu == EOF) /* Fin de fichier */
	break;
      lg_mot++;
    }
    if ( (c_lu == '\n') || (c_lu == '\r') || (c_lu == EOF) ) /* Fin de ligne, de fichier */
      sortie = VRAI;
    else {
      c_lu = fgetc(fichier); /* Caractere apres l'espace */
      if ( (c_lu == EOF) || (isspace(c_lu) != 0) ) /* C'est aussi un espace - fini */
	sortie = VRAI;
      else { /* Un seul espace => mot pas fini */
	lg_mot += 2; /* Un pour l'espace, un pour le caractere */
      }
    }
  } while (sortie == FAUX);

  chaine = (char *) calloc(lg_mot + 1, 1); /* +1 = zéro final */
  if (chaine == NULL) {
    fprintf(stderr, "lit_ascii.c/lit_mot : MÉMOIRE INSUFFISANTE\n");
    fprintf(stderr, "  Réservation de la chaîne impossible\n");
    fprintf(stderr, "  Taille demandée = %lu octets\n", (unsigned long) (lg_mot + 1) );
    return NULL;
  }

  fseek(fichier, pos_debut, SEEK_SET);
  for (i = 0 ; i < lg_mot ; i++)
    chaine[i] = fgetc(fichier);
  chaine[lg_mot] = 0;
  return chaine;
}

/* -------- Lit une suite de n caractères et les mets dans la chaîne indiquée ---\
 |  Renvoie la chaîne contenant le résultat (NULL en cas de problème)            |
 |     S'interrompt sur une fin de ligne ou de fichier                           |
 |                                                                               |
 | fichier : le fichier où il faut lire                                          |
 | chaine  : la chaîne où écrire (elle doit être assez grande !)                 |
 |           si NULL, la fonction crée une chaîne                                |
 | n_cars  : le nombre de caractères à lire                                      |
 \------------------------------------------------------------------------------ */

char *lit_cars(FILE *fichier, char *chaine, unsigned int n_cars)
{
  unsigned int i;

  if ( (fichier == NULL) || (n_cars == 0)) return NULL;

  if (chaine == NULL) { /* On crée la chaîne, avec malloc() */
    chaine = (char *) malloc( n_cars + 1 ); /* 0 final */
    if (chaine == NULL) {
      fprintf(stderr, "lit_ascii.c/lit_cars : MÉMOIRE INSUFFISANTE\n");
      fprintf(stderr, "  Réservation de la chaîne impossible\n");
      fprintf(stderr, "  Taille demandée = %lu octets\n", (unsigned long) (n_cars + 1) );
      return NULL;
    }
  }

  for (i = 0 ; i < n_cars ; i++) {
    int c = fgetc(fichier);
    if (c == EOF) break;
    if ( (c == '\r') || (c == '\n') ) break;
    chaine[i] = c;
  }

  chaine[ i ] = 0; /* En cas de sortie anticipée par fin de ligne ! */
  chaine[ n_cars ] = 0; /* De toute façon... */
  /* On supprime les espaces terminaux */
  while ( isspace( chaine[n_cars - 1] ) ) {
    chaine[n_cars - 1] = 0;
    n_cars--;
  }

  return chaine;
}

/* -------- Lire des valeurs numériques _sur une même ligne_, en colonnes -------- */

int lit_valeurs(FILE *fichier, unsigned int nbr_colonnes, int separateur, double *valeurs, double *erreurs)
{
  char *ligne, *debut, *fin;
  unsigned int i_c;

  if ( ( NULL == fichier ) || ( NULL == valeurs ) ) return -1;

  /* On initialise les valeurs à 0 = valeur remarquable */
  for ( i_c = 0 ; i_c < nbr_colonnes ; i_c ++ ) {
    valeurs[ i_c ]= 0.0;
    if ( erreurs != NULL )
      erreurs[ i_c ] = 0.0;
  }
  i_c = 0; /* Et on revient au début */

  /* On lit la ligne en une fois */
  ligne = lit_ligne_quelconque( fichier );
  if ( NULL == ligne ) {
    (void) fprintf(stderr, "lit_valeurs : ERREUR dans lit_ligne_quelconque()\n");
    return -2;
  }
/*   printf("%s : ligne lue = %s\n", __FUNCTION__, ligne); */
  debut = ligne;

  if ( 0 == separateur ) {
    do {
      double x, err_x;

      /* On est entre deux colonnes, ou avant la première */
      while ( isspace( *debut ) ) {
	debut++;
	if ( *debut == 0 ) {
	  (void) fprintf(stderr, "lit_valeurs : il manque des colonnes [%u lues, %u demandées\n",
			 i_c, nbr_colonnes);
	  free( ligne );
	  return +1; /* Erreur, mais mineure */
	}
      }

      /* Là, on est sur un début de colonne */
      fin = debut;
      while ( FAUX == isspace( *fin ) ) {
	fin++;
	if ( 0 == *fin ) break; /* Ligne se termine par un caractère... */
      }

      /* On décode, avec éventuellement une erreur associée */
      x = TXT_CNV_vNombre_D( debut, NULL, &err_x );
      if ( ( 0.0 == x ) && ( TXT_DOUBLE_CHAINE_VIDE == err_x ) ) {
	x = NAN;		/* On met un NaN qui sera filtré ensuite par l'utilisateur [en théorie !] */
      }

      /* On mémorise le résultat */
      valeurs[ i_c ] = x;
      if ( erreurs != NULL ) erreurs[ i_c ] = err_x;
#ifdef _DEVELOPPE_
/*       else if ( err_x > 0.0 ) printf("lit_valeurs : erreur ignorée [%g, valeur %g]\n", err_x, x); */
#endif

      i_c++;
      debut = fin; /* On attaque la colonne suivante */
    } while ( ( i_c < nbr_colonnes ) && ( *debut != 0 ) );
  } else {
    /* On peut supposer que l'on ne commence pas par un séparateur, sauf si la colonne est vide ! */
    do {
      double x, err_x;

      /* On décode donc ce que l'on peut --- suppose que le séparateur ne soit pas un caractère autorisé ! */
      x = TXT_CNV_vNombre_D( debut, NULL, &err_x );
      if ( ( 0.0 == x ) && ( TXT_DOUBLE_CHAINE_VIDE == err_x ) ) {
	x = NAN;		/* On met un NaN qui sera filtré ensuite par l'utilisateur [en théorie !] */
      }
      /* On mémorise le résultat */
      valeurs[ i_c ] = x;
      if ( erreurs != NULL ) erreurs[ i_c ] = err_x;
#ifdef _DEVELOPPE_
/*       else if ( err_x > 0.0 ) printf("lit_valeurs : erreur ignorée [%g, valeur %g]\n", err_x, x); */
#endif

      i_c++;

      /* On cherche le début de la colonne suivante */
      fin = strchr( debut, separateur );
      if ( NULL == fin ) break;
      debut = fin + 1;
    } while ( i_c < nbr_colonnes );
  }

  if ( i_c != nbr_colonnes ) {
    fprintf(stderr, "lit_valeurs : seulement %u colonnes lues sur %u demandées [%.20s]\n",
	    i_c, nbr_colonnes, ligne );
  }

  /* Tout s'est bien passé */
  free( ligne );
  return 0;
}
