/* Lecture d'une liste de solvants & de leurs proprits
   (C) 2010, Emmanuel Curis
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "ascii_utils.h"
#include "chimie_utils.h"
#include "txt_utils.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

Solvant *Chimie_CreerSolvants(unsigned long nombre_solvants)
{
  Solvant *solvants, *s;
  unsigned long i;

  if ( 0 == nombre_solvants ) {
    (void) fprintf( stderr, "%s,%s l.%u : Aucun solvant demand !\n",
		    __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Rservation */
  solvants = (Solvant *) malloc( sizeof( Solvant ) * nombre_solvants );
  if ( NULL == solvants ) {
    (void) fprintf( stderr, "%s,%s l.%u : ERREUR dans malloc pour rserver %lu solvants\n",
		    __FILE__, __FUNCTION__, __LINE__, nombre_solvants );
    return NULL;
  }

  /* Initialisation */
  for ( i = 0, s = solvants ; i < nombre_solvants ; i++, s++ ) {
    s->nom = NULL;
    s->formule = NULL;
    s->masse_molaire = 0.0;
    s->T_min = 0.0;
    s->T_max = 0.0;
    s->pKe = 0.0;
    s->densite = 0.0;
  }

  /* Termin */
  return solvants;
}

Solvant *Chimie_LireSolvants(const char *nom_fichier, unsigned long *nombre_solvants)
{
  FILE *fichier;
  Solvant *solvants;
  char *ligne, **colonnes;
  int c;
  unsigned long i, nombre_lignes, nombre_colonnes;

  if ( NULL == nom_fichier ) {
    (void) fprintf( stderr, "%s,%s l.%u : Aucun fichier indiqu\n",
		    __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  if ( NULL == nombre_solvants ) {
    (void) fprintf( stderr, "%s,%s l.%u : Indiquer o mmoriser le nombre de solvants\n",
		    __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  *nombre_solvants = 0;

  /* On ouvre le fichier */
  fichier = fopen( nom_fichier, "rb" );
  if ( NULL == fichier ) {
    (void) fprintf( stderr, "%s,%s l.%u : ERREUR en voulant ouvrir le fichier %s\n",
		    __FILE__, __FUNCTION__, __LINE__, nom_fichier );
    return NULL;
  }

  /* On compte le nombre de lignes */
  nombre_lignes = compte_lignes( fichier, '#' );
  if ( 0 == nombre_lignes ) {
    (void) fprintf( stderr, "%s,%s l.%u : Aucun solvant trouv dans le fichier\n",
		    __FILE__, __FUNCTION__, __LINE__);
    (void) fclose( fichier );
    return NULL;
  }
  
  /* On rserve pour les solvants */
  solvants = Chimie_CreerSolvants( nombre_lignes );
  if ( NULL == solvants ) {
    (void) fprintf( stderr, "%s,%s l.%u : Impossible de crer les solvants. Chargement annul\n",
		    __FILE__, __FUNCTION__, __LINE__);
    (void) fclose( fichier );
    return NULL;
  }

  /* On saute le dbut */
  c = fgetc( fichier ); ungetc( c, fichier );
  while ( '#' == c ) {
    c = ignore_ligne( fichier );
  }
  if ( EOF == c ) {
    (void) fprintf( stderr, "%s,%s l.%u : Fin de fichier atteinte : aucun solvant dans le fichier\n",
		    __FILE__, __FUNCTION__, __LINE__);
    (void) fclose( fichier );
    free( solvants ); solvants = NULL;
    return NULL;
  }

  /* On lit chaque ligne  utile  */
  i = 0;
  do {
    c = fgetc( fichier ); ungetc( c, fichier );
    if ( EOF == c ) {
      break;
    }

    /* L : caractre normal => on lit la ligne */
    ligne = lit_ligne( fichier, 0 );
    if ( NULL == ligne ) {
      /* Fin de fichier atteinte => termin */
      break;
    }

/*     printf( "Ligne lue : %s\n", ligne ); */
    if ( '#' != ligne[ 0 ] ) {
      /* Ligne utile : on peut l'analyser */

      /* 1. On dcompose en colonnes */
      colonnes = TXT_DecomposerChaine( ligne, ';', &nombre_colonnes );
      if ( ( NULL == colonnes ) || ( 0 == nombre_colonnes ) ) {
	(void) fprintf( stderr, "%s,%s l.%u : Erreur dans la dcomposition en colonnes --- ligne ignore\n",
			__FILE__, __FUNCTION__, __LINE__);
      } else {
	char *nom;
	size_t lg_nom;
	/* Nom en 1re colonne : toujours prsent */
	nom = strdup( colonnes[ 0 ] );

	/* On limine les espaces finaux du nom */
	lg_nom = strlen( nom );
	if ( lg_nom > 0 ) {
	  while ( isspace( nom[ lg_nom - 1 ] ) ) {
	    nom[ lg_nom - 1 ] = 0;
	    lg_nom--;
	    if ( 0 == lg_nom ) {
	      free( nom ); nom = strdup( "<Sans nom>" );
	      break;
	    }
	  }
	}

	/* 2. On mmorise les infos du solvant */
	solvants[ i ].nom = nom;
	if ( nombre_colonnes > 1 ) {
	  Composition *composition_solvant;

	  solvants[ i ].formule = strdup( colonnes[ 1 ] );

	  composition_solvant = Chimie_AnalyseFormule( colonnes[ 1 ] );
	  solvants[ i ].masse_molaire = Chimie_MasseMolaire( composition_solvant );
	  free( composition_solvant ); composition_solvant = NULL;
	}
	if ( nombre_colonnes > 2 ) {
	  solvants[ i ].densite = TXT_CNV_vNombre_D( colonnes[ 2 ], NULL, NULL );
	}

	/* On remplira ensuite le solvant suivant */
	i++;
	free( colonnes[ 0 ] ); colonnes[ 0 ] = NULL;
      }

      /* Termin avec les colonnes */
      free( colonnes ); colonnes = NULL;
    }

    /* Termin avec la ligne : on libre donc... */
    free( ligne ); ligne = NULL;
  } while ( c != EOF );

  /* Plus besoin du fichier */
  (void) fclose( fichier );
  
  /* Termin */
  *nombre_solvants = i;
  return solvants;
}

/* Pour tests... */

#ifdef _AUTONOME_
int main(void)
{
  Solvant *solvants;
  unsigned long i, n;

  printf( "--------------------\n"
	  " Banque de solvants\n"
	  "--------------------\n" );

  solvants = Chimie_LireSolvants( "solvants.txt", &n );
  if ( NULL == solvants ) {
    (void) fprintf( stderr, "%s,%s l.%u : ERREUR au chargement\n",
		    __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* Affichages pour contrle */
  printf( "On a charg %lu solvants\n", n );

  for ( i = 0 ; i < n ; i++ ) {
    printf( " -- Solvant %lu --\n", i );
    printf( "Nom           : %s\n", solvants[ i ].nom );
    printf( "Formule       : %s\n", solvants[ i ].formule );
    printf( "Masse molaire : %g g/mol\n",  solvants[ i ].masse_molaire );
    printf( "Densit       : %g [eau = 1]\n",  solvants[ i ].densite );
  }

  return 0;
}
#endif
