/* Fonctions utilitaires gnrales
   Dveloppes pour LASE, mais plus gnriques
   (C) 1996 - 2003, Emmanuel CURIS

   Calculs statistiques fondamentaux sur une table de valeurs
*/

#include <stdio.h>
#include <stdlib.h>

#include "stats_utils.h"

/* ~-~-~-~-~-~-~-~- Recherche des extrema ~-~-~-~-~-~-~-~- */

/* -------- Table de doubles -------- */

int STAT_ChercherExtrema_D(double *table, unsigned long nbr_valeurs, double *min, double *max)
{
  double m, M;
  unsigned long i;

  if ( ( NULL == table ) || ( 0 == nbr_valeurs ) ) return -1;

  m = *table; M = m;
  for ( i = 1, table++ ; i < nbr_valeurs ; i++, table++ ) {
    if ( *table > M ) M = *table;
    if ( *table < m ) m = *table;
  }

  if ( min != NULL ) *min = m;
  if ( max != NULL ) *max = M;

  return 0;
}

/* -------- Table de ints -------- */

int STAT_ChercherExtrema_I(int *table, unsigned long nbr_valeurs, int *min, int *max)
{
  int m, M;
  unsigned long i;

  if ( ( NULL == table ) || ( 0 == nbr_valeurs ) ) return -1;

  m = *table++; M = m;
  for ( i = 1 ; i < nbr_valeurs ; i++, table++ ) {
    if ( *table > M ) M = *table;
    if ( *table < m ) m = *table;
  }

  if ( min != NULL ) *min = m;
  if ( max != NULL ) *max = M;

  return 0;
}

/* -------- Table de longs -------- */

int STAT_ChercherExtrema_L(long *table, unsigned long nbr_valeurs, long *min, long *max)
{
  long m, M;
  unsigned long i;

  if ( ( NULL == table ) || ( 0 == nbr_valeurs ) ) return -1;

  m = *table++; M = m;
  for ( i = 1 ; i < nbr_valeurs ; i++, table++ ) {
    if ( *table > M ) M = *table;
    if ( *table < m ) m = *table;
  }

  if ( min != NULL ) *min = m;
  if ( max != NULL ) *max = M;

  return 0;
}

/* -------- Table de unsigned ints -------- */

int STAT_ChercherExtrema_UI(unsigned int *table, unsigned long nbr_valeurs, unsigned int *min, unsigned int *max)
{
  unsigned int m, M;
  unsigned long i;

  if ( ( NULL == table ) || ( 0 == nbr_valeurs ) ) return -1;

  m = *table++; M = m;
  for ( i = 1 ; i < nbr_valeurs ; i++, table++ ) {
    if ( *table > M ) M = *table;
    if ( *table < m ) m = *table;
  }

  if ( min != NULL ) *min = m;
  if ( max != NULL ) *max = M;

  return 0;
}

/* -------- Table de unsigned longs -------- */

int STAT_ChercherExtrema_UL(unsigned long *table, unsigned long nbr_valeurs, unsigned long *min, unsigned long *max)
{
  unsigned long m, M;
  unsigned long i;

  if ( ( NULL == table ) || ( 0 == nbr_valeurs ) ) return -1;

  m = *table++; M = m;
  for ( i = 1 ; i < nbr_valeurs ; i++, table++ ) {
    if ( *table > M ) M = *table;
    if ( *table < m ) m = *table;
  }

  if ( min != NULL ) *min = m;
  if ( max != NULL ) *max = M;

  return 0;
}

/* ~-~-~-~-~-~-~-~- Calcul des moyennes ~-~-~-~-~-~-~-~- */

#define SQUELETTE_MOYENNE( idx, table, nbr_valeurs, moyenne ) {\
  for ( idx = 0, moyenne = 0.0 ; idx < nbr_valeurs ; idx++, table++ ) {\
    moyenne += (double) *table;\
  }\
  moyenne /= (double) nbr_valeurs;\
}

/* -------- Table de doubles -------- */

int STAT_Moyenne_D(double *table, unsigned long nbr_valeurs, double *moyenne)
{
  unsigned long i;

  SQUELETTE_MOYENNE( i, table, nbr_valeurs, *moyenne );
  return 0;
}

int STAT_Moyenne_I (int  *table, unsigned long nbr_valeurs, double *moyenne)
{
  unsigned long i;

  SQUELETTE_MOYENNE( i, table, nbr_valeurs, *moyenne );
  return 0;
}

int STAT_Moyenne_L (long *table, unsigned long nbr_valeurs, double *moyenne)
{
  unsigned long i;

  SQUELETTE_MOYENNE( i, table, nbr_valeurs, *moyenne );
  return 0;
}

int STAT_Moyenne_UI(unsigned int *table, unsigned long nbr_valeurs, double *moyenne)
{
  unsigned long i;

  SQUELETTE_MOYENNE( i, table, nbr_valeurs, *moyenne );
  return 0;
}

int STAT_Moyenne_UL(unsigned long *table, unsigned long nbr_valeurs, double *moyenne)
{
  unsigned long i;

  SQUELETTE_MOYENNE( i, table, nbr_valeurs, *moyenne );
  return 0;
}
