/* Bibliothque d'interface
   (C) 1998-2002, Emmanuel CURIS

   Fonctions generales de traitement des chaines de caractere
*/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h> /* Pour is... */
#include <math.h>  /* Pour log10 */

#include "complexe.h"
#include "deboguage.h"
#include "txt_utils.h"

#ifndef CODE_SEPARATEUR		/* Par dfaut, sparateur de dossiers = convention UNIX */
# define CODE_SEPARATEUR '/'
#endif

/* #define AUTONOME */

static int TXT_AnalyseDouble(double valeur, unsigned long *avant, unsigned int *exposant);

#define TXT_Ecrit_I(chaine,i,controle) TXT_Ecrit_L( chaine, (long) i, controle )
#define TXT_Ecrit_UI(chaine,i,controle) TXT_Ecrit_UL( chaine, (unsigned long) i, controle )

static unsigned int TXT_Ecrit_C (char *chaine, Complexe *c, struct affiche_valeur *controle);
static unsigned int TXT_Ecrit_D (char *chaine, double d, struct affiche_valeur *controle);
static unsigned int TXT_Ecrit_F (char *chaine, const char *nom_fichier);
static unsigned int TXT_Ecrit_L (char *chaine, long li, struct affiche_valeur *controle);
static unsigned int TXT_Ecrit_UL(char *chaine, unsigned long uli, struct affiche_valeur *controle);

/* -------- Gnre une chane  partir du masque et des paramtres passs --------\
 |   Renvoie NULL en cas d'erreur, la chaine enregistre avec malloc sinon        |
 |                                                                                |
 | Codes de conversion : cf la routine d'analyse de format                        |
 |                       (TXT_AnalyseCodeAffichage)                               |
 \------------------------------------------------------------------------------- */

char *TXT_genere_chaine(const char *masque, ...)
{
  char *resultat;
  va_list parametres ;

  if ( NULL == masque ) return NULL;

  va_start(parametres, masque);
  resultat = TXT_vGenereChaine(masque, parametres);
  va_end(parametres);

  return resultat;
}

char *TXT_vGenereChaine(const char *masque, va_list parametres)
{
  size_t lg_chaine = 0;
  char *resultat, *car_ecrit/* , c */;
  const char *car_lu;
  va_list cp_parametres;

  if ( NULL == masque ) return NULL;
  if ( strchr( masque, '%' ) == NULL ) { /* Pas de parametres => copie brutale... */
    return strdup( masque );
  }

  /* Analyse de la longueur de la chaine
     On passe par une copie de la liste des paramtres pour viter de mauvaises surprises ventuelles
  */
  va_copy( cp_parametres, parametres );
  lg_chaine = TXT_lg_chaine_finale( masque, cp_parametres );
  va_end( cp_parametres );
  if ( 0 == lg_chaine ) {
    (void) fprintf( stderr, "%s : le masque conduit  une chane de taille nulle [%s]\n",
		    __FUNCTION__, masque );
    return NULL;
  }

  /* On rserve la chane, sans oublier le 0 final ! ; on l'initialise avec des 0 */
  resultat = (char *) calloc( lg_chaine + 1, sizeof(char) );
  if ( NULL == resultat ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante pour rserver la chane rsultat [demand : %lu octets]\n",
		   __FILE__, __FUNCTION__, __LINE__, (unsigned long) lg_chaine);
    return NULL;
  }

  /* On recopie la chane en l'adaptant */
  car_ecrit = resultat;
  car_lu = masque;
  do {
#ifdef _DEVELOPPE_
    if ( car_ecrit >= ( resultat + lg_chaine ) ) {
      (void) fprintf(stderr, "%s,%s l.%u : On dborde de la chane rserve !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      break;
    }
#endif

    if ( '%' == *car_lu ) {
      Complexe *cplx;
      const char *chaine;
      double d, e;
      int c, i;
      long li;
      size_t lg_code = 0;
      struct affiche_valeur infos_code;
      unsigned int ui;
      unsigned long int uli;

      car_lu = TXT_AnalyseCodeAffichage( car_lu, &infos_code );
      switch ( infos_code.type )
      {
      case TXT_AFFICHE_CARACTERE: /* On affiche un seul caractre */
	c = va_arg( parametres, int );
	if ( ' ' == c ) {
	  strcat( car_ecrit, "<espace>" );
	  lg_code = 8;
	} else if ( c < ' ' ) {
	  if ( c > 0 ) {
	    sprintf( car_ecrit, "<ctrl:%02x>", (unsigned int) abs( c ) );
	    lg_code = 9;
	  }
	  /* 0 : code special, caractere vide => pas mis pour eviter une fausse fin de chaine ; pas en <ctrl:> pour pouvoir avoir des textes vides ! */
	} else {
	  *car_ecrit = c;
	  lg_code = 1;
	}
	break;

      case TXT_AFFICHE_I:
	i = va_arg( parametres, int );
	lg_code = TXT_Ecrit_I( car_ecrit, i, &infos_code );
/* 	printf("%s : Lg crite = %lu cars\n", __FUNCTION__, (unsigned long) lg_code); */
	break;

      case TXT_AFFICHE_UI:
	ui = va_arg( parametres, unsigned int );
	lg_code = TXT_Ecrit_UI( car_ecrit, ui, &infos_code );
#ifdef AUTONOME
	printf("On va placer %u : %lu caractere(s)\n", ui, (unsigned long) lg_code );
#endif
	break;

      case TXT_AFFICHE_L:
	li = va_arg( parametres, long );
	lg_code = TXT_Ecrit_L( car_ecrit, li, &infos_code );
/* 	printf("%s : Lg crite = %lu cars\n", __FUNCTION__, (unsigned long) lg_code); */
	break;

      case TXT_AFFICHE_UL:
	uli = va_arg( parametres, unsigned long );
	lg_code = TXT_Ecrit_UL( car_ecrit, uli, &infos_code );
	break;

      case TXT_AFFICHE_D:
	d = va_arg( parametres, double );
	lg_code = TXT_Ecrit_D( car_ecrit, d, &infos_code );
	break;

      case TXT_AFFICHE_POURCENT:
	d = va_arg( parametres, double );
	lg_code = TXT_Ecrit_D( car_ecrit, d * 100.0, &infos_code );
	car_ecrit[ lg_code ] = '%';
	lg_code++;
	break;

      case TXT_AFFICHE_ERREUR:
	d = va_arg( parametres, double );
	e = va_arg( parametres, double );
	lg_code = TXT_Ecrit_D( car_ecrit, d, &infos_code );
	car_ecrit[ lg_code++ ] = ' ';
	car_ecrit[ lg_code++ ] = '';
	car_ecrit[ lg_code++ ] = ' ';
	lg_code += TXT_Ecrit_D( car_ecrit + lg_code, e, &infos_code );
	break;

      case TXT_AFFICHE_FICHIER:
	chaine = va_arg( parametres, const char * );
	lg_code = TXT_Ecrit_F( car_ecrit, chaine );
	break;

      case TXT_AFFICHE_CHAINE:
	chaine = va_arg( parametres, const char * );
	if ( NULL == chaine ) {
	  strcat( car_ecrit, "<NULL>" );
	  lg_code = 6;
	} else {
	  strcat( car_ecrit, chaine );
	  lg_code = strlen( chaine );
	}
	break;

      case TXT_AFFICHE_CAR_PC:
	lg_code = 1;
	*car_ecrit = '%';
	break;

      case TXT_AFFICHE_COMPLEXE:
	cplx = va_arg( parametres, Complexe * );
	lg_code = TXT_Ecrit_C( car_ecrit, cplx, &infos_code );
	break;

      default:
	(void) fprintf(stderr, "%s,%s l.%u : Code de contrle inconnu [%d] : ignor\n",
		       __FILE__, __FUNCTION__, __LINE__, infos_code.type);
	lg_code = 0;
      }

      car_ecrit += lg_code;

      /* On s'occupe de l'unit, si ncessaire */
      if ( infos_code.proprietes & TXT_CODE_UNITE_INDIQUEE ) {
	const char *unite;

	*car_ecrit = ' ';
	car_ecrit++;
	*car_ecrit = 0;		/* Peut ne pas etre NUL si chiffre a virgules et arrondi fait => on voit reapparaitre les decimales effacees dans ce cas ! */
	unite = va_arg( parametres, const char * );
	if ( NULL == unite ) {
	  /* Rien  crire */
	} else {
	  strcat( car_ecrit, unite );
	  car_ecrit += strlen( unite );
	}
      }
    } else {
      *car_ecrit = *car_lu;	/* Caractre inchang */
      car_ecrit++;

      /* On passe au caractre suivant */
      car_lu++;
    }

#ifdef AUTONOME
    printf("Chaine : %s\n", resultat );
#endif
  } while ( *car_lu != 0 );

#ifdef AUTONOME
  printf("Rsultat de la conversion : %s\n", resultat);
#endif
#ifdef _DEVELOPPE_
# ifdef DEBOGUE_MALLOC_
   Malloc_Logue("TXT_genere_chaine, contrle : taille de la chane finale [%p], %lu caractres\n", resultat, (unsigned long) strlen( resultat ));
# endif
#endif
  return resultat;
}

/* ~~~~~~~~ Briques pour l'criture de la chane ~~~~~~~~ */

/* -------- criture d'un nombre complexe dans une chane -------- */

static unsigned int TXT_Ecrit_C(char *chaine, Complexe *c, struct affiche_valeur *controle)
{
  unsigned int n_cars = 0, lg_ecrite;

  if ( ( NULL == chaine ) || ( NULL == controle ) || ( NULL == c ) ) return 0;

  if ( ( 0.0 == c->re ) && ( 0.0 == c->im ) ) {
    *chaine = '0';
    return 1;
  }

  if ( c->re != 0.0 ) {
    /* criture de la partie relle */
    lg_ecrite = TXT_Ecrit_D( chaine, c->re, controle );

    /* On s'avance dans la chane */
    n_cars += lg_ecrite;
    chaine += lg_ecrite;

    /* Devra-t-on mettre une partie imaginaire ? */
    if ( c->im != 0.0 ) {
      *chaine = ' '; chaine++;
      n_cars++;

      if ( c->im > 0.0 ) {
	*chaine = '+'; chaine++;
	n_cars++;
      } else {
	*chaine = '+'; chaine++;
	n_cars++;	
      }

      *chaine = ' '; chaine++;
      n_cars++;
    }
  }

  /* Doit-on mettre une partie imaginaire ? */
  if ( c->im != 0.0 ) {
    if ( ( 0.0 == c->re ) && ( c->im < 0.0 ) ) {
      /* Partie imaginaire ngative, imaginaire pur -> le signe n'a pas t mis avant */
      *chaine = '-'; chaine++;
      n_cars++;
    }

    /* criture de la partie imaginaire */
    lg_ecrite = TXT_Ecrit_D( chaine, fabs( c->im ), controle );

    /* On s'avance dans la chane */
    n_cars += lg_ecrite;
    chaine += lg_ecrite;

    /* On indique le i */
    *chaine = ' '; chaine++;
    n_cars++;

    *chaine = 'i';
    n_cars++;
  }

  /* Termin */
  return n_cars;
}

/* -------- criture d'un nombre  virgule dans une chane -------- */

static unsigned int TXT_Ecrit_D(char *chaine, double d, struct affiche_valeur *controle)
{
  unsigned int n_cars = 0, chiffres_exposant, lg_ecrite;
  unsigned int lg_avant, lg_apres;
  unsigned long chiffres_avant;
  int retour, av_exposant = FAUX;

  if ( ( NULL == chaine ) || ( NULL == controle ) ) return 0;

/*   printf("%s -- On va crire %g en %s\n", __FUNCTION__, d, chaine ); */

  /* Gestion du signe */
  if ( d < 0 ) {
    *chaine = '-';
    chaine++;
    d = -d;
    n_cars++;
  } else if ( controle->proprietes & TXT_CODE_AVEC_SIGNE ) {
    /* Positif, mais on impose qu'il y ait un signe ! */
    *chaine = '+';
    chaine++;
    n_cars++;    
  }

  /* Dtection de la longueur de la partie entire et de la partie dcimale */
  retour = TXT_AnalyseDouble( d, &chiffres_avant, &chiffres_exposant );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans TXT_AnalyseDouble [code %d / valeur %g]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour, d);
    return 0;
  }

  /* Contrle en fonction de la prcision demande */
  /* a. Partie entire */
  if ( controle->proprietes & TXT_CODE_ENTIER_IMPOSE ) {
    /* Taille de la partie entire impose */
    lg_avant = controle->lg_avant;
  } else {
    /* Arbitraire : si non prcis, au-dessus de 5 chiffres on passe en notation scientifique */
    if ( chiffres_avant > 5 ) {
      lg_avant = 1;
      av_exposant = VRAI;
    } else {
      lg_avant = chiffres_avant;
    }
  }

  /* b. Partie dcimale */
  if ( controle->proprietes & TXT_CODE_DECIMALE_IMPOSEE ) {
    /* Taille de la partie entire impose */
    lg_apres = controle->lg_apres;
  } else {
    /* Arbitraire : si non prcis, 10 chiffres */
    lg_apres = 10;
  }

  /* criture de la valeur */
  if ( VRAI == av_exposant ) {
    printf( "EXPOSANT demand, mais pas encore gr...\n" );
  }
  (void) TXT_CNV_D_vTexte( d, 0.0, lg_avant, lg_apres, chiffres_exposant, chaine, &lg_ecrite );

  n_cars += lg_ecrite;
  return n_cars;
}

/* -------- Affichage d'un nom de fichier, sans le chemin -------- */

static unsigned int TXT_Ecrit_F(char *chaine, const char *nom_fichier)
{
  const char *pos_dernier_separateur;

  if ( NULL == chaine ) return 0;

  /* A-t-on bien pass un nom de fichier ? */
  if ( NULL == nom_fichier ) {
    strcat( chaine, "<absent>" );
    return 8;
  }

  /* On cherche s'il contient un sparateur : chemin complet ? */
  pos_dernier_separateur = strrchr( nom_fichier, CODE_SEPARATEUR );
  if ( pos_dernier_separateur != NULL ) {
    nom_fichier = pos_dernier_separateur + 1; /* On ne garde que le nom de fichier ! */
  }

  /* On ajoute le nom de fichier */
  strcat( chaine, nom_fichier );
  return strlen( nom_fichier );
}

/* -------- criture d'un entier dans la chane --------
    Renvoie le nombre de caractres crit

    chaine   = l o crire [doit tre assez long !]
    i        = entier  crire
    controle = informations sur l'criture
 */

static unsigned int TXT_Ecrit_L(char *chaine, long li, struct affiche_valeur *controle)
{
  unsigned int n_cars = 0, n_chiffres;

  if ( ( NULL == chaine ) || ( NULL == controle ) ) return 0;

/*   printf("%s -- On va crire %ld en %s\n", __FUNCTION__, li, chaine ); */

  /* Gestion du signe */
  if ( li < 0 ) {
    *chaine = '-';
    chaine++;
    li = -li;
    n_cars++;
  } else if ( controle->proprietes & TXT_CODE_AVEC_SIGNE ) {
    /* Positif, mais on impose qu'il y ait un signe ! */
    *chaine = '+';
    chaine++;
    n_cars++;    
  }

  /* Combien de chiffres dans l'criture dcimale de l'entier ? */
  n_chiffres = TXT_nbr_car_L( li );

  /* criture de la valeur */
  if ( ( controle->lg_avant > 0 ) && ( controle->proprietes & TXT_CODE_ENTIER_IMPOSE ) ) {
    unsigned int j;

    /* Taille limite pour la partie entire */
    if ( n_chiffres > controle->lg_avant ) {
      chaine[ 0 ] = '>';	/* Indique que l'on dborde */
      for ( j = 1 ; j < controle->lg_avant ; j++ ) {
	chaine[ j ] = '9';	/* On remplace par 999, nombre affichable le plus grand... */
      }
    } else {
      if ( controle->proprietes & TXT_CODE_ALIGNE_A_DROITE ) {
	/* On met des espaces */
	for ( j = 0 ;  j < controle->lg_avant ; j++ ) {
	  chaine[ j ] = ' ';
	}

	/* On met les nombres, aligns  droite */
	for ( j = 0 ; j < n_chiffres ; j++ ) {
	  chaine[ ( controle->lg_avant - 1 ) - j ] = '0' + (li % 10);
	  li = li/10;
	}
      } else {
	/* On met les nombres, aligns  droite */
	for ( j = 0 ; j < n_chiffres ; j++ ) {
	  chaine[ ( n_chiffres - 1 ) - j ] = '0' + (li % 10);
	  li = li/10;
	}
      }
    }

    n_cars += controle->lg_avant;
  } else {
    unsigned int j;

#ifdef _DEVELOPPE_
/*     (void) fprintf(stderr, "%s,%s l.%u : Pas de taille limite, %u chiffres pour %ld\n", */
/* 		   __FILE__, __FUNCTION__, __LINE__, n_chiffres, li); */
#endif

    /* Pas de limitation sur la partie entire : a bien t faite */
    for ( j = 0 ; j < n_chiffres ; j++ ) {
      chaine[ ( n_chiffres - 1 ) - j ] = '0' + (li % 10);
      li = li/10;
    }

    n_cars += n_chiffres;
  }

  return n_cars;
}

static unsigned int TXT_Ecrit_UL(char *chaine, unsigned long uli, struct affiche_valeur *controle)
{
  unsigned int n_cars = 0, n_chiffres;

  if ( ( NULL == chaine ) || ( NULL == controle ) ) return 0;

  /* Gestion du signe */
  if ( controle->proprietes & TXT_CODE_AVEC_SIGNE ) {
    /* Positif, mais on impose qu'il y ait un signe ! */
    *chaine = '+';
    chaine++;
    n_cars++;    
  }

  /* Combien de chiffres dans l'criture dcimale de l'entier ? */
  n_chiffres = TXT_nbr_car_UL( uli );

#ifdef AUTONOME
  printf("%s : valeur %lu  placer dans %p [%d = %c]\n", __FUNCTION__, uli, chaine, (int) *chaine, *chaine);
#endif

  /* criture de la valeur */
  if ( ( controle->lg_avant > 0 ) && ( controle->proprietes & TXT_CODE_ENTIER_IMPOSE ) ) {
    unsigned int j;

#ifdef AUTONOME
    printf("  Taille impose : lg_avant = %u\n", controle->lg_avant );
#endif

    /* Taille limite pour la partie entire */
    if ( n_chiffres > controle->lg_avant ) {
      chaine[ 0 ] = '>';	/* Indique que l'on dborde */
      for ( j = 1 ; j < controle->lg_avant ; j++ ) {
	chaine[ j ] = '9';	/* On remplace par 999, nombre affichable le plus grand... */
      }
    } else {
      if ( controle->proprietes & TXT_CODE_ALIGNE_A_DROITE ) {
	/* On met des espaces */
	for ( j = 0 ;  j < controle->lg_avant ; j++ ) {
	  chaine[ j ] = ' ';
	}

	/* On met les nombres, aligns  droite */
	for ( j = 0 ; j < n_chiffres ; j++ ) {
	  chaine[ ( controle->lg_avant - 1 ) - j ] = '0' + (uli % 10);
	  uli = uli/10;
	}
      } else {
	/* On met les 0... */
	for ( j = 0 ;  j < controle->lg_avant ; j++ ) {
	  chaine[ j ] = '0';
	}

	/* On met les nombres, aligns  droite */
	for ( j = 0 ; j < n_chiffres ; j++ ) {
	  chaine[ ( controle->lg_avant - 1 ) - j ] = '0' + (uli % 10);
	  uli = uli/10;
	}
      }
    }

    n_cars = controle->lg_avant;
  } else {
    unsigned int j;

#ifdef AUTONOME
    printf("  Taille libre : lg_avant = %u\n", controle->lg_avant );
#endif

    /* Pas de limitation sur la partie entire : a bien t faite */
    for ( j = 0 ; j < n_chiffres ; j++ ) {
      chaine[ ( n_chiffres - 1 ) - j ] = '0' + ( uli % 10 );
      uli = uli/10;
    }

    n_cars = n_chiffres;
  }

  return n_cars;
}

/* ~~~~~~~~ Estime la longueur d'une chane aprs remplacement des paramtres ~~~~~~~~
   ATTENTION, le 0 final n'est pas compt
 */

size_t TXT_lg_chaine_finale(const char *masque, va_list parametres)
{
  const char *caractere;
  size_t lg_texte = 0;

  if ( NULL == masque ) {
    (void) fprintf(stderr, "%s,%s l.%u : chane NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0;
  }

#ifdef _DEVELOPPE_
/*   printf("%s. Masque : %s\n", __FUNCTION__, masque); */
#endif

  caractere = masque;
  do {
    if ( '%' == *caractere ) {
      Complexe *cplx;
      const char *chaine;
      double d, e;
      int c, i;
      long li;
      size_t lg_code;
      struct affiche_valeur infos_code;
      unsigned int ui;
      unsigned long int uli;

      caractere = TXT_AnalyseCodeAffichage( caractere, &infos_code );
      switch ( infos_code.type )
      {
      case TXT_AFFICHE_CARACTERE: /* On affiche un seul caractre */
	/* On lit le paramtre, mais juste pour garder la correspondance ensuite */
	c = va_arg( parametres, int ); /* Attention, char promu en int dans le passage de paramtres */

	if ( ' ' == c ) {
	  lg_code = 8;		/* espace : on affiche <espace> */
	} else if ( c < 32 ) {
	  lg_code = 9;		/* Code de contrle : on affiche <ctrl:xx> */
	} else {
	  lg_code = 1;		/* Un caractre : occupe une place */
	}
	break;

      case TXT_AFFICHE_I:	/* On affiche un entier classique */
	i = va_arg( parametres, int );

	lg_code = 0;
	if ( ( i < 0 ) || ( infos_code.proprietes & TXT_CODE_AVEC_SIGNE ) ) {
	  lg_code++;
	  i = abs( i );	/* Peut tre positif  l'origine... */
	}
	if ( 0 == i ) {
	  lg_code++;
	} else {
	  lg_code += 1 + (int) log10( (double) i );
	}
	/* On vrifie qu'il ne faudra pas complter avec des 0 si prcision impose ! */
	if ( ( lg_code < infos_code.lg_avant ) && ( infos_code.proprietes & TXT_CODE_ENTIER_IMPOSE ) ) {
	  lg_code = infos_code.lg_avant;
	  if ( infos_code.proprietes & TXT_CODE_AVEC_SIGNE ) {
	    lg_code++;
	  }
	}
	break;

      case TXT_AFFICHE_UI:	/* On affiche un entier positif */
	ui = va_arg( parametres, unsigned int );

	lg_code = 0;
	if ( infos_code.proprietes & TXT_CODE_AVEC_SIGNE ) {
	  lg_code++;
	}
	if ( 0 == ui ) {
	  lg_code++;
	} else {
	  lg_code += 1 + (int) log10( (double) ui );
	}
	/* On vrifie qu'il ne faudra pas complter avec des 0 si prcision impose ! */
	if ( ( lg_code < infos_code.lg_avant ) && ( infos_code.proprietes & TXT_CODE_ENTIER_IMPOSE ) ) {
	  lg_code = infos_code.lg_avant;
	  if ( infos_code.proprietes & TXT_CODE_AVEC_SIGNE ) {
	    lg_code++;
	  }
	}
	break;

      case TXT_AFFICHE_L:	/* On affiche un entier long */
	li = va_arg( parametres, long );

	lg_code = 0;
	if ( ( li < 0 ) || ( infos_code.proprietes & TXT_CODE_AVEC_SIGNE ) ) {
	  lg_code++;
	  li = labs( li );	/* Peut tre positif  l'origine... */
	}
	if ( 0 == li ) {
	  lg_code++;
	} else {
	  lg_code += 1 + (int) log10( (double) li );
	}
	/* On vrifie qu'il ne faudra pas complter avec des 0 si prcision impose ! */
	if ( ( lg_code < infos_code.lg_avant ) && ( infos_code.proprietes & TXT_CODE_ENTIER_IMPOSE ) ) {
	  lg_code = infos_code.lg_avant;
	  if ( infos_code.proprietes & TXT_CODE_AVEC_SIGNE ) {
	    lg_code++;
	  }
	}
	break;

      case TXT_AFFICHE_UL:	/* On affiche un entier long positif */
	uli = va_arg( parametres, unsigned long int );

	lg_code = 0;
	if ( infos_code.proprietes & TXT_CODE_AVEC_SIGNE ) {
	  lg_code++;
	}
	if ( 0 == uli ) {
	  lg_code++;
	} else {
	  lg_code += 1 + (int) log10( (double) uli );
	}
	/* On vrifie qu'il ne faudra pas complter avec des 0 si prcision impose ! */
	if ( ( lg_code < infos_code.lg_avant ) && ( infos_code.proprietes & TXT_CODE_ENTIER_IMPOSE ) ) {
	  lg_code = infos_code.lg_avant;
	  if ( infos_code.proprietes & TXT_CODE_AVEC_SIGNE ) {
	    lg_code++;
	  }
	}
	break;

      case TXT_AFFICHE_D:	/* Nombre rel */
	d = va_arg( parametres, double );
	lg_code = TXT_nbr_car_D( d, &infos_code );
/* 	printf("%s : double [%g], lg rserve = %lu, %u chiffres avant la virgule, %u aprs\n", */
/* 	       __FUNCTION__, d, (unsigned long) lg_code, infos_code.lg_avant, infos_code.lg_apres); */
	break;

      case TXT_AFFICHE_POURCENT:
	d = va_arg( parametres, double );
	lg_code = TXT_nbr_car_D( d * 100.0, &infos_code );
	lg_code++;		/* Pour le signe % */
	break;

      case TXT_AFFICHE_ERREUR:
	d = va_arg( parametres, double );
	e = va_arg( parametres, double );
	lg_code = TXT_nbr_car_D( d, &infos_code );
	lg_code += TXT_nbr_car_D( e, &infos_code );
	lg_code += 3;		/* "  ", 3 caractres */
	/* On a l le pire des cas possibles, a priori ! */
	break;

      case TXT_AFFICHE_FICHIER:
	chaine = va_arg( parametres, const char * );
	if ( NULL == chaine ) {
	  lg_code = 8;		/* "<absent>, 8 caractres */
	} else {
	  char *pos_sep;

	  pos_sep = strrchr( chaine, CODE_SEPARATEUR ); /* sparateur unix */
	  if ( NULL == pos_sep ) {
	    lg_code = strlen( chaine ); /* %F remplac par le nom complet */
	  } else {
	    lg_code = strlen( pos_sep ) - 1; /* %F remplac par le nom du dernier niveau, sans sparateur */
	  }
	}
	break;

      case TXT_AFFICHE_CHAINE:
	chaine = va_arg( parametres, const char * );
	if ( NULL == chaine ) {
	  lg_code = 6;		/* "<null>, 6 caractres */
	} else {
	  lg_code = strlen( chaine );
	}
	break;

      case TXT_AFFICHE_CAR_PC:
	lg_code = 1;
	break;

      case TXT_AFFICHE_COMPLEXE:
	cplx = va_arg( parametres, Complexe * );
	if ( NULL == cplx ) {
	  lg_code = 1;
	} else {
	  lg_code  = TXT_nbr_car_D( cplx->re, &infos_code );
	  lg_code += TXT_nbr_car_D( cplx->im, &infos_code );
	  lg_code += 5;		/* " + (..) i", 5 caractres */
	}
	/* On a l le pire des cas possibles, a priori ! */
	break;

      default:
	(void) fprintf(stderr, "%s,%s l.%u : Code de contrle inconnu [%d] : ignor\n",
		       __FILE__, __FUNCTION__, __LINE__, infos_code.type);
	lg_code = 0;
      }

      /* On ajoute la longueur trouve au texte */
      lg_texte += lg_code;

      /* On s'occupe de l'unit ventuelle si ncessaire */
      if ( infos_code.proprietes & TXT_CODE_UNITE_INDIQUEE ) {
	const char *unite;

	lg_texte++;		/* Espace sparateur */
	unite = va_arg( parametres, const char * );
	if ( NULL == unite ) {
	  /* On ne met aucune unit ! */
	} else {
	  lg_texte += strlen( unite );
	}
      }
    } else {
      /* Caractre normal : compte pour 1 */
      lg_texte++;

      /* On passe au suivant */
      caractere++;
    }
  } while ( *caractere != 0 );

  return lg_texte;
}

/* ~~~~~~~~ Manipulations simples sur les chanes de caractres ~~~~~~~~ */

/* -------- Ajouter un texte  une chane existante --------
   Renvoie l'adresse de la nouvelle chane (NULL en cas d'erreur)
   Elle peut tre identique  l'ancienne [realloc() ==> PAS DE CHANE CONSTANTE EN 1ER PARAMTRE]
   L'ancienne chane est perdue, en gnral
 */

char *TXT_AjouteTexte(char *chaine1, const char *chaine2)
{
  char *resultat;

  if ( NULL == chaine2 ) return chaine1;

  if ( NULL == chaine1 ) { /* On cre alors la chane */
    resultat = strdup( chaine2 );
    if ( NULL == resultat ) {
      (void) fprintf( stderr, "%s,%s l.%u : Impossible de copier la chane 2\n",
		      __FILE__, __FUNCTION__, __LINE__ );
    }

    return resultat;
  }

  /* On rserve la place pour la nouvelle */
  resultat = (char *) realloc( chaine1, strlen( chaine1 ) + strlen( chaine2 ) + 1 );
  if ( NULL == resultat ) {
    (void) fprintf( stderr, "%s,%s l.%u : Mmoire insuffisante pour ajouter le texte \"%s\" au texte \"%s\"\n",
		    __FILE__, __FUNCTION__, __LINE__, chaine2, chaine1 );
    /* La chane 1 n'est pas modifie */
    return NULL;
  }

  /* On copie la chane 2  la suite de la chaine 1 */
  strcat( resultat, chaine2 );

  /* On renvoie la chane ainsi cre */
  return resultat;
}

/* ~~~~~~~~ Conversions diverses ~~~~~~~~ */

/* -------- Nombre de chiffres pour crire le double, en fonction du contrle indiqu -------- */

unsigned long TXT_nbr_car_D (double valeur, struct affiche_valeur *controle)
{
  int retour, av_exposant = FAUX;
  unsigned long chiffres_avant;
  unsigned int chiffres_exposant;
  unsigned long longueur = 0;

  if ( controle->proprietes & TXT_CODE_AVEC_SIGNE ) {
    longueur = 1;		/* Signe quoiqu'il arrive */
    valeur = fabs( valeur );	/* Pour ne pas compter deux fois le signe - */
  }

  /* On regarde les chiffres avant la virgule, ainsi que ceux de l'exposant */
  retour = TXT_AnalyseDouble( valeur, &chiffres_avant, &chiffres_exposant );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans TXT_AnalyseDouble [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return 0;
  }

  /* Place occupe par la partie entire */
  if ( controle->proprietes & TXT_CODE_ENTIER_IMPOSE ) {
    longueur += controle->lg_avant;
    if ( controle->lg_avant < chiffres_avant ) {
      /* Si on dpasse, on utilisera la notation scientifique ! */
      av_exposant = VRAI;
    }
  } else {
    /* Arbitraire : si suprieur  5 chiffres, on utilise la notation scientifique */
    if ( chiffres_avant > 5 ) {
      av_exposant = VRAI;
      longueur++;	/* Donc un seul chiffre utilis */
    } else {
      longueur += chiffres_avant;
    }
  }

  /* Place occupe par la virgule [toujours compte, par sret] */
  longueur++;

  /* Place occupe par la partie dcimale */
  if ( controle->proprietes & TXT_CODE_DECIMALE_IMPOSEE ) {
    longueur += controle->lg_apres;
    if ( log10( fabs(valeur) ) < ( controle->lg_apres / 2 ) ) {
      av_exposant = VRAI;	/* Notation scientifique... */
    }
  } else {
    /* Par dfaut : 10 chiffres */
    longueur += 10;
    if ( fabs( valeur ) < 0.001 ) {
      av_exposant = VRAI;	/* Notation scientifique... */
    }
  }

  /* Place occupe par l'exposant, si ncessaire */
  if ( VRAI == av_exposant ) {
    longueur += chiffres_exposant;
    longueur += 6;		/* "  10^" = 6 caractres */
  }

  return longueur;
}

/* -------- Nombre de chiffres (donc de caractres) pour crire la valeur indique, en base 10 -------- */

unsigned long TXT_nbr_car_I(int valeur)
{
  unsigned long nbr_cars = 1;

  if (valeur == 0) return 1;
  if (valeur < 0) { /* Ngative : le signe - */
    nbr_cars++;
    valeur = -valeur;
  }
  nbr_cars += (unsigned long) log10( (double) valeur ); /* les chiffres */

  return nbr_cars;
}

unsigned long TXT_nbr_car_UI(unsigned int valeur)
{
  unsigned long nbr_cars = 1;

  if (valeur < 10) return 1;
  nbr_cars += (unsigned long) log10( (double) valeur ); /* les chiffres */

#ifdef _DEVELOPPE_
/*   printf("TXT_nbr_car_UI : %u a %lu chiffres\n", valeur, nbr_cars); */
#endif
  return nbr_cars;
}

unsigned long TXT_nbr_car_L(long valeur)
{
  unsigned long nbr_cars = 1;

  if (valeur == 0) return 1;
  if (valeur < 0) { /* Ngative : le signe - */
    nbr_cars++;
    valeur = -valeur;
  }
  nbr_cars += (unsigned long) log10( (double) valeur ); /* les chiffres */

  return nbr_cars;
}

unsigned long TXT_nbr_car_UL(unsigned long valeur)
{
  if (valeur < 10) return 1;

  return 1 + (unsigned long) log10( (double) valeur );
}

/* ~~~~~~~~ Comptages divers ~~~~~~~~ */

/* -------- Nombre d'occurences d'un caractre dans une chane -------- */

unsigned long TXT_compte_caractere(const char *chaine, int caractere)
{
  const char *c;
  unsigned long i = 0;

  if (chaine == NULL) return 0;

  if ( 0 == caractere ) {	/* On compte les quivalents d'espaces */
    c = chaine;
    while (*c != 0) { /* On n'est pas en fin de chane */
      if ( isspace( *c ) ) i++;
      c++;
    }
  } else {
    c = chaine;
    while (*c != 0) { /* On n'est pas en fin de chane */
      if (*c == caractere) i++;
      c++;
    }
  }

  return i;
}

/* ~~~~~~~~ Suite d'intervalles  partir d'une chane dcrivant cette suite ~~~~~~~~ */

static int traite_intervalle(const char *chaine, Intervalle_I *intervalle)
{
  char *fin_debut;
  unsigned long i = 0;
  int c;

  do {
    c = chaine[i];
    if (c == 0) { /* Fin de chane */
      return FAUX; /* Problme */
    }
  } while ( (isspace(c)) || (c == ',') );

  if (c == '-') { /* Intervalle de la forme -xxx */
    intervalle->debut = -1; /* Pas de minimum */
    i++;
    intervalle->fin = strtol(chaine + i, NULL, 10);

    return VRAI;
  }

  intervalle->debut = strtol(chaine + i, &fin_debut, 10);
  if ( NULL == fin_debut ) { /* Chiffre isol */
    intervalle->fin = intervalle->debut;
    return VRAI;
  }
  while ( isspace( *fin_debut ) )
    fin_debut++;
  if ( 0 == *fin_debut ) { /* Chiffre isol suivi d'espaces, ignors */
    intervalle->fin = intervalle->debut;
    return VRAI;
  }
  if ( *fin_debut == '-' ) { /* Nombre- ??? */
    fin_debut++;
    while ( isspace( *fin_debut ) )
      fin_debut++;

    if ( isdigit( *fin_debut ) ) { /* Nombre - Nombre */
      intervalle->fin = strtol( fin_debut , NULL, 10);
      if (intervalle->fin < intervalle->debut) {
	int i = intervalle->fin; intervalle->fin = intervalle->debut; intervalle->debut = i;
      }
    } else { /* Nombre - xxx : pas de borne suprieure */
      intervalle->fin = -1;
    }
    return VRAI;
  }

  /* Chane incorrecte ! Considre comme chiffre isol */
  if ( (*fin_debut != ',') && (*fin_debut != 0 ) ) {
    (void) fprintf(stderr, "traite_intervalle : FORMAT DE DFINITION D'INTERVALLE INCORRECT [%s]\n", chaine);
  }

  intervalle->fin = intervalle->debut;

  return VRAI;
}

/* -------- Fonction principale --------------------------------------------\
   |  Renvoie la table des intervalles crs,  librer avec free()         |
   |                                                                        |
   | Format : -> les intervalles sont spars par des virgules              |
   |          -> chiffre seul : intervalle rduit  l'entier considr      |
   |          -> a-b          : tous les entiers entre a et b (inclus)      |
   |          -> -b, a-       : idem, mais illimit                         |
   \----------------------------------------------------------------------- */

Intervalle_I *TXT_genere_intervalles(const char *chaine, unsigned long *nbr_intervalles)
{
  const char *chaine_traitee;
  unsigned long nbr_i = 0, i;
  Intervalle_I *intervalles;

  if ( (chaine == NULL) || (nbr_intervalles == NULL) ) return NULL;

  /* On compte le nombre d'intervalles = nombre de virgules + 1 */
  nbr_i = 1 + TXT_compte_caractere(chaine, ',');
  intervalles = (Intervalle_I *) malloc( sizeof(Intervalle_I) * nbr_i );
  if (intervalles == NULL) {
    fprintf(stderr, "txt_utils.c/TXT_genere_intervalles : MMOIRE INSUFFISANTE\n");
    fprintf(stderr, "  Rservation de la liste d'intervalles impossible\n");
    fprintf(stderr, "  Taille demande = %lu octets\n", (unsigned long) (sizeof(Intervalle_I) * nbr_i) );
    return NULL;
  }
  *nbr_intervalles = nbr_i;

  chaine_traitee = chaine;
  if (nbr_i == 1) { /* Un seul intervalle */
    traite_intervalle(chaine_traitee, intervalles);
  } else {
    for (i = 0 ; i < nbr_i ; i++) {
      traite_intervalle(chaine_traitee, intervalles + i);
      chaine_traitee = strchr(chaine_traitee, ',');
      if (chaine_traitee != NULL)
	chaine_traitee++; /* On se place juste aprs la virgule */
    }
  }

#ifdef _DEVELOPPE_
  for (i = 0 ; i < nbr_i ; i++) {
    printf("Intervalle %lu : [%d, %d]\n", i, intervalles[i].debut, intervalles[i].fin);
  }
#endif

  return intervalles;
}

/* -------- Renvoie VRAI si la valeur indique appartient  l'un des intervalles donns -------- */

int TXT_est_dans_intervalle(int valeur, Intervalle_I *intervalles, unsigned long nbr_intervalles)
{
  unsigned long i;

  if (intervalles == NULL) return FAUX;

  for (i = 0 ; i < nbr_intervalles ; i++, intervalles++) {
    if (intervalles->debut == -1) { /* Intervalle ]-infini, valeur] */
      if (valeur <= intervalles->fin) return VRAI;
    } else if (intervalles->fin == -1) { /* Intervalle [valeur, +infini[ */
      if (valeur >= intervalles->debut) return VRAI;
    } else { /* Intervalle classique, ventuellement restreint  1 point */
      if ( (valeur >= intervalles->debut) && (valeur <= intervalles->fin) ) {
/*	printf("Valeur %d bien dans [%d, %d]\n", valeur, intervalles->debut, intervalles->fin); */
	return VRAI;
      }
    }
  }

  return FAUX; /* Aucun intervalle ne convient */
}

/* ~-~-~-~-~-~-~-~-~ Routines de dcomposition d'un code de format ~-~-~-~-~-~-~-~-~ */

/* -------- Analyser le code de format --------
   Renvoie la position du premier caractre aprs le code de format dans la chane
           inchang en cas d'erreur

   Codage : commence forcment par un % [%% : spcial, pour remplacer un %]
            modificateurs : facultatifs ; ordre impos
                1er : * ; si prsent = il y a une unit d'indique [char *, aprs la valeur]
                2e  :   ; si indiqu = alignement  droite [espaces ajoutes avant le 1er chiffre]
                3e  : + ; si indiqu = ajouter le signe quoi qu'il arrive [- est aussi accept]
            prcision     : facultatif, de la forme x ou x.y ou .y [x seul autoris pour les entiers...]
                x = nombre entier positif, nombre de chiffres avant la virgule
                y = nombre entier positif, nombre de chiffres aprs la virgule
            code          : quel type de valeur(s) doi(ven)t tre lue(s)
                c   : caractre
                C   : nombre complexe [Re puis Im, deux doubles]
		d   : entier          [int]
		e   : rel + erreur   [X, erreur (en cart-type), deux doubles]
		f   : rel            [double]
                F   : nom de fichier  [char *]
		ld  : entier long     [long]
                lu  : entier long >0  [unsigned long]
		P   : pourcentage     [double, entre 0 et 1 -> affichage *100]
		s   : chane          [char *]
		u   : entier > 0      [unsigned int]

   -------- */

const char *TXT_AnalyseCodeAffichage(const char *code, struct affiche_valeur *valeurs)
{
  const char *code_original;

  /* Contrles prliminaires */
  if ( NULL == code ) return NULL;
  if ( NULL == valeurs ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucun lieu de stockage de l'analyse !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return code;
  }
  if ( *code != '%' ) {
    (void) fprintf(stderr, "%s, %s l.%u : Code ne commenant pas par un %% [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, code);
    return code;
  }
  code_original = code;

  /* On met  0 */
  valeurs->type = TXT_AFFICHE_INCONNU;
  valeurs->proprietes = 0;
  valeurs->n_max = 0;
  valeurs->lg_avant = 0;
  valeurs->lg_apres = 10;	/* Pas plus de 10 chiffres par dfaut */
  valeurs->longueur = 0;

  /* Analyse du caractre suivant */
  code++;
  if ( 0 == *code ) {
    (void) fprintf(stderr, "%s, %s l.%u : %% isol en fin de chane, compt comme %%%% !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    valeurs->type = TXT_AFFICHE_CAR_PC;
    return code;
  }
  if ( '%' == *code ) { /* "%%" : caractre % */
    valeurs->type = TXT_AFFICHE_CAR_PC;
    return (code + 1 );
  }
  /* Contrle du format d'affichage */
  if ( '*' == *code ) {	/* "%*" : on indique une unit */
    valeurs->proprietes |= TXT_CODE_UNITE_INDIQUEE;
    code++;
  }
  if ( ' ' == *code ) { /* "% " : on impose l'alignement  droite sur le nombre de chiffres indiqu */
    valeurs->proprietes |= TXT_CODE_ALIGNE_A_DROITE;
    code++;
  }
  if ( ( '+' == *code ) || ( '-' == *code ) ) {	/* "%+" ou "% -" : on met un signe mme si valeur positive */
    valeurs->proprietes |= TXT_CODE_AVEC_SIGNE;
    code++;
  }
  if ( isdigit( *code ) ) {	/* "%5" : on fixe la taille maximale [nombre de chiffres avant la virgule] */
    valeurs->proprietes |= TXT_CODE_ENTIER_IMPOSE;
    valeurs->lg_avant = (unsigned int) TXT_CNV_vNombre_UL( code, &code );
  }
  if ( '.' == *code ) { /* . : indique que l'on impose le nombre de chiffres aprs la virgule */
    valeurs->proprietes |= TXT_CODE_DECIMALE_IMPOSEE;
    code++;
  }
  if ( isdigit( *code ) ) {	/* "%.5" ou quivalent : on indique le nombre de dcimales */
    valeurs->lg_apres = (unsigned int) TXT_CNV_vNombre_UL( code, &code );
  }
  if ( 0 == *code ) {		/* On vrifie qu'il n'y a pas de problme ! */
    (void) fprintf(stderr, "%s, %s l.%u : Code de format non termin [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, code_original);
    return code;
  }

  /* Type de donne  afficher */
  switch ( *code )
  {
  case 'c':
    valeurs->type = TXT_AFFICHE_CARACTERE;
    break;

  case 'C':
    valeurs->type = TXT_AFFICHE_COMPLEXE;
    break;

  case 'd':
    valeurs->type = TXT_AFFICHE_I;
    break;

  case 'e':
    valeurs->type = TXT_AFFICHE_ERREUR;
    break;

  case 'f':
    valeurs->type = TXT_AFFICHE_D;
    break;

  case 'F':
    valeurs->type = TXT_AFFICHE_FICHIER;
    break;

  case 'l':
    code++;
    if ( 'd' == *code ) {
      valeurs->type = TXT_AFFICHE_L;
    } else if ( 'u' == *code ) {
      valeurs->type = TXT_AFFICHE_UL;
    } else {
      (void) fprintf(stderr, "%s, %s l.%u : Code de format inconnu [%s]\n",
		     __FILE__, __FUNCTION__, __LINE__, code_original);
      return code;
    }
    break;

  case 'P':
    valeurs->type = TXT_AFFICHE_POURCENT;
    break;

  case 's':
    valeurs->type = TXT_AFFICHE_CHAINE;
    break;

  case 'u':
    valeurs->type = TXT_AFFICHE_UI;
    break;

  default:
    (void) fprintf(stderr, "%s, %s l.%u : Code de format inconnu [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, code_original);
    code--; /* Pour ne pas passer ce caractre  tort */
  }
  code++;

  return code;
}

/* -------- Analyse un double pour prciser combien de chiffres avant la virgule (signe inclus)
                                         et combien de chiffres dans l'exposant --------- */

static int TXT_AnalyseDouble(double valeur, unsigned long *avant, unsigned int *exposant)
{
  double log_valeur;

#ifdef _DEVELOPPE_
  if ( ( NULL == avant ) || ( NULL == exposant ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel avec un paramtre NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }
#endif

  if ( 0.0 == valeur ) {
    *avant = 1;
    *exposant = 0;
    return 0;
  }

  *avant = 0;

  if ( valeur < 0.0 ) {
    (*avant)++;			/* Le signe - */
    valeur = -valeur;
  }

  /* On prend le log dcimal de la valeur */
  log_valeur = log10( valeur );
#ifdef _DEVELOPPE_
/*   printf( "%s : %g [log=%g], ", __FUNCTION__, valeur, log_valeur ); */
#endif
  if ( log_valeur < 0 ) {
    /* log(x) < 0 <=> x < 1 : 1 chiffre avant la virgule, exposant ngatif */
    (*avant)++;
    *exposant = 1; /* Signe moins de l'exposant */
    log_valeur *= -1.0;		/* On veut un exposant >0 */
  } else {
    /* log(x) >= 0 <=> x >= 1 : le nombre de chiffres de la partie entire vaut 1 + (int) log(x) ; exposant positif  */
    (*avant) += 1 + (unsigned long) log_valeur;
    *exposant = 0;
  }

  /* Calcul de l'exposant : x = n * 10^E ==> log(x) = log(n) + E
     comme 1 <= n < 10, 0 <= log(n) < 1 donc E = (int) log(x) et on veut le nombre de chiffres de E */
  *exposant += TXT_nbr_car_UI( (unsigned int) log_valeur );

  /* Termin, tout s'est bien pass */
#ifdef _DEVELOPPE_
/*   printf( "%lu chiffres avant la virgule, %u chiffres dans l'exposant\n", *avant, *exposant ); */
#endif
  return 0;
}

/* ~~~~~~~~ Version autonome, pour tests ~~~~~~~~ */
#ifdef AFFICHAGE_

static void afficher_code( char *code )
{
  struct affiche_valeur code_traite;
  const char *fin_code;
  
  fin_code = TXT_AnalyseCodeAffichage( code, &code_traite );
  if ( NULL == fin_code ) {
    (void) fprintf(stderr, "%s, %s l.%u : Problme grave dans l'analyse\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }
  if ( *fin_code != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Caractres ignor en fin de code : %s\n",
		   __FILE__, __FUNCTION__, __LINE__, fin_code);
  }

  switch ( code_traite.type )
  {
  case TXT_AFFICHE_CARACTERE:
    printf( "Afficher un caractre..." );
    break;

  case TXT_AFFICHE_I:
    printf( "Afficher un entier..." );
    break;

  case TXT_AFFICHE_UI:
    printf( "Afficher un entier positif..." );
    break;

  case TXT_AFFICHE_L:
    printf( "Afficher un entier long..." );
    break;

  case TXT_AFFICHE_UL:
    printf( "Afficher un entier long positif..." );
    break;

  case TXT_AFFICHE_D:
    printf( "Afficher un rel..." );
    break;

  case TXT_AFFICHE_POURCENT:
    printf( "Afficher un pourcentage..." );
    break;

  case TXT_AFFICHE_ERREUR:
    printf( "Afficher un rel avec son incertitude" );
    break;

  case TXT_AFFICHE_FICHIER:
    printf( "Afficher un nom de fichier..." );
    break;

  case TXT_AFFICHE_CHAINE:
    printf( "Afficher une chane de caractres" );
    break;

  case TXT_AFFICHE_CAR_PC:
    printf( "Afficher le caractre %%" );
    break;

  case TXT_AFFICHE_COMPLEXE:
    printf( "Afficher un nombre complexe" );
    break;

  case TXT_AFFICHE_INCONNU:
    printf( "Afficher autre chose !" );
    break;
  }

  printf("[maximal = %u, avant = %u, aprs = %u]", code_traite.n_max, code_traite.lg_avant, code_traite.lg_apres);
  if ( code_traite.proprietes & TXT_CODE_AVEC_SIGNE ) {
    printf( " Signe impos" );
  }
  if ( code_traite.proprietes & TXT_CODE_ENTIER_IMPOSE ) {
    printf( " lg avant impose" );
  }
  if ( code_traite.proprietes & TXT_CODE_DECIMALE_IMPOSEE ) {
    printf( " lg aprs impose" );
  }
  if ( code_traite.proprietes & TXT_CODE_UNITE_INDIQUEE ) {
    printf( " avec unit" );
  }

  printf("\n");
}

int main(void)
{
  char *chaine = NULL;
  char code[ 200 ];
  struct affiche_valeur infos_code;

  char texte[ 500 ];
  double d, e, Re, Im;
  int c, i;
  long int li;
  long unsigned int uli;
  unsigned int ui;

  /* Pr-test */
  const char *masque = "1 %s = %*f";

  chaine = TXT_genere_chaine( masque, "A", 1e10, "m" );
  printf( "Chane obtenue :\n%s\n", chaine );
  free( chaine ); chaine = NULL;

  printf("Entrez le code de format :\n" );
  scanf( "%[^\r\n]", code );

  afficher_code( code );

  /* Analyse du code */
  (void) TXT_AnalyseCodeAffichage( code, &infos_code );

  switch ( infos_code.type )
  {
  case TXT_AFFICHE_CARACTERE: /* On affiche un seul caractre */
    printf( "Entrez un caractre (par son code ASCII dcimal)...\n" );
    scanf( "%d", &c );

    chaine = TXT_genere_chaine( code, c );
    printf("Caractre code %d [%c] : %s", c, c, chaine );
    break;

  case TXT_AFFICHE_I:
    printf( "Entrez un entier dcimal...\n" );
    scanf("%d", &i );

    chaine = TXT_genere_chaine( code, i );
    printf( "Entier I, %d = %s\n", i, chaine );
    break;

  case TXT_AFFICHE_UI:
    printf( "Entrez un entier dcimal positif...\n" );
    scanf("%u", &ui );

    chaine = TXT_genere_chaine( code, ui );
    printf( "Entier UI, %u = %s\n", ui, chaine );
    break;

  case TXT_AFFICHE_L:
    printf( "Entrez un entier dcimal ...\n" );
    scanf("%ld", &li );

    chaine = TXT_genere_chaine( code, li );
    printf( "Entier L, %ld = %s\n", li, chaine );
    break;

  case TXT_AFFICHE_UL:
    printf( "Entrez un entier dcimal positif...\n" );
    scanf("%lu", &uli );

    chaine = TXT_genere_chaine( code, uli );
    printf( "Entier UL, %lu = %s\n", uli, chaine );
    break;

  case TXT_AFFICHE_D:
    printf( "Entrez un nombre rel...\n" );
    scanf("%lf", &d );

    chaine = TXT_genere_chaine( code, d );
    printf( "Rel, %.15g (%.15f, %.15e) = %s\n", d, d, d, chaine );
    break;

  case TXT_AFFICHE_POURCENT:
    printf( "Entrez un nombre rel...\n" );
    scanf("%lf", &d );

    chaine = TXT_genere_chaine( code, d );
    printf( "Pourcentage, %.15g (%.15f, %.15e soit %g %%) = %s\n", d, d, d, d, chaine );
    break;

  case TXT_AFFICHE_ERREUR:
    printf( "Entrez un nombre rel...\n" );
    scanf("%lf", &d );

    printf( "Entrez l'erreur associe...\n" );
    scanf("%lf", &e );

    chaine = TXT_genere_chaine( code, d, e );
    printf( "Valeur exp., %.15g  %.15g (%.15f  %.15f, %.15e  %.15e) = %s\n", d, e, d, e, d, e, chaine );
    break;

  case TXT_AFFICHE_FICHIER:
    printf( "Entrez un nom de fichier...\n" );
    scanf( "%s", texte );

    chaine = TXT_genere_chaine( code, texte );
    printf( "Fichier, %s = %s\n", texte, chaine );
    break;
    
  case TXT_AFFICHE_CHAINE:
    printf( "Entrez un texte...\n" );
    scanf( "%s", texte );

    chaine = TXT_genere_chaine( code, texte );
    printf( "Texte, %s = %s\n", texte, chaine );
    break;

  case TXT_AFFICHE_CAR_PC:
    chaine = TXT_genere_chaine( code );
    printf( "Pourcentage, %% = %s\n", chaine );
    break;

  case TXT_AFFICHE_COMPLEXE:
    printf( "Entrez la partie relle...\n" );
    scanf("%lf", &Re );

    printf( "Entrez la partie imaginaire...\n" );
    scanf("%lf", &Im );

    chaine = TXT_genere_chaine( code, Re, Im );
    printf( "Complexe, %.15g + %.15gi (%.15f + %.15fi, %.15e + %.15ei) = %s\n", Re, Im, Re, Im, Re, Im, chaine );
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Code de contrle inconnu [%d] : ignor\n",
		   __FILE__, __FUNCTION__, __LINE__, infos_code.type);
  }

  free( chaine ); chaine = NULL;

  return 0;
}

#endif /* AFFICHAGE_ */
