/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 1999, Emmanuel CURIS
   Cration : 11 fvrier 1999

   Fonctions mathmatiques diverses
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>

#include "science.h"

/* ~~~~~~~~ Variables internes ~~~~~~~~ */

/* ~~~~~~~~ Fonctions internes ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~~ */

/* -------- Fonction gamma incomplte

             /infini
             |         1   a - 1  -t
   Q(a, x) = |       ---- t      e   dt o G est la fonction Gamma
             |       G(a)
            /x

   C'est en fait 1 - fonction de rpartition d'une loi Gamma(a, 1),
   c'est--dire p(X > x) si X suit une loi gamma, donc Q(<0) = 1

   -------- */

#define NBR_ITER_SERIE    100 /* Nombre maximal d'itrations pour le calcul avec la srie */
#define NBR_ITER_FRACTION 100 /* Nombre maximal d'itrations pour le calcul avec le dveloppement
			         en fractions continues*/

#define FP_MIN 1e30 /* Plus petite valeur reprsentable en flottant */

/* #define algo_n_et_r */
/*  dfinir si on veut utiliser l'algo. des N & R
   marche bien pour x < a + 1 (utilisation du dveloppement en srie)
   mais diverge pour x > a + 1 (utilisation des fractions continues), raison inconnue
*/

#ifdef algo_n_et_r

double gamma_q(double a, double x)
{
  double log_gamma;

  if ( (x < 0) || (a < 0) ) 
  {
    return 1;
  }

  log_gamma = lgamma(a); /* lgamma donne le logarithme de la fonction gamma */
  printf("Valeur trouve pour log(G(a) : %f (%g)\n", log_gamma, log_gamma);

  if (x < a + 1.0) /* Dveloppement en srie pour les petites valeurs de x */
  {
    unsigned short int n;
    double somme, delta = 1/a, ap;

    ap = a;
    somme = delta;

#ifdef _DEVELOPPE_
    printf("gamma_q : on utilise le dveloppement en srie\n", x, a);
#endif
    for (n = 1 ; n < NBR_ITER_SERIE ; n++)
    {
      ap++;
      delta *= x / ap;
      somme += delta;

      if ( fabs(delta) < fabs(somme) * 3.0e-7 ) /* On a converg */
      {
	double g_ser = -x + a * log(x) - log_gamma;
	g_ser = somme * exp(g_ser);
	return 1.0 - g_ser;
      }
    }
#ifdef _DEVELOPPE_
    printf("gamma_q : la convergence a chou (x = %f, a = %f)\n", x, a);
#endif
    return 0.0; /* Nombre trop grand */
  }
  else
  {
    /* Ici, on a x >= a+1 : on utilise le dveloppement en fractions continues */
    unsigned short int i;
    double b, c, d, delta, h, a_n;
    double g_fc;

#ifdef _DEVELOPPE_
    printf("gamma_q : on utilise le dveloppement en fractions continues\n", x, a);
#endif
    b = 1.0 + x - a;
    c = 1 / DLB_MIN;
    d = 1/b;
    h = d;

    for (i = 0 ; i < NBR_ITER_FRACTION ; i++)
    {
      a_n = - i * (i - a);
      b += 2;
      d *= a_n; d += b;
      if ( fabs(d) < DLB_MIN) d = DLB_MIN;
      d = 1/d;
      c = b + a_n/c;
      if ( fabs(c) < DLB_MIN) c = DLB_MIN;
      delta = d * c;
      h *= delta;
      if ( fabs(delta - 1) < 1e-7 )
	break;
    }

#ifdef _DEVELOPPE_
    if (i >= NBR_ITER_FRACTION)
      printf("gamma_q : la convergence a chou (x = %f, a = %f)\n", x, a);
#endif

    g_fc = -x + a*log(x) - log_gamma;
    return h * exp(g_fc);
  }
}

#else /* Algorithme lent mais sr : toujours le dveloppement en srie entire
	 Voir s'il y a une astuce si x >> a (le rsidu doit tre quasi-nul dans ces conditions !)
         viterait probablement les tests de dpassement de capacit (qui ralentissent) 
      */

double gamma_q(double a, double x)
{
  double log_gamma;
  unsigned short int n;
  double somme, delta, ap;

  if ( (x <= 0.0) || (a <= 0.0) ) 
  {
    return 1.0;
  }

  delta = 1/a;
  log_gamma = lgamma(a); /* lgamma donne le logarithme de la fonction gamma */

  ap = a;
  somme = delta;

  for (n = 1 ; n < 10000 ; n++) /* 10000 itrations pour tre sr de converger mme
				   si x > a+1 (convergence lente) */
  {
    double i;

    ap++;
    i = x / ap;

    if ( (log10(i) + log10(delta)) > (double) DBL_MAX_10_EXP ) /* Ceci ralenti ! Mais si dbordement de
								  capacit, FPE sur alpha !
							       */
    {
      fprintf(stderr, "Itration %d : on dpasse la capacit de la machine !\n", n);
      return 1.0 - somme * exp( -x + a * log(x) - log_gamma );
    }

    delta *= i; /* hack sinon floating point exception avec cc digital ! */
    somme += delta;

    if ( fabs(delta) < fabs(somme) * 3.0e-7 ) /* On a converg */
    {
      double g_ser = -x + a * log(x) - log_gamma;
      g_ser = somme * exp(g_ser);
#ifdef _DEVELOPPE_
      printf("Convergence en %d itrations\n", n+1);
#endif
      return 1.0 - g_ser;
    }
  }
#ifdef _DEVELOPPE_
  printf("gamma_q : la convergence a chou (x = %f, a = %f)\n", x, a);
#endif
  return 0.0; /* Nombre trop grand */
}
#endif /* algo_n_et_r */

#ifdef SANS_LGAMMA_ /* Pas de fonction lgamma dfinie dans la bib. math */

double lgamma(double x)
{
  double y, tmp, ser;
  static double cof[6] = {  76.18009172947146,
			   -86.50532032941677,
			    24.01409824083091,
			    -1.231739572450155,
			     0.1208650973866179e-2,
			    -0.5395239384953e-5 };
  int j;

  if (x <= 0) /* Fonction non dfinie */
    return 0;
  y = x;
  tmp  = x + 5.5;
  tmp -= (x + 0.5) * log(tmp);
  ser = 1.000000000190015;
  for (j = 0 ; j <= 5 ; j++)
    ser += cof[j] / ++y;

  return -tmp + log( 2.5066282746310005 * ser / x);
}

#endif /* SANS_LGAMMA_ */

/* -------------- Fonction de Bessel J0 ------------------------------------------------- */

double bessel(double x)
{
  double x_abs,x_carre,y1,y2,inter;
  double retour;
	
  x_abs = fabs(x); /* Valeur absolue */

  if ( x_abs < 8) /* Approximation polynomiale */
  {
    x *= x;
    retour  = 57568490574.0 + x * ( -13362590354.0 + x * ( 651619640.7 
	                    + x * ( -11214424.18   + x * ( 77392.33017
		            - x * 184.9052456 ) ) ) );
    retour /= 57568490411.0 + x * (  1029532985.0  + x * ( 9494680.718
		            + x * (   59272.64853  + x * ( 267.8532712
		            + x ) ) ) );
  }
  else
  {
    x_carre  = 8 / x_abs;
    x_carre *= x_carre;
    inter = x_abs - 0.785398164;
    y1 = 1 + x_carre * ( -0.1098628627e-2 + x_carre * ( 0.2734510407e-4
		       + x_carre * ( -0.2073370639e-5 + x_carre * ( 0.2093887211e-6 ) ) ) );
    y2 = -0.1562499995e-1 + x_carre * ( 0.1430488765e-3 + x_carre * ( -0.6911147651e-5
			  + x_carre * ( 0.7621095161e-6 - x_carre *    0.934935152e-7 ) ) );
    retour  = sqrt( 0.636619772 / x_abs );
    retour *= cos(inter) * y1 - 8 / x_abs * sin(inter) * y2;
  }
	
  return retour;
}

/* -------------- Fonction de Bessel modifie I0 ---------------------------------------- */

double bessel2(double x)
{
  double ans;
  double y;
	
  x = fabs(x);
  if ( x < 3.75 ) {
    y  = x / 3.75;
    y *= y;

    ans = 1.0 + y * (3.5156229 + y * (3.0899424 + y * ( 1.2067492 + y * ( 0.2659732
					+ y * (0.360768e-1 + y * 0.45813e-2) ) ) ) );
  } else {
    y = 3.75 / x;
    ans  = 0.39894228 + y * (0.1328592e-1 + y * ( 0.225319e-2  + y * (-0.157565e-2
					+ y * (-0.157565e-2 + y * ( 0.916281e-2  + y * (-0.2057706e-1
					+ y * (0.2635537e-1 + y * (-0.1647633e-1 + y * 0.392377e-2))))))));
    ans *= exp(x) / sqrt(x);
  }
	
  return ans;
}

#ifdef SANS_ERF_

/* -------- Fonction erf (demi-fonction de rpartition de la loi de Laplace-Gauss)

                        /x
                2       |         2
   erf(x) = ----------  |   exp(-t ) dt
            racine(Pi)  |
                        /0

   Algorithme des Numerical Recipes fond sur Tchebycheff [la formule est aussi dans le Saporta]
*/

double erf(double x)
{
  double erf;
  double t;

  t = 1.0 / (1.0 + 0.5 * fabs(x));

  erf  = t * 0.17087277;
  erf += -0.82215223;
  erf *= t;
  erf += 1.48851587;
  erf *= t;
  erf += -1.13520398;
  erf *= t;
  erf += 0.27886807;
  erf *= t;
  erf += -0.18628806;
  erf *= t;
  erf += 0.09678418;
  erf *= t;
  erf += 0.37409196;
  erf *= t;
  erf += 1.00002368;
  erf *= t;
  erf += -1.26551223 - x*x;
  erf = t * exp(erf);

  if (x >  0.0)
    return 1 - erf;
  else
    return erf - 1;
}

#endif /* SANS_ERF_ */

/* -------- Fonction beta B (intgrale eulrienne de 2e espce)

                        / 1
             G(x)G(y)   |    x - 1        y - 1
   B(x, y) = -------- = |   t      (1 - t)      dt
             G(x + y)   |
                        / 0

*/

double beta(double x, double y)
{
  double lbeta;

  lbeta = lgamma(x) + lgamma(y) - lgamma(x + y);
  return exp(lbeta);
}
