/* Bibliothèque X-Window pour les interfaces
            Routines générales
	    Compatible avec gem_gene.c (version GEM : référence)
   (C) 1998-2005, Emmanuel CURIS - Création le 19 mars 1998
*/

#define _LIBRAIRIE_X_ 

#ifdef _HP_UNIX_
# define _XOPEN_SOURCE_EXTENDED /* Pour que ça passe sous HP-Unix
				  -- mais ne pas le mettre tout le temps : bloque sous Irix 6.2 (...) */
#endif
#define _BSD_COMPAT /* Pour que ça passe à peu-près sous Irix avec le mode POSIX
		       Il reste cependant des warnings pour setitimer -> à voir ! */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <errno.h>
#include <sys/select.h>

#include "bib_interface.h"

/* ****** Variables globales limitées à ce module ************** */

static XEvent vx_evenement; /* Ancien événement souris - static global pour la gestion des clics multiples */
static int    BI_n_clic;    /* Version interne pour éviter des problèmes bêtes */

#define TEMPS_ATTENTE_DOUBLE_CLIC (150)	 /* Délai maximal entre deux clics (en ms) */

/**************** Fonctions internes à ce module *****************/

extern int BI_InitRedessins   (void); /* Dans X_redessins.c */
extern int BI_TermineRedessins(void); /* Dans X_redessins.c */

static int BIX_AttendAvecTimer(int masque, int rect, int x_rc, int y_rc, int larg_rc, int haut_rc, unsigned long tps_max);
static int BIX_AttendSansTimer(int masque, int rect, int x_rc, int y_rc, int larg_rc, int haut_rc);
static int BIX_VideEvenements (void);

static int BIX_ConvertitEvenement(XEvent evenement, int masque, int rect, int x_rc, int y_rc, int larg_rc, int haut_rc);
static int BIX_TraiteEvenement(Fenetre *fenetre, XEvent evenement, XEvent *vx_evenement);

/**************** Fonctions proprement dites ******************* */

/* ~~~~~~~~~~~~ Fonctions générales de traitement ~~~~~~~~~~~ */

/* ------------ Nouvelle mouture de evnt_multi -----------------\
 |    Renvoie un code BI_EVENEMENT_* [cf evenements.h]          |
 |            les valeurs de x, y, ... si non-NULL              |
 |                                                              |
 | masque : événements que veut l'utilisateur [cf evenements.h] |
 | rect   : événement Rectangle (comme evnt_multi)              |
 | x_rc,y_rc,larg_rc,haut_rc : coordonnées de ce rectangle      |
 | compteur : temps d'attente demandé                           |
 | x,y    : coordonnées de la souris                            |
 | bouton : bouton qui a été cliqué                             |
 | etat   : état des touches spéciales du clavier               |
 | touche : touche appuyée (octet 1 : ASCII, 2 : SCAN  )        |
 | n_clic : nombre de clics souris                              |
 \------------------------------------------------------------- */

int BI_AttendEvenement(int masque,
		       int rect, int x_rc, int y_rc, int larg_rc, int haut_rc,
		       unsigned long temps_attente,
		       int *x, int *y, int *bouton, int *etat, unsigned int *touche, int *n_clic)
{
  int final = BI_EVENEMENT_TRAITE;

  ENTREE_FONCTION( __FUNCTION__,
		   "masque=%d, rect=%d, x_rc=%d, y_rc=%d, larg_rc=%d, haut_rc=%d, temps_attente=%lu, x=%p, y=%p, bouton=%p, etat=%p, touche=%p, n_clic=%p",
		   masque, rect, x_rc, y_rc, larg_rc, haut_rc, temps_attente, x, y, bouton, etat, touche, n_clic );

  BI_n_clic = 0; /* Un clic normalement */

  /* On commence sans ancien événement */
  vx_evenement.type = 0;

  /* On appelle la routine la plus appropriée */
  if ( ( masque & BI_EVENEMENT_TIMER ) == FAUX ) { /* Pas de timer, donc beaucoup plus simple */
    final = BIX_AttendSansTimer( masque, rect, x_rc, y_rc, larg_rc, haut_rc );
  } else if ( 0 == temps_attente ) { /* TIMER mis à zéro -> on traite tous les événements X et on ressort */
    final = BIX_VideEvenements();
  } else {
    final = BIX_AttendAvecTimer( masque, rect, x_rc, y_rc, larg_rc, haut_rc, temps_attente );
  }

  /* On met à jour les informations demandées... */
  if ( x      != NULL ) *x      = BI_x_souris;
  if ( y      != NULL ) *y      = BI_y_souris;
  if ( bouton != NULL ) *bouton = BI_bouton;
  if ( n_clic != NULL ) *n_clic = BI_n_clic; /* Souris  */
  if ( touche != NULL ) *touche = 0/*BI_touche*/;
  if ( etat   != NULL ) *etat   = BI_etat;   /* Clavier */

  SORTIE_FONCTION( __FUNCTION__, "final=%d [normal]", final);
  return final;
}

/* ---------------- Attente d'un événement X, version avec TIMER ------\
 | Paramètres exactement comme ci-dessus                               |
 \-------------------------------------------------------------------- */

static int BIX_AttendAvecTimer(int masque, int rect, int x_rc, int y_rc, int larg_rc, int haut_rc, unsigned long tps_max)
{
  int final = BI_EVENEMENT_TRAITE;
  unsigned long temps;
  Window vx_fenetre = None;
  Fenetre *fenetre = NULL;
  XEvent evenement;

  /* Temps de départ */
  temps = clock();

  while ( BI_EVENEMENT_TRAITE == final ) { /* On traite en interne, donc rien à renvoyer... */
    Bool est_filtre;

    while ( XPending( BI_Display ) == 0 ) { /* Il n'y a pas d'autre événement */
      if ( ( clock() - temps ) > tps_max )
	return BI_EVENEMENT_TIMER; /* Événement timer */
    }

    /* Si on est là, c'est qu'un événement est arrivé avant que le temps ne soit écoulé */
    BI_n_clic = 0; /* Un clic normalement */

    XNextEvent( BI_Display, &evenement ); /* On récupère l'événement */
    est_filtre = XFilterEvent( &evenement, None );
    if ( True == est_filtre ) {
      (void) fprintf( stderr, "%s,%s l.%u : Événement X filtré, ignoré\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      continue;
    }

    BI_Fen_redessin = evenement.xany.window; /* Fenêtre concernée -> pour les redessin, ok... */
    if ( BI_Fen_redessin != vx_fenetre ) {
      vx_fenetre = BI_Fen_redessin;
      fenetre = BIX_QuelleFenetre( BI_Fen_redessin );
	/*      printf("Fenêtre concernée : %p\n", fenetre); */
    }

    final = BIX_TraiteEvenement( fenetre, evenement, &vx_evenement );
    if ( BI_EVENEMENT_FEN_FERMEE == final ) { /* Cf. attend_sstimer() pour explication */
      BI_Fen_redessin = None;
      vx_fenetre = None;
      final = BI_EVENEMENT_TRAITE;
    }

    /* On convertit un événement s'il n'a pas été traité */
    if ( BI_EVENEMENT_NON_TRAITE == final ) { /* Message non-traité */
#ifdef _DEVELOPPE_
      (void) printf("%s : Événement non-traité en interne\n", __FUNCTION__ );
#endif

      final = BIX_ConvertitEvenement( evenement, masque, rect, x_rc, y_rc, larg_rc, haut_rc );
    }
  }

  if ( BI_EVENEMENT_FIN_PRG != final ) { /* On demande de quitter : important... */
    if ( ( clock() - temps ) > tps_max )
      final |= BI_EVENEMENT_TIMER;
  }

  /* On a terminé */
  return final;
}

/* ---------------- Attente d'un événement X, version sans TIMER ------\
 | Paramètres exactement comme ci-dessus                               |
 \-------------------------------------------------------------------- */

static int BIX_AttendSansTimer(int masque,
			       int rect, int x_rc, int y_rc, int larg_rc, int haut_rc)
{
  int final = BI_EVENEMENT_TRAITE;
  Window vx_fenetre = None;
  Fenetre *fenetre = NULL;
  XEvent evenement;

  ENTREE_FONCTION( __FUNCTION__,
		   "masque=%d, rect=%d, x_rc=%d, y_rc=%d, larg_rc=%d, haut_rc=%d",
		   masque, rect, x_rc, y_rc, larg_rc, haut_rc );

  /* On commence sans clic mémorisé */
  BI_n_clic = 0;
  vx_evenement.type = 0;

  while ( BI_EVENEMENT_TRAITE == final ) { /* On traite en interne, donc rien à renvoyer... */
    XNextEvent(BI_Display, &evenement); /* On récupère l'évenement */

    BI_Fen_redessin = evenement.xany.window; /* Fenêtre concernée -> pour les redessins, ok... */
    if ( BI_Fen_redessin != vx_fenetre ) {
      vx_fenetre = BI_Fen_redessin;
      fenetre = BIX_QuelleFenetre( BI_Fen_redessin );
      /*      printf("Fenêtre concernée : %p\n", fenetre); */
    }

/*     printf( "%s : événement %d à traiter, ancien = %d\n", __FUNCTION__, evenement.type, vx_evenement.type ); */
    final = BIX_TraiteEvenement( fenetre, evenement, &vx_evenement );
    if ( BI_EVENEMENT_FIN_PRG == final ) /* On demande de quitter : important... */
      return BI_EVENEMENT_FIN_PRG;
    if ( BI_EVENEMENT_FEN_FERMEE == final ) {
      /* Fenêtre fermée, mais il peut arriver qu'il reste des événements dans la file d'attente
	 Ils peuvent être visibles si ils s'adressent à la même fenêtre X
	 => alors fenetre n'est pas remis à jour => plantages prévisibles
	 On force donc ici BI_Fen_redessin = None pour éviter les ennuis, donnera fenetre = NULL */
      BI_Fen_redessin = None;
      vx_fenetre = None;
      final = BI_EVENEMENT_TRAITE;
    }

    if ( BI_EVENEMENT_NON_TRAITE == final ) { /* Message non-traité */
#ifdef _DEVELOPPE_
      (void) printf("%s : Événement non-traité en interne\n", __FUNCTION__ );
#endif

      final = BIX_ConvertitEvenement( evenement, masque, rect, x_rc, y_rc, larg_rc, haut_rc );
    }
  }

  SORTIE_FONCTION( __FUNCTION__, "final=%d [normal]");
  return final;
}

/* -------- Traiter les événements en attente, puis revenir avec timer ! -------- */

static int BIX_VideEvenements(void)
{
  int final = BI_EVENEMENT_TRAITE;
  Window vx_fenetre = None;
  Fenetre *fenetre = NULL;
  XEvent evenement;

  /* Tant qu'il y a des événements X dans la queue */
  while ( XPending( BI_Display ) && ( BI_EVENEMENT_TRAITE == final ) ) {
    XNextEvent( BI_Display, &evenement ); /* On récupère l'évenement */

    BI_Fen_redessin = evenement.xany.window; /* Fenêtre concernée -> pour les redessin, ok... */
    if ( BI_Fen_redessin != vx_fenetre ) {
      vx_fenetre = BI_Fen_redessin;
      fenetre = BIX_QuelleFenetre( BI_Fen_redessin );
      /*      printf("Fenêtre concernée : %p\n", fenetre); */
    }

    /* On traite l'événement ainsi récupéré */
    final = BIX_TraiteEvenement( fenetre, evenement, &vx_evenement );
    if ( BI_EVENEMENT_FEN_FERMEE == final ) {
      /* Cf. attend_sstimer() pour explication */
      BI_Fen_redessin = None;
      vx_fenetre = None;
      final = BI_EVENEMENT_TRAITE;
    }
  }

  /* Sortie parce que plus d'événement -> on retourne avec le Timer */
  if ( BI_EVENEMENT_TRAITE == final )
    final = BI_EVENEMENT_TIMER;

  return final;
}

/* -------- Analyse l'événement X pour savoir si un événement simple peut lui être associé -------- */

static int BIX_ConvertitEvenement(XEvent evenement, int masque, int rect, int x_rc, int y_rc, int larg_rc, int haut_rc)
{
  if ( !( masque & BI_EVENEMENT_CLAVIER ) && ( KeyPress == evenement.type ) ) { /* Clavier, non demandé : ignoré */
    return BI_EVENEMENT_TRAITE;
  }

  if ( ( masque & BI_EVENEMENT_RECT_1 ) && ( MotionNotify == evenement.type ) ) { /* Rectangle souris compte */
    boolean dedans = FAUX;

    if ( ( evenement.xmotion.x >= x_rc ) && ( evenement.xmotion.x <= x_rc + larg_rc ) &&
	 ( evenement.xmotion.y >= y_rc ) && ( evenement.xmotion.y <= y_rc + haut_rc ) ) {
      dedans = VRAI;
    }
    if ( ( ( VRAI == dedans ) && ( 1 == rect ) ) ||
	 ( ( FAUX == dedans ) && ( 0 == rect ) ) ) {
      return BI_EVENEMENT_RECT_1;
    }  
  }

  /* Par défaut : événement inconnu --> non-traité ! */
  return BI_EVENEMENT_NON_TRAITE;
}

/* ~~~~~~~~~~~~~~~~ Traitement des événements X en fonction du type ~~~~~~~~~~~~ *
 |  Renvoie l'état de sortie correspondant au traitement de l'événement          |
 |                                                                               |
 | fenetre      = la fenêtre pour laquelle l'événement X a eu lieu               |
 | evenement    = l'événement X ayant eu lieu                                    |
 | vx_evenement = le précédent événement X ayant eu lieu (pour double clic)      |
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

static int BIX_TraiteEvenement(Fenetre *fenetre, XEvent evenement, XEvent *vx_evenement)
{
  int final = BI_EVENEMENT_TRAITE;

  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p, &evenement=%p, vx_evenement=%p", fenetre, &evenement, vx_evenement);

  if ( evenement.type == ButtonRelease ) { /* On relâche le bouton */
    if ( EVNT_relache_bouton( fenetre, &(evenement.xbutton) ) == BI_EVENEMENT_TRAITE ) {
      vx_evenement->type = 0;
    } else if ( vx_evenement->type == ButtonPress ) { /* Le clic a bien été reçu... */
      /* On vient de relacher après avoir cliqué : on attend donc un éventuel clic ultérieur */
      if ( XPending( BI_Display ) > 0 ) {
	XEvent evenement_suivant;

	/* Il y a d'autres événements en attente : on regarde ce dont il s'agit */
	XPeekEvent( BI_Display, &evenement_suivant );
	if ( evenement_suivant.xany.type != ButtonPress ) {
	  /* On fait autre chose qu'un nouveau clic : on gère donc les clics déjà enregistrés */
	  final = EVNT_TraiterClic( fenetre, &(vx_evenement->xbutton), BI_n_clic );

	  /* Et on remet le compteur de clics à 0 */
	  BI_n_clic = 0;
	  vx_evenement->type = 0;	  
	} else {
	  /* On mémorisera, ou non ce clic pour la suite : rien de spécial à faire donc... */
	}

	/* L'événement regardé n'est pas ôté de la queue : il sera traité normalement dans la boucle au tour suivant... */
      } else {
	int *descripteurs_X = NULL, nombre_descripteurs, i, retour, descripteur_maximal = 0;
	fd_set descripteurs;
	struct timeval temps_attente;

	/* On récupère la liste des connexions X */
	if ( 0 == XInternalConnectionNumbers( BI_Display, &descripteurs_X, &nombre_descripteurs ) ) {
	  (void) fprintf( stderr, "%s, %s l.%u : Erreur en obtenant la liste des descripteurs de connexion X\n",
			  __FILE__, __FUNCTION__, __LINE__ );
	  return BI_EVENEMENT_TRAITE; /* Événement traité */
	}

	/* On crée ce qu'il faut pour select */
	FD_ZERO( &descripteurs );
	for ( i = 0 ; i < nombre_descripteurs ; i++ ) {
	  FD_SET( descripteurs_X[ i ], &descripteurs );
	  descripteur_maximal = ( descripteur_maximal > descripteurs_X[ i ] ) ? ( descripteur_maximal ) : ( descripteurs_X[ i ] );
	}
	/* Assez bizarrement, XInternalConnectionNumbers ne renvoie rien, et en tout cas pas le ConnectionNumber ! */
	FD_SET( ConnectionNumber( BI_Display ), &descripteurs );
	if ( ConnectionNumber( BI_Display ) > descripteur_maximal ) {
	  descripteur_maximal = ConnectionNumber( BI_Display );
	}

	temps_attente.tv_sec  = 0;
	temps_attente.tv_usec = 1000 * TEMPS_ATTENTE_DOUBLE_CLIC;  /* 1 ms = 1000 µs */

	/* On vide le tampon des événements X, pour être sûr que ceux en attente soient détectés par select() */
	XFlush( BI_Display );

	/* On appelle select() */
	retour = select( descripteur_maximal + 1, &descripteurs, NULL, NULL, &temps_attente );
	if ( retour < 0 ) {
	  (void) fprintf( stderr, "%s, %s l.%u : Erreur dans select : %d\n",
			  __FILE__, __FUNCTION__, __LINE__, errno );
	  final = EVNT_TraiterClic( fenetre, &(vx_evenement->xbutton), BI_n_clic );
	} else if ( 0 == retour ) {
	  /* Le délai s'est écoulé sans événement X : on traite le clic */
	  final = EVNT_TraiterClic( fenetre, &(vx_evenement->xbutton), BI_n_clic );

	  /* Et on remet le compteur de clics à 0 */
	  BI_n_clic = 0;
	  vx_evenement->type = 0;
	} else {
	  /* Il y a des événements X qui ont surgi : on s'occupera du clic plus tard, en fonction de l'événement suivant */
	  /* Le clic a déjà été mémorisé avec ButtonPress ==> il n'y a rien à faire, l'événement est traité */
	  final = BI_EVENEMENT_TRAITE;
	}

	/* On libère la liste des descripteurs X */
	XFree( descripteurs_X ); descripteurs_X = NULL;
      }
    } else {
      /* Probablement déjà à 0, mais autant être sûr... */
/*       vx_evenement->type = 0; */
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal], ButtonRelease", final );
/*     printf("--> Traitement terminé, on repasse dans la boucle d'attente\n"); */
    return final; /* Événement traité */
  }

  if ( evenement.type == ButtonPress ) { /* On enfonce le bouton */
    boolean memorise;
    int retour;

    memorise = EVNT_MemoriserClic( fenetre, &(evenement.xbutton), vx_evenement, &BI_n_clic );
    if ( FAUX == memorise ) {
/*       printf("** Clic non mémorisé, traité directement\n" ); */
      /* Clic non mémorisé : on le traite directement, comme un clic simple */
      retour = EVNT_TraiterClic( fenetre, &(evenement.xbutton), 1 );
    } else {
/*       printf("** Clic mémorisé, sera traité dans le ButtonRelease\n" ); */
      /* Clic mémorisé : on peut passer à l'événement suivant */
      retour = BI_EVENEMENT_TRAITE;
    }
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, ButtonPress]", retour );
    return retour;
  }

  /* Si on est là, c'est que l'événement n'est pas un clic sur un bouton de la souris */
  if ( ( vx_evenement->type == ButtonRelease ) ||
       ( vx_evenement->type == ButtonPress   ) ) { /* Il y avait un clic enregistré */
    final = EVNT_TraiterClic( fenetre, &(vx_evenement->xbutton), BI_n_clic ); /* Clic */

    BI_n_clic = 0; /* Plus de clic */
    BI_Fen_redessin = evenement.xany.window; /* Car peut changer au cours du traitement... */
    vx_evenement->type = 0; /* Aucun événement */
/*     printf( "{Événement autre [%d] traité, ancien remis à 0}\n", evenement.type ); */
  }

  switch ( evenement.type )
  {
  case KeyPress: /* On a appuyé sur une touche */
    final = EVNT_clavier( fenetre, &(evenement.xkey), VRAI ); /* VRAI si a été traité ; on gère aussi les menus */
    break;

  case KeyRelease: /* On ignore */
    /*    EVNT_relache( &(evenement.xkey) ); */
    break;

  case ConfigureNotify: /* Changement de taille de la fenêtre - XConfigureEvent */
    DANS_FONCTION( "ConfigureNotify" );
    if ( fenetre != NULL ) { /* Il y a une fenêtre classique associée */
      int delta_x, delta_y, d_x, d_y;

      delta_x = evenement.xconfigure.width - fenetre->larg;
      delta_y = evenement.xconfigure.height - fenetre->haut;
      d_x = evenement.xconfigure.x - fenetre->x;
      d_y = evenement.xconfigure.y - fenetre->y;
      if ( ( 0 == delta_x ) && ( 0 == delta_y ) ) {
	if ( ( 0 == d_x ) && ( 0 == d_y ) ) {
	  /*	  printf("Configure Notify, sans changement de taille ni position - (%d, %d)\n",
		  evenement.xconfigure.x, evenement.xconfigure.y); */
	} else {
	  /*	  printf("Déplacement de la fenêtre\n"); */
	  fenetre->x += d_x;
	  fenetre->y += d_y;
	}
	SORTIE_FONCTION( __FUNCTION__, "%d [normal, ConfigureNotify, déplacement]", VRAI );
	return BI_EVENEMENT_TRAITE;
      }

      /* On effectue le changement de taille */	  
      change_taille_fenetre( fenetre, evenement.xconfigure.width, evenement.xconfigure.height );
      fenetre->x += d_x;
      fenetre->y += d_y;

      /* Si fenêtre demande un message spécial en cas de changement de taille, on le fait */
      if ( fenetre->msg_masque.taille == VRAI ) {
	Message message;

	message.type = WM_SIZED;
	message.fenetre.application = 0;
	message.fenetre.en_plus = 0;
	message.fenetre.handle = 0; /*quel_handle(fenetre);*/
	message.fenetre.x = fenetre->x;
	message.fenetre.y = fenetre->y;
	message.fenetre.larg = evenement.xconfigure.width;
	message.fenetre.haut = evenement.xconfigure.height;
	if ( (fenetre->message) != (boolean (*)(Fenetre *, Message *)) NULL ) {
	  DANS_FONCTION( "On va appeler fenetre->message() [ConfigureNotify]" );
	  final = (fenetre->message)(fenetre, &message);
#ifdef _DEVELOPPE_
	} else {
	  fprintf(stderr, "ATTENTION, pas de fonction message associée\n");
	  fprintf(stderr, "  mais message de changement de taille demandé !\n");
	  fprintf(stderr, "Fenêtre : Nom = %s\n", fenetre->titre);
#endif
	}
      }
    }
    break;

  case Expose: /* On affiche la fenêtre = redessin  - XExposeEvent */
    DANS_FONCTION( "Expose" );
    final = EVNT_redessin( fenetre, (XExposeEvent *) &evenement );
    break;

  case MotionNotify: /* Déplacement du curseur */
    DANS_FONCTION( "MotionNotify" );
    final = EVNT_pointeur( fenetre, &(evenement.xmotion) );
    break;

  case EnterNotify:  /* Entrée dans une fenêtre */
    DANS_FONCTION( "EnterNotify" );
    final = EVNT_entree( fenetre, &(evenement.xcrossing) );
    break;

  case LeaveNotify:  /* Sortie d'une fenêtre */
    DANS_FONCTION( "LeaveNotify" );
    final = EVNT_sortie( fenetre, &(evenement.xcrossing) );
    break;

  case ResizeRequest: /* Changement de taille */
    break;

  case ColormapNotify: /* Changement de palette
			   -> soit fenêtre image, soit fenêtre Open_GL */
    break;
    
  case DestroyNotify:  /* Fenêtre détruite      */
  case UnmapNotify:    /* Fenêtre plus affichée */
  case ReparentNotify: /* ? */
  case MapNotify:      /* Affichage effectif    */
    break;

  case FocusIn:			/* La fenêtre obtient le focus */
    final = EVNT_Activation( fenetre, &(evenement.xfocus) );
    break;

  case FocusOut:		/* La fenêtre perd le focus */
    break;

  case ClientMessage:
    DANS_FONCTION( "ClientMessage" );
    if ( (Atom) (evenement.xclient.data.l[ 0 ]) == BI_ProtocoleFermerFenetre ) {
      if ( NULL == fenetre ) {
	/* Se produit quand on clique sur la barre de menu ! */
	OPTION_A_FAIRE( "Gérer le clic sur le bouton de fermeture de la barre de menu" );
	return BI_EVENEMENT_TRAITE;
      }
      if ( fenetre->attributs & CLOSER ) {
	if ( VRAI == fenetre->msg_masque.fermeture ) {
	  Message message;

	  message.type = WM_CLOSED;
	  message.fenetre.application = 0;
	  message.fenetre.en_plus = 0;
	  message.fenetre.handle = 0; /*quel_handle(fenetre);*/
	  message.fenetre.x = 0; message.fenetre.y = 0;
	  message.fenetre.larg = 0; message.fenetre.haut = 0; /* Dans le cas où != 0 <=> fermeture forcée... */

	  if ( fenetre->message != ( int (*)(Fenetre *, Message *) ) NULL ) {
	    final = (fenetre->message)(fenetre, &message);
	  } else {
	    printf("Traite_evenement : À FAIRE, message / fenêtre sans fonction, fermeture\n");
	  }
	} else {
	  FEN_Detruire( fenetre );
	  final = BI_EVENEMENT_FEN_FERMEE;
	}
      } else {
#ifdef _DEVELOPPE_
      printf("Traite_evenement : cette fenêtre ne peut être fermée [attributs : %d] !\n", fenetre->attributs);
#endif
      }
    } else {
#ifdef _DEVELOPPE_
      printf("Traite_evenement : message client de type inconnu [%ld]\n", evenement.xclient.data.l[ 0 ]);
#endif
    }
    break;
	
#ifdef _DEVELOPPE_
  case NoExpose:
/*     (void) fprintf(stderr, "%s, %s l.%u : Événement NoExpose. Signification ?\n", */
/* 		   __FILE__, __FUNCTION__, __LINE__); */
    break;

  default:
      printf("Traite_evenement : message inconnu, type %d\n", evenement.xany.type);
#endif
  }

#ifdef _DEVELOPPE_
  if ( BI_EVENEMENT_NON_TRAITE == final ) {
    (void) fprintf( stderr, "%s,%s l.%u : Événement non-traité [type %d]\n",
		    __FILE__, __FUNCTION__, __LINE__, evenement.xany.type );
  }
  if ( VRAI == final ) {
    (void) fprintf( stderr, "%s,%s l.%u : final=VRAI, oubli possible d'un remplacement VRAI -> BI_EVENEMENT_TRAITE [type %d]\n",
		    __FILE__, __FUNCTION__, __LINE__, evenement.xany.type );
  }
#endif

  SORTIE_FONCTION( __FUNCTION__, "%d [normal, autre]", final );
  return final; /* Comment a pu être traité l'événement */
}

/* ~~~~~~~~~~~~~~~~ Gestion du clavier ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* Relâchement d'une touche -> touches d'état pour control et autres... */

int EVNT_relache(XKeyEvent *clavier)
{
  if ( NULL == clavier ) {
    return BI_EVENEMENT_ERREUR;
  }
  /* On récupère le code de la touche utilisée */

  /*  XLookupString(clavier, tampon, 128, &touche, (XComposeStatus *) NULL); */

  return VRAI;
}

/* -------------- Réception et traitement primaire de l'événement ----- */

/* ~~~~~~~~~~~~~~~~ Gestion des événements ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

int EVNT_relache_bouton(Fenetre *fenetre, XButtonEvent *souris)
{
  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p, souris=%p", fenetre, souris );

  if ( NULL == fenetre ) {
    /* Pas de fenêtre associée : barre de menus à priori ==> ignorer ! */
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, fenetre = NULL]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }

  /* Tous les gadgets de fenêtre : inutile d'enregistrer */
#ifdef XGEM_AVEC_CLOSER_
  if ( souris->window == fenetre->closer )
    return BI_EVENEMENT_TRAITE;
#endif
  if ( souris->window == fenetre->barre_d_infos ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, barre d'infos]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }
  if ( souris->window == fenetre->fleche_haute ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, flèche haute]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }
  if ( souris->window == fenetre->fond_asc_vertical) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, fond ascenseur vertical]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }
  if ( souris->window == fenetre->bouton_asc_vertical) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, bouton ascenseur vertical]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }
  if ( souris->window == fenetre->fleche_basse ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, flèche basse]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }
  if ( souris->window == fenetre->fleche_gauche ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, flèche gauche]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }
  if ( souris->window == fenetre->fond_asc_horizontal) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, fond ascenseur horizontal]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }
  if ( souris->window == fenetre->bouton_asc_horizontal) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, bouton ascenseur horizontal]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }
  if ( souris->window == fenetre->fleche_droite ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, flèche droite]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }

  /* Fenêtre utilisateur */
  SORTIE_FONCTION( __FUNCTION__, "%d [normal, utilisateur]", BI_EVENEMENT_NON_TRAITE );
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Traitement d'un clic de la souris -------- */

/* -------- Mémorisation d'un clic : si un second clic avec le même bouton, espacé de moins du délai indiqué [en MICROSECONDES] survient, on compte un clic de plus
    Renvoie VRAI si l'on a mémorisé le clic
            FAUX sinon
   -------- */

boolean EVNT_MemoriserClic(Fenetre *fenetre, XButtonEvent *souris, XEvent *ancien, int *n_clics)
{
  int retour  __attribute__((__unused__)) = BI_EVENEMENT_TRAITE ;

  /* On élimine les cas où la mémorisation du clic est inutile : pas de double-clic (actuellement)  */
  /* a. Molette de la souris */
  if ( ( souris->button == Button4 ) || ( souris->button == Button5 ) ) {
    return FAUX;
  }

  /* b. Fenêtre à usage interne (menu) */
  if ( NULL == fenetre ) {
    return FAUX;
  }

  /* c. Gadgets de fenêtre */
#ifdef XGEM_AVEC_CLOSER_
  if ( souris->window == fenetre->closer ) { /* Bouton de fermeture */
    return FAUX;
  }
#endif
  if ( souris->window == fenetre->barre_d_infos ) {
    return FAUX;
  }
  if ( souris->window == fenetre->fleche_haute ) {
    return FAUX;
  }
  if ( souris->window == fenetre->fond_asc_vertical ) {
    return FAUX;
  }
  if ( souris->window == fenetre->bouton_asc_vertical ) {
    return FAUX;
  }
  if ( souris->window == fenetre->fleche_basse ) {
    return FAUX;
  }
  if ( souris->window == fenetre->fleche_gauche ) {
    return FAUX;
  }
  if ( souris->window == fenetre->fond_asc_horizontal ) {
    return FAUX;
  }
  if ( souris->window == fenetre->bouton_asc_horizontal ) {
    return FAUX;
  }
  if ( souris->window == fenetre->fleche_droite ) {
    return FAUX;
  }

  /* d. Fenêtre d'onglets */
  if ( BI_FENETRE_ONGLETS == fenetre->type ) {
    return FAUX;
  }

  if ( ButtonPress == ancien->type ) { /* Il y avait un clic enregistré */
    if ( ( souris->time - ancien->xbutton.time ) > TEMPS_ATTENTE_DOUBLE_CLIC ) { /* clics distincts */
      /* On traite la souris des anciens clics */
      retour = EVNT_TraiterClic( fenetre, &(ancien->xbutton), *n_clics );
      *n_clics = 0;
    } else if ( souris->button != ancien->xbutton.button ) {  /* On change de boutons : clics distincts */
      /* On traite la souris des anciens clics */
      retour = EVNT_TraiterClic( fenetre, &(ancien->xbutton), *n_clics );
      *n_clics = 0;
    } else {
      /* On ajoute ce clic à la liste  */
      retour = BI_EVENEMENT_TRAITE;
    }

    /* On enregistre le nouvel événement souris */
    ancien->xbutton = *souris;
    (*n_clics)++; /* Un clic en plus */

    return VRAI;
  }

  /* Aucun événement enregistré -> on se contente de compter 1 clic */
  ancien->type = ButtonPress;
  ancien->xbutton = *souris; /* On enregistre le nouvel événement souris */
  *n_clics = 1;

  return VRAI;
}

int EVNT_TraiterClic(Fenetre *fenetre, XButtonEvent *souris, int n_clics)
{
  int retour = BI_EVENEMENT_ERREUR;

  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p, n_clics=%d", souris, n_clics);

  /* On vérifie les modificateurs, au cas où */
/*  printf("État de la souris : %d (Bouton 1 + alt : %d)\n", souris->state, Button1Mask|Mod1Mask); */
  BIBX_verifie_modificateurs( souris->state );

  BI_bouton = 0; /* Aucun bouton sinon */
  /* Convention GEM : 1 bit par bouton, donc bit 3 mis <=> 3e bouton mis */
  if ( souris->button == Button1 )
    BI_bouton = 1;
  else if ( souris->button == Button2 ) {
    if ( VRAI == BI_2_egale_3 ) /* On échange les boutons 2 & 3 */
      BI_bouton = 4;
    else
      BI_bouton = 2;
  } else if ( souris->button == Button3 ) { /* Attention, un vrai 3e bouton n'existe que sous X !*/
    if ( VRAI == BI_2_egale_3 ) { /* On échange les boutons 2 & 3 */
#ifdef _DEVELOPPE_
      printf("Bouton choisi = troisième bouton X, converti en deuxième bouton X\n");
#endif
      BI_bouton = 2;
    } else {
      BI_bouton = 4;
    }
  } else if ( Button4 == souris->button ) {
    BI_bouton = (1 << 4);
  } else if ( Button5 == souris->button ) {
    BI_bouton = (1 << 5);
  } else {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Bouton inconnu, non géré : %d\n",
		   __FILE__, __FUNCTION__, __LINE__, souris->button);
#endif
    BI_bouton = (1 << 6);
  }

  /* On récupère la fenêtre */
  if ( NULL == fenetre ) { /* Fenêtre à usage interne */
    if ( BI_Menu != NULL ) { /* Il y a bien un menu */
      unsigned int i;

      if ( souris->window == BI_Menu->racine ) {
	SORTIE_FONCTION( __FUNCTION__, "%d [normal, clic sur une racine de menu]", BI_EVENEMENT_TRAITE );
	return BI_EVENEMENT_TRAITE; /* Rien à faire si clic sur la racine */
      }

      /* On cherche sur quel titre de menu on a pu cliquer */
      for ( i = 0 ; i < BI_Menu->nbr_titres ; i++ ) {
	if ( BI_Menu->titres[ i ] == souris->window ) { /* Clic sur le titre i */
	  if ( OB_STATE( BI_Menu->adresse, BI_Menu->prem_titre + i ) & DISABLED ) { /* Inactif -> non */
	    SORTIE_FONCTION( __FUNCTION__, "%d [normal, clic sur un objet inactif]", BI_EVENEMENT_TRAITE );
	    return BI_EVENEMENT_TRAITE;
	  }

	  retour = CLIC_menu( BI_Menu, i );
	  SORTIE_FONCTION( __FUNCTION__, "%d [normal, clic sur un menu ]", retour );
	  return retour;
	}
      }
    }

#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : clic sur une fenêtre X non associée à une structure Fenetre et hors-menu !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
#endif
    SORTIE_FONCTION( __FUNCTION__, "%d [ANORMAL]", BI_EVENEMENT_ERREUR );
    return BI_EVENEMENT_ERREUR; /* Probablement une fenêtre fermée entre-temps */
  }

  /* --- À partir d'ici, on est sur une vraie fenêtre (non-NULL) --- */

  /* Test des boutons 3 et 4 = mollette, raccourci pour les ascenseurs verticaux */
  if ( ( souris->button == Button4 ) || ( souris->button == Button5 ) ) {
#ifdef _DEVELOPPE_
/*    printf("Mollette vers le %s\n", (souris->button == Button4) ? "Haut" : "Bas"); */
#endif
    if ( fenetre->fleche_haute != None )
      souris->window = (souris->button == Button4) ? fenetre->fleche_haute : fenetre->fleche_basse;
    else if ( fenetre->fleche_droite != None )
      souris->window = (souris->button == Button4) ? fenetre->fleche_gauche : fenetre->fleche_droite;
  }

#ifdef XGEM_AVEC_CLOSER_
  if ( souris->window == fenetre->closer ) { /* Bouton de fermeture */
    Message message;
    int retour;

    if ( fenetre->msg_masque.fermeture ) { /* On veut gérer les messages de fermeture */
      message.type = WM_CLOSED;
      message.fenetre.application = 0;
      message.fenetre.en_plus = 0;
      message.fenetre.handle = 0; /*quel_handle(fenetre);*/
      message.fenetre.x = 0; message.fenetre.y = 0;
      message.fenetre.larg = 0; message.fenetre.haut = 0; /* Dans le cas où != 0 <=> fermeture forcée... */

      if ( fenetre->message != ( boolean (*)(struct fenetre *, Message *) ) NULL ) {
	retour = (fenetre->message)( fenetre, &message );
      } else {
	OPTION_A_FAIRE( "Remplir le tampon pour le message GEM de clôture" );
	retour = BI_EVENEMENT_MESSAGE; /* On aura un message GEM à traiter ! */
      }
    } else {
      ferme_fenetre( fenetre );
      retour = BI_EVENEMENT_FEN_FERMEE;
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, closer]", retour );
    return retour;
  }
#endif

  if ( souris->window == fenetre->barre_d_infos ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, barre d'infos]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }

  if ( souris->window == fenetre->fleche_haute ) {
    if ( fenetre->ascenseur != ( void (*)(struct fenetre *, int , int , int) ) NULL ) {
      (fenetre->ascenseur)(fenetre, WM_ARROWED, WA_UPLINE, 0);
      retour = BI_EVENEMENT_TRAITE;
    } else {
      Message message __attribute__((__unused__));

      message.type = WM_ARROWED;
      message.fenetre.application = 0;
      message.fenetre.en_plus = 0;
      message.fenetre.handle = 0; /*quel_handle(fenetre);*/
      message.fenetre.x    = WA_UPLINE; message.fenetre.y    = 0;
      message.fenetre.larg = 0        ; message.fenetre.haut = 0;

      OPTION_A_FAIRE( "Enregistrer le message GEM dans le tampon..." );
      retour = BI_EVENEMENT_MESSAGE;
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, flèche haute]", retour );
    return retour;
  }

  if ( souris->window == fenetre->fond_asc_vertical ) {
    Window w;
    int x, y;
    unsigned int larg, haut, i;

    XGetGeometry( BI_Display, fenetre->bouton_asc_vertical, &w, &x, &y, &larg, &haut, &i, &i );

    if ( fenetre->ascenseur != ( void (*)(struct fenetre *, int , int , int) ) NULL ) {
      if ( souris->y < y ) /* Clic avant le poussoir */
	(fenetre->ascenseur)( fenetre, WM_ARROWED, WA_UPPAGE, 0 );
      else
	(fenetre->ascenseur)( fenetre, WM_ARROWED, WA_DNPAGE, 0 );
      retour = BI_EVENEMENT_TRAITE;
    } else {
      Message message __attribute__((__unused__));

      message.type = WM_ARROWED;
      message.fenetre.application = 0;
      message.fenetre.en_plus = 0;
      message.fenetre.handle = 0; /*quel_handle(fenetre);*/
      message.fenetre.y = 0;
      message.fenetre.larg = 0; message.fenetre.haut = 0;
      if (souris->y < y)
	message.fenetre.x = WA_UPPAGE; /* On prépare un message */
      else
	message.fenetre.x = WA_DNPAGE; /* On prépare un message */

      OPTION_A_FAIRE( "Enregistrer le message GEM dans le tampon..." );
      retour = BI_EVENEMENT_MESSAGE;
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, fond ascenseur vertical]", retour );
    return retour;
  }

  if ( souris->window == fenetre->fleche_basse ) {
    if ( fenetre->ascenseur != ( void (*)(struct fenetre *, int , int , int) ) NULL ) {
      (fenetre->ascenseur)( fenetre, WM_ARROWED, WA_DNLINE, 0 );
      retour = BI_EVENEMENT_TRAITE;
    } else {
      Message message __attribute__((__unused__));

      message.type = WM_ARROWED;
      message.fenetre.application = 0;
      message.fenetre.en_plus = 0;
      message.fenetre.handle = 0; /*quel_handle(fenetre);*/
      message.fenetre.x    = WA_DNLINE; message.fenetre.y    = 0;
      message.fenetre.larg = 0        ; message.fenetre.haut = 0;

      OPTION_A_FAIRE( "Enregistrer le message GEM dans le tampon..." );
      retour = BI_EVENEMENT_MESSAGE;
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, bouton ascenseur vertical]", retour );
    return retour;
  }

  if ( souris->window == fenetre->fleche_gauche ) {
    if ( fenetre->ascenseur != ( void (*)(struct fenetre *, int , int , int) ) NULL ) {
      (fenetre->ascenseur)(fenetre, WM_ARROWED, WA_LFLINE, 0);
      retour = BI_EVENEMENT_TRAITE;
    } else {
      Message message __attribute__((__unused__));

      message.type = WM_ARROWED;
      message.fenetre.application = 0;
      message.fenetre.en_plus = 0;
      message.fenetre.handle = 0; /*quel_handle(fenetre);*/
      message.fenetre.x = WA_LFLINE; message.fenetre.y = 0;
      message.fenetre.larg = 0; message.fenetre.haut = 0;

      OPTION_A_FAIRE( "Enregistrer le message GEM dans le tampon..." );
      retour = BI_EVENEMENT_MESSAGE;
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, flèche basse]", retour );
    return retour;
  }

  if ( souris->window == fenetre->bouton_asc_vertical ) {
    int nv_pos;

    nv_pos = X_ascenseur( fenetre->fond_asc_vertical, fenetre->bouton_asc_vertical, VERTICAL );
    if ( fenetre->ascenseur != ( void (*)(struct fenetre *, int , int , int) ) NULL ) {
      (fenetre->ascenseur)( fenetre, WM_VSLID, 0, nv_pos );
    } else {
      Message message __attribute__((__unused__));

      message.type = WM_VSLID;
      message.fenetre.application = 0;
      message.fenetre.en_plus = 0;
      message.fenetre.handle = 0; /*quel_handle(fenetre);*/
      message.fenetre.x = nv_pos; message.fenetre.y = 0;
      message.fenetre.larg = 0; message.fenetre.haut = 0;

      OPTION_A_FAIRE( "Enregistrer le message GEM dans le tampon..." );
      retour = BI_EVENEMENT_MESSAGE;
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, flèche gauche]", retour );
    return retour;
  }

  if ( souris->window == fenetre->fond_asc_horizontal ) {
    Window w;
    int x, y;
    unsigned int larg, haut, i;

    XGetGeometry( BI_Display, fenetre->bouton_asc_horizontal, &w, &x, &y, &larg, &haut, &i, &i );

    if ( fenetre->ascenseur != ( void (*)(struct fenetre *, int , int , int) ) NULL ) {
      if ( souris->x > x ) /* Clic avant le poussoir */
	(fenetre->ascenseur)( fenetre, WM_ARROWED, WA_RTPAGE, 0 );
      else
	(fenetre->ascenseur)( fenetre, WM_ARROWED, WA_LFPAGE, 0 );
      retour = BI_EVENEMENT_TRAITE;
    } else {
      Message message __attribute__((__unused__));

      message.type = WM_ARROWED;
      message.fenetre.application = 0;
      message.fenetre.en_plus = 0;
      message.fenetre.handle = 0; /*quel_handle(fenetre);*/
      message.fenetre.y = 0;
      message.fenetre.larg = 0; message.fenetre.haut = 0;
      if ( souris->x < x )
	message.fenetre.x = WA_LFPAGE; /* On prépare un message */
      else
	message.fenetre.x = WA_RTPAGE; /* On prépare un message */

      OPTION_A_FAIRE( "Enregistrer le message GEM dans le tampon..." );
      retour = BI_EVENEMENT_MESSAGE;
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, fond ascenseur horizontal]", retour );
    return retour;
  }

  if ( souris->window == fenetre->bouton_asc_horizontal ) {
    int nv_pos;

    nv_pos = X_ascenseur(fenetre->fond_asc_horizontal, fenetre->bouton_asc_horizontal, HORIZONTAL);
    if ( fenetre->ascenseur != ( void (*)(struct fenetre *, int , int , int) ) NULL ) {
      (fenetre->ascenseur)( fenetre, WM_HSLID, 0, nv_pos );
    } else {
      Message message __attribute__((__unused__));

      message.type = WM_HSLID;
      message.fenetre.application = 0;
      message.fenetre.en_plus = 0;
      message.fenetre.handle = 0; /*quel_handle(fenetre);*/
      message.fenetre.x = nv_pos; message.fenetre.y = 0;
      message.fenetre.larg = 0; message.fenetre.haut = 0;

      OPTION_A_FAIRE( "Enregistrer le message GEM dans le tampon..." );
      retour = BI_EVENEMENT_MESSAGE;
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, bouton ascenseur horizontal]", retour );
    return retour;
  }

  if ( souris->window == fenetre->fleche_droite ) {
    if ( fenetre->ascenseur != ( void (*)(struct fenetre *, int , int , int) ) NULL ) {
      (fenetre->ascenseur)(fenetre, WM_ARROWED, WA_RTLINE, 0);
      retour = BI_EVENEMENT_TRAITE;
    } else {
      Message message __attribute__((__unused__));

      message.type = WM_HSLID;
      message.fenetre.application = 0;
      message.fenetre.en_plus = 0;
      message.fenetre.handle = 0; /*quel_handle(fenetre);*/
      message.fenetre.x = WA_RTLINE; message.fenetre.y = 0;
      message.fenetre.larg = 0; message.fenetre.haut = 0;

      OPTION_A_FAIRE( "Enregistrer le message GEM dans le tampon..." );
      retour = BI_EVENEMENT_MESSAGE;
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, flèche droite]", retour );
    return retour;
  }

  /* --- À partir de là : on est réellement dans la partie utile de la fenêtre --- */
  if ( BI_FENETRE_ONGLETS == fenetre->type ) {
    int numero_onglet;
    boolean est_actif = FAUX;

    numero_onglet = BIX_QuelOnglet( fenetre, souris->window, &est_actif );
    if ( numero_onglet >= 0 ) {
      if ( VRAI == est_actif ) return BI_EVENEMENT_TRAITE; /* Clic sur onglet actif : rien à faire... */

      /* On active l'onglet correspondant */
      BI_Onglets_ChangerActif( fenetre, numero_onglet );

      /* L'événement est donc traité ! */
      SORTIE_FONCTION( __FUNCTION__, "%d [normal, clic sur un onglet]", BI_EVENEMENT_TRAITE );
      return BI_EVENEMENT_TRAITE;
    }
  }
  if ( BI_FENETRE_ARBRE == fenetre->type ) {
    unsigned int i;

    retour = CLIC_arbre(fenetre->arbre, souris, &i);
    SORTIE_FONCTION( __FUNCTION__, "%d [normal, BI_FENETRE_ARBRE]", retour);
    return retour;
  }
  if ( BI_FENETRE_SIMPLE_ARBRE == fenetre->type ) {
    unsigned int i;

    if ( fenetre->handle == souris->window ) {
      /* Clic sur la racine : sans intérêt ! */
      SORTIE_FONCTION( __FUNCTION__, "%d [normal, clic sur la racine de l'arbre]", BI_EVENEMENT_TRAITE );
      return BI_EVENEMENT_TRAITE;
    }

    printf( "%s -- Logiquement, a-t-on une chance d'être là ?\n", __FUNCTION__ );

    if ( souris->window == fenetre->ss_fenetres[ 1 ]->handle ) {
      OPTION_A_FAIRE( "Clic sur la partie utilisateur" );
    }

    if (souris->window != fenetre->handle) { /* Clic sur une autre fenêtre que la racine */
      retour = CLIC_arbre(fenetre->arbre, souris, &i); /* Clic sur l'arbre donc */
      SORTIE_FONCTION( __FUNCTION__, "%d [normal, MIXTE_FENETRE sur arbre]", retour);
      return retour;
    }
  }
  if ( BI_FENETRE_GL_ARBRE_SIMPLE == fenetre->type ) {
    unsigned int i;

    if ( ( souris->window != fenetre->handle )
#ifdef AVEC_OPEN_GL_
/* 	 && (souris->window != fenetre->open_gl->handle) */
#endif
	 ) {
      int retour;

      retour = CLIC_arbre(fenetre->arbre, souris, &i); /* Clic sur l'arbre donc */
      SORTIE_FONCTION( __FUNCTION__, "%d [normal, GL_TRIPLE_FENETRE sur arbre]", retour);
      return retour;
    }
  }

#ifdef AVEC_OPEN_GL_
  if ( ( BI_FENETRE_GL_ARBRE_SIMPLE == fenetre->type ) || ( BI_FENETRE_GL_ARBRE == fenetre->type ) ) {
    OPTION_A_FAIRE( "Adapter fenêtres BI_FENETRE_GL_ARBRE_SIMPLE et BI_FENETRE_GL_ARBRE aux sous-fenêtres OpenGL" );
  }
#endif

  if ( fenetre->clic != ( int (*)(Fenetre *, int, int, int, int, int) ) NULL ) { /* Il y a une fonction spéciale pour les clics souris */
    BI_Fen_redessin = fenetre->handle;

/*    printf("État des touches : %d (attendu pour Alt : %d)\n", _etat, K_ALT); */
    DANS_FONCTION( "On va appeler fenetre->clic()" );
    /* On appelle en convertissant les coordonnées en local / zone de travail de la fenêtre */
    retour = (fenetre->clic)(fenetre, BI_bouton, n_clics, BI_etat, souris->x - fenetre->w_x, souris->y - fenetre->w_y);

    SORTIE_FONCTION( __FUNCTION__, "%d [normal, fonction associée à la fenêtre]", retour);
    return retour;
  }

#ifdef _DEVELOPPE_
(void) fprintf(stderr, "%s,%s l.%u : Clic sur fenêtre %ld avec le bouton %d --- pas de routine associée !\n",
	       __FILE__, __FUNCTION__, __LINE__, souris->window, BI_bouton);
#endif

  SORTIE_FONCTION( __FUNCTION__, "%d [normal, clic ignoré ?]", VRAI);
  return BI_EVENEMENT_CLIC;
}

/* -------- Gestion des changements de focus des fenêtres -------- */

int EVNT_Activation(Fenetre *fenetre, XFocusChangeEvent *focus)
{
  if ( ( NULL == fenetre ) || ( NULL == focus ) ) {
    /* Pas de fenêtre : rien à faire */
    return BI_EVENEMENT_TRAITE;
  }

  if ( NULL == fenetre->xic ) {
    /* Fenêtre sans contexte pour l'entrée clavier : rien à faire */
    return BI_EVENEMENT_TRAITE;
  }

  /* On active la méthode d'entrée au clavier associée à la fenêtre */
  XSetICFocus( fenetre->xic );
  return BI_EVENEMENT_TRAITE;
}

/* --------- Rectangle intersection de 2 rectangles - renvoie FAUX s'il est vide -------------- */

boolean Rc_intersect(int x_1, int y_1, unsigned int larg_1, unsigned int haut_1,
		     int *x, int *y, unsigned int *larg, unsigned int *haut)
{
  int x_2, y_2;
  unsigned int larg_2, haut_2;

  if ( (x_1 > *x) || ( (x_1 == *x) && (y_1 < *y) ) ) { /* On échange les deux rectangles ==> 1 tjrs x minimal */
    x_2 = x_1;
    y_2 = y_1;
    larg_2 = larg_1;
    haut_2 = haut_1;
    x_1 = *x;
    y_1 = *y;
    larg_1 = *larg;
    haut_1 = *haut;
  } else {
    x_2 = *x;
    y_2 = *y;
    larg_2 = *larg;
    haut_2 = *haut;
  }
  
  if ( (x_1 + (int) larg_1) <= x_2 )  /* Le rectangle 1 finit avant que 2 commence, en x */
    return FAUX;
  
  *x=x_2;
  if ( (x_1 + larg_1) > (x_2 + larg_2) )
    *larg = larg_2;
  else
    *larg = (x_1 + larg_1) - x_2;
  
  if (y_1 >= y_2) {
    if ( (y_2 + (int) haut_2) <= y_1)
      return FAUX;
    
    *y = y_1;
    if ( (y_2 + haut_2) > (y_1 + haut_1) )
      *haut = haut_1;
    else
      *haut = haut_2 - (y_1 - y_2);
    
    return VRAI;   /* Les rectangles se coupent */
  } else {  /* y_1 < y_2 */
    if ( (y_1 + (int) haut_1) <= y_2 )
      return FAUX;

    *y = y_2;
    
    if ( (y_1 + haut_1) > (y_2 + haut_2) )
      *haut = haut_2;
    else
      *haut = haut_1 - (y_2 - y_1);
    
    return VRAI;
  }
}
