/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2006, Emmanuel CURIS
   Création : mai 2006

   Interface graphique avec l'utilisateur

   Généralités pour les stats [macros de gestion de boutons & appel de fonctions internes]
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#define I_STAT_DEFINITION_DES_MENUS_  /* Pour ne pas avoir la définition des sous-menus en double ! */

#include "interface.h"
#include "statistiques.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#include "int_stat_gene.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Réalisation pratique ~~~~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Appel des routines internes simples ~-~-~-~-~-~-~-~-~ */

/* -------- Construire la droite de Henry pour une série de données ----------- */

Graphe *I_STAT_DroiteDeHenry(Graphe *valeurs)
{
  Graphe *resultat;
  char *titre;
  struct serie *serie;

  if ( NULL == valeurs ) return NULL;
  if ( valeurs->genre_donnees != GRF_SERIE ) {
    (void) fprintf(stderr, "%s,%s l.%u : Genre de graphe incorrect [doit être une série _unique_ de valeurs !]\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  OPTION_A_FAIRE( "Demander les options" );

  /* On construit la droite de Henry */
  resultat = STAT_DroiteDeHenry( valeurs, STAT_FDR_EMPIRIQUE_NP1 );
  if ( NULL == resultat ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans STAT_DroiteDeHenry()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On met le nom, pour suivre si nécessaire */
  titre = TXT_genere_chaine( titre_droite_de_Henry, valeurs->nom );
  GRF_titre( resultat, titre );
  free( titre ); titre = NULL;

  /* On l'insère juste après la série de valeurs */
  serie = I_GRF_InsererDansSerie( NULL, valeurs, resultat );
  if ( NULL == serie ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problème dans l'insertion dans la série\n",
		   __FILE__, __FUNCTION__, __LINE__);
    GRF_eff_isole( resultat ); resultat = NULL;
    return NULL;
  }

  /* On met les unités, nom, légendes,... */
  /* En X : les valeurs */
  GRF_ChangerLegende( resultat, AXE_X, (char *) GRF_QuelleLegende( valeurs, AXE_Y ), VRAI );
  GRF_ChangerUnite  ( resultat, AXE_X, GRF_QuelleUnite  ( valeurs, AXE_Y ) );

  /* En Y : les fractiles d'une gaussienne centrée, réduite ; sans unité  */
  GRF_ChangerLegende( resultat, AXE_Y, "Fractiles gaussiens", VRAI );

  /* On a terminé sans souci */
  return resultat;
}

/* -------- Calculer la fonction de répartition d'une série de valeurs -------- */

Graphe *I_STAT_FonctionRepartition(Graphe *valeurs)
{
  Graphe *resultat = NULL;
  char *titre;
  struct serie *serie;

  if ( NULL == valeurs ) return NULL;
  if ( valeurs->genre_donnees != GRF_SERIE ) {
    (void) fprintf(stderr, "%s,%s l.%u : Genre de graphe incorrect [doit être une série _unique_ de valeurs !]\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On récupère la fonction de répartition empirique */
  resultat = STAT_FdREmpirique( valeurs, STAT_FDR_EMPIRIQUE_BASE, 0 );
  if ( NULL == resultat ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans STAT_FdREmpirique()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On met le nom, pour suivre si nécessaire */
  titre = TXT_genere_chaine( titre_fdr_empirique, valeurs->nom );
  GRF_titre( resultat, titre );
  free( titre ); titre = NULL;

  /* On l'insère juste après la série de valeurs */
  serie = I_GRF_InsererDansSerie( NULL, valeurs, resultat );
  if ( NULL == serie ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problème dans l'insertion dans la série\n",
		   __FILE__, __FUNCTION__, __LINE__);
    GRF_eff_isole( resultat ); resultat = NULL;
    return NULL;
  }

  /* On met les unités, nom, légendes,... */
  GRF_ChangerLegende( resultat, AXE_X, (char *) GRF_QuelleLegende( valeurs, AXE_Y ), VRAI );
  GRF_ChangerUnite  ( resultat, AXE_X, GRF_QuelleUnite  ( valeurs, AXE_Y ) );

  /* On a terminé sans souci */
  return resultat;
}

/* ~-~-~-~-~-~-~-~-~ Gestion de demandes génériques ~-~-~-~-~-~-~-~-~ */

/* -------- Demander les paramètres pour un test de comparaison à un témoin -------- */

int I_STAT_CMP_OptionsTemoin(unsigned int *r_methode, unsigned int *r_approximation, int *r_sens, double *r_alpha_global)
{
  int i, retour;
  BI_Champ_Demande questions[ 4 ] = {
    { BI_CHAMP_DEMANDE_LISTE, "M"e_AIGU"thode      :", NULL, 0, 0.0, 0.0, 0.0, ANVA_NOMBRE_METHODES_TEMOIN, options_methode_temoin, { 0 }, { 0 }, { 0 } },
    { BI_CHAMP_DEMANDE_LISTE, "Approximation :", NULL, 0, 0.0, 0.0, 0.0, STAT_IC_NOMBRE_METHODES_APPROXIMATION, options_methode_approximation, { 0 }, { 0 }, { 0 } },
    { BI_CHAMP_DEMANDE_LISTE, "Sens          :", NULL, 0, 0.0, 0.0, 0.0, 3, I_STAT_options_sens_test, { 0 }, { 0 }, { 0 } },
    { BI_CHAMP_DEMANDE_REEL ,  ALPHA" global  :", NULL, BI_EDITABLE_MIN|BI_EDITABLE_MAX|BI_EDITABLE_FRACTION, 0.0, 1.0, 0.0, 0, NULL, { 0 }, { 0 }, { 0 } } };

  /* Valeurs par défaut */
  questions[ 0 ].valeur_defaut.numero_option = ANVA_CMP_TEMOIN_DUNETT;	 /* Test exact de Dunett par défaut */
  questions[ 1 ].valeur_defaut.numero_option = STAT_IC_FRACTILE_EXACT;
  questions[ 2 ].valeur_defaut.numero_option = 0;  /* Test bilatéral par défaut */
  questions[ 3 ].valeur_defaut.reel = 0.05;  /* Travail au risque 5% par défaut */

  /* Valeurs initiales */
  for ( i = 0 ; i < 4 ; i++ ) {
    questions[ i ].valeur_initiale = questions[ i ].valeur_defaut;
  }
  if ( ( r_alpha_global != NULL ) && ( *r_alpha_global > 0.0 ) && ( *r_alpha_global < 1.0 ) ) {
    questions[ 3 ].valeur_initiale.reel = *r_alpha_global;
  }

  /* On demande */
  retour = BI_DemandeMultiple( NULL, &Q_ParametresTemoin, 4, questions );
  if ( ( retour <= 0 ) || ( 2 == retour ) ) {
    AVERTISSEMENT( "Annulation [code %d]", retour );
    return -1;
  }

  /* On a fini, sans annuler : on renvoie les résultats demandés */
  if ( r_methode != NULL ) {
    *r_methode = questions[ 0 ].valeur_donnee.numero_option;
  }
  if ( r_approximation != NULL ) {
    *r_approximation = questions[ 1 ].valeur_donnee.numero_option;
    OPTION_A_FAIRE( "Contrôles de cohérence" );
  }
  if ( r_sens != NULL ) {
    unsigned long sens;

    sens = questions[ 2 ].valeur_donnee.numero_option;
    if      ( 0 == sens ) *r_sens =  0; /* Bilatéral */
    else if ( 1 == sens ) *r_sens = +1;	/* Unilatéral positif */
    else if ( 2 == sens ) *r_sens = -1;	/* Unilatéral négatif */
    else {
      AVERTISSEMENT( "Choix de sens inconnu : %lu", sens );
      *r_sens = 0;		/* Bilatéral par défaut ! */
    }
  }
  if ( r_alpha_global != NULL ) {
    *r_alpha_global = questions[ 3 ].valeur_donnee.reel;
  }

  /* Tout s'est bien passé */
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Affichages génériques ~-~-~-~-~-~-~-~-~ */

/* -------- Affichage des résultats d'un test statistique -------- */

int I_STAT_TEST_AfficherResultat(struct test_stats *test)
{
  int reponse;
  char *texte;

  BI_Boite_Alerte message = {
    ERR_ICONE_INFO,
    I_STAT_TXT_TitreResultatTest,
    NULL, 2, 0, 0,
    { I_STAT_TXT_GarderH0, I_STAT_TXT_GarderH1, NULL, NULL, NULL }
  };

  if ( NULL == test ) return -5;

  texte = TXT_genere_chaine( "%s\nH0 : %s\nH1 : %s\n"
			     "Valeur attendue : %f\n"
			     "Valeur esp"e_AIGU"r"e_AIGU"e  : %e\n"
			     "Valeur observ"e_AIGU"e : %f\n"
			     "Probabilit"e_AIGU" critique : %f",
			     test->nom, test->H0, test->H1, test->v_ideale, test->v_esperee, sqrt( test->variance ),
			     test->resultat, test->p_critique );
  if ( NULL == texte ) {
    (void) fprintf( stderr, "%s,%s l.%u : Impossible de créer le message !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return -2;
  }
  if ( ( ( test->alpha > 0.0 ) && ( test->alpha < 1.0 ) ) &&
       ( ( test->p_critique >= 0.0 ) && ( test->p_critique <= 1.0 ) ) ) {
    if ( test->p_critique > test->alpha ) {
      /* Probabilité critique "de se tromper en rejettant H0" trop forte : plutôt garder H0 */
      message.bouton_ok = 1;
    } else {
      /* Probabilité critique "de se tromper en rejettant H0" suffisamment faible : plutôt choisir H1 */
      message.bouton_ok = 2;
    }
  }

  message.texte = texte;
  reponse = BI_ERR_alerte( principale, &message, VRAI );
  free( texte ); texte = NULL;

  return reponse;
}
